#include <cstdio>
#include <cstring>
#include <memory>
#include <utils.h>
#include <fstream>
#include <sstream>
#include <iomanip>
#include "json.h" 
#include "glext.h"
#include "sir_application.h"

extern GdkCursor *hand_cursor;

using namespace std;

namespace main_app {
       extern Application JavApp;
                   }

static const char *bibliography[] = {"authors", "title", "journal", "year",
                               "volume", "firstpage", "lastpage"};
static const char *sep[] = {"\n", "\n", ", ", ", ",
                               ", ", "-", "."};
//vector<string> bibliography = {"authors", "title", "journal", "year",
//                               "volume", "firstpage", "lastpage"};
//vector<string> sep = {"\n", "\n", ", ", ", ",
//                               ", ", "-", "."};

enum colonne {COD, SG, COMMON, CHEM, MIN, FORM, CELLP, CELLV, BIBLIO, DOI};

typedef struct {
   string el;
   int         am;
               } form_el_type;
   
typedef struct {
   string codnumber;
   string sg;
   string commonname;
   string chemname;
   string mineral;
   string formula;
   string cella;
   string volume;
   string biblio;
   string doi;
               } frag_info_type;

bool GetRemoteFile(string &url, string &localfile, bool ifSilent, long *code);
namespace GLextmr
{
void importFileCOD(std::string fn, int tipo);
}

extern "C" int   get_content_from_formula_(char *param, int Amount[], int Z[],
                    size_t idum);
extern "C" void  jav_cod_(int *act, int *whichModel, int *kadd);


static bool GetFile(string &url, string &localfile, bool ifSilent)
{
    long code;
    return  GetRemoteFile(url, localfile, ifSilent, &code);
}

bool sort_Hill ( form_el_type elem1, form_el_type elem2 )
{
   return (elem1.el < elem2.el);
}

bool form_per_COD(char *form1, string &form2)
{
   size_t sz = strlen(form1);
   int *Z = g_new0(int, MAX_SPECIES);
   int *N = g_new0(int, MAX_SPECIES);
   int nElements = get_content_from_formula_(form1, N, Z, sz);
   vector<form_el_type> form_vet;
   if(nElements > 0)
   {
      for(int i=0; i<nElements; i++)
      {
          form_el_type f1;
          f1.el = get_nth_element(Z[i]-1);
          f1.am = N[i];
          form_vet.push_back(f1);
      }
      sort(form_vet.begin(), form_vet.end(), sort_Hill);
      stringstream formula;
      formula << "formula=";
      for(int i=0; i<nElements; i++)
      {
          if(i)
             formula << "%20";
          formula << form_vet[i].el;
          if(form_vet[i].am > 1)
             formula << form_vet[i].am;
      }
      form2.assign(formula.str());
      return false;
   }
      g_free(Z);
      g_free(N);
      return true;
}

void ClearListaFragV(GtkWidget *list)
{
    GtkTreeView *treeview = (GtkTreeView *)list;
    GtkTreeModel *model = gtk_tree_view_get_model(treeview);
    GtkTreeSelection *sele = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
    int hnd = get_int(list, "hnd");
    g_signal_handler_block(G_OBJECT(sele), hnd);
    gtk_list_store_clear(GTK_LIST_STORE(model));
    g_signal_handler_unblock(G_OBJECT(sele), hnd);
}

size_t RiempiListaFragV(GtkWidget *list, vector<frag_info_type> v)
{
    GtkTreeView *treeview = (GtkTreeView *)list;
    GtkTreeModel *model = gtk_tree_view_get_model(treeview);
    GtkTreeSelection *sele = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
    int hnd = get_int(list, "hnd");
    g_signal_handler_block(G_OBJECT(sele), hnd);

    gtk_list_store_clear(GTK_LIST_STORE(model));

    size_t szv = v.size();
    GtkTreeIter iter;
    for(size_t i=0; i<szv; i++)
    {
        gtk_list_store_append (GTK_LIST_STORE (model), &iter);
        gtk_list_store_set (GTK_LIST_STORE (model), &iter,
          0, trim(v[i].codnumber).c_str(), 1, trim(v[i].sg).c_str(),
          2, trim(v[i].commonname).c_str(), 3, trim(v[i].chemname).c_str(),
          4, trim(v[i].mineral).c_str(), 5, trim(v[i].formula).c_str(),
          6, trim(v[i].cella).c_str(), 7, atof(v[i].volume.c_str()),
          8, trim(v[i].biblio).c_str(), 9, trim(v[i].doi).c_str(), -1);
    }
    g_signal_handler_unblock(G_OBJECT(sele), hnd);
    return szv;
}

GtkWidget *crealistafrag(GtkWidget *vbox, GtkWidget *father)
{
  const char *titles[] = { "COD Number", "Spacegroup", "Common Name",
                           "Chemical Name", "Mineral", "Formula",
                           "Cell parameters", "Cell volume", "Biblio","doi"};
  GType types[] = { G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
                    G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
                    G_TYPE_STRING, G_TYPE_FLOAT, G_TYPE_STRING, G_TYPE_STRING};
  int  sizes[] = { 0, 0, -1, -1, -1 , 0, 0, 0, -1, -1};
  int  precs[] = { 0, 0, 0, 0, 0 , 0, 0, 0, 0, 0};
  int sort_type[] = {COD, SG, -1, -1, -1, FORM, -1, CELLV, -1, -1};
  int ncol = (int)(sizeof(sizes)/sizeof(sizes[0]));

  int lar, alt;
  PangoFontDescription *fontlist = get_fontN(MONOSPACE_FONT);
  get_metrics(fontlist, &lar, &alt);


  GtkWidget *list =
  new_selection(father, vbox, "frags", ncol, titles, sizes, types, precs);
  g_object_set_data(G_OBJECT(list), "father_b", vbox);
  widget_modify_font(list, fontlist);
  gtk_widget_set_size_request(get_widget(list, "scrollwin"), -1, 15*alt);

  for(int i=0; i<ncol; i++)
  {
      GtkTreeViewColumn *column = gtk_tree_view_get_column(GTK_TREE_VIEW(list), i);
      GtkWidget *label = gtk_tree_view_column_get_widget(column);
      if(label)
         widget_modify_font(label, fontlist);
      g_object_set_data(G_OBJECT(column), "father", list);
      if(sort_type[i] >= 0)
         gtk_tree_view_column_set_sort_column_id(column, i);
  }
  set_wrapwidth(list,300, FORM);
  set_wrapwidth(list,300, CELLP);

  return list;
}

string filtraFormula(string formula)
{
     string f2;
     f2 = filtrastr(formula, '"');
     size_t found = f2.find("- ");
     if(found != string::npos)
     {
        size_t found2 = f2.find(" -");
        if(found2 != string::npos)
           f2 = f2.substr(found+2, found2-found-2);
     }
     return f2;
}

void CheckFile(string local)
{
  ifstream ifs(local.c_str());
  filebuf* pbuf = ifs.rdbuf();

  size_t size = pbuf->pubseekoff (0,ifs.end,ifs.in);
  pbuf->pubseekpos (0,ifs.in);

  char* buffer=new char[size];
  pbuf->sgetn (buffer,size);
  ifs.close();
 
  string b(buffer);
  size_t ini = b.find("[");
  if((ini != 0) && (ini != string::npos))
  {
     ofstream ofs(local.c_str());
     ofs.write(b.substr(ini).c_str(), size-ini);
     ofs.close();
  }
  delete[] buffer;
}

int RiempiListaFrag(GtkWidget *list, string command)
{
    GtkWidget *dlg = gtk_widget_get_toplevel(list);
    ShowWaitCursor(dlg);
    string remote("http://www.crystallography.net/cod/result.php?");
    remote.append(command);
    remote.append("&format=json");
    //cout << remote << "\n";
    string local("tmp.json");
    GetFile(remote, local, false);
    //CheckFile(local);
    ifstream ifs(local.c_str());
    
    Json::CharReaderBuilder rbuilder;
    std::unique_ptr<Json::CharReader> const reader(rbuilder.newCharReader());
    string errs;
    Json::Value val;
    bool b = Json::parseFromStream(rbuilder, ifs, &val, &errs);
    if (!b)
    {
        cout << "Error: " << errs << endl;
        HideWaitCursor(dlg);
        ClearListaFragV(list);
        gtk_label_set_text(GTK_LABEL(get_widget(dlg, "label_info2")), 
           "No records found.");
        gtk_widget_set_visible(get_widget(dlg, "label_info"), FALSE);
        gtk_widget_set_sensitive(get_widget(dlg, "download"), FALSE);
        return -1;
    }
    vector<frag_info_type> v;
    frag_info_type el;

    switch (val.type())
    {
    default: break;

    case Json::arrayValue:
        {
        Json::ArrayIndex size = val.size();
        for (Json::ArrayIndex i=0; i<size; i++)
        {
             el.codnumber = filtrastr(val[i]["file"].toStyledString(), '"');
             if(val[i]["sg"].type() == Json::nullValue)
                el.sg = " ";
             else
                el.sg = filtrastr(val[i]["sg"].toStyledString(), '"');
             if(val[i]["commonname"].type() == Json::nullValue)
                el.commonname = " ";
             else
                el.commonname = filtrastr(val[i]["commonname"].asString(), '"');
             if(val[i]["chemname"].type() == Json::nullValue)
                el.chemname = " ";
             else
                el.chemname = filtrastr(val[i]["chemname"].asString(), '"');
             if(val[i]["mineral"].type() == Json::nullValue)
                el.mineral = " ";
             else
                el.mineral = filtrastr(val[i]["mineral"].asString(), '"');
             if(val[i]["a"].type() == Json::nullValue)
                el.cella = " ";
             else
             {
                el.cella = filtrastr(val[i]["a"].asString(), '"');
                el.cella.append(" ");
                el.cella.append(filtrastr(val[i]["b"].asString(), '"'));
                el.cella.append(" ");
                el.cella.append(filtrastr(val[i]["c"].asString(), '"'));
                el.cella.append(";");
                el.cella.append(filtrastr(val[i]["alpha"].asString(), '"'));
                el.cella.append(" ");
                el.cella.append(filtrastr(val[i]["beta"].asString(), '"'));
                el.cella.append(" ");
                el.cella.append(filtrastr(val[i]["gamma"].asString(), '"'));
             }
             el.formula = filtraFormula(val[i]["formula"].asString());
             if(val[i]["vol"].type() == Json::nullValue)
                el.volume = " ";
             else
                el.volume = filtrastr(val[i]["vol"].asString(), '"');
             el.biblio = "";
             for(size_t j=0; j<sizeof(bibliography)/sizeof(bibliography[0]); j++)
             {
                 if(val[i][bibliography[j]].type() != Json::nullValue)
                 {
                    el.biblio.append(val[i][bibliography[j]].asString());
                    el.biblio.append(sep[j]);
                 }
             }
             if(val[i]["doi"].type() == Json::nullValue)
                el.doi = " ";
             else
                el.doi = filtrastr(val[i]["doi"].asString(), '"');
             v.push_back(el);
        }
        }
        break;
    case Json::objectValue:
        el.codnumber = filtrastr(val["file"].toStyledString(), '"');
        if(val["sg"].type() == Json::nullValue)
           el.sg = " ";
        else
           el.sg = filtrastr(val["sg"].toStyledString(), '"');
        if(val["commonname"].type() == Json::nullValue)
           el.commonname = " ";
        else
           el.commonname = filtrastr(val["commonname"].asString(), '"');
        if(val["chemname"].type() == Json::nullValue)
           el.chemname = " ";
        else
           el.chemname = filtrastr(val["chemname"].asString(), '"');
        if(val["mineral"].type() == Json::nullValue)
           el.mineral = " ";
        else
           el.mineral = filtrastr(val["mineral"].asString(), '"');
        if(val["a"].type() == Json::nullValue)
                el.cella = " ";
        else
        {
            el.cella = filtrastr(val["a"].asString(), '"');
            el.cella.append(" ");
            el.cella.append(filtrastr(val["b"].asString(), '"'));
            el.cella.append(" ");
            el.cella.append(filtrastr(val["c"].asString(), '"'));
            el.cella.append(";");
            el.cella.append(filtrastr(val["alpha"].asString(), '"'));
            el.cella.append(" ");
            el.cella.append(filtrastr(val["beta"].asString(), '"'));
            el.cella.append(" ");
            el.cella.append(filtrastr(val["gamma"].asString(), '"'));
        }
        el.formula = filtraFormula(val["formula"].asString());
        if(val["vol"].type() == Json::nullValue)
           el.volume = " ";
        else
           el.volume = filtrastr(val["vol"].asString(), '"');
        el.biblio = "";
        for(size_t j=0; j<sizeof(bibliography)/sizeof(bibliography[0]); j++)
        {
           if(val[bibliography[j]].type() != Json::nullValue)
           {
              el.biblio.append(val[bibliography[j]].asString());
              el.biblio.append(sep[j]);
           }
        }
        if(val["doi"].type() == Json::nullValue)
           el.doi = " ";
        else
           el.doi = filtrastr(val["doi"].asString(), '"');

        v.push_back(el);
        break;
    }

    size_t szv = RiempiListaFragV(list, v);
    v.clear();
    select_row(list, 0);

    gtk_label_set_text(GTK_LABEL(get_widget(dlg, "label_info2")), (szv>0)?
           g_strdup_printf("%d records found.", (int)szv):
           "No records found.");
    gtk_widget_set_visible(get_widget(dlg, "label_info"), szv>0);

    HideWaitCursor(dlg);
    return 0;
}

void disp_names_in_table(GtkTreeSelection  *sele, GtkWidget *table)
{
    GtkTreeModel *model;
    GtkTreeIter   iter;

    if (! gtk_tree_selection_get_selected (sele, &model, &iter))
    {
      gtk_widget_hide(table);
      gtk_widget_set_sensitive(get_widget(gtk_widget_get_toplevel(table), "download"), FALSE);
      return;
    }
    gtk_widget_set_sensitive(get_widget(gtk_widget_get_toplevel(table), "download"), TRUE);
    GtkTreeView *list = gtk_tree_selection_get_tree_view(sele);
    GtkWidget *label = get_widget(table, "1l");
    gtk_label_set_text(GTK_LABEL(label), "Chemical names: ");
    //gtk_label_set_text(GTK_LABEL(label), "Common name: ");
    //label = get_widget(table, "2l");
    //gtk_label_set_text(GTK_LABEL(label), "Chemical name: ");
    //label = get_widget(table, "3l");
    //gtk_label_set_text(GTK_LABEL(label), "Mineral name: ");
    //label = get_widget(table, "4l");
    //gtk_label_set_text(GTK_LABEL(label), "Bibliography: ");
    //label = get_widget(table, "5l");
    //gtk_label_set_text(GTK_LABEL(label), "doi: ");
    label = get_widget(table, "2l");
    gtk_label_set_text(GTK_LABEL(label), "Bibliography: ");
    label = get_widget(table, "3l");
    //gtk_label_set_text(GTK_LABEL(label), "doi: ");
    gtk_label_set_text(GTK_LABEL(label), " ");
    string chemnames("");
    char *commonname = get_selected_row_col(GTK_WIDGET(list), COMMON);
    if(!str_white(commonname))
    {
       chemnames.append(commonname);
       chemnames.append("; ");
    }
    label = get_widget(table, "1lv");
    //label = get_widget(table, "1lv");
    //gtk_label_set_text(GTK_LABEL(label), (commonname !=NULL)?
    //    commonname: " ");
    char *chemname = get_selected_row_col(GTK_WIDGET(list), CHEM);
    if(!str_white(chemname))
    {
       chemnames.append(chemname);
       chemnames.append("; ");
    }
    //label = get_widget(table, "2lv");
    //gtk_label_set_text(GTK_LABEL(label), (chemname !=NULL)?
    //    chemname: " ");
    char *mineral = get_selected_row_col(GTK_WIDGET(list), MIN);
    if(!str_white(mineral))
    {
       chemnames.append(mineral);
       chemnames.append("; ");
    }
    //label = get_widget(table, "3lv");
    //gtk_label_set_text(GTK_LABEL(label), (mineral !=NULL)?
    //    mineral: " ");
    if(!str_white(chemnames.c_str()))
    {
       size_t found = chemnames.find_last_of(";");
       if(found != string::npos)
          chemnames = chemnames.substr(0, found);
    }
    label = get_widget(table, "1lv");
    gtk_label_set_text(GTK_LABEL(label), chemnames.c_str());
    char *biblio = get_selected_row_col(GTK_WIDGET(list), BIBLIO);
    //label = get_widget(table, "4lv");
    label = get_widget(table, "2lv");
    gtk_label_set_text(GTK_LABEL(label), (biblio !=NULL)?
        biblio: " ");
    gtk_widget_show(table);
    char *doi = get_selected_row_col(GTK_WIDGET(list), DOI);
    label = get_widget(table, "3lv");
    //label = get_widget(table, "5lv");

    gtk_widget_hide(gtk_widget_get_parent(label));
    if(!str_white(doi))
    {
       std::string str_lab = "https://doi.org/";
       str_lab.append(doi);
       set_hypertext(label, str_lab.c_str());
       gtk_widget_show(gtk_widget_get_parent(label));
    }

    char *filen = get_selected_row_col(GTK_WIDGET(list), COD);
    if(filen !=NULL)
    {
       string cif_file("http://www.crystallography.net/cod/");
       cif_file.append(g_strstrip(filen));
       cif_file.append(".cif");
       string cif_local("tmp.cif");
       GetFile(cif_file, cif_local, false);
       g_object_set_data(G_OBJECT(gtk_widget_get_toplevel(table)), "modificato", GINT_TO_POINTER(1));
       GLextmr::importFileCOD(cif_local, 0);
    }

}

string getElements(string s, string w, size_t lim)
{
    vector<string> v = split_str(s, " ");
    stringstream s1;
    if(!v.empty())
    {       
       size_t fin = MRmin(v.size(), lim);
       for(size_t i=0; i<fin; i++)
       {
           if(i)
              s1 << "&" << w << i+1 << "=" <<v.at(i);
           else
              s1 << w << i+1 << "=" <<v.at(i);
       }
       v.clear();
    }       
    return s1.str();
}

static gboolean link_clicked(GtkWidget *wid, GdkEvent *, GtkWidget *label)
{
     std::string tit = gtk_label_get_text(GTK_LABEL(label));
     URLOpen(tit, main_app::JavApp.InternetBrowser(), gtk_widget_get_toplevel(wid));
     return TRUE;
}

GtkWidget *create_label_info()
{
    int nrows = 3; //5;
    GtkWidget *table = table_new(nrows, 2, FALSE);
    table_set_row_spacings(table, 6);
    table_set_col_spacings(table, 6);
    
    for(int row=0; row<nrows; row++)
    {
        stringstream s;
        s << row+1 << "l";
        for(int col=0; col<2; col++)
        {
            GtkWidget *w;
            if(col)
            {
               w = LABEL_NEW(" ", NULL);
               if(row == (nrows-1))
               {
                  GtkWidget *eb = gtk_event_box_new();
                  gtk_container_add(GTK_CONTAINER(eb), w);
                  g_signal_connect (G_OBJECT (eb), "button_press_event",
                      G_CALLBACK (link_clicked), w);
                  gtk_widget_set_events(eb, GDK_BUTTON_PRESS_MASK );
                  gtk_widget_show (eb);
                  table_attach_exp(table, eb, col, col+1, row, row+1);
                  g_object_set_data(G_OBJECT(table), "eb", eb);
               }
            }
            else
               w = LABEL_BOLD(" ", NULL, 1, 0);
            gtk_widget_show (w);
            set_alignment(w, 0., 0.5);
            gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
            if(gtk_widget_get_parent(w) == NULL)
               table_attach_exp(table, w, col, col+1, row, row+1);
            if(col)
            {
               s << "v";
               gtk_label_set_line_wrap(GTK_LABEL(w), TRUE);
               if(row < (nrows-1))
                  gtk_label_set_selectable(GTK_LABEL(w), TRUE);
            }
            g_object_set_data(G_OBJECT(table), s.str().c_str(), w);
        }
    }
    return table;
}

void accept_formula(GtkWidget *wid, gpointer data)
{
using namespace gtkext;
     GtkWidget *dlg = gtk_widget_get_toplevel(wid);
     GtkWidget *window = (GtkWidget *) data;

     GtkWidget *w = get_widget(dlg, "formula");
     char *form1 = g_strdup( gtk_entry_get_text(GTK_ENTRY(w)));
     string command;
     bool ier = false;
     bool first_com = true;
     if(!str_white(form1))
     {
        string formula;
        ier = form_per_COD(form1, formula);
        if(ier)
        {
           dlg_errmsg(GTK_WINDOW(window), "Sorry. Wrong formula.");
           return;
        }
        else
        {
           command.append(formula);
           first_com = false;
        }
        g_free(form1);
     }
     w = get_widget(dlg, "text");
     form1 = g_strdup( gtk_entry_get_text(GTK_ENTRY(w)));
     if(!str_white(form1))
     {
        string s(form1);
        if(!first_com)
           command.append("&");
        first_com = false;
        command.append(getElements(s, "text", 2));
        g_free(form1);
     }
     w = get_widget(dlg, "elements");
     form1 = g_strdup( gtk_entry_get_text(GTK_ENTRY(w)));
     if(!str_white(form1))
     {
        string s(form1);
        if(!first_com)
           command.append("&");
        first_com = false;
        command.append(getElements(s, "el", 8));
        g_free(form1);
     }
     w = get_widget(dlg, "not_elements");
     form1 = g_strdup( gtk_entry_get_text(GTK_ENTRY(w)));
     if(!str_white(form1))
     {
        string s(form1);
        if(!first_com)
           command.append("&");
        first_com = false;
        command.append(getElements(s, "nel", 8));
        g_free(form1);
     }
     w = get_widget(dlg, "strictmin");
     form1 = g_strdup( gtk_entry_get_text(GTK_ENTRY(w)));
     if(!str_white(form1))
     {
        if(!first_com)
           command.append("&");
        first_com = false;
        command.append("strictmin=");
        command.append(form1);
        g_free(form1);
     }
     w = get_widget(dlg, "strictmax");
     form1 = g_strdup( gtk_entry_get_text(GTK_ENTRY(w)));
     if(!str_white(form1))
     {
        if(!first_com)
           command.append("&");
        first_com = false;
        command.append("strictmax=");
        command.append(form1);
        g_free(form1);
     }

     if(!ier)
        RiempiListaFrag(get_widget(dlg, "frags"), command);
}

void ok_COD(GtkWidget *w, gpointer data)
{
     GtkWidget *dlg = gtk_widget_get_toplevel(w);
     int ifAdd = get_int(dlg, "ifAdd");
     char *filen = get_selected_row_col(get_widget(dlg, "frags"), COD);
     if(filen !=NULL)
     {
        string cif_file("http://www.crystallography.net/cod/");
        string cif_local(g_strstrip(filen));
        cif_local.append(".cif");
        cif_file.append(g_strstrip(filen));
        cif_file.append(".cif");
        GetFile(cif_file, cif_local, false);
        GLextmr::importFileCOD(cif_local, 0);
        int whichModel = get_int(dlg, "whichModel") + 1;
        int act = 1;
        jav_cod_(&act, &whichModel, &ifAdd);
     }
     gtk_widget_hide(dlg);
}

void download_COD(GtkWidget *w, gpointer data)
{
using namespace gtkext;
     GtkWidget *dlg = gtk_widget_get_toplevel(w);
     char *filen = get_selected_row_col(get_widget(dlg, "frags"), COD);
     //gtk_widget_hide(dlg);
     if(filen !=NULL)
     {
        string cif_file("http://www.crystallography.net/cod/");
        string cif_local(g_strstrip(filen));
        cif_local.append(".cif");
        cif_file.append(cif_local);
        FilselItem local_item = saveas(dlg, "*.cif", cif_local.c_str());
        if(local_item.exitCode > 0)
        {
            string local_cif;
            local_cif = local_item.cwd;
            local_cif.append(local_item.fileinp);
            GetFile(cif_file, local_cif, false);
        }
     }
     //gtk_widget_show(dlg);
}

void cancel_COD(GtkWidget *w, gpointer data)
{
     GtkWidget *dlg = gtk_widget_get_toplevel(w);
     int ifAdd = get_int(dlg, "ifAdd");
     int modi = get_int(dlg, "modificato");
     if(modi)
     {
        int whichModel = get_int(dlg, "whichModel") + 1;
        int act = 2;
// Restore current model
        jav_cod_(&act, &whichModel, &ifAdd);
     }
     gtk_widget_hide(dlg);
}

GtkWidget *create_addCOD(GtkWidget *window, int whichModel)
{
   GtkWidget *dlg, *vbox, *button, *w;

   dlg = gtk_dialog_new ();
   gtk_container_set_border_width(GTK_CONTAINER(dlg), 8);
   g_object_set_data(G_OBJECT(dlg), "whichModel", GINT_TO_POINTER(whichModel-1));
   gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(window));
   gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);

   vbox = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
   GtkWidget *hbox = table_new(6, 2, FALSE);
   gtk_widget_show (hbox);
   //gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 4);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 4);
   table_set_row_spacings(hbox, 6);
   table_set_col_spacings(hbox, 6);

   int row = 0;
   w = LABEL_NEW("Text:", NULL);
   gtk_widget_show (w);
   set_alignment(w, 0., 0.5);
   gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
   table_attach(hbox, w, 0, 1, row, row+1);

   w = gtk_entry_new ();
   gtk_widget_show (w);
   table_attach(hbox, w, 1, 2, row, row+1);
   g_object_set_data(G_OBJECT(dlg), "text", w);
   set_tip(w, "A keyword that must be contained in the metadata of the entry. "
           "The metadata search covers data such as the bibliographic "
           "information of the associated publication, common and systematic "
           "name of the sample compound and the name of the sample mineral.");
   row++;

   w = LABEL_NEW("Formula:", NULL);
   gtk_widget_show (w);
   set_alignment(w, 0., 0.5);
   gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
   table_attach(hbox, w, 0, 1, row, row+1);

   w = gtk_entry_new ();
   gtk_widget_show (w);
   table_attach(hbox, w, 1, 2, row, row+1);
   g_object_set_data(G_OBJECT(dlg), "formula", w);
   set_tip(w, "The empirical chemical formula of the crystal. Chemical "
         "element symbols in the formula must be separated by a space symbol.");
   row++;

   w = LABEL_NEW("Elements:", NULL);
   gtk_widget_show (w);
   set_alignment(w, 0., 0.5);
   gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
   table_attach(hbox, w, 0, 1, row, row+1);

   w = gtk_entry_new ();
   gtk_widget_show (w);
   table_attach(hbox, w, 1, 2, row, row+1);
   g_object_set_data(G_OBJECT(dlg), "elements", w);
   set_tip(w, "Chemical element symbols that must appear in the chemical "
              "formula. They must be separated by a space symbol.");
   row++;

   w = LABEL_NEW("NOT these elements:", NULL);
   gtk_widget_show (w);
   set_alignment(w, 0., 0.5);
   gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
   table_attach(hbox, w, 0, 1, row, row+1);

   w = gtk_entry_new ();
   gtk_widget_show (w);
   table_attach(hbox, w, 1, 2, row, row+1);
   g_object_set_data(G_OBJECT(dlg), "not_elements", w);
   set_tip(w, "Chemical element symbols that must not appear in the chemical "
              "formula. They must be separated by a space symbol.");
   row++;

   w = LABEL_NEW("No. of elements min and max:", NULL);
   gtk_widget_show (w);
   set_alignment(w, 0., 0.5);
   gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
   table_attach(hbox, w, 0, 1, row, row+1);

   GtkWidget *hb = hbox_new(TRUE, 0);
   gtk_widget_show (hb);
   table_attach(hbox, hb, 1, 2, row, row+1);

   w = gtk_entry_new ();
   gtk_widget_show (w);
   gtk_box_pack_start(GTK_BOX(hb), w, TRUE, TRUE, 0);
   set_width_chars(GTK_ENTRY(w), 6);
   g_object_set_data(G_OBJECT(dlg), "strictmin", w);
   set_tip(w, "the minimum number of distinct chemical elements that must "
               "appear in the chemical formula");
   w = gtk_entry_new ();
   gtk_widget_show (w);
   set_width_chars(GTK_ENTRY(w), 6);
   gtk_box_pack_end(GTK_BOX(hb), w, TRUE, TRUE, 0);
   g_object_set_data(G_OBJECT(dlg), "strictmax", w);
   set_tip(w, "the maximum number of distinct chemical elements that must "
               "appear in the chemical formula");
   row++;

#ifdef IN_GTK3
    w = new_alignment(GTK_ALIGN_CENTER, GTK_ALIGN_CENTER, FALSE, FALSE);
#else
    w = gtk_alignment_new(0.5f, 0.5f, 0, 0);
#endif

   gtk_widget_show (w);
   button = myimage_button("  _Search", "find3.png");
   gtk_widget_show (button);
   table_attach(hbox, w, 0, 2, row, row+1);
   gtk_container_add(GTK_CONTAINER(w), button);
   g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(accept_formula),
     window);

   GtkWidget *list = crealistafrag(vbox, dlg);
   gtk_widget_show (list);

   GtkWidget *label_info = create_label_info();
   gtk_widget_show (label_info);
   gtk_box_pack_start (GTK_BOX (vbox), label_info, FALSE, FALSE, 4);
   g_object_set_data(G_OBJECT(dlg), "eb", get_widget(label_info, "eb"));
   g_object_set_data(G_OBJECT(dlg), "label_info", label_info);

   GtkTreeSelection *sele = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
   int hnd = g_signal_connect (G_OBJECT (sele), "changed",
                      G_CALLBACK (disp_names_in_table), label_info);
   g_object_set_data(G_OBJECT(list), "hnd", GINT_TO_POINTER(hnd));

   GtkWidget *label_info2 = LABEL_NEW(" ", NULL);
   set_alignment(label_info2, 0., 0.5);
   gtk_label_set_justify(GTK_LABEL(label_info2), GTK_JUSTIFY_LEFT);
   gtk_widget_show (label_info2);
   gtk_box_pack_end (GTK_BOX (vbox), label_info2, FALSE, FALSE, 4);
   g_object_set_data(G_OBJECT(dlg), "label_info2", label_info2);

   button = OK_button();
   gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_OK);
   gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
      GTK_BUTTONBOX_SPREAD);
   g_object_set_data(G_OBJECT(dlg), "ok", button);
   g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(ok_COD),
     NULL);

   button = myimage_button("_Download CIF", "download.png");
   gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_ACCEPT);
   gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
      GTK_BUTTONBOX_SPREAD);
   g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(download_COD),
     NULL);
   gtk_widget_set_sensitive(button, FALSE);
   g_object_set_data(G_OBJECT(dlg), "download", button);

   button = CANCEL_button();
   gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_CANCEL);
   g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(cancel_COD),
     NULL);
   return dlg;
}

void addCOD(GtkWidget *window, int whichModel, int ifAdd)
{
   static GtkWidget *dlg = NULL;
   if(!dlg)
      dlg = create_addCOD(window, whichModel);
   if(ifAdd)
   {
      gtk_window_set_title(GTK_WINDOW(dlg), "Add fragment from COD");
      myimage_button_set_text(get_widget(dlg, "ok"), "  Add  ");
   }
   else
   {
      gtk_window_set_title(GTK_WINDOW(dlg), "Import fragment from COD");
      myimage_button_set_text(get_widget(dlg, "ok"), "Import ");
   }
   g_object_set_data(G_OBJECT(dlg), "ifAdd", GINT_TO_POINTER(ifAdd));
   if(!gtk_widget_get_visible(dlg))
   {
// Save current model
     int act = 0;
     jav_cod_(&act, &whichModel, &ifAdd);
     //cout << "save current model\n";
     g_object_set_data(G_OBJECT(dlg), "modificato", GINT_TO_POINTER(0));
   }

   gtk_widget_show(dlg);
   gdk_window_set_cursor (gtk_widget_get_window(get_widget(dlg, "eb")), hand_cursor);
}

