MODULE bb_bc

 USE progtype

implicit none

type bb_bc_condition
   integer :: nump     ! size of population
   integer :: niter    ! number of iterations
   real    :: tol      ! Stopping criterion
   integer :: nlocmax  ! numero di local search
   real    :: beta     ! parametro di controllo
   logical :: auto     ! automatic setting of niter
end type bb_bc_condition

type pop_type
   real                            :: cfval
   real, dimension(:), allocatable :: x
end type pop_type

type bb_bc_iter
   type(param_type),dimension(:), allocatable :: par         ! pTarametri
   type(pop_type), dimension(:), allocatable     :: pop
   real, dimension(:,:),allocatable :: valdiff
end type bb_bc_iter

integer :: maxr = 3
type(pop_type), dimension(:), allocatable                      :: pop
type(pop_type), dimension(:), allocatable                      :: xcm1_gen   !centri di massa generati
type(pop_type), dimension(:), allocatable                      :: pop_app
type(pop_type), dimension(:), allocatable                      :: pop_app1
type(pop_type), dimension(:), allocatable                      :: pop_sav
type(pop_type), dimension(:), allocatable                      :: temppop
integer, dimension(:), allocatable                             :: iord
logical  :: alloc=.false.
CONTAINS

   subroutine bb_bc_initialize(cond)
   type(bb_bc_condition) :: cond
!
   cond%nump = 100
   cond%niter = 20
   cond%tol = 0.1  
   cond%nlocmax = 3
   cond%beta = 0.3
   cond%auto = .true.
!
   end subroutine bb_bc_initialize

!-------------------------------------------------------------------------------

   subroutine bb_bc_algo(par,cond,ginfo_ini,bestcfvalue,func,funcs,sc_ini,neval,kevent)
   USE rand_mod
   USE simulated_anneal
   USE nr
   USE stat
   USE SA_INTERFACE
   USE arrayutil
  !USE variables, only: cryst

   INTERFACE
!
!     Function per il calcolo della funzione costo
      real function func(xpar,hpar)
      real, intent(in), dimension(:) :: xpar
      integer, intent(in)            :: hpar
      end function func
!
      subroutine funcs(xp,fval,grad,icod)
      real, dimension(:), intent(in)            :: xp
      real, intent(out)                         :: fval
      real, dimension(:), intent(out), optional :: grad
      integer, intent(in)                       :: icod
      end subroutine funcs

   END INTERFACE
   type(param_type), dimension(:), intent(inout) :: par         ! parametri
   type(bb_bc_condition), intent(in)             :: cond
   type(opti_info_type), intent(in)              :: ginfo_ini
   real, intent(out)                             :: bestcfvalue ! valore finale della funzione costo
   type(SimAnn_Conditions_type), intent(in)      :: sc_ini
   integer, intent(out)                          :: kevent
   integer                                       :: neval 
   type(opti_info_type)                          :: ginfo
   type(SimAnn_Conditions_type)                  :: sc
   integer                                       :: npar

   type(bb_bc_iter), dimension(:),allocatable    :: itervalues
   integer, dimension(1)                         :: loc

   integer                                       :: i,j,k,it,l   !!!,jj
   real, dimension(size(par))                    :: xc,xcm1,xcm2!!!,xcm3
   integer                                       :: kmethod
   real                                          :: xrand
   real                                          :: val,rangev
   real                                          :: alpha, alphastar
   real                                          :: cfxc
   real                                          :: alpha_min,alpha_max
   real                                          :: coef
   integer                                       :: nloc,nlocc   !,nlocmax
   integer                                       :: i_do
   type(param_type), dimension(size(par)) :: sapar
   real, dimension(size(par)) :: xopt
   real :: fopt,tfin
   integer :: nacc,nobdss
   logical :: bestfound
  !integer, dimension(:), allocatable :: iord
   integer :: jord    
   real :: cfave,cfvar,stopc
   integer                         :: nacf
   integer                         :: ngen
   real                            :: fmin    
   integer                         :: kpr = 0
   real, dimension(maxr,size(par)) :: valdiff ! NC
   real   :: minrand, pop_savbest
   integer :: r, sabest=0, ok, newmaxr, sacount=0
   integer :: nrmax, limitrmax, start=1
   logical :: valuta=.false., esplora=.false.,newmin=.false.
   logical :: struttura = .false.
   logical :: yesval = .true.
   logical :: yesgrasp = .true.
   integer :: step=1
   integer                          :: nsol_rand
   real                             :: cfval
   common/provvisorio/sabest_tot,sacount_tot
   integer :: sabest_tot,sacount_tot
!

   yesval = .true.
   maxr=3

   npar = size(par)   ! numero di parametri
   kmethod = 2
  !kmethod = 1

   ngen = 1*cond%nump    ! numero di elementi da generare
  !ngen = cond%nump + nint(0.5*(cryst(1)%numfrag() - 1))*cond%nump

! allocations
  call reallocate_pop_bb(pop,ngen,npar)
  call reallocate_pop_bb(pop_sav,ngen,npar)
  call reallocate_pop_bb(temppop,ngen,npar)
  call resize_array(iord,ngen)

  if(.not.allocated(itervalues)) then
    allocate(itervalues(cond%niter))

    do i=1,cond%niter                  ! loop sul numero di iterazioni
       allocate(itervalues(i)%par(npar))
       allocate(itervalues(i)%pop(ngen))
       allocate(itervalues(i)%valdiff(maxr,npar))
    end do
  endif

!  Starting population is generating
   pop(1)%x = par(:)%val
   pop(1)%cfval = func(pop(1)%x,0)
!
! Applica un fast SA sul modello importato
   fmin = pop(1)%cfval
   neval = 1
   ginfo = ginfo_ini
   sc = sc_ini
   sapar(:) = par(:)
   call sa(sapar,sc,ginfo,tfin,xopt,fopt,nacc,neval,nobdss,kevent,func,funcs,fmin)
   if((fopt/pop(1)%cfval)<0.1) then
       pop(1)%cfval = fopt
       pop(1)%x = xopt
       par(:)%val=xopt
   endif
   if (kevent == 1 .or. kevent == 2) then ! gestione eventi grafica
       bestcfvalue = fopt
       return    
   endif

! genero la restante popolazione 
   do i=2,ngen
      do j=1,npar
         pop(i)%x(j) = randvalue(par(j)%low,par(j)%high)
      enddo
   enddo
!
!applico il GRASP ad un certo numero di elementi della popolazione scelti a caso
     i_do=10
     do l=1,int(0.7*ngen)
      yesgrasp = .true.
      call random_number(xrand)
      nsol_rand=1+mod(int(xrand*l),l)
      xcm1(:) = pop(nsol_rand)%x                  ! prende il mod con cfval max

      call grasp(npar,par,xcm1,func,ginfo,cfval,i_do,yesgrasp)

      if(yesgrasp) pop(nsol_rand)%x = xcm1(:)      ! migliore selezionato dalla lista
     enddo

! calcolo la FOM
   do i=1,ngen
      pop(i)%cfval = func(pop(i)%x,0)
   end do
!
   call indexx(pop%cfval,iord)    ! dal piu piccolo al piu grande
   pop(:) = pop(iord)
!
   xcm1=pop(1)%x           !  centro di massa della popolazione iniziale
   bestcfvalue=pop(1)%cfval
   par(:)%val = xcm1
!  xcm2(:) = xcm1(:)
   valdiff(:,:)=1.0
!
!  New population is generated
   ginfo = ginfo_ini
   call fcnprint(ginfo,ginfo%popt,xcm1,bestcfvalue,0.0,neval,0)   ! print best solution
   alpha_min = 0.0001
   alpha_max = 1.0
   sc = sc_ini
   coef = log(alpha_min/alpha_max) / cond%niter
   nacf = 0
   stopc = 1.0
   alpha = 1
!CE
   sabest=0
   sacount=0
   limitrmax = 10
   pop_sav(1)=pop(1)
   pop_savbest=pop(1)%cfval
   temppop(:3)=pop(:3)

   do it=1,cond%niter                  ! loop sul numero di iterazioni
      alpha = alpha_max * exp(coef*it)
      alphastar=alpha
      valdiff(:,:)=1.0
      newmaxr=maxr
! Big bang phase
      xc(:) = xcm1(:)
      itervalues(it)%valdiff(:,:)=1
      valuta=.false.
      call random_number(minrand)
      if (it<=2) then
        pop_sav(1)=pop(1)
        pop_sav(2)=pop(2)
        pop_sav(3)=pop(ngen)
      else
        valuta=.true.
        limitrmax=limitrmax-1
        ok=0
        call valutamassimo(par,npar,itervalues(it-1)%pop,itervalues(it-1)%valdiff,itervalues(it-2)%pop,ok,yesval)
        if (newmin) then       
! se newmin=true. Cioe' se passando da una pop. alla successiva ho ridotto l'Rwp di entrambi o uno solo dei due modelli migliori della popolazione 
!miglioro
            valdiff=itervalues(it-1)%valdiff
            pop_sav(1)=itervalues(it-1)%pop(1)
            pop_sav(2)=itervalues(it-1)%pop(2)
            pop_sav(3)=itervalues(it-1)%pop(ngen)
            struttura=.true.
        else
!miglioro
            struttura=.false.
            valdiff=itervalues(it-2)%valdiff
            pop_sav(1)=itervalues(it-2)%pop(1)
            pop_sav(2)=itervalues(it-2)%pop(2)
            pop_sav(3)=itervalues(it-2)%pop(ngen)
        endif
      endif
      nrmax=0
      start=1
      if (it>(cond%niter/4)) start=2
      pop(1)=pop_sav(1)

  do step=1,2
!definisco il c.m. intorno a cui generare
      nrmax=0
      do j=start,ngen    ! loop sulla popolazione. Start=1 oppure 2 da it>(cond%niter/4)-->lascio il migliore nella pop
         if (step==1) then
            call random_number(minrand)
            r=1+floor(newmaxr*minrand)
            if (valuta) then                        ! se valuta=true --> niter>2 

               if (nrmax>limitrmax .and. r>2) r=1   ! genera intorno a 1 da niter>=13 (limitrmax diventa =-1) e se r=3
               if (it>(cond%niter/4)) r=1           ! genera intorno a 1 dopo che ho lasciato il migliore nella popolazione

               if (r==maxr) then                    ! maxr=3
                  esplora=.true.   ! se r=3 usa alpha=1
               else
                  esplora=.false.  ! se r=/3 usa alpha=valdiff(r)
               endif
            endif
            xc=pop_sav(r)%x
         else
            xc=pop(1)%x
         endif
         struttura=.false.
         do k=1,npar      ! loop sui parametri della popolazione 
           if (.not.esplora) then     ! se esplora=F
                alpha=valdiff(r,k)    ! se r=/3
           else
               alpha=1
           endif
           call random_number(xrand) 
           if (step==1) then
                 val = xc(k) + (xrand*2 -1) * alpha
           else ! step==2
!genero variando x y z per il 50% della pop., angoli e torsioni per il restante 50%
             if (mod(floor(2*xrand),2) == 0) then
               if (k>=4) then 
                 val = xc(k) + (xrand*2 -1) * alphastar
               else
                 val = xc(k)
               endif
             else
               if (k<4) then
                 val = xc(k) + (xrand*2 -1) * alphastar
               else
                 val = xc(k)
               endif
             endif
           endif
!    
           rangev = par(k)%high - par(k)%low
          !if (par(k)%bound == 0) then     ! Fixed boundaries
          !     if(val < par(k)%low .or. val > par(k)%high) then
          !         call random_number(xrand)
          !         val = par(k)%low + rangev*xrand
          !     endif
          !else                            ! Periodic boundaries
                if(val < par(k)%low) val = val + rangev
                if(val > par(k)%high) val = val - rangev
          !endif
            if (step==1) then
               pop(j)%x(k) = val
            else
               temppop(j)%x(k) = val
            endif
         enddo       ! do su k, num di parametri

! Calcola funzione costo per ogni elemento della popolazione
      enddo    ! fine do su j, popolazione

      if (step==1) then
       do j=1,ngen
         pop(j)%cfval = func(pop(j)%x,0)
       end do
       call indexx(pop(:)%cfval,iord)
       pop(:) = pop(iord)
      else
       do j=1,ngen
         temppop(j)%cfval = func(temppop(j)%x,0)
       end do
       loc=minloc(temppop%cfval)
       if (temppop(loc(1))%cfval<=pop(1)%cfval) pop(1)=temppop(loc(1))
      endif
  enddo    ! fine do su step

!Big crunch phase

      call indexx(pop(:)%cfval,iord)
      pop(:) = pop(iord)

      nloc = 0
      nlocc = 0
      fmin = bestcfvalue           ! confronto SA con il minimo assoluto
      call avevar(pop(:)%cfval,cfave,cfvar)
      do j=1,ngen
         jord = ngen+1-j      ! solo su al max i 3 modelli peggiori
         nloc = nloc + 1
         if (nloc > cond%nlocmax) exit    ! esco da SA se ho considerato tutti gli cond%nlocmax
         if (jord < 0) cycle
! Applica un fast SA
         sapar(:) = par(:)
         sapar(:)%val = pop(jord)%x
         sacount=sacount+1
         sacount_tot=sacount_tot+1

         call sa(sapar,sc,ginfo,tfin,xopt,fopt,nacc,neval,nobdss,kevent,func,funcs,fmin)
         nlocc = nlocc + 1
         if (pop(iord(ngen))%cfval > fopt) sabest=sabest+1
         if (pop(iord(ngen))%cfval > fopt) sabest_tot=sabest_tot+1
         if (fopt < fmin) fmin = fopt
         if (kpr > 0) then
             write(6,'(a,i0,a,i0,a,i0,2x,f0.3,a,f0.3)')'n.',it,'/',cond%niter, &
                ' local search on pop. n.',jord,pop(jord)%cfval,'-->',fopt
         endif

         if(fopt<pop(jord)%cfval) then
            pop(jord)%x = xopt
            pop(jord)%cfval = fopt
         endif

         if (kevent == 1 .or. kevent == 2) exit    ! gestione eventi grafica
         if(fmin<=nint(0.15*cfave)) then      ! uscite *.out_1sa_alpha5
          if(kpr>0) write(6,'(a,2x,i0,2x,a,i0,2x,a,i4,2x,a,i4)')'STOP SA. niter',cond%niter,   &
                  'it= ',it,'sabest=',sabest,' sacount=',sacount
           exit
         endif
      enddo       ! ngen

      call indexx(pop(:)%cfval,iord)
      pop(:) = pop(iord)
      xcm1(:) = pop(1)%x  ! prende il mod con cfval minimo

      if (pop(1)%cfval<=(pop_sav(1)%cfval*1.0) .and. pop(1)%cfval<=(pop_savbest*1.5)) then
        newmin=.true.
        itervalues(it)%par=par
        itervalues(it)%pop=pop
        if (pop(1)%cfval<pop_savbest) pop_savbest=pop(1)%cfval
      else
        if (it>2) then
           newmin=.false.
           itervalues(it)%par=itervalues(it-1)%par
           itervalues(it)%pop=itervalues(it-1)%pop
        else
           newmin=.true.
           itervalues(it)%par=par
           itervalues(it)%pop=pop
        endif
      endif 
!
      cfxc = pop(1)%cfval
      neval = neval + 1
      bestfound = cfxc < bestcfvalue
      if (bestfound) then
          bestcfvalue = cfxc
          par(:)%val = xcm1(:)
          xcm2(:) = xcm1(:)
          call fcnprint(ginfo,ginfo_ini%popt,xcm1,bestcfvalue,0.0,neval,it)
          if (kpr > 0) then
              write(6,*)it,'cfval=',cfxc,' alpha=',alpha
          endif
      endif
! va normalizzata la popolazione per fare questo
! Check stopping criterion
      call avevar(pop(:)%cfval,cfave,cfvar)
      stopc = sqrt(cfvar)/cfave
      if (kpr > 0) then
          write(6,*)'Stopping criterion. Tol:',stopc
      endif
      if (stopc < cond%tol .or. kevent == 1 .or. kevent == 2) then
         if (kpr > 0) then
            write(6,'(a,f8.5,4(a,i8,2x))')'Stopping criterion verified. Tol:',stopc,    & 
                    ' neval=',neval,'it= ',it,'sabest=',sabest,' sacount=',sacount

         endif
          exit
      endif

   enddo
!!corr -fix this or remove, problem with mpi
!!corr   write(6,*)''
!!corr   write(6,'(10x,a,2x,i0,a,i0)') 'Num. of executed iterations =', it,'/',cond%niter
!!corr   write(6,'(10x,a,i4)') 'Num. of SA accesses =',sacount_tot
!!corr   write(6,'(10x,a,2x,f6.2)') '% of SA improvements =',(float(sabest_tot)/float(sacount_tot))*100.0

   bestcfvalue = func(par(:)%val,0)
!
   end subroutine bb_bc_algo

!-------------------------------------------------------------------------------

  !subroutine puteleminarray(pop, finalpop, x, newval, newcfval)
  !real, dimension(:), intent(in)   :: newval
  !real,intent(in)   ::newcfval
  !integer, intent(in)  :: x
  !integer, intent(in)    :: finalpop
  !type(pop_type), dimension(:),intent(inout) :: pop
  !integer :: j
  !logical :: found
  !real :: tempcfval
  !real, dimension(size(newval)):: tempval

  !found=.false.
! !write(6,*) 'TROVATO',found 
  !tempval=newval
  !tempcfval=newcfval
  ! 
! ! write(6,*) 'puteleminarray'
! ! write(6,*) pop%cfval
  !do j=finalpop,2,-1
! !       write(6,*) ' ',(j-1),' -> ',j,pop(j)%cfval, tempcfval
  !       if (pop(j-1)%cfval>=tempcfval) then
  !            pop(j)=pop(j-1)
  !       else
  !          found=.true.
  !          pop(j)%x=tempval
  !          pop(j)%cfval=tempcfval
! !           write(6,*) 'found' ,j
! !           write(6,*) pop%cfval
  !          return
  !       endif
  !enddo
! !write(6,*) 'TROVATO1',found 
  !if(.not.found) then            ! if .false.
  !   pop(1)%x=tempval
  !   pop(1)%cfval=tempcfval
  !endif
! !write(6,*) 'found' ,j
! !write(6,*) 'FINALI' 
! !write(6,*) pop%cfval
! !
  !end subroutine puteleminarray
!-------------------------------------------------------------------------------

  !real function pop_distance(pop1,pop2,par)  result(pdist)
  !type(pop_type), intent(in)   :: pop1,pop2
  !type(param_type), dimension(:),intent(in) :: par
  !real, dimension(size(pop1%x)) :: pdiff
  !integer                       :: nd
  !real                          :: dist
  !real                          :: cfdiff
  !integer                       :: i
  !integer                       :: npar
!
  !pdist = 1
  !!pdiff(:) = (pop1%x - pop2%x) / max(pop1%x,1.0)
  !!pdist = sqrt(dot_product(pdiff,pdiff))
  !npar = size(pop1%x)
  !nd = 0
  !do i=1,npar
  !   if (par(i)%typepar == 5) then
  !       nd = nd + 1
  !       pdiff(nd) = (pop1%x(i) - pop2%x(i)) / max(pop1%x(i),1.0)
  !   endif
  !enddo
  !if (nd > 0) then
  !    dist = sqrt(dot_product(pdiff(:nd),pdiff(:nd)))
  !else
  !    dist = 0.0
  !endif
  !cfdiff = abs(pop1%cfval - pop2%cfval)
  !    !write(0,*)'diff=',dist,cfdiff
  !if (dist < 0.1 .and. cfdiff < 0.5) then
  !    pdist = -1
  !endif
!
  !end function pop_distance

!-------------------------------------------------------------------------------

  !function centerpoint(pop,kmethod,np)  result(xc)
!
!  Trova il centro di massa della popolazione
!
  !type(pop_type), dimension(:), intent(in) :: pop
  !integer, intent(in)                      :: kmethod   
  !integer, intent(in) :: np
!corr   real, dimension(size(pop(1)%x))          :: xc
  !real, dimension(np) :: xc
  !integer                                  :: npop
  !integer, dimension(1)                    :: loc
  !real                                     :: num,den
  !integer                                  :: i,j


!
  !if (kmethod == 1) then
  !    !np = size(pop(1)%x)
  !    npop = size(pop)
  !    den = sum(1/pop(:)%cfval)
  !    do i=1,np
  !       num = 0
  !       do j=1,npop
  !          num = num + pop(j)%x(i) / pop(j)%cfval
  !       enddo
  !       xc(i) = num / den
  !    enddo
  !else
  !    loc = minloc(pop%cfval)
  !   !write(6,*)'MINIMO',loc
  !    i=loc(1)
  !    xc(:) = pop(i)%x
  !endif 
!
  !end function centerpoint

!--------------------------------------------------------------------------------------------------

   subroutine bb_bc_print_conditions(bcond,unt)
   type(bb_bc_condition), intent(in) :: bcond
   integer, intent(in)               :: unt
!
   write(unt,"(/8x,a)")'Global optimization by Hybrid Big Bang Big Crunch combined with GRASP (GHBB-BC)'
   write(unt,'(1x,a,i0)')  'Polulation size:              ',bcond%nump
   write(unt,'(1x,a,i0)')  'Number of iterations:         ',bcond%niter
   write(unt,'(1x,a,f0.4)')'Stopping criterion:           ',bcond%tol
   write(unt,'(1x,a,i0)')  'Number of local search:       ',bcond%nlocmax
!corr   write(unt,'(1x,a,i0)')'Control parameter alfa: ',bcond%beta
!
   end subroutine bb_bc_print_conditions

!--------------------------------------------------------------------------------------------------
  
 !subroutine newitempop(cond, par, xc, alpha, it, j)
 !    type(bb_bc_condition), intent(in)             :: cond
 !    real                                       :: xrand, rangev
 !    real, intent(in)                           :: alpha
 !    real                                       :: val
 !    type(param_type), dimension(:), intent(in) :: par
 !    real, dimension(size(par)),intent(in)      :: xc
 !    integer  :: it, k, j

 !    do k=1,size(par)

 !      call random_number(xrand)               ! da 0 a 1
!CROS
!           if(ginfo%dofi>10) then
 !          if(it<=(0.5*cond%niter)) then
 !             if(j<=nint(0.7*cond%nump)) then
 !                 val = xc(k) + (xrand*2 -1) * alpha      ! +-1
 !             else
 !                 val = xc(k) + (xrand*2 -1)
 !             endif
 !          else
 !             val = xc(k) + (xrand*2 -1) * alpha      ! +-1
 !          endif
!!          else
!!              val = xc(k) + (xrand*2 -1) * alpha      ! +-1
!!          endif
!CROS
 !         rangev = par(k)%high - par(k)%low
 !        !if (par(k)%bound == 0) then     ! Fixed boundaries
 !        !     if(val < par(k)%low .or. val > par(k)%high) then
 !        !         call random_number(xrand)
 !        !         val = par(k)%low + rangev*xrand
 !        !     endif
 !        !else                            ! Periodic boundaries
 !              if(val < par(k)%low) val = val + rangev
 !              if(val > par(k)%high) val = val - rangev
 !        !endif
 !     end do
 !end subroutine

!--------------------

 subroutine valutamassimo(par, np, last, val, xold,it,yesval)
     integer, intent(in) :: np
     type(param_type), dimension(:), intent(inout) :: par
     type(pop_type), dimension(:)  :: last, xold
     real, dimension(maxr,np), intent(inout) :: val
     real    :: rangei, tempdiff
     integer :: i, j, it
     logical :: yesval

   if(abs(last(1)%cfval-xold(1)%cfval)<=0.05.and.abs(last(1)%cfval-xold(1)%cfval)/=0.0000) then 
      yesval=.false.
   endif
   do j=1,maxr-1
    if (last(j)%cfval<xold(j)%cfval) then
      do i=1,np
        rangei = par(i)%high - par(i)%low
        tempdiff = abs(last(j)%x(i)-xold(j)%x(i))
        val(j,i) = 0.2*tempdiff + tempdiff/abs(rangei) 
      end do
    else
       val(j,:)=1
      !val(j,:)=0
    endif
   end do

   if (it==1) then
!   do j=size(last)-1,size(last)
!    it=j - size(last)+maxr
!    if (last(it)%cfval<xold(it)%cfval) then
!    do i=1,np
!       rangei = par(i)%high - par(i)%low
!       val(it,i)=abs(last(it)%x(i)-xold(it)%x(i))/abs(rangei)
!    end do
!    endif
!enddo
    it=size(last)
    it=maxr
!  if (last(it)%cfval<xold(it)%cfval) then
!   do i=1,np
!      rangei = par(i)%high - par(i)%low
!       val(it,i)=abs(last(it)%x(i)-xold(it)%x(i))/abs(rangei)
!    end do
!    endif
!    do i=1,np
!       rangei = par(i)%high - par(i)%low
       val(it,:)=1
!   end do

  end if
  end subroutine valutamassimo


!--------------------------------------------------------------------------------------------------
   subroutine reallocate_pop(vetr,n,savevet)
!
!  Rialloca ad n un vettore di tipo model
!  Se savevet = .true. o non esiste si salva il suo contenuto.
!
   type(pop_type), allocatable, intent(inout) :: vetr(:)
   integer, intent(in)                     :: n
   logical, optional, intent(in)           :: savevet
   logical                                 :: savev
   integer                                 :: nv
   type(pop_type), allocatable                :: vsav(:)
   integer                                 :: nsav
!  se n = 0 (riallocazione a 0): dealloca ed esci
   if (n == 0) then
       if (allocated(vetr)) deallocate(vetr)
       return
   endif
   if (.not.allocated(vetr)) then
       allocate(vetr(n))
   else
       nv = size(vetr)
       if (present(savevet)) then
           savev = savevet
       else
           savev = .true.
       endif
       if (savev) then
!          nsav contiene qual è la porzione di vetr da salvare
           select case(nv-n)
             case (1:)       ! compatta x ad n
               nsav = n
             case (:-1)      ! espandi x ad n
               nsav = nv
             case (0)
               return        ! n=nv non fare niente
           end select
!
!          salva vetr fino a nsav
           allocate(vsav(nsav))
           vsav = vetr(:nsav)
!
!          dealloca vetr
           deallocate(vetr)
!
!          rialloca vetr su n
           allocate(vetr(n))
!
!          ripristina vetr fino a nsav
           vetr(:nsav) = vsav(:nsav)
       else
           if (nv /= n) then
               deallocate(vetr)
               allocate(vetr(n))
           endif
       endif
   endif
!
   end subroutine reallocate_pop

!--------------------------------------------------------------------------------------------------
   subroutine reallocate_pop_bb(vetr,n,npar)
!
!  Rialloca ad n un vettore di tipo model
!  Se savevet = .true. o non esiste si salva il suo contenuto.
!
   type(pop_type), allocatable, intent(inout)    :: vetr(:)
   integer, intent(in)                           :: n
   integer                                       :: npar
   integer                                       :: i

!  se n = 0 (riallocazione a 0): dealloca ed esci
   if (n == 0) then
       if (allocated(vetr)) deallocate(vetr)
       return
   endif
   if (.not.allocated(vetr)) then
       allocate(vetr(n))
       do i=1,n
          allocate(vetr(i)%x(npar))
       enddo
   endif
!
   end subroutine reallocate_pop_bb
!--------------------------------------------------------------------------------------------------

  !subroutine init_random_seed()

  !   INTEGER :: i, n, clock
  !   INTEGER, DIMENSION(:), ALLOCATABLE :: seed

  !   CALL RANDOM_SEED(size = n)
  !   ALLOCATE(seed(n))

  !   CALL SYSTEM_CLOCK(COUNT=clock)

  !   seed = clock + 37 * (/ (i - 1, i = 1, n) /)
  !   CALL RANDOM_SEED(PUT = seed)

  !   DEALLOCATE(seed)
  !end subroutine

!--------------------------------------------------------------------------------------------------
   subroutine grasp(npar,par,xcm1,func,ginfo,cfval,i_do,yesgrasp)

   USE nr
   USE arrayutil

   INTERFACE
!     Function per il calcolo della funzione costo
      real function func(xpar,hpar)
      real, intent(in), dimension(:) :: xpar
      integer, intent(in)            :: hpar
      end function func
   END INTERFACE

   type(opti_info_type)                                           :: ginfo
   type(param_type), dimension(:), intent(inout)                  :: par        !parametri

   real, dimension(10)                                             :: prob,interval,somma,average,num  !,num1 
   real, dimension(size(par))                                     :: xcm1,xcm2!!!!,xcm3 
   real, dimension(nint(1.5*size(par)))                           :: bias,prob_bias

   integer, dimension(nint(1.5*size(par)),size(par))              :: xcm1_par_var  ! parametri variati per quel cm1
   integer, dimension(nint(1.5*size(par)))                        :: lista_RCL,cm1_sav,npar_mod
   integer, dimension(1)                                          :: loc
   integer, dimension(10)                                          :: nvol,nvol_app

   integer                                                        :: npar,npar_min,npar_max,cm1_gen  !!!,npar_mod
   integer                                                        :: i,ii,j,jj,k      !,l,kkk,jjj
   integer                                                        :: nsol_rand
   integer                                                        :: np,pval,nval,ppval
   integer                                                        :: nnpar
   integer                                                        :: i_do,num_do,n_iter_prob

   real                                                           :: somma_bias
   real                                                           :: xrand,alpha
   real                                                           :: rangev,kk,den
   real                                                           :: val,cfval,cfval_min_iniz,cfval_min_fin,cfval_minass

   logical                                                        :: yesgrasp
   
   kk=0.1
   nval=10
   prob(:)=1.0/nval
   do i=1,nval
      interval(i)=kk   ! valori da assegnare ad alpha=0.1,0.2, ...0.9
      kk=kk+0.1
   end do

   somma_bias=0.0

   num(:)=0
   somma(:)=0.0
   nvol(:)=0
   nvol_app(:)=0
   average(:)=0.0
   npar_mod(:)=0

   den=0.0
   np=0
   ppval=0

   n_iter_prob=nint(i_do/3.0)

   xcm2(:)=xcm1(:)
  !xcm3(:)=xcm1(:)
   cfval_min_iniz=func(xcm1,0)
   cfval_min_fin=cfval_min_iniz
   cfval_minass=cfval_min_iniz

   if (ginfo%res<2.0) then
      cm1_gen=nint(1.2*npar)  ! numero di Centri di Massa da generare
   else
      cm1_gen=nint(1.5*npar)  ! numero di Centri di Massa da generare
   endif

   if (.not.alloc) then
    alloc=.true.
    call reallocate_pop_bb(xcm1_gen,cm1_gen,npar)
   endif
!genero cm1_gen elementi a partire da cm1. Alcuni parametri di questi cm1_gen verranno poi modificati random
  do k=1,i_do      ! numero di cicli GRASP
   if(k<=n_iter_prob) then
!valori di alpha campionati tutti con la stessa probabiltita' per i primi n_iter_prob cicli
      call random_number(xrand)
      ppval=1+mod(nint(xrand*nval),nval)
      nvol(ppval)=nvol(ppval)+1
      nvol_app(ppval)=nvol_app(ppval)+1
      alpha=interval(ppval)
   else
!campionati con una probabiltita' che dipende da prob
      call random_number(xrand)
      ppval=1+mod(nint(xrand*nval),nval)
      if(prob(ppval)<(maxval(prob(:))*0.6).and.mod(k,8)/=0) then
         cycle
      else
!prendo tutte le prob. alte e tutte le eventuali prob. basse se il resto della divisione di k
!per 8 e' = 0 --> ogni 8 controllo
         nvol(ppval)=nvol(ppval)+1
         nvol_app(ppval)=nvol_app(ppval)+1
         alpha=interval(ppval)
      endif
   endif

   if(sum(nvol_app(:))==0) alpha=0.2   ! quasi pure greedy. Se non ho selezionato nessun valore per alpha
   jj=0
   cm1_sav(:)=0

!copio xcm1 in cm1_gen elementi della popolazione
   do i=1,cm1_gen
      xcm1_gen(i)%x(:)=xcm1(:)
   end do

!genero un numero di cm1 variando in modo random i parametri
   xcm1_par_var(1:cm1_gen,1:npar) = 0
   do i=1,cm1_gen
     call random_number(xrand)
! genero numero intero tra 1 e npar
     np=1+mod(int(xrand*npar),npar)                ! num di parametri da variare 

     do j=1,np
       call random_number(xrand)
       pval=1+mod(int(xrand*npar),npar)             ! parametro da variare

       if(xcm1_par_var(i,pval)==1) cycle            ! ciclo se ho gia variato il parametro 

       val=xcm1(pval)+(xrand*2 -1)                  ! tra +-1

!controllo che i nuovi valori siano nel range
       rangev=par(pval)%high-par(pval)%low
      !if(par(pval)%bound==0) then           ! Fixed boundaries
      !  if(val<par(pval)%low.or.val>par(pval)%high) then
      !     call random_number(xrand)
      !     val=par(pval)%low+rangev*xrand
      !  endif
      !else                               ! Periodic boundaries
         if(val<par(pval)%low)  val=val+rangev
         if(val>par(pval)%high) val=val-rangev
      !endif
       xcm1_gen(i)%x(pval) = val     ! nuovo valore del parametro
       xcm1_par_var(i,pval) = 1      ! parametro variato = 1. Creo una tabella con 0 e 1 
     end do
        xcm1_gen(i)%cfval=func(xcm1_gen(i)%x,0)
   end do

   cfval=minval(xcm1_gen(:)%cfval)
   if(cfval<cfval_minass) then
      cfval_minass=cfval
     !loc=minloc(xcm1_gen(:)%cfval)
     !xcm3(:) = xcm1_gen(loc(1))%x         ! minimo assoluto
   endif

!inizio procedura GRASP
   num_do=0
   do 
   num_do=num_do+1

    if(sum(xcm1_par_var(1:cm1_gen,1:npar))==0) exit   ! esco se li ho considerati tutti
    do i=1,cm1_gen
      npar_mod(i) = sum(xcm1_par_var(i,1:npar))       ! num. totale di parametri modificati per ogni elemento
    end do
    npar_min=minval(npar_mod,mask=npar_mod>0)
    npar_max=maxval(npar_mod)

    lista_RCL(:)=0
    bias(:)=0.0
    prob_bias(:)=0.0
    ii=0

! definisco la soglia per mettere le soluzioni in RCL
    nnpar=nint(npar_min+alpha*(npar_max-npar_min))

    do i=1,cm1_gen
       if(npar_mod(i)==0) cycle
        if(npar_mod(i)>=npar_min.and.npar_mod(i)<=nnpar) then
          ii=ii+1
          lista_RCL(ii)=i             !riempio RCL
       endif
    end do

! su lista_RCL
    if(ii==0) exit
    jj=jj+1

    loc = minloc(xcm1_gen(lista_RCL(1:ii))%cfval)
    nsol_rand=loc(1)

    cm1_sav(jj)=lista_RCL(nsol_rand)         ! lista dei cm1 salvati e selezionati random da lista_RCL

!elimino da tutti i cm1 generati il parametro variato
    do i=1,npar
       if(xcm1_par_var(lista_RCL(nsol_rand),i)==1) xcm1_par_var(:,i)=0
    end do
   end do     ! fine do del GRASP

!affinamento locale sui modelli salvati
    call mod_pop(jj,npar,par,xcm1_gen,cm1_sav,func)

!trovo il minimo tra i cm1 salvati e selezionati random da lista_RCL ed eventualmente affinati
    cfval=minval(xcm1_gen(cm1_sav(1:jj))%cfval)
    if(cfval<cfval_min_fin) then
       cfval_min_fin=cfval
       loc=minloc(xcm1_gen(cm1_sav(1:jj))%cfval)
!salvo in uscita quello con cfval minimo tra i cm1 salvati e selezionati random da lista_RCL
       xcm2(:) = xcm1_gen(cm1_sav(loc(1)))%x
    endif

!Reactive GRASP
!somma di tutti i valori della FOM trovata con quel valore di alpha

    if(sum(nvol_app(:))/=0) somma(ppval)=somma(ppval)+minval(xcm1_gen(cm1_sav(1:jj))%cfval)
      
!mod(1,10)=1; mod(2,10)=2;...;mod(10,10)=0;mod(11,10)=1;mod(12,10)=2;...;mod(20,10)=0 
    if(mod(k,n_iter_prob)==0) then    ! rivalutata periodicamente
      do i=1,nval
         if(nvol(i)==0) cycle
         average(i)=somma(i)/nvol(i)        ! Ai
         num(i)=(cfval_minass/average(i))   ! qi
      end do
      den=sum(num)

      do i=1,nval
         if(nvol(i)==0) cycle
         prob(i)=num(i)/den
      end do
    endif

    nvol_app(:)=0
  end do   ! fine do su k=n. di cicli GRASP

    cfval=func(xcm2,0)
    if(cfval<cfval_min_iniz) then
       xcm1(:) = xcm2(:)
    else
       cfval= cfval_min_iniz
       yesgrasp=.false.
    endif

   end subroutine grasp

!----------------------------------------------------------------------------------

   subroutine mod_pop(jj,npar,par,xcm1_gen,cm1_sav,func)
!subroutine per la modifica della popolazione. Cerco nell'intorno di ogni elemento
!della popolazione se c'e' qualcosa di meglio, e nel caso lo sostituisco al suo
!corrispondente. 

   INTERFACE
!   Function per il calcolo della funzione costo
    real function func(xpar,hpar)
     real, intent(in), dimension(:) :: xpar
     integer, intent(in)            :: hpar
    end function func
   END INTERFACE
   
   type(pop_type), dimension(:), allocatable                      :: xcm1_gen,pop_app,pop_app1
   type(param_type), dimension(:), intent(inout)                  :: par        !parametri
   integer, dimension(nint(1.5*size(par)))                        :: cm1_sav

   integer                                                        :: i,j,k,jj
   integer                                                        :: npar

   real, dimension(size(par))                                     :: xc
   real                                                           :: xrand,val,rangev
   real                                                           :: cfval_min
   real                                                           :: somma_med_i,somma_med_f
   real                                                           :: med_i,med_f

   logical                                                        :: yesmod = .false.

    allocate(pop_app1(jj))
    allocate(pop_app(jj))
    do i=1,jj
      allocate(pop_app1(i)%x(npar))
      allocate(pop_app(i)%x(npar))
    end do

   somma_med_i=sum(xcm1_gen(cm1_sav(1:jj))%cfval)

   do i=1,jj
     pop_app(i)=xcm1_gen(cm1_sav(i))
     xc=xcm1_gen(cm1_sav(i))%x
     cfval_min=xcm1_gen(cm1_sav(i))%cfval

     do j=1,5 
      do k=1,npar
        call random_number(xrand)
        val = xc(k) + (xrand*2 -1)*0.05
        rangev = par(k)%high - par(k)%low
       !if (par(k)%bound == 0) then     ! Fixed boundaries
       !  if(val < par(k)%low .or. val > par(k)%high) then
       !     call random_number(xrand)
       !     val = par(k)%low + rangev*xrand
       !  endif
       !else                            ! Periodic boundaries
          if(val < par(k)%low) val = val + rangev
          if(val > par(k)%high) val = val - rangev
       !endif
        pop_app1(i)%x(k) = val
      end do
      pop_app1(i)%cfval = func(pop_app1(i)%x,0)

      if(pop_app1(i)%cfval<(0.50*cfval_min)) then
       yesmod=.true.
       cfval_min=pop_app1(i)%cfval
       pop_app(i)%x=pop_app1(i)%x
       pop_app(i)%cfval = func(pop_app(i)%x,0)
      endif
     end do     ! fine do su j
   end do       ! fine do su i

   if(yesmod) then
    somma_med_f=sum(pop_app(1:jj)%cfval)
    med_i=nint(somma_med_i/jj)
    med_f=nint(somma_med_f/jj)

!Salvo la popolazione solo se la media sui jj dopo la mod_pop e' piu' bassa 
    if(med_f<=med_i) xcm1_gen(cm1_sav(1:jj))=pop_app(1:jj)
   endif
   
   end subroutine mod_pop
!------------------------------------------------------------------------------------------------
   subroutine AllocaBB_BC()
   USE arrayutil

    call reallocate_pop_bb(pop,0,0)
    call reallocate_pop_bb(pop_sav,0,0)
    call reallocate_pop_bb(temppop,0,0)
    call reallocate_pop_bb(xcm1_gen,0,0)
    call reallocate_pop_bb(pop_app,0,0)
    call resize_array(iord,0)   
   end subroutine AllocaBB_BC
END MODULE bb_bc
