#ifndef __CRY_H
#define __CRY_H
using namespace std;

#define PEARSON7  1
#define PVOIG     2
#define TCHZ      3

typedef struct  {
     int          ftype;
     r_param_type par[10];
                } profile_function_t;

typedef struct {
                  char           cname[300];
                  float          cell[6];
                  int            cellcod[6];
                  char           spgname[16];
                  float          scal;
                  int            scalcod;
                  int            kopcorr;
                  int            hklpo[3];
                  int            gcode;
                  float          gpar;
                  int            riding_model;
                  float          adp_factor;
                } crystal_t;

typedef struct {
                  float           zero;
                  float           sdisp;
                  float           stran;
                  int             zcode,sdcode,stcode;
                  int             bcode;
               } dataset_t;

typedef struct {
     int     code;   // codice identificativo del tipo di
     int     *vpar;  // i parametri coinvolti
     int     *vat;   // gli atomi coinvolti
     float   *coef;  // coefficienti
     float   sumc;   // known term in constraint equation
     int     npar;   // numero totale di parametri coinvolti
     int     npari;  // numero di parametri indipendenti
     int     ref;    // codice di affinamento
     int     typec;  // settato da 1=user 2=programma
               } constraint_type;

typedef struct {
    int ph;          // fase cristallina 
    int katom;       // atomo 
    int kpar;        // parametro
               } constraint_ref;


class Crystal
{
public:
   void FillCrystal(int nats, int nelems, crystal_t c, atom_type *atm, element_type *el);
   void PrintCrystal();
   string    GetName()                  {return cname->substr(0, 16);}
   string    GetSpg()                   {return trim(*spgname);}
   float     GetCell(int n)             {return cell[n];}
   int       GetCellCod(int n)          {return cellcod[n];}
   float     *Cell()                    {return cell;}
   int       *CellCode()                {return cellcod;}
   int       GetNats()                  {return natoms;}
   atom_type GetAtom(int n)             {return at[n];}
   int       SC()                       {return scalcod;}
   float     GetScale()                 {return scal;}
   int       GetNelems()                {return nelems;}
   string    GetElemName(int n);
   string    GetAtomName1(int n);
   int       GetElemZ(int n)            {return elem[n].z;}
   int       GetElemptab(int n)         {return elem[n].ptab;}
   void      SetScal(float val)         {scal = val;}
   void      SetCell(int n, float val)  {cell[n] = val;}
   void      SetCellCod(int n, int val) {cellcod[n] = val;}
   void      SetScalCod(int val)        {scalcod = val;}
   void      SetXC(float v,int a,int i) {at[a].xc[i] = v;}
   void      SetBiso(float val, int atm){at[atm].biso = val;}
   void      SetOch(float val, int atm) {at[atm].och = val;}
   void      SetRcod(int v,int a,int i) {at[a].rcod[i] = v;}
   atom_type *GetAtoms()                {return at;}
   int       HKLpo(int n)               {return hklpo[n];}
   int       *HKLpoV()                   {return hklpo;}
   int       Kopcorr()                  {return kopcorr;}
   int       Gcod()                     {return gcode;}
   float     Gpar()                     {return gpar;}
   void      SetHKLpo(int n, int val)   {hklpo[n] = val;}
   void      SetGpar(float val)         {gpar = val;}
   void      SetGcod(int val)           {gcode = val;}
   void      SetKopcorr(int val)        {kopcorr = val;}
   int       RidingModel()              {return riding_model;}
   float     ADP()                      {return adp_factor;}
   void      SetRM(int val)             {riding_model = val;}
   void      SetADP(float val)          {adp_factor = val;}
   void      add_constraint(int code, int npar, int ref, int vpar[], int vat[],
             float coef[], float sumc, int i, int typec, int npari);
   int       par_in_constraint(int kat, int npar);
   void      del_constraint(int i);
   int       Nconstraints()             {return nconstr;}
   constraint_type Constr(int i)        {return vconstr[i];}
   void      free_constraints();
   void      SetNconstr(int nc);
   void      IncNconstr()               {nconstr++;}
   void      DecNconstr()               {nconstr--;}
   void      SetRefine(int kc, int val) {vconstr[kc].ref = val;}
   bool      Reallocconstr();
   int       *dispo;
   int       *selected_r;
   int       *ind_Con;
private:
   int            natoms;
   int            nelems;
   string         *cname;
   float          cell[6];
   int            cellcod[6];
   string         *spgname;
   float          scal;
   int            scalcod;
   element_type   *elem;
   atom_type      *at;
   int            kopcorr;
   int            hklpo[3];
   int            gcode;
   float          gpar;
   int            riding_model;
   float          adp_factor;
   int            nconstr;
   constraint_type *vconstr;
};

class Dataset
{
public:
   float           zero;
   float           sdisp;
   float           stran;
   int             zcode,sdcode,stcode;
   int             bcode;
   int             *typefun;
   profile_function_t *pear;
   profile_function_t *pvoi;
   profile_function_t *tch;
};

#endif
