module datautil

implicit none

contains

   subroutine load_datafile(this,filename,gui,ier,filetyp,sync)
!
!  Load dataset from external file
!
   USE prog_constants
   USE filereading
   USE fileutil
   USE counts
   USE elements, only: RX_SOURCE,NEUTRON_SOURCE
   USE datasetmod
   class(dataset_type), intent(inout)          :: this
   character(len=*), intent(in)                :: filename
   logical, intent(in)                         :: gui
   integer, intent(out)                        :: ier
   integer, intent(in), optional               :: filetyp
   logical, intent(in), optional               :: sync
   integer                                     :: count_type
   integer                                     :: ftype
   real                                        :: thmino, thstepo, thmaxo   !, wavelen
   integer                                     :: ncount
   type(point_type), allocatable, dimension(:) :: countp
   type(error_type)                            :: error
   integer                                     :: irad,nwave
   real, dimension(3)                          :: wave,ratio
!
   if (present(filetyp)) then
       ftype = filetyp
   else
       ftype = filetype_from_filename(filename)
   endif
   call read_datafile(filename,ftype,thmino,    &
        thstepo,thmaxo,countp,ncount,.false.,nwave,wave,ratio,count_type,error)
   if (error%signal) then
       call error%print()
       ier = 1
   else
       ier = 0
       this%radtype = RX_SOURCE
       if (present(sync)) then
           this%sync = sync
       else
           this%sync = .false.
       endif
       this%tstep = thstepo
       this%tmin = thmino
       this%tmax = thmaxo
!
!      Set wavelength 
       if (this%nwave == 0) then     ! wavelenght is not still defined
           if (nwave == 0) then      ! wavelength is not in data file
               if (gui) then
                   wave(1:2) = [DEF_WAVE,DEF_WAVE2]
                   ratio(1:2) = [1.0,0.5]
                   call openwave(irad,nwave,wave,ratio)
                   select case (irad)
                     case (2)
                       this%sync = .true.
                     case (3)
                       this%radtype = NEUTRON_SOURCE
                   end select
               endif
           endif
           call this%set_wave(nwave,wave,ratio)
       endif
!
       if (count_type == COUNT_D_TYPE) countp = thvalue(countp,this%wave(1))
       call this%set_data(countp(:)%x,countp(:)%y,size(countp),filename,POW_DATA)
   endif
   end subroutine load_datafile

end module datautil
