MODULE drawatoms

implicit none

CONTAINS 

   subroutine draw_atoms(atom,legm,icon)
!
!  Interfaccia per passare da JAV alla vecchia grafica e viceversa
!
   USE Atomi
   USE atom_type_util
   USE ilmviewmod
   USE unit_cell
   USE General, only: cell,spacegr,celem
   USE connect_mod
   type(atom_type), dimension(:), allocatable, intent(in)           :: atom
   type(bond_type), dimension(:), allocatable, intent(in), optional :: legm    
   integer, dimension(:), intent(in), optional                      :: icon
   type(bond_type), dimension(:), allocatable                       :: legmi
   type(cell_type)                                                  :: cellt
!
   if (present(icon)) then
       cellt = set_cell_type(cell)
       call iconn_to_legm(icon,legmi,atom,cellt%get_g())
       call ilmview_draw(atom,legmi,celem,cellt,spacegr)
   else
       call ilmview_draw(atom,legm,celem,cellt,spacegr)
   endif
!
   end subroutine draw_atoms

!-----------------------------------------------------------------------

   subroutine draw_xyzs(conn,icon)
   USE Atomi
   USE ilmviewmod
   USE atom_type_util
   USE molpnew
   USE variables, only:lconn
   USE General, only: cell,spacegr,celem
   USE unit_cell
   USE connect_mod
   logical, optional, intent(in)               :: conn
   integer, dimension(:), intent(in), optional :: icon
   type(atom_type), dimension(:), allocatable  :: atom
   type(cell_type)                             :: cellt
!
   call xyz_to_atom(atom,xyzs(:3,:nats),bfac,occr=xyzs(5,:),occh=occpch,nz=nz,lab=aLab,elem=celem)
   if (present(icon)) then
       cellt = set_cell_type(cell)
       call iconn_to_legm(icon,lconn,atom,cellt%get_g())
   else
       if (present(conn)) then
           if (conn) then
               call create_connectivity(atom,lconn,set_cell_type(cell),spacegr)
           endif
       endif
   endif
   call ilmview_draw(atom,lconn,celem,cellt,spacegr)
!
   end subroutine draw_xyzs

!-------------------------------------------------------------------------------------------------
!corr
!corr   subroutine importa0_mpi(filename,ier,kadd)
!corr   USE mpi
!corr   character(len=*), intent(in) :: filename
!corr   integer, intent(out)         :: ier
!corr   logical, intent(in)          :: kadd
!corr   integer                      :: proc_id, mpi_err, np
!corr   integer                      :: file_free
!corr   integer                      :: status(MPI_STATUS_SIZE)
!corr
!corr   call MPI_Comm_size(MPI_COMM_WORLD, np, mpi_err)
!corr   call MPI_Comm_rank(MPI_COMM_WORLD, proc_id, mpi_err)
!corr
!corr   if (proc_id == 0) then
!corr       file_free = 1
!corr   else
!corr       call MPI_Recv(file_free,1,MPI_INTEGER, proc_id-1,1,MPI_COMM_WORLD,status,mpi_err)
!corr   endif
!corr   if (file_free == 1) call importa0(filename,ier,kadd)
!corr   if (proc_id /= np-1) call MPI_Send(file_free,1,MPI_INTEGER,proc_id+1,1,MPI_COMM_WORLD,mpi_err)
!corr
!corr   end subroutine importa0_mpi
!corr
!-------------------------------------------------------------------------------------------------

   subroutine importa0(strdir,atom,legm,ier,kadd)
!
!  Import model parsing string from command '%fragment'
!
   USE errormod
   USE general, only:StructureName,celem,cell,spacegr,neutro, ifAutomatic
   USE molcom,  only:jscreen
   USE importexport
   USE connect_mod
   USE ilmviewmod
   USE exportfiles
   USE variables
   USE atom_type_util
   USE gen_frm
   USE unit_cell
   USE elements
   USE proginterface
   USE sinterface
   USE elements
!for ochemdb
      !use ochemdb
!end for ochemdb
   character(len=*), intent(in)                              :: strdir
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   integer, intent(out)                                      :: ier
   logical, intent(in)                                       :: kadd
   type(error_type)                                          :: err
   type(cell_type)                                           :: cellt
   logical                                                   :: gui,has_symmetry
!
   ier = 0
   cellt = set_cell_type(cell)
   gui = (jscreen > 0 .and. ifAutomatic == 0)
   if (kadd) then
       has_symmetry = numatoms(atm) > 0
   else
       has_symmetry = is_cell_defined_list() ! in 'import' check for cell in list
   endif
   call import_fragment(strdir,atom,legm,cellt,spacegr,celem,neutro,err,kadd,gui,has_symmetry)
!
   if (jscreen > 0) call ilmview_open(StructureName) ! this open empty windows in case of error
   if (.not.err%signal) then
       call action_after_import(strdir,has_symmetry,celem,cellt,spacegr)
       call update_model_and_export(atom,legm,1,1)
       if(jscreen>0) then
          call ilmview_draw(atom,legm,celem,cellt,spacegr)
       endif
!!!for ochemdb
       !call export_for_ochemdb(atom,legm,celem,cellt,spacegr,structurename,expand=.false.,ftype='DAT',lcharge=.false.,kpr=-1)
!!!end for ochemdb
   else
       if (jscreen == 0) ier = -1
   endif
!
   end subroutine importa0

END MODULE drawatoms
