   MODULE GREXPOMOD
!-----------------------------------------------------------------------
!     List of subroutines (S) & functions (F) in this module
!-----------------------------------------------------------------------
!                 -- Funzioni primitive per la grafica --
! S gr_open_viewer(xlarg,yalt,string)            : apre gtk-viewer e inizializza variabili
! S gr_close_viewer()                            : chiude gtk-viewer
! S gr_select_viewer(nv)                         : attiva il viewer per disegnarci
! S gr_area(xleft,xright,ybottom,ytop)           : selezione area grafica
! S gr_setwin(xmin,xmax,ymin,ymax)               : min e max nell' area grafica
! S gr_sfs(fs)                                   : set fill-area style
! S gr_rect(xleft,xright,ybottom,ytop)           : disegna un rettangolo
! S gr_drawmarkers(n,xp,yp,symbol)               : draw markers
! S gr_draw1marker(xp,yp,symbol)                 : draw one marker
! S gr_setcolor(icol)                            : set color
! S gr_clear_all_area()                          : clear all area
! S gr_clear_area(xleft,xright,ybottom,ytop)     : clear selected area
! S gr_setbackground(icol)                       : set background color
! S gr_setsize(size)                             : set size marker
! S gr_drawline(n,xp,yp)                         : draw polyline
! S gr_box(xopt,xtick,nxsub,yopt,ytick,nysub)    : draw axis, labels, ticks
! S gr_setlinestyle(style)                       : set line style
! S gr_line(x1,y1,x2,y2)                         : draw line
! S gr_drawtext(xp,yp,just,stringa,fsize)        : write text
! S gr_reldrawtext(side,disp,coord,just,stringa,fsize) 
!                                                : write text at position relative to viewport
! S gr_hist(data,nbin)                           : draw histogram
!-------------------------------------------------------------------------------------
!                 -- Funzioni per disegnare grafici --
! S gr_create_graph(graph)                       : crea un grafico
! S gr_init_graph(graph)                         : inizializza grafico
! S gr_set_graph(graph,...)                      : modifica le caratteristiche del grafico
! S gr_draw_graph(graph,plot)                    : disegna un grafico   
! S gr_new_plot(plot,initsize)                : crea un plot
! S gr_init_plot(plot)                           : inizializza plot
! S gr_set_plot(plot,...)                        : modifica le caratteristiche del plot
! S gr_add_point_plot(plot,xval,yval)            : aggiungi un punto al plot
! S gr_add_points_plot(plot,xval,yval)           : aggiungi punti al plot
! S gr_update_points(plot,xval,yval)             : aggiorna i punti del plot
! S gr_draw_plot(plot,scale)                     : disegna plot
!-------------------------------------------------------------------------------------
   USE pointmod, only: point_type
    
    implicit none
    
    private :: gr_init_graph, gr_init_plot    
    
    integer, parameter          :: nmaxview = 2             ! massimo num. di viewer gestibili
    integer                     :: iselview = 0             ! viewer selezionato
    real, dimension(0:nmaxview) :: klargv                   ! larghezza viewer
    real, dimension(0:nmaxview) :: kaltv                    ! altezza viewer
    real, dimension(0:nmaxview) :: scalex, scaley           ! scala
    real, dimension(0:nmaxview) :: dispx, dispy             ! spostamento
    real, dimension(0:nmaxview) :: scalex0, scaley0         ! scala per max=1 e min=0
    real, dimension(0:nmaxview) :: dispx0, dispy0           ! spostamento per max=1 e min=0
    real                        :: xgrinf,xgrsup            ! min e max asse x
    real                        :: ygrinf,ygrsup            ! min e max asse y
    integer                     :: gr_color                 ! colore attivo (0-10)
    integer                     :: gr_color1                ! colore di riempimento
    integer                     :: gr_size                  ! size
    integer                     :: gr_colbackg              ! colore di background
    logical                     :: openviewer = .false.     ! 0/1 = viewer chiuso/aperto
    logical                     :: newviewer = .true.       ! nuovo viewer?
    integer                     :: xresize,yresize          ! dimensioni del nuovo viewer
    logical                     :: fortran_work = .false.

    integer, parameter   :: SOLIDLINE = 1
    integer, parameter   :: DASHLINE = 2
    integer, parameter   :: TICKLINE = 3
    integer, parameter   :: LINEWIDTH_DEF = 0
    integer              :: lstyle=SOLIDLINE                ! line style (1-2)
    integer              :: lwidth=LINEWIDTH_DEF            

    type plot_type
      type(point_type), dimension(:), allocatable :: point         ! i punti del plot
      integer                                     :: npoints       ! num. di punti 
      integer                                     :: initsize      ! valore iniziale di allocazione per points
      logical                                     :: existline     ! plot con linee? 
      logical                                     :: existmarkers  ! plot con markers?
      integer                                     :: colorline     ! colore delle linee
      integer                                     :: colormarkers  ! colore dei markers
      integer                                     :: n1,n2         ! puntatore al primo e ultimo punto da plottare
    end type

    type text_type
      character(len=100) :: str
      character(len=1)   :: side
      real               :: disp
      real               :: coord
      real               :: just
    end type text_type

    type graph_type
      character(len=30)             :: stringx,stringy    ! stringhe per asse X e Y
      integer                       :: color              ! colore assi
      integer                       :: xscale             ! <= 0 non riscala; > 0: riscala se range X > scale_number
      real                          :: zymin
      real                          :: zymax
      real                          :: zxmin
      real                          :: zxmax
      logical                       :: zoom 
      logical                       :: y_resize
      real                          :: y_resize_val
      integer                       :: ntext
      type(text_type), dimension(2) :: text    ! testo nel grafico
      real                          :: spacex  ! spaziatura agli estremi sull'asse x
      real                          :: spacey  ! spaziatura agli estremi sull'asse y
      character(len=6)              :: stk     ! stringa per definire tacche e label (vedi gr_box)
    end type

    type(plot_type), dimension(:), allocatable  :: grplot
    type(graph_type)                            :: grgraph

   CONTAINS

   subroutine gr_init(nv,xlarg,yalt)
!
!  Inizializza variabili
!
   integer, intent(in) :: nv
   integer, intent(in) :: xlarg,yalt
!
   iselview = nv
   klargv(iselview) = xlarg
   kaltv(iselview)  = yalt
   xresize = xlarg
   yresize = yalt
!
   call gr_area (0.0,1.0,0.0,1.0) ! tutta l'area e' usata per il disegno
!
   end subroutine gr_init
!-------------------------------------------------------------------------------------
   subroutine gr_open_viewer(xlarg,yalt,string,newv)
!
!  Apre GTK-viewer e inizializza variabili
!  xlarg  : larghezza del viewer
!  yalt   : altezza del viewer
!  string : titolo della finestra
!
   USE Molcom
!
   integer, intent(in)           :: xlarg,yalt
   character(len=*), intent(in)  :: string
   logical, intent(in), optional :: newv
!
   if (present(newv)) then
       newviewer = newv
   else     
       newviewer = .false.
   endif
   if ((viewerOpened == 0) .or. newviewer) then  ! con newviewer esegui comunque perche' e' sempre aperto
!                                                
       gr_color       = color_BRIGHTWHITE
       gr_color1      = -1
!
       lstyle = 1
       lwidth = LINEWIDTH_DEF
!
       if (newviewer) then
           call NVisualizzatore(1,xlarg,yalt,string)  ! apro nuovo visualizzatore
       else         
           call Visualizzatore(2,xlarg,yalt,string)   ! apro vecchio visualizzatore
       endif    
!
!      chiama la gr_init dopo la visulazziatore perche nella visulaz. vengono
!      processati eventi pendenti che possono cambiare iselview
       call gr_init(1,xlarg,yalt)   ! set iselview,scale,disp
!
   endif   
!
   end subroutine gr_open_viewer
!-------------------------------------------------------------------------------------
   subroutine gr_close_viewer()
   USE Molcom
!
!  Close GTK-viewer
!
   if (viewerOpened == 1)then
       if (newviewer) then
           call NVisualizzatore(5,0,0,' ')
       else         
           call Visualizzatore(5,0,0,' ')
       endif    
       viewerOpened = 0
       iselview = 0
   endif
!
   end subroutine gr_close_viewer
!-------------------------------------------------------------------------------------
   subroutine gr_select_viewer(nv)
!
!  Attiva viewer.
!  nv : codice che identifica il viewer attivo
!  nv = 0 --> attiva la finestra principale
!  nv = 1 --> attiva il viewer secondario
!
   integer :: nv
!
   if (nv == 0) then
       call Visualizzatore(3,0,0,' ')
       iselview = 0
   else if (nv == 1) then
       if (newviewer) then
           call NVisualizzatore(4,0,0,' ')
       else         
           call Visualizzatore(4,0,0,' ')
       endif    
       iselview = 1
   endif
!
   end subroutine gr_select_viewer
!-------------------------------------------------------------------------------------
   subroutine gr_area(xleft,xright,ybottom,ytop)
!
!  Selezione l' area grafica su cui disegnare.
!  Se questa subroutine non viene chiamata tutta l'area e usata per il disegno
!  xleft   : x-coordinate of left hand edge of viewport  (0-1)
!  xright  : x-coordinate of right hand edge of viewport (0-1)
!  ybottom : y-coordinate of left hand edge of viewport  (0-1)
!  ytop    : y-coordinate of left hand edge of viewport  (0-1)
!
   real :: xleft, xright
   real :: ybottom, ytop
!
   scalex(iselview) = (xright - xleft) * klargv(iselview)
   scaley(iselview) = -(ytop - ybottom) * kaltv(iselview)
   dispx(iselview)  = xleft * klargv(iselview)
   dispy(iselview)  = -ybottom * kaltv(iselview) + kaltv(iselview)
   scalex0(iselview) = scalex(iselview)
   scaley0(iselview) = scaley(iselview)
   dispx0(iselview) = dispx(iselview)
   dispy0(iselview) = dispy(iselview)
   xgrinf = 0.0
   xgrsup = 1.0
   ygrinf = 0.0
   ygrsup = 1.0
!
   end subroutine gr_area
!-------------------------------------------------------------------------------------
   subroutine gr_setwin(xmin,xmax,ymin,ymax)
!
!  Max e min dell'area grafica.
!  Se questa subroutine non viene chiamata il max e 1 il min e 0
!  xmin : x-coordinate of left hand edge of viewport
!  xmax : x-coordinate of right hand edge of viewport
!  ymin : y-coordinate of top hand edge of viewport
!  ymax : y-coordinate of bottom hand edge of viewport
!
   real, intent(in) :: xmin,xmax
   real, intent(in) :: ymin,ymax
!
   scalex(iselview) = scalex0(iselview)/(xmax - xmin)
   scaley(iselview) = scaley0(iselview)/(ymax - ymin)
   dispx(iselview) = dispx0(iselview) - xmin*scalex(iselview)
   dispy(iselview) = dispy0(iselview) - ymin*scaley(iselview)
   xgrinf = xmin
   xgrsup = xmax
   ygrinf = ymin
   ygrsup = ymax
!
   end subroutine gr_setwin
!-------------------------------------------------------------------------------------
   subroutine gr_sfs(fs)
!
   integer, intent(in) :: fs
!
   if (fs /= 0) then
       gr_color1 = gr_color ! area piena di colore gr_color
   else
       gr_color1 = -1       ! area vuota
   endif
!
   end subroutine gr_sfs
!-------------------------------------------------------------------------------------
   subroutine gr_rect(xleft,xright,ybottom,ytop)
!
!  Draw rectangle
!  xleft , xright : the horizontal range of the rectangle
!  ybottom , ytop : the vertical range of the rectangle
!
   real :: xleft, xright
   real :: ybottom, ytop
!
   call Rettangolo                                                                &
        (nint(xleft*scalex(iselview)+dispx(iselview)) ,nint(ybottom*scaley(iselview)+dispy(iselview)),                    &
         nint(xright*scalex(iselview)+dispx(iselview)),nint(ytop*scaley(iselview)+dispy(iselview)),gr_color1)
!
   end subroutine gr_rect
!-------------------------------------------------------------------------------------
   subroutine gr_drawmarkers(xp,yp,symbol)
!
!  Draw several markers
!  xp : x-coordinates of the points
!  yp : y-coordinates of points
!  n  : number of points to mark
!
   real, dimension(:), intent(in) :: xp,yp
   integer, intent(in)            :: symbol
   integer                        :: n
   integer, dimension(size(xp))   :: xg,yg
   integer                        :: i
!
   n = size(xp)
   xg = nint(xp*scalex(iselview)+dispx(iselview))
   yg = nint(yp*scaley(iselview)+dispy(iselview))
!
   select case(symbol)

     case(1)             ! croce 'x'
       do i=1,n
          call croce (xg(i),yg(i),gr_size,1)
       enddo

     case(2)             ! croce '+'
       do i=1,n
          call croce (xg(i),yg(i),gr_size,0)
       enddo

     case(3)             ! cerchio vuoto
       do i=1,n
          call cerchio (xg(i),yg(i),gr_size,0)
       enddo

     case(4)             ! cerchio pieno
       do i=1,n
          call cerchio (xg(i),yg(i),gr_size,-1)
       enddo

     case(5)             ! quadrato vuoto
       do i=1,n
          call Rettangolo (xg(i)-gr_size,yg(i)-gr_size,xg(i)+gr_size,yg(i)+gr_size,-1)
       enddo

     case(6)             ! quadrato pieno
       do i=1,n
          call Rettangolo (xg(i)-gr_size,yg(i)-gr_size,xg(i)+gr_size,yg(i)+gr_size,0)
       enddo

     case default        ! simbolo char(symbol)
       do i=1,n
          call DrawLabel (xg(i), yg(i), char(symbol), 2)
       enddo

   end select
!
   end subroutine gr_drawmarkers
!-------------------------------------------------------------------------------------
   subroutine gr_draw1marker(xp,yp,symbol)
!
!  Draw one marker
!  xp     : x-coordinates of the point
!  yp     : x-coordinates of the point
!  symbol : code number of the symbol
!
   real, intent(in)    :: xp,yp
   integer, intent(in) :: symbol
   real,dimension(1)   :: xxp,yyp
!
   xxp = xp
   yyp = yp
   call gr_drawmarkers(xxp,yyp,symbol)
!
   end subroutine gr_draw1marker
!-------------------------------------------------------------------------------------
   subroutine gr_setcolor(icol)
!
!  Set color
!  icol : il codice del colore deve variare da 0 a 10
!
   integer, intent(in) :: icol
!
   gr_color = icol
   call SetColor(gr_color)
!
   end subroutine gr_setcolor
!-------------------------------------------------------------------------------------
   subroutine gr_clear_all_area()
!
!  Clear area
!
   call gr_setbackground(gr_colbackg)
!
   end subroutine gr_clear_all_area
!-------------------------------------------------------------------------------------
   subroutine gr_clear_area(xleft,xright,ybottom,ytop)
!
!  Clear area
!  xleft   : x-coordinate of left hand edge of area  (0-1)
!  xright  : x-coordinate of right hand edge of area (0-1)
!  ybottom : y-coordinate of left hand edge of area  (0-1)
!  ytop    : y-coordinate of left hand edge of area  (0-1)
!
   real, intent(in) :: xleft, xright
   real, intent(in) :: ybottom, ytop
   real             :: colsav,colsav1
!
   colsav = gr_color
   colsav1 = gr_color1
!!corr   gr_color = gr_colbackg
   call gr_setcolor(gr_colbackg)
   gr_color1 = gr_colbackg
   call gr_area(xleft,xright,ybottom,ytop)
   call gr_rect(0.0,1.0,0.0,1.0)
   gr_color = colsav
   gr_color1 = colsav1
!
   end subroutine gr_clear_area
!-------------------------------------------------------------------------------------
   subroutine gr_setbackground(icol)
!
!  Seleziona il colore per il background
!
   integer, intent(in) :: icol
!
   gr_colbackg = icol
   call gr_setcolor(gr_colbackg)
   call Rettangolo(0,0,nint(klargv(iselview)),nint(kaltv(iselview)),0)
!
   end subroutine gr_setbackground
!-------------------------------------------------------------------------------------
   subroutine gr_setsize(size)
!
!  Seleziona le dimensioni dei markers
!
   integer, intent(in) :: size
!
   gr_size = size
!
   end subroutine gr_setsize
!-------------------------------------------------------------------------------------
   subroutine gr_drawline(xp,yp)
!
!  Draw a polyline
!  xp : x-coordinates of the points
!  yp : y-coordinates of points
!  n : numero di punti che definiscono la linea. la linea e' formata da n-1 segmenti
!
   real, dimension(:), intent(in) :: xp,yp
   integer                        :: n
!
   n = size(xp)
   !call profileplot(1,n,nint(xp(:n)*scalex(iselview)+dispx(iselview)),nint(yp(:n)*scaley(iselview)+dispy(iselview)),lstyle)
   call mrwinnew3(1,n,nint(xp(:n)*scalex(iselview)+dispx(iselview)),nint(yp(:n)*scaley(iselview)+dispy(iselview)),lstyle,lwidth)
!
   end subroutine gr_drawline
!-------------------------------------------------------------------------------------
   subroutine gr_box(xopt,xtick,nxsub,yopt,ytick,nysub)
   USE Molcom, only : FntPixHeight
!
!  Draw axis, labels, ticks
!
!  xopt  : strings of options for X axis
!  xtick : interval between majors tick marks on X axis.
!          If xtick=0.0 the interval is chosen to gr_box
!  nxsub : the number of subintervals to divide the major coordinate interval into.
!          If xtick=0.0 or nxsub=0 the number is chosen to gr_box
!  yopt  : strings of options for Y axis. Coding is the same as for xopt
!  ytick : like xtick for the Y axis
!  nysub : like nxsub for the Y axis
!
!  Options for parameters XOPT and YOPT
!  A : draw bottom (for X axis) or left  (for Y axis) edge of frame
!  B : draw top    (for X axis) or right (for Y axis) edge of frame
!  G : draw Grid
!  I : draw tick marks outside the viewport instead inside
!  L : draw label
!  T : draw major ticks
!  S : draw minor ticks
!  Z : draw ticks for top and right
!
   character(len = *), intent(in) :: xopt
   character(len = *), intent(in) :: yopt
   real              , intent(in) :: xtick
   real              , intent(in) :: ytick
   integer           , intent(in) :: nxsub
   integer           , intent(in) :: nysub
   logical                        :: xopta,xoptb,xoptg,xopti,xoptl,xoptt,xopts,xoptz
   logical                        :: yopta,yoptb,yoptg,yopti,yoptl,yoptt,yopts,yoptz
   real                           :: xint,xinte,xinte2
   real                           :: yint,yinte,yinte2
   !real                           :: gr_round
   integer                        :: i,j,i1,i2
   real                           :: tikl1
   real                           :: tikl2
   character(len = 20)            :: clbl
   integer                        :: nc,np,nv
   real                           :: xc,yc
   real                           :: xpt,ypt
   integer                        :: nsubx
   integer                        :: nsuby
   real                           :: xndsp,yndsp
!
   xopta = index(xopt,'A') /= 0
   xoptb = index(xopt,'B') /= 0
   xoptg = index(xopt,'G') /= 0
   xopti = index(xopt,'I') /= 0
   xoptl = index(xopt,'L') /= 0
   xoptt = index(xopt,'T') /= 0
   xopts = index(xopt,'S') /= 0
   xoptz = index(xopt,'Z') /= 0
!
   yopta = index(yopt,'A') /= 0
   yoptb = index(yopt,'B') /= 0
   yoptg = index(yopt,'G') /= 0
   yopti = index(yopt,'I') /= 0
   yoptl = index(yopt,'L') /= 0
   yoptt = index(yopt,'T') /= 0
   yopts = index(yopt,'S') /= 0
   yoptz = index(yopt,'Z') /= 0
!
!  Spostamento delle labels dagli assi
!
   xndsp = 1.2
   yndsp = 0.4
   if (xopti) xndsp = xndsp + 0.3
   if (yopti) yndsp = yndsp + 0.3
!
!  Disegna assi
!
   if (xopta) call gr_line(xgrinf,ygrinf,xgrsup,ygrinf)
   if (xoptb) call gr_line(xgrinf,ygrsup,xgrsup,ygrsup)
   if (yopta) call gr_line(xgrinf,ygrinf,xgrinf,ygrsup)
   if (yoptb) call gr_line(xgrsup,ygrinf,xgrsup,ygrsup)
!
!  Ampiezza intervalli e num. di subintervalli per l'asse X
!
   if (xtick == 0.0) then
       xinte = gr_round(7.0*FntPixHeight*(xgrsup-xgrinf)/klargv(iselview),nsubx)
   else
       xinte = xtick
       nsubx = max(nxsub,1)
   endif
!
!  Disegna tacche per asse X
!
   if (xoptt) then
!
!      Lunghezza tacche
!
       tikl1 = -FntPixHeight*0.55/scaley(iselview) ! lunghezza tacche maggiori
       tikl2 = tikl1*0.5                 ! lunghezza tacche minori
       if (xopti) then
           tikl1 = -tikl1
           tikl2 = -tikl2
       endif
!
       call gr_box1(xgrinf,xgrsup,xinte,i1,i2)
!
!      Tacche asse X in basso
!
       if (xopta) then
           do i=i1,i2
!
!             Tacche maggiori
!
              xint = xinte*real(i)
              call gr_line(xint,ygrinf,xint,ygrinf+tikl1)
!
!             Tacche minori
!
              if (xopts) then
                  xinte2 = xinte / nsubx
                  do j=1,nsubx-1
                     xpt = xint+xinte2*real(j)
                     if (xpt > xgrsup) exit
                     call gr_line(xpt,ygrinf,xpt,ygrinf+tikl2)
                  enddo
                  if ( i == i1 ) then    ! tacche minori al di sotto della prima tacca maggiore
                     j = 1
                     xpt = xint-xinte2
                     do while (xpt > xgrinf)
                        call gr_line(xpt,ygrinf,xpt,ygrinf+tikl2)
                        j = j + 1
                        xpt = xint-xinte2*real(j)
                     enddo
                  endif
              endif
           enddo
       endif
!
!      Tacche asse X in alto
!
       if (xoptb .and. xoptz) then
           do i=i1,i2
!
!             Tacche maggiori
!
              xint = xinte*real(i)
              call gr_line(xint,ygrsup,xint,ygrsup-tikl1)
!
!             Tacche minori
!
              if (xopts) then
                  xinte2 = xinte / nsubx
                  do j=1,nsubx-1
                     xpt = xint+xinte2*real(j)
                     if (xpt > xgrsup) exit
                     call gr_line(xpt,ygrsup,xpt,ygrsup-tikl2)
                  enddo
                  if ( i == i1 ) then   ! tacche minori al di sotto della prima tacca maggiore
                     j = 1
                     xpt = xint-xinte2
                     do while (xpt > xgrinf)
                        call gr_line(xpt,ygrsup,xpt,ygrsup-tikl2)
                        j = j + 1
                        xpt = xint-xinte2*real(j)
                     enddo
                  endif
              endif
           enddo
       endif
   endif
!
!  Disegna griglie per l'asse X
!
   if (xoptg) then
       call gr_box1(xgrinf,xgrsup,xinte,i1,i2)
       do i=i1,i2
          xint = xinte*real(i)
          call gr_line(xint,ygrinf,xint,ygrsup)
       enddo
   endif
!
!  Disegna label sull'asse X
!
   if (xoptl) then
       np = int(log10(xinte)) - 4
       nv = nint(xinte/10.**np)
       call gr_box1(xgrinf,xgrsup,xinte,i1,i2)
       do i=i1,i2
          xc = (i*xinte-xgrinf)/(xgrsup-xgrinf)
          call grnumb(i*nv,np,0,clbl,nc)
          call gr_reldrawtext('B',xndsp,xc,0.5,clbl(1:nc),2)
       enddo
   endif
!
!  Ampiezza intervalli e num. di subintervalli per l'asse Y
!
   if (ytick == 0.0) then
       !yinte = gr_round(0.05*(ygrsup-ygrinf),nsuby)
       yinte = gr_round(3.0*FntPixHeight*(ygrsup-ygrinf)/kaltv(iselview),nsuby)
   else
       yinte = ytick
       nsuby = max(nysub,1)
   endif
!
!  Disegna tacche per asse Y
!
   if (yoptt) then
!
!      Lunghezza tacche
!
       tikl1 = FntPixHeight*0.55/scalex(iselview) ! lunghezza tacche maggiori
       tikl2 = tikl1*0.5                ! lunghezza tacche minori
       if (yopti) then
           tikl1 = -tikl1
           tikl2 = -tikl2
       endif
!
       call gr_box1(ygrinf,ygrsup,yinte,i1,i2)
!
!      Tacche asse Y a sinistra
!
       if (yopta) then
           do i=i1,i2
!
!             Tacche maggiori
!
              yint = yinte*real(i)
              call gr_line(xgrinf,yint,xgrinf+tikl1,yint)
!
!             Tacche minori
!
              if (yopts) then
                  yinte2 = yinte / nsuby
                  do j=1,nsuby-1
                     ypt = yint+yinte2*real(j)
                     if (ypt > ygrsup) exit
                     call gr_line(xgrinf,ypt,xgrinf+tikl2,ypt)
                  enddo
                  if ( i == i1 ) then   ! tacche minori al di sotto della prima tacca maggiore
                     j = 1
                     ypt = yint-yinte2
                     do while (ypt > ygrinf)
                        call gr_line(xgrinf,ypt,xgrinf+tikl2,ypt)
                        j = j + 1
                        ypt = yint-yinte2*real(j)
                     enddo
                  endif
              endif
           enddo
       endif
!
!      Tacche asse Y a destra
!
       if (yoptb .and. yoptz) then
           do i=i1,i2
!
!             Tacche maggiori
!
              yint = yinte*real(i)
              call gr_line(xgrsup,yint,xgrsup-tikl1,yint)
!
!             Tacche minori
!
              if (yopts) then
                  yinte2 = yinte / nsuby
                  do j=1,nsuby-1
                     ypt = yint+yinte2*real(j)
                     if (ypt > ygrsup) exit
                     call gr_line(xgrsup,ypt,xgrsup-tikl2,ypt)
                  enddo
                  if ( i == i1 ) then   ! tacche minori al di sotto della prima tacca maggiore
                     j = 1
                     ypt = yint-yinte2
                     do while (ypt > ygrinf)
                        call gr_line(xgrsup,ypt,xgrsup-tikl2,ypt)
                        j = j + 1
                        ypt = yint-yinte2*real(j)
                     enddo
                  endif
              endif
           enddo
       endif
   endif
!
!  Disegna griglie per l'asse Y
!
   if (yoptg) then
       call gr_box1(ygrinf,ygrsup,yinte,i1,i2)
       do i=i1,i2
          yint = yinte*real(i)
          call gr_line(xgrinf,yint,xgrsup,yint)
       enddo
   endif
!
!  Disegna label sull'asse Y
!
   if (yoptl) then
       np = int(log10(yinte)) - 4
       nv = nint(yinte/10.**np)
       call gr_box1(ygrinf,ygrsup,yinte,i1,i2)
       do i=i1,i2
          yc = (i*yinte-ygrinf + 0.5*FntPixHeight/scaley(iselview))/(ygrsup-ygrinf)
          call grnumb(i*nv,np,1,clbl,nc)
          call gr_reldrawtext('L',yndsp,yc,1.0,clbl(1:nc),2)
       enddo
   endif
!
   end subroutine gr_box
!-------------------------------------------------------------------------------------
   subroutine gr_box_sir(xopt,xtick,nxsub,yopt,ytick,nysub,LabTrials,LabTrialsV)
   USE Molcom, only : FntPixHeight
!
!  Draw axis, labels, ticks
!
!  xopt  : strings of options for X axis
!  xtick : interval between majors tick marks on X axis. 
!          If xtick=0.0 the interval is chosen to gr_box
!  nxsub : the number of subintervals to divide the major coordinate interval into. 
!          If xtick=0.0 or nxsub=0 the number is chosen to gr_box
!  yopt  : strings of options for Y axis. Coding is the same as for xopt
!  ytick : like xtick for the Y axis
!  nysub : like nxsub for the Y axis
!  LabTrials  : number of X labels initialized by users
!  LabTrialsV : X labels initialized by users
!
!  Options for parameters XOPT and YOPT
!  A : draw bottom (for X axis) or left  (for Y axis) edge of frame 
!  B : draw top    (for X axis) or right (for Y axis) edge of frame 
!  G : draw Grid 
!  I : draw tick marks outside the viewport instead inside
!  L : draw label
!  T : draw major ticks 
!  S : draw minor ticks
!  Z : draw ticks for top and right
!
   character(len = *), intent(in) :: xopt
   character(len = *), intent(in) :: yopt
   real              , intent(in) :: xtick
   real              , intent(in) :: ytick
   integer           , intent(in) :: nxsub
   integer           , intent(in) :: nysub
   integer           , intent(in) :: LabTrials
   character(len = *), intent(in) :: LabTrialsV(LabTrials)
   logical                        :: xopta,xoptb,xoptg,xopti,xoptl,xoptt,xopts,xoptz
   logical                        :: yopta,yoptb,yoptg,yopti,yoptl,yoptt,yopts,yoptz
   real                           :: xint,xinte,xinte2
   real                           :: yint,yinte,yinte2,ayinte
   real, dimension(2)             :: vyinte
   !real                           :: gr_round
   integer                        :: i,j,i1,i2
   real                           :: tikl1              
   real                           :: tikl2             
   character(len = 20)            :: clbl
   integer                        :: nc,np,nv
   real                           :: xc,yc
   real                           :: xpt,ypt
   integer                        :: nsubx 
   integer                        :: nsuby 
   real                           :: xndsp,yndsp
   real                           :: vyintei,segno
   integer                        :: Lung

!
   xopta = index(xopt,'A') /= 0
   xoptb = index(xopt,'B') /= 0
   xoptg = index(xopt,'G') /= 0
   xopti = index(xopt,'I') /= 0
   xoptl = index(xopt,'L') /= 0
   xoptt = index(xopt,'T') /= 0
   xopts = index(xopt,'S') /= 0
   xoptz = index(xopt,'Z') /= 0
!
   yopta = index(yopt,'A') /= 0
   yoptb = index(yopt,'B') /= 0
   yoptg = index(yopt,'G') /= 0
   yopti = index(yopt,'I') /= 0
   yoptl = index(yopt,'L') /= 0
   yoptt = index(yopt,'T') /= 0
   yopts = index(yopt,'S') /= 0
   yoptz = index(yopt,'Z') /= 0
!
!  Spostamento delle labels dagli assi
!
   xndsp = 1.2
   yndsp = 0.4
   if (xopti) xndsp = xndsp + 0.3
   if (yopti) yndsp = yndsp + 0.3
!
!  Disegna assi 
!
   if (xopta) call gr_line(xgrinf,ygrinf,xgrsup,ygrinf)
   if (xoptb) call gr_line(xgrinf,ygrsup,xgrsup,ygrsup)
   if (yopta) call gr_line(xgrinf,ygrinf,xgrinf,ygrsup)
   if (yoptb) call gr_line(xgrsup,ygrinf,xgrsup,ygrsup)
!
!  Ampiezza intervalli e num. di subintervalli per l'asse X
!
   if (xtick == 0.0) then
       xinte = gr_round(7.0*FntPixHeight*(xgrsup-xgrinf)/klargv(iselview),nsubx)
   else
       xinte = xtick
       nsubx = max(nxsub,1)
   endif
!
!  Disegna tacche per asse X 
!
   if (xoptt) then
!
!      Lunghezza tacche
!
       tikl1 = -FntPixHeight*0.55/scaley(iselview) ! lunghezza tacche maggiori
       tikl2 = tikl1*0.5                 ! lunghezza tacche minori
       if (xopti) then 
           tikl1 = -tikl1
           tikl2 = -tikl2
       endif
!
       call gr_box1(xgrinf,xgrsup,xinte,i1,i2)
!
!      Tacche asse X in basso
!
       if (xopta) then
           do i=i1,i2
!
!             Tacche maggiori
!
              xint = xinte*real(i)
              call gr_line(xint,ygrinf,xint,ygrinf+tikl1)
!
!             Tacche minori
!
              if (xopts) then
                  xinte2 = xinte / nsubx
                  do j=1,nsubx-1
                     xpt = xint+xinte2*real(j)
                     if (xpt > xgrsup) exit
                     call gr_line(xpt,ygrinf,xpt,ygrinf+tikl2) 
                  enddo 
                  if ( i == i1 ) then    ! tacche minori al di sotto della prima tacca maggiore
                     j = 1
                     xpt = xint-xinte2
                     do while (xpt > xgrinf) 
                        call gr_line(xpt,ygrinf,xpt,ygrinf+tikl2) 
                        j = j + 1
                        xpt = xint-xinte2*real(j)
                     enddo
                  endif
              endif
           enddo
       endif
!
!      Tacche asse X in alto
!
       if (xoptb .and. xoptz) then
           do i=i1,i2
!
!             Tacche maggiori
!
              xint = xinte*real(i)
              call gr_line(xint,ygrsup,xint,ygrsup-tikl1)
!
!             Tacche minori
!
              if (xopts) then
                  xinte2 = xinte / nsubx
                  do j=1,nsubx-1
                     xpt = xint+xinte2*real(j)
                     if (xpt > xgrsup) exit
                     call gr_line(xpt,ygrsup,xpt,ygrsup-tikl2) 
                  enddo 
                  if ( i == i1 ) then   ! tacche minori al di sotto della prima tacca maggiore
                     j = 1
                     xpt = xint-xinte2
                     do while (xpt > xgrinf) 
                        call gr_line(xpt,ygrsup,xpt,ygrsup-tikl2) 
                        j = j + 1
                        xpt = xint-xinte2*real(j)
                     enddo
                  endif
              endif
           enddo
       endif
   endif
!
!  Disegna griglie per l'asse X
!
   if (xoptg) then 
       call gr_box1(xgrinf,xgrsup,xinte,i1,i2)
       do i=i1,i2
          xint = xinte*real(i)
          call gr_line(xint,ygrinf,xint,ygrsup)
       enddo
   endif
!
!  Disegna label sull'asse X
!
   if (xoptl) then
       np = int(log10(xinte)) - 4   ! ATTENZIONE xinte = -50000000
       nv = nint(xinte/10.**np)
       call gr_box1(xgrinf,xgrsup,xinte,i1,i2)
       if (LabTrials == 0) then
           do i=i1,i2
              xc = (i*xinte-xgrinf)/(xgrsup-xgrinf)
              call grnumb(i*nv,np,0,clbl,nc)
              call gr_reldrawtext('B',xndsp,xc,0.5,clbl(1:nc),2)
           enddo
       else
!!         do i=i1+1,i2-1
           do i=1, LabTrials
              xc = (i*xinte-xgrinf)/(xgrsup-xgrinf)
              nc = Lung(LabTrialsV(i))
              clbl= ' '
              clbl= LabTrialsV(i)(1:nc)
              call gr_reldrawtext('B',xndsp,xc,0.5,clbl(1:nc),2)
           enddo
       endif
   endif
!
!  Ampiezza intervalli e num. di subintervalli per l'asse Y
!
   if (ytick == 0.0) then
       !yinte = gr_round(0.20*(ygrsup-ygrinf),nsuby)
       yinte = gr_round(0.05*(ygrsup-ygrinf),nsuby)
       ayinte = yinte
   else
       yinte = ytick
       ayinte = abs(yinte)
       nsuby = max(nysub,1)
   endif
!
!  Disegna tacche per asse Y 
!
   if (yoptt) then
!
!      Lunghezza tacche
!
       tikl1 = FntPixHeight*0.55/scalex(iselview) ! lunghezza tacche maggiori
       tikl2 = tikl1*0.5                ! lunghezza tacche minori
       if (yopti) then 
           tikl1 = -tikl1
           tikl2 = -tikl2
       endif
!
       if (yinte < 0.0) then
           if (nint(ayinte) == 1) then
               i1 = 1
               i2 = 1
           else
               i1 = 1
               i2 = 2
           endif
           vyinte(1) = ygrinf
           vyinte(2) = ygrsup
       else
           call gr_box1(ygrinf,ygrsup,ayinte,i1,i2)
       endif
!
!      Tacche asse Y a sinistra
!
       if (yopta) then
           do i=i1,i2
!
!             Tacche maggiori
!
              if (yinte < 0) then
                  yint = vyinte(i)
              else
                  yint = yinte*real(i)
              endif
              call gr_line(xgrinf,yint,xgrinf+tikl1,yint)
!
!             Tacche minori
!
              if (yopts .and. yinte > 0.0) then
                  yinte2 = yinte / nsuby
                  do j=1,nsuby-1
                     ypt = yint+yinte2*real(j)
                     if (ypt > ygrsup) exit
                     call gr_line(xgrinf,ypt,xgrinf+tikl2,ypt)
                  enddo 
                  if ( i == i1 ) then   ! tacche minori al di sotto della prima tacca maggiore
                     j = 1
                     ypt = yint-yinte2
                     do while (ypt > ygrinf) 
                        call gr_line(xgrinf,ypt,xgrinf+tikl2,ypt)
                        j = j + 1
                        ypt = yint-yinte2*real(j)
                     enddo
                  endif
              endif
           enddo
       endif
!
!      Tacche asse Y a destra
!
       if (yoptb .and. yoptz) then
           do i=i1,i2
!
!             Tacche maggiori
!
              yint = yinte*real(i)
              call gr_line(xgrsup,yint,xgrsup-tikl1,yint)
!
!             Tacche minori
!
              if (yopts) then
                  yinte2 = yinte / nsuby
                  do j=1,nsuby-1
                     ypt = yint+yinte2*real(j)
                     if (ypt > ygrsup) exit
                     call gr_line(xgrsup,ypt,xgrsup-tikl2,ypt)
                  enddo 
                  if ( i == i1 ) then   ! tacche minori al di sotto della prima tacca maggiore
                     j = 1
                     ypt = yint-yinte2
                     do while (ypt > ygrinf) 
                        call gr_line(xgrsup,ypt,xgrsup-tikl2,ypt)
                        j = j + 1
                        ypt = yint-yinte2*real(j)
                     enddo
                  endif
              endif
           enddo
       endif
   endif
!
!  Disegna griglie per l'asse Y
!
   if (yoptg) then 
       call gr_box1(ygrinf,ygrsup,yinte,i1,i2)
       do i=i1,i2
          yint = yinte*real(i)
          call gr_line(xgrinf,yint,xgrsup,yint)
       enddo
   endif
!
!  Disegna label sull'asse Y
!
   if (yoptl) then
       if (yinte < 0.0) then
           if (nint(ayinte) == 1) then
               i1 = 1
               i2 = 1
           else
               i1 = 1
               i2 = 2
           endif
           vyinte(1) = ygrinf
           vyinte(2) = ygrsup
       else
           np = int(log10(ayinte)) - 4
           nv = nint(ayinte/10.**np)
           call gr_box1(ygrinf,ygrsup,ayinte,i1,i2)
       endif
       do i=i1,i2
          if (yinte < 0.0) then
              if (vyinte(i) < 0.0) then
                  vyintei =  -vyinte(i)
                  segno = -1
              else
                  vyintei =  vyinte(i)
                  segno = 1
              endif
              np = int(log10(vyintei+0.001)) - 4
              nv = nint(vyintei/10.**np)
              yc = (  vyinte(i)-ygrinf + 0.5*FntPixHeight/scaley(iselview))   &
                 / (ygrsup-ygrinf)
              nv = nv * segno
              call grnumb(  nv,np,1,clbl,nc)
          else
              yc = (i*ayinte-ygrinf + 0.5*FntPixHeight/scaley(iselview))    &
                 / (ygrsup-ygrinf)
              call grnumb(i*nv,np,1,clbl,nc)
          endif
          call gr_reldrawtext('L',yndsp,yc,1.0,clbl(1:nc),2)
       enddo
   endif
!
   end subroutine gr_box_sir
!-------------------------------------------------------------------------------------
   subroutine gr_box1(xi,xf,xint,i1,i2)
!
   real   , intent(in)  :: xi,xf,xint
   integer, intent(out) :: i1,i2
   real                 :: xlo,xhi
!
   xlo = xi / xint
   xhi = xf / xint
   i1 = nint(xlo)
   if (i1 < xlo) i1 = i1 + 1
   i2 = nint(xhi)
   if (i2 > xhi) i2 = i2 - 1
!
   end subroutine gr_box1
!-------------------------------------------------------------------------------------
   subroutine gr_setlinestyle(style,width)
!
!  Set line style
!  style : line style code (1-3)
!      1 = linea continua
!      2 = linea tratteggiata
!      3 = linea spessa
!
   integer, intent(in)           :: style
   integer, intent(in), optional :: width
!
   lstyle = style
   if (present(width)) then
       lwidth = width
   endif
!
   end subroutine gr_setlinestyle
!-------------------------------------------------------------------------------------
   subroutine gr_line(x1,y1,x2,y2)
!
!  Disegna un segmento
!  x1 , y1 : coordinate punto iniziale
!  x2 , y2 : coordinate punto finale
!
   real, intent(in) :: x1,x2,y1,y2
!
   select case (lstyle)
      case (SOLIDLINE)                     ! linea continua
       call Linea(nint(x1*scalex(iselview)+dispx(iselview)),nint(y1*scaley(iselview)+dispy(iselview)),           &
              nint(x2*scalex(iselview)+dispx(iselview)),nint(y2*scaley(iselview)+dispy(iselview)),lwidth)

      case (DASHLINE)                      ! linea tratteggiata
       call Linea_dash(nint(x1*scalex(iselview)+dispx(iselview)),nint(y1*scaley(iselview)+dispy(iselview)),      &
              nint(x2*scalex(iselview)+dispx(iselview)),nint(y2*scaley(iselview)+dispy(iselview)),lwidth)

      case (TICKLINE)                      ! linea spessa
       call LineaSpessa(nint(x1*scalex(iselview)+dispx(iselview)),nint(y1*scaley(iselview)+dispy(iselview)),     &
              nint(x2*scalex(iselview)+dispx(iselview)),nint(y2*scaley(iselview)+dispy(iselview)),1)
   end select
!
   end subroutine gr_line
!-------------------------------------------------------------------------------------
   subroutine gr_drawtext(xp,yp,just,stringa,fsize)
   USE Molcom
!
!  Write text
!  x       : x-coordinate of start of string
!  y       : y-coordinate of start of string
!  just    : control justification of the string (0-1)
!  stringa : the character string to be plotted
!
   real, intent(in)                :: xp,yp
   character (len = *), intent(in) :: stringa
   integer, intent(in)             :: fsize
   real, intent(in)                :: just
   integer                         :: LenInPix
!
   call DrawLabel (nint(xp*scalex(iselview)+dispx(iselview)-just*LenInPix(stringa,fsize)), &
                   nint(yp*scaley(iselview)+dispy(iselview)), stringa, fsize)
!
   end subroutine gr_drawtext
!-------------------------------------------------------------------------------------
   subroutine gr_reldrawtext(side,disp,coord,just,stringa,fsize,verticale)
   USE Molcom, only:FntPixHeight
!
!  Write text at position relative to viewport
!  side    : 'B', 'L', 'T', 'R' signify the bottom, left, top , right of the vieport.
!  disp    : the displacement of the character from the specified edge in units of character height.
!            Use negative value to write inside the viewport, a positive value to write outside.
!  just    : control justification of the string (0-1)
!  stringa : the character string to be plotted
!
   character(len = *), intent(in) :: side
   real              , intent(in) :: disp
   real              , intent(in) :: coord
   real              , intent(in) :: just
   character(len = *), intent(in) :: stringa
   integer, intent(in), optional  :: verticale
   integer, intent(in)            :: fsize
   integer                        :: xp,yp
   integer                        :: LenInPix
   integer                        :: verticals
!
   if (present(verticale)) then
       verticals = verticale
   else
       verticals = 0
   endif
!
   if (index(side,'L') /= 0) then
       xp = nint(dispx0(iselview) - just*LenInPix(stringa,fsize) - disp*FntPixHeight)
       yp = nint(coord*scaley0(iselview) + dispy0(iselview))
   else if (index(side,'R') /= 0) then
       xp = nint(scalex0(iselview) + dispx0(iselview) - just*LenInPix(stringa,fsize) + disp*FntPixHeight)
       yp = nint(coord*scaley0(iselview) + dispy0(iselview))
   else if (index(side,'B') /= 0) then
       xp = nint(scalex0(iselview)*coord + dispx0(iselview)  - just*LenInPix(stringa,fsize))
       yp = nint(dispy0(iselview) + disp*FntPixHeight)
   else if (index(side,'T') /= 0) then
       xp = nint(scalex0(iselview)*coord + dispx0(iselview) - just*LenInPix(stringa,fsize))
       yp = nint(scaley0(iselview) + dispy0(iselview) - disp*FntPixHeight)
   endif
!
!  Controllo per stringa fuori area
   if (xp <= 0) xp = 1
   if (xp > klargv(iselview)) xp = klargv(iselview) - 1
   if (yp <= 0) yp = 1
   if (yp > kaltv(iselview)) yp = kaltv(iselview) - 1
!
   if (verticals == 0) then
       call DrawLabel (xp, yp, stringa, fsize)
   else
       call DrawLabelV(xp, yp, stringa, fsize)
   endif
!
   end subroutine gr_reldrawtext
!-------------------------------------------------------------------------------------
   real function gr_round(x,nsub)
!
   real, intent(in)              :: x
   integer, intent(out)          :: nsub
   real                          :: xx,xlog,pwr,frac
   integer                       :: ilog,i
   real, dimension(3), parameter :: nice = (/2.0,5.0,10.0/)
!
   if ( x == 0.0) then
      gr_round = 0.0
      nsub = 2
      return
   endif
   xx = abs(x)
   xlog = log10(xx)
   ilog = xlog
   if (xlog < 0) ilog = ilog - 1
   pwr = 10.0 ** ilog
   frac = xx / pwr
   i = 3
   if (frac <= nice(2)) i = 2
   if (frac <= nice(1)) i = 1
   gr_round = sign(pwr*nice(i),x)
   nsub = 5
   if (i == 1) nsub = 2
!
   end function gr_round
!-------------------------------------------------------------------------------------
   SUBROUTINE GRNUMB (MM, PP, FORM, STRING, NC)
   INTEGER MM, PP, FORM
   CHARACTER*(*) STRING
   INTEGER NC
!
! This routine converts a number into a decimal character
! representation. To avoid problems of floating-point roundoff, the
! number must be provided as an integer (MM) multiplied by a power of 10
! (10**PP).  The output string retains only significant digits of MM,
! and will be in either integer format (123), decimal format (0.0123),
! or exponential format (1.23x10**5). Standard escape sequences \u, \d
! raise the exponent and \x is used for the multiplication sign.
! This routine is used by PGBOX to create numeric labels for a plot.
!
! Formatting rules:
!   (a) Decimal notation (FORM=1):
!       - Trailing zeros to the right of the decimal sign are
!         omitted
!       - The decimal sign is omitted if there are no digits
!         to the right of it
!       - When the decimal sign is placed before the first digit
!         of the number, a zero is placed before the decimal sign
!       - The decimal sign is a period (.)
!       - No spaces are placed between digits (ie digits are not
!         grouped in threes as they should be)
!       - A leading minus (-) is added if the number is negative
!   (b) Exponential notation (FORM=2):
!       - The exponent is adjusted to put just one (non-zero)
!         digit before the decimal sign
!       - The mantissa is formatted as in (a), unless its value is
!         1 in which case it and the multiplication sign are omitted
!       - If the power of 10 is not zero and the mantissa is not
!         zero, an exponent of the form \x10\u[-]nnn is appended,
!         where \x is a multiplication sign (cross), \u is an escape
!         sequence to raise the exponent, and as many digits nnn
!         are used as needed
!   (c) Automatic choice (FORM=0):
!         Decimal notation is used if the absolute value of the
!         number is less than 10000 or greater than or equal to
!         0.01. Otherwise exponential notation is used.
!
! Arguments:
!  MM     (input)
!  PP     (input)  : the value to be formatted is MM*10**PP.
!  FORM   (input)  : controls how the number is formatted:
!                    FORM = 0 -- use either decimal or exponential
!                    FORM = 1 -- use decimal notation
!                    FORM = 2 -- use exponential notation
!  STRING (output) : the formatted character string, left justified.
!                    If the length of STRING is insufficient, a single
!                    asterisk is returned, and NC=1.
!  NC     (output) : the number of characters used in STRING:
!                    the string to be printed is STRING(1:NC).
!--
! 23-Nov-1983
!  9-Feb-1988 [TJP] - Use temporary variable to avoid illegal character
!                     assignments; remove non-standard DO loops.
! 15-Dec-1988 [TJP] - More corrections of the same sort.
! 27-Nov-1991 [TJP] - Change code for multiplication sign.
! 23-Jun-1994 [TJP] - Partial implementation of FORM=1 and 2.
!-----------------------------------------------------------------------
   CHARACTER*1 BSLASH
   CHARACTER*2 TIMES, UP, DOWN
   CHARACTER*20 WORK, WEXP, TEMP
   INTEGER M, P, ND, I, J, K, NBP
   LOGICAL MINUS
!
! Define backslash (escape) character and escape sequences.
!
   BSLASH = CHAR(92)
   TIMES  = BSLASH//'x'
   UP     = BSLASH//'u'
   DOWN   = BSLASH//'d'
!
! Zero is always printed as "0".
!
   IF (MM.EQ.0) THEN
       STRING = '0'
       NC = 1
       RETURN
   END IF
!
! If negative, make a note of that fact.
!
   MINUS = MM.LT.0
   M = ABS(MM)
   P = PP
!
! Convert M to a left-justified digit string in WORK. As M is a
! positive integer, it cannot use more than 10 digits (2147483647).
!
   J = 10
10 IF (M.NE.0) THEN
       K = MOD(M,10)
       M = M/10
       WORK(J:J) = CHAR(ICHAR('0')+K)
       J = J-1
    GOTO 10
   END IF
   TEMP = WORK(J+1:)
   WORK = TEMP
   ND = 10-J
!
! Remove right-hand zeros, and increment P for each one removed.
! ND is the final number of significant digits in WORK, and P the
! power of 10 to be applied. Number of digits before decimal point
! is NBP.
!
20 IF (WORK(ND:ND).EQ.'0') THEN
       ND = ND-1
       P = P+1
    GOTO 20
   END IF
   NBP = ND+MIN(P,0)
!
! Integral numbers of 4 or less digits are formatted as such.
!
   IF ((P.GE.0) .AND. ((FORM.EQ.0 .AND. P+ND.LE.4) .OR.                   &
                       (FORM.EQ.1 .AND. P+ND.LE.10))) THEN
       DO 30 I=1,P
           ND = ND+1
           WORK(ND:ND) = '0'
30     CONTINUE
       P = 0
!
! If NBP is 4 or less, simply insert a decimal point in the right place.
!
   ELSE IF (FORM.NE.2.AND.NBP.GE.1.AND.NBP.LE.4.AND.NBP.LT.ND) THEN
       TEMP = WORK(NBP+1:ND)
       WORK(NBP+2:ND+1) = TEMP
       WORK(NBP+1:NBP+1) = '.'
       ND = ND+1
       P = 0
!
! Otherwise insert a decimal point after the first digit, and adjust P.
!
   ELSE
       P = P + ND - 1
       IF (FORM.NE.2 .AND. P.EQ.-1) THEN
           TEMP = WORK
           WORK = '0'//TEMP
           ND = ND+1
           P = 0
       ELSE IF (FORM.NE.2 .AND. P.EQ.-2) THEN
           TEMP = WORK
           WORK = '00'//TEMP
           ND = ND+2
           P = 0
       END IF
       IF (ND.GT.1) THEN
           TEMP = WORK(2:ND)
           WORK(3:ND+1) = TEMP
           WORK(2:2) = '.'
           ND = ND + 1
       END IF
   END IF
!
! Add exponent if necessary.
!
   IF (P.NE.0) THEN
!!!corrado       WORK(ND+1:ND+6) = TIMES//'10'//UP
!!!corrado       ND = ND+6
       WORK(ND+1:ND+4) = 'x10^'
       ND = ND+4
!!!corrado
       IF (P.LT.0) THEN
           P = -P
           ND = ND+1
           WORK(ND:ND) = '-'
       END IF
       J = 10
40     IF (P.NE.0) THEN
           K = MOD(P,10)
           P = P/10
           WEXP(J:J) = CHAR(ICHAR('0')+K)
           J = J-1
        GOTO 40
       END IF
       WORK(ND+1:) = WEXP(J+1:10)
       ND = ND+10-J
       IF (WORK(1:3).EQ.'1'//TIMES) THEN
           TEMP = WORK(4:)
           WORK = TEMP
           ND = ND-3
       END IF
!!!corrado       WORK(ND+1:ND+2) = DOWN
!!!corrado       ND = ND+2
   END IF
!
! Add minus sign if necessary and move result to output.
!
   IF (MINUS) THEN
      TEMP = WORK(1:ND)
      STRING = '-'//TEMP
      NC = ND+1
   ELSE
      STRING = WORK(1:ND)
      NC = ND
   END IF
!
! Check result fits.
!
   IF (NC.GT.LEN(STRING)) THEN
       STRING = '*'
       NC = 1
   END IF
   end subroutine grnumb
!-------------------------------------------------------------------------------------
   subroutine gr_hist(data,nbin,iflag)
!
!  Draw a histogram of a variable data
!  data : the data values
!  nbin  : the number of bins to use
!  iflag = 0 -> gr_setwin e' chiamata automaticamente da gr_hist
!
   real, dimension(:), intent(in) :: data
   integer, intent(in)            :: nbin
   integer, intent(in)            :: iflag
   real                           :: datamin,datamax
   integer, dimension(nbin)       :: numb
   integer                        :: ndat
   integer                        :: ibin
   integer                        :: i
   real                           :: cur
   real                           :: binsiz
   real                           :: xl
!
   ndat = size(data)                 ! numero di valori
   if (iflag == 0) then
       datamax = maxval(data)*1.01   ! allargo l'intervallo per far entrare nell'istogramma il massimo
       datamin = minval(data)
       call gr_setwin(datamin,datamax,0.0,real(maxval(numb)))
   else
       datamin = xgrinf
       datamax = xgrsup
   endif
!
!  Calcola il numero di valori per rettangolo
!
   numb = 0
   do i=1,ndat
      ibin = (data(i)-datamin)/(datamax-datamin)*nbin+1
      if (ibin <= nbin) numb(ibin) = numb(ibin) + 1
   enddo
!
!  Definisci il massimo e minimo dell'area grafica se iflag = 0
!
   binsiz = (datamax - datamin)/nbin
!
!  Disegna l'istogramma
!
   xl = datamin
   do ibin=1,nbin
      cur = numb(ibin)
      if (cur == 0.0) cycle
      xl = datamin + binsiz*(ibin-1)
      call gr_rect(xl,xl+binsiz,0.0,cur)
   enddo
!
   end subroutine gr_hist
!-------------------------------------------------------------------------------------
   subroutine gr_create_graph(graph)
!
!  Crea un grafico
!   
   type(graph_type), intent(out) :: graph
!
   call gr_init_graph(graph)
!
   end subroutine gr_create_graph
!-------------------------------------------------------------------------------------
   subroutine gr_init_graph(graph)
!
!  Inizializza grafico
!   
   USE Molcom
   type(graph_type), intent(out) :: graph
!
   graph%stringx = 'X'
   graph%stringy = 'Y'
   graph%color = color_BLACK
   graph%xscale = -1
   graph%zoom = .false.
   graph%y_resize = .false.
   graph%y_resize_val = 0
   graph%ntext = 0
   graph%spacex = 0.0
   graph%spacey = 0.05
   graph%stk = 'ABLTS'
!
   end subroutine gr_init_graph      
!-------------------------------------------------------------------------------------
   subroutine gr_set_graph(graph,stringx,stringy,color,xscale,spacex,spacey,stk,y_resize,y_resize_val)
!
!  Modifica le caratteristiche del grafico
!   
   type(graph_type), intent(inout)        :: graph
   character(len=*), intent(in), optional :: stringx,stringy
   integer, intent(in), optional          :: color
   real, intent(in), optional             :: xscale
   real, intent(in), optional             :: spacex,spacey
   character(len=*), optional             :: stk
   logical, intent(in), optional          :: y_resize
   real, intent(in), optional             :: y_resize_val
!
   if (present(stringx)) graph%stringx = stringx
   if (present(stringy)) graph%stringy = stringy
   if (present(color)) graph%color = color
   if (present(xscale)) graph%xscale = xscale
   if (present(spacex)) graph%spacex = spacex
   if (present(spacey)) graph%spacey = spacey
   if (present(stk)) graph%stk = stk
   if (present(y_resize)) graph%y_resize = y_resize
   if (present(y_resize_val)) graph%y_resize_val = y_resize_val
!
   end subroutine gr_set_graph   
!-------------------------------------------------------------------------------------

   subroutine gr_graph_set_text(graph,string,side,disp,coord,just)
   type(graph_type), intent(inout):: graph
   character(len = *), intent(in) :: string
   character(len = *), intent(in) :: side
   real              , intent(in) :: disp
   real              , intent(in) :: coord
   real              , intent(in) :: just
!
   graph%ntext = graph%ntext + 1
   if (graph%ntext > size(graph%text)) return
!
   graph%text(graph%ntext)%str = string
   graph%text(graph%ntext)%side = side
   graph%text(graph%ntext)%disp = disp
   graph%text(graph%ntext)%coord = coord
   graph%text(graph%ntext)%just = just
!
   end subroutine gr_graph_set_text

!-------------------------------------------------------------------------------------
   subroutine gr_zoom_graph(graph,action,perc)
!   
!  Attiva/disattiva lo zoom ridefinendo i limiti del grafico
!
   type(graph_type), intent(inout) :: graph       
   integer, intent(in)             :: action
   real, intent(in)                :: perc
   real                            :: diff
!
   if (action == 0) then
       graph%zoom = .false.
   else    
       graph%zoom = .true.
       diff = graph%zymax - graph%zymin
       if (diff <= 10e-2) return
       graph%zymax = graph%zymax - action*perc*diff
       graph%zymin = graph%zymin
   endif
!
   end subroutine gr_zoom_graph
!-------------------------------------------------------------------------------------
   subroutine gr_draw_graph(graph,plot)
!
!  Disegna un grafico   
!
   type(graph_type), intent(inout)            :: graph       ! il grafico
   type(plot_type), dimension(:), intent(inout)  :: plot        ! i plot da inserire nel grafico
   real, dimension(size(plot))                :: minyv,maxyv
   real, dimension(size(plot))                :: minxv,maxxv
   real                                       :: miny,maxy
   real                                       :: minx,maxx
   real                                       :: diff
   integer                                    :: nplot
   integer                                    :: npp
   integer                                    :: i
   real                                       :: nexp
   real                                       :: scalexval
   logical                                    :: autox_scale
   character(len=20)                          :: strn
   integer                                    :: np1,np2
!
!  Numero di plot
   nplot = size(plot)
!
!  Calcola il min e max su tutti i plot
   if (graph%zoom) then
       maxy = graph%zymax
       miny = graph%zymin
       call gr_get_limit(plot,miny,maxy)
   else        
       do i=1,nplot
          if (plot(i)%npoints == 0) cycle
          plot(i)%n1 = 1
          plot(i)%n2 = plot(i)%npoints
       enddo
   endif        
   npp = 0
   do i=1,nplot
      if (plot(i)%npoints == 0) cycle
      np1 = plot(i)%n1
      np2 = plot(i)%n2
      if (np2-np1+1 <= 0)cycle
      npp = npp + 1
      maxyv(npp) = maxval(plot(i)%point(np1:np2)%y)
      minyv(npp) = minval(plot(i)%point(np1:np2)%y)
      maxxv(npp) = maxval(plot(i)%point(np1:np2)%x)
      minxv(npp) = minval(plot(i)%point(np1:np2)%x)
      if (graph%y_resize) then    ! auto resize of y axis if  minimun val < y_resize_val
          if (maxyv(npp) > graph%y_resize_val .and. minyv(npp) < graph%y_resize_val) maxyv(npp) = graph%y_resize_val
      endif
   enddo
   if (npp == 0) return
   if (.not.graph%zoom) then
       maxy = maxval(maxyv(:npp))
       miny = minval(minyv(:npp))
       diff= maxy - miny
       if (abs(diff) <= 10e-2) diff = 0.05 
       maxy = maxy + diff*graph%spacey
       miny = miny - diff*graph%spacey
   endif
   graph%zymin = miny
   graph%zymax = maxy
   maxx = maxval(maxxv(:npp))
   minx = minval(minxv(:npp))
   diff= maxx - minx
   if (abs(diff) <= 10e-5) then
      diff = 10e-3
      maxx = maxx + diff*0.05
      minx = minx - diff*0.05
   else
      maxx = maxx + diff*graph%spacex
      minx = minx - diff*graph%spacex
   endif
!
!  Se xscale attivo l'asse x viene automaticamnte riscalato se diff > xscale
!  -- N.B. questa parte va generalizzata all'asse y e a valori piccoli --
   autox_scale = graph%xscale > 0 .and. diff > graph%xscale    
   if (autox_scale) then
       nexp = int(log10(diff))
       scalexval = 10**(-nexp)
       minx = minx*scalexval
       maxx = maxx*scalexval
   else
       scalexval = 1.0
   endif
!
!  Set viewport   
   call gr_setwin(minx,maxx,miny,maxy)
!
!  Axis
   call gr_setcolor(graph%color)
   call gr_box(graph%stk,0.0,0,graph%stk,0.0,0)
!
!  Stringhe degli assi
   if (autox_scale) then       
       write(strn,'(i0)')nint(10**nexp)
       call gr_reldrawtext('B',2.0,0.5,0.5,trim(graph%stringx)//'/'//trim(strn),2)
   else
       call gr_reldrawtext('B',2.0,0.5,0.5,graph%stringx,2)
   endif
!  call gr_reldrawtext('L',2.0,0.5,0.5,graph%stringy,2,0)   
   call gr_reldrawtext('L',4.0,0.5,0.0,graph%stringy,2,1)   
!
!  Plot
   do i=1,nplot
      call gr_draw_plot(plot(i),scalexval)
   enddo
!
!  Testo
   do i=1,graph%ntext
      call gr_reldrawtext(graph%text(i)%side,graph%text(i)%disp,graph%text(i)%coord,   &
                          graph%text(i)%just,graph%text(i)%str,2)
   enddo
!
   end subroutine gr_draw_graph
!-------------------------------------------------------------------------------------
   subroutine gr_new_plot(plot,initsize)
!
!  Crea un plot
!   
   type(plot_type), intent(inout) :: plot
   integer, intent(in)            :: initsize
!
   call gr_init_plot(plot)
   allocate(plot%point(initsize))
   plot%initsize = initsize
   plot%npoints = 0
!
   end subroutine gr_new_plot   
!-------------------------------------------------------------------------------------
   subroutine gr_init_plot(plot)
!
!  Inizializza plot
!   
   USE Molcom
   type(plot_type), intent(out) :: plot
!
   plot%existline = .true.
   plot%existmarkers = .false.
   plot%colorline = color_BLACK
   plot%colormarkers = color_BLACK
   plot%n1 = 1
!
   end subroutine gr_init_plot
!-------------------------------------------------------------------------------------
   subroutine gr_set_plot(plot,existline,existmarkers,colorline,colormarkers,n1)
!
!  Modifica le caratteristiche del plot
!   
   type(plot_type), intent(inout) :: plot
   logical, intent(in), optional  :: existline
   logical, intent(in), optional  :: existmarkers
   integer, intent(in), optional  :: colorline
   integer, intent(in), optional  :: colormarkers
   integer, intent(in), optional  :: n1
!
   if (present(existline)) plot%existline = existline
   if (present(existmarkers)) plot%existmarkers = existmarkers
   if (present(colorline)) plot%colorline = colorline
   if (present(colormarkers)) plot%colormarkers = colormarkers
   if (present(n1)) plot%n1 = n1
!
   end subroutine gr_set_plot
!-------------------------------------------------------------------------------------
   subroutine gr_add_point_plot(plot,xval,yval)
!
!  Aggiungi un punto al plot
!   
   USE pointmod
   type(plot_type), intent(inout) :: plot
   real, intent(in)               :: xval,yval
   integer                        :: npsize
!
   plot%npoints = plot%npoints + 1
   npsize = size(plot%point)
   if (plot%npoints > npsize) then
       call resize_points(plot%point,npsize+plot%initsize)
   endif
   plot%point(plot%npoints)%x = xval
   plot%point(plot%npoints)%y = yval
!
   end subroutine gr_add_point_plot
!-------------------------------------------------------------------------------------
   subroutine gr_add_points_plot(plot,xval,yval)
!
!  Aggiungi più punti al plot
!   
   USE pointmod
   type(plot_type), intent(inout) :: plot
   real, dimension(:), intent(in) :: xval,yval
   integer                        :: npsize
   integer                        :: initp
!
   initp = plot%npoints
   plot%npoints = plot%npoints + size(xval)
   npsize = size(plot%point)
   if (plot%npoints > npsize) then
       call resize_points(plot%point,npsize+plot%initsize)
   endif
   plot%point(initp+1:plot%npoints)%x = xval
   plot%point(initp+1:plot%npoints)%y = yval
!
   end subroutine gr_add_points_plot
!-------------------------------------------------------------------------------------
   subroutine gr_update_points(plot,xval,yval)
!
!  Aggiorna i punti del plot
!
   type(plot_type), intent(inout) :: plot
   real, dimension(:), intent(in) :: xval,yval
!
   plot%npoints = 0
   call gr_add_points_plot(plot,xval,yval)
!
   end subroutine gr_update_points
!-------------------------------------------------------------------------------------
   subroutine gr_get_limit(plot,ymin,ymax)
!   
!  Cerca i limiti n1 e n2 dei plot a partire dai limiti del grafico ymin ymax
!
   type(plot_type), dimension(:), intent(inout) :: plot        
   real, intent(in)                             :: ymin,ymax
   integer                                      :: npo,npn,nplim
   integer                                      :: i,j
!   
   do i=1,size(plot)
      npo = plot(i)%npoints
      if (npo == 0) cycle
      npn = 0
      do j=1,npo
         if (plot(i)%point(j)%y >= ymin .and. plot(i)%point(j)%y <= ymax) then
             npn = npn + 1    
             nplim = j
             if (npn == 1) plot(i)%n1 = nplim
         endif        
      enddo
      if (npn > 0) then
          plot(i)%n2 = nplim
      else    
          plot(i)%n1 = 1
          plot(i)%n2 = 0
      endif
   enddo
!
   end subroutine gr_get_limit
!-------------------------------------------------------------------------------------
   subroutine gr_draw_plot(plot,scale)
!
!  Disegna plot
!   
   type(plot_type), intent(in) :: plot
   real, intent(in)            :: scale
   integer                     :: np1,np2
!
   if (plot%npoints == 0) return  ! non ci sono punti da disegnare
!   
!  Devi disegnare punti da np1 a np2
   np1 = plot%n1
   np2 = plot%n2
   if (np2 - np1 + 1 == 1) then
!           
!      Un solo punto da disegnare
       call gr_setsize(2)
       call gr_setcolor(plot%colormarkers)
       call gr_drawmarkers(plot%point(np1:np2)%x*scale,plot%point(np1:np2)%y,4)
   else           
!
!      Line
       if (plot%existline) then
           call gr_setcolor(plot%colorline)
           call gr_drawline(plot%point(np1:np2)%x*scale,plot%point(np1:np2)%y)
       endif
!
!      Markers
       if (plot%existmarkers) then
           call gr_setsize(2)
           call gr_setcolor(plot%colormarkers)
           call gr_drawmarkers(plot%point(np1:np2)%x*scale,plot%point(np1:np2)%y,4)
       endif
   endif
!
   end subroutine gr_draw_plot
   
!----------------------------------------------------------------------------------------------------

   subroutine clear_plot(plots,iview)
   type(plot_type), dimension(:), allocatable, intent(inout) :: plots
   integer, intent(in), optional                             :: iview
!
   if (allocated(plots)) deallocate(plots)
   if (present(iview)) then
       call gr_select_viewer(iview)
       call gr_clear_all_area()
       call zflush()
       call gr_select_viewer(0)
   else
       call gr_clear_all_area()
   endif
!
   end subroutine clear_plot

!----------------------------------------------------------------------------------------------------

   subroutine PlotXY(xval,yval,wtitle,xtitle,ytitle)
   USE Molcom
   real, dimension(:), intent(in)         :: xval,yval
   character(len=*), intent(in), optional :: wtitle
   character(len=*), intent(in), optional :: xtitle
   character(len=*), intent(in), optional :: ytitle
!   
   call gr_create_graph(grgraph)
   if (present(xtitle))call gr_set_graph(grgraph,stringx=xtitle)
   if (present(ytitle))call gr_set_graph(grgraph,stringy=ytitle)
   if (allocated(grplot)) deallocate(grplot)
   allocate(grplot(1))
   call gr_new_plot(grplot(1),1000)
   call gr_set_plot(grplot(1),colorline=color_LIGHTRED)
   if (present(wtitle)) then
       call gr_open_viewer(500,270,wtitle,newv=.true.)
   else
       call gr_open_viewer(500,270,' ',newv=.true.)
   endif           
   call gr_setbackground(color_BRIGHTWHITE)
   call gr_add_points_plot(grplot(1),xval,yval)
   call gr_select_viewer(1)
   call gr_area(0.08,0.99,0.10,0.99)
   call gr_draw_graph(grgraph,grplot)
   call zflush()
   call gr_select_viewer(0)
!
   end subroutine PlotXY

END MODULE GREXPOMOD
