module mol2_frm

implicit none

contains

   subroutine read_mol2file(filename,atom,legm,spg,cellmol2,error)
!
!  Legge da MDL (Molecular Design Limited) MolFile specie,coord,connettivita'
!   
   USE atom_type_util
   USE strutil
   USE cgeom
   USE errormod
   USE connect_mod
   USE spginfom
   USE elements
   USE fileutil
   USE unit_cell
!
   character(len=*), intent(in)                              :: filename
   type(atom_type), allocatable, dimension(:), intent(inout) :: atom  ! atomi in uscita
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm  ! connettivita'
   type(spaceg_type), intent(out)                            :: spg   ! space group
   type(cell_type), intent(out)                              :: cellmol2
   type(error_type), intent(out)                             :: error ! errore in lettura
   integer                                                   :: i,j
   character(len=500)                                        :: line,line2
   integer                                                   :: pos
   integer                                                   :: natom
   integer                                                   :: nconn
   real, dimension(50)                                       :: vet
   integer, dimension(50)                                    :: ivet
   integer                                                   :: iv
   integer                                                   :: icc
   integer                                                   :: nline
   integer                                                   :: ier
   character(len=10)                                         :: strn
   integer                                                   :: nlong1,nlong2
   type(file_handle)                                         :: ff
   integer                                                   :: j_in
!
   call ff%fopen(filename)
   if (ff%fail()) then
       call error%set('Cannot open: '//trim(filename)//char(10)//'Message: '//trim(ff%err_msg()))
       return
   endif
   j_in = ff%handle()
!
!  Leggi blocco @<TRIPOS>MOLECULE
   nline = 0
   do
      nline = nline + 1
      read(j_in,'(a)',iostat=ier)line
      if (ier < 0) then
          call error%set('It does not seem mol2 file')
          go to 20
      endif    
      if(index(line,'@<TRIPOS>MOLECULE') > 0 ) exit
   enddo
   nline = nline + 1
   read(j_in,'(a)',err=10)line
   nline = nline + 1
   read(j_in,'(a)',err=10)line
   call Getnum(line,vet,ivet,iv)
   if (iv < 2) go to 10
   natom = ivet(1)
   if (natom == 0) then
       go to 10
   else           
       allocate(atom(natom))
   endif
   nconn = ivet(2)
   spg = init_spaceg_type()
!corr   cellmol2(:) = (/10.0,10.0,10.0,90.,90.,90./) 
   cellmol2 = cell_init()
!   
!  Leggi blocco @<TRIPOS>ATOM
   call find_key_file(j_in,'@<TRIPOS>ATOM',nline,line,ier)
   if (ier < 0) then
       call error%set('It does not seem mol2 file')
       go to 20
   endif    
   do i=1,natom
      nline = nline + 1
      read(j_in,'(a)',err=10)line2
      call cutst(line2,nlong1,line,nlong2)
      call cutst(line2,nlong1,line,nlong2)
!      
!     Leggi le coordinate
      do j=1,3
         call cutst(line2,nlong1,line,nlong2)      
         if (nlong2 == 0) go to 10
         ier = s_to_r(line,atom(i)%xc(j),nlong1)
         if (ier > 0) go to 10
      enddo
!      
!     Leggi atom type
      call cutst(line2,nlong1,line,nlong2)
      if (nlong2 == 0) go to 10
      pos = index(line,'.') - 1
      if (pos <= 0) then
          atom(i)%lab = line(:nlong2)
      else              
          atom(i)%lab = line(:pos)
      endif
   enddo
!
!  Leggi blocco @<TRIPOS>BOND
   call find_key_file(j_in,'@<TRIPOS>BOND',nline,line,ier)
   if (ier < 0) then
       call error%set('Cannot find <TRIPOS>BOND section')
       go to 20
   endif    
   call new_bonds(legm,nconn)
   icc = 1
   do i=1,nconn
      nline = nline + 1
      read(j_in,'(a)',err=10)line
      call Getnum(line,vet,ivet,iv)
      if (iv < 3) go to 10
      if (ivet(2) > natom) then
          write(0,30)nline,ivet(2)
          cycle
      endif    
      if (ivet(3) > natom) then
          write(0,30)nline,ivet(3)
          cycle
      endif    
      legm(i)%n1 = ivet(2)
      legm(i)%n2 = ivet(3)
      legm(i)%dist = distanzaC(atom(ivet(2))%xc,atom(ivet(3))%xc)
      legm(i)%sigma = 0.03
   enddo   
!
!  Read crystallografic information
   call find_key_file(j_in,'@<TRIPOS>CRYSIN',nline,line,ier)
   if (ier == 0) then
       read(j_in,'(a)',iostat=ier)line
       if (ier == 0) then
           call Getnum(line,vet,ivet,iv)
           if (iv == 8) then
!corr               cellmol2(:6) = vet(:6)
               cellmol2 = set_cell_type(vet(:6))
               spg = init_spaceg_type(numb = ivet(7),code=ivet(8))
           endif
       endif
       !!!!write(0,*)'LINE=',vet(:8)
   endif
!
!  Metti in nz il puntatore al file xen
   do i=1,natom
      atom(i)%ptab = pxen_from_specie(atom(i)%lab)
   enddo
!
   return      
!      
10 write(strn,'(i0)')nline
!corr   call set_error(error,'Error on reading line '//trim(strn)//' of mol2 file')   
   call error%set('Error on reading line '//trim(strn)//' of mol2 file')   
30 format('WARNING (line ',i0,'): Atom with serial #',i0,' was not found')
20 continue
!
   end subroutine read_mol2file  

end module mol2_frm
