      subroutine pirum(ier)
C***********************************************************************
C     January 2000
C     Extinction condition for rhombohedral cells using hexagonal
C     settings included. Rhombohedral cells can now be refined using
C     hexagonal setting and the only extiction condition R 
C     ....or it may be refined in the rhombohedral form.
C     (See CONDITION>INFO ). 
C     As only positive (HKL)s' are generated for hexagonal cells the
C     kind of condition (H -H  L) L=2N is not needed.
C     For example: (1 -1 2) has exactly identical 2-theta as (102) and
C     (012) (The last one is allowed for a rhombohedral cell using
C     hexagonal setting.)
C     Thus, no other extinction condition than R should be used for
C     rhombohedral cells in hexagonal setting.
C
C     If a rhombohedral reflexion file is generated (IP=1) identical
C     extinction conditions must be given in the following IP=2 run
C     (i.e. when fixed indices are used). 
C
C
C     It is possible to convert rhombohedral to/from hexagonal
C     cell parameters. (See the CHANGE commands). 
C
C     Errors found in subroutine PW57 (for IP=2 cubic, tetragonal and
C     hexagonal cells) are now corrected.  (January 2000, P.E.Werner)
C     
C
C     PIRUM (PROGRAM FOR INDEXING AND REFINEMENT OF UNIT-CELL MATRIX)
C     DETERMINATION OF ACCURATE CELL PARAMETERS FROM POWDER DATA BY THE
C     METHOD OF THE LEAST SQUARES. MAX 200 SIN SQUARE THETA OBS.
C
C     (WRITTEN BY P-E WERNER,STOCKHOLM,1967)
C     PIRUM VERSION  1980.05.05
C
C     MENUE-CONTROLLED VERSION MADE BY L.ERIKSSON FEBR. 1992
C
C     HESS WEIGTHS, USE OF D1 AS MAX DEVIATIONS IN DEG. 2-THETA
C     (AND SSQTL SKIPPED), FIXED (HKL) REFINEMENTS (cf. PURUM)
C     P.-E. WERNER AUG. 1992
C***********************************************************************
C     SIO2 850125          ! This is the title line.
C     0.03285   2108.00    ! Sine square theta and Intensity. 
C     0.05282   9253.00    ! (intensity can be omitted...)
C     0.09851    877.00    !     .
C     0.11424   1009.00    !     .
C     0.11885    499.00    !     .
C     0.13138    565.00    !     .
C     0.15165    522.00    !     .
C     0.17987   2375.00    !     .
C     0.18320     34.00    !     .
C     0.21270    614.00    !     .
C     0.21599    281.00    !     .
C     0.22966     34.00    !     .
C     0.25015   1564.00    !     .
C     0.28174    334.00    !     .
C     0.29530     46.00    !     .
C     0.31124   1314.00    !     .
C     0.31462   1774.00    !     .
C     0.31588    629.00    !     .
C     0.35854    492.00    !     .
C     0.37713    661.00    !     .
C     0.39413    342.00    !     .
C     0.41326   1017.00    !     .
C     0.42453    778.00    !     .
C     0.42713    620.00    !     .
C     0.44750    514.00    !     .
C     0.45732     62.00    !     .
C     0.47924     81.00    !     .
C                          ! a blank line to indicate end of SSQ-data.
C     0.030000  0.000000  0.000000  1.540598   0   0   0  4  8  0
C     4.913400  5.405200  0.000000  0.000000   0.000000  0.000000  1
C-----------------------------------------------------------------------
C   The last two lines are the control parameters.
C-----------------------------------------------------------------------
C
c---------------------------------------------------------------------
      USE General, only:lo,dtor,alambda,pi
      USE peak_mod
      common /ureq1/ ncycl,nroset,nundset,nsec,jback,uvwt(8),
     *               itry,pfwhm,kequirip
      common /treoriterato/mwoolf(22,10),
     *                     celltreor(22,6,10),cellbest(22,6),mbest(22),
     *                     ntrialtreor,nsolut(22),negtrial,shiftzero(22)
      common/CellPirum/mpirbest(3),cellpirbest(3,6),zeroshiftpir(3),
     *                 iespir
      common /unindexed/nunindex(660),volc(660),ncalc(660),nobs(660)
C-----------------------------------------------------------------------
      common /bestone/icheckbest,shiftzerobest,IHKLbest(200,4),nrefbest
     *               ,m20best,mnew20
      common /pirumprint/isolpir,mbestprint(3,10,22),m20print(3,10,22),
     *                   shiftprint(3,10,22),cellbestprint(3,6,10,22)
     *                   ,noindprint(3,10,22),volcprint(3,10,22)
     *                   ,isysprint(3,10,22),centringprint(3,10,22)
      common/nodeparityInfo/warnpar(2),infopar(2)
      character*80 warnpar,infopar
C-----------------------------------------------------------------------
      common /fast/deltath,mnew20_old,iend,iminus,iplus,imatrice
!corr      common/genob/irho,thetamin
      common/genob2/thmin,thmax,thstep
      common/sistemacris/isys,isyss(660),isyst(10)
      common /centering/CellType,CellAxis,nruntre,iesdircell
      character CellType,CellAxis*3
      common /centringsav/CellTypeb(660),celltypesav(660)
      character CellTypeb,celltypesav,centringprint
*      common/trig/sint(45000),pi,twopi,dtor,rtod
!
      common/per_m20all/usedt
      common /ratiod/dratio,iproteina,volprot
      common/celltotal/celltot(6,100),m20tot(100),m20ptot(100),
!    *                 nobstot(100),ncalctot(100),voltot(100),
     *                 voltot(100),isimadd(100),
     *                 tthetatot(100),nixtot(100),isistot(100),nsoltot,
     *                 isognoind,imernoind,celladd(6),
!    *                 isymadd,mpm20,n1add,volm
     *                 isymadd,mpm20,zzzero,volm,iesadd,isoladd
!
      character cff
      character titstr*80, answer*1, command*80, cpar*80 !, fname*80
      integer   ludisp
      integer   exsymb(50,5),numext,ilatt
      logical   okflag
      integer :: npeak

      dimension amat(6,6),bmat(6,1),s(6,6),iht(400),
     >          ikt(400),ilt(400),iqe(400),t1(200),t2(400),tagut(200),
     >          t11(400),t22(400),wg(400),ihi(400),iki(400),ili(400)
c
c --- Logical unit numbers ...
c
      ikoref = 0
      iespir = 0
      do 3 i=1,3
         mpirbest(i)= -99999
         zeroshiftpir(i)= 0.0
         do 4 jj=1,6
            cellpirbest(i,jj)= - 99999.0
 4       continue
 3    continue
      do 1 i=1,200
        do 2 jj=1,4
         IHKLbest(i,jj) = 0
 2      continue
 1    continue
      icheckref = 0
      iApplyShift = 0
      iend = 0
      iplus = 0
      iminus = 0
      imatrice = 0
      iescheck=0
      isavpar = 0
c
      lukeys = 5
****  ludisp = lo
      ludisp = 9 
      iin    = 7
      iout   = 8
c-- lulist : output file
***** lulist = lo
      lulist = 9 
      ioup   = 9
      linfo  = 10
c
c --- some defalt values ...
c
      deltath = thstep/4.
      mnew20_old = 0
      ifind=0
      irun =0
      irefine =0
      ilast=0
      istop=0
      okflag = .false.
      numext = 0
      ilatt  = 1
      isub   = 0
      inr    = 0 ! Num. refinement cycles
!     isym   = isyss(ntrialtreor)
!
      if (iesadd.eq.0) then
          isym   = isyst(isolpir)
         else
          isym = isymadd
      endif
!
      iomv   = 1
      a11    = 10.0
      a22    = 0.0
      a33    = 0.0
      a13    = 0.0
      a12    = 0.0
      a23    = 0.0
      dt     = 0.03
      dtmax  = 0.08
!peso HESS      wt     = 0.0
      wt     = 0.0
      d2     = 0.0
      ip     = 0
      wave   = alambda
      nunind = 0
      ihm    = 15
      ikm    = 15
      ilm    = 15
      n1     = 0
      titstr = ' '
c-------------------------------
c-- Fill tagut
c-------------------------------
      npeak = numpeaks(pkind)
      if (iesadd.eq.0) then  ! verificare che e' corretto
!!!!! if (iesext.eq.1.and.npeak.gt.25) then
!
         npeak = min0(npeak,100)  ! anche se ho npeak > 100 prendo i primi 100 in ordine di 2-theta per indicizzazione
!
         do 15 i = 1,npeak
!corr            shiftth= twothesav(i,1)/2. - shiftzero(ntrialtreor)
            shiftth= pkind(i)%getx()/2. - shiftzero(ntrialtreor)
            if (iproteina.eq.0) then
                tagut(i) = (sin(shiftth*dtor))**2
              else
                tagut(i) = (dratio*sin(shiftth*dtor))**2
            endif
            t1(i) = tagut(i) 
 15      continue
!!!!!   else
!!!!!    do 20 i = 1,npeak
!!!!!       if (iproteina.eq.0) then
!!!!!           tagut(i) = (sin(0.5*xmax(i)*dtor))**2
!!!!!         else
!!!!!           tagut(i) = (dratio*sin(0.5*xmax(i)*dtor))**2
!!!!!       endif
!!!!!       t1(i) = tagut(i) 
!!!!! 20      continue
!!!!! endif
      endif
c-------------------------------
      if (iesadd.gt.0) then  ! verificare che e' corretto
!        write(6,*)' isoladd tthetatot(isoladd',
!    *         isoladd,tthetatot(isoladd)
         do i = 1,npeak
!corr            shiftth= twothesav(i,1)/2. - tthetatot(isoladd)/2.
            shiftth= pkind(i)%getx()/2. - tthetatot(isoladd)/2.
            if (iproteina.eq.0) then
                tagut(i) = (sin(shiftth*dtor))**2
              else
                tagut(i) = (dratio*sin(shiftth*dtor))**2
            endif
            t1(i) = tagut(i) 
         enddo
      endif
      n1 = npeak
      noindold = n1
c-------------------------------
c-  Load centring type
c-------------------------------
      if (CellType.eq.'P') then
          write (ludisp,'(a)') ' '
          write (ludisp,'(a)') '  Primitive lattice '
          write (ludisp,'(a)') ' '
      endif
      if (CellType.eq.'I') then
          ilatt = 2
          write (ludisp,'(a)') ' '
          write (ludisp,'(a)') '  I-centred lattice '
          write (ludisp,'(a)') ' '
      endif
      if (CellType.eq.'F') then
          ilatt = 3
          write (ludisp,'(a)') ' '
          write (ludisp,'(a)') '  F-centred lattice '
          write (ludisp,'(a)') ' '
      endif
      if (CellType.eq.'A') then
          ilatt = 4
          write (ludisp,'(a)') ' '
          write (ludisp,'(a)') '  A-centred lattice '
          write (ludisp,'(a)') ' '
      endif
      if (CellType.eq.'B') then
          ilatt = 5
          write (ludisp,'(a)') ' '
          write (ludisp,'(a)') '  B-centred lattice '
          write (ludisp,'(a)') ' '
      endif
      if (CellType.eq.'C') then
          ilatt = 6
          write (ludisp,'(a)') ' '
          write (ludisp,'(a)') '  C-centred lattice '
          write (ludisp,'(a)') ' '
      endif
      if (CellType.eq.'R') then
          ilatt = 7
          write (ludisp,'(a)') ' '
          write (ludisp,'(a)') '  R-centred lattice '
          write (ludisp,'(a)') ' '
      endif
      if (ilatt.eq.1) then
          isub = 0
       else
          isub = 1
      endif
c----------------------------------
c---  Load cell found by N-TREOR
c----------------------------------
      if (iesadd.eq.0) then
          call chaaij (isym,a11,a22,a33,a13,a12,a23)
        else
          call chaaij_add (isym,a11,a22,a33,a13,a12,a23)
      endif
      if (CellAxis.eq.'A/2') then
          a11=a11/2.
          CellAxis='   '
      endif
      if (CellAxis.eq.'B/2') then
          a22=a22/2.
          CellAxis='   '
      endif
      if (CellAxis.eq.'C/2') then
          a33=a33/2.
          CellAxis='   '
      endif
c-------------------------------
      a11sav = a11
      a22sav = a22
      a33sav = a33
      a12sav = a12
      a13sav = a13
      a23sav = a23
!     write (ludisp,'(a)')
!  10 format(///,a1,78('+'),//,
!    1 39h expo2005  : PIRUM   routine           ,26x,14hRelease 02.01
*    2 ,/,/,20x,20a4,/,1h ,78('+'),//)
*    2 ,/,46h             by Courtesy of Prof. P. E. Werner              
!    3 ,/,20x,20a4,/,1h ,78('+'),//)
  19  format(1x,///,
!!!  *62h The following cell has been selected for refinement by PIRUM:,
     *40h ***************************************,/,
     *14h    The cell #,i3,20h is refined by PIRUM,/,
     *40h ***************************************,/,
     *//,8x,48h     ****  PIRUM: Starting cell parameters **** ,/)
!
               a1=cos(cellbest(ntrialtreor,2)*pi/180.0)
               a2=cos(cellbest(ntrialtreor,4)*pi/180.0)
               a3=cos(cellbest(ntrialtreor,6)*pi/180.0)
               arg=1.0-a1*a1
     1             -a2*a2
     2             -a3*a3
     3             +a1*a2*a3*2.0
        volumes=cellbest(ntrialtreor,1)*cellbest(ntrialtreor,3)*
     *          cellbest(ntrialtreor,5)*sqrt(arg)

!
      if (iesadd.eq.0) then
!
!!!   write (lo,19)
      write (lo,19) isolpir
      write (lo,18) cellbest(ntrialtreor,1),cellbest(ntrialtreor,3)
     *      ,cellbest(ntrialtreor,5),cellbest(ntrialtreor,2),
     *      cellbest(ntrialtreor,4),cellbest(ntrialtreor,6),volumes
!!!   write (ludisp,19)
      write (ludisp,19) isolpir
      write (ludisp,18) cellbest(ntrialtreor,1),cellbest(ntrialtreor,3)
     *      ,cellbest(ntrialtreor,5),cellbest(ntrialtreor,2),
     *      cellbest(ntrialtreor,4),cellbest(ntrialtreor,6)
         else
!
!!!   write (lo,19)
      write (lo,19) isoladd
      write (lo,18) celltot(1,isoladd),celltot(3,isoladd)
     *      ,celltot(5,isoladd),celltot(2,isoladd),
     *      celltot(4,isoladd),celltot(6,isoladd)
!!!   write (ludisp,19)
      write (ludisp,19) isoladd
      write (ludisp,18) celltot(1,isoladd),celltot(3,isoladd)
     *      ,celltot(5,isoladd),celltot(2,isoladd),
     *      celltot(4,isoladd),celltot(6,isoladd)

!
      endif
!18   format(9h A     = ,f9.5,10h   B    = ,f9.5,11h   C     = ,f9.5,/,
!    * 9h Alpha = ,f9.5,10h   Beta = ,f9.5,11h   Gamma = ,f9.5,/)
 18   format(6h    A ,' =',f10.5,3x,6h    B ,' =',
     *        f10.5,3x,6h    C ,' =',f10.5,/,6h Alpha,' =',
     *        f10.5,3x,6h Beta ,' =',f10.5,3x,6h Gamma,' =',f10.5,//,
     *        '  Unit cell volume =',f10.2,' A**3',/)
!!!mettere     if (iesadd.eq.0) then
!iprot      if (iproteina.eq.1) then
!iprot          a = cellbest(ntrialtreor,1)
!iprot          b = cellbest(ntrialtreor,3)
!iprot          c = cellbest(ntrialtreor,5)
!iprot          al = cellbest(ntrialtreor,2)
!iprot          be =  cellbest(ntrialtreor,4)
!iprot          ga =  cellbest(ntrialtreor,6)
!!!       mpm20 = mbest(ntrialtreor)
!iprot          mpm20 = mwoolf(ntrialtreor,isolpir)
!iprot          noind = nunindex(isolpir)
!iprot          if (isym.eq.1)then
!iprot              m20 = mpm20/6
!iprot          elseif (isym.eq.2) then
!iprot              m20 = mpm20/5
!iprot          elseif (isym.eq.3) then
!iprot              m20 = mpm20/4
!iprot          elseif (isym.eq.4) then
!iprot              m20 = mpm20/5
!iprot          elseif (isym.eq.5) then
!iprot              m20 = mpm20/3
!iprot          elseif (isym.eq.6) then
!iprot              m20 = mpm20
!iprot          endif
!iprot          iend = 0
!iprot          irefine = 1
!
!iprot          imatrice = 1
!
!iprot          noindp = 0
!iprot          istop = 2
!iprot          volm=volprot
!!!!      call FillCell(M20,MpM20,irefine,a,al,b,be,c,ga,noindp,
!iprot          call FillCell(M20,MpM20,irefine,a,al,b,be,c,ga,noind,
!iprot     *                  volm,icheckref)
!iprot          go to 680
!iprot      endif
!!!mettere      endif
c-------------------------------
c--- dt automatic choice
c-------------------------------
c  
  130 continue
       if(ip .eq. 3) goto 700
      goto 140
  700 continue
c-----------------------------------------------------------------------
c     Open the output file and then run the program PIRUM part. ...
c-----------------------------------------------------------------------
      iout = lulist
c-----------------------------------------------------------------------

  780 CONTINUE
C 
C    Now use fixed (hkl) to calc. a11.....a23
       iout=ludisp
      i=0
      do while (i .lt. n1)
      i=i+1
      ha=sqrt(t1(i))
      hb=sqrt(1.-t1(i))
      t11(i)=114.59156*atan(ha/hb)
      ssqto=t1(i)
      ha=sqrt(ssqto)
      hb=sqrt(1.-ssqto)
      t22(i)=114.59156*atan(ha/hb)
       if(wt .eq. 1.) then
         wg(i)=1.
         else 
       wg(i)=1./((2.*ha*hb)**2)
!        elseif (wt.eq.0.0) then
!      wg(i)=1./((2.*ha*hb)**2)
!        elseif (wt.eq.2.0) then
!      wg(i)=wave/(2.*ha)
       endif
      iqe(i)=1
      enddo
      call pw58(ihi,iki,ili,n1,isym)
      call p55w(t1,wg,iqe,iht,ikt,ilt,n1,isym,amat,bmat,iekv)
      if(isym .eq. 1) nsy=1
      if(isym .eq. 2 .or. isym .eq. 4 .or. isym .eq. 7) nsy=2
      if(isym .eq. 3) nsy=3
      if(isym .eq. 5) nsy=4
      if(isym .eq. 6) nsy=6
      call pw54(amat,nsy,bmat)
      a11=bmat(1,1)
      a22=bmat(2,1)
      a33=bmat(3,1)
      a13=bmat(4,1)
      a12=bmat(5,1)
      a23=bmat(6,1)
       iout=ludisp
      write(iout,'(6f10.6)') a11,a22,a33,a13,a12,a23
c     calc. error square sum (v2) and (delta-2heta) square sum.
      call pww71(t1,wg,iht,ikt,ilt,iqe,isym,n1,a11,a22,a33,a13,a12,a23,
     >          v2,v22,t22,nekv)      
c     calc. direct cell parameters.
      call pw63(a11,a22,a33,a13,a12,a23,wave,isym,a,b,c,al,be,ga)
c     calc. partial derivatives of direct cell parameters (in s(6,6))
       if(isym .eq. 6) then
       call pw61(a,b,c,al,be,ga,s)
       else
       call pww62(a,b,c,be,s,isym,al)
       endif
c     calc. direct cell standard deviations --from amat(6,6), s(6,6)
c     and cell parameters.
      call pww52(nsy,nekv,isym,v2,wave,amat,s,a,b,c,al,be,ga,
     >          da,db,dc,dal,dbe,dga,volm)
c-----------------------------------------------------------
c
c
c     calc. ssqt-max and 
      ssqtm=(sin((0.5*t11(n1)+2.0*dt)*0.01745329))**2
      if(n1-30) 710,711,711
  711 nt30=30
      sq30=t1(30)
      go to 715
  710 nt30=0
      sq30=0
      if(n1-20) 716,715,715
  715 nt20=20
      sq20=t1(20)
      go to 717
  716 nt20=0
      sq20=0.0
  717 n2=0
      ih=0
      ik=0
      il=0
      jo=0
      n20=0
      n30=0
      nfi=0
      sqfi=t1(n1)
  733 call pww50
     >  (a11,a22,a33,a13,a12,a23,isym,ihm,ikm,ilm,ih,ik,il,ssqt,ia,jo)
      if(ia) 730,750,730
  730 if(ssqt-ssqtm) 731,731,733
  731 if(isub) 732,734,732
  732 call exti(ih,ik,il,ij,numext,exsymb,ilatt)
      if(ij) 734,733,734
  734 call pww51(ssqt,t1,n1,t11,dt,ssqto,ia)
      if(ssqt-sqfi) 735,735,736
  735 call pww80(ssqt,t2,n2,iht,ikt,ilt,ih,ik,il,isym,inne)
      if(inne) 737,737,738
  737 nfi=nfi+1
  736 if(ssqt-sq30) 739,739,740
  739 n30=n30+1
  740 if(ssqt-sq20) 741,741,738
  741 n20=n20+1
  738 if(ia) 742,733,742
  742 call pww53(ssqt,t2,t22,wg,n2,iht,ikt,ilt,iqe,ih,ik,il,isym,wt)
      if(ssqt) 743,733,733
  743 n2=n2+1
      if(n2-400) 733,750,750
750   continue
         write(iout,751)
     >   ' Number of lines (=number of equations) =',NEKV,
     >   ' A =',A,DA,' A   Alfa  =',AL,DAL,' Degrees',
     >   ' B =',B,DB,' A   Beta  =',BE,DBE,' Degrees',
     >   ' C =',C,DC,' A   Gamma =',GA,DGA,' Degrees',
     >   ' Unit cell volume =',VOLM,' A**3'
 751     format(' ',A,I5,/,3(' ',A,2F10.6,A,2F10.6,A,/),A,F10.2,A)
      call pw57(t1,t2,iht,ikt,ilt,n1,n2,iout,dt,wt,wave,tagut,n20,
     >          n30,nfi,sq20,sq30,sqfi,m20,noind,t11,t22,ihi,iki,
     >          ili,a11,a22,a33,a13,a12,a23,isym,ilatt)
      iout=-iout
c----
c---- noindp = num. righe non indic. calc. considerando tutti i picchi osservati
c---- noind  
      call pw56(t1,t2,iht,ikt,ilt,n1,n2,iout,dt,wt,ip,wave,tagut,n20,
     >         n30,nfi,sq20,sq30,sqfi,m20,noind,t11,t22,ioup,ilatt,
     >         noindold,ifind,ilast,irun,irefine,istop,noindp,usedt)
      write(iout,'(a,f10.6)')' Summa(Delta(2th)**2)=',v22
c-----
       do 776 i=1,n1
       iht(i)=ihi(i)
       ikt(i)=iki(i)
       ilt(i)=ili(i)
 776   continue
       goto 777
c      write(iout,'(a,f10.6)')
c     >      ' Summa(Delta(2th)**2)=',v22
c----------------------------------------------------------------------+
c --- For this time end of the calculations, close output file and     !
c     set the number of entries in the sine square theta table to      !
c     the original value (It is decreased to zero by some routines).   !
c     At last return to the main many.                                 !
CC PEW 2008       stop     Why? Cannot go to this...
  140 continue
c
c
c --- First check that all spacing data are less than 1.0
c
      do 141 i = 1,n1
         if (t1(i) .gt. 1.0) then
            write (ludisp,'(a)') ' '
            write (ludisp,'(a,i3,a,f12.6,a)')
     >      ' WARNING, observed spacing:',i,' = ',t1(i),' which is > 1'
            write (ludisp,'(a)')' Check your data'
            return
         endif
      ha=sqrt(t1(i))
      hb=sqrt(1.-t1(i))
      t11(i)=114.59156*atan(ha/hb)
C      write(ludisp,'(a,f10.5,a,f10.5)')
C     >      ' ssq-obs:',t1(i),' two-theta:',t11(i)
 141  continue

c-----------------------------------------------------------------------
c     Open the output file and then run the program PIRUM ...
c-----------------------------------------------------------------------
         iout = lulist
  160 CONTINUE
C
      CONTINUE
c
c --- If IOMV <> 0 then convert the real cell parameters
c     to normal linear parameters, and set IOMV = 0
c
      if (iomv .ne. 0) then
         call rdcel(a11,a22,a33,a13,a12,a23,isym,wave)
         iomv = 0
      endif
c
c --- Write the title to the output file.
c
      ler1 = istart(titstr)
      ler2 = last(titstr)
      if (ler2 .gt. ler1) then
         write(iout,'(a,a)') ' ',titstr(ler1:ler2)
      else
         write(iout,'(a)') ' Input data.'
         if (isavpar.eq.0) then
             a11sav = a11
             a22sav = a22
             a33sav = a33
             a12sav = a12
             a13sav = a13
             a23sav = a23
             isavpar = 1
         endif
      endif

      write(iout,'(6f10.6,i10)') a11,a22,a33,a13,a12,a23,iomv
!!!!!!!!!!!
      if (a11.lt.0.0.or.a22.lt.0.0.or.a33.lt.0.0) then   ! da not a number per struttura pt34 e hydrocholoro
      write(iout,'(a)')' **********************************************'
      write(iout,'(a)') ' '
      write(iout,'(a)') '  Refinement problems.'
      write(iout,'(a)') '  The refinement procedure has been stopped. '
      write(iout,'(a)') ' '
      write(iout,'(a)')' **********************************************'
      write(lo,'(a)')' **********************************************'
      write(lo,'(a)') ' '
      write(lo,'(a)') '  Refinement problems.'
      write(lo,'(a)') '  The refinement procedure has been stopped. '
      write(lo,'(a)') ' '
      write(lo,'(a)')' **********************************************'
             a11 = a11sav
             a22 = a22sav
             a33 = a33sav
             a12 = a12sav
             a13 = a13sav
             a23 = a23sav
             inr = 0
             ier = -111
             return
      endif
!!!!!!!!!!!
      write(iout,'(a,i5)') ' Number of observed lines = ',N1
      write(iout,'(a)') ' Cycle results.'
      if(ihm+ikm+ilm) 280,190,280
C  190 ssqtm=t1(n1)+2*d2
  190 ssqtm=(sin((0.5*t11(n1)+2.0*dt)*0.01745329))**2
  200 ihm=ihm+1
      if(ssqtm-ihm*ihm*a11) 210,200,200
  210 ihm=ihm-1
      go to(220,230,250,230,250,250,230),isym
  220 ikm=ihm
      ilm=ihm
      go to 280
  230 ikm=ihm
  240 ilm=ilm+1
      if(ssqtm-ilm*ilm*a22) 280,240,240
  250 ikm=ikm+1
      if(ssqtm-ikm*ikm*a22) 260,250,250
  260 ikm=ikm-1
  270 ilm=ilm+1
      if(ssqtm-ilm*ilm*a33) 280,270,270
  280 iv=inr
      idir=inr
C      ssqtm=t1(n1)+d2
      ssqtm=(sin((0.5*t11(n1)+2.0*dt)*0.01745329))**2  
      if(n1-30) 300,290,290
  290 nt30=30
      sq30=t1(30)
      go to 310
  300 nt30=0
      sq30=0
      if(n1-20) 320,310,310
  310 nt20=20
      sq20=t1(20)
      go to 330
  320 nt20=0
      sq20=0.0
  330 n2=0
      ih=0
      ik=0
      il=0
      jo=0
      n20=0
      n30=0
      nfi=0
      sqfi=t1(n1)
  340 continue
      call pww50
     >  (a11,a22,a33,a13,a12,a23,isym,ihm,ikm,ilm,ih,ik,il,ssqt,ia,jo)
      if(ia) 350,510,350
  350 if(ssqt-ssqtm) 360,360,340
  360 if(isub) 370,380,370
  370 call exti(ih,ik,il,ij,numext,exsymb,ilatt)
      if(ij) 380,340,380
  380 call pww51(ssqt,t1,n1,t11,dt,ssqto,ia)
      if(ssqt-sqfi) 390,390,410
  390 call pww80(ssqt,t2,n2,iht,ikt,ilt,ih,ik,il,isym,inne)
      if(inne) 400,400,450
  400 nfi=nfi+1
  410 if(ssqt-sq30) 420,420,430
  420 n30=n30+1
  430 if(ssqt-sq20) 440,440,450
  440 n20=n20+1
  450 if(ia) 460,340,460
  460 if(inr) 490,470,490
  470 call pww53(ssqt,t2,t22,wg,n2,iht,ikt,ilt,iqe,ih,ik,il,isym,wt)
      if(ssqt) 480,340,340
  480 n2=n2+1
      if(n2-400) 340,670,670
  490 call pww53(ssqto,t2,t22,wg,n2,iht,ikt,ilt,iqe,ih,ik,il,isym,wt)
      if(ssqto) 500,340,340
  500 n2=n2+1
      if(n2-400) 340,510,510
  510 if(inr) 520,660,520
  520 if(iv) 530,540,530
  530 iv=iv-1
      go to 580
  540 call pww71(t2,wg,iht,ikt,ilt,iqe,isym,n2,a11,a22,a33,a13,a12,a23,
     >          v2,v22,t22,nekv)
      inr=0
      call pw63(a11,a22,a33,a13,a12,a23,wave,isym,a,b,c,al,be,ga)
      if(isym-6) 560,550,560
  550 call pw61(a,b,c,al,be,ga,s)
      go to 570
  560 call pww62(a,b,c,be,s,isym,al)
  570 continue
      call pww52(n,nekv,isym,v2,wave,amat,s,a,b,c,al,be,ga,
     >          da,db,dc,dal,dbe,dga,volm)
      go to 330
  580 call p55w(t2,wg,iqe,iht,ikt,ilt,n2,isym,amat,bmat,iekv)
      if(iekv.ne.0) goto 600
      write(iout,'(a)') ' '
      write(iout,'(a)')' **********************************************'
      write(iout,'(a)') ' '
      write(iout,'(a)') '  Number of eq less than number of parameters.'
      write(iout,'(a)') '  The refinement procedure has been stopped. '
      write(iout,'(a)') ' '
      write(iout,'(a)')' **********************************************'
      write(lo,'(a)') ' '
      write(lo,'(a)')' **********************************************'
      write(lo,'(a)') ' '
      write(lo,'(a)') '  Number of eq less than number of parameters.'
      write(lo,'(a)') '  The refinement procedure has been stopped. '
      write(lo,'(a)') ' '
      write(lo,'(a)')' **********************************************'
          ier = -111
          mnew20_old=-999.
          if (iend.eq.0) imatrice = 1
          if (iend.eq.1) then
              imatrice = imatrice + 1
              if (iplus.eq.0.and.imatrice.eq.1) then
                 goto 1333
               elseif (iminus.eq.0.and.imatrice.ge.1) then
                 goto 1333
              endif
              if (imatrice.ge.2) then
                  goto 681
              endif
          endif
      goto 680
  600 n=1
      go to (650,640,630,640,620,610,640),isym
  610 n=n+2
  620 n=n+1
  630 n=n+1
  640 n=n+1
  650 continue
      call pw54(amat,n,bmat)
      a11=bmat(1,1)
      a22=bmat(2,1)
      a33=bmat(3,1)
      a13=bmat(4,1)
      a12=bmat(5,1)
      a23=bmat(6,1)
!!!! check se ci sono problemi di affinamento (in tal caso il num. di aij=0 è > num. param. non affinati) 
      nzero= 0
      do inpar=1,6
         if(abs(bmat(inpar,1)) <=  1.e-10) nzero  =  nzero+1
      enddo
      if ( (isym ==1 .and. nzero > 5).or. ! cubico   
     *     (isym ==2 .and .nzero > 4).or.  ! tetragonale
     *     (isym ==3 .and .nzero > 3).or. ! ortorombico
     *     (isym== 4. and. nzero > 4).or. ! esagonale   
     *     (isym== 5. and. nzero > 2).or. ! monoclino
     *     (isym== 6. and .nzero > 0) )   then ! triclino
      write(iout,'(a)')' **********************************************'
      write(iout,'(a)') ' '
      write(iout,'(a)') '  Refinement problems.'
      write(iout,'(a)') '  The refinement procedure has been stopped. '
      write(iout,'(a)') ' '
      write(iout,'(a)')' **********************************************'
      write(lo,'(a)')' **********************************************'
      write(lo,'(a)') ' '
      write(lo,'(a)') '  Refinement problems.'
      write(lo,'(a)') '  The refinement procedure has been stopped. '
      write(lo,'(a)') ' '
      write(lo,'(a)')' **********************************************'
             a11 = a11sav
             a22 = a22sav
             a33 = a33sav
             a12 = a12sav
             a13 = a13sav
             a23 = a23sav
      write(iout,'(6f10.6)') a11,a22,a33,a13,a12,a23
             inr = 0
             ier = -111
             return
      endif
!!!!!!!!!!!
      write(iout,'(6f10.6)') a11,a22,a33,a13,a12,a23
      go to 330
  660 continue
      if (idir .gt. 1) then
! 659 format(/,11x,40h     **** Refined cell parameters ****  ,//,1x,
  659 format(/,11x,43h  **** PIRUM: refined cell parameters **** ,//,1x,
     *        6h    A ,' =',f10.5,3x,6h    B ,' =',
     *        f10.5,3x,6h    C ,' =',f10.5,/,1x,6h Alpha,' =',
     *        f10.5,3x,6h Beta ,' =',f10.5,3x,6h Gamma,' =',f10.5,//,
     *        '  Unit cell volume =',f10.2,' A**3',/)
  658 format(//,1x,
     *        6h    A ,' =',f10.5,3x,6h    B ,' =',
     *        f10.5,3x,6h    C ,' =',f10.5,/,1x,6h Alpha,' =',
     *        f10.5,3x,6h Beta ,' =',f10.5,3x,6h Gamma,' =',f10.5,//,
     *        '  Unit cell volume =',f10.2,' A**3',/)
         write(iout,661)
     >   ' Number of single indexed lines =',NEKV,
     >   ' Total number of lines          =',N1,
     >   ' A =',A,DA,' A   Alfa  =',AL,DAL,' Degrees',
     >   ' B =',B,DB,' A   Beta  =',BE,DBE,' Degrees',
     >   ' C =',C,DC,' A   Gamma =',GA,DGA,' Degrees',
     >   ' Unit cell volume =',VOLM,' A**3'
 661     format(2(' ',A,I5,/),3(' ',A,2F10.6,A,2F10.6,A,/),A,F10.2,A)
      endif
c----
c---- noindp = num. righe non indic. calc. considerando tutti i picchi osservati
c---- noind  
  670 call pw56(t1,t2,iht,ikt,ilt,n1,n2,iout,dt,wt,ip,wave,tagut,n20,
     >         n30,nfi,sq20,sq30,sqfi,m20,noind,t11,t22,ioup,ilatt,
     >         noindold,ifind,ilast,irun,irefine,istop,noindp,usedt)
      if (iminus.ge.1.and.iescheck.eq.1) deltath = - (thstep/4.)
      if (isym.eq.6) MpM20 = M20     ! Triclinic
      if (isym.eq.5) MpM20 = M20*3   ! Monoclinic
      if (isym.eq.3) MpM20 = M20*4   ! Orthorhombic
      if (isym.eq.4) MpM20 = M20*5   ! Hexagonal
      if (isym.eq.2) MpM20 = M20*5   ! Tetragonal
      if (isym.eq.1) MpM20 = M20*6   ! Cubic
c---
      if (istop.eq.2.and.(iplus.eq.0.and.iminus.eq.0.and.iend.eq.0.and.
     *irefine.ge.1).
     *or.(iend.eq.1.and.irefine.eq.1).or.(iend.eq.1.and.irefine.eq.100))
     * then
         if (icheckref.eq.0) then
            write(lo,659)a,b,c,al,be,ga,volm
          else
            write(lo,658)a,b,c,al,be,ga,volm
         endif
!!!    write(lo,611)m20,mpm20,n1,n2,noindp,usedt
       if (n1.ge.20) then
!!!        write(lo,611)m20,mpm20,n1,n20,noindp,usedt
           write(lo,611)m20,mpm20,n1,n20,noind,usedt
          else
!!!        write(lo,611)m20,mpm20,n1,nfi,noindp,usedt
           write(lo,611)m20,mpm20,n1,nfi,noind,usedt
       endif
 611     format(1x,8h M(20) =,i7,6x,8hM'(20) =,i7,//,
     *   1x,33h Total number of observed lines =,i7,//,
     *   1x,56h Number of calculated lines (up to the 20th observed d)=,
     *   i7,//,1x,29h Number of unindexed  lines =,i7,//,
     *   41h  DT = abs[2-theta(calc)-2-theta(obs)] = ,f7.4,//)
       write(lo,612)
 612     format(2x, 'DT is the maximum difference above which a line is
     * not indexed ')
      endif
c---
c-- To store the refined cells
c---
!!!!   call FillCell(M20,MpM20,irefine,a,al,b,be,c,ga,noindp,volm,
       if (iesadd.eq.0) then
       call FillCell(M20,MpM20,irefine,a,al,b,be,c,ga,noind,volm,
     *               icheckref)
         else
       call FillCellad(M20,irefine,a,al,b,be,c,ga,noind,icheckref)
       endif
       if (irefine.eq.100) goto 681
       if (istop.eq.2.or.irefine.gt.2) then
          goto 680
       endif
       if (irefine.eq.0.and.istop.le.1) then
          if (ifind.eq.1) then
             ifind = 0
             dt = dt + 0.01
             if (dt.gt.dtmax) then
                dt = dtmax
                istop = 1
             endif
             irun = 1
             lernum = inr
             goto 130 
          endif
          if (ilast.eq.1) then
             ilast = 0
             if (irefine.eq.0) then
                if (dt.gt.0.03) then
                    dt = dt - 0.01
                  else
corig               inr = 5
                    inr = 3
!
!iprot                    if (iproteina.eq.1) inr = 0
!
                    lernum = inr
                    goto 130 
                endif
              else
                goto 680
             endif
             irun = 1
             istop = 1
          endif
       endif
       if (irun.eq.1) then
           if (istop.eq.1) then
               istop = 2
           endif
           if (irefine.eq.0) irefine = 1
corig      inr = 5
           inr = 3
!
!iprot           if (iproteina.eq.1) inr = 0
!
           lernum = inr
           goto 130 
       endif
       if (istop.eq.2.and.irefine.eq.0) then
corig     inr = 5
          inr = 3
!
!iprot          if (iproteina.eq.1) inr = 0
!
          lernum = inr
          irun = 1
          goto 130 
       endif
       if (irefine.gt.0) then
           dt = dt + 0.01
           if (dt.gt.dtmax.or.noindold.eq.0) then
              istop = 2
              goto 680
          endif
corig     inr = 5
          inr = 3
!
!iprot          if (iproteina.eq.1) inr = 0
!
          lernum = inr
          goto 130 
       endif
c---------------
c      write(iout,'(a,f10.6)')
c     >      ' Summa(Delta(2th)**2)=',v22
c----------------------------------------------------------------------+
c --- For this time end of the calculations, close output file and     !
c     set the number of entries in the sine square theta table to      !
c     the original value (It is decreased to zero by some routines).   !
c     At last return to the main many.                                 !
c----------------------------------------------------------------------+
  777 inr = lernum
c
c
c
c --- This is the very end of this program.
c
  680 continue
c---------------------
      if (imatrice.eq.1) iend=1
      if (iescheck.eq.1) iescheck=0
      if (imatrice.eq.0) then
        if (icheckref.eq.0) then
!           call SearchNode(iNok,n2)
            call SearchNode(iNok,n2,isym,npeak)
!           CellTypeb(ntrialtreor) = celltype
            call SearchParity(iPok,n2,npeak)
            if (iPok.gt.0.or.iNok.gt.0) then
            call UseParityNode(isym,a11,a22,a33,ilatt,isub,iescellaxis)
                if (iescellaxis.eq.1) then
                    iomv = 1
                    if (iesadd.eq.0) then
                        call chaaij (isym,a11,a22,a33,a13,a12,a23)
                      else
                        call chaaij_add (isym,a11,a22,a33,a13,a12,a23)
                    endif
                endif
*               write(lo,'(a)')
*               write(lo,'(a)')warnpar(1)
*               write(lo,'(a)')warnpar(2)
*               write(lo,'(a)')
*               write(lo,'(a)')infopar(1)
*               write(lo,'(a)')infopar(2)
!corr-annag                if (dt.gt.0.03)dt = dt - 0.01
corig           inr = 5
                inr = 3
!
!iprot          if (iproteina.eq.1) inr = 0
!
                lernum = inr
                irun = 1
                irefine = 1
                icheckref = 1
                iescheck=1
                goto 130 
              else
                istop = 2
                iend = 1
            endif
         else
            iend = 1
        endif
      endif
c---------------------
 1333 continue
c---------------------
 681  continue
c--
      CellAxis='   '
      CellType = 'P'
c---------------------
      call eliminacelledoppie
c---------------------
      return
      end
c-----------------------------------------------------------------------

      function torad (x)
      torad = x * 0.01745329251994
      return
      end

c-----------------------------------------------------------------------
         
      function sqr(x)
      sqr=x*x
      return
      end

c-----------------------------------------------------------------------
      SUBROUTINE P55W(T2,WG,IQE,IHT,IKT,ILT,NTOT,ISYM,A,B,IEKV)
C-----------------------------------------------------------------------
C     BILDA NORMALEKV. MATRISERNA A OCH B. ORIML.PROBLEM GER IEKV=0
C-----------------------------------------------------------------------
**    DIMENSION T2(1),WG(1),IQE(1),IHT(1),IKT(1),ILT(1),A(6,6),B(6,1)
      DIMENSION T2(NTOT),WG(NTOT),IQE(NTOT),IHT(NTOT),IKT(NTOT),
     *ILT(NTOT),A(6,6),B(6,1)
      DO 10 I=1,6
      DO 10 J=1,6
   10 A(I,J)=0.0
      DO 20 I=1,6
   20 B(I,1)=0.0
      IEKV=0
      GO TO(30,80,160,130,160,160,300),ISYM
   30 DO 50 I=1,NTOT
      IF(IQE(I))40,50,40
   40 U1=FLOAT(IHT(I)**2+IKT(I)**2+ILT(I)**2)
      A(1,1)=WG(I)*U1**2+A(1,1)
      B(1,1)=WG(I)*U1*T2(I)+B(1,1)
      IEKV=IEKV+1
   50 CONTINUE
      IF(IEKV-1)60,60,70
   60 IEKV=0
   70 RETURN
   80 DO 100 I=1,NTOT
      IF(IQE(I)) 90,100,90
   90 U1=FLOAT(IHT(I)**2+IKT(I)**2)
      A(1,1)=WG(I)*U1**2+A(1,1)
      A(1,2)=WG(I)*FLOAT(ILT(I)**2)*U1+A(1,2)
      A(2,2)=WG(I)*FLOAT(ILT(I))**4+A(2,2)
      B(1,1)=WG(I)*U1*T2(I)+B(1,1)
      B(2,1)=WG(I)*FLOAT(ILT(I)**2)*T2(I)+B(2,1)
      IEKV=IEKV+1
  100 CONTINUE
  110 IF(IEKV-2) 60,60,120
  120 A(2,1)=A(1,2)
      RETURN
  130 DO 150 I=1,NTOT
      IF(IQE(I))140,150,140
  140 U1=FLOAT(IHT(I)**2+IHT(I)*IKT(I)+IKT(I)**2)
      A(1,1)=WG(I)*U1**2+A(1,1)
      A(1,2)=WG(I)*U1*FLOAT(ILT(I)**2)+A(1,2)
      A(2,2)=WG(I)*FLOAT(ILT(I))**4+A(2,2)
      B(1,1)=WG(I)*U1*T2(I)+B(1,1)
      B(2,1)=WG(I)*FLOAT(ILT(I)**2)*T2(I)+B(2,1)
      IEKV=IEKV+1
  150 CONTINUE
      GO TO 110
  300 do 350 i=1,ntot
      if(iqe(i)) 340,350,340
  340 U1=FLOAT(IHT(I)**2+IKT(I)**2+ILT(I)**2)
      U2=FLOAT(IHT(I)*IKT(I)+IHT(I)*ILT(I)+IKT(I)*ILT(I))
      a(1,1)=wg(i)*u1**2+a(1,1)
      a(1,2)=wg(i)*u1*u2+a(1,2)
      a(2,2)=wg(i)*u2**2+a(2,2)
      b(1,1)=wg(i)*u1*t2(i)+b(1,1)
      b(2,1)=wg(i)*u2*t2(i)+b(2,1)
      iekv=iekv+1
  350 continue
      go to 110
  160 DO 200 I=1,NTOT
      IF(IQE(I)) 170,200,170
  170 U1=FLOAT(IHT(I)**2)
      U2=FLOAT(IKT(I)**2)
      U3=FLOAT(ILT(I)**2)
      A(1,1)=WG(I)*U1**2+A(1,1)
      A(1,2)=WG(I)*U1*U2+A(1,2)
      A(1,3)=WG(I)*U1*U3+A(1,3)
      B(1,1)=WG(I)*U1*T2(I)+B(1,1)
      B(2,1)=WG(I)*U2*T2(I)+B(2,1)
      B(3,1)=WG(I)*U3*T2(I)+B(3,1)
      A(2,2)=WG(I)*U2**2+A(2,2)
      A(2,3)=WG(I)*U2*U3+A(2,3)
      A(3,3)=WG(I)*U3**2+A(3,3)
      IEKV=IEKV+1
      GO TO(60,60,200,60,180,180),ISYM
  180 U4=FLOAT(IHT(I)*ILT(I))
      A(1,4)=WG(I)*U1*U4+A(1,4)
      A(2,4)=WG(I)*U2*U4+A(2,4)
      A(3,4)=WG(I)*U3*U4+A(3,4)
      A(4,4)=WG(I)*U4**2+A(4,4)
      B(4,1)=WG(I)*U4*T2(I)+B(4,1)
      GO TO(60,60,60,60,200,190),ISYM
  190 U5=FLOAT(IHT(I)*IKT(I))
      U6=FLOAT(IKT(I)*ILT(I))
      A(1,5)=WG(I)*U1*U5+A(1,5)
      A(1,6)=WG(I)*U1*U6+A(1,6)
      A(2,5)=WG(I)*U2*U5+A(2,5)
      A(2,6)=WG(I)*U2*U6+A(2,6)
      A(3,5)=WG(I)*U3*U5+A(3,5)
      A(3,6)=WG(I)*U3*U6+A(3,6)
      A(4,5)=WG(I)*U4*U5+A(4,5)
      A(4,6)=WG(I)*U4*U6+A(4,6)
      A(5,5)=WG(I)*U5**2+A(5,5)
      A(5,6)=WG(I)*U5*U6+A(5,6)
      A(6,6)=WG(I)*U6**2+A(6,6)
      B(5,1)=WG(I)*U5*T2(I)+B(5,1)
      B(6,1)=WG(I)*U6*T2(I)+B(6,1)
  200 CONTINUE
      A(2,1)=A(1,2)
      A(3,1)=A(1,3)
      A(3,2)=A(2,3)
      GO TO(60,60,210,60,220,220),ISYM
  210 IF(IEKV-3)60,60,70
  220 A(4,1)=A(1,4)
      A(4,2)=A(2,4)
      A(4,3)=A(3,4)
      GO TO(60,60,60,60,230,240),ISYM
  230 IF(IEKV-4)60,60,70
  240 A(5,1)=A(1,5)
      A(5,2)=A(2,5)
      A(5,3)=A(3,5)
      A(5,4)=A(4,5)
      A(6,1)=A(1,6)
      A(6,2)=A(2,6)
      A(6,3)=A(3,6)
      A(6,4)=A(4,6)
      A(6,5)=A(5,6)
      IF(IEKV-6)60,60,70
      END

c-----------------------------------------------------------------------
      logical function extinct (h,k,l,numext,ixs,ilatt)
c
c --- this checks if a reflexion is allowed, according to lattice
c     centering and other entered symmetry restrictions.
c
      integer h,k,l
      logical symmok,even,lattok
      integer ixs(50,5),numext
c
c --- check lattice extinctions.
c
      if (ilatt .eq. 1) then
         lattok = .true.
      elseif (ilatt.eq.2.and.even(h+k+l)) then
         lattok = .true.
      elseif (ilatt.eq.3.and.even(h+k).and.even(h+l).and.even(k+l)) then
         lattok = .true.
      elseif (ilatt.eq.4.and.even(k+l)) then
         lattok = .true.
      elseif (ilatt.eq.5.and.even(h+l)) then
         lattok = .true.
      elseif (ilatt.eq.6.and.even(h+k)) then
         lattok = .true.
      else
         lattok = .false.
      endif
      if (ilatt.eq.7) then
         if (mod((-h+k+l),3).eq.0 .or. mod((-k+h+l),3).eq.0) then
         lattok = .true.
         endif
      endif
c
c --- check other types of extinctions than lattice extintions.
c     extinction codes stored in the array ixs(i,j)
c
      symmok = .true.

      do 1 i = 1,numext
         isum = h*ixs(i,1) + k*ixs(i,2) + l*ixs(i,3)

c ------ HK0 (type = 1)
         if (l.eq.0 .and.
     >       ixs(i,5).eq.1 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

c ------ H0L (type = 2)
         elseif (k.eq.0 .and.
     >           ixs(i,5).eq.2 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

c ------ 0KL (type = 3)
         elseif (h.eq.0 .and.
     >           ixs(i,5).eq.3 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

c ------ HHL (type = 4)
         elseif (k.eq.h .and.
     >           ixs(i,5).eq.4 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

c ------ H00 (type = 5)
         elseif (k.eq.0 .and. l.eq.0 .and.
     >           ixs(i,5).eq.5 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

c ------ 0K0 (type = 6)
         elseif (h.eq.0 .and. l.eq.0 .and.
     >           ixs(i,5).eq.6 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

c ------ 00L (type = 7)
         elseif (h.eq.0 .and. k.eq.0 .and.
     >           ixs(i,5).eq.7 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

c ------ HH0 (type = 8)
         elseif (k.eq.h .and. l.eq.0 .and.
     >           ixs(i,5).eq.8 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

         endif
 1    continue
c
c --- set return value of this function.
c
 2    continue
      if (lattok .and. symmok) then
         extinct = .false.
      else
         extinct = .true.
      endif

      return
      end
    
c-----------------------------------------------------------------------
      subroutine chaaij (isym,a1,a2,a3,a4,a5,a6)
      common /treoriterato/mwoolf(22,10),
     *                     celltreor(22,6,10),cellbest(22,6),mbest(22),
     *                     ntrialtreor,nsolut(22),negtrial,shiftzero(22)
c------
c----- a     =cellbest(i,1)
c----- alpha =cellbest(i,2)
c----- b     =cellbest(i,3)
c----- beta  =cellbest(i,4)
c----- c     =cellbest(i,5)
c----- gamma =cellbest(i,6)
c------
c--------
c---CUBIC
c--------
      if (isym .eq. 1) then
         a1 = cellbest(ntrialtreor,1)
         a2 = 0.0
         a3 = 0.0
         a4 = 0.0
         a5 = 0.0
         a6 = 0.0
c--------
c---TETRAGONAL
c--------
      elseif (isym .eq. 2) then
         a1 = cellbest(ntrialtreor,1)
         a2 = cellbest(ntrialtreor,5)
         a3 = 0.0
         a4 = 0.0
         a5 = 0.0
         a6 = 0.0
c--------
c---ORTHORHOMBIC
c--------
      elseif (isym .eq. 3) then
         a1 = cellbest(ntrialtreor,1)
         a2 = cellbest(ntrialtreor,3)
         a3 = cellbest(ntrialtreor,5)
         a4 = 0.0
         a5 = 0.0
         a6 = 0.0
         call pworder(3,a1,a2,a3,a4,a5,a6)
c--------
c---HEXAGONAL
c--------
      elseif (isym .eq. 4) then
         a1 = cellbest(ntrialtreor,1)
         a2 = cellbest(ntrialtreor,5)
         a3 = 0.0
         a4 = 0.0
         a5 = 0.0
         a6 = 0.0
c--------
c---MONOCLINIC
c--------
      elseif (isym .eq. 5) then
         a1 = cellbest(ntrialtreor,1)
         a2 = cellbest(ntrialtreor,3)
         a3 = cellbest(ntrialtreor,5)
         a4 = cellbest(ntrialtreor,4)
         a5 = 0.0
         a6 = 0.0
         call pworder(5,a1,a2,a3,a4,a5,a6)
c--------
c---TRICLINIC
c--------
      elseif (isym .eq. 6) then
         a1 = cellbest(ntrialtreor,1)
         a2 = cellbest(ntrialtreor,3)
         a3 = cellbest(ntrialtreor,5)
         a4 = cellbest(ntrialtreor,2)
         a5 = cellbest(ntrialtreor,4)
         a6 = cellbest(ntrialtreor,6)
CCC  PEW 2008  OBS. ISYM=6 (NOT 5)
CC   It is possible to skip this call pworder
CC   and keep the order from REDDUCT
CC       ... a matter of taste...
         call pworder(6,a1,a2,a3,a4,a5,a6)
      endif
      return
      end

c-----------------------------------------------------------------------
      SUBROUTINE HSORT2 (V,IX,ANTAL)
C-----------------------------------------------------------------------
C This routine sorts the numbers in the array V in increasing order
C with heap sort ...
C-----------------------------------------------------------------------
      INTEGER   ANTAL
      DIMENSION V(1),IX(1)
C
C --- Create the heap.
C
      CALL CRHEP2 (V,IX,ANTAL)
C
C --- Sort the heap.
C
      DO 100 I = ANTAL,2,-1
         CALL SWAPR (V(I),V(1))
         CALL SWAPI (IX(I),IX(1))
         IROT = 1
         CALL ADHEP2 (V,IX,IROT,I-1)
  100 CONTINUE
C
      RETURN
      END


      SUBROUTINE ADHEP2 (V,IX,ROT,N)
C-----------------------------------------------------------------------
C This routine adjusts the heap ...
C-----------------------------------------------------------------------
      INTEGER   ROT,N
      DIMENSION V(1),IX(1)
C
 4711 CONTINUE
C
      J = 2*ROT
      IF (J .LE. N) THEN
         IF (J .LT. N) THEN
            IF (V(J) .LT. V(J+1)) THEN
               J = J+1
            ENDIF
         ENDIF
         IF (V(ROT) .LT. V(J)) THEN
            CALL SWAPR (V(J),V(ROT))
            CALL SWAPI (IX(J),IX(ROT))
            ROT = J
            GOTO 4711
         ENDIF
      ENDIF
C
      RETURN
      END


      SUBROUTINE CRHEP2 (V,IX,ANTAL)
C-----------------------------------------------------------------------
C This routine creates the heap ...
C-----------------------------------------------------------------------
      INTEGER   ANTAL
      DIMENSION V(1),IX(1)
C
      DO 100 I = ANTAL/2,1,-1
         IROT = I
         CALL ADHEP2 (V,IX,IROT,ANTAL)
  100 CONTINUE
C
      RETURN
      END
 

      SUBROUTINE SFIXA (X,Y,N)
C-----------------------------------------------------------------------
C This routine sort the values in the array "X" in increasing order
C with the help of heap-sort.  The values in the array "Y" corresponds
C to the same x-values as they did before, ie. they are sorted in a
C similar way as X are ...
C
C The actual dimension of X and Y in this case are 200 so the 
C index array are given this dimension.
C-----------------------------------------------------------------------
      DIMENSION X(1),Y(1)
      INTEGER   IND(200)
      REAL      YTEMP(200)
C
C --- First set the N first elements in the index vector to the
C     corresponding ordinal numbers.
C
      do 200 i = 1,n
         ind(i) = i
  200 continue
c
c --- Sort the X-array.
c
      call hsort2 (x,ind,n)
c
c --- Rearrange the values in the Y-array according to the IND-array.
c
      do 100 i = 1,n
         ytemp(i) = y(ind(i))
  100 continue
      do 101 i = 1,n
         y(i) = ytemp(i)
  101 continue
c
      return
      end

c-----------------------------------------------------------------------

      subroutine swapi (ia,ib)
c
c --- This routine swaps the two integer variables IA and IB.
c
      itemp = ia
      ia    = ib
      ib    = itemp

      return
      end

c-----------------------------------------------------------------------

      subroutine swapr (ra,rb)
c
c --- This routine swaps the two real variables RA and RB.
c
      temp = ra
      ra   = rb
      rb   = temp

      return
      end

c-----------------------------------------------------------------------

      subroutine blkstr (c80,ifrom,ito)
c
c --- This routine sets the positions beginning at IFROM up to and 
c     including ITO in the character-string C80 to blanks.
c
      character c80*80

      i = ifrom
  100 continue
      if (i .le. ito) then
         c80(i:i) = ' '
         i = i+1
         goto 100
      endif

      return
      end

c-----------------------------------------------------------------------
      integer function last (cline)
      character cline*80
      do 10 i = 80,1,-1
         if (cline(i:i) .ne. ' ') then
            last = i
            return
         endif
   10 continue
      last = 0
      return
      end

c-----------------------------------------------------------------------
      SUBROUTINE PWW50(A11,A22,A33,A13,A12,A23,ISYM,IHM,IKM,ILM,
     >                IH,IK,IL,SSQT,IA,jo)
C-----------------------------------------------------------------------
C     SE P-E 850
C     Updated August 2008
C-----------------------------------------------------------------------
      IA=1
      GO TO(10,90,160,230,300,390,521),ISYM
   10 IF(IH-IHM) 20,30,30
   20 IH=IH+1
      GO TO 70
   30 IF(IK-IKM) 40,50,50
   40 IK=IK+1
      IH=IK
      GO TO 70
   50 IF(IL-ILM) 60,80,80
   60 IL=IL+1
      IK=IL
      IH=IK
   70 SSQT=FLOAT(IH*IH+IK*IK+IL*IL)*A11
      RETURN
   80 IA=0
      RETURN
   90 IF(IH-IHM) 100,110,110
  100 IH=IH+1
      GO TO 150
  110 IF(IK-IKM) 120,130,130
  120 IK=IK+1
      IH=IK
      GO TO 150
  130 IF(IL-ILM) 140,80,80
  140 IL=IL+1
      IH=0
      IK=0
  150 SSQT=FLOAT(IH*IH+IK*IK)*A11+FLOAT(IL*IL)*A22
      RETURN
  160 IF(IH-IHM) 170,180,180
  170 IH=IH+1
      GO TO 220
  180 IH=0
      IF(IK-IKM) 190,200,200
  190 IK=IK+1
      GO TO 220
  200 IF(IL-ILM) 210,80,80
  210 IK=0
      IL=IL+1
  220 SSQT=FLOAT(IH*IH)*A11+FLOAT(IK*IK)*A22+FLOAT(IL*IL)*A33
      RETURN
C
C     CHANGE S50/46,47,48,49,50,54
C
  230 IF(IH-IHM) 240,250,250
  240 IH=IH+1
      GO TO 290
  250 IF(IK-IKM) 260,270,270
  260 IK=IK+1
      IH=IK
      GO TO 290
  270 IF(IL-ILM) 280,80,80
  280 IH=0
      IK=0
      IL=IL+1
  290 SSQT=FLOAT(IH*IH+IH*IK+IK*IK)*A11+FLOAT(IL*IL)*A22
      RETURN
  300 IF(IH*IL)320,320,310
  310 IH=-IH
      GO TO 380
CC PEW 2008
  320 IF(IABS(IH)-IHM) 330,340,340
!!!  320 IF(IABS(IH)-IHM-1) 330,340,340
  330 IH=IABS(IH)+1
      GO TO 380
  340 IH=0
CC PEW 2008
      IF(IK-IKM)350,360,360
! io  IF(IK-IKM-1)350,360,360
  350 IK=IK+1
      GO TO 380
  360 IF(IL-ILM) 370,80,80
! io  360 IF(IL-ILM-1) 370,80,80
  370 IK=0
      IL=IL+1
  380 SSQT=FLOAT(IH*IH)*A11+FLOAT(IK*IK)*A22+FLOAT(IL*IL)*A33+
     >     FLOAT(IH*IL)*A13
      RETURN
  390 IF(IABS(IK)+IL) 400,420,400
  400 IF(IH) 420,420,410
  410 IH=-IH
      GO TO 510
CC PEW  2008
  420 IF(IABS(IH)-IHM) 430,440,440
!io  420 IF(IABS(IH)-IHM-1) 430,440,440
  430 IH=IABS(IH)+1
      GO TO 510
  440 IH=0
      IF(IL) 470,470,450
  450 IF(IK) 470,470,460
  460 IK=-IK
      GO TO 510
CC PEW 2008
  470 IF(IABS(IK)-IKM) 480,490,490
! io  470 IF(IABS(IK)-IKM-1) 480,490,490
  480 IK=IABS(IK)+1
      GO TO 510
  490 IK=0
CC PEW 2008
      IF(IL-ILM) 500,80,80
! io      IF(IL-ILM-1) 500,80,80
  500 IL=IL+1
  510 SSQT=FLOAT(IH*IH)*A11+FLOAT(IK*IK)*A22+FLOAT(IL*IL)*A33+
     >     FLOAT(IH*IL)*A13+FLOAT(IH*IK)*A12+FLOAT(IK*IL)*A23
      RETURN
  521 jo=jo+1
      goto(531,532,533,534),jo
  531 ih=iabs(ih)
      ik=iabs(ik)
      il=iabs(il)
      go to 540
  532 ih=-ih
      if(ih.eq.0) then
      goto 521
      else
      goto 527
      endif
  533 ih=iabs(ih)
      ik=-ik
      if(ik.gt.0) then
      jo=0
      goto 540
      else
      goto 527
      endif
  534 ik=iabs(ik)
      il=-il
      jo=0
      if(il.gt.0) then
      goto 521
      else
      goto 527
      endif
  540 IF(IH-IHM) 522,523,523
  522 IH=IH+1
      GO TO 527
  523 IF(IK-IKM) 524,525,525
  524 IK=IK+1
      IH=IK
      GO TO 527
  525 IF(IL-ILM) 526,80,80
  526 IL=IL+1
      IK=IL
      IH=IK
  527 SSQT=FLOAT(IH*IH+IK*IK+IL*IL)*A11+FLOAT(IH*IK+IH*IL+IK*IL)*A22
      RETURN
      END


      SUBROUTINE PWW51(SSQT,T1,N,T11,DT,SSQTO,IA)
C-----------------------------------------------------------------------
C     SE P-E 852
C-----------------------------------------------------------------------
**    DIMENSION T1(1),T11(1)
      DIMENSION T1(200),T11(200)
      HA=SQRT(SSQT)
      HB=SQRT(1.-SSQT)
      TTH=114.59156*ATAN(HA/HB)
      DO 40 I=1,N
      IF(ABS(TTH-T11(I))-DT) 50,50,40
C      IF(SSQT-SSQTL) 10,20,20
C   10 D=DT
C      GO TO 30
C   20 D=D2
C   30 DO 40 I=1,N
C      IF(ABS(SSQT-T1(I))-D) 50,50,40
   40 CONTINUE
      IA=0
      RETURN
   50 IA=1
      SSQTO=T1(I)
      RETURN
      END


      SUBROUTINE PWW52(N,NEKV,ISYM,V2,WAVE,AMAT,S,A,B,C,AL,BE,GA,
     >                DA,DB,DC,DAL,DBE,DGA,VOLM)
C-----------------------------------------------------------------------
      common/stand_dev_cell_vol/ stand_cell(6),stand_vol
      DIMENSION AMAT(6,6),S(6,6),F(6)
      SIG=SQRT(V2/(FLOAT(NEKV-N)))
      SIG=(4.0*SIG)/(WAVE**2)
!     nuit = 9
      DO 20 L=1,N
      F(L)=0.0
      DO 10 J=1,N
      DO 10 K=1,N
   10 F(L)=F(L)+AMAT(J,K)*S(L,J)*S(L,K)
   20 F(L)=SIG*SQRT(F(L))
      DB=0.0
      DC=0.0
      DAL=0.0
      DBE=0.0
      DGA=0.0
      stand_cell(1:6) = 0.0
      stand_vol = 0.0
      RAD=0.0174533
      GRAD=57.2958
      DA=F(1)
      stand_cell(1) = DA 
      GO TO(30,40,50,60,70,80,201),ISYM
   30 B=A
      C=A
      DB=DA
      DC=DA
      stand_cell(3) = DA 
      stand_cell(5) = DA 
      GO TO 150
   40 B=A
      DB=DA
      DC=F(2)
      stand_cell(3) = DA 
      stand_cell(5) = DC 
      GO TO 150
   50 DB=F(2)
      DC=F(3)
      stand_cell(3) = DB 
      stand_cell(5) = DC 
      GO TO 150
   60 DB=DA
      B=A
      DC=F(2)
      GA=120.0
      stand_cell(3) = DA 
      stand_cell(5) = DC 
      GO TO 180
   70 DB=F(2)
      DC=F(3)
      BE=GRAD*BE
      DBE=GRAD*F(4)
      stand_cell(3) = DB 
      stand_cell(5) = DC 
      stand_cell(4) = DBE 
      GO TO 190
   80 DB=F(2)
      DC=F(3)
      stand_cell(3) = DB 
      stand_cell(5) = DC 
      AL=GRAD*AL
      IF(AL) 90,100,100
   90 AL=AL+180.0
  100 DAL=GRAD*F(4)
      stand_cell(2) = DAL 
      BE=GRAD*BE
      IF(BE) 110,120,120
  110 BE=BE+180.0
  120 DBE=GRAD*F(5)
      stand_cell(4) = DBE 
      GA=GRAD*GA
      IF(GA) 130,140,140
  130 GA=GA+180.0
  140 DGA=GRAD*F(6)
      stand_cell(6) = DGA 
      GO TO 200
  150 VOLM=A*B*C
!-standard deviation volm
      siga2 = (da/a)**2.
      sigb2 = (db/b)**2.
      sigc2 = (dc/c)**2.
      stand_vol = volm * sqrt(siga2 + sigb2 + sigc2)
!     write(nuit,*) ' a=',a,' stand_a=',stand_cell(1)
!     write(nuit,*) ' b=',b,' stand_b=',stand_cell(3)
!     write(nuit,*) ' c=',c,' stand_c=',stand_cell(5)
!     write(nuit,*) ' volm=',volm,' stand_vol=',stand_vol
!-standard deviation volm
      BE=90.0
  160 GA=90.0
  170 AL=90.0
      GO TO 210
  180 VOLM=0.8660254*A*B*C
!-standard deviation volm
      siga2 = (da/a)**2.
      sigb2 = (db/b)**2.
      sigc2 = (dc/c)**2.
      stand_vol = volm * sqrt(siga2 + sigb2 + sigc2)
!     write(nuit,*) ' volm=',volm,' stand_vol=',stand_vol
!     write(nuit,*) ' a=',a,' stand_a=',stand_cell(1)
!     write(nuit,*) ' b=',b,' stand_b=',stand_cell(3)
!     write(nuit,*) ' c=',c,' stand_c=',stand_cell(5)
!-standard deviation volm
      BE=90.0
      GO TO 170
  190 VOLM=A*B*C*SIN(RAD*BE)
!-standard deviation volm
      siga2 = (da/a)**2.
      sigb2 = (db/b)**2.
      sigc2 = (dc/c)**2.
      sigang1 = ((a*b*c)**4.)/(volm**2.)
      sigang2 = ( sin(rad*be)*cos(rad*be)*(dbe*rad) )**2.
      stand_vol = (volm**2.)*(siga2+sigb2+sigc2)+sigang1*sigang2 
      stand_vol = sqrt( stand_vol ) 
!     write(nuit,*) ' volm=',volm,' stand_vol=',stand_vol
!     write(nuit,*) ' a=',a,' stand_a=',stand_cell(1)
!     write(nuit,*) ' b=',b,' stand_b=',stand_cell(3)
!     write(nuit,*) ' c=',c,' stand_c=',stand_cell(5)
!     write(nuit,*) ' beta=',be,' stand_beta=',stand_cell(4)
!-standard deviation volm
      GO TO 160
  201 b=a
      c=a
      db=da
      dc=da
      al=grad*al
      be=al
      ga=al
      dal=grad*f(2)
      dbe=dal
      dga=dal
  200 CA=COS(RAD*AL)
      CB=COS(RAD*BE)
      CC=COS(RAD*GA)
      VOLM=A*B*C*SQRT(1.0-CA**2-CB**2-CC**2+2.0*CA*CB*CC)
!-standard deviation volm
      siga2 = (da/a)**2.
      sigb2 = (db/b)**2.
      sigc2 = (dc/c)**2.
      sigang1 = ((a*b*c)**4.)/(volm**2.)
      sigb11 = ( sin(rad*al)*( cos(rad*al) - cos(rad*be)*cos(rad*ga) )
     * *(dal*rad) )**2.
      sigb12 = ( sin(rad*be)*( cos(rad*be) - cos(rad*al)*cos(rad*ga) )
     * *(dbe*rad) )**2.
      sigb13 = ( sin(rad*ga)*( cos(rad*ga) - cos(rad*al)*cos(rad*be) )
     * *(dga*rad) )**2.
      sigang2 = sigb11 + sigb12 + sigb13
      stand_vol = (volm**2.)*(siga2+sigb2+sigc2)+sigang1*sigang2
      stand_vol = sqrt( stand_vol ) 
!     write(nuit,*) ' volm=',volm,' stand_vol=',stand_vol
!     write(nuit,*) ' a=',a,' stand_a=',stand_cell(1)
!     write(nuit,*) ' b=',b,' stand_b=',stand_cell(3)
!     write(nuit,*) ' c=',c,' stand_c=',stand_cell(5)
!     write(nuit,*) ' alfa=',al,' stand_alfa=',stand_cell(2)
!     write(nuit,*) ' beta=',be,' stand_beta=',stand_cell(4)
!     write(nuit,*) ' gamma=',ga,' stand_gamma=',stand_cell(6)
!-standard deviation volm
  210 RETURN
      END


      SUBROUTINE PWW53(SSQTO,T2,T22,WG,N2,IHT,IKT,ILT,IQE,IH,IK,IL,ISYM,
     >                WT)
C-----------------------------------------------------------------------
C     SE P-E 853 OBS. N2 START=0
C-----------------------------------------------------------------------
**    DIMENSION T2(1),IHT(1),IKT(1),ILT(1),IQE(1),T22(1),WG(1)
      common /ureq1/ ncycl,nroset,nundset,nsec,jback,uvwt(8),
     *               itry,pfwhm,kequirip
      DIMENSION T2(400),IHT(400),IKT(400),ILT(400),IQE(400),T22(400),
     *WG(400)
      I=1
   10 IF(I-N2) 30,30,20
   20 T2(I)=SSQTO
      HA=SQRT(SSQTO)
      HB=SQRT(1.-SSQTO)
      T22(I)=114.59156*ATAN(HA/HB)
       IF(WT .EQ. 1.) THEN
         WG(I)=1.
         ELSE 
       WG(I)=1./((2.*HA*HB)**2)
!        ELSEif (WT.eq.0.0) then 
!      WG(I)=1./((2.*HA*HB)**2)
!        ELSEif (WT.eq.2.0) then 
!      WG(I)=alambda/(2.*HA)
       ENDIF
      IHT(I)=IH
      IKT(I)=IK
      ILT(I)=IL
      IQE(I)=1
      GO TO 160
   30 IF(SSQTO-T2(I)) 40,50,40
   40 I=I+1
      GO TO 10
   50 GO TO (60,80,100,120,130,140,180),ISYM
   60 IF((IH**2+IK**2+IL**2)-(IHT(I)**2+IKT(I)**2+ILT(I)**2))70,170,70
   70 IQE(I)=0
      RETURN
   80 IF((IH**2+IK**2)-(IHT(I)**2+IKT(I)**2)) 70,90,70
   90 IF(IL**2-ILT(I)**2) 70,170,70
  100 IF(IH**2-IHT(I)**2) 70,110,70
  110 IF(IK**2-IKT(I)**2) 70,90,70
  120 IF((IH**2+IH*IK+IK**2)-(IHT(I)**2+IHT(I)*IKT(I)+IKT(I)**2))
     >  70,90,70
  130 IF(IH*IL-IHT(I)*ILT(I)) 70,100,70
  140 IF(IH*IK-IHT(I)*IKT(I)) 70,150,70
  150 IF(IK*IL-IKT(I)*ILT(I)) 70,130,70
  180 if((ih*ik+ih*il+ik*il)-(iht(i)*ikt(i)+iht(i)*ilt(i)+
     > ikt(i)*ilt(i))) 70,60,70
  160 SSQTO=-SSQTO
  170 RETURN
      END


      SUBROUTINE PW54(A,N,B)
C-----------------------------------------------------------------------
C     M=0 GER AINV,M=1 DESSUTOM X I B-VEKTORN
C-----------------------------------------------------------------------
      DIMENSION IPIV(6),A(6,6),B(6,1),INDEX(6,2),PIVOT(6)
      EQUIVALENCE (IROW,JROW),(ICOL,JCOL),(AMAX,T,SWAP)
      M=1
      DET=1.0
      DO 10 J=1,N
   10 IPIV(J)=0
      DO 200 I=1,N
      AMAX=0.0
      DO 60 J=1,N
      IF(IPIV(J)-1) 20,60,20
   20 DO 50 K=1,N
      IF(IPIV(K)-1) 30,50,240
   30 IF(ABS(AMAX)-ABS(A(J,K))) 40,50,50
   40 IROW=J
      ICOL=K
      AMAX=A(J,K)
   50 CONTINUE
   60 CONTINUE
      IPIV(ICOL)=IPIV(ICOL)+1
      IF(IROW-ICOL) 70,110,70
   70 DET=-DET
      DO 80 L=1,N
      SWAP=A(IROW,L)
      A(IROW,L)=A(ICOL,L)
   80 A(ICOL,L)=SWAP
      IF(M)110,110,90
   90 DO 100 L=1,M
      SWAP=B(IROW,L)
      B(IROW,L)=B(ICOL,L)
  100 B(ICOL,L)=SWAP
  110 INDEX(I,1)=IROW
      INDEX(I,2)=ICOL
      PIVOT(I)=A(ICOL,ICOL)
      DET=DET*PIVOT(I)
      A(ICOL,ICOL)=1.0
      DO 120 L=1,N
  120 A(ICOL,L)=A(ICOL,L)/PIVOT(I)
      IF(M) 150,150,130
  130 DO 140 L=1,M
  140 B(ICOL,L)=B(ICOL,L)/PIVOT(I)
  150 DO 200 L1=1,N
      IF(L1-ICOL) 160,200,160
  160 T=A(L1,ICOL)
      A(L1,ICOL)=0.0
      DO 170 L=1,N
  170 A(L1,L)=A(L1,L)-A(ICOL,L)*T
      IF(M) 200,200,180
  180 DO 190 L=1,M
  190 B(L1,L)=B(L1,L)-B(ICOL,L)*T
  200 CONTINUE
      DO 230 I=1,N
      L=N+1-I
      IF(INDEX(L,1)-INDEX(L,2)) 210,230,210
  210 JROW=INDEX(L,1)
      JCOL=INDEX(L,2)
      DO 220 K=1,N
      SWAP=A(K,JROW)
      A(K,JROW)=A(K,JCOL)
      A(K,JCOL)=SWAP
  220 CONTINUE
  230 CONTINUE
  240 RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE PW56(T1,T2,IHT,IKT,ILT,N1,N2,IOUT,DT,WT,IP,WAVE,
     >           TAGUT,N20,N30,NFI,SQ20,SQ30,SQFI,M20,NOIND,
     >           T11,T22,IOUP,ilatt,noindold,ifind,ilast,irun,irefine,
     >           istop,noindp,usedt)
      USE General
      USE peak_mod, only: pkind
!corr      common/genob/irho,thetamin
      common/genob2/thmin,thmax,thstep
!corr      common/corr2th/thstepcor,fintepeak(200) 
      common /bestone/icheckbest,shiftzerobest,IHKLbest(200,4),nrefbest
     *               ,m20best,mnew20
      common /treoriterato/mwoolf(22,10),
     *                     celltreor(22,6,10),cellbest(22,6),mbest(22),
     *                     ntrialtreor,nsolut(22),negtrial,shiftzero(22)
      common /fast/deltath_old,mnew20_old,iend,iminus,iplus,imatrice
      common/CellPirum/mpirbest(3),cellpirbest(3,6),zeroshiftpir(3),
     *                 iespir
      DIMENSION T1(200),T2(400),IHT(400),IKT(400),ILT(400),TAGUT(200),
     * T11(200),T22(400)
      real :: ymaxint
      isense=0
       if (iout .lt. 0) then
       isense=1
       iout=-iout
       write(iout,500) DT
  500   format(' ',' All calculated lines where...',/,
     >  '  ABS(2theta-obs  minus 2theta-calc) < ',F6.3,' deg.',/,
     >  '  are listed below. ')
       endif
      NOIND=0
      NWIND=0
      M20=0
      IF(SQ20.EQ.0.) GO TO 10
      SAK=SQ20
      GO TO 20
   10 SAK=SQFI
   20 CONTINUE
      do 401 i=1,n2
      if(ilatt .eq. 7) then
         if(mod((-iht(i)+ikt(i)+ilt(i)),3).eq. 0) then
         goto 401
         elseif(mod((-ikt(i)+iht(i)+ilt(i)),3).eq. 0) then
         call swapi(iht(i),ikt(i))
        endif
      endif
  401 continue
      DO 50 J1=1,N2
      ISH=IHT(J1)
      ISK=IKT(J1)
      ISL=ILT(J1)
      TS=T2(J1)
      TSS=T22(J1)
      IMH=ISH
      IMK=ISK
      IML=ISL
      TM=TS
      TMM=TSS
      DO 40 J2=J1,N2
      IF(TM-T2(J2)) 40,30,30
   30 TM=T2(J2)
      TMM=T22(J2)
      IMH=IHT(J2)
      IMK=IKT(J2)
      IML=ILT(J2)
      J3=J2
   40 CONTINUE
      IHT(J1)=IMH
      IKT(J1)=IMK
      ILT(J1)=IML
      T2(J1)=TM
      T22(J1)=TMM
      IHT(J3)=ISH
      IKT(J3)=ISK
      ILT(J3)=ISL
      T22(J3)=TSS
   50 T2(J3)=TS
      DSUM=0.0
      DSUMA=0.0
      DSUMB=0.0
      DSUMC=0.0
      TSUMA=0.0
      TSUMB=0.0
      TSUMC=0.0
      NTALA=0
      NTALB=0
      NTALC=0
      GRAD=114.59156
!corr
      ymaxint = maxval(pkind%yint)
!corr
      WRITE(IOUT,60)
   60 FORMAT(' ','   H   K   L SST-OBS  SST-CALC   DELTA   2TH-OBS',
**** >           ' 2TH-CALC D-OBS   FREE PARAM.')
     >           ' 2TH-CALC D-OBS   100*I/Imax ')
      J=1
      DO 280 I=1,N2
   70 IF(J-N1) 80,80,260
   80 IF(I-N2) 90,100,100
   90 IF(ABS(T11(J)-T22(I))-ABS(T11(J)-T22(I+1))) 100,100,260  
  100 IF(ABS(T11(J)-T22(I))-DT) 180,180,130  
  130 IF(T11(J)-T22(I)) 140,140,260
  140 HA=SQRT(T1(J))
      DOBS=WAVE/(2.*HA)
      HD=SQRT(1.-T1(J))
      THO=GRAD*ATAN(HA/HD)
      NOIND=NOIND+1
***** WRITE(IOUT,150) T1(J),THO,DOBS,TAGUT(J)
      WRITE(IOUT,'(1h ,12X,F9.6,19X,F8.3,8X,F8.4)') T1(J),THO,DOBS
  150 FORMAT(' ',12X,F9.6,19X,F8.3,8X,F8.4,2X,F10.2)
      IF(T1(J)-SAK) 160,160,170
  160 NWIND=NWIND+1
  170 J=J+1
      GO TO 70
  180 HA=SQRT(T1(J))
      HB=SQRT(T2(I))
      DOBS=WAVE/(2.*HA)
      DCALC=WAVE/(2.*HB)
      DELTA=T1(J)-T2(I)
      HD=SQRT(1.-T2(I))
      HE=SQRT(1.-T1(J))
      THO=ATAN(HA/HE)*GRAD
      THC=ATAN(HB/HD)*GRAD
      DELTB=THO-THC
C      WRITE(IOUT,190) IHT(I),IKT(I),ILT(I),T1(J),T2(I),DELTA,THO,THC,
      WRITE(IOUT,190) IHT(I),IKT(I),ILT(I),T1(J),T2(I),DELTB,THO,THC,
***  >                DOBS,TAGUT(J)
!corr     >                DOBS,fintepeak(J)
     >                DOBS,100*pkind(j)%yint/ymaxint
c-----
      if (J.le.200) then
      IHKLbest(j,1) = iht(i)
      IHKLbest(j,2) = ikt(i)
      IHKLbest(j,3) = ilt(i)
      IHKLbest(j,4) = 1
      endif
c-----
  190 FORMAT(' ',3I4,2F9.6,F10.3,2F8.3,F8.4,2X,F10.2)
      IF(IP .EQ. 1) THEN
      WRITE(IOUP,201) IHT(I),IKT(I),ILT(I),T1(J),TAGUT(J)
  201 FORMAT(3I5,F10.6,F10.2)
      ENDIF
      IF(SQ20-T1(J)) 210,200,200
  200 DSUMA=DSUMA+ABS(DELTA)
      TSUMA=TSUMA+ABS(DELTB)
      NTALA=NTALA+1
  210 IF(SQ30-T1(J)) 230,220,220
  220 DSUMB=DSUMB+ABS(DELTA)
      TSUMB=TSUMB+ABS(DELTB)
      NTALB=NTALB+1
  230 IF(SQFI-T1(J)) 250,240,240
  240 DSUMC=DSUMC+ABS(DELTA)
      TSUMC=TSUMC+ABS(DELTB)
      NTALC=NTALC+1
  250 J=J+1
      GO TO 280
  260 HC=SQRT(T2(I))
      HD=SQRT(1.-T2(I))
      THC=GRAD*ATAN(HC/HD)
      WRITE(IOUT,270) IHT(I),IKT(I),ILT(I),T2(I),THC
  270 FORMAT(' ',3I4,9X,F9.6,18X,F8.3)
  280 CONTINUE
  290 IF(J-N1) 300,300,330
  300 HA=SQRT(T1(J))
      HD=SQRT(1.-T1(J))
      DOBS=WAVE/(2.*HA)
      NOIND=NOIND+1
      THO=GRAD*ATAN(HA/HD)
****  WRITE(IOUT,150) T1(J),THO,DOBS,TAGUT(J)
      WRITE(IOUT,'(1h ,12X,F9.6,19X,F8.3,8X,F8.4)') T1(J),THO,DOBS
C     omkastat tidigare....
      IF(T1(J)-SAK) 310,310,320
  310 NWIND=NWIND+1
  320 J=J+1
      GO TO 290
  330 CONTINUE
      WRITE(IOUT,331)
  331 FORMAT(' ')
      WRITE(IOUT,340) N1
  340 FORMAT(' Number of obs. lines  =',I5)
      WRITE(IOUT,350) N2
  350 FORMAT(' Number of calc. lines =',I5)
      IF(NTALA) 380,380,360
  360 TMEDA=TSUMA/NTALA
      DMEDA=DSUMA/NTALA
      MER=SQ20/(2.0*DMEDA*N20)+0.5
      M20=MER
      F20=20.0/(TMEDA*N20)+0.5
      NDEF=20
      if(isense .eq. 0) then
      WRITE(IOUT,331)
      WRITE(IOUT,370) NDEF,MER,DMEDA,NDEF,F20,TMEDA,N20
  370 FORMAT(' M(',I3,') =',I4,'  AV.EPS.=',F10.7,/,
     >       ' F(',I3,') =',F5.0,'(',F9.6,',',I5,')')
      endif
  380 IF(NTALB) 400,400,390
  390 TMEDB=TSUMB/NTALB
      DMEDB=DSUMB/NTALB
      MER=SQ30/(2.0*DMEDB*N30)+0.5
      F30=30.0/(TMEDB*N30)+0.5
      NDEF=30
      if(isense .eq. 0) then
      WRITE(IOUT,331)
      WRITE(IOUT,370) NDEF,MER,DMEDB,NDEF,F30,TMEDB,N30
      endif
  400 IF(NTALC) 420,420,410
  410 DMEDC=DSUMC/NTALC
      TMEDC=TSUMC/NTALC
      FN1=N1/(TMEDC*NFI)+0.5
      MER=SQFI/(2.0*DMEDC*NFI)+0.5
      IF(M20.EQ.0) M20=MER
      if(isense .eq. 0) then
      WRITE(IOUT,331)
      WRITE(IOUT,370) N1,MER,DMEDC,N1,FN1,TMEDC,NFI
      endif
  420 CONTINUE
      WRITE(IOUT,331)
C
C --- Write references for the M- and F-indexes.
C
      if(isense .eq. 0) then
       if(wt .eq. 1.) then
         write(iout,'(a)') ' UNIT weights used '
         else
         write(iout,'(a)') ' HESS weights used '
!        elseif (wt.eq.0.0) then
!        write(iout,'(a)') ' HESS weights used '
!        elseif (wt.eq.2.0) then
!        write(iout,'(a)') ' (1/dobs) weights used '
       endif
      WRITE(IOUT,331)
      write(iout,430)
  430 format(' M  cf. J.Appl.Cryst.  1 (1968) 108 ',/,
     >       ' F  cf. J.Appl.Cryst. 12 (1979)  60 ')
      endif
C
C --- Write the number of indexed lines (in a nice format) ...
C
      write(iout,331)
      if (noind .lt. 10) then
         write(iout,441) noind
      elseif (noind .lt. 100) then
         write(iout,442) noind
      elseif (noind .lt. 1000) then
         write(iout,443) noind
      elseif (noind .lt. 10000) then
         write(iout,444) noind
      else
         write(iout,445) noind
      endif
  441 format(' ',I1,' lines are unindexed. ')
  442 format(' ',I2,' lines are unindexed. ')
  443 format(' ',I3,' lines are unindexed. ')
  444 format(' ',I4,' lines are unindexed. ')
  445 format(' ',I5,' lines are unindexed. ')
C
       noindp=noind
c----- Calcolo di M'20 = M20 * [10 * (1.-DT) * (3 - NOIND) ]
c-----
c-----
      if (irefine.ge.1) then
          usedt =dt
      endif
      if (iend.eq.1) then
         if (iplus.eq.0)  iplus=1
         idelt = int ( 10.*(1.-10*DT) )
         MNEW20 = M20 * (idelt * (3 - noind) ) * 0.1
         thsog = thstep/2.
         if (iplus.ne.2.or.iminus.eq.1) then
             if (MNEW20.gt.mnew20_old) then
                   if (iplus.eq.1) then
                       iplus=2
                       if (iminus.eq.0) then
                            iminus=1
                            irefine = 1
                       endif
                       return
                   endif
                   if (iminus.eq.1) then
                       iminus=2
                       irefine = 100
                       istop = 2
                       return
                   endif
                mnew20_old = mnew20 
                NOINDOLD= NOIND
                irefine = 1
                irun = 1
                return
              else
                if (noindold.gt.0) then
                   if (dt.lt.0.07) dt = dt + 0.01
                   irefine = irefine + 1
                   irun = 1
                   if (iplus.eq.1)  then
                       iplus=2
                       if (iminus.eq.0) then
                            iminus=1
                            dt = dt - 0.01
                            irefine = 1
                       endif
                       return
                   endif
                   if (iminus.eq.1)  then
                       iminus=2
                       irefine = 100
                       istop = 2
                       return
                   endif
                   return
                endif  
                if (noindold.eq.0.or.noind.eq.0) then
                   if (iplus.le.1)  then
                       iplus=2
                       if (iminus.eq.0) then
                            iminus=1
                            irefine = 1
                       endif
                       return
                   endif
                   if (iminus.eq.1)  then
                       iminus=2
                       irefine = 100
                       istop = 2
                       return
                   endif
                   return
                endif  
             endif  
         endif
       else
         idelt = int ( 10.*(1.-10.*DT) )
         mnew20_old = M20 * (idelt * (3 - noind) ) * 0.1
      endif
      if (irefine.eq.2) istop = 2
      write(iout,*)' '
      if (noind.eq.0) then
          if (irefine.eq.0) then
              irun = 1
              return
            else
              istop = 2
              return
          endif
      endif
      if (noind.gt.0) then
         if (noind.lt.noindold) then
            if (noind.gt.2) then
               ifind = 1
              else
               irun = 1
               if (irefine.ge.1) then
                  if (dt.lt.0.06) then
                     dt = dt + 0.01
                   else
                     istop = 2
                  endif
               endif
            endif
           else
            if (irefine.eq.0) ilast = 1
            if (irefine.ge.1) then
                irun = 1
                if (dt.lt.0.06) then
                   dt = dt + 0.01
                 else
                   istop = 2
                endif
c----
            endif
            if (irefine.gt.1) istop= 2
         endif
         if (irefine.gt.0.and.noind.ge.noindold) irefine = irefine + 1
      endif
c-----
      NOINDOLD= NOIND
      NOIND=NWIND
      RETURN
      END
      SUBROUTINE PW57(T1,T2,IHT,IKT,ILT,N1,N2,IOUT,DT,WT,WAVE,
     >           TAGUT,N20,N30,NFI,SQ20,SQ30,SQFI,M20,NOIND,T11,T22,
     >           IHI,IKI,ILI,A11,A22,A33,A13,A12,A23,isym,ilatt)
C-----------------------------------------------------------------------
**    DIMENSION T1(1),T2(1),IHT(1),IKT(1),ILT(1),TAGUT(1),T11(1),T22(1),
**   >          IHI(1),IKI(1),ILI(1)
      DIMENSION T1(200),T2(400),IHT(400),IKT(400),ILT(400),TAGUT(200),
     > T11(200),T22(400),IHI(400),IKI(400),ILI(400)
      DIMENSION ITAG(200)
      do 237 i=1,n1
      itag(i)=tagut(i)
      if(ilatt .eq. 7) then
        if(mod((-ihi(i)+iki(i)+ili(i)),3).eq. 0) then
        goto 237
        elseif(mod((-iki(i)+ihi(i)+ili(i)),3).eq. 0) then
        call swapi(ihi(i),iki(i))
        endif
      endif
  237 continue
      do 401 i=1,n2
      if(ilatt .eq. 7) then
         if(mod((-iht(i)+ikt(i)+ilt(i)),3).eq. 0) then
         goto 401
        elseif(mod((-ikt(i)+iht(i)+ilt(i)),3).eq. 0) then
         call swapi(iht(i),ikt(i))
        endif
      endif
  401 continue
      WRITE(IOUT,331)
      write(iout,195)
  195 format(' No cycles as (hkl)s are fixed ',/,
     >' Any input of cell parameters irrelevant ',/,
     >' Extinction conditions will affect figures of merits ')
      write(iout,196) dt
  196 format(' The diff. parameter=',F6.3,
     >       ' has no influence on cell parameters ')
      WRITE(IOUT,331)
      NOIND=0
      NWIND=0
      NR=1
      M20=0
      IF(SQ20.EQ.0.) GO TO 10
      SAK=SQ20
      GO TO 20
   10 SAK=SQFI
   20 CONTINUE
      DO 50 J1=1,N2
      ISH=IHT(J1)
      ISK=IKT(J1)
      ISL=ILT(J1)
      TS=T2(J1)
      TSS=T22(J1)
      IMH=ISH
      IMK=ISK
      IML=ISL
      TM=TS
      TMM=TSS
      DO 40 J2=J1,N2
      IF(TM-T2(J2)) 40,30,30
   30 TM=T2(J2)
      TMM=T22(J2)
      IMH=IHT(J2)
      IMK=IKT(J2)
      IML=ILT(J2)
      J3=J2
   40 CONTINUE
      IHT(J1)=IMH
      IKT(J1)=IMK
      ILT(J1)=IML
      T2(J1)=TM
      T22(J1)=TMM
      IHT(J3)=ISH
      IKT(J3)=ISK
      ILT(J3)=ISL
      T22(J3)=TSS
   50 T2(J3)=TS
      DSUM=0.0
      DSUMA=0.0
      DSUMB=0.0
      DSUMC=0.0
      TSUMA=0.0
      TSUMB=0.0
      TSUMC=0.0
      NTALA=0
      NTALB=0
      NTALC=0
      GRAD=114.59156
      WRITE(IOUT,61)
      WRITE(IOUT,60)
   61 FORMAT(' ','                                              '
     >           '  Calculated')
   60 FORMAT(' ','   H   K   L  2Th-obs 2Th-calc    D-obs  I-obs'
     >           '   H   K   L Delta(2Th)')
C   60 FORMAT(' ','   H   K   L SST-OBS  SST-CALC   DELTA   2TH-OBS',
C     >           ' 2TH-CALC D-OBS   FREE PARAM.')
      J=1
      DO 280 I=1,N2
   70 IF(J-N1) 80,80,260
   80 IF(I-N2) 90,100,100
   90 IF(ABS(T11(J)-T22(I))-ABS(T11(J)-T22(I+1))) 100,100,260  
  100 IF(ABS(T11(J)-T22(I))-DT) 180,180,130  
  130 IF(T11(J)-T22(I)) 140,140,260
  140 HA=SQRT(T1(J))
      DOBS=WAVE/(2.*HA)
      HD=SQRT(1.-T1(J))
      THO=GRAD*ATAN(HA/HD)
      NOIND=NOIND+1
      lh=ihi(j)
      lk=iki(j)
      ll=ili(j)
      if(isym.eq.1) then
       ssqtc=(lh*lh+lk*lk+ll*ll)*a11
      elseif(isym.eq.2) then
       ssqtc=(lh*lh+lk*lk)*a11+ll*ll*a33
      elseif(isym.eq.4) then
       ssqtc=(lh*lh+lh*lk+lk*lk)*a11+ll*ll*a22
      elseif(isym.eq.7) then
       ssqtc=(lh*lh+lk*lk+ll*ll)*a11+(lh*lk+lh*ll+lk*ll)*a22
      else
      ssqtc=lh*lh*a11+lk*lk*a22+ll*ll*a33+lh*ll*a13+lh*lk*a12+lk*ll*a23
      endif
      ha=sqrt(ssqtc)
      hb=sqrt(1.-ssqtc)
      thcalc=atan(ha/hb)*grad      
      deltc=tho-thcalc
      if (dobs .lt. 2.501) then
      write(iout,150) lh,lk,ll,tho,thcalc,dobs,itag(j),     
     >                lh,lk,ll,deltc,dt
  150 FORMAT(' ',3I4,2F9.3,F9.4,I7,3I4,F11.3,' >'F6.3)
      else if(dobs .lt. 5.001) then
      write(iout,151) lh,lk,ll,tho,thcalc,dobs,itag(j),     
     >                lh,lk,ll,deltc,dt
  151 FORMAT(' ',3I4,2F9.3,F8.3,1X,i7,3I4,F11.3,' >'F6.3)
      else 
      write(iout,152) lh,lk,ll,tho,thcalc,dobs,itag(j),     
     >                lh,lk,ll,deltc,dt
  152 FORMAT(' ',3I4,2F9.3,F7.2,2X,I7,3I4,F11.3,' >'F6.3)
      endif
      IF(T1(J)-SAK) 160,160,170
  160 NWIND=NWIND+1
  170 J=J+1
      GO TO 70
  180 HA=SQRT(T1(J))
      HB=SQRT(T2(I))
      DOBS=WAVE/(2.*HA)
      DCALC=WAVE/(2.*HB)
      DELTA=T1(J)-T2(I)
      HD=SQRT(1.-T2(I))
      HE=SQRT(1.-T1(J))
      THO=ATAN(HA/HE)*GRAD
      THC=ATAN(HB/HD)*GRAD
      DELTB=THO-THC
            if(ihi(j) .eq. iht(i) .and. 
     >         iki(j) .eq. ikt(i) .and.
     >         ili(j) .eq. ilt(i)) then
      if(dobs .lt. 2.501) then
      WRITE(IOUT,190)  IHI(J),IKI(J),ILI(J),THO,THC,DOBS,ITAG(J),
     >                 IHT(I),IKT(I),ILT(I),DELTB
  190 FORMAT(' ',3I4,2F9.3,F9.4,I7,3I4,F11.3)
      else if(dobs .lt. 5.001) then
      WRITE(IOUT,153)  IHI(J),IKI(J),ILI(J),THO,THC,DOBS,ITAG(J),
     >                 IHT(I),IKT(I),ILT(I),DELTB
  153 FORMAT(' ',3I4,2F9.3,F8.3,1X,I7,3I4,F11.3)     
      else
      WRITE(IOUT,154)  IHI(J),IKI(J),ILI(J),THO,THC,DOBS,ITAG(J),
     >                 IHT(I),IKT(I),ILT(I),DELTB
  154 FORMAT(' ',3I4,2F9.3,F7.2,2X,I7,3I4,F11.3)     
      endif
            else
            lh=ihi(j)
            lk=iki(j)
            ll=ili(j)
      if(isym.eq.1) then
       ssqtc=(lh*lh+lk*lk+ll*ll)*a11
      elseif(isym.eq.2) then
       ssqtc=(lh*lh+lk*lk)*a11+ll*ll*a33
      elseif(isym.eq.4) then
       ssqtc=(lh*lh+lh*lk+lk*lk)*a11+ll*ll*a22
      elseif(isym.eq.7) then
       ssqtc=(lh*lh+lk*lk+ll*ll)*a11+(lh*lk+lh*ll+lk*ll)*a22
      else
      ssqtc=lh*lh*a11+lk*lk*a22+ll*ll*a33+lh*ll*a13+lh*lk*a12+lk*ll*a23
      endif
            ha=sqrt(ssqtc)
            hb=sqrt(1.-ssqtc)
            thcalc=atan(ha/hb)*grad
      if(dobs .lt. 2.501) then
      WRITE(IOUT,191)  IHI(J),IKI(J),ILI(J),THO,THC,DOBS,ITAG(J),
     >                 IHT(I),IKT(I),ILT(I),DELTB
  191 FORMAT(' ',3I4,2F9.3,F9.4,I7,3I4,F11.3,' *')
      else if(dobs .lt. 5.001) then
      WRITE(IOUT,197)  IHI(J),IKI(J),ILI(J),THO,THC,DOBS,ITAG(J),
     >                 IHT(I),IKT(I),ILT(I),DELTB
  197 FORMAT(' ',3I4,2F9.3,F8.3,1X,I7,3I4,F11.3,' *')
      else
      WRITE(IOUT,198)  IHI(J),IKI(J),ILI(J),THO,THC,DOBS,ITAG(J),
     >                 IHT(I),IKT(I),ILT(I),DELTB
  198 FORMAT(' ',3I4,2F9.3,F7.2,2X,I7,3I4,F11.3,' *')
      endif
            deltc=tho-thcalc
      if(dobs .lt. 2.501) then
      write(iout,192) thcalc,dobs,lh,lk,ll,deltc
  192 format(' ',21X,F9.3,F9.4,7X,3I4,F11.3,' !')
      else if(dobs .lt. 5.001) then
      write(iout,231) thcalc,dobs,lh,lk,ll,deltc
  231 format(' ',21X,F9.3,F8.3,8X,3I4,F11.3,' !')
      else 
      write(iout,232) thcalc,dobs,lh,lk,ll,deltc
  232 format(' ',21X,F9.3,F9.4,7X,3I4,F11.3,' !')
      endif
            endif
      IF(SQ20-T1(J)) 210,200,200
  200 DSUMA=DSUMA+ABS(DELTA)
      TSUMA=TSUMA+ABS(DELTB)
      NTALA=NTALA+1
  210 IF(SQ30-T1(J)) 230,220,220
  220 DSUMB=DSUMB+ABS(DELTA)
      TSUMB=TSUMB+ABS(DELTB)
      NTALB=NTALB+1
  230 IF(SQFI-T1(J)) 250,240,240
  240 DSUMC=DSUMC+ABS(DELTA)
      TSUMC=TSUMC+ABS(DELTB)
      NTALC=NTALC+1
  250 J=J+1
      GO TO 280
  260 HC=SQRT(T2(I))
      HD=SQRT(1.-T2(I))
      THC=GRAD*ATAN(HC/HD)
  280 CONTINUE
  290 IF(J-N1) 300,300,330
  300 HA=SQRT(T1(J))
      HD=SQRT(1.-T1(J))
      DOBS=WAVE/(2.*HA)
      NOIND=NOIND+1
      THO=GRAD*ATAN(HA/HD)
      WRITE(IOUT,501) IHI(J),IKI(J),ILI(J),THO,DOBS,ITAG(J)
  501 FORMAT(' ',3I4,F9.3,9X,F9.4,I7)
      IF(T1(J)-SAK) 310,310,320
  310 NWIND=NWIND+1
  320 J=J+1
      GO TO 290
  330 CONTINUE
      WRITE(IOUT,331)
  331 FORMAT(' ')
      WRITE(IOUT,350) DT,N2
  350 format(' Number of calc. lines. within ',F6.3,
     >       ' deg. from obs. 2-Theta=',i5)
      IF(NTALA) 380,380,360
  360 TMEDA=TSUMA/NTALA
      DMEDA=DSUMA/NTALA
      MER=SQ20/(2.0*DMEDA*N20)+0.5
      M20=MER
      F20=20.0/(TMEDA*N20)+0.5
      NDEF=20
      WRITE(IOUT,331)
      WRITE(IOUT,370) NDEF,MER,DMEDA,NDEF,F20,TMEDA,N20
  370 FORMAT(' M(',I3,') =',I4,'  AV.EPS.=',F10.7,/,
     >       ' F(',I3,') =',F5.0,'(',F9.6,',',I5,')')
  380 IF(NTALB) 400,400,390
  390 TMEDB=TSUMB/NTALB
      DMEDB=DSUMB/NTALB
      MER=SQ30/(2.0*DMEDB*N30)+0.5
      F30=30.0/(TMEDB*N30)+0.5
      NDEF=30
      WRITE(IOUT,331)
      WRITE(IOUT,370) NDEF,MER,DMEDB,NDEF,F30,TMEDB,N30
  400 IF(NTALC) 420,420,410
  410 DMEDC=DSUMC/NTALC
      TMEDC=TSUMC/NTALC
      FN1=N1/(TMEDC*NFI)+0.5
      MER=SQFI/(2.0*DMEDC*NFI)+0.5
      IF(M20.EQ.0) M20=MER
      WRITE(IOUT,331)
      WRITE(IOUT,370) N1,MER,DMEDC,N1,FN1,TMEDC,NFI
  420 CONTINUE
      WRITE(IOUT,331)
C
C --- Write references for the M- and F-indexes.
C
      if(wt .eq. 1.) then
        write(iout,'(a)') ' UNIT weights used '
         else
        write(iout,'(a)') ' HESS weights used '
!        elseif (wt.eq.0.0) then
!       write(iout,'(a)') ' HESS weights used '
!        elseif (wt.eq.2.0) then
!        write(iout,'(a)') ' (1/dobs) weights used '
      endif
      WRITE(IOUT,331)
      write(iout,430)
  430 format(' M  cf. J.Appl.Cryst.  1 (1968) 108 ',/,
     >       ' F  cf. J.Appl.Cryst. 12 (1979)  60 ')
C
C --- Write the number of indexed lines (in a nice format) ...
C
      write(iout,331)
      write(iout,441) dt,noind
  441 format(' Number of differences >',F6.3,' deg. =',i4)
      NOIND=NWIND
      RETURN
      END

      SUBROUTINE PWW62(A,B,C,BE,S,ISYM,al)
C-----------------------------------------------------------------------
C     PARTIAL DERIVATIVES OF DIRECT LATTICE PARAMETERS(NOT TRICLINIC)
C-----------------------------------------------------------------------
      DIMENSION S(6,6)
      DO 10 I=1,6
      DO 10 J=1,6
   10 S(I,J)=0.0
      GO TO(70,60,50,30,20,80,90),ISYM
   90 S(1,1)=-0.5*a**3
      S(1,2)=0.0
      S(2,1)=0.0
      S(2,2)=-0.5*A*A*SIN(AL)
      goto 80
   20 S(1,1)=-0.5*A*A*A
      S(1,2)=0.0
      S(1,3)=-0.5*A*C*C*COS(BE)**2
      S(1,4)=-0.5*A*A*C*COS(BE)
      S(2,1)=0.0
      S(2,2)=-0.5*B*B*B
      S(2,3)=0.0
      S(2,4)=0.0
      S(3,1)=-0.5*A*A*C*COS(BE)**2
      S(3,2)=0.0
      S(3,3)=-0.5*C*C*C
      S(3,4)=-0.5*A*C*C*COS(BE)
      S(4,1)=0.25*A*A*SIN(2.*BE)
      S(4,2)=0.0
      S(4,3)=0.25*C*C*SIN(2.*BE)
      S(4,4)=0.5*A*C*SIN(BE)
      GO TO 80
   30 S(1,1)=-(3.*A*A*A)/8.
   40 S(1,2)=0.0
      S(2,1)=0.0
      S(2,2)=-0.5*C*C*C
      GO TO 80
   50 S(1,1)=-0.5*A*A*A
      S(1,2)=0.0
      S(1,3)=0.0
      S(2,1)=0.0
      S(2,2)=-0.5*B*B*B
      S(2,3)=0.0
      S(3,1)=0.0
      S(3,2)=0.0
      S(3,3)=-0.5*C*C*C
      GO TO 80
   60 S(1,1)=-0.5*A*A*A
      GO TO 40
   70 S(1,1)=-0.5*A*A*A
   80 RETURN
      END


      SUBROUTINE PW63(A11,A22,A33,A13,A12,A23,WAVE,ISYM,A,B,C,AL,BE,GA)
C-----------------------------------------------------------------------
C     CALC. OF DIRECT LATTICE PARAMETERS
C-----------------------------------------------------------------------
      A=0.0
      B=0.0
      C=0.0
      AL=0.0
      BE=0.0
      GA=0.0
      GO TO(10,20,30,40,50,90,41),ISYM
   10 A=WAVE/(2.*SQRT(A11))
      GO TO 190
   20 C=WAVE/(2.*SQRT(A22))
      GO TO 10
   30 B=WAVE/(2.*SQRT(A22))
      C=WAVE/(2.*SQRT(A33))
      GO TO 10
   40 A=WAVE/SQRT(3.*A11)
      C=WAVE/(2.*SQRT(A22))
      GO TO 190
   41 calst=a22/(2.0*a11)
      alst=acos(calst)
      alsth=alst/2.0
      calh=1.0/(2.0*cos(alsth))
      alh=acos(calh)
      al=2.0*alh
      ast=(2.0*sqrt(a11))/wave
      a=1.0/(ast*sin(al)*sin(alst))
      go to 190
   50 CBST=A13/(2.*SQRT(A11*A33))
      SBST=SQRT(1.-CBST*CBST)
      IF(CBST-0.0005) 60,70,70
   60 BE=1.570796
      GO TO 80
   70 TBST=SBST/CBST
      BST=ATAN(TBST)
      BE=3.141593-BST
   80 A=WAVE/(2.*SBST*SQRT(A11))
      B=WAVE/(2.*SQRT(A22))
      C=WAVE/(2.*SBST*SQRT(A33))
      GO TO 190
   90 S=4./(WAVE*WAVE)
      AST=SQRT(A11*S)
      BST=SQRT(A22*S)
      CST=SQRT(A33*S)
      CAST=A23/(2.*SQRT(A22*A33))
      CBST=A13/(2.*SQRT(A11*A33))
      CGST=A12/(2.*SQRT(A11*A22))
      VST=AST*BST*CST*SQRT(1.-CAST**2-CBST**2-CGST**2+2.*CAST*CBST*CGST)
      SAST=SQRT(1.-CAST**2)
      SBST=SQRT(1.-CBST**2)
      SGST=SQRT(1.-CGST**2)
      CAL=(CBST*CGST-CAST)/(SBST*SGST)
      CBE=(CAST*CGST-CBST)/(SAST*SGST)
      CGA=(CAST*CBST-CGST)/(SAST*SBST)
      IF(ABS(CAL)-0.0005) 100,110,110
  100 AL=1.570796
      GO TO 120
  110 SAL=SQRT(1.-CAL**2)
      AL=ATAN(SAL/CAL)
  120 IF(ABS(CBE)-0.0005) 130,140,140
  130 BE=1.570796
      GO TO 150
  140 SBE=SQRT(1.-CBE**2)
      BE=ATAN(SBE/CBE)
  150 IF(ABS(CGA)-0.0005) 160,170,170
  160 GA=1.570796
      GO TO 180
  170 SGA=SQRT(1.-CGA**2)
      GA=ATAN(SGA/CGA)
  180 A=(BST*CST*SAST)/VST
      B=(AST*CST*SBST)/VST
      C=(AST*BST*SGST)/VST
  190 RETURN
      END


      SUBROUTINE PWW71(T2,WG,IHT,IKT,ILT,IQE,ISYM,N2,A11,A22,A33,A13,A12
     >                ,A23,V2,V22,T22,NEKV)
C-----------------------------------------------------------------------
**    DIMENSION T2(1),WG(1),IHT(1),IKT(1),ILT(1),IQE(1),T22(1)
      DIMENSION T2(N2),WG(N2),IHT(N2),IKT(N2),ILT(N2),
     *IQE(N2),T22(N2)
      V2=0
      V22=0
      NEKV=0
      DO 90 I=1,N2
      IF(IQE(I)) 10,90,10
   10 GO TO(20,30,40,50,60,70,100),ISYM
   20 SQ=(IHT(I)**2+IKT(I)**2+ILT(I)**2)*A11
      GO TO 80
   30 SQ=(IHT(I)**2+IKT(I)**2)*A11+ILT(I)*ILT(I)*A22
      GO TO 80
   40 SQ=(IHT(I)**2)*A11+(IKT(I)**2)*A22+(ILT(I)**2)*A33
      GO TO 80
   50 SQ=(IHT(I)**2+IHT(I)*IKT(I)+IKT(I)**2)*A11+(ILT(I)**2)*A22
      GO TO 80
   60 SQ=(IHT(I)**2)*A11+(IKT(I)**2)*A22+(ILT(I)**2)*A33+
     >   (IHT(I)*ILT(I)*A13)
      GO TO 80
   70 SQ=(IHT(I)**2)*A11+(IKT(I)**2)*A22+(ILT(I)**2)*A33+
     >    IHT(I)*ILT(I)*A13+IHT(I)*IKT(I)*A12+IKT(I)*ILT(I)*A23
            GO TO 80
  100 SQ=(IHT(I)**2+IKT(I)**2+ILT(I)**2)*A11+
     >    (iht(i)*ikt(i)+iht(i)*ilt(i)+ikt(i)*ilt(i))*a22
   80 V2=V2+WG(I)*((T2(I)-SQ))**2
      HA=SQRT(SQ)
      HB=SQRT(1.-SQ)
      TTH=114.59156*ATAN(HA/HB)
      V22=V22+(T22(I)-TTH)**2
      NEKV=NEKV+1
   90 CONTINUE
      RETURN
      END


      SUBROUTINE PWW80(SSQT,T2,N2,IHT,IKT,ILT,IH,IK,IL,ISYM,INNE)
C-----------------------------------------------------------------------
**    DIMENSION T2(1),IHT(1),IKT(1),ILT(1)
      DIMENSION T2(400),IHT(400),IKT(400),ILT(400)
      I=1
   10 IF(I-N2) 30,30,20
   20 INNE=0
      RETURN
   30 IF(SSQT-T2(I)) 40,50,40
   40 I=I+1
      GO TO 10
   50 GO TO (60,80,100,120,130,140,170),ISYM
   60 IF((IH**2+IK**2+IL**2)-(IHT(I)**2+IKT(I)**2+ILT(I)**2)) 70,160,70
   70 INNE=0
      RETURN
   80 IF((IH**2+IK**2)-(IHT(I)**2+IKT(I)**2)) 70,90,70
   90 IF(IL**2-ILT(I)**2) 70,160,70
  100 IF(IH**2-IHT(I)**2) 70,110,70
  110 IF(IK**2-IKT(I)**2) 70,90,70
  120 IF((IH**2+IH*IK+IK**2)-(IHT(I)**2+IHT(I)*IKT(I)+ILT(I)**2))
     >  70,90,70
  130 IF(IH*IL-IHT(I)*ILT(I)) 70,100,70
  140 IF(IH*IK-IHT(I)*IKT(I)) 70,150,70
  150 IF(IK*IL-IKT(I)*ILT(I)) 70,130,70
  170 if((ih*ik+ih*il+ik*il)-(iht(i)*ikt(i)+iht(i)*ilt(i)+
     > ikt(i)*ilt(i))) 70,60,70
  160 INNE=1
      RETURN
      END


      SUBROUTINE RDCEL(A11,A22,A33,A13,A12,A23,ISYM,WAVE)
C-----------------------------------------------------------------------
***   GD=0.0174533
      pi=4.0*atan(1.0)
      GD=pi/180.
      SM=(WAVE*WAVE)/4.0
      GO TO(10,50,70,80,90,100,110),ISYM
   10 AST=1.0/A11
      A11=AST*AST*SM
      A22=0.0
   20 A33=0.0
   30 A13=0.0
   40 A12=0.0
      A23=0.0
      RETURN
   50 AST=1.0/A11
   60 A11=AST*AST*SM
      CST=1.0/A22
      A22=CST*CST*SM
      GO TO 20
   70 AST=1.0/A11
      A11=AST*AST*SM
      BST=1.0/A22
      A22=BST*BST*SM
      CST=1.0/A33
      A33=CST*CST*SM
      GO TO 30
   80 AST=2.0/(A11*SQRT(3.0))
      GO TO 60
   90 SB=SIN(GD*A13)
      CBST=-COS(GD*A13)
      AST=1.0/(A11*SB)
      BST=1.0/A22
      CST=1.0/(A33*SB)
      A11=AST*AST*SM
      A22=BST*BST*SM
      A33=CST*CST*SM
      A13=2.0*AST*CST*CBST*SM
      GO TO 40
  110 alr=a22*gd
c      ainv=1.0/a11
      calsth=1.0/(2.0*cos(alr/2.0))
      alst=2.0*acos(calsth)
      ast=1.0/(a11*sin(alr)*sin(alst))
      a11=ast*ast*sm
      a22=2.0*ast*ast*cos(alst)*sm
c      a11=ainv*ainv*sm
c      a22=2.0*ainv*ainv*cos(alst)*sm
      go to 20
  100 SA=SIN(A13*GD)
      CA=COS(A13*GD)
      SB=SIN(A12*GD)
      CB=COS(A12*GD)
      SC=SIN(A23*GD)
      CC=COS(A23*GD)
      TRIG=SQRT(1.0-CA*CA-CB*CB-CC*CC+2.0*CA*CB*CC)
      A=A11
      B=A22
      C=A33
      V=A*B*C*TRIG
      AST=(B*C*SA)/V
      BST=(A*C*SB)/V
      CST=(A*B*SC)/V
      A11=AST*AST*SM
      A22=BST*BST*SM
      A33=CST*CST*SM
      CAST=(CB*CC-CA)/(SB*SC)
      CBST=(CA*CC-CB)/(SA*SC)
      CCST=(CA*CB-CC)/(SA*SB)
      A13=2.0*AST*CST*CBST*SM
      A12=2.0*AST*BST*CCST*SM
      A23=2.0*BST*CST*CAST*SM
      RETURN
      END

c-----------------------------------------------------------------------

      subroutine exti (h,k,l,flag,numext,exsymb,ilatt)
c
c --- This is a routine that sets flag = 1 if the reflex is allowed,
c     else flag = 0
c
      integer h,k,l,flag
      logical extinct
      integer exsymb(50,5),numext,ilatt
c
      if (extinct(h,k,l,numext,exsymb,ilatt)) then
         flag = 0
      else
         flag = 1
      endif
c
      return
      end
    
c-----------------------------------------------------------------------

      integer function lastch(str80)
      character str80*80
      do 1 i = 80,1,-1
         if (str80(i:i) .ne. ' ') then
            lastch = i
            return
         endif
    1 continue
      lastch = 0
      return
      end

c-----------------------------------------------------------------------

      logical function even(i)
      if (mod(i,2) .eq. 0) then
         even = .true.
      else
         even = .false.
      endif
      return
      end

c-----------------------------------------------------------------------

      subroutine get_int (str,istat,itemp)
      character str*80,s*80
      call stripp(str)
      n = lastch(str)
      s = ' '
      s(80-n+1:80) = str(1:n)
      read (s,'(I80)',iostat=istat) itemp
      return
      end
c-----------------------------------------------------------------------
      subroutine pw58(ihi,iki,ili,n1,isym)
      dimension ihi(1),iki(1),ili(1)
      do 99 i=1,n1
      ih=ihi(i)
      ik=iki(i)
      il=ili(i)
      go to(11,12,99,12,15,16,30),isym
   11 if (ih .ge. ik .and. ik .ge. il) then
        goto 30
       elseif (ih .ge. ik .and. ik .lt. il) then
        ia=ik
        ik=il
        il=ia
        goto 11
       elseif (ih .lt. ik) then
        ia=ik
        ik=ih
        ih=ia
        goto 11
       else
       goto 30
      endif
   12 if (ih .ge. ik) then
       goto 30
       else
        ia=ih
        ih=ik
        ik=ia
       endif
      goto 30
   15 if(il .lt. 0) then
       ih=-ih
       il=-il
       endif
       goto 30
   16 if(il .lt. 0) then
       ih=-ih
       ik=-ik
       il=-il
      endif    
   30 ihi(i)=ih
      iki(i)=ik
      ili(i)=il
   99 continue
      return
      end
c------------------------------------------------------------------
      subroutine rhex(ludisp,isym,wave,a1,a2,a3,a4,a5,a6,iomv,irh)
c     irh=1 rhomb. to hex   irh=2 hex. to rhomb.
      rad=6.2831853/360.
      if(irh .eq. 2) then
      goto 10
      endif
C     rhombohedral to hexagonal
      if(iomv .eq. 0) then
       smk=(wave/2.0)**2
       ast=sqrt(a1/smk)
       h1=a2/smk
       calst=h1/(2.*ast*ast)
       alst=acos(calst)
       alsth=0.5*alst
       calsth=cos(alsth)
       calh=1.0/(2.0*calsth)
       alh=acos(calh)
       al=2.0*alh
       alpha=al/rad
       a=1.0/(ast*sin(alst)*sin(al))
      endif
      if (iomv .eq. 1) then
       a=a1
       alpha=a2
      endif
      h1=cos(alpha*rad)
      ch=a*sqrt(3.*(1.+2.*h1))
      volh=3.*a**3*sqrt(1.-3.*h1**2+2.*h1**3)
      h2=(volh*2.)/(ch*sqrt(3.))
      ah=sqrt(h2)
      write(ludisp,1) ah,ch,volh
    1 format(' hexagonal cell: A=',f10.5,' C=',f10.5,' vol=',f8.2)
      write(ludisp,'(a)') ' This cell will be used in the refinements.'
      a1=ah
      a2=ch
      isym=4
      iomv=1
!corr-gfortran non accetta i pause      
!!!      pause
      write(0,'("pause")')
      read(0,*)
!corr
      return
C     irh .eq. 2  hexagonal to rhombohedral
   10 continue
      if(iomv .eq. 0) then
       smk=(wave/2.0)**2
       ast=sqrt(a1/smk)
       cst=sqrt(a2/smk)
       ah=2./(ast*sqrt(3.0))
       ch=1./cst
      endif
      if (iomv .eq. 1) then
       ah=a1
       ch=a2
      endif
      a3 = 0.0
      a4 = 0.0
      a5 = 0.0
      a6 = 0.0
      a=sqrt(ch*ch/9. + ah*ah/3.)
      h1=((3.*ch*ch)/(2.*ch*ch + 6.*ah*ah)-0.5)
      alfr=acos(h1)/rad
      volr=0.5*ah*ah*ch*sqrt(3.)/3.0
      write(ludisp,2) a,alfr
    2 format(' rhombohedral cell: A=',f10.5,' alpha=',f10.5,' deg.')
      write(ludisp,'(a,f8.2)')' rhombohedral cell volume=',volr
C    3 format(' rhombohedral cell volume=',f8.2)
      a1=a
      a2=alfr
      isym=7
      iomv=1
      write(ludisp,'(a)') ' This cell may be used in the refinements.'
c     pause
      return
      end
c-----------------------------------------------------------------------
      subroutine UseParityNode(isym,a11,a22,a33,ilatt,isub,iescellaxis)
c---
      USE General
      common /treoriterato/mwoolf(22,10),
     *                     celltreor(22,6,10),cellbest(22,6),mbest(22),
     *                     ntrialtreor,nsolut(22),negtrial,shiftzero(22)
      common /centering/CellType,CellAxis,nruntre,iesdircell
      character CellType,CellAxis*3
      common/nodeparityInfo/warnpar(2),infopar(2)
      character*80 warnpar,infopar
     
c---
      iescellaxis = 0
      iescelltype = 0
      do 1 i=1,80
           infopar(1)(i:i)=' '
           infopar(2)(i:i)=' '
           warnpar(1)(i:i)=' '
           warnpar(2)(i:i)=' '
  1   continue
c---
      if (CellType.eq.'P') then
          infopar(1)='             ****   Refined cell'//
     *      ' parameters  obtained    ****'
          infopar(2)='                  considering  the'//
     *'  lattice as primitive  '
          write(lo,'(a)')
          write(lo,'(a)')infopar(1)
          write(lo,'(a)')infopar(2)
      endif
      if (CellType.eq.'I') then
          ilatt = 2
          warnpar(1)=
     *'                     *********   WARNING *********      '
          warnpar(2)=
     *'                        Probable I-centred cell        '
          infopar(1)='             ****   Refined cell'//
     *      ' parameters  obtained    ****'
          infopar(2)='                  considering  the'//
     *'  lattice as I-centred '
          iescelltype = 1
      endif
      if (CellType.eq.'F') then
          ilatt = 3
          warnpar(1)=
     *'                     *********   WARNING *********      '
          warnpar(2)=
     *'                        Probable F-centred cell        '
          infopar(1)='             ****   Refined cell'//
     *      ' parameters  obtained    ****'
          infopar(2)='                  considering  the'//
     *'  lattice as F-centred '
          iescelltype = 1
      endif
      if (CellType.eq.'A') then
          ilatt = 4
          warnpar(1)=
     *'                     *********   WARNING *********      '
          warnpar(2)=
     *'                        Probable A-centred cell        '
          infopar(1)='             ****   Refined cell'//
     *      ' parameters  obtained    ****'
          infopar(2)='                  considering  the'//
     *'  lattice as A-centred '
          iescelltype = 1
      endif
      if (CellType.eq.'B') then
          ilatt = 5
          warnpar(1)=
     *'                     *********   WARNING *********      '
          warnpar(2)=
     *'                        Probable B-centred cell        '
          infopar(1)='             ****   Refined cell'//
     *      ' parameters  obtained    ****'
          infopar(2)='                  considering  the'//
     *'  lattice as B-centred '
          iescelltype = 1
      endif
      if (CellType.eq.'C') then
          ilatt = 6
          warnpar(1)=
     *'                     *********   WARNING *********      '
          warnpar(2)=
     *'                        Probable C-centred cell        '
          infopar(1)='             ****   Refined cell'//
     *      ' parameters  obtained    ****'
          infopar(2)='                  considering  the'//
     *'  lattice as C-centred '
          iescelltype = 1
      endif
      if (CellType.eq.'R') then
          ilatt = 7
          warnpar(1)=
     *'                     *********   WARNING *********      '
          warnpar(2)=
     *'                        Probable R-centred cell        '
          infopar(1)='             ****   Refined cell'//
     *      ' parameters  obtained    ****'
          infopar(2)='            considering  the'//
     *               '  lattice as R-centred (hexagonal)'
          iescelltype = 1
      endif
      if (iescelltype.eq.1) then
          write(lo,'(a)')
          write(lo,'(a)')warnpar(1)
          write(lo,'(a)')warnpar(2)
          write(lo,'(a)')
          write(lo,'(a)')infopar(1)
          write(lo,'(a)')infopar(2)
      endif
      if (CellAxis.eq.'A/2') then
          a11=a11/2.
          CellAxis='   '
          warnpar(1)=
     *'                     ********   WARNING *******         '
          warnpar(2)=
     *'                        Probable double cell            '
          infopar(1)='             ****   Refined cell'//
     *      ' parameters  obtained starting   ****'
          infopar(2)='                  from the New A axis ='
          cellbest(ntrialtreor,1) = cellbest(ntrialtreor,1)/2.0
          write(infopar(2)(44:),'(f9.4)')cellbest(ntrialtreor,1)
          iescellaxis = 1
      endif
      if (CellAxis.eq.'B/2') then
          a22=a22/2.
          CellAxis='   '
          warnpar(1)=
     *'                     ********   WARNING *******         '
          warnpar(2)=
     *'                        Probable double cell            '
          infopar(1)='             ****   Refined cell'//
     *      ' parameters  obtained starting   ****'
          infopar(2)='                  from the New B axis ='
          cellbest(ntrialtreor,3) = cellbest(ntrialtreor,3)/2.0
          write(infopar(2)(44:),'(f9.4)')cellbest(ntrialtreor,3)
          iescellaxis = 1
      endif
      if (CellAxis.eq.'C/2') then
c--per kuos 
          a33=a33/2.
c--- non tetragonale o esagonale 
          if (isym.eq.2.or.isym.eq.4) a22=a22/2.
          CellAxis='   '
          warnpar(1)=
     *'                     ********   WARNING *******         '
          warnpar(2)=
     *'                        Probable double cell            '
          infopar(1)='             ****   Refined cell'//
     *      ' parameters  obtained starting   ****'
          infopar(2)='                  from the New C axis ='
CCCCCCC   write(infopar(2)(44:),'(f9.4)')a33
          cellbest(ntrialtreor,5) = cellbest(ntrialtreor,5)/2.0
          write(infopar(2)(44:),'(f9.4)')cellbest(ntrialtreor,5)
          iescellaxis = 1
      endif
      if (CellAxis.eq.'A/3') then
          a11=a11/3.
          CellAxis='   '
          warnpar(1)=
     *'                     ********   WARNING *******         '
          warnpar(2)=
     *'                        Probable triple cell            '
          infopar(1)='             ****   Refined cell'//
     *      ' parameters  obtained starting   ****'
          infopar(2)='                  from the New A axis ='
          cellbest(ntrialtreor,1) = cellbest(ntrialtreor,1)/3.0
          write(infopar(2)(44:),'(f9.4)')cellbest(ntrialtreor,1)
          iescellaxis = 1
      endif
      if (CellAxis.eq.'B/3') then
          a22=a22/3.
          CellAxis='   '
          warnpar(1)=
     *'                     ********   WARNING *******         '
          warnpar(2)=
     *'                        Probable triple cell            '
          infopar(1)='             ****   Refined cell'//
     *      ' parameters  obtained starting   ****'
          infopar(2)='                  from the New B axis ='
          cellbest(ntrialtreor,3) = cellbest(ntrialtreor,3)/3.0
          write(infopar(2)(44:),'(f9.4)')cellbest(ntrialtreor,3)
          iescellaxis = 1
      endif
      if (CellAxis.eq.'C/3') then
          a33=a33/3.
c--- non tetragonale o esagonale 
          if (isym.eq.2.or.isym.eq.4) a22=a22/3.
          CellAxis='   '
          warnpar(1)=
     *'                     ********   WARNING *******         '
          warnpar(2)=
     *'                        Probable triple cell            '
          infopar(1)='             ****   Refined cell'//
     *      ' parameters  obtained starting   ****'
          infopar(2)='                  from the New C axis ='
CCCCCCC   write(infopar(2)(44:),'(f9.4)')a33
          cellbest(ntrialtreor,5) = cellbest(ntrialtreor,5)/3.0
          write(infopar(2)(44:),'(f9.4)')cellbest(ntrialtreor,5)
          iescellaxis = 1
      endif
      if (iescellaxis.eq.1) then
          write(lo,'(a)')
          write(lo,'(a)')warnpar(1)
          write(lo,'(a)')warnpar(2)
          write(lo,'(a)')
          write(lo,'(a)')infopar(1)
          write(lo,'(a)')infopar(2)
      endif
c
      if (ilatt.eq.1) then
          isub = 0
       else
          isub = 1
      endif
      return
      end
c------------------------------------------------------------------
      subroutine FillCell(M20,MpM20,irefine,a,al,b,be,c,ga,noind,volm,
     *                    icheckref)
      common /treoriterato/mwoolf(22,10),
     *                     celltreor(22,6,10),cellbest(22,6),mbest(22),
     *                     ntrialtreor,nsolut(22),negtrial,shiftzero(22)
!corr      common/genob/irho,thetamin
      common/genob2/thmin,thmax,thstep
c--
      common /bestone/icheckbest,shiftzerobest,IHKLbest(200,4),nrefbest
     *               ,m20best,mnew20
      common/sistemacris/isys,isyss(660),isyst(10)
      common /pirumprint/isolpir,mbestprint(3,10,22),m20print(3,10,22),
     *                   shiftprint(3,10,22),cellbestprint(3,6,10,22)
     *                   ,noindprint(3,10,22),volcprint(3,10,22)
     *                   ,isysprint(3,10,22),centringprint(3,10,22)
      common/CellPirum/mpirbest(3),cellpirbest(3,6),zeroshiftpir(3),
     *                 iespir
      common /fast/deltath,mnew20_old,iend,iminus,iplus,imatrice
      common /centering/CellType,CellAxis,nruntre,iesdircell
      character CellType,CellAxis*3,centringprint
      common /ratiod/dratio,iproteina,volprot
      common/stand_dev_cell_vol/ stand_cell(6),stand_vol
      common/st_pirum/st_cellbestprint(3,6,10,22),st_volcprint(3,10,22)
     *               ,st_cellsav(6,660),st_volc(660)
c--
      iespir=0
      if (iplus.eq.0.and.iminus.eq.0.and.iend.eq.0.and.irefine.ge.1)then
         iespir = 1 
         if (icheckref.eq.0) then
             mpirbest(iespir)= MpM20
             zeroshiftpir(iespir)= shiftzero(ntrialtreor)
             shiftprint(1,isolpir,ntrialtreor)=2.*shiftzero(ntrialtreor)
         endif
      endif
c----------------
       if (iproteina.eq.1) volm = volm*dratio*dratio*dratio
c----------------
       if (iespir.ge.1.and.iespir.le.3) then
          if (icheckref.eq.0.or.
     *       (icheckref.eq.1.and.MpM20.ge.mpirbest(iespir))) then
               mpirbest(iespir)= MpM20
              cellpirbest(iespir,1)= A
              cellpirbest(iespir,2)= AL
              cellpirbest(iespir,3)= B
              cellpirbest(iespir,4)= BE
              cellpirbest(iespir,5)= C
              cellpirbest(iespir,6)= GA
              cellbestprint(iespir,1,isolpir,ntrialtreor)= A
              cellbestprint(iespir,2,isolpir,ntrialtreor)= AL
              cellbestprint(iespir,3,isolpir,ntrialtreor)= B
              cellbestprint(iespir,4,isolpir,ntrialtreor)= BE
              cellbestprint(iespir,5,isolpir,ntrialtreor)= C
              cellbestprint(iespir,6,isolpir,ntrialtreor)= GA
!standard deviation cella
       do isd = 1,6
       st_cellbestprint(iespir,isd,isolpir,ntrialtreor)= stand_cell(isd)
       enddo
!
              mbestprint(iespir,isolpir,ntrialtreor)=MpM20
              m20print(iespir,isolpir,ntrialtreor)=M20
              noindprint(iespir,isolpir,ntrialtreor)=noind
              isysprint(iespir,isolpir,ntrialtreor)=isyst(isolpir)
              centringprint(iespir,isolpir,ntrialtreor)=CellType
              shiftprint(iespir,isolpir,ntrialtreor) =
     *                        2.*shiftzero(ntrialtreor)
              volcprint(iespir,isolpir,ntrialtreor)=volm
!standard deviation volume
              st_volcprint(iespir,isolpir,ntrialtreor)=stand_vol
          endif
       endif
c---------------
       if (iespir.eq.1) then
           do 30 ii=1,6
              cellbest(ntrialtreor,ii) = cellpirbest(1,ii)
  30       continue
              mbest(ntrialtreor) = mpirbest(1)
              shiftzero(ntrialtreor) = zeroshiftpir(1)     
       endif
       return
       end
c------------------------------------------------------------------
      subroutine eliminacelledoppie
!
      USE General, only:lo
!
      common /treoriterato/mwoolf(22,10),
     *                     celltreor(22,6,10),cellbest(22,6),mbest(22),
     *                     ntrialtreor,nsolut(22),negtrial,shiftzero(22)
      common /pirumprint/isolpir,mbestprint(3,10,22),m20print(3,10,22),
     *                   shiftprint(3,10,22),cellbestprint(3,6,10,22)
     *                   ,noindprint(3,10,22),volcprint(3,10,22)
     *                   ,isysprint(3,10,22),centringprint(3,10,22)
      common/doppioni/idoppioni,jdopp
      character centringprint
!
c--
      dimension adiff1(6),adiff2(6)
c--
!  verifico che le celle affinate non siano coincidenti
c--
!
       jdopp = 0
!
        do iln=ntrialtreor,1,-1! per elimare celle simili a quelle gia' memorizzate nei trial precedenti
!
!!!     if (isolpir.gt.1) then
!!!        do j=1,isolpir-1
           do j=1,isolpir
             do ii=1,3
               if (mbestprint(ii,j,ntrialtreor).lt.0) goto 16
!!!!           do il=1,6
!!!!              adiff1(il) = cellbestprint(ii,il,j,ntrialtreor)
!!!!           enddo
!!!!
               adiff1(1) = cellbestprint(ii,1,j,ntrialtreor)
               adiff1(2) = cellbestprint(ii,3,j,ntrialtreor)
               adiff1(3) = cellbestprint(ii,5,j,ntrialtreor)
               adiff1(4) = cellbestprint(ii,2,j,ntrialtreor)
               adiff1(5) = cellbestprint(ii,4,j,ntrialtreor)
               adiff1(6) = cellbestprint(ii,6,j,ntrialtreor)
!!!!
!!             do jjj=j+1,isolpir
               if (nsolut(iln)==0) cycle
               do jjj=1,nsolut(iln)
             if (iln.eq.ntrialtreor.and.jjj.eq.j) cycle
                   do iij=1,3
!!!                 if (mbestprint(iij,jjj,ntrialtreor).lt.0) goto 18
                    if (mbestprint(iij,jjj,iln).lt.0) goto 18
!!!!                do ill=1,6
!!!!              adiff2(ill)=cellbestprint(iij,ill,jjj,iln)
!!!!                enddo
               adiff2(1) = cellbestprint(iij,1,jjj,iln)
               adiff2(2) = cellbestprint(iij,3,jjj,iln)
               adiff2(3) = cellbestprint(iij,5,jjj,iln)
               adiff2(4) = cellbestprint(iij,2,jjj,iln)
               adiff2(5) = cellbestprint(iij,4,jjj,iln)
               adiff2(6) = cellbestprint(iij,6,jjj,iln)
!
               iokdif1 = 0
               call permutecell(adiff1,adiff2,iokdif1)
               if (iokdif1==1) then
              if (iln.eq.ntrialtreor) write(lo,100) j,jjj
              if (iln.lt.ntrialtreor) write(lo,101)j
                   idoppioni = idoppioni + 1
!         write(6,*) j,cellbestprint(iij,1:6,j,ntrialtreor)
!         write(6,*) jjj,cellbestprint(iij,1:6,jjj,ntrialtreor)
!!               if (mbestprint(iij,jjj,ntrialtreor).le.
!!   *                mbestprint(ii,j,ntrialtreor)) then
!!                       mbestprint(iij,jjj,ntrialtreor)=-9999.
!!!!!!!!!!!      if (mbestprint(iij,jjj,iln).le.
                 if (mbestprint(iij,jjj,iln).lt.
     *                mbestprint(ii,j,ntrialtreor)) then
                         mbestprint(iij,jjj,iln)=-9999.
                         m20print(iij,jjj,iln)=-9999.
                  elseif (mbestprint(iij,jjj,iln).eq.
     *                mbestprint(ii,j,ntrialtreor)) then
                       if (noindprint(iij,jjj,iln).ge.
     *                     noindprint(ii,j,ntrialtreor)) then ! scelgo soluzione con NIX piu' basso
                         mbestprint(iij,jjj,iln)=-9999.
                         m20print(iij,jjj,iln)=-9999.
                           else
                        mbestprint(ii,j,ntrialtreor)=-9999.
                        m20print(ii,j,ntrialtreor)=-9999.
                          jdopp=j
                       endif
                       else
!!!                     mbestprint(ii,j,ntrialtreor)=-9999.
                        mbestprint(ii,j,ntrialtreor)=-9999.
                        m20print(ii,j,ntrialtreor)=-9999.
                          jdopp=j
                   endif
               endif
!
 18                        continue
                         enddo
                      enddo
 16   continue
             enddo
           enddo
!!!     endif
!
        enddo    ! per elimare celle simili a quelle gia' memorizzate nei trial precedenti
!
 100   format(//,16h WARNING: cell #,i3,11h and cell #,i3,
     *        46h are similar, only one of them will be stored.,//)
 101  format(//,
     *  58h**********************************************************,/,
     *   21h WARNING: The cell # ,i3,25h is similar to a cell 
     *   /,45h          found in a previous run of N-TREOR./,
     *   46h          Only one of them will be considered.,/,
     * 58h**********************************************************,//)
c-
       return
       end
c---------------------------------------------------------
      subroutine chaaij_add (isym,a1,a2,a3,a4,a5,a6)
      common/celltotal/celltot(6,100),m20tot(100),m20ptot(100),
!    *                 nobstot(100),ncalctot(100),voltot(100),
     *                 voltot(100),isimadd(100),
     *                 tthetatot(100),nixtot(100),isistot(100),nsoltot,
     *                 isognoind,imernoind,celladd(6),
!    *                 isymadd,mpm20,n1add,volm
     *                 isymadd,mpm20,zzzero,volm,iesadd,isoladd
c------
c----- a     =celltot(1,i)
c----- alpha =celltot(2,i)
c----- b     =celltot(3,i)
c----- beta  =celltot(4,i)
c----- c     =celltot(5,i)
c----- gamma =celltot(6,i)
c------
c--------
c---CUBIC
c--------
      if (isym .eq. 1) then
         a1 = celltot(1,isoladd)
         a2 = 0.0
         a3 = 0.0
         a4 = 0.0
         a5 = 0.0
         a6 = 0.0
c--------
c---TETRAGONAL
c--------
      elseif (isym .eq. 2) then
         a1 =  celltot(1,isoladd)
         a2 =  celltot(5,isoladd) 
         a3 = 0.0
         a4 = 0.0
         a5 = 0.0
         a6 = 0.0
c--------
c---ORTHORHOMBIC
c--------
      elseif (isym .eq. 3) then
         a1 = celltot(1,isoladd)
         a2 = celltot(3,isoladd)
         a3 = celltot(5,isoladd)
         a4 = 0.0
         a5 = 0.0
         a6 = 0.0
c--------
c---HEXAGONAL
c--------
      elseif (isym .eq. 4) then
         a1 = celltot(1,isoladd)
         a2 = celltot(5,isoladd)
         a3 = 0.0
         a4 = 0.0
         a5 = 0.0
         a6 = 0.0
c--------
c---MONOCLINIC
c--------
      elseif (isym .eq. 5) then
         a1 = celltot(1,isoladd)
         a2 = celltot(3,isoladd)
         a3 = celltot(5,isoladd)
         a4 = celltot(4,isoladd)
         a5 = 0.0
         a6 = 0.0
c--------
c---TRICLINIC
c--------
      elseif (isym .eq. 6) then
         a1 = celltot(1,isoladd)
         a2 = celltot(3,isoladd)
         a3 = celltot(5,isoladd)
         a4 = celltot(2,isoladd)
         a5 = celltot(4,isoladd)
         a6 = celltot(6,isoladd)
      endif
      return
      end

c-----------------------------------------------------------------------
      subroutine FillCellad(M20,irefine,a,al,b,be,c,ga,noind,icheckref)
!
      common/celltotal/celltot(6,100),m20tot(100),m20ptot(100),
!    *                 nobstot(100),ncalctot(100),voltot(100),
     *                 voltot(100),isimadd(100),
     *                 tthetatot(100),nixtot(100),isistot(100),nsoltot,
     *                 isognoind,imernoind,celladd(6),
!    *                 isymadd,mpm20,n1add,volm
     *                 isymadd,mpm20,zzzero,volm,iesadd,isoladd
!
c--
      common /treoriterato/mwoolf(22,10),
     *                     celltreor(22,6,10),cellbest(22,6),mbest(22),
     *                     ntrialtreor,nsolut(22),negtrial,shiftzero(22)
      common /fast/deltath,mnew20_old,iend,iminus,iplus,imatrice
      common /centering/CellType,CellAxis,nruntre,iesdircell
!!!   character CellType,CellAxis*3,centringprint
      character CellType,CellAxis*3
c--
      iespir=0
      if (iplus.eq.0.and.iminus.eq.0.and.iend.eq.0.and.irefine.ge.1)then
         iespir = 1 
         if (icheckref.eq.0) then
             m20tot(isoladd)= M20
             m20ptot(isoladd)= MpM20
!!!!         tthetatot(isoladd)=2.*shiftzero(ntrialtreor)
         endif
      endif
c----------------
       if (iespir.ge.1.and.iespir.le.3) then
          if (icheckref.eq.0.or.
     *       (icheckref.eq.1.and.MpM20.ge.m20ptot(isoladd))) then
              m20ptot(isoladd)= MpM20
              celltot(1,isoladd)= A
              celltot(2,isoladd)= AL
              celltot(3,isoladd)= B
              celltot(4,isoladd)= BE
              celltot(5,isoladd)= C
              celltot(6,isoladd)= GA
!!!!!!!!      isysprint(iespir,isolpir,ntrialtreor)=isyst(isolpir)
!!!!!!!!      centringprint(iespir,isolpir,ntrialtreor)=CellType
             nixtot(isoladd)=noind
             voltot(isoladd)=volm
             m20tot(isoladd)= M20
             m20ptot(isoladd)= MpM20
!!!!!!       tthetatot(isoladd)=2.*shiftzero(ntrialtreor)
          endif
       endif
c---------------
!!!!       if (iespir.eq.1) then
!!!!           do 30 ii=1,6
!!!!              cellbest(ntrialtreor,ii) = cellpirbest(1,ii)
!!!!  30       continue
!!!!              mbest(ntrialtreor) = mpirbest(1)
!!!!              shiftzero(ntrialtreor) = zeroshiftpir(1)     
!!!!       endif
c---------------
       return
       end
c------------------------------------------------------------------
      subroutine pworder(isym,a1,a2,a3,a4,a5,a6)
      dimension va(6),vb(6)
      va(1)=a1
      va(2)=a2
      va(3)=a3
      va(4)=a4
      va(5)=a5
      va(6)=a6
      if(isym .eq. 3 .or. isym .eq. 6) then
       goto 3
      elseif (isym .eq. 5) then
       goto 5
      endif
      return
   3  amax=0.0
      do 20 j=1,3
      do 10 i=1,3
      if(va(i) .gt. amax) then
      amax=va(i)
      index=i
      endif
   10 continue   
      vb(j)=amax
      vb(j+3)=va(index+3)
      va(index)=0.0
      amax=0.0
   20 continue
CCC  change  PEW 2008 
CC   already in order  
CC      a2=vb(1)
CC      a1=vb(2)
      a1=vb(1)
      a2=vb(2)
      a3=vb(3)
      a4=vb(4)
      a5=vb(5)
      a6=vb(6)
      if(isym .eq. 3) then
      a1=vb(2)
      a2=vb(1)
      endif
      return
    5 if(va(1) .lt. va(3)) then
      a1=va(3)
      a3=va(1)
      endif
      return
      end
c-----------------------------------------------------------------------
