#include "mrconfig.h"
#include "aliasx.h"
//#include "forcomm1.h"
extern GtkWidget *finestre[6];

typedef struct {
                unsigned short red;
                unsigned short green;
                unsigned short blue;
               } rgbCol;

//**********************************************************************//
//                                                                      //
//                       P l o t   O p t i o n s                        //
//                                                                      //
//**********************************************************************//

extern "C" void set_style_pattern_(int *code, style_type style[]);
void update_style(void);
extern GdkColor color[MAX_COLORS];
extern style_type Style[NITEMS];
extern rgbCol pal_colors[16];

static GdkColor colorSave[16];
static GdkColor colorWhite = White;
static float WhiteCol[4] = {1, 1, 1, 1};

static const char *label_tab[] = {"Observed",
                                  "Background",
                                  "Calculated",
                                  "Difference",
                                  "Reflections",
                                  "Peaks",
                                  "Axis",
                                  "View"};

static const char *label_check[] = {"Show observed plot",
                                    "Show background plot",
                                    "Show calculated plot",
                                    "Show difference plot",
                                    "Show cumulative chi2 plot",
                                    "Show reflections ticks",
                                    "Show peaks",
                                    "Show axis",
                                    "Show background points",
                                    " ",
                                    " ",
                                    "Show Legend"};

GtkWidget *ComboMix(int nlines, const char *files[], const char *labels[],
                    const char *altlabels[], GtkWidget *hbox)
{
  GtkWidget *combo;
#ifdef GTK_COMBO_BOX
  GtkListStore *store;
  if(labels == NULL)
     store = gtk_list_store_new(1, GDK_TYPE_PIXBUF);
  else if(files == NULL)
     store = gtk_list_store_new(1, G_TYPE_STRING);
  else
     store = gtk_list_store_new(2, G_TYPE_STRING, GDK_TYPE_PIXBUF);
  GtkTreeIter iter;
  gtk_list_store_clear(store);
  GdkPixbuf **pixbufs;
  if(files != NULL)
     pixbufs = g_new0(GdkPixbuf *, nlines);
  for(int i=0; i<nlines; i++)
  {
      if(files != NULL)
          pixbufs[i] = create_pixbuf(files[i]);
      gtk_list_store_append (store, &iter);
      if(labels == NULL)
           gtk_list_store_set (store, &iter, 0, pixbufs[i],-1);
      else if(files == NULL)
           gtk_list_store_set (store, &iter, 0, labels[i], -1);
      else
           gtk_list_store_set (store, &iter, 0, labels[i],
                                        1, pixbufs[i],-1);
  }
  GtkTreeModel *model = GTK_TREE_MODEL(store);

  combo = gtk_combo_box_new_with_model(model);
  GtkCellRenderer *cell;
  int col = 0;
  if(labels != NULL)
  {
     cell = gtk_cell_renderer_text_new ();
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo),
                                    cell, TRUE);
     gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (combo), cell,
                                        "text", col);
     col++;
  }
  if(files != NULL)
  {
     cell = gtk_cell_renderer_pixbuf_new ();
     gtk_cell_layout_pack_end (GTK_CELL_LAYOUT (combo),
                                    cell, FALSE);
     gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (combo), cell,
                                        "pixbuf", col);
  }
  gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 6);
#else
  combo = create_combo (NULL, alt_labels, nlines, 2, NULL, NULL);
  gtk_editable_set_editable(GTK_EDITABLE(get_widget(combo, "entry")), FALSE);
  set_widget_fontN(combo);
  gtk_box_pack_start (GTK_BOX (hbox), get_widget(combo, "parent"), FALSE, FALSE, 6);
#endif
  return combo;
}

GtkWidget *comboPlotType(GtkWidget *father, int page)
{
  const char *plot_type[] = {"Line",
                            "Scatter",
                            "Line + Symbol"};

  std::stringstream suffix;
  std::string nome;
  suffix << page;

  GtkWidget *hbox = hbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(hbox), 6);

  GtkWidget *label = LABEL_NEW("Plot Type", NULL);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  GtkWidget *combo = ComboMix(static_cast<int>(NPSTYLES), NULL, plot_type,
                    plot_type, hbox);
  nome = "plot" + suffix.str();
  g_object_set_data(G_OBJECT(father), nome.c_str(), combo);
  return hbox;
}

GtkWidget *FrameLine(GtkWidget *father, int page)
{
  std::stringstream suffix;
  std::string nome, nomestile, nomecolore, nomesize;
  suffix << page;
  nome       = "frameline" + suffix.str();
  nomestile  = "lines" + suffix.str();
  nomecolore = "linec" + suffix.str();
  nomesize   = "linewidth" + suffix.str();
  const char *line_stipples[] = {"Solid     ",
                                 "Dashes    "};
  const char *old_line_stipples[] = {"Solid      _________",
                                     "Dashes     _ _ _ _ _"};
  const char *filenames[] = {"ls1.png", "ls2.png"};
  const int nstipples = (int) (sizeof(line_stipples)/sizeof(line_stipples[0]));

  GtkWidget *frame = FRAME_NEW("Line ", MyBLUE);
  g_object_set_data(G_OBJECT(father), nome.c_str(), frame);
  GtkWidget *hbox = hbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(hbox), 6);
  gtk_container_add(GTK_CONTAINER(frame), hbox);

  GtkWidget *label = LABEL_NEW("Style", NULL);
  set_alignment(label, 0, 0.5f);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  GtkWidget *combo = ComboMix(nstipples, filenames, line_stipples,
                    old_line_stipples, hbox);
  g_object_set_data(G_OBJECT(father), nomestile.c_str(), combo);

  label = LABEL_NEW("Colour", NULL);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 20);
  set_alignment(label, 0, 0.5f);
  label = color_button(WhiteCol, "Pick a colour");
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
  g_object_set_data(G_OBJECT(father), nomecolore.c_str(), label);

  GtkWidget *spin  = SPIN_WITH_LABEL("Width ", NULL, 1, 30, 1, 1, 0);
  g_object_set_data(G_OBJECT(father), nomesize.c_str(), get_widget(spin, "spin"));
  gtk_container_set_border_width(GTK_CONTAINER(spin), 4);
  gtk_box_pack_start (GTK_BOX (hbox), spin, FALSE, FALSE, 10);

  return frame;
}

GtkWidget *FrameMarkers(GtkWidget *father, int page)
{
  const char *markers[] = {"Crossx       ",
                           "Cross        ",
                           "Empty Circle ",
                           "Filled Circle",
                           "Empty Square ",
                           "Filled Square"};

  const char *markers_xpm[] = {"crossx.xpm",
                         "cross.xpm",
                         "empty_circle.xpm",
                         "fill_circle.xpm",
                         "square_empty.xpm",
                         "square.xpm"};
  int ntypes = (int) (sizeof(markers_xpm)/sizeof(markers_xpm[0]));
  std::stringstream suffix;
  std::string nome, nomemark, nomecolore, nomesize;
  suffix << page;
  nome       = "framemark" + suffix.str();
  nomemark   = "markers" + suffix.str();
  nomecolore = "markerc" + suffix.str();
  nomesize   = "markersize" + suffix.str();

  GtkWidget *frame = FRAME_NEW("Markers ", MyBLUE);
  g_object_set_data(G_OBJECT(father), nome.c_str(), frame);
  GtkWidget *hbox = hbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(hbox), 6);
  gtk_container_add(GTK_CONTAINER(frame), hbox);

  GtkWidget *label = LABEL_NEW("Type", NULL);
  set_alignment(label, 0, 0.5f);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  GtkWidget *combo = ComboMix(ntypes, markers_xpm, markers,
                    markers, hbox);
  g_object_set_data(G_OBJECT(father), nomemark.c_str(), combo);

  label = LABEL_NEW("Colour", NULL);
  set_alignment(label, 0, 0.5f);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 10);
  label = color_button(WhiteCol, "Pick a colour");
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
  g_object_set_data(G_OBJECT(father), nomecolore.c_str(), label);

  GtkWidget *spin  = SPIN_WITH_LABEL("Size ", NULL, 1, 30, 8, 1, 0);
  g_object_set_data(G_OBJECT(father), nomesize.c_str(), get_widget(spin, "spin"));
  gtk_container_set_border_width(GTK_CONTAINER(spin), 4);
  gtk_box_pack_start (GTK_BOX (hbox), spin, FALSE, FALSE, 10);

  return frame;
}

void cb_type(GtkWidget *w, gpointer data)
{
   int tipo;
   GtkWidget *dlg = gtk_widget_get_toplevel(w);
   std::stringstream suffix;
   std::string nome;
   int page = GPOINTER_TO_INT(data);
   suffix << page;
   get_comboselection(w, &tipo);
   nome       = "framemark" + suffix.str();
   GtkWidget *frame = get_widget(dlg, nome.c_str());
   gtk_widget_set_sensitive(frame, (tipo > SCATTERPLOT));
   nome       = "frameline" + suffix.str();
   frame = get_widget(dlg, nome.c_str());
   gtk_widget_set_sensitive(frame, (tipo != (SCATTERPLOT+1)));
}

void FrameObserved(GtkWidget *notebook, GtkWidget *father)
{
  int npag = STYLE_OBS;
  std::stringstream suffix;
  std::string nome;
  suffix << npag;
  GtkWidget *vbox1 = vbox_new (FALSE, 2);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (notebook), vbox1);
  gtk_container_set_border_width(GTK_CONTAINER(vbox1), 8);

  GtkWidget *check = CHECK_BUTTON_WITH_LABEL (label_check[npag], NULL);
  gtk_box_pack_start (GTK_BOX (vbox1), check, FALSE, FALSE, 6);
  nome = "check" + suffix.str();
  g_object_set_data(G_OBJECT(father), nome.c_str(), check);

  GtkWidget *combo = comboPlotType (father, npag);
  gtk_box_pack_start (GTK_BOX (vbox1), combo, FALSE, FALSE, 6);

  GtkWidget *frame_line = FrameLine(father, npag);
  gtk_box_pack_start (GTK_BOX (vbox1), frame_line, FALSE, FALSE, 6);
  gtk_widget_set_sensitive(frame_line, FALSE);

  GtkWidget *frame_markers = FrameMarkers(father, npag);
  gtk_box_pack_start (GTK_BOX (vbox1), frame_markers, FALSE, FALSE, 6);
  gtk_widget_set_sensitive(frame_markers, FALSE);

  combo = get_widget(father, "plot0");
#ifndef GTK_COMBO_BOX
      g_signal_connect(G_OBJECT(GTK_COMBO(combo)->entry), "changed",
          G_CALLBACK(cb_type), GINT_TO_POINTER(npag));
#else
      g_signal_connect(G_OBJECT(combo), "changed", G_CALLBACK(cb_type), GINT_TO_POINTER(npag));
#endif

  set_tab_label (GTK_NOTEBOOK (notebook), npag, label_tab[npag]);
}

void FrameBackground(GtkWidget *notebook, GtkWidget *father)
{
  int npag = STYLE_BACK;
  std::stringstream suffix;
  std::string nome;
  suffix << npag;
  GtkWidget *vbox1 = vbox_new (FALSE, 2);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (notebook), vbox1);
  gtk_container_set_border_width(GTK_CONTAINER(vbox1), 8);

  GtkWidget *check = CHECK_BUTTON_WITH_LABEL (label_check[npag], NULL);
  gtk_box_pack_start (GTK_BOX (vbox1), check, FALSE, FALSE, 6);
  nome = "check" + suffix.str();
  g_object_set_data(G_OBJECT(father), nome.c_str(), check);

  (void) comboPlotType (father, npag);

  GtkWidget *frame_line = FrameLine(father, npag);
  gtk_box_pack_start (GTK_BOX (vbox1), frame_line, FALSE, FALSE, 6);
  (void) FrameMarkers(father, npag);

  int npag_p = STYLE_BACKP;
  std::stringstream suff1;
  suff1 << npag_p;
  check = CHECK_BUTTON_WITH_LABEL (label_check[npag_p], NULL);
  gtk_box_pack_start (GTK_BOX (vbox1), check, FALSE, FALSE, 6);
  nome = "check" + suff1.str();
  g_object_set_data(G_OBJECT(father), nome.c_str(), check);

  (void) comboPlotType (father, npag_p);
  (void) FrameLine(father, npag_p);
  GtkWidget *frame_markers = FrameMarkers(father, npag_p);
  gtk_box_pack_start (GTK_BOX (vbox1), frame_markers, FALSE, FALSE, 6);

  set_tab_label (GTK_NOTEBOOK (notebook), npag, label_tab[npag]);
}

void FrameCalculated(GtkWidget *notebook, GtkWidget *father)
{
  int npag = STYLE_CALC;
  std::stringstream suffix;
  std::string nome;
  suffix << npag;
  GtkWidget *vbox1 = vbox_new (FALSE, 2);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (notebook), vbox1);
  gtk_container_set_border_width(GTK_CONTAINER(vbox1), 8);

  GtkWidget *check = CHECK_BUTTON_WITH_LABEL (label_check[npag], NULL);
  gtk_box_pack_start (GTK_BOX (vbox1), check, FALSE, FALSE, 6);
  nome = "check" + suffix.str();
  g_object_set_data(G_OBJECT(father), nome.c_str(), check);

  (void) comboPlotType (father, npag);

  GtkWidget *frame_line = FrameLine(father, npag);
  gtk_box_pack_start (GTK_BOX (vbox1), frame_line, FALSE, FALSE, 6);

  (void) FrameMarkers(father, npag);

  set_tab_label (GTK_NOTEBOOK (notebook), npag, label_tab[npag]);
}

void FrameDifference(GtkWidget *notebook, GtkWidget *father)
{
  int npag = STYLE_DIFF;
  GtkWidget *vbox1 = vbox_new (FALSE, 2);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (notebook), vbox1);
  gtk_container_set_border_width(GTK_CONTAINER(vbox1), 8);

  for(int i=0; i<2; i++)
  {
      std::stringstream suffix;
      std::string nome;
      suffix << npag+i;
      GtkWidget *check = CHECK_BUTTON_WITH_LABEL (label_check[npag+i], NULL);
      gtk_box_pack_start (GTK_BOX (vbox1), check, FALSE, FALSE, 6);
      nome = "check" + suffix.str();
      g_object_set_data(G_OBJECT(father), nome.c_str(), check);

      (void) comboPlotType (father, npag+i);

      GtkWidget *frame_line = FrameLine(father, npag+i);
      gtk_box_pack_start (GTK_BOX (vbox1), frame_line, FALSE, FALSE, 6);

      (void) FrameMarkers(father, npag+i);
  }
  set_tab_label (GTK_NOTEBOOK (notebook), npag, label_tab[npag]);
}

void FrameReflections(GtkWidget *notebook, GtkWidget *father)
{
  int npag = STYLE_REFL;
  std::stringstream suffix;
  std::string nome;
  suffix << npag;
  GtkWidget *vbox1 = vbox_new (FALSE, 2);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (notebook), vbox1);
  gtk_container_set_border_width(GTK_CONTAINER(vbox1), 8);

  GtkWidget *check = CHECK_BUTTON_WITH_LABEL (label_check[npag], NULL);
  gtk_box_pack_start (GTK_BOX (vbox1), check, FALSE, FALSE, 6);
  nome = "check" + suffix.str();
  g_object_set_data(G_OBJECT(father), nome.c_str(), check);

  (void) comboPlotType (father, npag);

  GtkWidget *frame_line = FrameLine(father, npag);
  gtk_box_pack_start (GTK_BOX (vbox1), frame_line, FALSE, FALSE, 6);

  (void) FrameMarkers(father, npag);

  set_tab_label (GTK_NOTEBOOK (notebook), npag-1, label_tab[npag-1]);
}

void FramePeaks(GtkWidget *notebook, GtkWidget *father)
{
  int npag = STYLE_PEAKS;
  std::stringstream suffix;
  std::string nome;
  suffix << npag;
  GtkWidget *vbox1 = vbox_new (FALSE, 2);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (notebook), vbox1);
  gtk_container_set_border_width(GTK_CONTAINER(vbox1), 8);

  GtkWidget *check = CHECK_BUTTON_WITH_LABEL (label_check[npag], NULL);
  gtk_box_pack_start (GTK_BOX (vbox1), check, FALSE, FALSE, 6);
  nome = "check" + suffix.str();
  g_object_set_data(G_OBJECT(father), nome.c_str(), check);

  (void) comboPlotType (father, npag);

  GtkWidget *frame_line = FrameLine(father, npag);
  gtk_box_pack_start (GTK_BOX (vbox1), frame_line, FALSE, FALSE, 6);

  (void) FrameMarkers(father, npag);

  set_tab_label (GTK_NOTEBOOK (notebook), npag-1, label_tab[npag-1]);
}

void FrameAxes(GtkWidget *notebook, GtkWidget *father)
{
  int npag = STYLE_AXIS;
  std::stringstream suffix;
  std::string nome;
  suffix << npag;
  GtkWidget *vbox1 = vbox_new (FALSE, 2);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (notebook), vbox1);
  gtk_container_set_border_width(GTK_CONTAINER(vbox1), 8);

  GtkWidget *check = CHECK_BUTTON_WITH_LABEL (label_check[npag], NULL);
  gtk_box_pack_start (GTK_BOX (vbox1), check, FALSE, FALSE, 6);
  nome = "check" + suffix.str();
  g_object_set_data(G_OBJECT(father), nome.c_str(), check);

  (void) comboPlotType (father, npag);

  GtkWidget *frame_line = FrameLine(father, npag);
  gtk_box_pack_start (GTK_BOX (vbox1), frame_line, FALSE, FALSE, 6);

  (void) FrameMarkers(father, npag);

  set_tab_label (GTK_NOTEBOOK (notebook), npag-1, label_tab[npag-1]);
}

void FrameView(GtkWidget *notebook, GtkWidget *father)
{
  int npag = STYLE_INTERVAL;
  GtkWidget *vbox1 = vbox_new (FALSE, 2);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (notebook), vbox1);
  gtk_container_set_border_width(GTK_CONTAINER(vbox1), 8);

  for(int i=0; i<3; i++)
  {
      std::stringstream suffix;
      std::string nome;
      suffix << npag+i;
      GtkWidget *check = CHECK_BUTTON_WITH_LABEL (label_check[npag+i], NULL);
      if((npag+i) == STYLE_LEGEND)
         gtk_box_pack_start (GTK_BOX (vbox1), check, FALSE, FALSE, 6);
      nome = "check" + suffix.str();
      g_object_set_data(G_OBJECT(father), nome.c_str(), check);

      (void) comboPlotType (father, npag+i);
      (void) FrameLine(father, npag+i);
      (void) FrameMarkers(father, npag+i);

      if((npag+i) == STYLE_BG)
      {
         GtkWidget *frame = FRAME_NEW("Background color ", MyBLUE);
         GtkWidget *vbox = vbox_new(FALSE, 0);
         gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
         gtk_container_add(GTK_CONTAINER(frame), vbox);
         gtk_box_pack_start (GTK_BOX (vbox1), frame, FALSE, FALSE, 6);
         GSList *group = NULL;
         GtkWidget *button = RADIO_BUTTON_WITH_LABEL (group, "Black", NULL);
         group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
         gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
         g_object_set_data(G_OBJECT(father), "blackbg", button);
         button = RADIO_BUTTON_WITH_LABEL (group, "White", NULL);
         group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
         gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
         g_object_set_data(G_OBJECT(father), "whitebg", button);
      }
  }
  set_tab_label (GTK_NOTEBOOK (notebook), npag-2, label_tab[npag-2]);
}

static void UpdateColors()
{
  int i;
  float base = 65535.f;

  for(i=0;i<16;i++)
  {
        pal_colors[i].red   =
                  (unsigned short)((float) color[i].red * 255.f / base);
        pal_colors[i].green =
                  (unsigned short)((int) color[i].green * 255.f / base);
        pal_colors[i].blue  =
                  (unsigned short)((int) color[i].blue  * 255.f / base);
  }
  c_allocacolori(1);
  update_style();
}

static void SaveColors()
{
  for(int i=0;i<16;i++)
  {
      colorSave[i].red   = color[i].red;
      colorSave[i].green = color[i].green;
      colorSave[i].blue  = color[i].blue;
  }
}

static void RestoreColors()
{
  for(int i=0;i<16;i++)
  {
      color[i].red   = colorSave[i].red;
      color[i].green = colorSave[i].green;
      color[i].blue  = colorSave[i].blue;
  }
}

static void GetStyle(GtkWidget *dlg)
{
  float colore[4];
  for(int i=0; i<NITEMS; i++)
  {
     std::stringstream suffix;
     suffix << i;
     std::string nome = "check" + suffix.str();
     GtkWidget *check = get_widget(dlg, nome.c_str());
     if(gtk_widget_get_visible(check))
     {
        if(gtk_widget_get_sensitive(check))
           Style[i].vis = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));
        else
           Style[i].vis = -1;
     }
     nome = "plot" + suffix.str();
     GtkWidget *w = get_widget(dlg, nome.c_str());
     if(gtk_widget_get_visible(w))
        get_comboselection(w, &Style[i].plot_type);
     nome = "lines" + suffix.str();
     w = get_widget(dlg, nome.c_str());
     if(gtk_widget_get_visible(w))
        get_comboselection(w, &Style[i].linetype);
     nome = "linec" + suffix.str();
     w = get_widget(dlg, nome.c_str());
     if(gtk_widget_get_visible(w))
     {
        if(i != STYLE_BG)
        {
           color_button_get_color(w, colore);
           color[Style[i].colline] = *ToGdkColor(colore);
        }
     }
     nome = "linewidth" + suffix.str();
     w = get_widget(dlg, nome.c_str());
     if(gtk_widget_get_visible(w))
        Style[i].linewidth = spin_button_get_value_as_int(w);
     nome = "markers" + suffix.str();
     w = get_widget(dlg, nome.c_str());
     if(gtk_widget_get_visible(w))
        get_comboselection(w, &Style[i].markertype);
     nome = "markerc" + suffix.str();
     w = get_widget(dlg, nome.c_str());
     if(gtk_widget_get_visible(w))
     {
        color_button_get_color(w, colore);
        color[Style[i].colmarker] = *ToGdkColor(colore);
     }
     nome = "markersize" + suffix.str();
     w = get_widget(dlg, nome.c_str());
     if(gtk_widget_get_visible(w))
        Style[i].sizemarker = spin_button_get_value_as_int(w);
  }
  Style[STYLE_BG].colline = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget(dlg, "whitebg")));
  UpdateColors();
}

static void SetStyle(GtkWidget *dlg)
{
  for(int i=0; i<NITEMS; i++)
  {
     std::stringstream suffix;
     suffix << i;
     std::string nome = "check" + suffix.str();
     GtkWidget *check = get_widget(dlg, nome.c_str());
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), (Style[i].vis == 1));
     gtk_widget_set_sensitive(check, (Style[i].vis != -1));
     nome = "plot" + suffix.str();
     GtkWidget *w = get_widget(dlg, nome.c_str());
     combo_set_val(w, Style[i].plot_type);
     nome = "lines" + suffix.str();
     w = get_widget(dlg, nome.c_str());
     combo_set_val(w, Style[i].linetype);
     nome = "linec" + suffix.str();
     w = get_widget(dlg, nome.c_str());
     GdkColor colore = color[Style[i].colline];
     color_button_set_color(w, ToColor4f(&colore));
     nome = "linewidth" + suffix.str();
     w = get_widget(dlg, nome.c_str());
     spin_button_set_value(w, Style[i].linewidth);
     nome = "markers" + suffix.str();
     w = get_widget(dlg, nome.c_str());
     combo_set_val(w, Style[i].markertype);
     nome = "markerc" + suffix.str();
     w = get_widget(dlg, nome.c_str());
     colore = color[Style[i].colmarker];
     color_button_set_color(w, ToColor4f(&colore));
     nome = "markersize" + suffix.str();
     w = get_widget(dlg, nome.c_str());
     spin_button_set_value(w, Style[i].sizemarker);
  }
  if(Style[STYLE_BG].colline)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget(dlg, "whitebg")), TRUE);
  else
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget(dlg, "blackbg")), TRUE);
}
void on_style_changed(GtkWidget *but, gpointer data)
{
      GtkWidget *dialog1 = gtk_widget_get_toplevel(but);
      int result = GPOINTER_TO_INT(data);
      int code = 3;

      switch(result)
      {
      default:
      case GTK_RESPONSE_CANCEL:
           RestoreColors();
           UpdateColors();
           break;
      case GTK_RESPONSE_APPLY:
           GetStyle(dialog1);
           code = 1;
           break;
      case GTK_RESPONSE_OK:
           GetStyle(dialog1);
           code = 2;
           update_style();
           break;
      }
      set_style_pattern_(&code, Style);
      if(result != GTK_RESPONSE_APPLY)
         gtk_widget_destroy(dialog1);
}
void OpenStyleWindow(GtkWidget *item, gpointer data)
{
//  GdkColor colorvet[16];
  int code = 0;

  set_style_pattern_(&code, Style);
  GtkWidget *dialog1 = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dialog1), "Plot Options");
  gtk_container_set_border_width(GTK_CONTAINER(dialog1), 8);
  GtkWidget *vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dialog1));
  gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog1), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(dialog1), GTK_WINDOW(finestre[0]));

  GtkWidget *notebook = NewNotebook (vbox1);
  //gtk_box_pack_start (GTK_BOX (vbox1), notebook, TRUE, TRUE, 0);
  widget_modify_bg(notebook, GTK_STATE_NORMAL, &colorWhite);
  g_object_set_data(G_OBJECT(dialog1), "nb", notebook);
  gtk_widget_show (notebook);

  FrameObserved(notebook, dialog1);
  FrameBackground(notebook, dialog1);
  FrameCalculated(notebook, dialog1);
  FrameDifference(notebook, dialog1);
  FrameReflections(notebook, dialog1);
  FramePeaks(notebook, dialog1);
  FrameAxes(notebook, dialog1);
  FrameView(notebook, dialog1);
  SetStyle(dialog1);
  SaveColors();

  GtkWidget *button = OK_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dialog1), button, GTK_RESPONSE_OK);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(on_style_changed), 
     GINT_TO_POINTER(GTK_RESPONSE_OK));
  button = APPLY_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dialog1), button, GTK_RESPONSE_APPLY);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(on_style_changed), 
     GINT_TO_POINTER(GTK_RESPONSE_APPLY));
  button = CANCEL_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dialog1), button, GTK_RESPONSE_CANCEL);
  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(on_style_changed), 
     GINT_TO_POINTER(GTK_RESPONSE_CANCEL));
  gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)), GTK_BUTTONBOX_SPREAD);
  gtk_widget_show_all(dialog1);

#if 0
  bool loop = true;
  while(loop)
  { 
      loop = false;
      int result = gtk_dialog_run(GTK_DIALOG(dialog1));
      code = 3;
      switch(result)
      {
      default:
      case GTK_RESPONSE_CANCEL:
           RestoreColors(colorvet);
           UpdateColors();
           break;
      case GTK_RESPONSE_APPLY:
           GetStyle(dialog1);
           loop = true;
           code = 1;
           break;
      case GTK_RESPONSE_OK:
           GetStyle(dialog1);
           code = 2;
           update_style();
           break;
      }
      set_style_pattern_(&code, Style);
  }
  gtk_widget_destroy(dialog1);
#endif
}

extern int altSaveP, larSaveP;
extern PangoFontDescription *fontdescP;
gpointer getCurrentPix(), getCurrentContext();

namespace legenda
{
    int SIZE_LEGENDA = 200;
    int    leg_x = 0;      //x inizio legenda
    int    leg_y = 0;      //y inizio legenda
    int    leg_w = 0;      //larghezza legenda
    int    leg_h = 0;      //altezza legenda
    int    leg_m = 0;      //legenda spostata?
    int    leg_dx = 0;     //deltax legenda
    int    leg_dy = 0;     //deltay legenda
    MyRegion *Legenda;
    gboolean moveLegenda = FALSE;

void draw_legend(gboolean ifShow)
{
   int deltax;
   int nlege, nwi, x0, y0, yi;
   GdkRectangle rleg;
   GtkWidget *drawingarea = lookup_widget(finestre[0], drawingareaname);
   gpointer outPix = getCurrentPix();
   gpointer context = getCurrentContext();

   int code = -1;
   set_style_pattern_(&code, Style);

   gpointer tog_legend = get_object(finestre[0], "legendTog");
   obj_set_active(tog_legend, Style[STYLE_LEGEND].vis, true);

   if(Legenda)
   {
      region_destroy (Legenda);
      Legenda = NULL;
   }
   GtkAllocation allocation;
   gtk_widget_get_allocation(drawingarea, &allocation);

   nlege = 0;
   for(int i=0; i<=STYLE_DIFF; i++)
       nlege += (Style[i].vis  == 1);

   nwi = (nlege+1)/3;
   nwi *= 2;
   deltax = allocation.width;

   if(!leg_m)
   {
      leg_x = deltax -(120 + larSaveP*10);
      leg_y = 20;
   }
   leg_w = 80 + larSaveP*10;
   leg_h = nlege*altSaveP+10;
//   set_forecolor(Style[STYLE_BG].colline);
//   DrawRectangle(leg_x, leg_y, leg_w,leg_h, !ifShow);
   if(!ifShow)
      return;

   int nvis = 0;
   for(int kk=0; kk<NITEMS;kk++)
       nvis += Style[kk].vis;
   if (!nvis)
       return;
   set_forecolor(Style[STYLE_BG].colline);
   DrawRectangle(leg_x, leg_y, leg_w,leg_h, 1);
   set_forecolor(1-Style[STYLE_BG].colline);
   DrawRectangle(leg_x, leg_y, leg_w,leg_h, 0);
   rleg.x = leg_x;
   rleg.y = leg_y;
   rleg.width = leg_w;
   rleg.height = leg_h;
   Legenda = (MyRegion *)region_rectangle(&rleg);
   x0 = leg_x + 10;
   y0 = leg_y + 5 ;
   yi = leg_y + 5 ;
   if(Style[STYLE_OBS].vis == 1)
   {
      if(Style[STYLE_OBS].plot_type > SCATTERPLOT)
      {
         GdkPoint punto[1];

         punto[0].x = x0+10;
         punto[0].y = y0+altSaveP/2;
         DrawPoints(finestre[0], punto, 1, 
              &color[Style[STYLE_OBS].colmarker],
              Style[STYLE_OBS].markertype, Style[STYLE_OBS].sizemarker);
      }
      if(Style[STYLE_OBS].plot_type != (SCATTERPLOT+1))
         DrawLine(x0,y0+altSaveP/2, x0+20, y0+altSaveP/2, 
                     &color[Style[STYLE_OBS].colline], Style[STYLE_OBS].linewidth);
      draw_string(outPix, fontdescP, context, x0+25, y0, "Observed", 
         &color[Style[STYLE_OBS].colline], NULL);
      yi += altSaveP;
   }

   if(Style[STYLE_CALC].vis == 1)
   {
      DrawLine(x0,yi+altSaveP/2, x0+20, yi+altSaveP/2, 
              &color[Style[STYLE_CALC].colline], Style[STYLE_CALC].linewidth);
      draw_string(outPix, fontdescP, context, x0+25, yi, "Calculated ", 
              &color[Style[STYLE_CALC].colline], NULL);
      yi += altSaveP;
   }
   if(Style[STYLE_BACK].vis == 1)
   {
      DrawLine(x0,yi+altSaveP/2, x0+20, yi+altSaveP/2, 
              &color[Style[STYLE_BACK].colline], Style[STYLE_BACK].linewidth);
      draw_string(outPix, fontdescP, context, x0+25, yi, "Background",
              &color[Style[STYLE_BACK].colline], NULL);
      yi += altSaveP;
   }
   if(Style[STYLE_DIFF].vis == 1)
   {
      DrawLine(x0,yi+altSaveP/2, x0+20, yi+altSaveP/2, 
              &color[Style[STYLE_DIFF].colline], Style[STYLE_DIFF].linewidth);
      draw_string(outPix, fontdescP, context, x0+25, yi, "Difference", 
           &color[Style[STYLE_DIFF].colline], NULL);
      yi += altSaveP;
   }
}
}             // end namespace Legenda

void TogLegend(int iflegend)
{
using namespace legenda;
   Style[STYLE_LEGEND].vis = iflegend;
   update_style();
   int code = 2;
   set_style_pattern_(&code, Style);
}
