  MODULE SPACECORR

   implicit none
   integer, allocatable :: vethkl(:)
!
   type absref_type
      integer               :: num       ! num. d'ordine del rifless
      integer, dimension(3) :: hkl       ! h,k,l
      character(len=50)     :: string    ! stringa con operatore e condizione
      real                  :: prob      ! probabilita' di estinzione
      integer               :: ktype     ! singolo (1) o in sovrapposizione (> 1)
   end type absref_type
   type(absref_type), dimension(:), allocatable :: absref    ! sistematically absent reflections
   integer                                      :: nabsref   ! numero di riflessi estinti

   CONTAINS

   subroutine search_extintion(space,hkl,csys,indov,vet1,spgl,kpr,kopenv,reflim)
!
!  Variabli in input
!  space     = stringa contenente il simbolo di estinzione
!  hkl(nr,i) = riflessi, nr=1:nrefl e i=1:3
!  csys      = sistema cristallino
!  indov     = se = 1 il riflesso e' singolo
!  vet1      = variabile rispetto alla quale calcolare la distribuzione
!  kpr       = unita' di stampa
!  kopenv    = se = 1 e' aperto il secondo viewer su cui disegnare gli istogrammi
!
!  Variabili locali
!  kprint            = 0/1 (non stampa/stampa sull'unita' kpr)
!  nrefl              = num. di riflessi
!  nl                = lunghezza della stringa space
!  ncond             = numero di condizioni per classe (3 per cella F)
!  cond(1:ncond,1:4) = condizioni di riflessione (es. 1,1,0,4 per h+k=4n)
!  class(1:3)        = classi di riflessi (es. 1,0,3 per h0l; 1,1,3 per hhl)
!  nhist             = num. di istogrammi da disegnare
!  kst               = puntatore alla stringa dell'elemento di simmetria
!  vethkl(1:nrefl)    = (>1 per i riflessi assenti) serve alla grafica per evedenziare i riflessi estinti
!  winh              = definisce la posizione degli istogrammi nell'area grafica
!  ne                = num. di riflessi estinti
!
   USE GREXPOMOD
   USE arrayutil
   USE spginfom
   implicit none
   character(len=*), intent(in)                 :: space
   integer, dimension(:,:), intent(in)          :: hkl
   integer, intent(in)                          :: csys
   integer, dimension(:), intent(in)            :: indov
   real, intent(in), dimension(:)               :: vet1
   type(spaceg_type), intent(in)                :: spgl
   integer, intent(in), optional                :: kpr
   integer, intent(in), optional                :: kopenv
   integer,dimension(4),intent(in out),optional :: reflim  !ivan refl. para las que d<limite
   integer                                      :: kprint
   integer                                      :: ne
   integer                                      :: nl
   integer                                      :: nrefl
   integer                                      :: pos
   integer                                      :: oper
   character(len=2), dimension(5)               :: strop
   character(len=6)                             :: strop1='ICABFR'
   character(len=2), dimension(13)              :: strop2=(/'2 ','21','a ','b ','c ','n ','d ','41','42','31','61','62','63'/)
   integer                                      :: kst
   integer                                      :: nre
   integer, dimension(3)                        :: class
   integer, dimension(3,4)                      :: cond
   integer                                      :: ncond
   integer                                      :: ncref
   integer, dimension(5,size(hkl,1))            :: vethist
   integer                                      :: nhist
   integer                                      :: nh
   real, dimension(5,4)                         :: winh
!corr   integer, parameter                           :: MONOCLINIC = 1
!corr   integer, parameter                           :: ORTOROMBIC = 2
!corr   integer, parameter                           :: TETRAGONAL = 3
!corr   integer, parameter                           :: TRIG = 4
!corr   integer, parameter                           :: ESAG = 5
!corr   integer, parameter                           :: CUBIC = 6
   logical                                      :: cubic205  !ivan .true. cuando estamos en Cubico y Pa-- (grupo 205)
!
   nrefl = size(hkl,1)     ! calcola il num. di riflessi
   nl = len_trim(space)    ! calcola la lunghezza della stringa
!
   if (present(kpr)) then  ! metti kprint a 0 se kpr non esiste
       kprint = 1
       if(allocated(absref))deallocate(absref)
       allocate(absref(nrefl))
       nabsref = 0
   else
       kprint = 0
   endif
!
!  Inizializza variabili
!
   class(:) = (/1,2,3/)
   ncond = 1
   !call reallocate(vethkl,nrefl,savevet=.false.)
   call new_array(vethkl,nrefl)
   vethkl(:) = 0
   nhist = 0
   oper = 1
   pos = 3
   cubic205 = .false.
!
!  - Definisci le condizioni di riflessione per l'operatore cella
!  - Cerca i riflessi estinti
!
   select case (space(1:1))
       case ('I')
         kst=1; cond(1,:4) = (/1,1,1,2/)    ! h + k + l
       case ('C')
         kst=2; cond(1,:4) = (/1,1,0,2/)    ! h + k
       case ('A')
         kst=3; cond(1,:4) = (/0,1,1,2/)    ! k + l
       case ('B')
         kst=4; cond(1,:4) = (/1,0,1,2/)    ! h + k
       case ('F')
         kst=5
         ncond = 3
         cond(1,:4) = (/1,1,0,2/)           ! h + k
         cond(2,:4) = (/0,1,1,2/)           ! k + l
         cond(3,:4) = (/1,0,1,2/)           ! h + l
       case ('R')
         kst=6
         pos=9
         if (space(4:4) == 'o') then
             cond(1,:4) = (/-1,1,1,3/)      ! obv: -h + k + l = 3n
         else
             cond(1,:4) = (/1,-1,1,3/)      ! rev:  h - k + l = 3n
         endif
   end select
   if (space(1:1) /= 'P') call searchref(strop1(kst:kst),cubic205,spgl)   ! cerca e stampa i riflessi estinti
!
!  - Definisci le condizioni di riflessione per ogni elemento di simmetria
!  - Cerca i riflessi estinti
!
   kst = 0
   ncond = 1
   select case(csys)
     !case (MONOCLINIC,ORTOROMBIC)
     case (CS_Monoclinic,CS_Orthorhombic)
      do while (pos <= nl)
         select case (space(pos:pos))

           case ('2')
             class(:) = 0
             class(oper) = oper
             cond(1,:) = (/0,0,0,2/)
             cond(1,oper)  = 1
             kst = 2
             pos = pos + 1
           case ('a')
             class(:) = (/1,2,3/)
             class(oper) = 0
             cond(1,:)  = (/1,0,0,2/)
             kst = 3
           case ('b')
             class(:) = (/1,2,3/)
             class(oper) = 0
             cond(1,:)  = (/0,1,0,2/)
             kst = 4
           case ('c')
             class(:) = (/1,2,3/)
             class(oper) = 0
             cond(1,:)  = (/0,0,1,2/)
             kst = 5
           case ('n')
             class(:) = (/1,2,3/)
             class(oper) = 0
             cond(1,:) = (/1,1,1,2/)
             cond(1,oper) = 0
             kst = 6
           case ('d')
             class(:) = (/1,2,3/)
             class(oper) = 0
             cond(1,:) = (/1,1,1,4/)
             cond(1,oper) = 0
             kst = 7
           case ('-','1','/','(',')')
             kst=0
           case (' ')
             kst = 0
             oper = oper + 1

         end select

         if (kst>0) call searchref(strop2(kst),cubic205,spgl)   ! cerca e stampa i riflessi estinti
         pos = pos + 1

      enddo

!corr     case (TETRAGONAL)
     case (CS_Tetragonal)
      do while (pos <= nl)
         select case (space(pos:pos))

           case ('2')                             ! asse 21: 0k0, k=2n
             class(:) = (/0,2,0/)
             cond(1,:) = (/0,1,0,2/)
             kst = 2
             pos = pos + 1
           case ('4')
             class(:) = (/0,0,3/)
             cond(1,:3) = (/0,0,1/)
             pos = pos + 1
             if (space(pos:pos) == '1') then      ! asse 41: 00l, l=4n
                 cond(1,4) = 4
                 kst = 8
             elseif (space(pos:pos) == '2') then  ! asse 42: 00l, l=2n
                 cond(1,4) = 2
                 kst = 9
             endif
           case ('a')                             ! piano a: hk0, h=2n
             class(:) = (/1,2,0/)
             cond(1,:) = (/1,0,0,2/)
             kst = 3
           case ('b')                             ! piano b: 0kl, k=2n
             class(:) = (/0,2,3/)
             cond(1,:) = (/0,1,0,2/)
             kst = 4
           case ('c')
             if (oper == 2) then                  ! piano c perpendicolare ad a: 0kl, l=2n
                 class(:) = (/0,2,3/)
                 cond(1,:) = (/0,0,1,2/)
             elseif (oper == 3) then              ! piano c perpendicolare alla diagonale: hhl, l=2n
                 class(:) = (/1,1,3/)
                 cond(1,:) = (/0,0,1,2/)
             endif
             kst = 5
           case ('n')
             if (oper == 1) then                  ! piano n perpendicolare c: hk0, h+k=2n
                 class(:) = (/1,2,0/)
                 cond(1,:) = (/1,1,0,2/)
             elseif (oper == 2) then              ! piano n perpendicolare a: 0kl, k+l=2n
                 class(:) = (/0,2,3/)
                 cond(1,:) = (/0,1,1,2/)
             endif
             kst = 6
           case ('d')                             ! piano d: hhl, 2h+l=4n
             class(:) = (/1,1,3/)
             cond(1,:) = (/2,0,1,4/)
             kst = 7
           case ('-','1','/')
             kst=0
           case (' ')
             kst = 0
             oper = oper + 1

         end select

         if (kst>0) call searchref(strop2(kst),cubic205,spgl)                   ! cerca i riflessi estinti
         pos = pos + 1
      enddo

!corr     case (TRIG,ESAG)
     case (CS_Trigonal,CS_Hexagonal)
      do while (pos <= nl)
         select case (space(pos:pos))

           case ('3')                             ! asse 31: 00l, l=3n
             class(:) = (/0,0,3/)
             cond(1,:) = (/0,0,1,3/)
             kst = 10
             pos = pos + 1
           case ('6')
             class(:) = (/0,0,3/)
             cond(1,:3) = (/0,0,1/)
             pos = pos + 1
             if (space(pos:pos) == '1') then      ! asse 61: 00l, l=6n
                 cond(1,4) = 6
                 kst = 11
             elseif (space(pos:pos) == '2') then  ! asse 62: 00l, l=3n
                 cond(1,4) = 3
                 kst = 12
             elseif (space(pos:pos) == '3') then  ! asse 63: 00l, l=2n
                 cond(1,4) = 2
                 kst = 13
             endif
           case ('c')
             if (oper == 2) then                  !ivan piano c perpendicolare (diagonal larga???): h-hl, l=2n
                 class(:) = (/1,-1,3/)
                 cond(1,:) = (/0,0,1,2/)
             elseif (oper == 3) then              !ivan piano c perpendicolare (diagonal corta???): hhl, l=2n
                 class(:) = (/1,1,3/)
                 cond(1,:) = (/0,0,1,2/)
             endif
             kst = 5
           case ('-','1','/')
             kst=0
           case (' ')
             kst = 0
             oper = oper + 1

         end select

         if (kst>0) call searchref(strop2(kst),cubic205,spgl)                   ! cerca i riflessi estinti
         pos = pos + 1
      enddo

!corr     case (CUBIC)
     case (CS_Cubic)
      do while (pos <= nl)
         select case (space(pos:pos))

           case ('2')                             ! asse 21: 00l, l=2n
             class(:) = (/0,0,3/)
             cond(1,:) = (/0,0,1,2/)
             kst = 2
           case ('4')
             class(:) = (/0,0,3/)
             cond(1,:3) = (/0,0,1/)
             pos = pos + 1
             if (space(pos:pos) == '1') then      ! asse 41: 00l, l=4n
                 cond(1,4) = 4
                 kst = 8
             elseif (space(pos:pos) == '2') then  ! asse 42: 00l, l=2n
                 cond(1,4) = 2
                 kst = 9
             endif
           case ('a')                             ! piano a: 0kl, k=2n
             class(:) = (/0,2,3/)
             cond(1,:4) = (/0,1,0,2/)
             kst = 3
             if (space(1:1)=='P') cubic205=.true. !ivan caso grupo numero 205
           case ('c')                             ! piano c: hhl, l=2n
             class(:) = (/1,1,3/)
             cond(1,:) = (/0,0,1,2/)
             kst = 5
           case ('d')
             if (oper == 1) then                  ! piano d perpendicolare ad ad a: 0kl, k+l=4n
                 class(:) = (/0,2,3/)
                 cond(1,:) = (/0,1,1,4/)
             elseif(oper == 3) then               ! piano d perpendicolare alla diagonale: hhl, 2h+l=4n
                 class(:) = (/1,1,3/)
                 cond(1,:) = (/2,0,1,4/)
             endif
             kst = 7
           case ('n')
             if (oper == 1) then                  ! piano n perpendicolare ad a: 0kl, k+l=2n
                 class(:) = (/0,2,3/)
                 cond(1,:) = (/0,1,1,2/)
             elseif (oper == 3) then              ! piano n perpendicolare alla diagonale: hhl, l=2n
                 class(:) = (/1,1,3/)
                 cond(1,:) = (/0,0,1,2/)
             endif
             kst = 6
           case ('-','1','/')
             kst=0
           case (' ')
             kst = 0
             oper = oper + 1

         end select

         if (kst>0) call searchref(strop2(kst),cubic205,spgl)       ! cerca i riflessi estinti
         pos = pos + 1
      enddo
   end select
!
!  Riempi vethkl per la grafica
!
   vethkl = 0
   do nh=1,nhist
!      where(vethist(nh,:) /= 0)vethkl(:nrefl) = vethist(nh,:)
      where(vethist(nh,:) == 1) vethkl(:nrefl) = 1
      where(vethist(nh,:) > 1) vethkl(:nrefl) = 2
   enddo

   !corr call scanreflections(vethkl(:nrefl))
   call scanreflections(vethkl)
!
!  Disegna gli istogrammi -- questa parte va condizionata all'apertura de viewer
!
   if (present(kopenv)) then
       if(kopenv==1) then
          call gr_select_viewer(1)
          winh(1,:) = (/0.08,0.46,0.69,0.93/)
          winh(2,:) = (/0.56,0.95,0.69,0.93/)
          winh(3,:) = (/0.08,0.46,0.39,0.63/)
          winh(4,:) = (/0.56,0.95,0.39,0.63/)
          winh(5,:) = (/0.08,0.46,0.10,0.34/)
          if (nhist<3) then
              winh(1:2,3) = 0.20
              winh(1:2,4) = 0.90
          else if (nhist==3 .or. nhist==4) then
              winh(1:2,3) = 0.60
              winh(3:4,3) = 0.10
              winh(3:4,4) = 0.40
          end if
          do nh=0,nhist
             call drawhist2()
          enddo
          call gr_select_viewer(0)
       endif
   endif
!
   CONTAINS

      function str_hklcond()    result(string)
!
!     Genera una stringa per stampare le condizioni di riflessione
!
      character(len=30)                 :: string
      integer                           :: nc,ncd
      integer                           :: ns,nst
      character(len=2), dimension(-3:3) :: sh = (/'-l','-k','-h',' 0',' h',' k',' l'/)
!
      string = ' '
      string(1:8) = sh(class(1))//sh(class(2))//sh(class(3))//' :'
      !write(string,'(3a2,1x,a1)')sh(class(1:3)),':'
      !write(string(1:5),'(a5)')'pippo'
      ns = 8
      do nc=1,ncond
         nst = 0
         if (nc > 1) then      ! scrivi ',' se ho piu' di una condizione
            ns = ns + 1
            string(ns:ns) = ','
         endif
         do ncd=1,3
            if (cond(nc,ncd) /= 0) then
                if (cond(nc,ncd) > 0)then             ! scrivi '+'
                    if (nst > 0) then
                        ns = ns+2
                        string(ns-1:ns) = ' +'
                    endif
                else                                  ! scrivi '-'
                    if (nst >= 0) then
                        ns = ns+2
                        string(ns-1:ns) = ' -'
                    endif
                endif
                if (cond(nc,ncd) > 1)then ! scrivi '2' (caso 2h+l=4n)
                    ns = ns+2
                    string(ns-1:ns) = ' 2'
                endif
                nst = nst + 1
                ns = ns + 2
                string(ns-1:ns) = sh(ncd)             ! scrivi indici h, k, l
            endif
         enddo
         if (cond(nc,4) > 2)then                      ! scrivi ' = Xn',X=3,4,6
             write(string(ns+1:ns+5),'(a3,i1,a1)')' = ',cond(nc,4),'n'
             ns = ns+5
         endif
      enddo
!
      end function str_hklcond

      subroutine searchref(stroper,cubic205,spgl)
      USE nr
              !USE prnutil
              !USE symmetry, only: kmat
!corr      USE Symmetry, only : nsym,icent
!corr      USE General, only: spacegr
!
      character(len=*), intent(in)  :: stroper
      logical, intent(in)           :: cubic205
      type(spaceg_type), intent(in) :: spgl
      integer, dimension(-3:3)      :: ind
      integer, dimension(48,3)      :: indeq
      integer                       :: nc,kncd
      integer, dimension(nrefl)     :: iorder
      integer                       :: ip
      integer                       :: i
      integer                       :: k
      integer                       :: l
      integer                       :: m
      integer                       :: nsymoo
      character(len=50)             :: strcond
!
!     - Cerca i riflessi estinti
!     - Riempie la variabile vethist usata per gli istogrammi
!       vethist = 0 riflesso presente
!       vethist = 1 riflesso assente singolo
!       vethist = 2 riflesso assente non singolo
!
!corr      nsymoo= nsym
!corr      if (icent.ne.0) nsymoo = nsymoo*2
      nsymoo = spgl%nsymop_prim
              !write(0,*)'NSYM=',nsymoo, nsym,icent, spacegr%nsymop_prim
      !write(6,*) ' Numero di operatori di simmetria =',nsymoo
      nhist = nhist + 1
      vethist(nhist,:) = 0
      ind(0) = 0
      ncref = 0
      ne = 0
      k = 0
      l = 0
      m = 0
!!!!!!!!!!
       ! do i=1,nsymoo
       !    write(6,*)'MATR kmat'
       !    call print_matrix(kmat(i,:,:),6)
       !    write(6,*)'MATR rot'
       !    call print_matrix(spgl%symop(i)%rot,6)
       ! enddo
!!!!!!!!!!

      do nre=1,nrefl
         !indeq = 0
         !call equi(hkl(nre,:),indeq(:,:),nsymoo)  !ivan calcula equivalentes
         do i=1,nsymoo
            indeq(i,:) = matmul(hkl(nre,:),spgl%symop(i)%rot)
         enddo
!
         do i=1,nsymoo
               !if (any(indeq(i,:) - matmul(hkl(nre,:),spgl%symop(i)%rot) /= 0)) then
               !    write(6,*)i,'INDEQ=',all(indeq(i,:) - matmul(hkl(nre,:),spgl%symop(i)%rot) == 0)
               !    write(6,*)i,'INDEQ=',indeq(i,:)
               !    write(6,*)i,'INDEQ=',matmul(hkl(nre,:),spgl%symop(i)%rot)
               !endif
            ind(1:3) = indeq(i,:); ind (-1:-3:-1) = -indeq(i,:)
            if (all(ind(1:3)==0)) exit
!
!           Mi chiedo se il riflesso i appartiene alla classe ncl ed e' presente
            if (sum( abs( ind(1:3) - ind(class) ) ) == 0) then
!
!               Mi chiedo se verifica la condizione
                kncd = 0
                do nc=1,ncond
                   if (mod(sum (ind(1:3)*cond(nc,:3)),cond(nc,4)) == 0) then
                       kncd = kncd + 1
                   endif
                enddo

                if (all(cond(1,:4)==(/-1,1,1,3/)).and.kncd/=0) exit !ivan si uno de los equiv. es presente,
                if (all(cond(1,:4)==(/1,-1,1,3/)).and.kncd/=0) exit !ivan   entonces el R es permitido
                if (cubic205) then
                   if (all(ind(2:3)/=0)) then
                      if (mod(ind(2),2)==0 .or. mod(ind(3),2)==0) exit !ivan si uno de los �ndices es par, el operador es permitido
                   end if
                end if

                if (kncd /= ncond) then       ! se tutte le condizioni sono contemp. verificate ...
                    if((all(cond(1,:4)==(/-1,1,1,3/)).or.all(cond(1,:4)==(/1,-1,1,3/))).and.i/=48) then
                       if(.not.all(indeq(i+1,:)==0)) cycle
                    end if
                    ne = ne + 1               ! conta gli estinti
                    if(indov(nre) == 1)then
                       vethist(nhist,nre) = 1
                    else
                       vethist(nhist,nre) = 2
                    endif
                    exit
                endif
            endif
         end do
      enddo
!
!     Stampa i riflessi estinti segnando con 's' i singoli
      if (kprint == 1) then
          strcond = trim(stroper)//' ('//trim(str_hklcond())//')'
          write(kpr,'(a)')' Simm. Operator: '//stroper//' Condition: '//trim(str_hklcond())
          write(kpr,'(a)')'     n.   h  k  l   w*E**2'
      endif
      if (ne > 0) then
          call scanreflections(vethist(nhist,:))       ! cerca gruppi in cui tutti i riflessi sono estinti
          call indexx(vet1,iorder)                     ! crea un indice dei riflessi in base a vet1
          iorder = iorder(nrefl:1:-1)                  ! in ordine decrescente
          do nre=1,nrefl             
             ip = iorder(nre)             
             if (vethist(nhist,ip) == 1) then          ! riflesso singolo
                 if (vet1(ip) > 1.0) ncref = ncref + 1 ! conta i riflessi singoli con vet1 > 1.0
                 if (present(reflim)) then
                    if (ip<reflim(1).and.vet1(ip)>0.45) k=k+1  !ivan cuenta refl. singles que cumplen los criterios
                    if (ip<reflim(2).and.vet1(ip)>0.45) l=l+1
                    if (ip<reflim(3).and.vet1(ip)>0.35) m=m+1
                    if (ip<reflim(4).and.vet1(ip)>0.5) then
                        k=k+1
                        l=l+1
                        m=m+1
                    end if
                    if (ip>reflim(3)) exit
                 end if
                 if (kprint == 1) then
                     write(kpr,'(2x,i5,1x,3i3,1x,f10.3,2x,a1)')ip,hkl(ip,:),vet1(ip),'s'
                     nabsref = nabsref + 1
                     absref(nabsref)%num = ip
                     absref(nabsref)%hkl(:) = hkl(ip,:)
                     absref(nabsref)%string = trim(strcond)
                     absref(nabsref)%prob = vet1(ip)
                     absref(nabsref)%ktype = 1
                 endif
             else if (vethist(nhist,ip) == 2) then     ! riflesso in gruppo ma non tutti i riflessi del gru. sono estinti
                      if (kprint == 1) then
                          write(kpr,'(2x,i5,1x,3i3,1x,f10.3)')ip,hkl(ip,:),vet1(ip)
                          nabsref = nabsref + 1
                          absref(nabsref)%num = ip
                          absref(nabsref)%hkl(:) = hkl(ip,:)
                          absref(nabsref)%string = trim(strcond)
                          absref(nabsref)%prob = vet1(ip)
                          absref(nabsref)%ktype = 2
                      endif
             else if (vethist(nhist,ip) > 2) then      ! riflesso in gruppo ma tutti i refl del gruppo sono estinti
                 if (present(reflim)) then
!
!                   ivan cuenta las refl. con todo el grupo extinto que cumplen los criterios
                    if (ip<reflim(1).and.vet1(ip)>0.45) k=k+1
                        if (ip<reflim(2).and.vet1(ip)>0.45) l=l+1
                        if (ip<reflim(3).and.vet1(ip)>0.35) m=m+1
                        if (ip<reflim(4).and.vet1(ip)>0.5) then
                            k=k+1
                            l=l+1
                            m=m+1
                        end if
                        if (ip>reflim(3)) exit
                     end if
                     if (kprint == 1) then
                         write(kpr,'(2x,i5,1x,3i3,1x,f10.3,2x,a2,i2,a1,i4)')   &
                           ip,hkl(ip,:),vet1(ip),'g(',indov(vethist(nhist,ip)-2),')',vethist(nhist,ip)-2
                         nabsref = nabsref + 1
                         absref(nabsref)%num = ip
                         absref(nabsref)%hkl(:) = hkl(ip,:)
                         absref(nabsref)%string = trim(strcond)
                         absref(nabsref)%prob = vet1(ip)
                         absref(nabsref)%ktype = 2
                     endif
                 endif
         enddo
         if(present(reflim)) then !ivan verifica si se cumplen los criterios para descartar el grupo de extinci�n
            if (any((/'I','A','B','C','F','R'/)==stroper)) then
                if (k>=4) reflim(1) = -abs(reflim(1))
            else if (any((/'a ','b ','c ','n ','d '/)==stroper)) then
               if (l>=2) reflim(2) = -abs(reflim(2))
            else if (any((/'21','41','42','31','61','62','63'/)==stroper)) then
               if (m>=3) reflim(3) = -abs(reflim(3))
            end if
         end if
!!
!!             Se almeno un riflesso singolo ha vet1 > 1.0 ...
!!
!!          if (kprint == 1) then
!!              if (ncref > 0) write(kpr,'(a)') 'WARNING: questo operatore non esiste!'
!!              write(kpr,*)
!!          endif
          strop(nhist) = stroper           ! memorizza la stringa dell'elemento di simmetria
      else
          nhist = nhist - 1                ! se non ci sono riflessi estinti non si disegna l'histogramma
          if (kprint == 1)write(kpr,*)'NON CI SONO RIFLESSI ESTINTI'
      endif
!
      end subroutine searchref

      subroutine drawhist2()
      USE Molcom, only : color_BLACK,color_LIGHTRED,color_LIGHTBLUE
      real                         :: datamin,datamax
      integer                      :: nbins             ! num. di barre nell'istogramma
      real                         :: maxb2             ! massimo della distribuzione
      real, dimension(nrefl)        :: veth
      integer                      :: next              ! num. di riflessi estinti
      integer                      :: nsing             ! num. di riflessi estinti singoli
!corr      integer                      :: ive
!
!!!!!      if (openviewer) then
!!!!!          call gr_select_viewer(2)
          if (nh == 0) then
              call gr_clear_area(0.0,1.0,0.0,1.0)
          else
              nbins = 40
!
!           - Conta i riflessi estinti
!           - Riempi veth in modo che da 1:nsing contenga i singoli e da nsing+1:next gli altri
!
              nsing = 0
              next = sum (vethist(nh,:),mask=vethist(nh,:)==1)
              do nre=1,nrefl
                 if (vethist(nh,nre) == 1) then
                     nsing = nsing + 1
                     veth(nsing) = vet1(nre)
                 elseif (vethist(nh,nre) >= 2) then
                     next = next + 1
                     veth(next) = vet1(nre)
                 endif
              enddo
!
              call gr_area(winh(nh,1),winh(nh,2),winh(nh,3),winh(nh,4))       !definisci l'area grafica
!
!             N.B. moltiplico per 1.01 per far entrare anche maxval(veth(:next)) in istogramma
!
              datamax = maxval(veth(:next))*1.01                              !massimo asse x dell'istogr.
              datamin = minval(veth(:next))                                   !minimo asse x dell'istogr.
              maxb2 = MAXVAL(distrib(next,veth(:next),datamin,datamax,nbins)) !massimo della distribuzione (asse y)
!
!             Istogramma per riflessi non singoli
!
              call gr_setcolor(color_LIGHTRED)
              call gr_sfs(1)
              call gr_setcolor(color_BLACK)
              call gr_setwin(datamin,datamax,0.0,maxb2)
              call gr_hist(veth(:next),nbins,1)
!
!             Istogramma per riflessi singoli
!
              call gr_setcolor(color_LIGHTBLUE)
              call gr_sfs(1)
              call gr_setcolor(color_BLACK)
              call gr_hist(veth(:nsing),nbins,1)
!
              call gr_box('AILTS',0.0,0,'AILTS',0.0,0)
              call gr_reldrawtext('T',-1.0,0.5,0.5,strop(nh),2)
          endif
!
          call zflush()
!!!!!          call gr_select_viewer(1)
!!!!!      endif
!
      end subroutine drawhist2

      subroutine scanreflections(vetin)
!
!     Cerca gruppi in cui tutti i riflessi sono estinti
!
      integer                              :: i
      integer                              :: finr
      integer,dimension(:),intent(in out)  :: vetin
!
      i=1
      do while (i <= nrefl)
         if (indov(i) > 1) then
             finr = i+indov(i)-1
!             if (all(vethist(nhist,i:finr)==2)) then ! vero se tutti i riflessi del gruppo sono estinti
!                 vethist(nhist,i:finr) = i+2         ! memorizza il puntatore al primo riflesso del gruppo
             if (all(vetin(i:finr)==2)) then ! vero se tutti i riflessi del gruppo sono estinti
                 vetin(i:finr) = i+2         ! memorizza il puntatore al primo riflesso del gruppo
             endif
         endif
         i = i + indov(i)
      enddo

      end subroutine scanreflections

   end subroutine search_extintion

   function distrib(ndat,data,datamin,datamax,nbin) result(numb)
!
   integer, intent(in)               :: ndat
   real, dimension(ndat), intent(in) :: data
   real                              :: datamin,datamax
   integer, intent(in)               :: nbin
   integer, dimension(nbin)          :: numb
   integer                           :: i
   integer                           :: ibin
!
   numb = 0
   do i=1,ndat
      ibin = (data(i)-datamin)/(datamax-datamin)*nbin+1
      if (ibin >= 1 .and. ibin <= nbin) numb(ibin) = numb(ibin) + 1
   enddo
!
   end function distrib


  END MODULE SPACECORR
!------------------------------------------------------------------------------------------------
