#include "mrconfig.h"

typedef struct {
      int   num;        // num. d'ordine del rifless
      int   hkl[3];     // h,k,l
      char  string[50]; // stringa con operatore e condizione
      float prob;       // probabilita' di estinzione
      int   ktype;      // singolo (1) o in sovrapposizione (> 1)
               } absref_type;
typedef struct {
      int num;
      char str1[10];
      char str2[60];
      float prob;
      char str3[12];
               } absref_dicomodo;

typedef struct  {
     int          num;        // Space group number
     char         symb[11];   // Space group symbol
     char         exts[20];   // Extinction symbol
     float        fig;        // Figure of merit
     int          nabs;       // Number of absent reflections
     int          nasym;      // Atoms in the asymmetric unit
     int          freq_no;    // No. in CSD
     float        freq_perc;  // % in CSD
     int          freq_rank;  // Rank in CSD
     int          chiral;     // Chiral space group?
                } spgext_info_type; 

extern "C" void get_nabsref_(int *rowspace, int *nabsref);
extern "C" void get_absref_(absref_type *nabsref);
extern "C" void selectednum_(int *num);
extern "C" void spg_selection_(int *num);
extern "C" void spg_determine_();
extern "C" void spg_list_();

bool Alphagreater(char *str1, char *str2);
extern GtkWidget *finestre[6];

void scelta_riga(GtkTreeSelection  *sele, gpointer data)
{
     GtkTreeView *tree_view;

     tree_view = gtk_tree_selection_get_tree_view(sele);
     GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree_view));
     GtkTreeIter iter;
     if(gtk_tree_selection_get_selected(sele, &model, &iter))
     {
          int num;
          gtk_tree_model_get (model, &iter, 0, &num, -1);
          selectednum_(&num);
     }
}

void RiempiLista(GtkListStore *list_store, int nabsref, std::vector<absref_dicomodo> absref)
{
  GtkTreeIter iter;
  gtk_list_store_clear(list_store);
  for(int i=0; i<nabsref; i++)
  {
      gtk_list_store_append (list_store, &iter);
      gtk_list_store_set (list_store, &iter, 0, absref[i].num,
           1, absref[i].str1, 2, absref[i].str2, 3, absref[i].prob,
           4, absref[i].str3, -1);
  }
}

bool sort_by_numd( absref_dicomodo elem1, absref_dicomodo elem2 )
{
   return (elem1.num > elem2.num);
}
bool sort_by_num ( absref_dicomodo elem1, absref_dicomodo elem2 )
{
   return (elem1.num < elem2.num);
}

bool sort_by_probd( absref_dicomodo elem1, absref_dicomodo elem2 )
{
   return (elem1.prob > elem2.prob);
}
bool sort_by_prob ( absref_dicomodo elem1, absref_dicomodo elem2 )
{
   return (elem1.prob < elem2.prob);
}

bool sort_by_hkld( absref_dicomodo elem1, absref_dicomodo elem2 )
{
   return Alphagreater(elem1.str1, elem2.str1);
}
bool sort_by_hkl ( absref_dicomodo elem1, absref_dicomodo elem2 )
{
   return Alphagreater(elem2.str1, elem1.str1);
}

bool sort_by_condd( absref_dicomodo elem1, absref_dicomodo elem2 )
{
   return Alphagreater(elem1.str2, elem2.str2);
}
bool sort_by_cond ( absref_dicomodo elem1, absref_dicomodo elem2 )
{
   return Alphagreater(elem2.str2, elem1.str2);
}

bool sort_by_ktyped( absref_dicomodo elem1, absref_dicomodo elem2 )
{
   return Alphagreater(elem1.str3, elem2.str3);
}
bool sort_by_ktype ( absref_dicomodo elem1, absref_dicomodo elem2 )
{
   return Alphagreater(elem2.str3, elem1.str3);
}

void sort_col_list(GtkWidget *column, gpointer data)
{
    bool (*sort_by_col)(absref_dicomodo, absref_dicomodo);
    std::vector<absref_dicomodo> absrefv;

    int ncol = GPOINTER_TO_INT(data);
    int ordin = get_int(column, "set_order");
    GtkWidget *treeview = get_widget(column, "father");
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
    GtkListStore *list_store = GTK_LIST_STORE(model);
    GtkTreeIter iter;
    int nabsref = n_children(treeview);
    gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
    while (valid)
    {
       absref_dicomodo absref;
       char *str1 = g_new0(char, 10);
       char *str2 = g_new0(char, 60);
       char *str3 = g_new0(char, 12);
       gtk_tree_model_get (model, &iter, 0, &(absref.num), 
                                         1, &str1,
                                         2, &str2,
                                         3, &(absref.prob),
                                         4, &str3, -1);
       CopiaStringa(absref.str1, 10, str1);
       CopiaStringa(absref.str2, 60, str2);
       CopiaStringa(absref.str3, 12, str3);
       valid = gtk_tree_model_iter_next (model, &iter);
       absrefv.push_back(absref);
    }

    if(ordin > 0)
       ordin = -1;
    else
       ordin = 1;
    g_object_set_data(G_OBJECT(column), "set_order", GINT_TO_POINTER(ordin));

    switch (ordin*(ncol+1))
    {
    case -1:
         sort_by_col = sort_by_numd;
         break;
    case -2:
         sort_by_col = sort_by_hkld;
         break;
    case -3:
         sort_by_col = sort_by_condd;
         break;
    case -4:
         sort_by_col = sort_by_probd;
         break;
    case -5:
         sort_by_col = sort_by_ktyped;
         break;
    default:
    case 1:
         sort_by_col = sort_by_num;
         break;
    case 2:
         sort_by_col = sort_by_hkl;
         break;
    case 3:
         sort_by_col = sort_by_cond;
         break;
    case 4:
         sort_by_col = sort_by_prob;
         break;
    case 5:
         sort_by_col = sort_by_ktype;
         break;
    }

    sort(absrefv.begin(), absrefv.end(), sort_by_col);
    RiempiLista(list_store, nabsref, absrefv);
}

void chiudi_absref(GtkWidget *but, GtkWidget *dlg1)
{
   gtk_widget_set_sensitive(dlg1, TRUE);
   gtk_widget_destroy(gtk_widget_get_toplevel(but));
}

void c_getabsref(GtkButton *b, GtkWidget *dlg1)
{
   int nabsref;
   absref_type *absref;
   GtkTreeSelection *sele = (GtkTreeSelection *)get_object(dlg1, "selection");
   int rowspace = row_selected(sele) + 1;
   get_nabsref_(&rowspace, &nabsref);
   absref = g_new0(absref_type, nabsref);
   get_absref_(absref);

   gtk_widget_set_sensitive(dlg1, FALSE);
   GtkWidget *dlg = gtk_dialog_new();
   gtk_window_set_title (GTK_WINDOW (dlg), "List of systematically absent reflections");
   gtk_container_set_border_width(GTK_CONTAINER(dlg), 8);
   gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(dlg1));
   PangoFontDescription *fontlist = get_fontN(MONOSPACE_FONT);
   gtk_widget_set_size_request(dlg, -1, 300);
   int lar, alt;
   get_metrics(fontlist, &lar, &alt);

   GtkWidget *vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
   const char *titles[] = { "Num", "  h  k  l", "Extinction condition",
                            "Probability for extinction", "Type"};
   GType types[] = { G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, 
                     G_TYPE_FLOAT, G_TYPE_STRING };
// la seguente setta la larghezza (fissa) delle colonne alla lunghezza del titolo)
//   int  sizes[] = { 8*lar, 12*lar, 22*lar, 28*lar, 14*lar};
// la seguente dice al gtk di autoridimensionare le colonne alla larghezza 'ottimale'
   int  sizes[] = { 0, 0, 0, 0, 0};
   int  precs[] = { 0, 0, 0, 3, 0};
   GtkWidget *list =
   new_selection(dlg, vbox1, "absref", 5, titles, sizes, types, precs);
   g_object_set_data(G_OBJECT(dlg), "absref_conn", get_widget(list, "frame"));
   widget_modify_font(list, fontlist);

   for(int i=0; i<5; i++)
  {
      GtkTreeViewColumn *column = gtk_tree_view_get_column(GTK_TREE_VIEW(list), i);
      GtkWidget *label = gtk_tree_view_column_get_widget(column);
      if(label)
         widget_modify_font(label, fontlist);
      gtk_tree_view_column_set_clickable(column, TRUE);
      g_object_set_data(G_OBJECT(column), "set_order", GINT_TO_POINTER(-1));
      g_object_set_data(G_OBJECT(column), "father", list);
      g_signal_connect(G_OBJECT(column), "clicked", G_CALLBACK(sort_col_list),
                 GINT_TO_POINTER(i));
  }

  GtkListStore *list_store = GTK_LIST_STORE(gtk_tree_view_get_model(
                             GTK_TREE_VIEW(list)));
  std::vector<absref_dicomodo> absrefv;
  for(int i=0; i<nabsref; i++)
  {
      absref_dicomodo absref1;
      const char *tipo[] = {"Overlapped", "Single    "};
      int kk = absref[i].ktype;
      if(kk != 1)
         kk = 0;
      absref1.num = absref[i].num;
      absref1.prob = absref[i].prob;
//      CopiaStringa(absref1.str2, 50, absref[i].string);
      ScriviStringa(absref1.str2, 60, "    %s", absref[i].string);
      ScriviStringa(absref1.str1, 10, "%3d%3d%3d", absref[i].hkl[0],
                                      absref[i].hkl[1],
                                      absref[i].hkl[2]);
      CopiaStringa(absref1.str3, 12, tipo[kk]);
      absrefv.push_back(absref1);
  }

  RiempiLista(list_store, nabsref, absrefv);

  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
  set_selection_mode(list, GTK_SELECTION_SINGLE);
  (void) g_signal_connect (G_OBJECT (selection), "changed",
                      G_CALLBACK (scelta_riga), dlg);

  GtkWidget *button = CLOSE_button();
  gtk_dialog_add_action_widget (GTK_DIALOG (dlg), button, 1);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
         GTK_BUTTONBOX_SPREAD);

  (void) g_signal_connect(G_OBJECT (button), "clicked",
         G_CALLBACK(chiudi_absref), dlg1);

   (void) g_signal_connect_swapped(G_OBJECT(dlg), "delete_event",
        G_CALLBACK(gtk_button_clicked), G_OBJECT(button));
   (void) g_signal_connect (G_OBJECT (dlg), "delete_event",
                      G_CALLBACK(true_cb), NULL);
  gtk_widget_show_all(dlg);
}

void scelta_riga_spg(GtkTreeSelection  *sele, gpointer data)
{
    int row = row_selected(sele) + 1;
    spg_selection_(&row);
}

static void end_dlg(GtkWidget *b, GtkWidget *dlg)
{
    int response = gtk_dialog_get_response_for_widget(GTK_DIALOG(dlg), b);
    g_object_set_data(G_OBJECT(dlg), "response", GINT_TO_POINTER(response));
    gtk_main_quit();
}

extern "C"  {

void c_open_spacegroup_(spgext_info_type spglist[], int *kselect, int *numspg)
{
      const char *titles[] = { 
        "Space Group", "Extinction symbol", "FoM", "Nabs", "Nasym",
        "No. in CSD", "% of CSD", "Rank", "Chiral"};
      GType types[] = { G_TYPE_STRING, G_TYPE_STRING, G_TYPE_FLOAT, G_TYPE_INT,
                        G_TYPE_INT, G_TYPE_INT, G_TYPE_FLOAT, G_TYPE_INT,
                        G_TYPE_STRING };
      int ncols = G_N_ELEMENTS(types);
      int  sizes[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0};
      int  precs[] = { 0, 0, 3, 0, 0, 0, 2, 0, 0};
#ifndef GTK_TOOLTIP
      GtkTooltips *tooltips = gtk_tooltips_new();
#endif

      GtkWidget *dlg = gtk_dialog_new();
      gtk_window_set_title (GTK_WINDOW (dlg), "Find space group");
      gtk_container_set_border_width(GTK_CONTAINER(dlg), 8);
      gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(finestre[0]));
      PangoFontDescription *fontlist = get_fontN(MONOSPACE_FONT);
      gtk_widget_set_size_request(dlg, -1, 300);
      int lar, alt;
      get_metrics(fontlist, &lar, &alt);

      GtkWidget *vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dlg));

      GtkWidget *list =
      new_selection(dlg, vbox1, "spglist", ncols, titles, sizes, types, precs);
      widget_modify_font(list, fontlist);
      GtkListStore *list_store = GTK_LIST_STORE(gtk_tree_view_get_model(
                             GTK_TREE_VIEW(list)));
      GtkTreeIter iter;

      for(int i=0; i<*numspg; i++)
      {
          std::string symb(spglist[i].symb, 11);
          std::string exts(spglist[i].exts, 20);
          gtk_list_store_append (list_store, &iter);
          gtk_list_store_set (list_store, &iter, 0, symb.c_str(),
          1, exts.c_str(), 2, spglist[i].fig, 3, spglist[i].nabs, 
          4, spglist[i].nasym, 5, spglist[i].freq_no, 6, spglist[i].freq_perc,
          7, spglist[i].freq_rank, 8, spglist[i].chiral?"yes":"no", -1);
      }

      GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
      set_selection_mode(list, GTK_SELECTION_SINGLE);
      g_object_set_data(G_OBJECT(dlg), "selection", selection);
      (void) g_signal_connect (G_OBJECT (selection), "changed",
                      G_CALLBACK (scelta_riga_spg), NULL);

      GtkWidget *button = myimage_button("List", "output.xpm");
      gtk_dialog_add_action_widget (GTK_DIALOG (dlg), button, 0);
#ifdef GTK_TOOLTIP
      gtk_widget_set_tooltip_text( button, "List of systematically absent reflections");
#else
      gtk_tooltips_set_tip (tooltips, button, "List of systematically absent reflections", NULL);
#endif
      g_signal_connect(G_OBJECT(button),"clicked",G_CALLBACK(c_getabsref),dlg);
      gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
         GTK_BUTTONBOX_END);
      gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(gtk_widget_get_parent(button)), button, TRUE);

      button = OK_button();
      gtk_dialog_add_action_widget (GTK_DIALOG (dlg), button, 1);
      g_signal_connect(G_OBJECT(button),"clicked",G_CALLBACK(end_dlg),dlg);
      button = CANCEL_button();
      gtk_dialog_add_action_widget (GTK_DIALOG (dlg), button, 2);
      g_signal_connect(G_OBJECT(button),"clicked",G_CALLBACK(end_dlg),dlg);
      g_signal_connect_swapped(G_OBJECT(dlg), "delete_event",
        G_CALLBACK(gtk_button_clicked), G_OBJECT(button));
      g_signal_connect (G_OBJECT (dlg), "delete_event",
                      G_CALLBACK(true_cb), NULL);
      gtk_widget_show_all(dlg);
      g_object_set_data(G_OBJECT(dlg), "response", GINT_TO_POINTER(0));
      gtk_main();
      int response = get_int(dlg, "response");

      *kselect = 0;
      if(response == 1)
         *kselect = row_selected(selection) + 1;
      gtk_widget_destroy(dlg);
}

}
void  c_spgdet (gpointer item, gpointer data)
{
     spg_determine_();
}

void c_spglist (gpointer item, gpointer data)
{
     spg_list_();
}

