MODULE plotstyle

  implicit none

  type style_type
       integer         :: plot_type  ! linee, punti, linee+punti
       integer         :: colline    ! colore delle linee
       integer         :: colmarker  ! colore dei marker
       integer         :: linetype   ! tipo della linea
       integer         :: linewidth  ! tipo della linea
       integer         :: markertype ! tipo di marker
       integer         :: sizemarker ! dimensione del marker
       integer         :: font_size  ! dimensione del carattere
       integer         :: vis        ! visibile? 0 non visibile, 1 visibile, -1 disabilitato
  end type style_type

!
! plot_type puo' assumere i seguenti valori
  integer, parameter   :: LINEPLOT = 1      ! solo linee
  integer, parameter   :: SCATTERPLOT = 2   ! solo punti
  integer, parameter   :: LINESCATTER = 3   ! linee+punti

  integer, parameter   :: NITEMS = 13

  type(style_type), dimension(NITEMS) :: style 

  integer, parameter :: STYLE_OBS = 1
  integer, parameter :: STYLE_BACK = 2
  integer, parameter :: STYLE_CALC = 3
  integer, parameter :: STYLE_DIFF = 4
  integer, parameter :: STYLE_CUMUL = 5
  integer, parameter :: STYLE_REFL = 6
  integer, parameter :: STYLE_PEAKS = 7
  integer, parameter :: STYLE_AXIS = 8
  integer, parameter :: STYLE_BACKP = 9
  integer, parameter :: STYLE_INTERVALS = 10
  integer, parameter :: STYLE_BG = 11
  integer, parameter :: STYLE_LEGEND = 12
  integer, parameter :: STYLE_UND_PEAKS = 13

END MODULE plotstyle
!
MODULE view

   implicit none

   private

   public  :: vedinew,plottanew,zoom_next,individua_patt,zoomnew1
   public  :: set_plot_style, add_to_selection, reset_selection

   private :: asserefn 

   type :: peak_type
     real    :: xpos,ypos  ! posizione del picco
     integer :: col        ! colore del picco
   end type 

   type gr_phase_type
     integer, dimension(:,:), allocatable :: hkl
     real, dimension(:), allocatable      :: tth
   end type
   type :: pd_graph_type
     type(gr_phase_type), dimension(:), allocatable :: grph
   end type pd_graph_type

   logical, public :: kzoomn = .FALSE.
   integer, public :: xzleft,xzright
   real, private :: xzoom1,xzoom2

   real, private   :: yzmax
   real, private   :: maxvv
   real, private   :: ttminc,ttmaxc

   integer, public                                     :: pd_action 
   integer, public                                     :: pinioss_sav,pfinoss_sav
   integer, public                                     :: pinicalc_sav,pfincalc_sav
   integer, public                                     :: iAct_sav = 1
   integer, parameter                                  :: NMAXSEL = 3000
   integer, dimension(NMAXSEL)                         :: vselection
   integer                                             :: nselection

   CONTAINS

     subroutine vedinew(iAct,pinioss,pfinoss,pinicalc,pfincalc,lsetstyle)  !!!!stylev)
     USE PatternRef
     USE Conteggi, only : theta_int,back_int
     USE CommonExpo, only : backp,scala,nback
     USE Riflessi, only : TThet
     USE General, only : nrefl 
     USE Molcom, only : kscrx,kscry
     USE GrexpoMod, only : gr_init,gr_setbackground,gr_clear_all_area,scalex,dispx,iselview
     USE plotstyle
     USE variables, only: dataset
     USE datasetmod, only: ndataset
     USE arrayutil
!
     integer, intent(in), optional                 :: iAct
     integer, intent(in), optional                 :: pinioss,pfinoss   !limiti per l'osservato
     integer, intent(in), optional                 :: pinicalc,pfincalc !limiti per il calcolato
     logical, intent(in), optional                 :: lsetstyle         !se vero setta lo stile all'interno
     integer                                       :: pinicalc1,pfincalc1
     real, dimension(:,:), allocatable             :: backpur
     integer                                       :: i
     integer, dimension(:), allocatable :: inid,ifid
!
     if (iselview == 0) call gr_init(iselview,kscrx,kscry)
     call gr_setbackground(style(STYLE_BG)%colline)
!
     call gr_clear_all_area()
!     
     if (present(iAct)) then
         pd_action = iAct
         if (present(lsetstyle)) then
             if (lsetstyle) call set_plot_style(pd_action)
         else
             call set_plot_style(pd_action)
         endif
         if (pd_action /= iAct_sav) kzoomn = .false. ! rimuovo lo zoom al cambio di azione
     endif
     if (present(pinioss))then         
         xzleft = pinioss
         xzright = pfinoss
     endif
     if (present(pinicalc)) then
         pinicalc1 = pinicalc
         pfincalc1 = pfincalc
     else
         pinicalc1 = xzleft
         pfincalc1 = xzright
     endif
!     
          !write(0,*)'iACTS=',pd_action
     select case (pd_action)
       case (1)
         if (allocated(dataset(1)%yb) .and. dataset(1)%npoints_back() > 0) then
             call dataset(1)%get_points(backpur)
             call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),tthr=TThet,   &
                            ybac=dataset(1)%yb(pinicalc1:pfincalc1),bacp=backpur)
         else
             call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),tthr=TThet)
         endif

       case (2)
         !call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),tthr=TThet(:nrefl))
         call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),tthr=TThet)

       case (3)
         !call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),tthr=TThet(:nrefl),  &
         !ybac=back_int(pinicalc1:pfincalc1,1),bacp=backp(:nback,:))
         call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),tthr=TThet,  &
         ybac=back_int(pinicalc1:pfincalc1,1),bacp=backp(:nback,:))

       case (4)
         !call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),     & 
         !scala(jjj)*theta_int(pinicalc1:pfincalc1,3),TThet(:nrefl),back_int(pinicalc1:pfincalc1,1), &
         !bacp=backp(:nback,:))
         call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),     & 
         scala(jjj)*theta_int(pinicalc1:pfincalc1,3),TThet,back_int(pinicalc1:pfincalc1,1), &
         bacp=backp(:nback,:))

       case (5)
         call dataset(1)%get_points(backpur)
!corr         allocate(backpur(npointback,2))
!corr         do i=1,npointback
!corr            backpur(i,1) = pointb(i)%x
!corr            backpur(i,2) = pointb(i)%y
!corr         enddo
         if (nrefl == 0) then
             call plottanew(xzleft,xzright,pinicalc1,pfincalc1,dataset(1)%x0,dataset(1)%y,ybac=dataset(1)%yb(pinicalc1:pfincalc1), &
             bacp=backpur)
         else
             call plottanew(xzleft,xzright,pinicalc1,pfincalc1,dataset(1)%x0,dataset(1)%y,ybac=dataset(1)%yb(pinicalc1:pfincalc1), &
             tthr=TThet,bacp=backpur)
         endif

       case (6)
         if (nrefl == 0) then
             !style(STYLE_REFL)%vis = -1
             call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),      &
             scala(1)*theta_int(pinicalc1:pfincalc1,3),ybac=back_int(pinicalc1:pfincalc1,1),bacp=backp(:nback,:))
         else
             !call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),      &
             !scala(1)*theta_int(pinicalc1:pfincalc1,3),ybac=back_int(pinicalc1:pfincalc1,1),tthr=TThet(:nrefl),  & 
             !bacp=backp(:nback,:))
             call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),      &
             scala(1)*theta_int(pinicalc1:pfincalc1,3),ybac=back_int(pinicalc1:pfincalc1,1),tthr=TThet,  & 
             bacp=backp(:nback,:))
         endif
  
       case (7)
         !call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),      &
         !scala(1)*theta_int(pinicalc1:pfincalc1,3),ybac=back_int(pinicalc1:pfincalc1,1),tthr=TThet(:nrefl),  &   
         !bacp=backp(:nback,:))
         call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),      &
         scala(1)*theta_int(pinicalc1:pfincalc1,3),ybac=back_int(pinicalc1:pfincalc1,1),tthr=TThet,  &   
         bacp=backp(:nback,:))

       case (8)
         allocate(inid(ndataset(dataset)))
         allocate(ifid(ndataset(dataset)))
         if (kzoomn) then
             do i=1,ndataset(dataset)
                inid(i) = clocate(dataset(i)%x0,xzoom1)
                ifid(i) = clocate(dataset(i)%x0,xzoom2)         
             enddo
         else
             do i=1,ndataset(dataset)
                inid(i) = 1
                ifid(i) = size(dataset(i)%x)
             enddo
         endif
         call plottanew2(inid,ifid)   !!!!,pinicalc1,pfincalc1)

     end select
!
!    salva limiti per redraw
     pinioss_sav = xzleft
     pfinoss_sav = xzright
     pinicalc_sav = pinicalc1
     pfincalc_sav = pfincalc1
     iAct_sav = pd_action
!
!    passa al C scala e spostamento sulla x
     xscaleC(1) = scalex(iselview)
     xdispC(1) = dispx(iselview)     
!
!    mr 210908  disegna legenda!!!
     call disegna_display_legend()
!
     end subroutine vedinew

  !-----------------------------------------------------------------------------------------------

     subroutine plottanew(inic,ific,inic1,ific1,ttet,yoss,ycal,tthr,ybac,bacp)
     USE PatternRef
     USE Molcom
     USE GREXPOMOD
     USE CommonExpo, only : nnt,ncoun1,ncoun2  !!!!,iviewpeaks
     USE peak_mod
     USE plotstyle
                  USE ssmoothing
                  !USE math_util
!
     integer, intent(in) :: inic  ! conteggio iniziale per l'osservato
     integer, intent(in) :: ific  ! conteggio finale per l'osservato
     integer, intent(in) :: inic1 ! conteggio iniziale per background e calcolato
     integer, intent(in) :: ific1 ! conteggio finale per background e calcolato
     real, dimension(:), intent(in)             :: ttet  ! 2theta del conteggio
     real, dimension(:), intent(in)             :: yoss  ! conteggio osservato
     real, dimension(:), intent(in), optional   :: ycal  ! conteggio calcolato
     real, dimension(:), intent(in), allocatable, optional :: tthr  ! 2theta del riflesso 
     real, dimension(:), intent(in), optional   :: ybac  ! background
     real, dimension(:,:), intent(in), optional :: bacp  ! punti di background
     real, dimension(ific1-inic1+1)               :: diffc 
     real, dimension(ific1-inic1+1)             :: ycalb,ydiff
     real, dimension(4)                         :: maxview,minview
     real                                       :: maxv,minv,diff
     integer                                    :: nplot
     real                                       :: trdiff    !!!!!,trpeak
     integer                                    :: i,j,ninf,nsup
     real                                       :: minint
     integer                                    :: nbp
     real                                       :: maxdiff,mindiff
     real                                       :: adiff,ypeak
     integer                                    :: intr
     integer                                    :: ncalcc
     integer                                    :: jumpd 
     logical                                    :: drawref, drawcalc, drawdiff
!
     jumpd = 0 
     call gr_area(0.07, 0.99, 0.07, 0.99)   ! Seleziona area grafica
     ncalcc = ific1 - inic1 + 1
!
!    Calcola il massimo e minimo valore lungo y
     nplot = 1
     maxview(nplot) = MAXVAL(yoss(inic:ific))
     minview(nplot) = MINVAL(yoss(inic:ific))
     drawcalc = style(STYLE_CALC)%vis == 1 .and. present(ycal) .and. present(ybac)
     if (drawcalc) then
         nplot = nplot + 1
         ycalb = ycal(:ncalcc)+ybac(:ncalcc)
         maxview(nplot) = MAXVAL(ycalb)
         minview(nplot) = MINVAL(ycalb)
     endif
     if (style(STYLE_BACK)%vis == 1 .and. present(ybac)) then
         nplot = nplot + 1
         maxview(nplot) = MAXVAL(ybac)
         minview(nplot) = MINVAL(ybac)
     endif
         !write(0,*)'REFLp=',style(STYLE_REFL)%vis
     drawdiff = style(STYLE_DIFF)%vis == 1 .and. present(ycal) .and. present(ybac)
     if (drawdiff) then
         trdiff = -nint(0.07*MAXVAL(maxview(1:nplot)))  ! traslazione asse differenza
         nplot = nplot + 1
         ydiff = yoss(inic1:ific1)-ycal(:ncalcc)-ybac(:ncalcc)
         maxview(nplot)=MAXVAL(ydiff+trdiff)
         minview(nplot)=MINVAL(ydiff+trdiff)
     endif
     maxvv = MAXVAL(maxview(:nplot))
     if (kzoomn) then
         maxv = yzmax    ! sono in zoom
     else
         maxv = maxvv
     endif
     minv = MINVAL(minview(:nplot))
     if (minv > maxv) then
         jumpd = 1
         minv = maxv - 10
     endif
     diff= maxv - minv
     maxv = maxv !+ trpeak
!
     drawref = .false.
     if (style(STYLE_REFL)%vis == 1 .and. present(tthr)) then
         if (allocated(tthr)) then
             minv = minv - diff*0.04
             drawref = .true.
         endif
     endif
!
     if (style(STYLE_INTERVALS)%vis == 1) minv = minv - diff*0.03
     ttminc = ttet(inic)
     ttmaxc = ttet(ific)
     call gr_setwin(ttminc,ttmaxc,minv,maxv)
!
!    Disegna il profilo osservato
     if (style(STYLE_OBS)%vis == 1 .and. jumpd == 0) then
         select case(style(STYLE_OBS)%plot_type)
           case (LINEPLOT)
             call gr_setlinestyle(style(STYLE_OBS)%linetype,style(STYLE_OBS)%linewidth)
             call gr_setcolor(style(STYLE_OBS)%colline)
             call gr_drawline(ttet(inic:ific),yoss(inic:ific))
!!!view smooth
             !call gr_setcolor(style(STYLE_CALC)%colline)
             !call gr_drawline(ttet(inic:ific),100*savgol_smooth(yoss(inic:ific),15,15,3,9)+minv)
             !call gr_setcolor(style(STYLE_BACK)%colline)
             !call gr_drawline(ttet(inic:ific),derivative(yoss(inic:ific)))
!!!view smooth
           case (SCATTERPLOT)
             call gr_setsize(style(STYLE_OBS)%sizemarker)
             call gr_setcolor(style(STYLE_OBS)%colmarker)
             call gr_drawmarkers(ttet(inic:ific),yoss(inic:ific),style(STYLE_OBS)%markertype)
           case (LINESCATTER)
             call gr_setlinestyle(style(STYLE_OBS)%linetype,style(STYLE_OBS)%linewidth)
             call gr_setcolor(style(STYLE_OBS)%colline)
             call gr_drawline(ttet(inic:ific),yoss(inic:ific))
             call gr_setsize(style(STYLE_OBS)%sizemarker)
             call gr_setcolor(style(STYLE_OBS)%colmarker)
             call gr_drawmarkers(ttet(inic:ific),yoss(inic:ific),style(STYLE_OBS)%markertype)
         end select
     endif
!
!    Disegna il profilo calcolato
!corr     if (style(STYLE_CALC)%vis == 1 .and. jumpd == 0) then
     if (drawcalc .and. jumpd == 0) then
         call gr_setlinestyle(style(STYLE_CALC)%linetype,style(STYLE_CALC)%linewidth)
         call gr_setcolor(style(STYLE_CALC)%colline)
         call gr_drawline(ttet(inic1:ific1),ycalb)
     endif
!
!    Disegna il background
     if (style(STYLE_BACK)%vis == 1 .and. present(ybac) .and. jumpd == 0) then
         call gr_setlinestyle(style(STYLE_BACK)%linetype,style(STYLE_BACK)%linewidth)
         call gr_setcolor(style(STYLE_BACK)%colline)
         call gr_drawline(ttet(inic1:ific1),ybac(:ncalcc))
         if (style(STYLE_BACKP)%vis == 1 .and. present(bacp)) then    ! disegna i punti di background
             nbp = size(bacp,1)
             call gr_setsize(style(STYLE_BACKP)%sizemarker)
             call gr_setcolor(style(STYLE_BACKP)%colmarker)
             do i=1,nbp
                if (bacp(i,1) >= ttminc .and. bacp(i,1) <= ttmaxc) then
                    call gr_draw1marker(bacp(i,1),bacp(i,2),style(STYLE_BACKP)%markertype)
                endif
             enddo
         endif
     endif
!
!    Disegna differenza
!corr     if (style(STYLE_DIFF)%vis == 1 .and. jumpd == 0) then
     if (drawdiff .and. jumpd == 0) then
         call gr_setlinestyle(style(STYLE_DIFF)%linetype,style(STYLE_DIFF)%linewidth)
         call gr_setcolor(style(STYLE_DIFF)%colline)
         call gr_drawline(ttet(inic1:ific1),ydiff+trdiff)
!
!        disegna asse differenza
!corr         call gr_setcolor(cp(color_OBS+4))
         call gr_line(ttminc,trdiff,ttmaxc,trdiff)
     endif
!
!    calcola e disegna differrenza cumulativa
     if (style(STYLE_CUMUL)%vis == 1 .and. jumpd == 0) then
         call gr_setlinestyle(style(STYLE_CUMUL)%linetype,style(STYLE_CUMUL)%linewidth)
         call gr_setcolor(style(STYLE_CUMUL)%colline)
         diffc(1) = abs(ydiff(inic1))
         do i=inic1+1,ific1
            j = i-inic1+1
            diffc(j) = diffc(j-1) + abs(ydiff(i))
         enddo
!
!        riscala diffc tra 0 e il massimo
         maxdiff = diffc(j)
         mindiff = diffc(1)
         adiff = diffc(j) - diffc(1)
         diffc = diffc*(maxv)/adiff - maxv*diffc(1)/adiff
         call gr_drawline(ttet(inic1:ific1),diffc)
     endif
!
!    Disegna gli intervalli
     if (style(STYLE_INTERVALS)%vis == 1) then
         call gr_setlinestyle(style(STYLE_INTERVALS)%linetype)
         do i=1,nnt-1
            ninf = ncoun1(i)
            if (ninf > ific) cycle ! intervallo tutto a destra dell'area di zoom 
            nsup = ncoun2(i)
            if (nsup < inic) cycle ! intervallo tutto a sinistra dell'area di zoom
            ninf = max(ninf,inic)
            nsup = min(nsup,ific)
            call gr_setsize(6)
            if (mod(i,2) == 0) then
                call gr_setcolor(style(STYLE_INTERVALS)%colline)
                minint = minv + diff*0.05  
            else
                call gr_setcolor(style(STYLE_INTERVALS)%colmarker)
                minint = minv + diff*0.06  
            endif
            call gr_draw1marker(ttet(ninf),yoss(ninf),2)
            call gr_draw1marker(ttet(nsup),yoss(nsup),2)
            call gr_line(ttet(ninf),minint,ttet(nsup),minint)
         enddo
!corr         call gr_setlinestyle(1)
     endif
!
!    Disegna i marcatori di picco 
     if (style(STYLE_PEAKS)%vis == 1) then
         call gr_setcolor(style(STYLE_PEAKS)%colline)
         do i=1,numpeaks(pkind)
            if (pkind(i)%getx() >= ttminc .and. pkind(i)%getx() <= ttmaxc) then
                call gr_setlinestyle(style(STYLE_PEAKS)%linetype,style(STYLE_PEAKS)%linewidth)
                ypeak = peak_intensity(pkind(i)%getx(),ttet,yoss)
                call gr_line(pkind(i)%getx(),minv,pkind(i)%getx(),ypeak)
                if (style(STYLE_UND_PEAKS)%vis == 1 .and. pkind(i)%info == 1) call gr_draw1marker(pkind(i)%getx(),ypeak+diff*0.01,4)
                if (pkind(i)%sel == SELECT_PEAK) then
                    call gr_setlinestyle(3)
                    call gr_line(pkind(i)%getx(),ypeak+diff*0.01,pkind(i)%getx(),ypeak+diff*0.04)
                endif
            endif
         enddo
         call gr_setlinestyle(1)
     endif
!
!    Disegna i riflessi
     if (drawref) then
         call gr_setlinestyle(style(STYLE_REFL)%linetype,style(STYLE_REFL)%linewidth)
         call gr_setcolor(style(STYLE_REFL)%colline)
         call asserefn(tthr,diff*0.03,minv)
     endif
!
!    kprofile =  1 --> calcola e disegna profili 
     if (kprofile == 1) then
         call profrefnew(ttet(inic),ttet(ific))  
     endif
!
!    Copri con un rettangolo la zona esterna agli assi (in zoom)
     if (kzoomn) then
         if (maxvv > yzmax) then
             call gr_setcolor(gr_colbackg)
             call gr_sfs(1)
             call gr_setcolor(gr_colbackg)
             call gr_rect(0.0,ttmaxc,maxv,maxvv)
             call gr_sfs(0)
         endif
     endif
!
!    Assi e label
     if (style(STYLE_AXIS)%vis == 1) then
         call gr_setcolor(style(STYLE_AXIS)%colline)
         call gr_setlinestyle(style(STYLE_AXIS)%linetype,style(STYLE_AXIS)%linewidth)
         call gr_box('ABILTS',0.0,0,'ABILTS',0.0,0)
     endif
!
!    Disegna barre degli intervalli-alle fine perch chiama la redrawpattern
     if (kruler == 1) then
         intr = iact_interval + 1
         call disegna_rulers(theta_left(intr),theta_right(intr))
     endif     
!
     call zflush()
!
     end subroutine plottanew

  !-----------------------------------------------------------------------------------------------

     subroutine plottanew2(inic,ific)   !!!!,inic1,ific1)
     USE PatternRef
     USE Molcom
     USE GREXPOMOD
     USE CommonExpo, only : nnt,ncoun1,ncoun2  !!!!,iviewpeaks
     USE peak_mod
     USE plotstyle
     USE variables, only: cryst,dataset
     USE crystal_phase
     USE datasetmod
     USE arrayutil
                  USE ssmoothing
                  !USE math_util
!
     integer, intent(in), dimension(:)          :: inic  ! conteggio iniziale per l'osservato
     integer, intent(in), dimension(:)          :: ific  ! conteggio finale per l'osservato
!corr     integer, intent(in) :: inic1 ! conteggio iniziale per background e calcolato
!corr     integer, intent(in) :: ific1 ! conteggio finale per background e calcolato
!corr     real, dimension(ific1-inic1+1)               :: diffc 
!corr     real, dimension(ific1-inic1+1)             :: ycalb,ydiff
     integer :: inic1,ific1
     real, dimension(:), allocatable :: ycalb,ydiff,diffc
     real                         :: maxview,minview
     real                                       :: maxv,minv,diff
     real                                       :: trdiff    !!!!!,trpeak
     integer                                    :: i,j
     real                                       :: maxdiff,mindiff
     real                                       :: adiff,ypeak
     integer                                    :: intr
     integer                                    :: ncalc
     integer                                    :: jumpd 
     logical :: drawref, drawcalc, drawdiff
     real :: space_ref,ypos_ref
     integer :: nset_ref
     integer :: ids
!
     jumpd = 0 
     call gr_area(0.07, 0.99, 0.07, 0.99)   ! Seleziona area grafica
!!!TODO for multipattern
!
!    Compute maximum and minimum along y
     maxview = tiny(0.0)
     minview = huge(0.0)
     do ids=1,ndataset(dataset)
        maxview = max(maxview,MAXVAL(dataset(ids)%y(inic(ids):ific(ids))))
        minview = min(minview,MINVAL(dataset(ids)%y(inic(ids):ific(ids))))
        drawcalc = style(STYLE_CALC)%vis == 1 .and. allocated(dataset(ids)%yb)  &
                   .and. allocated(dataset(ids)%yc)
        inic1 = max(dataset(ids)%nc1,inic(ids))
        ific1 = min(dataset(ids)%nc2,ific(ids))
        ncalc = ific1 - inic1 + 1
        if (drawcalc) then
            call new_array(ycalb,ncalc)
            ycalb = dataset(ids)%yc(inic1:ific1)+dataset(ids)%yb(inic1:ific1)
            maxview = max(maxview,MAXVAL(ycalb))
            minview = min(minview,MINVAL(ycalb))
        endif
        if (style(STYLE_BACK)%vis == 1 .and. allocated(dataset(ids)%yb) ) then
            maxview = max(maxview,MAXVAL(dataset(ids)%yb))
            minview = min(minview,MINVAL(dataset(ids)%yb))
        endif
        drawdiff = style(STYLE_DIFF)%vis == 1 .and. allocated(dataset(ids)%yb)  &
                   .and. allocated(dataset(ids)%yc)
        if (drawdiff) then
            call new_array(ydiff,ncalc)
            trdiff = -nint(0.07*maxview)  ! traslazione asse differenza
            if (drawcalc) then
                ydiff = dataset(ids)%y(inic1:ific1)-ycalb
            else
                ydiff = dataset(ids)%y(inic1:ific1)-dataset(ids)%yc(inic1:ific1)-dataset(ids)%yb(inic1:ific1)
            endif
            maxview = max(maxview,MAXVAL(ydiff+trdiff))
            minview = min(minview,MINVAL(ydiff+trdiff))
        endif
     enddo
     maxvv = maxview
     if (kzoomn) then
         maxv = yzmax    ! sono in zoom
     else
         maxv = maxvv
     endif
     minv = minview
     if (minv > maxv) then
         jumpd = 1
         minv = maxv - 10
     endif
     diff= maxv - minv
     maxv = maxv !+ trpeak
!
     drawref = .false.
     if (style(STYLE_REFL)%vis == 1) then
         space_ref = diff*0.04
         nset_ref = count(cryst%numref() > 0)
         if (nset_ref > 0) then
             minv = minv - nset_ref * space_ref
             drawref = .true.
         endif
     endif
!
!    Compute minimum and maximum along x
     ttminc = huge(0.0)
     ttmaxc = tiny(0.0)
     do ids=1,ndataset(dataset)
        ttminc = min(ttminc,dataset(ids)%x0(inic(ids)))
        ttmaxc = max(ttmaxc,dataset(ids)%x0(ific(ids)))
     enddo
     call gr_setwin(ttminc,ttmaxc,minv,maxv)
!
!    Disegna il profilo osservato
     do ids=1,ndataset(dataset)
        if (style(STYLE_OBS)%vis == 1 .and. jumpd == 0) then
            select case(style(STYLE_OBS)%plot_type)
              case (LINEPLOT)
                call gr_setlinestyle(style(STYLE_OBS)%linetype,style(STYLE_OBS)%linewidth)
                call gr_setcolor(style(STYLE_OBS)%colline+ids-1)
                call gr_drawline(dataset(ids)%x0(inic(ids):ific(ids)),dataset(ids)%y(inic(ids):ific(ids)))
              case (SCATTERPLOT)
                call gr_setsize(style(STYLE_OBS)%sizemarker)
                call gr_setcolor(style(STYLE_OBS)%colmarker)
                call gr_drawmarkers(dataset(ids)%x0(inic(ids):ific(ids)),  &
                                    dataset(ids)%y(inic(ids):ific(ids)),style(STYLE_OBS)%markertype)
              case (LINESCATTER)
                call gr_setlinestyle(style(STYLE_OBS)%linetype,style(STYLE_OBS)%linewidth)
                call gr_setcolor(style(STYLE_OBS)%colline)
                call gr_drawline(dataset(ids)%x0(inic(ids):ific(ids)),dataset(ids)%y(inic(ids):ific(ids)))
                call gr_setsize(style(STYLE_OBS)%sizemarker)
                call gr_setcolor(style(STYLE_OBS)%colmarker)
                call gr_drawmarkers(dataset(ids)%x0(inic(ids):ific(ids)),  &
                                    dataset(ids)%y(inic(ids):ific(ids)),style(STYLE_OBS)%markertype)
            end select
        endif
     enddo
!
!    Disegna il profilo calcolato
!corr     if (style(STYLE_CALC)%vis == 1 .and. jumpd == 0) then
     if (drawcalc .and. jumpd == 0) then
         call gr_setlinestyle(style(STYLE_CALC)%linetype,style(STYLE_CALC)%linewidth)
         call gr_setcolor(style(STYLE_CALC)%colline)
         call gr_drawline(dataset(1)%x0(inic1:ific1),ycalb)
     endif
!
!    Disegna il background
!corr     if (style(STYLE_BACK)%vis == 1 .and. present(ybac) .and. jumpd == 0) then
     if (style(STYLE_BACK)%vis == 1 .and. jumpd == 0 .and. allocated(dataset(1)%yb)) then
         call gr_setlinestyle(style(STYLE_BACK)%linetype,style(STYLE_BACK)%linewidth)
         call gr_setcolor(style(STYLE_BACK)%colline)
         call gr_drawline(dataset(1)%x0(inic1:ific1),dataset(1)%yb(inic1:ific1))
!corr         if (style(STYLE_BACKP)%vis == 1 .and. present(bacp)) then    ! disegna i punti di background
         if (style(STYLE_BACKP)%vis == 1 .and. dataset(1)%npoints_back() > 0) then
             !nbp = size(bacp,1)
             call gr_setsize(style(STYLE_BACKP)%sizemarker)
             call gr_setcolor(style(STYLE_BACKP)%colmarker)
             do i=1,dataset(1)%npoints_back()
                !if (bacp(i,1) >= ttminc .and. bacp(i,1) <= ttmaxc) then
                if (dataset(1)%points(i)%x >= ttminc .and. dataset(1)%points(i)%x <= ttmaxc) then
                    !call gr_draw1marker(bacp(i,1),bacp(i,2),style(STYLE_BACKP)%markertype)
                    call gr_draw1marker(dataset(1)%points(i)%x,dataset(1)%points(i)%y,style(STYLE_BACKP)%markertype)
                endif
             enddo
         endif
     endif
!
!    Disegna differenza
!corr     if (style(STYLE_DIFF)%vis == 1 .and. jumpd == 0) then
     if (drawdiff .and. jumpd == 0) then
         call gr_setlinestyle(style(STYLE_DIFF)%linetype,style(STYLE_DIFF)%linewidth)
         call gr_setcolor(style(STYLE_DIFF)%colline)
         call gr_drawline(dataset(1)%x0(inic1:ific1),ydiff+trdiff)
!
!        disegna asse differenza
!corr         call gr_setcolor(cp(color_OBS+4))
         call gr_line(ttminc,trdiff,ttmaxc,trdiff)
     endif
!
!    calcola e disegna differrenza cumulativa
     if (style(STYLE_CUMUL)%vis == 1 .and. jumpd == 0) then
         call new_array(diffc,ncalc)
         call gr_setlinestyle(style(STYLE_CUMUL)%linetype,style(STYLE_CUMUL)%linewidth)
         call gr_setcolor(style(STYLE_CUMUL)%colline)
         diffc(1) = abs(ydiff(inic1))
         do i=inic1+1,ific1
            j = i-inic1+1
            diffc(j) = diffc(j-1) + abs(ydiff(i))
         enddo
!
!        riscala diffc tra 0 e il massimo
         maxdiff = diffc(j)
         mindiff = diffc(1)
         adiff = diffc(j) - diffc(1)
         diffc = diffc*(maxv)/adiff - maxv*diffc(1)/adiff
         call gr_drawline(dataset(1)%x0(inic1:ific1),diffc)
     endif
!
!    Disegna i marcatori di picco 
     if (style(STYLE_PEAKS)%vis == 1) then
         call gr_setcolor(style(STYLE_PEAKS)%colline)
         do i=1,numpeaks(pkind)
            if (pkind(i)%getx() >= ttminc .and. pkind(i)%getx() <= ttmaxc) then
                call gr_setlinestyle(1,style(STYLE_PEAKS)%linewidth)
                ypeak = peak_intensity(pkind(i)%getx(),dataset(1)%x0,dataset(1)%y)
                call gr_line(pkind(i)%getx(),minv,pkind(i)%getx(),ypeak)
                if (pkind(i)%sel == SELECT_PEAK) then
                    call gr_setlinestyle(3)
                    call gr_line(pkind(i)%getx(),ypeak+diff*0.01,pkind(i)%getx(),ypeak+diff*0.04)
                endif
            endif
         enddo
         call gr_setlinestyle(1)
     endif
!
!    Disegna i riflessi
     if (drawref) then
         call gr_setlinestyle(style(STYLE_REFL)%linetype,style(STYLE_REFL)%linewidth)
         call gr_setcolor(style(STYLE_REFL)%colline)
         do i=1,numphase(cryst)
            if (cryst(i)%numref() > 0) then
                ypos_ref = minv + (numphase(cryst)-i)*space_ref
                if (numphase(cryst) > 1) call gr_drawtext(ttminc,ypos_ref,0.,cryst(i)%cr_name,2)
                call asserefn(cryst(i)%ref%tthd(1),0.75*space_ref,ypos_ref)
            endif
         enddo
     endif
!
!    kprofile =  1 --> calcola e disegna profili 
     if (kprofile == 1) then
         call profrefnew(dataset(1)%x0(inic(1)),dataset(1)%x0(ific(1)))  
     endif
!
!    Copri con un rettangolo la zona esterna agli assi (in zoom)
     if (kzoomn) then
         if (maxvv > yzmax) then
             call gr_setcolor(gr_colbackg)
             call gr_sfs(1)
             call gr_setcolor(gr_colbackg)
             call gr_rect(0.0,ttmaxc,maxv,maxvv)
             call gr_sfs(0)
         endif
     endif
!
!    Assi e label
     if (style(STYLE_AXIS)%vis == 1) then
         call gr_setcolor(style(STYLE_AXIS)%colline)
         call gr_setlinestyle(style(STYLE_AXIS)%linetype,style(STYLE_AXIS)%linewidth)
         call gr_box('ABILTS',0.0,0,'ABILTS',0.0,0)
     endif
!
!    Disegna barre degli intervalli-alle fine perch chiama la redrawpattern
     if (kruler == 1) then
         intr = iact_interval + 1
         call disegna_rulers(theta_left(intr),theta_right(intr))
     endif     
!
     call zflush()
!
     end subroutine plottanew2

  !-----------------------------------------------------------------------------------------------

     subroutine set_plot_style(kaction)
     USE plotstyle
     integer, intent(in) :: kaction
!
     select case (kaction)
       case (1)
         style(STYLE_OBS)%vis = 1
         style(STYLE_BACK)%vis = 0
         style(STYLE_BACKP)%vis = 0
         style(STYLE_CALC)%vis = -1
         style(STYLE_DIFF)%vis = -1
         style(STYLE_CUMUL)%vis = -1
         style(STYLE_REFL)%vis = 1
         style(STYLE_PEAKS)%vis = 0  !!!!!-1
         style(STYLE_UND_PEAKS)%vis = 0  !!!!!-1
         style(STYLE_INTERVALS)%vis = -1

       case (2)
         style(STYLE_OBS)%vis = 1
         style(STYLE_BACK)%vis = -1
         style(STYLE_BACKP)%vis = -1
         style(STYLE_CALC)%vis = -1
         style(STYLE_DIFF)%vis = -1
         style(STYLE_CUMUL)%vis = -1
         style(STYLE_REFL)%vis = 1
         style(STYLE_PEAKS)%vis = 0  !!!!!-1
         style(STYLE_UND_PEAKS)%vis = 0  !!!!!-1
         style(STYLE_INTERVALS)%vis = 1

       case (3)
         style(STYLE_OBS)%vis = 1
         style(STYLE_BACK)%vis = 1
         style(STYLE_BACKP)%vis = 1
         style(STYLE_CALC)%vis = -1
         style(STYLE_DIFF)%vis = -1
         style(STYLE_CUMUL)%vis = -1
         style(STYLE_REFL)%vis = 1
         style(STYLE_PEAKS)%vis = 0  !!!!!-1
         style(STYLE_UND_PEAKS)%vis = 0  !!!!!-1
         style(STYLE_INTERVALS)%vis = 1

       case (4)
         style(STYLE_OBS)%vis = 1
         style(STYLE_BACK)%vis = 1
         style(STYLE_BACKP)%vis = 0
         style(STYLE_CALC)%vis = 1
         style(STYLE_DIFF)%vis = 1
         style(STYLE_CUMUL)%vis = 0
         style(STYLE_REFL)%vis = 1
         style(STYLE_PEAKS)%vis = 0  !!!!!-1
         style(STYLE_UND_PEAKS)%vis = 0  !!!!!-1
         style(STYLE_INTERVALS)%vis = -1

       case (5)      ! background per peak-search
         style(STYLE_OBS)%vis = 1
         style(STYLE_BACK)%vis = 1
         style(STYLE_BACKP)%vis = 1
         style(STYLE_CALC)%vis = -1
         style(STYLE_DIFF)%vis = -1
         style(STYLE_CUMUL)%vis = -1
         style(STYLE_REFL)%vis = 1
         style(STYLE_PEAKS)%vis = 1
         style(STYLE_UND_PEAKS)%vis = 0  !!!!!-1
         style(STYLE_INTERVALS)%vis = -1

       case (6)
         style(STYLE_OBS)%vis = 1
         style(STYLE_BACK)%vis = 1
         style(STYLE_BACKP)%vis = 0
         style(STYLE_CALC)%vis = 1
         style(STYLE_DIFF)%vis = 1
         style(STYLE_CUMUL)%vis = 0
         style(STYLE_REFL)%vis = 1
         style(STYLE_PEAKS)%vis = 1
         style(STYLE_UND_PEAKS)%vis = 0
         style(STYLE_INTERVALS)%vis = -1
  
       case (7)
         style(STYLE_OBS)%vis = 1
         style(STYLE_BACK)%vis = 1
         style(STYLE_BACKP)%vis = 0
         style(STYLE_CALC)%vis = 1
         style(STYLE_DIFF)%vis = 1
         style(STYLE_CUMUL)%vis = 0
         style(STYLE_REFL)%vis = 1
         style(STYLE_PEAKS)%vis = 0 
         style(STYLE_UND_PEAKS)%vis = 0
         style(STYLE_INTERVALS)%vis = -1

       case (8)    ! Rietveld
         style(STYLE_OBS)%vis = 1
         style(STYLE_BACK)%vis = 1
         style(STYLE_BACKP)%vis = 0
         style(STYLE_CALC)%vis = 1
         style(STYLE_DIFF)%vis = 1
         style(STYLE_CUMUL)%vis = 0
         style(STYLE_REFL)%vis = 1
         style(STYLE_PEAKS)%vis = 0 
         style(STYLE_UND_PEAKS)%vis = 0
         style(STYLE_INTERVALS)%vis = -1

     end select
     if (style(STYLE_PEAKS)%vis == 1) then
         call wmenu_set_button('Hide Peaks',0)
     else
         call wmenu_set_button('Hide Peaks',1)
     endif
!
     end subroutine set_plot_style

  !-----------------------------------------------------------------------------------------------

     subroutine asserefn(tthr,lungr,yini)
     USE GREXPOMOD
     USE SPACECORR, only:vethkl
     USE Molcom
     USE arrayutil
     USE PatternRef, only:pos_absref
     USE plotstyle
!
     real, dimension(:), intent(in) :: tthr
     real, intent(in)               :: lungr  ! lunghezza delle tacche dei riflessi
     real, intent(in)               :: yini   ! posizione y dalla quale disegnare le tacche
     integer                        :: minref,maxref
     integer, dimension(1)          :: posref
     integer                        :: i
     real                           :: yyini
     integer                        :: jsel
!
     posref = MINLOC(tthr,MASK=tthr>=ttminc)
     minref = posref(1)
     if (minref > 0) then
         posref = MAXLOC(tthr(minref:),mask=tthr(minref:)<=ttmaxc)
         maxref = minref+posref(1)-1
!
         do i=minref,maxref
            call gr_line(tthr(i),yini,tthr(i),yini+lungr)
         enddo
!
         if (nselection > 0) then
             yyini = yini !+ lungr + 0.20*lungr
             call gr_setsize(6)
             do i=1,nselection
                jsel = vselection(i)
                if (jsel >= minref .and. jsel <= maxref) then
                    call gr_setcolor(1-style(STYLE_BG)%colline)
                    call gr_draw1marker(tthr(jsel),yyini+lungr/2,1)
                    call gr_draw1marker(tthr(jsel),yyini+lungr/2,3)
                endif
             enddo
         endif
!     
!        Disegna tacche supplementari durante la ricerca del gruppo spaziale
         if (allocated(vethkl)) then
             yyini = yini + lungr + 0.20*lungr
             do i=minref,min(maxref,size(vethkl))
             !do i=minref,maxref
                if (vethkl(i) == 1 .or. vethkl(i) > 2) then
                    call gr_setcolor(1-style(STYLE_BG)%colline)
                    call gr_line (tthr(i),yyini,tthr(i),yyini+lungr)
                elseif (vethkl(i) == 2) then
                    call gr_setcolor(color_LIGHTMAGENTA)
                    call gr_line (tthr(i),yyini,tthr(i),yyini+lungr)
                endif
                if (pos_absref > 0) then
                    call gr_setcolor(1-style(STYLE_BG)%colline)
                    call gr_draw1marker(tthr(pos_absref),yyini+lungr/2,1)
                    call gr_draw1marker(tthr(pos_absref),yyini+lungr/2,3)
                endif
             enddo        
         endif
     endif
!
     end subroutine asserefn  
     
  !-----------------------------------------------------------------------------------------------

     subroutine zoomnew1(ixp,iyp,iw)
     USE GREXPOMOD
     USE Conteggi, only : theta_int,npunti
     USE patternref, only : kprofile,iriet
     USE arrayutil
     USE variables, only: dataset
!
     integer, intent(in) :: ixp,iyp
     integer, intent(in) :: iw
     real                :: x1,x2
     real                :: y2
!     
!    kprofile = -1 --> lo zoom e' stato richiamato da 'Profile'
!    kprofile =  1 --> segnala alla plottaN di calcolare e disegnare i profili
     if (kprofile == -1) then
         kprofile = 1
     endif
!
!    Converti in 2-theta gli estremi     
     x1 = (ixp - dispx(iselview)) / scalex(iselview)                            
     x2 = x1 + iw/scalex(iselview)
     xzoom1 = x1
     xzoom2 = x2
!
!    Prendi i puntatori agli estremi dell'area     
     if (iriet == 1) then
         xzleft = clocate(dataset(1)%x0,x1)
         xzright = clocate(dataset(1)%x0,x2)         
     else
         xzleft = clocate(theta_int(:npunti,7),x1)
         xzright = clocate(theta_int(:npunti,7),x2)        
     endif
!    
     if (xzleft /= xzright) then
         kzoomn = .TRUE.
         y2 = (iyp - dispy(iselview)) / scaley(iselview)         ! calcola il massimo conteggio
         yzmax = y2
         call vedinew()
     else                                                        ! zoom fuori area -> disegna area vuota
         call gr_clear_all_area()
         kzoomn = .FALSE.
         call zflush()
     endif
!
     end subroutine zoomnew1

  !-----------------------------------------------------------------------------------------------

     subroutine zoom_next(icod)
     USE GREXPOMOD
     USE Conteggi, only:npunti
!
     integer, intent(in) :: icod
     integer             :: diffz
     real                :: diffz1
!
     if (kzoomn) then

         diffz = nint((xzright - xzleft) / 5.0)
         diffz1 = (xzoom2 - xzoom1) / 5.0

         select case(icod)

             case (1)                       ! zoom left
               xzoom1 = xzoom1 - diffz1
               xzoom2 = xzoom2 - diffz1
               if (xzleft == 1) return
               xzleft = xzleft - diffz
               xzright = xzright - diffz

             case (2)                       ! zoom right
               xzoom1 = xzoom1 + diffz1
               xzoom2 = xzoom2 + diffz1
               if (xzright == npunti) return
               xzright = xzright + diffz
               xzleft = xzleft + diffz

             case (3)                       ! zoom high
               if (yzmax > maxvv) return
               yzmax = yzmax + yzmax / 5.0

             case (4)                       ! zoom low
               yzmax = yzmax - yzmax / 5.0

             case (5)                       ! autoscale
               yzmax = maxvv

         end select

         xzleft = max(1,xzleft)
         xzright = min(npunti,xzright)
         call vedinew()
     endif
!
     end subroutine zoom_next

  !-----------------------------------------------------------------------------------------------

     integer function individua_patt(ixp,iyp)   result(individua)
     USE GREXPOMOD, only : dispx,scalex,dispy,scaley,iselview
     USE Molcom, only : iQualeBecca,button_ev
     USE CommonExpo,only:nback,backp
     USE peak_mod
     USE PatternRef, only : iriet
     USE arrayutil
     USE messagemod
     USE counts, only:dvalue
     USE Conteggi, only:theta_int
     USE proginterface
     USE plotstyle
     USE variables, only: dataset
!
!    indiviua = 0 (-1) : disegna/non disegna croce
!
     integer, intent(in) :: ixp,iyp
     real                :: tth,xpix,ypix
     integer             :: ira 
     integer             :: i
     real                :: thp
     real                :: xc,yc
     integer             :: iok
     integer             :: ipmin,ixpmin
     integer             :: update_background
     type(peak_type)     :: padd
     real                :: pkx,pky,fwp
     integer             :: ierfw
!corr     integer, dimension(2) :: xp
!
     individua = -1
!
     tth = (ixp - dispx(iselview)) / scalex(iselview)        ! converti in 2-theta
!!     
!!    Controllo per selezioni fuori dall'area del plot
!     if(tth < ttminc .or. tth > ttmaxc) return
!
     ira = 5
!
     select case (iQualeBecca)

         case (41)                            ! Add/Del peak
!     
!          Controllo per selezioni fuori dall'area del plot
           if(tth < ttminc .or. tth > ttmaxc) return

           if (button_ev == 1) then      ! Aggiungo picco con tasto sinistro
               thp = (ixp - dispx(iselview)) / scalex(iselview)
               call check_peak(pkind,thp,xc,yc,iok) 
               if (iok == 1) then
!
!                  Set background is not available
                   if (.not.allocated(dataset(1)%yb)) then
                       call fillcounts()
                       call back_for_peaksearch(.false.)
                   endif
!
!                  Compute fwhm
                   call peak_gaussian_fit(theta_int(:,1),theta_int(:,2),clocate(theta_int(:,1),xc),4,pkx,pky,fwp,ierfw)
                   if (ierfw /= 0) fwp = peak_fwhm(pkind,xc)
!
                   padd = peak_type(fwhm=fwp)
                   call padd%sety(yc)
                   call padd%setxd(xc)
                   call padd%get_int(theta_int(:,1),theta_int(:,2),dataset(1)%yb(:))
!
                   call padd%integrated_intensity(theta_int(:,1),theta_int(:,2),dataset(1)%yb)
!
!                  add peak to list
                   call peak_add(pkind,padd,INSERT_PEAK)
                       !write(0,*)'IS_PEAK=',is_peak_position(pkindtot,padd%getx())
                   if (numpeaks(pkindtot) > 0) then
                       if (.not.is_peak_position(pkindtot,padd%getx()))  &
                           call peak_add(pkindtot,padd,INSERT_PEAK)  ! add to array pkindtot if absent
                   else
                       call peak_add(pkindtot,padd,INSERT_PEAK)
                   endif
!
                   style(STYLE_PEAKS)%vis = 1  ! force visualization
                   call vedinew()
                   call write_message('Number of peaks: ',inum=numpeaks(pkind),pos=2)
                   individua = 0
               endif

           elseif (button_ev == 3) then   ! Rimuovo picco con tasto destro

               if (numpeaks(pkind) == 0) return
!
!              Localizza il picco pi vicino alla posizione cliccata
               ipmin = clocate(pkind(:)%getx()*scalex(iselview) + dispx(iselview),real(ixp))
!
!              Valuta se il picco e' sufficientemente vicino
               ixpmin = nint(pkind(ipmin)%getx()*scalex(iselview) + dispx(iselview))  ! converti in pixel 
               if (abs(ixpmin - ixp) < ira) then
!
!                  Rimuovi il picco
                   call peak_delete(pkindtot,pkind(ipmin)%getx())
                   call peak_delete(pkind,ipmin)
                   call vedinew()
                   call write_message('Number of peaks: ',inum=numpeaks(pkind),pos=2)
                   individua = 0
               endif
           endif

         case (42)                            ! Del point to background
           individua = -1
           do i=1,nback
              xpix = backp(i,1)*scalex(iselview) + dispx(iselview)  
              ypix = backp(i,2)*scaley(iselview) + dispy(iselview)  
              if (abs(xpix-ixp) < ira .and. abs(ypix-iyp) < ira) then
                  call Beep(100,100)
                  individua = 0
                  exit 
              endif 
           enddo

         case (43)                            ! Add/Del point to background
           select case (pd_action)
              case (2,3,4)   ! extra
                  individua = update_background_extra(ixp,iyp)
              case default
                  individua = update_background(ixp,iyp)
           end select

     end select
!
     end function individua_patt

! ----------------------------------------------------------------------

     subroutine check_peak(peaks,tt,xc,yc,iok)
!
     USE Conteggi
     USE arrayutil
     USE peak_util
!
     type(peak_type), dimension(:), allocatable, intent(in) :: peaks
     real, intent(in)     :: tt
     real, intent(out)    :: xc,yc
     integer, intent(out) :: iok
     integer              :: jmin
     integer              :: iniz,ifin
     real                 :: yjmin
     real                 :: yjmaxr,yjmaxl
     integer              :: jmaxr,jmaxl
     integer              :: i
     integer              :: iadd
     logical              :: is_peak
!
     iok = 1
     xc = tt
     iadd = 3
     jmin = clocate(theta_int(:,1),tt)   ! localizza conteggio piu' vicino
     if (numpeaks(peaks) == 0) then
         yc = theta_int(jmin,2)
         return
     endif
!
!    Controlla subito se nella posizione cliccata non esiste gia' un picco
     !is_peak = any(abs(xmax(:npk) - theta_int(jmin,1)) < 1.e-07)
!corr     is_peak = any(abs(peaks%x - theta_int(jmin,1)) < 1.e-07)
     is_peak = is_peak_position(peaks,theta_int(jmin,1))
!
     if (jmin > 0 .and. is_peak .eqv. .false.) then
         iniz = jmin - iadd
         if (iniz.lt.1) iniz = 1
         ifin = jmin + iadd
         if (ifin.gt.npunti) ifin = npunti
         if (iniz.gt.ifin) iniz = ifin
         yjmin = theta_int(jmin,2)
!
!        Cerca il massimo + vicino a destra
         jmaxr = 0
         yjmaxr = yjmin
         do i=jmin+1,ifin
            if (theta_int(i,2) > yjmaxr) then
                yjmaxr = theta_int(i,2)
                jmaxr = i
            elseif (theta_int(i,2) < yjmaxr .and. jmaxr > 0) then
                exit
            endif
         enddo
         if (jmaxr == ifin) jmaxr = 0  ! il massimo non puo' coincidere col limite
         if (jmaxr > 0) then
         !if (any(abs(xmax(:npk) - theta_int(jmaxr,1)) < 1.e-07))jmaxr=-1  ! annulla il punto se esiste gia'
!corr         if (any(abs(peaks%x - theta_int(jmaxr,1)) < 1.e-07))jmaxr=-1  ! annulla il punto se esiste gia'
         if(is_peak_position(peaks,theta_int(jmaxr,1)))jmaxr=-1   ! annulla il punto se esiste gia'
         !if (jmaxr > 0)write(0,*)'massimo a destra=',jmaxr,theta_int(jmaxr,1),yjmaxr
         endif
!
!        Cerca il massimo + vicino a sinistra
         jmaxl = 0
         yjmaxl = yjmin
         do i=jmin-1,iniz,-1
            if (theta_int(i,2) > yjmaxl) then
                yjmaxl = theta_int(i,2)
                jmaxl = i
            elseif (theta_int(i,2) < yjmaxl .and. jmaxl > 0) then
                exit
            endif
         enddo
         if (jmaxl == iniz) jmaxl = 0  ! il massimo non puo' coincidere col limite
         if (jmaxl > 0) then
         !if (any(abs(peaks%x - theta_int(jmaxl,1)) < 1.e-07))jmaxl=-1  ! annulla il punto se esiste gia'
         if(is_peak_position(peaks,theta_int(jmaxl,1)))jmaxl=-1  ! annulla il punto se esiste gia'
         !if (any(abs(xmax(:npk) - theta_int(jmaxl,1)) < 1.e-07))jmaxl=-1  ! annulla il punto se esiste gia'
         !if (jmaxl > 0)write(0,*)'massimo a sinistra=',jmaxl,theta_int(jmaxl,1),yjmaxl
         endif
!
         if ((jmaxl + jmaxr == 0) .or. (jmaxl + jmaxr == -1)) then       ! non trova nessun massimo
             yc = yjmin                              ! accetta selezione dell'utente
             xc = theta_int(jmin,1)
         elseif (jmaxl > 0 .and. jmaxr <= 0) then    ! massimo a sinistra
             yc = yjmaxl
             xc = theta_int(jmaxl,1)
         elseif (jmaxr > 0 .and. jmaxl <= 0) then    ! massimo a destra
             yc = yjmaxr
             xc = theta_int(jmaxr,1)
         elseif (jmaxr > 0 .and. jmaxl > 0) then     ! massimo a destra e sinistra: prendi quello pi vicino
             if (jmaxr - jmin  > jmin - jmaxl) then
                 yc = yjmaxl               ! a sinistra e' piu' vicino
                 xc = theta_int(jmaxl,1)
             else
                 yc = yjmaxr               ! piu' vicino a destra o uguali
                 xc = theta_int(jmaxr,1)
             endif
         elseif (jmaxr < 0 .and. jmaxl < 0) then     ! il picco trovato esiste gia'
             !write(0,*)"picco annullato perche' esiste gia'"
             iok = 0
         endif
         !write(0,*)'scelgo massimo=',yc,xc,jmaxl,jmaxr,jmin
      else
         iok = 0
      endif

      end subroutine check_peak

  !-----------------------------------------------------------------------------------------------
                                                                                                                  
     integer function update_background_extra(xpixel,ypixel) result(ier)
     USE Molcom
     USE PatternRef, only:jjj
     USE Commonexpo, only:nnt,ntyp,ncoun1,ncoun2,nback,backp
     USE grexpomod
     USE conteggi
     integer, intent(in) :: xpixel,ypixel
     integer             :: iix2,iiy2
     integer             :: ira
     integer             :: i,j
     integer             :: jjjsav
!
     jjjsav = jjj  
     if (button_ev == 1) then      ! aggiungo punti col tasto sinistro
         ier = 0
         nback=nback+1
         backp(nback,1) = (xpixel-dispx(iselview)) / scalex(iselview)
         backp(nback,2) = (ypixel-dispy(iselview)) / scaley(iselview)
!
!        localizza l'intervallo jjj a cui appartiene il punto aggiunto
         do i=2,nnt-1
            if (backp(nback,1) >= theta_int(ncoun1(i),7) .and.    &
                backp(nback,1) <= theta_int(ncoun2(i),7)) then
                jjj = i
            endif
         enddo
         ntyp(nback)=jjj
         call CalcoBacExt(1)

     elseif (button_ev == 3) then  ! elimino punti col tasto destro
         ira=5
         ier = -1
         do i=1, nback
            iix2 = nint(backp(i,1)*scalex(iselview) + dispx(iselview))
            iiy2 = nint(backp(i,2)*scaley(iselview) + dispy(iselview))
            if(iabs(xpixel-iix2) < ira .and. iabs(ypixel-iiy2) < ira) then
               ier = 0
               backp(i:nback-1,:) = backp(i+1:,:)
               ntyp(i:nback-1) = ntyp(i+1:)
               nback = nback - 1
!
!              localizza l'intervallo jjj a cui appartiene il punto da eliminare
               do j=2,nnt-1
                  if (backp(i,1) >= theta_int(ncoun1(j),7) .and.    &
                      backp(i,1) <= theta_int(ncoun2(j),7)) then
                      jjj = j
                  endif
               enddo
               exit
            end if
         enddo
         call CalcoBacExt(1)
     endif
     jjj = jjjsav
     call vedinew()
!
     end function update_background_extra

  !-----------------------------------------------------------------------------------------------

     subroutine reset_selection()
!
     nselection = 0
!
     end subroutine reset_selection

  !-----------------------------------------------------------------------------------------------

     subroutine add_to_selection(vet)
     integer, dimension(:), intent(in) :: vet
     integer                           :: nsel
     integer                           :: i
!
     nsel = nselection
     do i=1,size(vet)
        nsel = nsel + i
        if (nsel > NMAXSEL) nsel = 1
        vselection(nsel) = vet(i)
     enddo
     nselection = nsel
!
     end subroutine add_to_selection

   END MODULE view

  !----------------------------------------------------------------------------------------------

   subroutine set_style_pattern(code,styleg)
   USE plotstyle        !!!!!!, only:NITEMS, style_type, style
   USE view
   implicit none
   integer, intent(in)                                :: code
   type(style_type), dimension(NITEMS), intent(inout) :: styleg
   type(style_type), dimension(NITEMS), save          :: style_sav
!
   select case (code)
      case (-1)  ! passa stile al C
        styleg(:) = style(:)

      case (0)   ! Salva style all'apertura 
        style_sav(:) = style(:)

      case (1)   ! Apply
        style(:) = styleg(:)
        call vedinew()

      case (2)   ! OK
        style(:) = styleg(:)
        call vedinew()

      case (3)   ! Cancel
        style(:) = style_sav(:)
        call vedinew()

   end select
!
   end subroutine set_style_pattern
