#include <utils.h>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <sir_application.h>
//#include <gtkdefs.h>
#include "glext.h"
#include "models.h"
#include "jav_cb.h"
#ifdef GDK_WINDOWING_X11
#include <gdk/gdkx.h>
#endif

float *cross_prod(float *x1, const float *x2, const float *x3);

#define color_helix NUM_COLORS-6
#define color_sheet NUM_COLORS-5
#define color_turn  NUM_COLORS-4
#define color_coils NUM_COLORS-3

   typedef struct {
                    int vertex;
                    int faces[4];
                  } _VertVett;
namespace main_app {
       extern Application JavApp;
                   }

static atom_color icolorSave[NUM_COLORS];

static const char *ps_string[] = {"Wires",
                           "Simple",
                           "Bonds",
                           "Rods",
                           "Stick and Balls",
                           "Thermal Ellipsoids",
                           "Van der Waals",
                           "Backbone",
                           "Chain trace",
                           "Ribbons",
                           "Strands",
                           "Cartoons"};
static const char *cm_string[] = {"by atom species",
                           "by temperature (B fact.)",
                           "by s.o.f.",
                           "by symmetry operator",
                           "monochromatic",
                           "by residue",
                           "by chain",
                           "by secondary structure",
                           "by position along chain"};

extern int altSaveN;

namespace GLextmr {
    Model *ModelInfo; //[5];
    extern GtkWidget *GLWINDOW;
    extern GLuint font_list_base;
    extern int defaultPlotStyle;
    extern int defaultColor;
    extern int defaultSymmetry;
    extern int defaultLabels;
    extern bool  moving;
    extern bool  in_selection;
    extern bool  PreviewMode;
    extern GLfloat sc_factor;
    extern GLint   npixels;
    extern float H[4][NKNOTS-2];
    extern int   nknots;
    extern GLuint  iTexture[8];
    bool noSecondary = false;
    bool IsProline(const char *resname);
    bool DetectSecondary(Model *md, std::string &msg);
    extern Vert SelectionRect[4];
#ifdef JAV
    extern bool ifDebug;
#else
    bool ifDebug = FALSE;
#endif
    void sync_radio(const char *menuname, bool Flag);
    void aggiornaMenuSym();
    RESIDUE GetResiduo(int whichModel, int whichChain, int whichResidue);
    void setMinMax(vet3 mini, vet3 maxi, float rad, float step, int k);

extern "C" void  applicasimmetria_(int *which);
void CApplicaSimmetria(int *which)
{
#ifdef MAPS
    if(ModelInfo[*which-1].ifSymmetry)
       SettaSurface(*which-1, 0);
#endif
#ifdef JAV
     SetActive(*which-1);
#endif
     applicasimmetria_(which);
}
   
static float CalcolaInten(vet3 vv)
{
    float m[4][4];
    mat3x3 m2;
    glGetFloatv(GL_MODELVIEW_MATRIX, &m[0][0]);
    for(int i=0; i<3; i++)
        for (int j=0; j<3; j++)
             m2.m[3*i+j] = m[j][i];
    vet3 x2 = MRMatrix3MultiplyVector3(m2, vv);

    float size = (float) MRVectorLength(x2);
    return (MRmax(0.1f, (float) MRabs(x2.z)/size));
}

static float CalcolaInten(float v1, float v2, float v3)
{
    vet3 v = {{v1, v2, v3}};
    v = MRVectorNormalize(v);
    return CalcolaInten(v);
}

static void GetSizeandInten(Knot *knot)
{
    vet3 x1 = {{knot->vnx, knot->vny, knot->vnz}};
    vet3 x2 = {{knot->hnx, knot->hny, knot->hnz}};
    if( !knot->vsize )
        knot->vsize = sqrt( knot->vnx*knot->vnx +
                            knot->vny*knot->vny +
                            knot->vnz*knot->vnz );
    knot->vinten = MRmax(0.1f, MRabs(knot->vnz)/knot->vsize);
    knot->vinten = CalcolaInten(x1);

    if( !knot->hsize )
        knot->hsize = sqrt( knot->vnx*knot->hnx +
                            knot->vny*knot->hny +
                            knot->vnz*knot->hnz );
    knot->hinten = MRmax(0.1f, MRabs(knot->hnz)/knot->hsize);
    knot->hinten = CalcolaInten(x2);
}

static void DrawPolygon(Vert v[])
{
    glBegin(GL_POLYGON);
    for(int i=0; i<4; i++)
    {
        glColor4fv(v[i].c);
        glVertex3f(v[i].x, v[i].y, v[i].z);
    }
    glEnd();
}

void CalcolaH(int n, float Hdit[4][NKNOTS-2])
{
   float t;
   float d = 1.f/(float)(n-1);
   for(int i=1; i<n/2; i++)
   {
       t = (float) i * d;
       Hdit[0][i-1] = (1 + 2*t) * (1 - t) * (1 - t);
       Hdit[1][i-1] = t * (1 - t) * (1 - t);
       Hdit[2][i-1] = t * t * (3 - 2*t);
       Hdit[3][i-1] = t * t * (t - 1);
       t = (float) (i + n/2) * d;
       Hdit[0][i+n/2-1] = (1 + 2*t) * (1 - t) * (1 - t);
       Hdit[1][i+n/2-1] = t * (1 - t) * (1 - t);
       Hdit[2][i+n/2-1] = t * t * (3 - 2*t);
       Hdit[3][i+n/2-1] = t * t * (t - 1);
   }
   t = 0.5;
   Hdit[0][n/2-1] = (1 + 2*t) * (1 - t) * (1 - t);
   Hdit[1][n/2-1] = t * (1 - t) * (1 - t);
   Hdit[2][n/2-1] = t * t * (3 - 2*t);
   Hdit[3][n/2-1] = t * t * (t - 1);
//   for(int i=0; i<n-2; i++)
//       g_print("%d %f, %f, %f, %f\n", i+1,Hdit[0][i],Hdit[1][i], Hdit[2][i],Hdit[3][i]);
}

void calcolaH()
{
   nknots = NKNOTS;
   CalcolaH(nknots, H);
}

void BuildRect(Vert *v, float p1[], float p2[])
{
//   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
   v[0].x = p1[0];
   v[0].y = p1[1];
   v[1].x = p1[0];
   v[1].y = p2[1];
   v[2].x = p2[0];
   v[2].y = p2[1];
   v[3].x = p2[0];
   v[3].y = p1[1];
   float colors[4] = {1.f, 0.85f, 1.f, 0.65f};
   for(int i=0; i<4; i++)
   {
       v[i].z = 0.f;
       memcpy(v[i].c, colors, 4*sizeof(float));
   }
}

void DrawCylinder(double v, double raggio, int subdivisions,
                      int d1=1, int d2=1, int ordleg=1)
{
  double rc=raggio*0.7;

  float delt = 1.1f*(float)raggio;
  
  switch (ordleg)
  {
  default:
  case 1:
  case 4:
       disegnaCilindro(raggio, v, subdivisions);
       break;
  case 2:
  case 5:
       glTranslatef(0,-delt,0 );
       disegnaCilindro(rc, v, subdivisions);
       glTranslatef(0,2*delt,0 );
       disegnaCilindro(rc, v, subdivisions);
       break;
  case 3:
       glTranslatef(-delt, 0,0 );
       disegnaCilindro(rc, v, subdivisions);
       glTranslatef(delt,-1.73f*delt,0 );
       disegnaCilindro(rc, v, subdivisions);
       glTranslatef(delt,1.73f*delt,0 );
       disegnaCilindro(rc, v, subdivisions);
       break;
  }
  if(!d1)
     disegnaSfera(raggio, subdivisions);
  glTranslatef( 0,0,(float) v );
  if(!d2)
     disegnaSfera(raggio, subdivisions);
}

void render3DArrow(vet3 p1, vet3 p2, double raggio, int subdivisions)
{
  glPushMatrix();
  double v; 
  mat4x4 m = MRCylinderMatrix(&v, p1, p2, MRVectorZero);
  glTranslated( p1.x, p1.y, p1.z);
  glMultMatrixd (m.m);
  disegnaCilindro(raggio, v*0.65, subdivisions);
  glTranslatef( 0,0,(float) v*0.65f );
  disegnaCono(raggio*2.0, v*0.35, subdivisions);
  glPopMatrix();
}

void drawCylinder( vet3 p1, vet3 p2,
                   float raggio, float color1[]=NULL)
{
  float colore[4];
  float color2[] = {1.0f, 0.85f, 1.0f, 0.5f};

  if(!color1)
     memcpy(colore, color2, 4*sizeof(float));
  else
     memcpy(colore, color1, 4*sizeof(float));
  vet3 p1p2N = MRVectorSub(p1, p2);
  vet3 firstPerpV = MRVectorFirstPerp(p1p2N);
  vet3 secondPerpV = MRVectorCrossProduct(p1p2N, firstPerpV);
  secondPerpV = MRVectorNormalize(secondPerpV);

  const int ANZ = 32; 
  const float FULL = (float) (2.0f*M_PI);
  Vert points[ANZ+1];

  for (int i=0; i<ANZ; i++) 
  {
    float angle = FULL*((float) i/(float) ANZ);

    points[i].x = (float) (raggio*(cos(angle)*firstPerpV.x+sin(angle)*secondPerpV.x));
    points[i].y = (float) (raggio*(cos(angle)*firstPerpV.y+sin(angle)*secondPerpV.y));
    points[i].z = (float) (raggio*(cos(angle)*firstPerpV.z+sin(angle)*secondPerpV.z));
    memcpy(points[i].c, colore,  4*sizeof(float));
    if(color1 != NULL)
    {
       float intens = 0.9f*CalcolaInten(points[i].x, points[i].y, points[i].z);
       TuneHSV( points[i].c, intens);
    }
  }
    points[ANZ].x = points[0].x;
    points[ANZ].y = points[0].y;
    points[ANZ].z = points[0].z;
    memcpy(points[ANZ].c, points[0].c, 4*sizeof(float));

  glBegin(GL_TRIANGLE_FAN);
  glVertex3dv(p1.v);
  for (int i=0; i<=ANZ; i++)
  {
    glColor4fv(points[i].c);
    glVertex3f((float)p1.x+points[i].x,
               (float)p1.y+points[i].y,
               (float)p1.z+points[i].z);
  }
  glEnd();

  glBegin(GL_TRIANGLE_FAN);
  glVertex3dv(p2.v);
  for (int i=0; i<=ANZ; i++)
  {
    glColor4fv(points[i].c);
    glVertex3f((float)p2.x+points[i].x,
               (float)p2.y+points[i].y,
               (float)p2.z+points[i].z);
  }
  glEnd();

  glBegin(GL_QUAD_STRIP);
  for (int i=0; i<=ANZ; i++) {
    glColor4fv(points[i].c);
    glVertex3f((float)p1.x+points[i].x,
               (float)p1.y+points[i].y,
               (float)p1.z+points[i].z);
    glVertex3f((float)p2.x+points[i].x,
               (float)p2.y+points[i].y,
               (float)p2.z+points[i].z);
  }
  glEnd();      
}

void renderCylinder(GLWindow *gl, vet3 p1, vet3 p2,
                    double raggio, float color1[] = NULL,
                    int subdivisions=0, int d1=1, int d2=1)
{
    glDisable(GL_LIGHTING);
    glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
    glEnable(GL_COLOR_MATERIAL);
    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    drawCylinder(p1, p2, (float) raggio, color1);

    glPushMatrix();  

    glDisable(GL_COLOR_MATERIAL);
    glEnable(GL_LIGHTING);
    glEnable(GL_NORMALIZE);

    glTranslated( p1.x, p1.y, p1.z);
    if(!d1)
       disegnaSfera(raggio, subdivisions);
    glPopMatrix();  
    glPushMatrix();  
    glTranslated( p2.z, p2.z, p2.z);
    if(!d2)
       disegnaSfera(raggio, subdivisions);
    glPopMatrix();  
}
void renderSphere(float x, float y, float z, double raggio, int subdivisions,
                  bool iftexture = false)
{
      glPushMatrix();
      glTranslatef( x,y,z );
      disegnaSfera(raggio, subdivisions, iftexture);
      glPopMatrix();
}

void drawSphere(float xc, float yc, float zc, float rad, float color1[])
{
    glDisable(GL_LIGHTING);
    glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
    glEnable(GL_COLOR_MATERIAL);
    glEnable (GL_BLEND);
    glShadeModel(GL_SMOOTH);
    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL ) ;
    glColor4fv(color1);

    glPushMatrix();
    glTranslated(xc, yc, zc);
    int lats=10, longs=10;
    for(int i = 0; i <= lats; i++)
    {
        double lat0 = M_PI * (-0.5 + (double) (i - 1) / lats);
        double z0  =  rad * sin(lat0);
        double zr0 =  rad * cos(lat0);

        double lat1 = M_PI * (-0.5 + (double) i / lats);
        double z1 =  rad * sin(lat1);
        double zr1 = rad * cos(lat1);

        glBegin(GL_QUAD_STRIP);
        for(int j = 0; j <= longs; j++)
        {
            double lng = 2 * M_PI * (double) (j - 1) / longs;
            double x = cos(lng);
            double y = sin(lng);
            glNormal3d(x * zr0, y * zr0, z0);
            glVertex3d(x * zr0, y * zr0, z0);
            glNormal3d(x * zr1, y * zr1, z1);
            glVertex3d(x * zr1, y * zr1, z1);
        }
        glEnd();
    }
    glPopMatrix();
}

void drawCylinderWithCaps(double radius, double h, int slices,
      bool ifCaps=true)
{
    int  DEF_D=5;
    if(slices > 180)
       DEF_D = MRmax(1, (int)(900.f/(float)slices));

    disegnaCilindro(radius, h, slices);
    if(!ifCaps)
       return;

    // top and bottom circles 
    for (int j=0;j<=1; j++)
    {
      double i = (double)j*h;
      glBegin(GL_TRIANGLE_FAN);
      for (int k=0;k<=360;k+=DEF_D)
      {
        glNormal3d(radius*Cos(k),radius*Sin(k),i);
        glVertex3d(radius*Cos(k),radius*Sin(k),i);
      }
      glEnd();
    }
}

void renderCircs(float x, float y, float z, double radius, int subdivisions,
               float scala[], float mat[4][4])
{
      float he = 0.003f;
      double raggio = radius;

      if((radius / he) > 5)
          raggio = radius + he/2;
      else
          raggio = radius;

      glPushMatrix();
      glTranslatef( x,y,z);
      glMultMatrixf (&mat[0][0]);
      glScalef(scala[0], scala[1], 1);
      glRotatef (90.0, 0.0, 0.0, 1.0);
      drawCylinderWithCaps(raggio, 2*he, subdivisions);
      glPopMatrix();

      glPushMatrix();
      glTranslatef( x,y,z);
      glMultMatrixf (&mat[0][0]);
      glScalef(1, scala[1], scala[2]);
      glRotatef (90.0, 0.0, 1.0, 0.0);
      drawCylinderWithCaps(raggio, 2*he, subdivisions);
      glPopMatrix();

      glPushMatrix();
      glTranslatef( x,y,z);
      glMultMatrixf (&mat[0][0]);
      glScalef(scala[0], 1, scala[2]);
      glRotatef (90.0, 1.0, 0.0, 0.0);
      drawCylinderWithCaps(raggio, 2*he, subdivisions);
      glPopMatrix();
}

void renderEll(float x, float y, float z, double raggio, int subdivisions,
               float scala[], float mat[4][4], bool zero)
{
      GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
      glPushMatrix();
      glTranslatef( x,y,z );
      glMultMatrixf (&mat[0][0]);
      glScalef(scala[0], scala[1], scala[2]);
      disegnaSfera(raggio, subdivisions);
      glScalef(gl->Scale, gl->Scale, gl->Scale);
      glPopMatrix();
}

Model* Model::Clone() const
{
  return new Model(*this);
}

Model::Model()
{
      int status[] = {1, 0, 1, 0, 0, 0, 0, -1, 1, 0, 0, 0, 0, 
                      1, 1, 1, 1};
      float fstatus[] = {0.1f, 0.1f, 0.1f};
      if(defaultPlotStyle >= 0)
         status[3] = defaultPlotStyle;
      if(defaultSymmetry >= 0)
         status[1] = defaultSymmetry;
      if(defaultLabels >= 0)
         status[5] = defaultLabels;
      if(defaultColor >= 0)
         status[4] = defaultColor;
      created = false;
      setValues(status, fstatus);
}

Model::Model(int status[], float fstatus[], statusSav vec)
{
      setValues(status, fstatus);

      for(int i=0; i<NUM_COLORS; i++)
          memcpy(light_diffuse[i], vec.light_diffuse[i], 4*sizeof(float));
      memcpy(labelcolor, vec.labelcolor, 4*sizeof(float));
      memcpy(light_chain, vec.light_chain, 4*sizeof(float));
      memcpy(LabelS, vec.LabelS, N_ELEMENTS*sizeof(int));

      created = true;
}

bool Model::IsType(int which, const char *type)
{
     std::string str = AtomTypes.at(which);
     size_t found = str.find("*");
     if(found != std::string::npos)
        str.at(found) = 0x27;   // '
     return (strcmp(str.c_str(), type) == 0);
}

int Model::FindAtom(const char *type, RESIDUE rr, int tipo)
{
     for(int i=0; i<(int)rr.atoms.size(); i++)
     {
         if(IsType(rr.atoms[i], type))
         {
            if(tipo == 0)
               return rr.atoms[i];
            else if(atm[rr.atoms[i]].Z == tipo)
               return rr.atoms[i];
         }
     }
     return -1;
}


void Model::InitColors()
{
      for(int i=0; i<NUM_COLORS; i++)
      {
          float *colori = main_app::JavApp.getfCol(i+N_OBJS);
          memcpy(light_diffuse[i], colori, 4*sizeof(float));
          g_free(colori);
      }
}

void Model::setValues(int status[], float fstatus[])
{
      InitColors();
      freeVectors(false);
      freeStructs(false);
      nucleic = 0;
      isPDB = false;
      nats = 0;
      nconn = 0;
      nconn_prot = 0;
      nconn_bb = 0;
      nconn_lig = 0;
      nconn_mix = 0;
      nconn_nuc = 0;
      nconn_ca = 0;
      nangbonds = 0;
      ntorsions = 0;
      nlegs_au = 0;
      nats_au = 0;
      nResAmino = 0;
      numSpec = 0;
      hasSecondary = false;
      nPoly = 0;
      nPolyStyles = 0;
      Polyhedra = NULL;
      PolyStyle = NULL;
      nlegsH = 0;
      nlegsW = 0;
      legsH = NULL;
      legsW = NULL;
      ifResize = true;
      Bmin = Bmax = Bmin_ca = Bmax_ca = 0;
      RF = 0;
      ifVis = (status[0] == 1);
      ifSymmetry = (status[1] == 1);
      ifDrawH = (status[2] == 1);
      plotStyle  = status[3];
      colorMode = status[4];
      ifLabels = (status[5] != 0);
      ifChangedSymmetry = (status[6] != 0);
      drawLigand = status[7];
      ifPoly = (status[9] != 0);
      ifDrawW = (status[10] == 1);
      specialLab = status[11];
      labelType = status[12];
      klevel[0] = fstatus[0];
      klevel[1] = fstatus[1];
      klevel[2] = fstatus[2];
      FitOpt = status[16];
      for(int i=0; i<N_ELEMENTS; i++)
          LabelS[i] = 0;
      for(int i=0; i<4; i++)
          numbStructs[i] = 0;
      for(int i=0; i<3; i++)
          baricentro[i] = 0;
      for (int i=0; i< MAX_SPECIES; i++)
      {
           Species[i] = 0;
           Content[i] = 0;
           ptab[i] = 0;
           radii[i][0] = 0;
           radii[i][1] = 0;
      }
      for(int i=0; i<NHDistances; i++)
         distHn[i] = 0;

      switch(main_app::JavApp.Background)
      {
      case TYPE_BLACK:      //Black background
         labelcolor[0] = 0.85f;//1.0f;
         labelcolor[1] = 0.85f;//1.0f;
         labelcolor[2] = 0.85f;//0.0f;
         break;
      case TYPE_WHITE:      //White background
         labelcolor[0] = 0.0f;
         labelcolor[1] = 0.0f;
         labelcolor[2] = 0.7f;
         break;
      default:      //Other
         labelcolor[0] = 1.0f;
         labelcolor[1] = 1.0f;
         labelcolor[2] = 1.0f;
         break;
      }
      labelcolor[3] = 1.0f;
      light_chain[0] = 0.7f;
      light_chain[1] = 0.7f;
      light_chain[2] = 0.7f;
      light_chain[3] = 1.0f;
      shininess = 0.8f;

      nResTot = 0;
      ifSurface = false;
      Surface_c = false;
}

void Model::freeStructs(bool tofree)
{
  if(tofree)
  {
     if(!Chains.empty())
     {
        for(int i=0; i<(int) Chains.size(); i++)
        {
           if(!Chains[i].Residues.empty())
           {
              for(int j=0; j<(int) Chains[i].Residues.size(); j++)
                  Chains[i].Residues[j].atoms.clear();
              Chains[i].Residues.clear();
            }
           if(!Chains[i].AminoRes.empty())
           { 
              for(int j=0; j<(int) Chains[i].AminoRes.size(); j++)
                  Chains[i].AminoRes[j].atoms.clear();
              Chains[i].AminoRes.clear();
           }
        }
       Chains.clear();
     }
     if(!Conns.empty())
         for(int i=0; i<(int) Conns.size(); i++)
            Conns[i].clear();
  }
  c_alpha.clear();
  c_alphab.clear();
  nChains = 0;
  atoms_sel.clear();
  bonds_sel.clear();
  resid_sel.clear();
  phlist.clear();
  atom_types.clear();
  AtomTypes.clear();
  Conns.clear();
}

void Model::freeVectors(bool tofree)
{
   if(tofree)
   {
      if(!Equivs.empty())
      for(int i=0; i<(int) Equivs.size(); i++)
          Equivs[i].clear();
      Equivs.clear();
      if(atm != NULL)
         g_free(atm);
      if(atm_au != NULL)
         g_free(atm_au);
      if(modname)
         g_free(modname);
      if(iEqui)
         g_free(iEqui);
      if(Secondary)
         g_free(Secondary);
      g_free(bonds);
      if(bonds_ca)
         g_free(bonds_ca);
      if(bonds_bb)
         g_free(bonds_bb);
      if(bonds_nuc)
         g_free(bonds_nuc);
      if(bonds_lig)
         g_free(bonds_lig);
      if(bonds_mix)
         g_free(bonds_mix);
      if(anglebonds)
         g_free(anglebonds);
      if(torsions)
         g_free(torsions);
      if(Polyhedra)
         g_free(Polyhedra);
      if(PolyStyle)
         g_free(PolyStyle);
      if (legsH)
         g_free(legsH);
      if (legsW)
         g_free(legsW);
      if (NConns)
         g_free(NConns);
      if (NEquivs)
         g_free(NEquivs);
   }
   modname = NULL;
   atm = NULL;
   atm_au = NULL;
   iEqui = NULL;
   bonds = NULL;
   bonds_ca = NULL;
   bonds_bb = NULL;
   bonds_nuc = NULL;
   bonds_lig = NULL;
   bonds_mix = NULL;
   anglebonds = NULL;
   torsions = NULL;
   legsH = NULL;
   legsW = NULL;
   Polyhedra = NULL;
   PolyStyle = NULL;
   Secondary = NULL;
   NConns = NULL;
   NEquivs = NULL;
}

void Model::freeModel()
{
#ifdef MAPS
      if(Surface_c)
      {
         if(GLWINDOW)
         {
            GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
            gl->usedC[nModello] = -1;
         }
         freeSurface(nModello);
      }
#endif
      Surface_c = false;
      freeVectors(true);
      freeStructs(true);
      created = false;
      ifSymmetry = false;
      ifPoly = false;
}

void Model::GetEquivs()
{
   NEquivs = g_new0(int, nats_au);
   for(int nat=0; nat<nats_au; nat++)
   {
       std::vector<int> eq;
       NEquivs[nat] = 1;
       Equivs.push_back(eq);
//       Equivs[nat].push_back(nat);
//       for(int i=nats_au; i<nats; i++)
       for(int i=0; i<nats; i++)
       {
          if(iEqui[i] == nat)
             Equivs[nat].push_back(i);
       }
       NEquivs[nat] = (int) Equivs[nat].size();
   }
}

void Model::ToggleLabels(int flag)
{
     bool labels = (flag != 0);
     if(labels != ifLabels)
     {
        sync_radio("Label All Atoms", labels);
        ifLabels = labels;
     }
}

float Model::VolumePerAtom(float volume, int nsymt)
{
   if(nats == 0)
   {
      int nTot = 0;
      for(int i=0; i<numSpec; i++)
      {
          if(Species[i] != 1)
            nTot += Content[i];
      }
      if(nTot > 0)
         return (volume / (float) nTot);
   }
   else
   {
      float somma_occ = 0;
      for(int i=0; i<nats_au; i++)
      {
          if(!IsHydrogen(i))
             somma_occ = somma_occ + (atm[i].och*atm[i].ocry);
      }
      somma_occ = nsymt * somma_occ;
      if(somma_occ > 0)
         return (volume / somma_occ);
   }
   return 0;
}

const char *Model::NomeResiduo(int which)
{
     if(which < 20)
     { 
        char *amino = g_strdup(get_nth_amino(which));
        return lower_case(g_strstrip(amino));
     }
     return "XXXX";
}

void Model::UpdateSelectedAtom(int whichAtom, bool to_add)
{
     bool stato = atm[whichAtom].selected;
     atm[whichAtom].selected = !stato;
     if(stato && !to_add)
     {
        std::vector<int>::iterator it;
        it = std::find(atoms_sel.begin(), atoms_sel.end(), (whichAtom+1));
        if(it != atoms_sel.end())
          atoms_sel.erase(it);
     }
     else
        atoms_sel.push_back(whichAtom+1);
}

void Model::UpdateSelectedResidue(int whichAlpha, bool to_add)
{
     bool stato = c_alphab[whichAlpha].selected;
     c_alphab[whichAlpha].selected = !stato;
     if(stato && !to_add)
     {
        std::vector<int>::iterator it;
        it = std::find(resid_sel.begin(), resid_sel.end(), (whichAlpha+1));
        if(it != resid_sel.end())
          resid_sel.erase(it);
     }
     else
        resid_sel.push_back(whichAlpha+1);
}

void Model::UpdateSelectedBond(int whichBond, int whichtype)
{
     int wb = whichBond;
     bool stato;
     switch(whichtype)
     {
     default:
     case 1:
          stato = bonds[wb].selected;
          bonds[wb].selected = !stato;
          if(stato)
          {
             std::vector<int>::iterator it;
             it = std::find(bonds_sel.begin(), bonds_sel.end(), (wb+1));
             if(it != bonds_sel.end())
               bonds_sel.erase(it);
          }
          else
             bonds_sel.push_back(wb+1);
          break;
     case 2:
          break;
     case 3:
          break;
     case 4:
          break;
     }
}

std::vector<char *> Model::ListaAtomi(std::vector<int> atoms, GtkWidget *dlg)
{
//     GList *listAtoms = NULL;
     std::vector<char *> listAtoms;
     for(int k=0; k<(int) atoms.size(); k++)
     {
        std::stringstream os;
        std::ofstream x;
        int i = atoms[k];
        x  << std::fixed <<std::setw(8);
        x.setf(std::ios_base::left, std::ios_base::adjustfield);
        os.copyfmt(x);
//        if(IsProtein())
        if(isPDB)
           os << atom_types.at(i) << " (";
        else
           os << std::setw(16) << atm[i].lab << " (";
        os << std::setprecision (3) << std::setw(7) << std::right
           << atm[i].xyzc[0] << ","
           << std::setprecision (3) << std::setw(7) << std::right
           << atm[i].xyzc[1] << ","
           << std::setprecision (3) << std::setw(7) << std::right
           << atm[i].xyzc[2] << ") "
           << std::setprecision (3) << std::setw(7) << std::right
           << atm[i].ocry
           << std::setprecision (3) << std::setw(7) << std::right
           << U_or_B(atm[i].B, dlg);

//        g_print("%s\n", os.str().c_str());
//          listAtoms = g_list_append(listAtoms, DuplicaStringa(os.str().c_str()));
          listAtoms.push_back(DuplicaStringa(os.str().c_str()));
     }
     return listAtoms;
}

void Model::DrawSticks(GLWindow *gl)
{
     DrawSticks(gl, 0, nconn_prot, bonds, false, NULL);
}

int Model::SetColor(int whichAtom, float color1[])
{
     int index1, ieq;
     for(int i=0; i<3; i++)
         color1[i] = 0;
     switch(colorMode)
     {
     case BY_SPECIES:
           index1  = atm[whichAtom].Z-1;
           if((index1  < 0)||(index1 >= N_ELEMENTS))
              index1 = OTHERS;
           break;
     case BY_SYMMETRY:
           index1 = atm[whichAtom].op.op - 1;
           if(index1 >= N_AMINO)
              index1 = OTHERS;
           else
              index1 += N_ELEMENTS;
           break;
     case BY_RESIDUE:
           ieq = iEqui[whichAtom];
           index1 = atm_au[ieq].numResiduo;
           if(index1 >= (N_AMINO + N_DNA_NUCLEOTIDES))
              index1 = OTHERS;
           else
              index1 += N_ELEMENTS;// + N_CHAINS;
           break;
     case BY_CHAIN:
           ieq = iEqui[whichAtom];
           index1 = atm_au[ieq].catena;
           if(index1 >= N_CHAINS)
              index1 = OTHERS;
           else
              index1 += N_ELEMENTS+N_AMINO;
           break;
     case BY_STRUCT:
           ieq = iEqui[whichAtom];
           if(IsHelix(ieq))
              index1 = color_helix;
           else if(IsBetaSheet(ieq))
              index1 = color_sheet;
           else if(IsTurn(ieq))
              index1 = color_turn;
           else if(IsDna(ieq))
              index1 = is_valid_symbol("P") - 1;
           else
              index1 = color_coils;
           break;
     case MONOC:
           index1 = MONOCOL;
           break;
     default:
           index1 = OTHERS;
           break;
     }
     if(colorMode == BY_GROUP)
     {
         ieq = iEqui[whichAtom];
         int ch = atm_au[ieq].catseq;
         int nRestot = Chains[ch].nResidues;
         int r = nRestot - atm_au[ieq].seq;
         TuneH(nRestot, r, color1);
         color1[3] = 1.0f;
     }
     else if(colorMode == BY_B)
     {
         float b, b2;
         if(plotStyle == C_ALPHA)
         {
            b  = Bmax_ca - atm[whichAtom].B;
            b2 = Bmax_ca - Bmin_ca;
         }
         else
         {
            b  = Bmax - atm[whichAtom].B;
            b2 = Bmax - Bmin;
         }
         TuneH(b2, b, color1);
         color1[3] = 1.0f;
     }
     else if(colorMode == BY_SOF)
     {
         float r = 1.f - atm[whichAtom].ocry;
         TuneH(1.f, r, color1);
         color1[3] = 1.0f;
     }
     else
        memcpy(color1, light_diffuse[index1], 4*sizeof(float));
     return index1;
}

void Model::SetRungColor(int whichAtom, float color1[])
{
    int index1, ieq;
     for(int i=0; i<3; i++)
         color1[i] = 0;
     ieq = iEqui[whichAtom];
     index1 = atm_au[ieq].numResiduo;
     if(index1 >= (N_AMINO + N_DNA_NUCLEOTIDES))
         index1 = OTHERS;
     else  
         index1 += N_ELEMENTS + N_CHAINS;
     memcpy(color1, light_diffuse[index1], 4*sizeof(float));
}

void Model::DrawSticks(GLWindow *gl, int i0, int i1, jav_bond_type *legami,
                       bool iflig, float *colore)
{
      int i;
      float color1[4];
      float color2[4];

      glPushMatrix();
      glDisable (GL_LIGHTING);
      if(gl->smooth)
      {
         glEnable (GL_LINE_SMOOTH);
         glEnable (GL_BLEND);
         glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
         glHint (GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
      }

      for(i=0; i<nats; i++)
          atm[i].disegnato = !iflig && (plotStyle >= RODS);     //!= WIRES);
      if((plotStyle == BONDSTYLE) || (iflig && (drawLigand - 1) == BONDSTYLE))
         glLineWidth(5) ;
      else
         glLineWidth(1.5f) ;

      glBegin(GL_LINES);
      for(i=i0; i<i1;i++)
      {
         float p1[3], p2[3], p3[3];
         int j1 = MRabs(legami[i].n1) - 1;
         if(j1 >= nats)
            continue;
         if(j1 < 0)
            continue;
         if(atm[j1].deleted || atm[j1].hidden)
            continue;
         int j2 = MRabs(legami[i].n2) - 1;
         if(j2 >= nats)
            continue;
         if(atm[j2].deleted || atm[j2].hidden)
            continue;
         if( (!ifDrawH) && (IsHydrogen(j1) || IsHydrogen(j2)))
            continue;
         if( (!ifDrawW) && ( atm_au[iEqui[j1]].isW  || atm_au[iEqui[j2]].isW ) )
            continue;
         for(int j=0; j<3; j++)
         {
             p1[j] =  atm[j1].xyzN[j];
             p2[j] =  atm[j2].xyzN[j];
         }
         atm[j1].disegnato = 1;
         atm[j2].disegnato = 1;

         int index1=0, index2=0;
         if(colore == NULL)
         {
            index1 = SetColor(j1, color1);
            index2 = SetColor(j2, color2);
         }
         else
         {
            memcpy(color1, colore, 4*sizeof(float));
            memcpy(color2, colore, 4*sizeof(float));
         }
         if(gl->Stereo && (gl->BgType < TYPE_FLAT))
         {
            Desaturate(color1);
            Desaturate(color2);
         }
         glColor4fv(color1);
         glVertex3f(p1[0], p1[1], p1[2]);
         if(index1 != index2)
         {
            for(int j=0; j<3; j++)
                p3[j] =  (p1[j] + p2[j]) / 2.f;
            glVertex3f(p3[0], p3[1], p3[2]);
            glColor4fv(color2);
            glVertex3f(p3[0], p3[1], p3[2]);
         }
         glVertex3f(p2[0], p2[1], p2[2]);
      }
      glEnd();
      glLineWidth(1.5f) ;

      glEnable (GL_LIGHTING);
//      glDisable(GL_LINE_SMOOTH);
//      glDisable(GL_BLEND);
      glPopMatrix();
}

bool Model::CalcolaBaricentro(int i0, int i1, float bc[3])
{
  if(i1 == i0)
     return false;
  for(int j=0; j<3; j++)
  {
      for(int i=i0; i<i1; i++)
          bc[j] += atm[i].xyzN[j];
     bc[j] /= (i1-i0);
  }
  return true;
}

void Model::DrawTubes(GLWindow *gl, int npoints, Knot knot[], float color[], double factor)
{
     for(int i=0; i<npoints-1; i++)
     {
          vet3 x1 = {{ knot[i].px, knot[i].py, knot[i].pz}};
          vet3 x2 = {{ knot[i+1].px, knot[i+1].py, knot[i+1].pz}};
          renderCylinder(gl, x1, x2, factor*1.5*gl->cylinder,color);
     }
}

void Model::DrawSolidRibbon(GLWindow *gl, Knot src, Knot dst, float colore[])
{
    Vert v[4];
    
    for(int i=0; i<4; i++)
        memcpy(v[i].c, colore, 4*sizeof(float));
    TuneHSV( v[0].c, src.vinten);
    TuneHSV( v[1].c, dst.vinten);
    TuneHSV( v[2].c, dst.vinten);
    TuneHSV( v[3].c, src.vinten);
// Top Surface 
    v[0].x = src.px+src.wx+src.dx;
    v[0].y = src.py+src.wy+src.dy;
    v[0].z = src.pz+src.wz+src.dz;

    v[1].x = dst.px+dst.wx+dst.dx;
    v[1].y = dst.py+dst.wy+dst.dy;
    v[1].z = dst.pz+dst.wz+dst.dz;

    v[2].x = dst.px-dst.wx+dst.dx;
    v[2].y = dst.py-dst.wy+dst.dy;
    v[2].z = dst.pz-dst.wz+dst.dz;

    v[3].x = src.px-src.wx+src.dx;
    v[3].y = src.py-src.wy+src.dy;
    v[3].z = src.pz-src.wz+src.dz;
    DrawPolygon(v);

// Bottom Surface 
    v[0].x = src.px+src.wx-src.dx;
    v[0].y = src.py+src.wy-src.dy;
    v[0].z = src.pz+src.wz-src.dz;

    v[1].x = dst.px+dst.wx-dst.dx;
    v[1].y = dst.py+dst.wy-dst.dy;
    v[1].z = dst.pz+dst.wz-dst.dz;

    v[2].x = dst.px-dst.wx-dst.dx;
    v[2].y = dst.py-dst.wy-dst.dy;
    v[2].z = dst.pz-dst.wz-dst.dz;

    v[3].x = src.px-src.wx-src.dx;
    v[3].y = src.py-src.wy-src.dy;
    v[3].z = src.pz-src.wz-src.dz;
    DrawPolygon(v);

    for(int i=0; i<4; i++)
        memcpy(v[i].c, colore, 4*sizeof(float));
    TuneHSV( v[0].c, src.hinten);
    TuneHSV( v[1].c, dst.hinten);
    TuneHSV( v[2].c, dst.hinten);
    TuneHSV( v[3].c, src.hinten);

// Left Surface 
    v[0].x = src.px+src.wx+src.dx;
    v[0].y = src.py+src.wy+src.dy;
    v[0].z = src.pz+src.wz+src.dz;

    v[1].x = dst.px+dst.wx+dst.dx;
    v[1].y = dst.py+dst.wy+dst.dy;
    v[1].z = dst.pz+dst.wz+dst.dz;

    v[2].x = dst.px+dst.wx-dst.dx;
    v[2].y = dst.py+dst.wy-dst.dy;
    v[2].z = dst.pz+dst.wz-dst.dz;

    v[3].x = src.px+src.wx-src.dx;
    v[3].y = src.py+src.wy-src.dy;
    v[3].z = src.pz+src.wz-src.dz;
    DrawPolygon(v);

// Right Surface 
    v[0].x = src.px-src.wx+src.dx;
    v[0].y = src.py-src.wy+src.dy;
    v[0].z = src.pz-src.wz+src.dz;

    v[1].x = dst.px-dst.wx+dst.dx;
    v[1].y = dst.py-dst.wy+dst.dy;
    v[1].z = dst.pz-dst.wz+dst.dz;

    v[2].x = dst.px-dst.wx-dst.dx;
    v[2].y = dst.py-dst.wy-dst.dy;
    v[2].z = dst.pz-dst.wz-dst.dz;

    v[3].x = src.px-src.wx-src.dx;
    v[3].y = src.py-src.wy-src.dy;
    v[3].z = src.pz-src.wz-src.dz;
    DrawPolygon(v);

}

void Model::DrawStrands(GLWindow *gl, int npoints, Knot knot[], float color[])
{
   glColor4fv(color);
   float c[NKNOTS][5][3];

   for(int k=-1; k<4; k++)
   {
      glBegin(GL_LINE_STRIP);
      for(int i=0; i<npoints; i++)
      {
          if(k == -1)
          {
          c[i][k+1][0] = knot[i].px;
          c[i][k+1][1] = knot[i].py;
          c[i][k+1][2] = knot[i].pz;
          }
          else if(k == 0)
          {
          c[i][k+1][0] = knot[i].px - knot[i].wx;
          c[i][k+1][1] = knot[i].py - knot[i].wy;
          c[i][k+1][2] = knot[i].pz - knot[i].wz;
          }
          else if(k == 1)
          {
          c[i][k+1][0] = knot[i].px - knot[i].wx/2;
          c[i][k+1][1] = knot[i].py - knot[i].wy/2;
          c[i][k+1][2] = knot[i].pz - knot[i].wz/2;
          }
          else if(k == 2)
          {
          c[i][k+1][0] = knot[i].px + knot[i].wx/2;
          c[i][k+1][1] = knot[i].py + knot[i].wy/2;
          c[i][k+1][2] = knot[i].pz + knot[i].wz/2;
          }
          else
          {
          c[i][k+1][0] = knot[i].px + knot[i].wx;
          c[i][k+1][1] = knot[i].py + knot[i].wy;
          c[i][k+1][2] = knot[i].pz + knot[i].wz;
          }
          glVertex3f(c[i][k+1][0], c[i][k+1][1], c[i][k+1][2]);
      }
      glEnd();
   }
}

void Model::DrawFlatRibbon(GLWindow *gl, Knot src, Knot dst, float colore[])
{
    Vert v[4];
    
    for(int i=0; i<4; i++)
        memcpy(v[i].c, colore, 4*sizeof(float));
    TuneHSV( v[0].c, src.vinten);
    TuneHSV( v[1].c, dst.vinten);
    TuneHSV( v[2].c, dst.vinten);
    TuneHSV( v[3].c, src.vinten);

    v[0].x = src.px+src.wx;
    v[0].y = src.py+src.wy;
    v[0].z = src.pz+src.wz;

    v[1].x = dst.px+dst.wx;
    v[1].y = dst.py+dst.wy;
    v[1].z = dst.pz+dst.wz;

    v[2].x = dst.px-dst.wx;
    v[2].y = dst.py-dst.wy;
    v[2].z = dst.pz-dst.wz;
                        
    v[3].x = src.px-src.wx;  
    v[3].y = src.py-src.wy;  
    v[3].z = src.pz-src.wz;
                    
    DrawPolygon(v);
}

void Model::DrawScaletta(GLWindow *gl)
{
    float ca[3], canext[3];
    vet3 p1, p2, p3;
    float color[4];
    int d = (int)phlist.size();
    int lfin = 1;
    if(ifSymmetry)
       lfin = NEquivs[0];
    for(int l=0; l<lfin; l++)
    {
      for(int i=0; i<d-1; i+=2)
      {
        int at1 = Equivs[phlist[i]].at(l);
        int at2 = Equivs[phlist[i+1]].at(l);
        memcpy(ca,  atm[at1].xyzN, 3*sizeof(float));
        memcpy(canext,  atm[at2].xyzN, 3*sizeof(float));
#if 0
// boh?
        for(int kk=0; kk<(int)c_alphab.size(); kk++)
        {
            if(at1 == c_alphab[kk].atom)
               memcpy(ca, c_alphab[kk].xyz, 3*sizeof(float));
            else if(at2 == c_alphab[kk].atom)
               memcpy(canext, c_alphab[kk].xyz, 3*sizeof(float));
        }
#endif
        p1 = MRVectorMake(ca);
        p2 = MRVectorMake(canext);
        p3 = MRVectorMedium(p1, p2);
        SetRungColor(at1, color);
        renderCylinder(gl, p1, p3, 1.5*gl->cylinder,color);
        SetRungColor(at2, color);
        renderCylinder(gl, p3, p2, 1.5*gl->cylinder,color);
      }
    }
}

void Model::DrawRibbon(int kc,  Knot knot[], int npoints, bool selected)
{
    GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
    float color1[4];
//    float colors[4] = {1.f, 0.6f, 1.f, 0.85f};
    float colors[4] = {1.f, 0.85f, 1.f, 0.65f};
    int ca = c_alphab[kc].atom;

    glDisable(GL_LIGHTING);
    glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
    glEnable(GL_COLOR_MATERIAL);
    glEnable (GL_BLEND);
    (void) SetColor(ca, color1);
    if(selected && c_alphab[kc].selected)
    {
//       glBlendFunc (GL_SRC_ALPHA,  GL_SRC_ALPHA);
       memcpy(color1, colors, 4*sizeof(float));
       glPolygonMode( GL_FRONT_AND_BACK, GL_FILL ) ;
    }
    else if(selected)
       return;
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    if(gl->Stereo && (gl->BgType < TYPE_FLAT))
       Desaturate(color1);

    int plotStyleSav = plotStyle;
    if(selected)
       plotStyle = RIBBONS;

    if(plotStyle == RIBBONS)
    {
       for(int kk=0; kk<npoints-1; kk++)
//240112           DrawFlatRibbon(gl, knot[kk], knot[kk+1], color1);
           DrawSolidRibbon(gl, knot[kk], knot[kk+1], color1);
    }
    else if(plotStyle == STRANDS)
        DrawStrands(gl, npoints, knot, color1);
    else
    {
        if(IsBetaSheet(ca))
        {
            for(int kk=0; kk<npoints-1; kk++)
                DrawSolidRibbon(gl, knot[kk], knot[kk+1], color1);
        }
        else if(IsHelix(ca))
        {
           for(int kk=0; kk<npoints-1; kk++)
              DrawSolidRibbon(gl, knot[kk], knot[kk+1], color1);
        }
        else if(IsDna(ca))
           DrawTubes(gl, npoints, knot, color1, 3);
        else
           DrawTubes(gl, npoints, knot, color1);
    }
    plotStyle = plotStyleSav;
}

void Model::DrawRibbons(GLWindow *gl, bool selected, int kmodel)
{
    Knot knot[NKNOTS];
    float wide, hwide, size, kf = 1.5f;
    float ca[3], canext[3], o[3], o2[3];
    bool prev = false;
    bool ifArrow;
    if(kmodel != -1)
    {
       int nomep = 5;
       glPushName(nomep);
       glPushName(kmodel);
    }

    memset(knot, 0, nknots*sizeof(Knot));
    int chain = -1;
    for(int i=0; i<(int)c_alphab.size(); i++)
    {
        if(kmodel != -1)
           glPushName(i);
// Defining peptide planes
        if(!prev)
           memset(knot, 0, nknots*sizeof(Knot));

        if(c_alphab[i].chain != chain)
        {
           prev = false;
           chain = c_alphab[i].chain;
           memset(knot, 0, nknots*sizeof(Knot));
        }
        if(c_alphab[i].o == -1)
        {
           prev = false;
           if(kmodel != -1)
              glPopName();
           continue;
        }
 
        for(int j=0; j<3; j++)
        {
            ca[j] = atm[c_alphab[i].atom].xyzN[j];
            if(c_alphab[i].isAmino)
               o[j] = atm[c_alphab[i].o].xyzN[j];
            else if(c_alphab[i].n == -1) // deoxyribonucleic acid
            {
               if(!c_alphab[i].breakF)
               {
                  o[j] = atm[c_alphab[i+1].o].xyzN[j];
                  if(c_alphab[i].o2 != -1)
                     o2[j] = atm[c_alphab[i].o2].xyzN[j];
               }
               else
                  o[j] = atm[c_alphab[i].o].xyzN[j];
            }
            else                          // ribonucleic acid
               o[j] = atm[c_alphab[i].o].xyzN[j];
            if(c_alphab[i].breakF)
               continue;
            canext[j] = atm[c_alphab[i+1].atom].xyzN[j];
        }
        if(c_alphab[i].breakF)            // siamo alla fine
        {
           if( IsBetaSheet(c_alphab[i].atom) && (plotStyle == CARTOONS))
           {
               wide = (float)(12*gl->cylinder);
               knot[nknots-1].px = ca[0] + (knot[nknots-1].tx/2);
               knot[nknots-1].py = ca[1] + (knot[nknots-1].ty/2);
               knot[nknots-1].pz = ca[2] + (knot[nknots-1].tz/2);
               ifArrow = true;
           } 
           else
           {
               wide = (float)(6*gl->cylinder);
               if(selected)
                  wide = (float)(8*gl->cylinder);
               knot[nknots-1].px = ca[0];
               knot[nknots-1].py = ca[1];
               knot[nknots-1].pz = ca[2];
               ifArrow = false;
           }
           if( (knot[0].wide!=wide) && knot[0].hsize )
           {   
               size = knot[0].hsize;
               knot[0].wx = (wide*knot[0].hnx)/size;
               knot[0].wy = (wide*knot[0].hny)/size;
               knot[0].wz = (wide*knot[0].hnz)/size;

               if( !ifArrow )
               {   knot[nknots-1].wx = knot[0].wx;
                   knot[nknots-1].wy = knot[0].wy;
                   knot[nknots-1].wz = knot[0].wz;
               } 
               else
               {   
                   knot[nknots-1].wx = 0;
                   knot[nknots-1].wy = 0;
                   knot[nknots-1].wz = 0;
               }
            } 
            else if( ifArrow )
            {   
                wide = (float)(12*gl->cylinder);
                knot[nknots-1].wx = 0;
                knot[nknots-1].wy = 0;
                knot[nknots-1].wz = 0;
            }
            knot[1] = knot[nknots-1];
            if(!prev)
            {
            knot[0] = knot[1];
            knot[0].px = o[0];
            knot[0].py = o[1];
            knot[0].pz = o[2];
            }
            prev = false;
            DrawRibbon(i, knot, 2, selected);
            c_alphab[i].xyz[0] = knot[0].px;
            c_alphab[i].xyz[1] = knot[0].py;
            c_alphab[i].xyz[2] = knot[0].pz;
            if((IsDna(c_alphab[i].atom))  && (plotStyle == CARTOONS))
            {
               float color1[4];
               SetColor(c_alphab[i].atom, color1);
               TuneHSV( color1, knot[1].vinten);
               drawSphere(knot[1].px, knot[1].py, knot[1].pz, 4.5f*(float)gl->cylinder,
                         color1);
            }
            if(kmodel != -1)
               glPopName();
            continue;
        }
// Get vector A = Ca(i+l) - Ca(i)
        knot[nknots-1].tx = canext[0] - ca[0];
        knot[nknots-1].ty = canext[1] - ca[1];
        knot[nknots-1].tz = canext[2] - ca[2];
// Get vector B = O(i) - Ca(i) 
        float bx, by, bz;
        if(!c_alphab[i].isAmino && (c_alphab[i].o2 != -1) && (c_alphab[i].n == -1))
        {
           bx = (o[0]+o2[0])/2 - ca[0];
           by = (o[1]+o2[0])/2 - ca[1];
           bz = (o[2]+o2[0])/2 - ca[2];
        }
        else
        {
           bx = o[0] - ca[0];
           by = o[1] - ca[1];
           bz = o[2] - ca[2];
        }

// Form point P as the midpoint of Ca(i) and Ca(i+1)
        knot[nknots-1].px = (ca[0] + canext[0])/2;
        knot[nknots-1].py = (ca[1] + canext[1])/2;
        knot[nknots-1].pz = (ca[2] + canext[2])/2;

// Form vector C = A x B
// Vector C is normal to the peptide plane approximated by atoms Ca(i), O(i), and Ca(i+1
        knot[nknots-1].vnx = knot[nknots-1].ty*bz - knot[nknots-1].tz*by;
        knot[nknots-1].vny = knot[nknots-1].tz*bx - knot[nknots-1].tx*bz;
        knot[nknots-1].vnz = knot[nknots-1].tx*by - knot[nknots-1].ty*bx;

// If the peptide plane is part of a helix, translate in the direction of vector C, 
// away from the helix axis
        if(IsHelix(c_alphab[i].atom) && IsHelix(c_alphab[i+1].atom))
        {
           size = sqrt(knot[nknots-1].vnx*knot[nknots-1].vnx + 
                       knot[nknots-1].vny*knot[nknots-1].vny + 
                       knot[nknots-1].vnz*knot[nknots-1].vnz);
        
           knot[nknots-1].vsize = size;
           if( size )
           {   // 1.00 Angstrom Displacement
               wide = 0.7f/gl->deltav;
               knot[nknots-1].px += ((wide*knot[nknots-1].vnx)/size);
               knot[nknots-1].py += ((wide*knot[nknots-1].vny)/size);
               knot[nknots-1].pz += ((wide*knot[nknots-1].vnz)/size);
           } 
        }
        else 
            knot[nknots-1].vsize = 0;

// Form vector D = C x A
// Vector D lies parallel to the peptide plane and is perpendicular to vector A. 
        float dx = knot[nknots-1].vny*knot[nknots-1].tz - knot[nknots-1].vnz*knot[nknots-1].ty;
        float dy = knot[nknots-1].vnz*knot[nknots-1].tx - knot[nknots-1].vnx*knot[nknots-1].tz;
        float dz = knot[nknots-1].vnx*knot[nknots-1].ty - knot[nknots-1].vny*knot[nknots-1].tx;

// Scale vector D by one half the desired ribbon width
        dx /= (float)(3*gl->cylinder);
        dy /= (float)(3*gl->cylinder);
        dz /= (float)(3*gl->cylinder);
        knot[nknots-1].hsize = sqrt(dx*dx + dy*dy + dz*dz);

// Handle Carbonyl Oxygen Flip 
        if( prev && ((knot[0].hnx*dx + knot[0].hny*dy + knot[0].hnz*dz)<0) )
        {
            knot[nknots-1].hnx = -dx;   knot[nknots-1].vnx = -knot[nknots-1].vnx;
            knot[nknots-1].hny = -dy;   knot[nknots-1].vny = -knot[nknots-1].vny;
            knot[nknots-1].hnz = -dz;   knot[nknots-1].vnz = -knot[nknots-1].vnz;
        }
        else
        {
            knot[nknots-1].hnx = dx;
            knot[nknots-1].hny = dy;
            knot[nknots-1].hnz = dz;
        }

//  Set Ribbon Width 
        wide = (float)(6*gl->cylinder);
        if(selected)
           wide = (float)(8*gl->cylinder);

        ifArrow = false;
        if(plotStyle == CARTOONS)
        {
           if(IsBetaSheet(c_alphab[i].atom))
           {
              if(!IsBetaSheet(c_alphab[i+1].atom))
              {
                 ifArrow = true;
                 wide = (float)(12*gl->cylinder);
              }
           }
           else if(!IsHelix(c_alphab[i].atom))
                wide = (float)(kf*gl->cylinder);
        }
        if( knot[nknots-1].hsize && !ifArrow)
        {
            size = knot[nknots-1].hsize;
            knot[nknots-1].wx = (wide*knot[nknots-1].hnx)/size;
            knot[nknots-1].wy = (wide*knot[nknots-1].hny)/size;
            knot[nknots-1].wz = (wide*knot[nknots-1].hnz)/size;
            knot[nknots-1].wide = wide;
        }
        else
        {
            knot[nknots-1].wide = 0;
            knot[nknots-1].wx = 0;
            knot[nknots-1].wy = 0;
            knot[nknots-1].wz = 0;
        }

        if(prev && knot[0].hsize)
        {
           size = knot[0].hsize;
           knot[0].wx = (float)(wide*knot[0].hnx)/size;
           knot[0].wy = (float)(wide*knot[0].hny)/size;
           knot[0].wz = (float)(wide*knot[0].hnz)/size;
        }
        hwide = (float)(kf*gl->cylinder);
//240112        if(IsHelix(c_alphab[i].atom))
        if(IsHelix(c_alphab[i].atom) || (plotStyle == RIBBONS))
           hwide /= 3.;
        if(prev && knot[0].vsize)
        {
           size = knot[0].vsize;
           knot[0].dx = (float)(hwide*knot[0].vnx)/size;
           knot[0].dy = (float)(hwide*knot[0].vny)/size;
           knot[0].dz = (float)(hwide*knot[0].vnz)/size;
        }
        GetSizeandInten(&knot[nknots-1]);

        size = knot[nknots-1].vsize;
        knot[nknots-1].dx = (float)(hwide)*knot[nknots-1].vnx/size;
        knot[nknots-1].dy = (float)(hwide)*knot[nknots-1].vny/size;
        knot[nknots-1].dz = (float)(hwide)*knot[nknots-1].vnz/size;

        if(prev)
        {
           for(int k=0; k<nknots-2; k++)
           {
// Calculate Hermite Spline Points 
               knot[k+1].px = H[0][k]*knot[0].px + H[1][k]*knot[0].tx +
                              H[2][k]*knot[nknots-1].px + H[3][k]*knot[nknots-1].tx;
               knot[k+1].py = H[0][k]*knot[0].py + H[1][k]*knot[0].ty +
                              H[2][k]*knot[nknots-1].py + H[3][k]*knot[nknots-1].ty;
               knot[k+1].pz = H[0][k]*knot[0].pz + H[1][k]*knot[0].tz +
                              H[2][k]*knot[nknots-1].pz + H[3][k]*knot[nknots-1].tz;

//  Calculate Hermite Spline Widths 
               knot[k+1].wx = H[0][k]*knot[0].wx + H[2][k]*knot[nknots-1].wx;
               knot[k+1].wy = H[0][k]*knot[0].wy + H[2][k]*knot[nknots-1].wy;
               knot[k+1].wz = H[0][k]*knot[0].wz + H[2][k]*knot[nknots-1].wz;

// Calculate Surface Intensity 
               knot[k+1].vsize = 0;
               knot[k+1].vnx = H[0][k]*knot[0].vnx + H[2][k]*knot[nknots-1].vnx;
               knot[k+1].vny = H[0][k]*knot[0].vny + H[2][k]*knot[nknots-1].vny;
               knot[k+1].vnz = H[0][k]*knot[0].vnz + H[2][k]*knot[nknots-1].vnz;
               GetSizeandInten(&knot[k+1]);

//240112               if(plotStyle == CARTOONS)
               {
// Calculate Spline Heights 
                  knot[k+1].dx =  (float)(hwide*knot[k+1].vnx/knot[k+1].vsize); 
                  knot[k+1].dy =  (float)(hwide*knot[k+1].vny/knot[k+1].vsize);
                  knot[k+1].dz =  (float)(hwide*knot[k+1].vnz/knot[k+1].vsize);
// Calculate Surface Intensity 
                  knot[k+1].hsize = 0;
                  knot[k+1].hnx = H[0][k]*knot[0].hnx + H[2][k]*knot[nknots-1].hnx;
                  knot[k+1].hny = H[0][k]*knot[0].hny + H[2][k]*knot[nknots-1].hny;
                  knot[k+1].hnz = H[0][k]*knot[0].hnz + H[2][k]*knot[nknots-1].hnz;
                  GetSizeandInten(&knot[k+1]);
               }
           }

           DrawRibbon(i, knot, nknots, selected);
           knot[0] = knot[nknots-1];
            c_alphab[i].xyz[0] = knot[4].px;
            c_alphab[i].xyz[1] = knot[4].py;
            c_alphab[i].xyz[2] = knot[4].pz;
        }
        else                     // siamo all'inizio
        {
           knot[1] = knot[nknots-1];
           knot[0] = knot[nknots-1];
           knot[0].px = ca[0];
           knot[0].py = ca[1];
           knot[0].pz = ca[2];
           knot[0].hnx = dx;
           knot[0].hny = dy;
           knot[0].hnz = dz;
           c_alphab[i].xyz[0] = knot[0].px;
           c_alphab[i].xyz[1] = knot[0].py;
           c_alphab[i].xyz[2] = knot[0].pz;
           if(kmodel != -1)
              DrawRibbon(i, knot, 2, selected);
           if((IsDna(c_alphab[i].atom)) && (plotStyle == CARTOONS))
           {
               float color1[4];
               SetColor(c_alphab[i].atom, color1);
               TuneHSV( color1, knot[0].vinten);
               drawSphere(knot[0].px, knot[0].py, knot[0].pz, 4.5f*(float)gl->cylinder,
                         color1);
           }

        }
        prev = true;
        if(kmodel != -1)
           glPopName();
    }
    if(kmodel != -1)
    {
       glPopName();
       glPopName();
    }
}

void Model::DrawIsolated(GLWindow *gl)
{
      int i;
      float color1[4];
      PangoFontDescription *font_desc = pango_font_description_from_string(gl->fontname);

      glPushMatrix();
      glDisable (GL_LIGHTING);

      for(i=0; i<nats; i++)
      {
         if(atm[i].disegnato)
            continue;
         if( (!ifDrawH) &&  IsHydrogen(i) )
            continue;
         if( (!ifDrawW) &&  atm_au[iEqui[i]].isW )
            continue;
         if(atm[i].deleted || atm[i].hidden)
            continue;
         if(IsPeak(i))
            continue;
         (void) SetColor(i, color1);
         if(gl->Stereo && (gl->BgType < TYPE_FLAT))
            Desaturate(color1);
         DrawText(-1, font_desc, PANGO_ALIGN_CENTER, color1, 
           MRVectorMake(atm[i].xyzN), "+", NULL, true);

      }
//      glEnd();
      glPopMatrix();
      pango_font_description_free(font_desc);
}

void Model::DrawBonds(GLWindow *gl, int j0)
{
    vet3 p1, p2;
    double radius_c = gl->cylinder*gl->sc_bonds;
    jav_bond_type *legami;

    int i0, i1;
    if(plotStyle < VANDERWAALS)
    {
       i0 = 0;
       i1 = nconn_prot;
       legami = bonds;
    }
    else if ( (plotStyle == POLYPEPTIDE) || (plotStyle == RIBBONS) )
    {
       i0 = 0;
       i1 = nconn_bb;
       legami = bonds_bb;
    }
    else if (plotStyle == C_ALPHA)
    {
       i0 = 0;
       i1 = nconn_ca;
       legami = bonds_ca;
    }
    else
       return;

    glColor3f(1.0f, 1.0f, 1.0f);
    glPushName(1);
    glPushName(j0);

    for(int i=i0; i<i1;i++)
    {
        int j1 = legami[i].n1 - 1;
        if(j1 >= nats)
            continue;
        if(j1 < 0)
           return;
         if(atm[j1].deleted || atm[j1].hidden)
            continue;
        int j2 = legami[i].n2 - 1;
        if(j2 >= nats)
            continue;
         if(atm[j2].deleted || atm[j2].hidden)
            continue;
         if( (!ifDrawH) && (IsHydrogen(j1) || IsHydrogen(j2)))
            continue;
        if( (!ifDrawW) && ( atm_au[iEqui[j1]].isW  || atm_au[iEqui[j2]].isW ) )
            continue;
        p1 =  MRVectorMake(atm[j1].xyzN);
        p2 =  MRVectorMake(atm[j2].xyzN);
        glPushName(i);
        renderCylinder(gl, p1, p2, radius_c);
        glPopName();
    }
    glPopName();
    glPopName();
}

void Model::DrawBonds2(GLWindow *gl, int j0, bool ifLigands)
{
    vet3 p1, p2, p3;
    double radius_c = gl->cylinder*gl->sc_bonds;
    jav_bond_type *legami;

    int i0, i1, nomep = 1;
    if(ifLigands)
    {
       i0 = 0;
       i1 = nconn_lig;
       legami = bonds_lig;
       nomep = 4;
    }
    else if(plotStyle < VANDERWAALS)
    {
       i0 = 0;
       i1 = nconn_prot;
       legami = bonds;
    }
    else if ( (plotStyle == POLYPEPTIDE) || (plotStyle == RIBBONS) )
    {
       i0 = 0;
       i1 = nconn_bb;
       legami = bonds_bb;
       nomep = 3;
    }
    else if (plotStyle == C_ALPHA)
    {
       i0 = 0;
       i1 = nconn_ca;
       legami = bonds_ca;
       nomep = 2;
    }
    else
       return;

    glColor3f(1.0f, 1.0f, 1.0f);
    glPushName(nomep);
    glPushName(j0);

    for(int i=i0; i<i1;i++)
    {
        int j1 = legami[i].n1 - 1;
        if(j1 >= nats)
            continue;
        if(j1 < 0)
           return;
         if(atm[j1].deleted || atm[j1].hidden)
            continue;
        int j2 = legami[i].n2 - 1;
        if(j2 >= nats)
            continue;
         if(atm[j2].deleted || atm[j2].hidden)
            continue;
         if( (!ifDrawH) && (IsHydrogen(j1) || IsHydrogen(j2)))
            continue;
        if( (!ifDrawW) && ( atm_au[iEqui[j1]].isW  || atm_au[iEqui[j2]].isW ) )
            continue;
        p1 =  MRVectorMake(atm[j1].xyzN);
        p2 =  MRVectorMake(atm[j2].xyzN);
        p3 = MRVectorMedium(p1, p2);
        glPushName(i);
//        glPushName(atm[j1].seq);
        renderCylinder(gl, p1, p3, radius_c);
//        glPopName();
//        glPushName(atm[j2].seq);
        renderCylinder(gl, p3, p2, radius_c);
        glPopName();
    }
    glPopName();
    glPopName();
}

void Model::DrawAtoms(GLWindow *gl, int j)
{
  int subdiv;
  subdiv = 8;

  glColor3f(1.0f, 1.0f, 1.0f);
  glPushName(0);
  glPushName(j);

  for(int i=0; i<nats; i++)
  {
      glPushName(i);
      renderSphere(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2],
                   atm[i].radN[0], subdiv);
      glPopName();
  }
  glPopName();
  glPopName();
}

void CalcolaNormale(vet3 a, vet3 b, vet3 c, vet3 *vn)
{
   vet3 v1 = MRVectorSub(b, a);
   vet3 v2 = MRVectorSub(c, b);
   vet3 v3 = MRVectorCrossProduct(v1, v2);
   v3 = MRVectorNormalize(v3);
   MRVectorCopy(v3, vn);
}

int Model::FindStyle(int whichPol)
{
  int vert1 = iEqui[Polyhedra[whichPol].xc];
  for(int i=0; i<nPolyStyles; i++)
  {
      if(PolyStyle[i].vert == vert1)
         return i;
  }
  return -1;
}

void DrawPolyhedron(Vert *points, Vert *norms, int nfaces, int *nlati)
{
  int p = 0;
  for(int i=0; i<nfaces; i++)
  {
    glBegin(GL_POLYGON);
    for(int k=0; k<nlati[i];k++)
    {
          glNormal3f(norms[p].x, norms[p].y, norms[p].z);
          glVertex3f(points[p].x, points[p].y, points[p].z);
          p++;
    }
    glEnd();
  }
}

void DrawPolyhedron_1n(Vert *points, Vert *norms, int nfaces, int *nlati)
{
  int p = 0;
  for(int i=0; i<nfaces; i++)
  {
      Vert n = norms[i];
      glBegin(GL_POLYGON);
      glNormal3f(n.x, n.y, n.z);
      for(int k=0; k<nlati[i];k++)
      {
          glVertex3f(points[p].x, points[p].y, points[p].z);
          p++;
      }
      glEnd();
  }
}

int NumberOfFaces(polyhedron_type Poly)
{
  int nf = 8;
  switch(Poly.nvert)
  {
  case 4:
       nf = 4;
       break;
  case 5:
       nf = 5;
       break;
  case 6:
  default:
       nf = 8;
       break;
  }
  return nf;
}

void Model::DrawPolyhedra(GLWindow *gl)
{
  if(Polyhedra == NULL)
     return;
  for (int i=0; i<nPoly; i++)
  {
       int ind = FindStyle(i);
       if(ind == -1)
          continue;
       if(!PolyStyle[ind].visible)
          continue;
       glEnable (GL_LIGHTING);
       glEnable (GL_NORMALIZE);
       int nf = NumberOfFaces(Polyhedra[i]);
       int *nlati = g_new0(int, nf);
       int np = 0;
       for(int j=0; j<nf; j++)
       {
           nlati[j] = Polyhedra[i].face[j].n;
           np += nlati[j];
       }
       Vert *points = g_new0(Vert, np);
       Vert *norms;
       if(gl->normalToV)
          norms  = g_new0(Vert, np);
       else
          norms  = g_new0(Vert, nf);
       float color[4];
       memcpy(color, PolyStyle[ind].colore, 4*sizeof(float));
       int jj = 0;
       for(int j=0; j<nf; j++)
       {
           for(int k=0; k<Polyhedra[i].face[j].n; k++)
           {
               points[jj].x = atm[Polyhedra[i].face[j].fp[k]].xyzN[0];
               points[jj].y = atm[Polyhedra[i].face[j].fp[k]].xyzN[1];
               points[jj].z = atm[Polyhedra[i].face[j].fp[k]].xyzN[2];
               if(gl->normalToV)
               {
                   norms[jj].x = atm[Polyhedra[i].face[j].fp[k]].vn[0];
                   norms[jj].y = atm[Polyhedra[i].face[j].fp[k]].vn[1];
                   norms[jj].z = atm[Polyhedra[i].face[j].fp[k]].vn[2];
               }
               jj++;
           }
           if(!gl->normalToV)
               norms[j] = norm(points[jj-3], points[jj-2], points[jj-1]);
       }
       glEnable (GL_BLEND);
       glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
       if(PolyStyle[ind].transparency)
       {
          glDepthMask (GL_FALSE);
          color[3] = (float) (100 - PolyStyle[ind].transparency)/100.f;
       }
       else
          glDepthMask (GL_TRUE);
       glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color);
       glPolygonMode( GL_FRONT_AND_BACK, GL_FILL ) ;
       if(gl->normalToV)
          DrawPolyhedron(points, norms, nf, nlati);
       else
          DrawPolyhedron_1n(points, norms, nf, nlati);

       if(PolyStyle[ind].ifBorder)
       {
          memcpy(color, PolyStyle[ind].Bcolore, 4*sizeof(float));
          glLineWidth(PolyStyle[ind].line_width) ;
          glEnable (GL_LINE_SMOOTH);
          glDisable (GL_LIGHTING);
          glPolygonMode( GL_FRONT_AND_BACK, GL_LINE ) ;
          glColor4fv(color);
          if(gl->normalToV)
             DrawPolyhedron(points, norms, nf, nlati);
          else
             DrawPolyhedron_1n(points, norms, nf, nlati);
       }
       g_free(nlati);
  }
  glEnable (GL_LIGHTING);
  glDepthMask (GL_TRUE);
}

void Model::DrawSBonds(GLWindow *gl, int nlegs, jav_bond_type *legs, int tipo)
{
      float colorMix[] = {0.3f, 0.3f, 1.f, 1.f};
      if(!nlegs)
         return;

      glLineStipple(1, 0x00FF);
      glEnable(GL_LINE_STIPPLE);
      float colore[4];
      if(tipo == 'H')
         memcpy(colore, gl->ColorHB, 4*sizeof(float));
      else if(tipo == 'W')
         memcpy(colore, gl->ColorSC, 4*sizeof(float));
      else
         memcpy(colore, colorMix, 4*sizeof(float));
     DrawSticks(gl, 0, nlegs, legs, false, colore);
     glDisable(GL_LINE_STIPPLE);
}

void Model::DrawModel(GLWindow *gl)
{
  glDisable (GL_BLEND);
  if((plotStyle >= POLYPEPTIDE) && (nconn_prot == 0) && (nconn_ca != 0))
     plotStyle = C_ALPHA;
  if((nconn_bb == 0) && (plotStyle >= POLYPEPTIDE))
      plotStyle = STICK_AND_BALLS;
  if((nconn_bb == 0) && (colorMode >= BY_RESIDUE))
      colorMode = BY_SPECIES;

//  if(plotStyle > BONDSTYLE)
  {
     if(plotStyle < RIBBONS)
     {
        glDisable(GL_COLOR_MATERIAL);
        glEnable (GL_LIGHTING);
        glEnable (GL_NORMALIZE);
     }
     else
     {
        glDisable(GL_FOG);
        glDisable(GL_CLIP_PLANE0);
        glDisable(GL_CLIP_PLANE1);
     }
     glShadeModel(GL_SMOOTH);
     glPolygonMode( GL_FRONT_AND_BACK, GL_FILL ) ;
  }

  int colorSave = colorMode;
  switch(plotStyle)
  {
  case WIRES:
  case BONDSTYLE:
       DrawSticks(gl);
       DrawCircles(gl,1);
       //DrawIsolated(gl);
       break;

  case SIMPLE:
       DrawSticks(gl);
       for(int i=0; i<nats;i++)
           atm[i].disegnato = 0;
       DrawCircles(gl);
       break;

  case VANDERWAALS:
       DrawBalls(gl, 1);
       break;

  case ELLIPSOIDS:
       DrawBalls(gl, 0, true);
       DrawCyls (gl);
       break;

  case STICK_AND_BALLS:
       DrawCyls (gl);
       for(int i=0; i<nats;i++)
           atm[i].disegnato = 0;
       DrawBalls(gl, gl->RadType?3:0);
       break;

  case RIBBONS:
  case STRANDS:
  case CARTOONS:
       DrawRibbons(gl);
       if(nucleic == 1)
       {
          DrawScaletta(gl);
/*
          DrawCyls(gl);
          for(int i=0; i<nats;i++)
              atm[i].disegnato = 1;
          for(int i=0;i<nconn_nuc;i++)
          {
              atm[bonds_nuc[i].n1-1].disegnato = 0;
              atm[bonds_nuc[i].n2-1].disegnato = 0;
          }
          DrawBalls(gl, 0);
*/
       }
  case C_ALPHA:
  case RODS:
  case POLYPEPTIDE:
       if(plotStyle < RIBBONS)
          DrawCyls (gl);
       if(plotStyle == RODS)
          DrawBalls(gl, 2);
       if((plotStyle >= POLYPEPTIDE) && ((drawLigand > 0) || ifDrawW ))
       {     
           colorMode = colorSave;
           glDisable(GL_COLOR_MATERIAL);
//           for(int i=0; i<nconn_lig;i++)
//           {
//               Secondary[bonds_lig[i].n1 - 1] = 'L';
//               Secondary[bonds_lig[i].n2 - 1] = 'L';
//           }
           if(colorMode >= BY_RESIDUE)
              colorMode = BY_SPECIES;
           if(drawLigand > 0)
           {
           switch(drawLigand-1)
           {
           case WIRES:
           case BONDSTYLE:
                DrawSticks(gl, 0, nconn_lig, bonds_lig, true);
                for(int i=0; i<nats;i++)
                    if(!atm[i].disegnato)
                        atm[i].disegnato = !IsHetero(i);
                DrawIsolated(gl);
                break;
           case SIMPLE:
                for(int i=0; i<nats;i++)
                    atm[i].disegnato = 1;
                DrawSticks(gl, 0, nconn_lig, bonds_lig, true);
                for(int i=0; i<nats;i++)
                        atm[i].disegnato = !IsHetero(i);
                DrawCircles(gl, 1);
                break;
           case STICK_AND_BALLS:
           case RODS:
                for(int i=0; i<nats;i++)
                    atm[i].disegnato = 1;
                DrawCyls(gl, false, true);
                if((drawLigand - 1) == STICK_AND_BALLS)
                {
                   for(int i=0; i<nats;i++)
                       atm[i].disegnato = !IsHetero(i);
                   DrawBalls(gl, 0);
                }
                else
                {
                   for(int i=0; i<nats;i++)
                       if(!atm[i].disegnato)
                          atm[i].disegnato = !IsHetero(i);
                   DrawBalls(gl, 2);
                }
           }
           }
           else
           {
              for(int i=0; i<nats;i++)
                  atm[i].disegnato = !atm_au[iEqui[i]].isW;
              DrawBalls(gl, 0);
           }
           colorMode = colorSave;
           if(plotStyle == POLYPEPTIDE)
              DrawSBonds(gl, nconn_mix, bonds_mix,'M');
       }
       break;
  }
  if(ifPoly && (nPoly > 0))
     DrawPolyhedra(gl);
  gpointer ck = get_object(GLWINDOW, "Hydrogen Bonds");
  if(obj_get_active(ck))
     DrawSBonds(gl, nlegsH, legsH,'H');
  ck = get_object(GLWINDOW, "Short Contacts");
  if(obj_get_active(ck))
     DrawSBonds(gl, nlegsW, legsW, 'W');

  if(gl->fog)
     glEnable(GL_FOG);
  if(gl->Clip)
  {
     glEnable(GL_CLIP_PLANE0);
     glEnable(GL_CLIP_PLANE1);
  }
}

void Model::DrawCyls(GLWindow *gl, bool selected, bool ifLigands)
{
  int i0=0, i1=0;
  jav_bond_type *legs=NULL;

  if(ifLigands)
  {
     i0 = 0;
     i1 = nconn_lig;
     legs = bonds_lig;
  }
  else if(plotStyle < VANDERWAALS)
  {
     i0 = 0;
     i1 = nconn_prot;
     legs = bonds;
  }
  else if ( (plotStyle == POLYPEPTIDE) || (plotStyle == RIBBONS) )
  {
     i0 = 0;
     i1 = nconn_bb;
     legs = bonds_bb;
  }
  else if (plotStyle == C_ALPHA)
  {
     i0 = 0;
     i1 = nconn_ca;
     legs = bonds_ca;
  }
  else if (plotStyle == CARTOONS)
  {
     i0 = 0;
     i1 = nconn_nuc;
     legs = bonds_nuc;
  }

  for(int i=0; i<nats;i++)
      atm[i].disegnato = ((plotStyle == STICK_AND_BALLS)||(plotStyle == ELLIPSOIDS));

//  DrawSticks(gl, i0, i1);               // controllo cilindri
  if(legs != NULL)
  {
     if (selected)
         DrawSelectedCylinder(gl, i0, i1, legs);
     else
         Draw2Cylinder(gl, i0, i1, legs);
  }
  if (selected)
  {
     if(nlegsH)
         DrawSelectedCylinder(gl, 0, nlegsH, legsH);
     if(nlegsW)
         DrawSelectedCylinder(gl, 0, nlegsW, legsW);
  }
}

void Model::DrawSelected(GLWindow *gl)
{
  glDisable (GL_BLEND);
  if((plotStyle >= POLYPEPTIDE) && (nconn_prot == 0) && (nconn_ca != 0))
     plotStyle = C_ALPHA;

  if(plotStyle < RIBBONS)
  {
     gl->SetLightForSelected(1);
     DrawCyls(gl, true);
     for(int i=0; i<nats;i++)
         atm[i].disegnato = 0;
     if(plotStyle >= POLYPEPTIDE)
        DrawCyls(gl, true, true);
     else
        DrawSelectedBalls(gl);
     gl->ResetLightForSelected();
  }
  else
  {
      glDisable(GL_FOG);
      glDisable(GL_CLIP_PLANE0);
      glDisable(GL_CLIP_PLANE1);
      DrawRibbons(gl, true);
  }

  if(gl->fog)
     glEnable(GL_FOG);
  if(gl->Clip)
  {
     glEnable(GL_CLIP_PLANE0);
     glEnable(GL_CLIP_PLANE1);
  }

}

void Model::DrawLigandsLabels(double model[], double proj[], GLint view[])
{
     GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
     GLdouble winx, winy, winz;
     GLdouble x, y, z;
     PangoFontDescription *font_desc = pango_font_description_from_string(gl->fontname);

     float molt = (float) (gl->sc_atoms*gl->Scale*sc_factor)/SCALA;
     int ind = 0;
     for(int i=0; i<nats; i++)
     {
         if(!IsHetero(i))
             continue;
         if(atm[i].deleted || (atm[i].hidden))
             continue;
         if( (!ifDrawH || (specialLab == NONH)) &&  IsHydrogen(i) )
             continue;
         if(((specialLab == HEAVY) &&  !IsHeavy(i)) )
             continue;
         if(((specialLab == SELECTED) &&  !atm[i].selected) )
             continue;
         if((specialLab < 0) && (LabelS[atm[i].Z-1] == 0))
             continue;
         if( (!ifDrawW) &&  atm_au[iEqui[i]].isW )
            continue;
// NEW
         FromCoordToWin(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2],
                    model, proj, view, &winx, &winy, &winz);
         double delta = atm[i].radN[ind]*molt;
         if(plotStyle < RODS)
            delta = delta/4;
         PangoRectangle *rect1 = GetLabelRect(atm[i], i, font_desc);
         double deltax = delta * gl->xoffset * npixels / 2;
         double deltay = delta * gl->yoffset * npixels / 2 - rect1->height/2;
         FromWinToCoord(winx+deltax, winy+deltay,
                winz-delta/10, model, proj, view, &x, &y, &z);
         DrawAtomLabel(atm[i], i, x, y, z, font_desc);
     }
     pango_font_description_free(font_desc);
}

void Model::DrawLabels(bool ifSingle)
{
  GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
  GLboolean val;
  glGetBooleanv(GL_LIGHTING, &val);
  glDisable (GL_LIGHTING);
  glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
  PangoFontDescription *font_desc = pango_font_description_from_string(gl->fontname);

  double *model = g_new0(double, 16);
  double *proj = g_new0(double, 16);
  GLint view[4];
  GLdouble winx, winy, winz;
  GLdouble x, y, z;
  glGetDoublev(GL_MODELVIEW_MATRIX, model);
  glGetDoublev(GL_PROJECTION_MATRIX, proj);
  glGetIntegerv(GL_VIEWPORT, view);
  float molt = (float) (gl->sc_atoms*gl->Scale*sc_factor)/SCALA;
  if(plotStyle < POLYPEPTIDE)
  {
     int ind = 0;
     if(plotStyle == VANDERWAALS)
        ind = 1;

     for(int i=0; i<nats; i++)
     {
         if(ifSingle)
         {
            if(atm[i].drawLabel < 1)
               continue;
         }
         else
         {
            if(atm[i].deleted || (atm[i].hidden))
               continue;
            if( (!ifDrawH || (specialLab == NONH)) &&  IsHydrogen(i) )
               continue;
            if(((specialLab == HEAVY) &&  !IsHeavy(i)) )
               continue;
            if(((specialLab == SELECTED) &&  !atm[i].selected) )
               continue;
            if((specialLab < 0) && (LabelS[atm[i].Z-1] == 0))
               continue;
         }
// NEW
         FromCoordToWin(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2],
                    model, proj, view, &winx, &winy, &winz);
         double delta = atm[i].radN[ind]*molt;
         if(plotStyle < RODS)
            delta = delta/4;
         PangoRectangle *rect1 = GetLabelRect(atm[i], i, font_desc);
         double deltax = delta * gl->xoffset * npixels / 2;
         double deltay = delta * gl->yoffset * npixels / 2 - rect1->height/2;
         FromWinToCoord(winx+deltax, winy+deltay,
                winz-delta/10, model, proj, view, &x, &y, &z);
         DrawAtomLabel(atm[i], i, x, y, z, font_desc);
     }
  }
  else
  {
     if(ifSingle || ifLabels || (specialLab == NONH))
     {
        for(int i=0; i<(int)c_alphab.size(); i++)
        {
            int ca = c_alphab[i].atom;
            if(atm[ca].deleted || atm[ca].hidden)
               continue;
            if(ifSingle)
            {
               if(atm[ca].drawLabel < 1)
                  continue;
            }
            FromCoordToWin(atm[ca].xyzN[0],atm[ca].xyzN[1], atm[ca].xyzN[2],
                    model, proj, view, &winx, &winy, &winz);
            double delta = 0.01*molt;
            FromWinToCoord(winx+delta*gl->Width, winy+delta*gl->Height,
                winz-delta, model, proj, view, &x, &y, &z);
            DrawAtomLabel(atm[ca], i, x, y, z, font_desc, c_alphab[i].ResLab.c_str());
        }
     }
     if(drawLigand > 0)
        if(ifLabels || (specialLab != 0))
           DrawLigandsLabels(model, proj, view);
  }
  if(val == GL_TRUE)
     glEnable (GL_LIGHTING);
  pango_font_description_free(font_desc);

}

void Model::DrawCircles(GLWindow *gl, int ind)
{
      int i;
      float color1[4];

      glPushMatrix();
//      glDisable (GL_LIGHTING);

      int subd = (int)(gl->Scale*sc_factor);
      if(subd <= 1)
         subd = 2;
      else if(subd > 20)
         subd = 20;
      for(i=0; i<nats; i++)
      {
         if( (!ifDrawH) &&  IsHydrogen(i) )
            continue;
         if( (!ifDrawW) &&  atm_au[iEqui[i]].isW )
            continue;
         if( ind && atm[i].disegnato)
            continue;
         if(atm[i].deleted || atm[i].hidden)
            continue;
         if(IsPeak(i))
            continue;
         (void) SetColor(i, color1);
         if(gl->Stereo && (gl->BgType < TYPE_FLAT))
            Desaturate(color1);
//         glColor4f(color1[0], color1[1], color1[2], color1[3]);
         glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color1);
         glLoadIdentity();
         double Scale =  gl->Scale;
         glTranslated( gl->obj_pos.x, gl->obj_pos.y, -gl->obj_pos.z );
         glScaled( Scale, Scale, Scale);
         glMultMatrixd (gl->Rotm.m);
         glTranslatef(atm[i].xyzN[0], atm[i].xyzN[1], atm[i].xyzN[2]);
         glCallList (subd);
//         glCallList (1);
      }
      glPopMatrix();
}

void Model::DrawCircles(GLWindow *gl)
{
     DrawCircles(gl, 0);
}

void GLWindow::CheckDistAng()
{
    bool atm_exist = true;
    std::vector<size_t> to_destroy;
    to_destroy.clear();
    for(size_t i=0; i<DistAng_vet.size(); i++)
    {
        int t = DistAng_vet.at(i).op;
        for(int k=0; k<t; k++)
        {
            int wm = DistAng_vet.at(i).wM[k];
            int wa = DistAng_vet.at(i).wA[k];
            if(wm >= nModels)
               atm_exist = false;
            else if(wa >= ModelInfo[wm].nats)
               atm_exist = false;
        }
        if(atm_exist)
        {
           for(int k=0; k<t; k++)
           {
               int wm = DistAng_vet.at(i).wM[k];
               int wa = DistAng_vet.at(i).wA[k];
               ModelInfo[wm].atm[wa].drawLabel = 1;
           }
        }
        else
           to_destroy.push_back(i);
    }
    for(size_t i=to_destroy.size(); i>0; i--)
        DistAng_vet.erase(DistAng_vet.begin() + to_destroy.at(i-1));
    to_destroy.clear();
}

void GLWindow::DrawDistAng()
{
    if(DistAng_vet.size() <= 0)
       return;
    PangoFontDescription *fdesc = pango_font_description_from_string(fontname);
//    float strpos[3];
    vet3 strpos;
    std::string str;

    glDisable (GL_DEPTH_TEST);
    glLineStipple(1, 0x00FF);
    glEnable(GL_LINE_STIPPLE);
    glLineWidth(1.5);
    glPushMatrix();
    glEnable (GL_BLEND);
    glDisable (GL_LIGHTING);
    if(smooth)
    {
       glEnable (GL_LINE_SMOOTH);
       glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
       glHint (GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
    }
    for(size_t i=0; i<DistAng_vet.size(); i++)
    {
        int t = DistAng_vet.at(i).op;
        str.assign(g_strdup_printf("%.3f", DistAng_vet.at(i).result));
        vet3 *p = g_new0(vet3, t);
        double ang = (double) DistAng_vet.at(i).result;
        for(int k=0; k<t; k++)
        {
            int wm = DistAng_vet.at(i).wM[k];
            int wa = DistAng_vet.at(i).wA[k];
            p[k] = MRVectorMake(ModelInfo[wm].atm[wa].xyzN);
        }
        if(t <= 3)
        {
           strpos = DrawDistLines(t, p);
           if(t == 3)
              strpos = DrawDistAngles(t, p, ang);
        }
        else
           strpos = DrawDistDihedral(t, p);
        DrawText(-1, fdesc, PANGO_ALIGN_CENTER, DistColor, strpos,
                str.c_str(), NULL, true);
    }
    glDisable(GL_LINE_STIPPLE);
    glDisable(GL_LINE_SMOOTH);
    glDisable(GL_BLEND);
    glPopMatrix();
    glEnable (GL_DEPTH_TEST);
    glLineWidth(1);
    pango_font_description_free(fdesc);
}

PangoRectangle *Model::GetLabelRect(jav_atom_type at, int i,
                                    PangoFontDescription *fdesc)
{
   std::string str;
   switch(labelType)
   {    
   default:
   case LABEL_ATOM:
        str.assign(at.lab);
        break;
   case LABEL_SPECIES:
        str.assign(get_nth_element(at.Z-1));
        break;
   case LABEL_NUMBER:
        str.assign(g_strdup_printf("%d", i+1));
        break;
   }    
   return get_text_rect(fdesc, str.c_str());
}

void Model::DrawAtomLabel(jav_atom_type at, int i, GLdouble x, GLdouble y,
                   GLdouble z, PangoFontDescription *font_desc,
                   const char *LabelAt)
{
//     if((!ifLabels) && (!notLabelH))
//        return;
//     glColor4f(labelcolor[0], labelcolor[1], labelcolor[2], labelcolor[3]);

//     glRasterPos3d(x, y, z);
     std::string str;
     if(LabelAt != NULL)
        str.assign(LabelAt);
     else
     {
        switch(labelType)
        {
        default:
        case LABEL_ATOM:
             str.assign(at.lab);
             break;
        case LABEL_SPECIES:
             str.assign(get_nth_element(at.Z-1));
             break;
        case LABEL_NUMBER:
//          str.assign(g_strdup_printf("%d", iEqui[i]+1));
             str.assign(g_strdup_printf("%d", i+1));
             break;
        }
     }

     vet3 xy = {{x, y, z}};
     float ico[] = {labelcolor[0], labelcolor[1], labelcolor[2]};
     int width = -1;
     size_t found;
     found=str.find_last_not_of(" ");
     if (found!=std::string::npos)
       str.erase(found+1);
     else
       str.clear();  
     DrawText(width, font_desc, PANGO_ALIGN_CENTER, ico, xy, str.c_str(), NULL,
     true);
}

void Model::DrawBalls(GLWindow *gl, int ind, bool ifEll)
{
  float color1[4];
  int subdiv;
;

  float subd = (float) (300. * maxRad[ind]) *sc_factor;
  if(ind != 2)
     subd *= (float) gl->sc_atoms;
  subdiv = (gl->Scale > 1.)? ((int) (subd * gl->Scale)) : ((int) subd);
//  if(moving && (nats > 100))
//     subdiv = MRmin(8, subdiv);

  for(int i=0; i<nats; i++)
  {
      int z = atm[i].Z-1;
      if((z< 0) || (z > N_TRUE_ELEMENTS))
         z = N_TRUE_ELEMENTS;
      if( (!ifDrawH) &&  IsHydrogen(i) )
          continue;
      if( (!ifDrawW) &&  atm_au[iEqui[i]].isW )
          continue;
      if(atm[i].deleted || atm[i].hidden)
            continue;
      if(IsPeak(i))
         continue;
      if( (ind != 1) && atm[i].disegnato)
            continue;

      (void) SetColor(i, color1);
      if(gl->Stereo && (gl->BgType < TYPE_FLAT))
         Desaturate(color1);
      glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color1);
      glEnable (GL_BLEND);
      glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
      float radius = atm[i].radN[ind];
      if(ind != 2)
         radius *=  gl->scale_atoms[z];
      else
         radius *= (float) gl->sc_bonds;
      if(ifEll && !IsHydrogen(i))
      {
//         static const float no_mat[] = {0.0f, 0.0f, 0.0f, 1.0f};
         GLfloat mat_planes[] = { color1[0], color1[1], color1[2], 0.8f};
         GLfloat mat_ell[]    = { color1[0], color1[1], color1[2], 1};
         Darker(mat_planes);
         float semi = atm[i].seminN * gl->scale_atoms[z];
         glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_planes);
         renderCircs(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2], 
                     semi, subdiv, atm[i].scala, atm[i].mat);
         mat_ell[3] = 0.01f * (100 - gl->ell_tr);
//         color1[3] = 0.2f;
//         glMaterialfv(GL_FRONT, GL_EMISSION, color1);
         glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_ell);
         renderEll(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2], 
                   semi, subdiv, atm[i].scala, atm[i].mat, false);
//         glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
      }
      else
      {
         renderSphere(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2], 
                   radius, subdiv);
//         renderSphere(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2], 
//                   radius, subdiv, quadric);
      }
      glDisable (GL_BLEND);
  }
}

void Model::DrawSelectedBalls(GLWindow *gl)
{
  int subdiv;

  bool ifEll = (plotStyle == ELLIPSOIDS);
  float subd;
  if((plotStyle == STICK_AND_BALLS) || (plotStyle == ELLIPSOIDS))
     subd = (float) (float) (300. * maxRad[0]) * sc_factor;
  else if(plotStyle == VANDERWAALS)
     subd = (float) (float) (300. * maxRad[1]) * sc_factor;
  else
     subd = (float) (float) (300. * maxRad[2]) * sc_factor;
  if( (plotStyle >= STICK_AND_BALLS) && (plotStyle <= VANDERWAALS) )
     subd *= (float) gl->sc_atoms;
  subdiv = (gl->Scale > 1.)? ((int) (subd * gl->Scale)) : ((int) subd);
  float add = (float)(0.2 / gl->deltav);

  for(int i=0; i<nats; i++)
  {
      int z = atm[i].Z-1;
      if((z< 0) || (z > N_ELEMENTS))
         z = N_ELEMENTS;
      if(!atm[i].selected || atm[i].deleted || atm[i].hidden)
            continue;
      float radius;
      if(plotStyle == STICK_AND_BALLS)
         radius = (float) gl->scale_atoms[z] * atm[i].radN[(gl->RadType?3:0)];
      else if(plotStyle == ELLIPSOIDS)
         radius = (float) gl->scale_atoms[z] * atm[i].radN[0];
      else if(plotStyle == VANDERWAALS)
         radius = (float) gl->scale_atoms[z] * atm[i].radN[1];
      else
         radius = (float) gl->sc_bonds * atm[i].radN[2];

      if(ifEll && !IsHydrogen(i))
      {
         float semi = atm[i].seminN * gl->scale_atoms[z];
         renderEll(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2],
                   semi+add, subdiv, atm[i].scala, atm[i].mat, false);
      }
      else
          renderSphere(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2], 
                   radius+add, subdiv);

  }
}

void Model::DrawPeaks(GLWindow *gl)
{
  int subdiv;
  GLboolean val;
  glGetBooleanv(GL_LIGHTING, &val);
  glDisable (GL_LIGHTING);

  glBindTexture(GL_TEXTURE_2D, iTexture[6]);
  glEnable (GL_TEXTURE_2D);

#if 0
   float mat_e[4] = {0.9f, 1.0f, 1.0f, 1.0f};
   gl->SetLightForSelected(0, mat_e, 0.25f);
#endif

  float subd;
  if((plotStyle == STICK_AND_BALLS) || (plotStyle == ELLIPSOIDS))
     subd = (float) (float) (300. * maxRad[0]) * sc_factor;
  else if(plotStyle == VANDERWAALS)
     subd = (float) (float) (300. * maxRad[1]) * sc_factor;
  else
     subd = (float) (float) (300. * maxRad[2]) * sc_factor;
  if( (plotStyle >= STICK_AND_BALLS) && (plotStyle <= VANDERWAALS) )
     subd *= (float) gl->sc_atoms;
  subdiv = (gl->Scale > 1.)? ((int) (subd * gl->Scale)) : ((int) subd);

  for(int i=0; i<nats; i++)
  {
      int z = atm[i].Z-1;
      if((z< 0) || (z > N_ELEMENTS))
         z = N_ELEMENTS;
      if(!IsPeak(i)) 
         continue;
      float radius;
      if(plotStyle == VANDERWAALS)
         radius = gl->scale_atoms[z] * atm[i].radN[1];
      else
         radius = gl->scale_atoms[z] * atm[i].radN[0];
//      glColor4fv(mat_e);
      renderSphere(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2], 
                   radius, subdiv, true);
//      gl->ResetLightForSelected();
  }
  glDisable (GL_TEXTURE_2D);
  if(val == GL_TRUE)
     glEnable (GL_LIGHTING);
}

bool Model::IsHeavy(int which)
{
  return(atm[which].Z >= 10);
}

bool Model::IsHydrogen(int which)
{
  return(atm[which].Z == 1);
}

bool Model::IsPeak(int which)
{
  return (atm[which].color == -1);
}

bool Model::IsDna(int which)
{
   int i = iEqui[which];
   return ( Secondary[i] == 'D') ;
}

bool Model::IsLigand(int which)
{
   int i = iEqui[which];
   return ( (Secondary[i] == 'L') | (Secondary[i] == 'l') );
}

bool Model::IsHelix(int which)
{
   int i = iEqui[which];
   return ( (Secondary[i] == 'H') | (Secondary[i] == 'h') |
            (Secondary[i] == 'P') | (Secondary[i] == 'p') |
            (Secondary[i] == 'G') | (Secondary[i] == 'g') 
          );
}

bool Model::IsBetaSheet(int which)
{
    int i = iEqui[which];
    return ( (Secondary[i] == 'E') | (Secondary[i] == 'e') |
             (Secondary[i] == 'S') | (Secondary[i] == 's'));
}

bool Model::IsTurn(int which)
{
    int i = iEqui[which];
    return ( (Secondary[i] == 'T') | (Secondary[i] == '&')  |
             (Secondary[i] == '1') | (Secondary[i] == '2')  |
             (Secondary[i] == '3') | (Secondary[i] == '4')  |
             (Secondary[i] == '5') | (Secondary[i] == '6')  |
             (Secondary[i] == '7') | (Secondary[i] == '8')  |
             (Secondary[i] == '@') );
}

void Model::Draw2Cylinder(GLWindow *gl, int i0, int i1, jav_bond_type *legs)
{
    vet3 p1, p2, p3;
    int index1, index2;
    float color1[4], color2[4];
    double radius_c = gl->cylinder*gl->sc_bonds;
    if(legs == NULL)
       legs = bonds;

    glEnable (GL_LIGHTING);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, light_chain);

    for(int i=i0; i<i1;i++)
    {
        int j1 = legs[i].n1 - 1;
        if(j1 >= nats)
            continue;
        if(j1 < 0)
           return;
         if(atm[j1].deleted || atm[j1].hidden)
            continue;
        int j2 = legs[i].n2 - 1;
        if(j2 >= nats)
            continue;
         if(atm[j2].deleted || atm[j2].hidden)
            continue;
        if( (!ifDrawH) &&  (IsHydrogen(j1) || IsHydrogen(j2)) )
            continue;
        if( (!ifDrawW) && ( atm_au[iEqui[j1]].isW  || atm_au[iEqui[j2]].isW ) )
            continue;
        index1 = SetColor(j1, color1);
        index2 = SetColor(j2, color2);
        int ordleg = legs[i].ordleg;
        if(!gl->ShowMultBonds)
           ordleg = 1;
        p1 = MRVectorMake(atm[j1].xyzN);
        p2 = MRVectorMake(atm[j2].xyzN);
        p3 = MRVectorMedium(p1, p2);
        
        double v;
        mat4x4 m;
        vet3 p4 = MRVectorZero;
        if(ordleg > 1)
        {
           for(int kat=0; kat<atm[j1].numleg; kat++)
           {
               int j3 = atm[j1].leg[kat];
               if(j3 != j2)
               {
                  p4 = MRVectorMake(atm[j3].xyzN);
                  break;
               }
           }
           if(IsZero(MRVectorLength(p4)))
           {
              for(int kat=0; kat<atm[j2].numleg; kat++)
              {
                  int j3 = atm[j2].leg[kat];
                  if(j3 != j1)
                  {
                     p4 = MRVectorMake(atm[j3].xyzN);
                     break;
                  }
              }
           }
        }
        if(gl->Stereo && (gl->BgType < TYPE_FLAT))
           Desaturate(color1);
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color1);
        if(index1 == index2)
        {
           glPushMatrix();
           m = MRCylinderMatrix(&v, p1, p2, p4);
           glTranslated( p1.x, p1.y, p1.z);
           glMultMatrixd (m.m);
           DrawCylinder(v, radius_c, gl->subdivisions, 
                         atm[j1].disegnato, atm[j2].disegnato, ordleg);
        }
        else
        {
           glPushMatrix();
           m = MRCylinderMatrix(&v, p1, p3, p4);
           glTranslated( p1.x, p1.y, p1.z);
           glMultMatrixd (m.m);
           DrawCylinder(v, radius_c, gl->subdivisions,
                         atm[j1].disegnato, 1, ordleg);
           if(gl->Stereo && (gl->BgType < TYPE_FLAT))
              Desaturate(color2);
           glPopMatrix();
           glPushMatrix();
           m = MRCylinderMatrix(&v, p3, p2, p4);
           glTranslated( p3.x, p3.y, p3.z);
           glMultMatrixd (m.m);
           glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color2);
           DrawCylinder(v, radius_c, gl->subdivisions,
                      1, atm[j2].disegnato, ordleg);
        }
        atm[j1].disegnato = 1;
        atm[j2].disegnato = 1;
        glPopMatrix();
    }
}

void Model::DrawSelectedCylinder(GLWindow *gl, int i0, int i1, jav_bond_type *leg)
{
    vet3 p1, p2;
    double radius_c = gl->cylinder*gl->sc_bonds*1.5;

    glEnable (GL_NORMALIZE);
    glEnable (GL_LIGHTING);
    glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
    glEnable(GL_COLOR_MATERIAL);
    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA,  GL_SRC_ALPHA);

    for(int i=i0; i<i1;i++)
    {
         if(!leg[i].selected)
            continue;
        int j1 = leg[i].n1 - 1;
        if(j1 >= nats)
            continue;
        if(atm[j1].deleted || atm[j1].hidden)
            continue;
        if(j1 < 0)
           return;
        int j2 = leg[i].n2 - 1;
        if(j2 >= nats)
            continue;
        if(atm[j2].deleted || atm[j2].hidden)
            continue;
        p1 =  MRVectorMake(atm[j1].xyzN);
        p2 =  MRVectorMake(atm[j2].xyzN);
        glPushMatrix();
        double v;
        mat4x4 m = MRCylinderMatrix(&v, p1, p2, MRVectorZero);
        glTranslated( p1.x, p1.y, p1.z);
        glMultMatrixd (m.m);
        DrawCylinder(v, radius_c, gl->subdivisions);
        atm[j1].disegnato = 1;
        atm[j2].disegnato = 1;
        glPopMatrix();
    }

    glDisable(GL_COLOR_MATERIAL);
    glDisable (GL_BLEND);
    gl->SetLightForSelected(1);
}

void Model::Normalizza(GLWindow *gl)
{
  float pi2_8 = (float) (8*M_PI*M_PI);
  for(int j=0; j<3; j++)
      maxRad[j] = 0;
  for(int i=0; i<nats; i++)
  {
      int index = atm[i].color;
      if(index >=0)
      {
         atm[i].radN[0] = radii[index][0] / gl->deltav;
         atm[i].radN[1] = 2.f * radii[index][1] / gl->deltav;
//         atm[i].radN[1] = radii[index][1] / gl->deltav;
      }
      else
      {
         atm[i].radN[0] = 0.37f / gl->deltav;
         atm[i].radN[1] = 2.2f  / gl->deltav;
      }
      atm[i].radN[2] = (float) gl->cylinder;
//      if(MRabs(Bmax-Bmin) > 1e-6)
//     atm[i].radN[3] = 1.5f * (float) gl->cylinder * (1 + (atm[i].B - Bmin) /
//                   (Bmax - Bmin));
//      else
//         atm[i].radN[3] = atm[i].radN[0];
      float bf = MRabs(atm[i].B);
      if(bf < 0.01f)
         bf = 0.01f;
       atm[i].radN[3] = 3.5f * (float) (sqrt(bf/pi2_8))/gl->deltav;

      atm[i].seminN = atm[i].semin / gl->deltav;
      for(int j=0; j<4; j++)
      {
         if((atm[i].radN[j] / gl->cylinder) <= 1.1)
             atm[i].radN[j] = (float) (gl->cylinder * 1.15);
          maxRad[j] = MRmax(maxRad[j], atm[i].radN[j]);
      }
      vet3 a = gl->Trasforma(MRVectorMake(atm[i].xyz));
      for(int j=0; j<3; j++)
          atm[i].xyzN[j] = (float) a.v[j];
  }
}

void populate_listv(GtkTreeView *list, std::vector <char *> values)
{
    GtkListStore *list_store;
    GtkTreeIter iter;
    char *entry;
    int i, numval;

    list_store = GTK_LIST_STORE(gtk_tree_view_get_model(list));
    gtk_list_store_clear(GTK_LIST_STORE(list_store));
    numval = (int) values.size();
    for(i=0; i<numval; i++)
    {
        entry = values.at(i);
        if(entry && strlen(entry))
        {
           gtk_list_store_append (list_store, &iter);
           gtk_list_store_set (list_store, &iter, 0, entry,-1);
        }
    }
}

void Model::FillLists(GtkWidget *dlg)
{
     if(isPDB)
     {
        gtk_widget_hide(get_widget(dlg, "frame_noPDB"));
        gtk_widget_show(get_widget(dlg, "frame_PDB"));
        GtkWidget *list = get_widget(dlg, "Chains");
        PopulateModel(GTK_TREE_VIEW(list), 1);
        select_row(list, 0);
        GtkTreePath *path = gtk_tree_path_new_from_string("0:0");
        gtk_tree_selection_select_path(gtk_tree_view_get_selection(GTK_TREE_VIEW(list)), path);
     }
     else
     {
        gtk_widget_hide(get_widget(dlg, "frame_PDB"));
        gtk_widget_show(get_widget(dlg, "frame_noPDB"));
        GtkWidget *list = get_widget(dlg, "atoms_o");
        gtk_widget_set_size_request(list, -1, MRmin((nats+1)*altSaveN, 10*altSaveN));
        std::vector<int> atoms;
        for(int i=0; i<nats_au; i++)
            atoms.push_back(Equivs[i].at(0));
        std::vector<char *> listAtoms = ListaAtomi(atoms, dlg);
        populate_listv(GTK_TREE_VIEW(list), listAtoms);
     }
}

void Model::FillListaAtomi(GtkWidget *dlg, int nodi[])
{
     GtkWidget *list = get_widget(dlg, "atoms_PDB");
     int whichChain = nodi[0];
     int whichResidue = nodi[1];
 
     RESIDUE rr = GetResiduo(nModello, whichChain, whichResidue);
     std::vector<char *> listAtoms = ListaAtomi(rr.atoms, dlg);
     populate_listv(GTK_TREE_VIEW(list), listAtoms);
}

int Model::FindSpec(std::string &spec)
{
    std::string spect = strstrip(spec);
    for(int i=0; i<numSpec; i++)
    {
        std::string spec1 (LabSpec[i], LEN_SPEC);
        std::string spec1t = strstrip(spec1);
        if(g_ascii_strcasecmp(spect.c_str(), spec1t.c_str()) == 0)
//      std::string spec1 = get_nth_element(Species[i] - 1);
//      std::string spec1t = strstrip(spec1);
//      if(g_ascii_strcasecmp(spect.c_str(), spec1t.c_str()) == 0)
//           return (Species[i]);
           return (ptab[i]);
    }
    return 0;
}

f_bond_type *Model::FromIlmbondToLegm()
{
    f_bond_type *legm = g_new0(f_bond_type, nconn_prot);
    for(int i=0; i<nconn_prot; i++)
    {
        legm[i].n1 = bonds[i].n1;
        legm[i].n2 = bonds[i].n2;
        legm[i].dist = bonds[i].dist;
        legm[i].sigma = bonds[i].sigma;
    }
    return legm;
}  

f_bond_type *Model::FromListToLegm(GtkWidget *list)
{
    int nlegs = n_children(list);
    f_bond_type *legm = g_new0(f_bond_type, nlegs);

    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(list));
    GtkTreeIter iter;

    int val;
    gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
    while (valid)
    {
       gtk_tree_model_get(model, &iter, 0, &val, -1);
       if(val > nlegs)
          break;
       float distance;
       legm[val-1].n1 = bonds[val-1].n1;
       legm[val-1].n2 = bonds[val-1].n2;
       legm[val-1].sigma = bonds[val-1].sigma;
       gtk_tree_model_get(model, &iter, 4, &distance, -1);
       legm[val-1].dist = distance;
       valid = gtk_tree_model_iter_next (model, &iter);
    }
    return legm;
}

void Model::FromLegmToList(GtkWidget *list, f_bond_type *legm)
{
    int nlegs = n_children(list);

    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(list));
    GtkTreeIter iter;

    int val;
    gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
    while (valid)
    {
       gtk_tree_model_get(model, &iter, 0, &val, -1);
       if(val > nlegs)
          break;
       int n1 = legm[val-1].n1 - 1;
       int n2 = legm[val-1].n2 - 1;
       char *label1 = g_strdup(atm[n1].lab);
       char *label2 = g_strdup(atm[n2].lab);
       gtk_list_store_set(GTK_LIST_STORE(model),  &iter, 0, val,
                 2, g_strstrip(label1),
                 3, g_strstrip(label2), 4,  legm[val-1].dist, -1);
       valid = gtk_tree_model_iter_next (model, &iter);
    }
}

atom_type *Model::FromIlmatomToAtom()
{
    atom_type *atml = g_new0(atom_type, nats);
    for(int i=0; i<nats_au; i++)
    {
        int ieq = Equivs[i].at(0);
        for(int j=0; j<3; j++)
            atml[ieq].xc[j] = atm[ieq].xyzc[j];
        atml[ieq].biso = atm[ieq].B;
        atml[ieq].och = atm[ieq].och;
        atml[ieq].ocry = atm[ieq].ocry;
        atml[ieq].nz = Species[atm[ieq].color];
        int klun = (int) strlen(atm[ieq].lab);
        for(int j=0; j<klun; j++)
            atml[ieq].lab[j] = atm[ieq].lab[j];
        for(int j=klun; j<16; j++)
            atml[ieq].lab[j] = ' ';
    }
    return atml;
}

atom_type *Model::FromJavatomToAtom()
{
    atom_type *atml = g_new0(atom_type, nats_au);
    for(int i=0; i<nats_au; i++)
    {
        for(int j=0; j<3; j++)
            atml[i].xc[j] = atm[i].xyz[j];
        atml[i].biso = atm[i].B;
        atml[i].och = atm[i].och;
        atml[i].ocry = atm[i].ocry;
        atml[i].nz = Species[atm[i].color];
        int klun = (int) strlen(atm[i].lab);
        for(int j=0; j<klun; j++)
            atml[i].lab[j] = atm[i].lab[j];
        for(int j=klun; j<16; j++)
            atml[i].lab[j] = ' ';
    }
    return atml;
}

atom_type *Model::FromListToAtom(GtkWidget *list, int *numatoms)
{               
    int natoms = n_children(list);
    atom_type *atml = g_new0(atom_type, natoms);

    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(list));
    GtkTreeIter iter;

    int i = 0, val;
    float xyzc[3], B, och;
    gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
    while (valid)
    {
       gchar *new_text;
       gtk_tree_model_get(model, &iter, 0, &val, -1);
       gtk_tree_model_get(model, &iter, 1, &new_text, -1);
       int klun = (int) strlen(new_text);
       if(val > natoms)
          break;
       int ieq = val-1; //Equivs[val-1].at(0);
       for(int j=0; j<klun; j++)
           atml[ieq].lab[j] = new_text[j];
       for(int j=klun; j<16; j++)
           atml[ieq].lab[j] = ' ';
       g_free(new_text);
       gtk_tree_model_get(model, &iter, 2, &new_text, -1);
       std::string spec = new_text;
       g_free(new_text);
       //atml[ieq].nz = FindSpec(spec);
       atml[ieq].ptab = FindSpec(spec);
       //g_print("new species %s, ptab %d\n", spec.c_str(), atml[ieq].ptab);
       gtk_tree_model_get(model, &iter, 4, &xyzc[0], 5, &xyzc[1], 6,&xyzc[2], -1);
       for(int j=0; j<3; j++)
           atml[ieq].xc[j] = xyzc[j];
       gtk_tree_model_get(model, &iter, 7, &B, 8, &och, -1);
       atml[ieq].biso = B;
       atml[ieq].och = och;

       i++;
       valid = gtk_tree_model_iter_next (model, &iter);
    }
    *numatoms = i;
    return atml;
}

jav_anglebond_type *Model::FromListToAngs(GtkWidget *list)
{
    int nangs = n_children(list);
    jav_anglebond_type *angs = g_new0(jav_anglebond_type, nangs);

    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(list));
    GtkTreeIter iter;

    int val;
    gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
    while (valid)
    {
       gtk_tree_model_get(model, &iter, 0, &val, -1);
       if(val > nangs)
          break;
       float angolo;
       angs[val-1].n1 = anglebonds[val-1].n1;
       angs[val-1].n2 = anglebonds[val-1].n2;
       angs[val-1].n3 = anglebonds[val-1].n3;
       gtk_tree_model_get(model, &iter, 5, &angolo, -1);
       angs[val-1].fv = angolo;
       angs[val-1].sigma = anglebonds[val-1].sigma;
       valid = gtk_tree_model_iter_next (model, &iter);
    }
    return angs;
}

void Model::FromAngsToList(GtkWidget *list, jav_anglebond_type *angs)
{
    int nangs = n_children(list);

    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(list));
    GtkTreeIter iter;

    int val;
    gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
    while (valid)
    {
       gtk_tree_model_get(model, &iter, 0, &val, -1);
       if(val > nangs)
          break;
       int n1 = angs[val-1].n1 - 1;
       int n2 = angs[val-1].n2 - 1;
       int n3 = angs[val-1].n3 - 1;
       char *label1 = g_strdup(atm_au[n1].lab);
       char *label2 = g_strdup(atm_au[n2].lab);
       char *label3 = g_strdup(atm_au[n3].lab);
       gtk_list_store_set(GTK_LIST_STORE(model),  &iter, 0, val,
                 2, g_strstrip(label1), 3, g_strstrip(label2),
                 4, g_strstrip(label3), 5,  angs[val-1].fv, -1);
       valid = gtk_tree_model_iter_next (model, &iter);
    }
}

void Model::FillAngs(int nangles, jav_anglebond_type *angs)
{
  nangbonds = nangles;
  if(anglebonds)
     g_free(anglebonds);
  anglebonds = g_new0(jav_anglebond_type,  nangbonds);
  for(int i=1; i<=nangbonds; i++)
  {
      anglebonds[i-1].n1 = angs[i-1].n1;
      anglebonds[i-1].n2 = angs[i-1].n2;
      anglebonds[i-1].n3 = angs[i-1].n3;
      anglebonds[i-1].fv = angs[i-1].fv;
      anglebonds[i-1].sigma = angs[i-1].sigma;
  }
}

jav_torsion_type *Model::FromListToTors(GtkWidget *list)
{
    int nangs = n_children(list);
    jav_torsion_type *angs = g_new0(jav_torsion_type, nangs);

    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(list));
    GtkTreeIter iter;

    int val;
    gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
    while (valid)
    {
       gtk_tree_model_get(model, &iter, 0, &val, -1);
       if(val > nangs)
          break;
       float angolo;
       angs[val-1].n1 = torsions[val-1].n1;
       angs[val-1].n2 = torsions[val-1].n2;
       angs[val-1].n3 = torsions[val-1].n3;
       angs[val-1].n4 = torsions[val-1].n4;
       gtk_tree_model_get(model, &iter, 6, &angolo, -1);
       angs[val-1].fv = angolo;
       valid = gtk_tree_model_iter_next (model, &iter);
    }
    return angs;
}

void Model::FromTorsToList(GtkWidget *list, jav_torsion_type *angs)
{
    int nangs = n_children(list);

    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(list));
    GtkTreeIter iter;

    int val;
    gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
    while (valid)
    {
       gtk_tree_model_get(model, &iter, 0, &val, -1);
       if(val > nangs)
          break;
       int n1 = angs[val-1].n1 - 1;
       int n2 = angs[val-1].n2 - 1;
       int n3 = angs[val-1].n3 - 1;
       int n4 = angs[val-1].n4 - 1;
       char *label1 = g_strdup(atm_au[n1].lab);
       char *label2 = g_strdup(atm_au[n2].lab);
       char *label3 = g_strdup(atm_au[n3].lab);
       char *label4 = g_strdup(atm_au[n4].lab);
       gtk_list_store_set(GTK_LIST_STORE(model),  &iter, 0, val,
                 2, g_strstrip(label1), 3, g_strstrip(label2),
                 4, g_strstrip(label3), 5, g_strstrip(label4),
                 6, angs[val-1].fv, -1);
       valid = gtk_tree_model_iter_next (model, &iter);
    }
}

void Model::FillTors(int nangles, jav_torsion_type *angs)
{
  ntorsions = nangles;
  if(torsions)
     g_free(torsions);
  torsions = g_new0(jav_torsion_type,  ntorsions);
  for(int i=1; i<=ntorsions; i++)
  {
      torsions[i-1].n1 = angs[i-1].n1;
      torsions[i-1].n2 = angs[i-1].n2;
      torsions[i-1].n3 = angs[i-1].n3;
      torsions[i-1].n4 = angs[i-1].n4;
      torsions[i-1].fv = angs[i-1].fv;
  }
}

//GLWindow for models

void GLWindow::RiNormalizzasuModello(bool fResize)
{
     if(!nModels)
        return;
     int i = GetActiveModel() - 1;
     if(i == -1)
        i = 0;
//     for(int i=0;  i<nModels; i++)
     {
         if(ModelInfo[i].ifVis)
         {
            if(ModelInfo[i].nats > 1)
               NormalizzasuModello(i, fResize);
            else
               NormalizzasuCella();
            Normalized = 1;
//            break;
         }
     }
}

void GLWindow::DrawSelection()
{
  glDisable (GL_DEPTH_TEST);
  SetLightForSelected(1, NULL, 0.2f);
  glDepthMask (GL_FALSE); 
  glPushMatrix();
  glLoadIdentity();
  DrawPolygon(SelectionRect);
  glEnable (GL_DEPTH_TEST);
  glDepthMask (GL_TRUE); 
  ResetLightForSelected();
  glPopMatrix();
}

void GLWindow::DrawModels()
{
    for(int i=0; i<nModels;i++)
    {
        if (ModelInfo[i].ifVis)
        {
            ModelInfo[i].DrawModel(this);
            ModelInfo[i].DrawPeaks(this);
        }
#ifdef MAPS
        if(ModelInfo[i].ifSurface && ModelInfo[i].nats_au)
        {
           if(!ModelInfo[i].Surface_c)
           {
              ModelInfo[i].CreaMappa(i+1);
              RunMCAll();
           }
           DisableClip();
           DrawSurface(i);
           EnableClip();
        }
#endif
    }
    for(int i=0; i<nModels;i++)
    {
        if (ModelInfo[i].ifVis)
        {
            if(!Stereo)
            {
               glDisable(GL_FOG);
               if((ModelInfo[i].ifLabels) || (ModelInfo[i].specialLab != 0))
                  ModelInfo[i].DrawLabels();
               else
                  ModelInfo[i].DrawLabels(true);
               ModelInfo[i].DrawSelected(this);
               if(in_selection)
                  DrawSelection();
               if(fog && (BgType < TYPE_FLAT))
                  glEnable(GL_FOG);
            }
        }
    }
}

void GLWindow::NormalizzasuModello(int k, bool fResize)
{
  if(!fResize && !ModelInfo[k].ifResize)
     return;
  std::vector<double> tvet;
  std::vector<double> mini, maxi;
  int i, j;

  for(j=0; j<3; j++)
  {
      for(i=0; i<ModelInfo[k].nats; i++)
      {
          int l = ModelInfo[k].iEqui[i];
          if( ModelInfo[k].atm[i].hidden )
               continue;
          if( !ModelInfo[k].ifDrawH &&  ModelInfo[k].IsHydrogen(i) )
               continue;
          if( !ModelInfo[k].ifDrawW &&  ModelInfo[k].atm_au[l].isW )
               continue;
          tvet.push_back(ModelInfo[k].atm[i].xyz[j]);
      }

// corretto 291111 per zoom
// ricorretto 250515 
      bool include_cell = true;
      if(include_cell)
      {
         //if(ifCell && IsZero(Scale -1))
         if(ifCell)
         {
            for(int i=0; i<8; i++)
                tvet.push_back(vc[i].v[j]);
         }
      }
      if(tvet.empty())
         return;
      mini.push_back(*min_element(tvet.begin(), tvet.end())-ModelInfo[k].radMax/2);
      maxi.push_back(*max_element(tvet.begin(), tvet.end())+ModelInfo[k].radMax/2);
      tvet.clear();
  }
  vet3 oldc = FractToCart(xyzC); //fraz. --> orto
  vet3 baricentro = FractToCart(ModelInfo[k].baricentro);
  vet3 diff = MRVectorSub(oldc, baricentro);
  for(j=0; j<3; j++)
  {
     mini[j] = mini[j] + diff.v[j];
     maxi[j] = maxi[j] + diff.v[j];
  }

  for(j=0; j<3; j++)
      tvet.push_back(maxi[j]-mini[j]);
  double delta0 = *max_element(tvet.begin(), tvet.end());
  tvet.clear();
  deltav = (float) delta0;
  calcolaO(mini, maxi);
}

void GLWindow::NormalizzaModelli()
{
  for(int i=0;  i<nModels; i++)
      ModelInfo[i].Normalizza(this);
}

//Callbacks 

void block_handlers(int i, GtkWidget *father, GtkWidget *window)
{
    std::stringstream suffix;
    suffix << i;
    std::string name;

    name  = "vischeckmd" + suffix.str();
    GtkWidget *check = get_widget(father, name.c_str());
    int hnd = get_int(check, "hnd");
    g_signal_handler_block(G_OBJECT(check), hnd);
    name = "symcheck" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_block(G_OBJECT(check), hnd);
    name = "labelcheck"  + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_block(G_OBJECT(check), hnd);
    name = "hydrocheck"  + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_block(G_OBJECT(check), hnd);
    name = "watercheck"  + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_block(G_OBJECT(check), hnd);
    name = "level_0_" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_block(G_OBJECT(check), hnd);
    name = "level_1_" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_block(G_OBJECT(check), hnd);
    name = "level_2_" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_block(G_OBJECT(check), hnd);
    name = "FitOpt" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_block(G_OBJECT(check), hnd);
    name = "ps" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_block(G_OBJECT(check), hnd);
    name = "cm" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_block(G_OBJECT(check), hnd);
    name = suffix.str() + "styleLigand";
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_block(G_OBJECT(check), hnd);
#ifdef MAPS
    name  = "surfacecheck" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_block(G_OBJECT(check), hnd);
    name  = "surfacestyle" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_block(G_OBJECT(check), hnd);
    name = "surfaceradius" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_block (G_OBJECT (check), hnd);
    name = "surfacestep" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_block (G_OBJECT (check), hnd);

#endif
}

void unblock_handlers(int i, GtkWidget *father, GtkWidget *window)
{
    std::stringstream suffix;
    suffix << i;
    std::string name;

    name  = "vischeckmd" + suffix.str();
    GtkWidget *check = get_widget(father, name.c_str());
    int hnd = get_int(check, "hnd");
    g_signal_handler_unblock(G_OBJECT(check), hnd);
    name = "symcheck" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_unblock(G_OBJECT(check), hnd);
    name = "labelcheck"  + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_unblock(G_OBJECT(check), hnd);
    name = "hydrocheck"  + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_unblock(G_OBJECT(check), hnd);
    name = "watercheck"  + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_unblock(G_OBJECT(check), hnd);
    name = "level_0_" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_unblock(G_OBJECT(check), hnd);
    name = "level_1_" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_unblock(G_OBJECT(check), hnd);
    name = "level_2_" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_unblock(G_OBJECT(check), hnd);
    name = "FitOpt" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_unblock(G_OBJECT(check), hnd);
    name = "ps" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_unblock(G_OBJECT(check), hnd);
    name = "cm" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_unblock(G_OBJECT(check), hnd);
    name = suffix.str() + "styleLigand";
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_unblock(G_OBJECT(check), hnd);
#ifdef MAPS
    name  = "surfacecheck" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_unblock(G_OBJECT(check), hnd);
    name  = "surfacestyle" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_unblock(G_OBJECT(check), hnd);
    name = "surfaceradius" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_unblock (G_OBJECT (check), hnd);
    name = "surfacestep" + suffix.str();
    check = get_widget(father, name.c_str());
    hnd = get_int(check, "hnd");
    g_signal_handler_unblock (G_OBJECT (check), hnd);
#endif
}

void set_values_for_model(int i, GtkWidget *father, GtkWidget *window)
{
    std::stringstream suffix;
    suffix << i;
    std::string name, name1;

//    block_handlers(i, father, window);
    name  = "vischeckmd" + suffix.str();
    GtkWidget *check = get_widget(father, name.c_str());
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), ModelInfo[i].ifVis);
    name  = "model" + suffix.str();
    GtkWidget *box =  get_widget(father, name.c_str());
    gtk_widget_set_sensitive(box, ModelInfo[i].ifVis);
    name = "symcheck" + suffix.str();
    check = get_widget(father, name.c_str());
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), ModelInfo[i].ifSymmetry);
    name = "hydrocheck"  + suffix.str();
    check = get_widget(father, name.c_str());
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), ModelInfo[i].ifDrawH);
    name = "watercheck"  + suffix.str();
    check = get_widget(father, name.c_str());
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), ModelInfo[i].ifDrawW);
    name = "labelcheck"  + suffix.str();
    check = get_widget(father, name.c_str());
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), ModelInfo[i].ifLabels);

    name = "level_0_" + suffix.str();
    check = get_widget(father, name.c_str());
    spin_button_set_value(check, ModelInfo[i].klevel[0]);

    name = "level_1_" + suffix.str();
    check = get_widget(father, name.c_str());
    spin_button_set_value(check, ModelInfo[i].klevel[1]);

    name = "level_2_" + suffix.str();
    check = get_widget(father, name.c_str());
    spin_button_set_value(check, ModelInfo[i].klevel[2]);

    name1 = "FitOpt" + suffix.str();
    check = get_widget(father, name1.c_str());
    MRcombo_set_val(check, ModelInfo[i].FitOpt);

    float light[4];
    memcpy(light, ModelInfo[i].labelcolor, 4*sizeof(float));
    name = "labelcolor"  + suffix.str();
    GtkWidget *cb = get_widget(father, name.c_str());
    color_button_set_color(cb, light);
//    std::stringstream base;
//    base << i+1;
    name1 = "ps" + suffix.str();
    GtkWidget *w = get_widget(father, name1.c_str());
    check = w;
    MRcombo_set_val(check, ModelInfo[i].plotStyle);
    GtkTreeModelFilter *filter = (GtkTreeModelFilter *) get_object(w, "filter");
    gtk_tree_model_filter_refilter (filter);
    name1 = "cm" + suffix.str();
    check = get_widget(father, name1.c_str());
    filter = (GtkTreeModelFilter *) get_object(check, "filter");
    gtk_tree_model_filter_refilter (filter);
    if(ModelInfo[i].colorMode < 0)
      ModelInfo[i].colorMode = BY_SPECIES;
    MRcombo_set_val(check, ModelInfo[i].colorMode,true);
    memcpy(light, ModelInfo[i].light_diffuse[MONOCOL], 4*sizeof(float));
    name = "monocolor"  + suffix.str();
    cb = get_widget(father, name.c_str());
    color_button_set_color(cb, light);
    obj_set_visible(G_OBJECT(cb), ModelInfo[i].colorMode == MONOC);
    name1 = suffix.str() + "styleLigand";
    GtkWidget *combo = get_widget(father, name1.c_str());
    if(ModelInfo[i].drawLigand > 0)
       combo_set_val(combo, ModelInfo[i].drawLigand);
    name1 = suffix.str() + "drawLigand";
    check = get_widget(father, name1.c_str());
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), 
         (ModelInfo[i].drawLigand > 0));
//    unblock_handlers(i, father, window);
}

void aggiornaProperties(int whichModel)
{
      if(GLWINDOW !=NULL)
      {
         if(g_object_get_data(G_OBJECT(GLWINDOW), SettingsWindow))
          {
             GtkWidget *dlg1 = get_widget(GLWINDOW, SettingsWindow);
             if(whichModel == -1)
                rewrite_models(dlg1, GLWINDOW);
             else
                update_model(dlg1, GLWINDOW);
          }
      }
}

#ifdef GTK_COLOR_BUTTON
void labelColor(GtkWidget *cb, GParamSpec *pspec, gpointer data)
#else
void labelColor(GtkWidget *cb, GtkStyle *style, gpointer data)
#endif
{
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
   int whichModel = GPOINTER_TO_INT(data);
   float light[4];
   color_button_get_color(cb, light);
   memcpy(ModelInfo[whichModel].labelcolor, light, 4*sizeof(float));
   gl->Redraw ();
}

#ifdef GTK_COLOR_BUTTON
void monoColor(GtkWidget *cb, GParamSpec *pspec, gpointer data)
#else
void monoColor(GtkWidget *cb, GtkStyle *style, gpointer data)
#endif
{
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
   float light[4];
   int whichModel = GPOINTER_TO_INT(data);
   color_button_get_color(cb, light);
   if(whichModel == 0)
      main_app::JavApp.putfCol(MONOCHROME, light);
   memcpy(ModelInfo[whichModel].light_diffuse[MONOCOL], light, 4*sizeof(float));
   gl->Redraw ();
}

void lig_ps_changed(GtkWidget *combo, int whichStyle)
{
    GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
     int whichModel = get_int(combo, "whichModel");
     ModelInfo[whichModel].drawLigand = whichStyle+1;
     gl->Redraw ();
}

void sistema_elltr(GtkWidget *dlg)
{
    GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
    GtkWidget *frame = get_widget(dlg, "ell_frame");
    GtkWidget *frame1 = get_widget(dlg, "radtype_frame");
    if(frame == NULL)
       return;
    gtk_widget_set_sensitive(frame, FALSE);
    gtk_widget_set_sensitive(frame1, FALSE);
    for(int i=0; i<gl->nModels; i++)
    {
        if(ModelInfo[i].plotStyle == ELLIPSOIDS)
           gtk_widget_set_sensitive(frame, TRUE);
        if(ModelInfo[i].plotStyle == STICK_AND_BALLS)
           gtk_widget_set_sensitive(frame1, TRUE);
    }
}

void ps_changed(GtkWidget *combo, int whichStyle)
{
    GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
     int whichModel = get_int(combo, "whichModel");
     ModelInfo[whichModel].plotStyle = whichStyle;
//#ifndef JAV
//     defaultPlotStyle = whichStyle;
//#endif
     GtkWidget *frame = get_widget(combo, "frame_lig");
     if(whichStyle < POLYPEPTIDE)
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(frame), FALSE);
     gtk_widget_set_sensitive(frame, (whichStyle >= POLYPEPTIDE));
     frame = get_widget(combo, "combo_lig");
     gtk_widget_set_sensitive(frame, (whichStyle >= POLYPEPTIDE));
     sistema_elltr(get_widget(combo, "father"));
     gl->Redraw ();
}

void cm_changed(GtkWidget *combo, int whichcolorMode)
{
     GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
     int whichModel = get_int(combo, "whichModel");
     std::stringstream suffix;
     suffix << whichModel;
     std::string name = "monocolor"  + suffix.str();
     GtkWidget *but = get_widget(combo, name.c_str());
     obj_set_visible(G_OBJECT(but), (whichcolorMode == MONOC));
//     if(!ModelInfo[whichModel].ifSymmetry && (whichcolorMode >= BY_SYMMETRY))
//        ModelInfo[whichModel].colorMode = whichcolorMode+1;
//     else
        ModelInfo[whichModel].colorMode = whichcolorMode;
     gl->Redraw ();
}

void sym_changed(GtkToggleButton *button, gpointer data)
{
      GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
      int whichModel = GPOINTER_TO_INT(data);
      int symcheck = gtk_toggle_button_get_active(button);

      if((get_object(GLWINDOW, "SymmOn") != NULL) && (whichModel == 0))
      {
         gpointer check = get_object(GLWINDOW, "SymmOn");
         obj_set_active(check, symcheck, true);
      }
      int i = whichModel + 1;
      ModelInfo[whichModel].ifSymmetry = (symcheck != 0);
      aggiornaProperties(whichModel);
      aggiornaMenuSym();
      CApplicaSimmetria(&i);
      gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
      gl->Redraw ();
}

void labelvis_changed(GtkToggleButton *button, gpointer data)
{
      GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
      int whichModel = GPOINTER_TO_INT(data);
      int active = gtk_toggle_button_get_active(button);
      if((get_object(GLWINDOW, "LabelsOn") != NULL) && (whichModel == 0))
      {
         gpointer check = get_object(GLWINDOW, "LabelsOn");
         obj_set_active(check, active, true);
         sync_radio("Label All Atoms", !ModelInfo[whichModel].ifLabels);
      }
      if(whichModel == (GetActiveModel() -1))
         sync_radio("Label All Atoms", (active != 0));
      ModelInfo[whichModel].ifLabels = (active != 0);
      gl->Redraw ();
}

void vis_changed(GtkToggleButton *button, gpointer data)
{
      GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
      int whichModel = GPOINTER_TO_INT(data);
      int check = gtk_toggle_button_get_active(button);
      ModelInfo[whichModel].ifVis = (check != 0);
//260513      gl->RiNormalizza();
      gl->Redraw ();
}

void hyd_changed(GtkToggleButton *button, gpointer data)
{
      GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
      int whichModel = GPOINTER_TO_INT(data);
      int check = gtk_toggle_button_get_active(button);
      ModelInfo[whichModel].ifDrawH = (check != 0);
      gl->RiNormalizza();
      gl->Redraw ();
}

void wat_changed(GtkToggleButton *button, gpointer data)
{
      GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
      int whichModel = GPOINTER_TO_INT(data);
      int check = gtk_toggle_button_get_active(button);
      ModelInfo[whichModel].ifDrawW = (check != 0);
      gl->RiNormalizza();
      gl->Redraw ();
}

void getLigandStyle(GtkWidget *button, gpointer data)
{
      GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
      int whichModel = GPOINTER_TO_INT(data);
      if(!gtk_widget_get_sensitive(button))
      {
         ModelInfo[whichModel].drawLigand = -1;
         return;
      }
      int check = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
      GtkWidget *dlg = gtk_widget_get_toplevel(button);
      std::stringstream name;
      name << whichModel << "styleLigand";
      GtkWidget *combo = get_widget(dlg, name.str().c_str());
      if(check)
         ModelInfo[whichModel].drawLigand = (combo_get_val(combo) > 0)?
               (combo_get_val(combo)):0;
      else
         ModelInfo[whichModel].drawLigand = -1;
      gl->Redraw ();
}

void lig_changed(GtkToggleButton *button, gpointer data)
{
      getLigandStyle(GTK_WIDGET(button), data);
}

void level_changed(GtkWidget *button, gpointer data)
{
   double EPS = 1e-6;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
   GtkWidget *dlg = gtk_widget_get_toplevel(button);
   int num = GPOINTER_TO_INT(data);
   int whichModel = num % 10;
   int whichSpin  = num / 10;
   float val = (float) spin_button_get_value(button);
   if(MRabs(val - ModelInfo[whichModel].klevel[whichSpin]) < EPS)
      return;
   ModelInfo[whichModel].klevel[whichSpin] = val;
   GtkWidget *chainb = get_widget(dlg, "chainbutton_level");
   gboolean joined = mr_chain_button_get_active(MR_CHAIN_BUTTON(chainb));
   if(joined)
   {
      for (int k=0; k<3; k++)
      {
           if(k == whichSpin)
              continue;
           std::stringstream name1;
           name1 << "level_" << k << "_" << whichModel;
           GtkWidget *spin = get_widget(dlg, name1.str().c_str());
           int hnd = get_int(spin, "hnd");
           g_signal_handler_block(G_OBJECT(spin), hnd);
           spin_button_set_value(spin, val);
           g_signal_handler_unblock(G_OBJECT(spin), hnd);
           ModelInfo[whichModel].klevel[k] = val;
      }
   }

   if(ModelInfo[whichModel].ifSymmetry)
   {
      int j = whichModel + 1;
      gtk_widget_set_sensitive(button, FALSE);
      ShowWaitCursor(dlg);
      CApplicaSimmetria(&j);
      gtk_widget_set_sensitive(button, TRUE);
      HideWaitCursor(dlg);
      gl->Redraw ();
   }
}

void show_dialog(GtkWidget *window, GtkWidget *father)
{
  GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
  gtk_widget_show_all(father);
  for(int i=0; i<gl->nModels; i++)
  {
    if(!ModelInfo[i].created)
       continue;
//    std::stringstream base;
    std::stringstream suffix;
//    base << i;
    suffix << i;
    std::string name1 = "ps" + suffix.str();
    GtkWidget *w = get_widget(father, name1.c_str());
    GtkTreeModelFilter *filter = (GtkTreeModelFilter *) get_object(w, "filter");
    gtk_tree_model_filter_refilter (filter);
    name1 = "cm" + suffix.str();
    w = get_widget(father, name1.c_str());
    filter = (GtkTreeModelFilter *) get_object(w, "filter");
    gtk_tree_model_filter_refilter (filter);
    gboolean show_lig = ((ModelInfo[i].nconn_bb != 0) ||
                         (ModelInfo[i].nconn_ca != 0));
    std::string name3 = suffix.str() + "drawLigand";
    gtk_widget_set_visible(get_widget(father, name3.c_str()), show_lig);
    name3 = suffix.str() + "styleLigand";
    gtk_widget_set_visible(get_widget(father, name3.c_str()), show_lig);
    std::string name4 = "watercheck" + suffix.str();
    if(ModelInfo[i].IsProtein())
       gtk_widget_show(get_widget(father, name4.c_str()));
    else
       gtk_widget_hide(get_widget(father, name4.c_str()));
    block_handlers(i, father, window);
    set_values_for_model(i, father, window);
#ifdef MAPS
    set_values_for_surface(i, father, window, !ModelInfo[i].ifSymmetry);
#endif
    unblock_handlers(i, father, window);
  }
  GtkWidget *frame_sel = get_widget(father, "model_selector");
  if(frame_sel)
     gtk_widget_set_visible(frame_sel, (gl->nModels > 1));
  GList *listModels = NULL;
  for(int i=0; i<gl->nModels; i++)
      listModels = g_list_append(listModels,
            g_path_get_basename(ModelInfo[i].modname));
  if(gl->nModels)
      listModels = g_list_append(listModels, g_strdup("All"));
  GtkWidget *combo = get_widget(father, "model_combo");
  int whichModel = get_int(father, "whichModel") - 1;
  if(gl->nModels > 1)
  {
     block_handler(combo);
     set_popdown_strings(combo, listModels);
     combo_set_val(combo, whichModel+1);
     unblock_handler(combo);
  }
  if(whichModel != -1)
  {
     for(int i=0; i<gl->nModels; i++)
     {
         std::stringstream nome, nome1, nome2;
         nome << "modelname" << i;
         nome1 << "surface" << i;
         nome2 << "symmetry" << i;
         obj_set_visible(get_object(father, nome.str().c_str()),
             ((i == whichModel) || (whichModel == gl->nModels)));
         obj_set_visible(get_object(father, nome1.str().c_str()),
             ((i == whichModel) || (whichModel == gl->nModels)));
         obj_set_visible(get_object(father, nome2.str().c_str()),
             ((i == whichModel) || (whichModel == gl->nModels)));
     }
  }
  block_handlers_for_advanced();
  set_values_for_advanced();
  unblock_handlers_for_advanced();
}

#ifdef GTK_TREE_MODEL_FILTER
static gboolean ps_visible_func (GtkTreeModel *model,
              GtkTreeIter  *iter,
              gpointer      data)
{
  int whichModel = GPOINTER_TO_INT(data);
  gchar *str;
  gboolean visible = FALSE;

  gtk_tree_model_get (model, iter, 0, &str, -1);
  if((ModelInfo[whichModel].nconn_ca != 0) && ModelInfo[whichModel].hasSecondary)
  {
      if(ModelInfo[whichModel].nconn_bb != 0)
         visible = TRUE;
      else if(strcmp (str, ps_string[POLYPEPTIDE]))
         visible = TRUE;
  }
  else if(ModelInfo[whichModel].IsProtein())// && !strcmp (str, ps_string[CARTOONS]))
      visible = TRUE;
  else
  {
      for(int j=0; j<POLYPEPTIDE; j++)
      {
          if(strcmp (str, ps_string[j]) == 0)
          {
             visible = TRUE;
             break;
          }
      }
  }
  g_free (str);
  return visible;
}

static gboolean cm_visible_func (GtkTreeModel *model,
              GtkTreeIter  *iter,
              gpointer      data)
{
  int whichModel = GPOINTER_TO_INT(data);
  gchar *str;
  gboolean visible = FALSE;

  gtk_tree_model_get (model, iter, 0, &str, -1);
  if(ModelInfo[whichModel].nconn_ca == 0)
  {
      if( (strcmp (str, cm_string[BY_SPECIES]) == 0) ||
          (strcmp (str, cm_string[BY_SOF]) == 0) ||
          (strcmp (str, cm_string[BY_SYMMETRY]) == 0) ||
          (strcmp (str, cm_string[MONOC]) == 0) ||
          (strcmp (str, cm_string[BY_B]) == 0))
             visible = TRUE;
  }
  else
     visible = TRUE;
  g_free (str);
  return visible;
}
#endif

void pack_changed(GtkWidget *combo, int whichPack)
{
    if(whichPack < 0)
       return;
    GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
    GtkWidget *dlg = gtk_widget_get_toplevel(combo);
     int whichModel = get_int(combo, "whichModel");
     ModelInfo[whichModel].FitOpt = whichPack;
     if(ModelInfo[whichModel].ifSymmetry)
     {
        int j = whichModel + 1;
        gtk_widget_set_sensitive(combo, FALSE);
        ShowWaitCursor(dlg);
        CApplicaSimmetria(&j);
        gtk_widget_set_sensitive(combo, TRUE);
        HideWaitCursor(dlg);
        gl->Redraw ();
     }
}

void  add_symm_line(GtkWidget *parent, GtkWidget *father, int i, int row)
{
   static const char *labels[] = {"a", "b", "c"};
   static const char *pack_string[] = {"atoms that fit",
                                       "molecules whose centroids fit",
                                       "molecules where any atoms fits",
                                       "molecules where all atoms fit"};
   std::stringstream suffix;
   suffix << i;
   std::string name;

   name = "symmetry"  + suffix.str();
   GtkWidget *frame = NEW_FRAME(NULL, false);
   gtk_box_pack_start(GTK_BOX(parent), frame, TRUE, TRUE, 2);
   g_object_set_data(G_OBJECT(father), name.c_str(), frame);
   GtkWidget *box = vbox_new(FALSE, 0);
   add_to_frame (frame, box);
   GtkWidget *check = CHECK_BUTTON_WITH_LABEL("Packing", NULL);
   g_signal_connect(G_OBJECT(check), "toggled", G_CALLBACK(on_cb_attiva), box);
   gtk_frame_set_label_widget(GTK_FRAME(frame), check);
   name = "symcheck" + suffix.str();
   g_object_set_data(G_OBJECT(father), name.c_str(), check);
   int hnd = g_signal_connect_after(G_OBJECT(check), "toggled", G_CALLBACK(sym_changed), GINT_TO_POINTER(i));
   g_object_set_data(G_OBJECT(check), "hnd", GINT_TO_POINTER(hnd));

   GtkWidget *table = table_new(2, 5, FALSE);
   table_set_row_spacings(table, 16);
#ifdef IN_GTK3
   table_set_col_spacings(table, 16);
#else
   table_set_col_spacings(table, 2);
#endif
   gtk_box_pack_start(GTK_BOX(box), table, FALSE, FALSE, 2);
   gtk_container_set_border_width(GTK_CONTAINER(table), 0);
   gtk_widget_set_sensitive(box, FALSE);

   GtkWidget *label = gtk_label_new("Range ");
   set_alignment(label, 0., 0.5);
   table_attach_exp(table, label, 0, 1, row, row+1);
   for(int k=0; k<3; k++)
   {
       std::string labspin = g_strdup_printf("%s:", labels[k]);
       GtkWidget *level_spin = SPIN_WITH_LABEL(labspin.c_str(), MyBLUE, 0,
            MAX_SYMLEVEL, 0.1, 0.1, 1);
       std::stringstream name1;
       name1 << "level_" << k << "_" << suffix.str();
       g_object_set_data(G_OBJECT(father), name1.str().c_str(), 
           get_widget(level_spin, "spin"));
       set_width_chars(GTK_ENTRY(get_widget(level_spin, "spin")), 3);
       table_attach_exp(table, level_spin, 1+k, 2+k, row, row+1);
       hnd = get_int(get_widget(level_spin, "spin"), "hnd");
       if(!hnd)
           hnd = g_signal_connect(G_OBJECT(get_widget(level_spin, "spin")),
          "value-changed", G_CALLBACK(level_changed), GINT_TO_POINTER(k*10+i));
       g_object_set_data(G_OBJECT(get_widget(level_spin, "spin")), "hnd",
          GINT_TO_POINTER(hnd));
   }
   GtkWidget *button = mr_chain_button_new();
   gtk_widget_show(button);
   table_attach(table, button, 4, 5, row, row+1);
   g_object_set_data(G_OBJECT(father), "chainbutton_level", button);
   row++;
   label = gtk_label_new("Include: ");
   set_alignment(label, 0., 0.5);
   table_attach_exp(table, label, 0, 1, row, row+1);
   GtkWidget *comboPack = MRcombo_new(pack_string, pack_changed, 1, 4);
   name = "FitOpt"  + suffix.str();
   g_object_set_data(G_OBJECT(father), name.c_str(), comboPack);
   g_object_set_data(G_OBJECT(comboPack), "father", father);
   g_object_set_data(G_OBJECT(comboPack), "whichModel", GINT_TO_POINTER(i));
   table_attach_exp(table, comboPack, 1, 4, row, row+1);
}

void add_model(GtkWidget *window, int i, GtkWidget *father, GtkWidget *parent)
{
    GtkWidget *check, *label;
    std::stringstream nome;
    nome << "modelname" << i;

    GtkWidget *frame = NEW_FRAME(NULL, false);
    gtk_box_pack_start(GTK_BOX(parent), frame, TRUE, TRUE, 2);
    g_object_set_data(G_OBJECT(father), nome.str().c_str(), frame);
    GtkWidget *box = vbox_new(FALSE, 0);
    gtk_container_add (GTK_CONTAINER (get_widget(frame, "align")), box);

    check = CHECK_BUTTON_WITH_LABEL(
          g_strdup_printf("Visualize Model n. %d: %s", i+1,
             g_path_get_basename(ModelInfo[i].modname)), MyBLUE, 0);
    g_signal_connect(G_OBJECT(check), "toggled", G_CALLBACK(on_cb_attiva), box);
    gtk_frame_set_label_widget(GTK_FRAME(frame), check);

    std::stringstream suffix;
    suffix << i;
    std::string name;
    name = "vischeckmd" + suffix.str();
    g_object_set_data(G_OBJECT(father), name.c_str(), check);
    int hnd = g_signal_connect_after(G_OBJECT(check), "toggled", G_CALLBACK(vis_changed), GINT_TO_POINTER(i));
    g_object_set_data(G_OBJECT(check), "hnd", GINT_TO_POINTER(hnd));

    name = "model" + suffix.str();
    g_object_set_data(G_OBJECT(father), name.c_str(), box);
    int nrows;
#ifdef PROTEINS
    nrows = 6;
#else
    nrows = 4;
#endif
    GtkWidget *table = table_new(nrows, 2, FALSE);
    gtk_box_pack_start(GTK_BOX(box), table, FALSE, FALSE, 2);
    gtk_container_set_border_width(GTK_CONTAINER(table), 0);
    table_set_row_spacings(table, 16);
#ifdef IN_GTK3
    table_set_col_spacings(table, 16);
#else
    table_set_col_spacings(table, 2);
#endif

    int row = 0;

    frame = LABEL_NEW("Plot Style: ", MyBLUE);
    table_attach_exp(table, frame, 0, 1, row, row+1);
    set_alignment(frame, 0., 0.5);
    if((ModelInfo[i].plotStyle >= POLYPEPTIDE) &&
         (ModelInfo[i].nconn_prot == 0) && (ModelInfo[i].nconn_ca != 0))
       ModelInfo[i].plotStyle = C_ALPHA;
    GtkWidget *comboPS = MRcombo_new(ps_string, ps_changed,
        ModelInfo[i].plotStyle, PLOT_STYLES);
    g_object_set_data(G_OBJECT(comboPS), "father", father);
    table_attach_exp(table, comboPS, 1, 2, row, row+1);
    g_object_set_data(G_OBJECT(comboPS), "whichModel", GINT_TO_POINTER(i));
//    std::stringstream base;
//    base << i+1;
    std::string name3 = "ps" + suffix.str();
    g_object_set_data(G_OBJECT(father), name3.c_str(), comboPS);
#ifdef GTK_TREE_MODEL_FILTER
   GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(comboPS));
   GtkTreeModel *model1 = gtk_tree_model_filter_new(model, NULL);
   gtk_tree_model_filter_set_visible_func ((GtkTreeModelFilter *)model1, ps_visible_func, GINT_TO_POINTER(i), NULL);
   gtk_combo_box_set_model (GTK_COMBO_BOX(comboPS), NULL);
   gtk_combo_box_set_model (GTK_COMBO_BOX(comboPS), model1);
   g_object_set_data(G_OBJECT(comboPS), "filter", (gpointer) model1);
#endif
   row++;

   frame = LABEL_NEW("Colours setting: ", MyBLUE);
   set_alignment(frame, 0., 0.5);
   table_attach_exp(table, frame, 0, 1, row, row+1);
   GtkWidget *comboCM = MRcombo_new(cm_string, cm_changed,
        ModelInfo[i].colorMode, COLOR_MODES);
   table_attach_exp(table, comboCM, 1, 2, row, row+1);
   g_object_set_data(G_OBJECT(comboCM), "whichModel", GINT_TO_POINTER(i));
   name3 = "cm" + suffix.str();
   g_object_set_data(G_OBJECT(father), name3.c_str(), comboCM);
#ifdef GTK_TREE_MODEL_FILTER
   model = gtk_combo_box_get_model(GTK_COMBO_BOX(comboCM));
   GtkTreeModel *model2 = gtk_tree_model_filter_new(model, NULL);
   gtk_tree_model_filter_set_visible_func ((GtkTreeModelFilter *)model2, cm_visible_func, GINT_TO_POINTER(i), NULL);
   gtk_combo_box_set_model (GTK_COMBO_BOX(comboCM), NULL);
   gtk_combo_box_set_model (GTK_COMBO_BOX(comboCM), model2);
   g_object_set_data(G_OBJECT(comboCM), "filter", (gpointer) model2);
#endif
   const float colore[] = {1, 1, 1, 1};
   GtkWidget *cb = color_button(colore, "Pick a color to be used for monochrome drawing");
   GtkWidget *hb = hbox_new(FALSE, 0);
   table_attach_exp(table, hb, 2, 3, row, row+1);
   gtk_box_pack_start(GTK_BOX(hb), cb, FALSE, FALSE, 0);
   name = "monocolor"  + suffix.str();
   g_object_set_data(G_OBJECT(father), name.c_str(), cb);
   g_object_set_data(G_OBJECT(comboCM), name.c_str(), cb);
#ifdef GTK_COLOR_BUTTON
   g_signal_connect(G_OBJECT(cb), "notify::color", G_CALLBACK(monoColor), GINT_TO_POINTER(i));
#else
   g_signal_connect(G_OBJECT(cb), COLOR_CHANGED, G_CALLBACK(monoColor), GINT_TO_POINTER(i));
#endif
   row++;

   label = CHECK_BUTTON_WITH_LABEL("Draw ligand(s)", MyBLUE);
   gtk_widget_set_sensitive(label, ModelInfo[i].plotStyle >= POLYPEPTIDE);
#ifdef PROTEINS
   table_attach_exp(table, label, 0, 1, row, row+1);
#endif
   g_signal_connect_after(G_OBJECT(label), "toggled", G_CALLBACK(lig_changed), GINT_TO_POINTER(i));
   name3 = suffix.str() + "drawLigand";
   g_object_set_data(G_OBJECT(father), name3.c_str(), label);
   GtkWidget *combo_lig = MRcombo_new(ps_string, lig_ps_changed,
                            ModelInfo[i].drawLigand+1, VANDERWAALS-1);
#ifdef PROTEINS
   table_attach_exp(table, combo_lig, 1, 2, row, row+1);
   row++;
#endif
   g_object_set_data(G_OBJECT(combo_lig), "whichModel", GINT_TO_POINTER(i));
   gtk_widget_set_sensitive(combo_lig, FALSE);
   g_signal_connect(G_OBJECT(label), "toggled",
                   G_CALLBACK(on_cb_attiva), G_OBJECT(combo_lig));
   name3 = suffix.str() + "styleLigand";
   g_object_set_data(G_OBJECT(father), name3.c_str(), combo_lig);
   name3 = "ps" + suffix.str();
   g_object_set_data(G_OBJECT(get_widget(father, name3.c_str())),
         "frame_lig", label);
   g_object_set_data(G_OBJECT(get_widget(father, name3.c_str())),
         "combo_lig", combo_lig);

   check = CHECK_BUTTON_WITH_LABEL ("Draw _Labels    ", MyBLUE);
   table_attach_exp(table, check, 0, 1, row, row+1);
   name = "labelcheck"  + suffix.str();
   g_object_set_data(G_OBJECT(father), name.c_str(), check);
   hnd = g_signal_connect(G_OBJECT(check), "toggled", G_CALLBACK(labelvis_changed), GINT_TO_POINTER(i));
   g_object_set_data(G_OBJECT(check), "hnd", GINT_TO_POINTER(hnd));
//   const float colore[] = {1, 1, 1, 1};
   cb = color_button(colore, "Pick a color to be used for draw labels");
   hb = hbox_new(FALSE, 0);
   table_attach_exp(table, hb, 1, 2, row, row+1);
   gtk_box_pack_start(GTK_BOX(hb), cb, FALSE, FALSE, 0);
   name = "labelcolor"  + suffix.str();
   g_object_set_data(G_OBJECT(father), name.c_str(), cb);
#ifdef GTK_COLOR_BUTTON
   g_signal_connect(G_OBJECT(cb), "notify::color", G_CALLBACK(labelColor), GINT_TO_POINTER(i));
#else
   g_signal_connect(G_OBJECT(cb), COLOR_CHANGED, G_CALLBACK(labelColor), GINT_TO_POINTER(i));
#endif
   row++;

   check = CHECK_BUTTON_WITH_LABEL ("Draw _Hydrogens", MyBLUE);
   table_attach_exp(table, check, 0, 1, row, row+1);
   name = "hydrocheck"  + suffix.str();
   g_object_set_data(G_OBJECT(father), name.c_str(), check);
   hnd = g_signal_connect_after(G_OBJECT(check), "toggled", G_CALLBACK(hyd_changed), GINT_TO_POINTER(i));
   g_object_set_data(G_OBJECT(check), "hnd", GINT_TO_POINTER(hnd));
   row++;

   check = CHECK_BUTTON_WITH_LABEL ("Draw _Water Mol.", MyBLUE);
#ifdef PROTEINS
   table_attach_exp(table, check, 0, 1, row, row+1);
   row++;
#endif
   name = "watercheck"  + suffix.str();
   g_object_set_data(G_OBJECT(father), name.c_str(), check);
   hnd = g_signal_connect_after(G_OBJECT(check), "toggled", G_CALLBACK(wat_changed), GINT_TO_POINTER(i));
   g_object_set_data(G_OBJECT(check), "hnd", GINT_TO_POINTER(hnd));

   add_symm_line(parent, father, i, 0);
   row++;

#ifdef MAPS
   add_surface_line(parent, father, i, 0);
#endif
}

static void selected_model(GtkWidget *combo, gpointer )
{
    GtkWidget *dlg = gtk_widget_get_toplevel(combo);
    int whichModel = combo_get_val(combo);
    if(whichModel > 0)
    {
       g_object_set_data(G_OBJECT(dlg), "whichModel", GINT_TO_POINTER(whichModel));
       show_dialog(GLWINDOW, dlg);
    }
}

GtkWidget *add_model_selector(GtkWidget *dlg)
{
   GtkWidget *hbox = hbox_new(FALSE, 0);
   GtkWidget *frame = LABEL_NEW("Select model", MyBLUE);
   g_object_set_data(G_OBJECT(dlg), "model_selector", hbox);
   gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE,FALSE, 0);

   GtkWidget *combo = create_combo (NULL, NULL, 0, 0, NULL, G_CALLBACK(selected_model));
   g_object_set_data(G_OBJECT(dlg), "model_combo", combo);
   gtk_editable_set_editable(GTK_EDITABLE(get_widget(combo, "entry")), FALSE);
   gtk_box_pack_start(GTK_BOX(hbox), get_widget(combo, "parent"), FALSE,FALSE, 10);
   gtk_container_set_border_width(GTK_CONTAINER(get_widget(combo, "parent")), 4);
   g_object_set_data(G_OBJECT(dlg), "whichModel", GINT_TO_POINTER(1));
   return hbox;
}

void model_settings(GtkWidget *window, GtkWidget *dlg, GtkWidget *vbox)
{
   int i;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   GList *listModels = NULL;
   GtkWidget *frame;
   for(int i=0; i<gl->nModels; i++)
       listModels = g_list_append(listModels,
              g_path_get_basename(ModelInfo[i].modname));
   if(gl->nModels)
       listModels = g_list_append(listModels, g_strdup("All"));

   GtkWidget *vbox1 = vbox_new(FALSE, 0);
   frame = NEW_FRAME(NULL);
   gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
   gtk_container_add(GTK_CONTAINER(get_widget(frame,"align")), vbox1);
   g_object_set_data(G_OBJECT(dlg), "frame_models", vbox1);
   gtk_container_set_border_width (GTK_CONTAINER (vbox1), 0);
   frame = add_model_selector(dlg);
   gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 4);

   for(i=0; i<gl->nModels; i++)
       add_model(window, i, dlg, vbox1);
   show_dialog(window, dlg);
}

void Model::CalcolaNormali()
{
   if(nPoly == 0)
      return;
   std::vector<_VertVett> vertvett;
   for (int i=0; i<nPoly; i++)
   {
        int nf = NumberOfFaces(Polyhedra[i]);
        vertvett.clear();
        int found[NMAX_FACE_POINTS] = {-1, -1, -1, -1};
        vet3 *nv = g_new0(vet3, nf);
        for(int j=0; j<nf; j++)
        {
           int nvf = Polyhedra[i].face[j].n;
           int *nn = g_new0(int, nvf);
           for(int l=0; l<nvf; l++)
               nn[l] = Polyhedra[i].face[j].fp[l];
           for(int ifound=0; ifound <NMAX_FACE_POINTS; ifound++)
           {
               for(size_t kk=0; kk<vertvett.size(); kk++)
               {
                   if(vertvett.at(kk).vertex == nn[ifound])
                      found[ifound] = (int) kk;
               }
               if(found[ifound] == -1)
               {
                  _VertVett v = {nn[ifound], {j, -1, -1, -1}};
                  vertvett.push_back(v);
               }
               else
               {
                   for(int ff=0; ff<4; ff++)
                   {
                       if(vertvett.at(found[ifound]).faces[ff] == -1)
                        {
                          vertvett.at(found[ifound]).faces[ff] = j;
                          break;
                       }
                   }
               }
           }
           vet3 a, b, c;
           for(int kk=0; kk<3; kk++)
           {
               a.v[kk] = (double)atm[Polyhedra[i].face[j].fp[0]].xyzN[kk];
               b.v[kk] = (double)atm[Polyhedra[i].face[j].fp[1]].xyzN[kk];
               c.v[kk] = (double)atm[Polyhedra[i].face[j].fp[2]].xyzN[kk];
           }
           CalcolaNormale(a, b, c, &(nv[j]));
           g_free(nn);
        }
        for(size_t kk=0; kk<vertvett.size(); kk++)
        {
            int nfc =(nf == 4)? 3:4;
            vet3 *v = g_new0(vet3, nfc);
            vet3 *vij = g_new0(vet3, nfc);
            for(int j=0; j<nfc;j++)
            {
                int jj = vertvett.at(kk).faces[j];
                MRVectorCopy(nv[jj], &v[j]);
            }
            for(int j=0; j<nfc;j++)
            {
                int jj = (j+1) % nfc;
                vij[j] = MRVectorCrossProduct(v[j], v[jj]);
                vij[j] = MRVectorNormalize(v[j]);
            }
            vet3 sumvet = {{0, 0, 0}};
            for(int j=0; j<nfc;j++)
                sumvet = MRVectorAdd(sumvet, vij[j]);
            sumvet = MRVectorNormalize(sumvet);
            memcpy(atm[vertvett.at(kk).vertex].vn, sumvet.v, 3*sizeof(float));
            g_free(v);
            g_free(vij);
        }
        g_free(nv);
   }
}

void Model::FillPolyhedra(int npoly, polyhedron_type *poly)
{
   if(Polyhedra != NULL)
      g_free(Polyhedra);
   Polyhedra = NULL;
   nPoly = npoly;
   if(nPoly == 0)
      return;
   Polyhedra = g_new0(polyhedron_type, nPoly);
   for (int i=0; i<nPoly; i++)
   {
        Polyhedra[i].nvert = poly[i].nvert;
        Polyhedra[i].xc = poly[i].xc-1;
//        g_print("Poliedro n.%d vertici %d\n", i+1, Polyhedra[i].nvert);
        int nf = NumberOfFaces(Polyhedra[i]);
        for(int j=0; j<nf; j++)
        {
           Polyhedra[i].face[j].n =  poly[i].face[j].n;
//           g_print("Poliedro n.%d faccia %d vertici %d\n", i+1, j+1, Polyhedra[i].face[j].n);
           for(int k=0; k<Polyhedra[i].face[j].n; k++)
               Polyhedra[i].face[j].fp[k] = poly[i].face[j].fp[k]-1;
        }
   }
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
   if(gl->normalToV)
      CalcolaNormali();
}

void Model::FillPolyhedraStyles()
{
   if(nPoly == 0)
      return;
   std::vector<polyhedron_style> PS;
   std::vector<polyhedron_style>::iterator it;
   PS.clear();
   for (int i=0; i<nPolyStyles; i++)
        PS.push_back(PolyStyle[i]);
   
   for (int i=0; i<nPoly; i++)
   {
        int vert = iEqui[Polyhedra[i].xc];
        bool found=false;
        for(it=PS.begin() ; it < PS.end(); it++ )
        {
            if((*it).vert == vert)
            {
               found = true;
               break;
            }
        }
        if(!found)       // nuovo poliedro
        {
           polyhedron_style polstyl;
           polstyl.visible = true;
           polstyl.transparency = 0;
           polstyl.ifBorder = true;
           int index1 = atm_au[vert].Z-1;
           memcpy(polstyl.colore, light_diffuse[index1], 4*sizeof(float));
           memcpy(polstyl.Bcolore, light_diffuse[index1], 4*sizeof(float));
           TuneHSV( polstyl.Bcolore, 0.2f);
           polstyl.line_width = 1.5f;
           polstyl.vert = vert;
           PS.push_back(polstyl);
        }
   }
   for(it=PS.begin() ; it < PS.end(); it++ )
   {
        bool found=false;
        int vert = (*it).vert;
        for (int i=0; i<nPoly; i++)
        {
             int vert1 = iEqui[Polyhedra[i].xc];
             if(vert1 == vert)
             {
                found = true;
                break;
             }
        }
        if(!found)          // poliedro scomparso
           PS.erase(it);
   }
   nPolyStyles = (int) PS.size();
   if(PolyStyle != NULL)
   {           
       g_free(PolyStyle);
       PolyStyle = NULL;
   }    
   PolyStyle = g_new0(polyhedron_style, nPolyStyles);
   for(int i=0; i<nPolyStyles; i++)
       memcpy(&PolyStyle[i], &PS.at(i), sizeof(polyhedron_style));
   PS.clear();
}

static const int posi[N_TRUE_ELEMENTS][2] = {
{0, 0}, {0,17},
{1, 0}, {1, 1}, {1, 12}, {1, 13}, {1, 14}, {1, 15}, {1, 16}, {1, 17},
{2, 0}, {2, 1}, {2, 12}, {2, 13}, {2, 14}, {2, 15}, {2, 16}, {2, 17},
{3, 0}, {3, 1}, {3, 2}, {3, 3}, {3, 4}, {3, 5}, {3, 6}, {3, 7}, {3, 8},
{3, 9}, {3,10}, {3,11}, {3,12}, {3,13}, {3,14}, {3,15}, {3,16}, {3,17},
{4, 0}, {4, 1}, {4, 2}, {4, 3}, {4, 4}, {4, 5}, {4, 6}, {4, 7}, {4, 8},
{4, 9}, {4,10}, {4,11}, {4,12}, {4,13}, {4,14}, {4,15}, {4,16}, {4,17},
{5, 0}, {5, 1}, 
{7, 3}, {7, 4}, {7, 5}, {7, 6}, {7, 7}, {7, 8},
{7, 9}, {7,10}, {7,11}, {7,12}, {7,13}, {7,14}, {7,15}, {7,16}, {7,17},
{5, 3}, {5, 4}, {5, 5}, {5, 6}, {5, 7}, {5, 8},
{5, 9}, {5,10}, {5,11}, {5,12}, {5,13}, {5,14}, {5,15}, {5,16}, {5,17},
{6, 0}, {6, 1}, 
{8, 3}, {8, 4}, {8, 5}, {8, 6}, {8, 7}, {8, 8},
{8, 9}, {8,10}, {8,11}, {8,12}, {8,13}, {8,14}, {8,15}, {8,16}, {8,17},
{6, 3}, {6, 4}, {6, 5}, {6, 6}, {6, 7}, {6, 8}, {6, 9}, {6,10}};
 
void elem_sel(GtkWidget *but, gpointer data)
{
    int response = GPOINTER_TO_INT(data);
    gtk_dialog_response(GTK_DIALOG(gtk_widget_get_toplevel(but)), response);
}

gboolean event_elem_sel(GtkWidget *but, GdkEvent *, gpointer data)
{
    elem_sel(but, data);
    return TRUE;
}

int select_from_table(GtkWidget *father)
{
    GtkWidget *dlg = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(dlg), "Select new species");
    gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(father));

    GtkWidget *vbox = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
    GtkWidget *table2 = table_new(9, 18, TRUE);
    gtk_container_add(GTK_CONTAINER(vbox), table2);
    table_set_col_spacings(table2, 0);
    table_set_row_spacings(table2, 0);
    for(int k=0; k<N_TRUE_ELEMENTS; k++)
    {
        std::string elem;
        elem.assign(get_nth_element(k));
        std::stringstream nome;
        nome << k;
        float *colore = get_elem_color(k);
        GtkWidget *cb = mycolor_button(colore, " ",
        G_CALLBACK(event_elem_sel),  GINT_TO_POINTER(k), elem.c_str());

//        GtkWidget *cb = gtk_button_new_with_label(elem.c_str());
//        g_signal_connect(G_OBJECT(cb), "clicked", G_CALLBACK(elem_sel), GINT_TO_POINTER(k));
        table_attach(table2, cb, posi[k][1], posi[k][1]+1, posi[k][0], posi[k][0] + 1, TRUE);
        gtk_container_set_border_width(GTK_CONTAINER(cb), 2);
    }
    gtk_widget_show_all(dlg);
    int result = gtk_dialog_run(GTK_DIALOG(dlg));
    gtk_widget_destroy(dlg);
    if(result < 0)
       return -1;
    return result;
}
  
#ifdef IN_GTK3
gboolean userColor_changed(GtkWidget *cb, gpointer data)
#else
gboolean userColor_changed(GtkWidget *cb, GtkStyle *style, gpointer data)
#endif
{
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
   int k = GPOINTER_TO_INT(data);
   float light[4];
   mycolor_button_get_color(cb, light);
   for(int i=0; i<gl->nModels; i++)
       memcpy(ModelInfo[i].light_diffuse[k], light, 4*sizeof(float));
   memcpy(main_app::JavApp.app_color[k+N_OBJS], light, 4*sizeof(float));
   gl->Redraw();
   return FALSE;
}

void ok_color_table(GtkWidget *button, gpointer data)
{
    main_app::JavApp.WritePalette();
    chiudi_dlg(button, data);
}

void defaults_color_table(GtkWidget *button, gpointer data)
{
    main_app::JavApp.ResettaPalette();
    main_app::JavApp.WritePalette();
    GtkWidget *dlg = gtk_widget_get_toplevel(button);
    for(int k=0; k<NUM_COLORS; k++)
    {
        std::stringstream nome;
        nome << k;
        float light[4];
        memcpy(light, main_app::JavApp.app_color[k+N_OBJS], 4*sizeof(float));
        GtkWidget *cb = get_widget(dlg, nome.str().c_str());
        if(cb != NULL)
           mycolor_button_set_color(cb, light);
    }
}

void close_color_table(GtkWidget *button, gpointer data)
{
// restore previous values
    for(int k=0; k<NUM_COLORS;k++)
        memcpy(main_app::JavApp.app_color[k+N_OBJS], icolorSave[k], 4*sizeof(float));
    chiudi_dlg(button, data);
}

GtkWidget *create_colortable(gpointer data)
{
    GtkWidget *window = (GtkWidget *) data;
    chiudi_dlg_data *st = g_new0(chiudi_dlg_data, 1);
    st->window = window;
    st->name = "colortable";
    for(int k=0; k<NUM_COLORS;k++)
        memcpy(icolorSave[k], main_app::JavApp.app_color[k+N_OBJS], 4*sizeof(float));

    GtkWidget *dlg = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(dlg), "Colours Preferences");
    GtkWidget *scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
    gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG(dlg))), scrolledwindow1, TRUE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (scrolledwindow1), 4);
    gtk_widget_set_size_request(scrolledwindow1, -1, 300);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1),
        GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

    GtkWidget *vbox = vbox_new(FALSE, 0);
    add_to_scrollwin (scrolledwindow1, vbox);
    GtkWidget *frame = NEW_FRAME("Elements");
    gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
    GtkWidget *table2 = table_new(9, 18, TRUE);
    gtk_container_add(GTK_CONTAINER(get_widget(frame, "align")), table2);
    table_set_col_spacings(table2, 0);
    table_set_row_spacings(table2, 0);
    for(int k=0; k<N_TRUE_ELEMENTS; k++)
    {
        std::string elem;
        elem.assign(get_nth_element(k));
        std::stringstream nome;
        nome << k;
        float light[4];
        memcpy(light, main_app::JavApp.app_color[k+N_OBJS], 4*sizeof(float));
        GtkWidget *cb = mycolor_button(light, "Pick a color to be used for draw atoms");
        g_signal_connect(G_OBJECT(cb), COLOR_CHANGED, G_CALLBACK(userColor_changed), GINT_TO_POINTER(k));
        g_object_set_data(G_OBJECT(dlg), nome.str().c_str(), cb);
        GtkWidget *label = LABEL_NEW(elem.c_str(), MyBLUE);
        GtkWidget *vb = vbox_new(FALSE, 0);
        gtk_box_pack_start(GTK_BOX(vb), label, FALSE, FALSE, 0);
        gtk_box_pack_start(GTK_BOX(vb), cb, FALSE, FALSE, 0);
        table_attach(table2, vb, posi[k][1], posi[k][1]+1, posi[k][0], posi[k][0] + 1, TRUE);
        gtk_container_set_border_width(GTK_CONTAINER(cb), 2);
    }
    frame = NEW_FRAME("Amino");
    gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
    GtkWidget *table1 = table_new(2, 10, TRUE);
    gtk_container_add(GTK_CONTAINER(get_widget(frame, "align")), table1);
    table_set_col_spacings(table1, 0);
    table_set_row_spacings(table1, 0);
    for(int k=0; k<N_AMINO; k++)
    {
        std::string elem = get_nth_amino(k);
        std::stringstream nome;
        int kk = k+N_ELEMENTS;
        nome << kk;
        float light[4];
        memcpy(light, main_app::JavApp.app_color[kk+N_OBJS], 4*sizeof(float));
        GtkWidget *cb = mycolor_button(light, "Pick a color to be used for draw amino acids");
        g_object_set_data(G_OBJECT(dlg), nome.str().c_str(), cb);
        g_signal_connect(G_OBJECT(cb), COLOR_CHANGED, G_CALLBACK(userColor_changed), GINT_TO_POINTER(kk));
        GtkWidget *label = LABEL_NEW(elem.c_str(), MyBLUE);
        GtkWidget *vb = vbox_new(FALSE, 0);
        gtk_box_pack_start(GTK_BOX(vb), label, FALSE, FALSE, 0);
        gtk_box_pack_start(GTK_BOX(vb), cb, FALSE, FALSE, 0);
        table_attach(table1, vb, k%10, k%10+1, k/10, k/10+1, TRUE);
        gtk_container_set_border_width(GTK_CONTAINER(cb), 2);
    }
    frame = NEW_FRAME("Chains");
    gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
    GtkWidget *table3 = table_new(2, 13, TRUE);
    gtk_container_add(GTK_CONTAINER(get_widget(frame, "align")), table3);
    for(int k=0; k<N_CHAINS; k++)
    {
        std::string elem = get_nth_chain(k);
        std::stringstream nome;
        int kk = k+N_ELEMENTS+N_AMINO;
        nome << kk;
        float light[4];
        memcpy(light, main_app::JavApp.app_color[kk+N_OBJS], 4*sizeof(float));
        GtkWidget *cb = mycolor_button(light, "Pick a color to be used for draw chains");
        g_object_set_data(G_OBJECT(dlg), nome.str().c_str(), cb);
        g_signal_connect(G_OBJECT(cb), COLOR_CHANGED, G_CALLBACK(userColor_changed), GINT_TO_POINTER(kk));
        GtkWidget *label = LABEL_NEW(elem.c_str(), MyBLUE);
        GtkWidget *vb = vbox_new(FALSE, 0);
        gtk_box_pack_start(GTK_BOX(vb), label, FALSE, FALSE, 0);
        gtk_box_pack_start(GTK_BOX(vb), cb, FALSE, FALSE, 0);
        table_attach(table3, vb, k%13, k%13+1, k/13, k/13+1, TRUE);
        gtk_container_set_border_width(GTK_CONTAINER(cb), 2);
    }
    frame = NEW_FRAME("Secondary structure");
    gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
    GtkWidget *table4 = table_new(1, N_STRUCT, TRUE);
    gtk_container_add(GTK_CONTAINER(get_widget(frame,"align")), table4);
    for(int k=0; k<N_STRUCT; k++)
    {
        std::string elem = get_nth_struct(k);
        std::stringstream nome;
        int kk = k+N_ELEMENTS+N_AMINO+N_CHAINS+N_DNA_NUCLEOTIDES;
        nome << kk;
        float light[4];
        memcpy(light, main_app::JavApp.app_color[kk+N_OBJS], 4*sizeof(float));
        GtkWidget *cb = mycolor_button(light, "Pick a color to be used for draw secondary structure elements");
        g_object_set_data(G_OBJECT(dlg), nome.str().c_str(), cb);
        g_signal_connect(G_OBJECT(cb), COLOR_CHANGED, G_CALLBACK(userColor_changed), GINT_TO_POINTER(kk));
        GtkWidget *label = LABEL_NEW(elem.c_str(), MyBLUE);
        GtkWidget *vb = vbox_new(FALSE, 0);
        gtk_box_pack_start(GTK_BOX(vb), label, FALSE, FALSE, 0);
        gtk_box_pack_start(GTK_BOX(vb), cb, FALSE, FALSE, 0);
        table_attach(table4, vb, k, k+1, 0, 1, TRUE);
        gtk_container_set_border_width(GTK_CONTAINER(cb), 2);
    }

    frame = NEW_FRAME("Bases");
    gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
    GtkWidget *table5 = table_new(1, N_DNA_NUCLEOTIDES, TRUE);
    gtk_container_add(GTK_CONTAINER(get_widget(frame, "align")), table5);
    for(int k=0; k<N_DNA_NUCLEOTIDES; k++)
    {
        std::string elem = get_nth_base(k);
        std::stringstream nome;
        int kk = k+N_ELEMENTS+N_AMINO+N_CHAINS;
        nome << kk;
        float light[4];
        memcpy(light, main_app::JavApp.app_color[kk+N_OBJS], 4*sizeof(float));
        GtkWidget *cb = mycolor_button(light, "Pick a color to be used for draw secondary structure elements");
        g_object_set_data(G_OBJECT(dlg), nome.str().c_str(), cb);
        g_signal_connect(G_OBJECT(cb), COLOR_CHANGED, G_CALLBACK(userColor_changed), GINT_TO_POINTER(kk));
        GtkWidget *label = LABEL_NEW(elem.c_str(), MyBLUE);
        GtkWidget *vb = vbox_new(FALSE, 0);
        gtk_box_pack_start(GTK_BOX(vb), label, FALSE, FALSE, 0);
        gtk_box_pack_start(GTK_BOX(vb), cb, FALSE, FALSE, 0);
        table_attach(table5, vb, k, k+1, 0, 1, TRUE);
        gtk_container_set_border_width(GTK_CONTAINER(cb), 2);
    }

    GtkWidget *button = OK_button();
    gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_OK);
    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(ok_color_table), (gpointer) st);
    button = DEFAULTS_button();
    gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_ACCEPT);
    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(defaults_color_table), (gpointer) st);

    button = CANCEL_button();
    gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_CANCEL);
    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(close_color_table), (gpointer) st);
    g_signal_connect_swapped(G_OBJECT(dlg), "delete-event", G_CALLBACK(gtk_button_clicked), button);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
         GTK_BUTTONBOX_SPREAD);
    gtk_widget_show_all(dlg);
    return dlg;
}

void ColorTable(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GtkWidget *dlg;
   if(g_object_get_data(G_OBJECT(window), "colortable"))
   {
      dlg = get_widget(window, "colortable");
      gdk_window_raise(gtk_widget_get_window(dlg));
      return;
   }
   dlg = create_colortable(data);
   g_object_set_data(G_OBJECT(window), "colortable", dlg);
}



bool IsProline(const char *resname)
{
    if(!resname)
       return false;
    const char *proline = "PRO";
    if(g_ascii_strcasecmp(proline, resname) == 0)
       return true;
    return false;
}

const char *get_chain(std::string &label)
{
   int p = (int) label.find_last_of( "_" );
   if (p >= 0)
   {
       char *chain = DuplicaStringa(label.substr(p+1).c_str());
       return g_strstrip(chain);
   }
   else
       return NULL;
}

const char *get_res_num(std::string &label)
{
   int p = (int) label.find_last_of( "_" );
   if (p >= 0)
   {
       std::string lab1 = label.substr(0,p);
       int q = (int) lab1.find_last_of( "_" );
       if(q >=0)
       {
          char *rn = DuplicaStringa(lab1.substr(q+1).c_str());
          return g_strstrip(rn);
       }
       else
          return NULL;
   }
   else
       return NULL;
}

bool CheckAtom(const char *str)
{
  static const char *Atom[] = {
    "AD1", "AD2", "AE1", "AE2", "C", "CA", "CB", "CD", "CD1", "CD2", "CE", "CE1", "CE2",
    "CE3", "CG", "CG1", "CG2", "CH2", "CH3", "CZ", "CZ2", "CZ3", "HG", "HG1", "HH", "HH2",
    "HZ", "HZ2", "HZ3", "N", "ND1", "ND2", "NE", "NE1", "NE2", "NH1", "NH2", "NZ", "O",
    "OD1", "OD2", "OE", "OE1", "OE2", "OG", "OG1", "OH", "OXT", "SD", "SG", "H", "HA", "HB",
    "HD1", "HD2", "HE", "HE1", "HE2", "HE3", "1H", "1HA", "1HB", "1HD", "1HD1", "1HD2",
    "1HE", "1HE2", "1HG", "1HG1", "1HG2", "1HH1", "1HH2", "1HZ", "2H", "2HA", "2HB", "2HD", 
    "2HD1", "2HD2", "2HE", "2HE2", "2HG", "2HG1", "2HG2", "2HH1", "2HH2", "2HZ", "3H", "3HB",
    "3HD1", "3HD2", "3HE", "3HG1", "3HG2", "3HZ",
     "O3'", "O4'", "O5'",
     "C1'", "C2'", "C3'", "C4'", "C5'", "OP1", "OP2",
     "N1", "N2", "N3", "N4", "N6", "O2", "O4", "O6"//,
//     "O3*", "O4*", "O5*","C1*", "C2*", "C3*", "C4*", "C5*"
    };

  if(!str)
     return false;
  for( int i=0; i<(int)(sizeof(Atom)/sizeof(Atom[0])); i++ )
    if( !g_ascii_strcasecmp(str,Atom[i]) )
        return true;
  return (is_valid_symbol(str) != 0);
}

const char *Model::GetSpecies(int whichNo)
{
    if((whichNo >= 0) && (whichNo < numSpec))
    {
//       int which = Species[whichNo]-1;
//       return get_nth_element(which);
         std::string spec (LabSpec[whichNo],LEN_SPEC);
         return g_strdup(spec.c_str());
    }
    else
         return "Q";
}

const char *Model::GetSpeciesName(int whichNo)
{
    if(whichNo >= 0)
    {
       int which = Species[whichNo]-1;
       return get_nth_element_name(which);
    }
    else
       return "Q";
}

bool getAtomTypes(std::string &label, std::string &typ, std::string &Typ)
{
   int p = (int) label.find_first_of( "_" );
   if (p >= 0)
   {
       typ = label.substr(0,p);
       p = (int) typ.find_first_of( "/" );
       if (p > 0)
           Typ = typ.substr(0,p);
       else
       {
           std::string typsav = typ;
           size_t found = typ.find("*");
           if(found != std::string::npos)
              typ.at(found) = 0x27;
           if(CheckAtom(typ.c_str()))
              Typ = typ;
           else
           {
              size_t found = typ.find_first_of("0123456789");
              if(found != std::string::npos)
                 Typ = typ.substr(0, found);
              else
                 Typ = typ;
#ifndef NDEBUG
              if(!is_valid_symbol(Typ.c_str()) && ifDebug)
                 g_print("Warning: Invalid atom id:%s\n", Typ.c_str());
#endif
              Typ = typ.substr(0,1);
           }
           typ = typsav;
       }
       return true;
   }
   return false;
}

const char *get_res_name(std::string &label)
{
   int p = (int) label.find_first_of( "_" );
   if (p >= 0)
   {
       std::string lab1 =  label.substr(p+1);
       int q = (int) lab1.find_first_of("_");
       if(q >=0)
          return DuplicaStringa(lab1.substr(0,q).c_str());
       else
          return NULL;
   }
   else
       return NULL;
}

int Model::GetNumResiduo(int ch, int r)
{
   int res0 = 0;
   for(int i=0; i<ch-1; i++)
       res0 = res0 + Chains[i].nAmino;
   return (res0+r);
}  

std::string Model::GetResName(int res1)
{
   int res0 = 0;
   int ch1 = 0;
   int r1 = 0;
   for(int i=0; i<nChains; i++)
   {
       int resi = Chains[i].nResidues;
       if(res1 <= (res0+resi))
       {
          r1 = res1 - res0;
          ch1 = i;
          break;
       }
       res0 = res0 + resi;
   }
   std::string tname (Chains[ch1].Residues[r1].name);
   return tname;
}  

int Model::FindChain(const char *curChain)
{
    std::vector<CHAIN>::iterator it;
    for(it=Chains.begin();it!=Chains.end();it++)
    {
         char *lastchain = (*it).id;
         if(strcmp(lastchain, curChain) == 0)
            return ((int)distance(Chains.begin(), it));
    }
    return -1;
}

int Model::FindChainId(const char *curChain)
{
    int kch = tolower(curChain[0]);
    if( (kch >= 'a') && (kch <='z') )
          return(kch - 'a');
    else 
          return 0;
}

int Model::FindResidue(int ch, const char *curResidue, RESIDUE *rr, int atom)
{
    for(int i=0; i<Chains[ch].nResidues; i++)
    {
         char *lastresidue = Chains[ch].Residues[i].PDBnum;
         if(strcmp(lastresidue, curResidue) == 0)
         {
            rr = &(Chains[ch].Residues[i]);
            Chains[ch].Residues[i].atoms.push_back(atom);
            return i;
         }
    }
    return -1;
}

int Model::FindConn(int at1, int at2, int nlegs)
{
    if(nlegs < 0)
       nlegs = nconn;
    for(int i=0; i<nlegs; i++)
    {
        int j1 = bonds[i].n1 - 1;
        int j2 = bonds[i].n2 - 1;
        if( ((j1 == at1) && (j2 == at2)) ||
            ((j1 == at2) && (j2 == at1)) )
           return i;
    }
    return -1;
}

void Model::DeleteResidue(int which)
{
    int nr = c_alpha[which].residue;
    int ch = c_alpha[which].chain;
    if(ifDebug)
       g_print("Which = %d\nDelete residue %d of chain %d\n", which+1, nr+1, ch+1);
    for(int i=0; i<(int) Chains[ch].Residues[nr].atoms.size(); i++)
    {
        int n = Chains[ch].Residues[nr].atoms[i];
        atm[n].deleted = true;
    }
}

bool Model::IsSideChain(int n)
{
    return(!IsType(n,"N") && !IsType(n, "CA") && !IsType(n, "C") 
       &&  !IsType(n, "CB") && !IsType(n, "O"));
}

bool Model::IsAromatic(int n)
{
    const char *aro[] = {"HOH", "CD", "CD1", "CD2", "CE", "CE1", "CE2",
    "CE3", "CG", "CG1", "CG2", "CH2", "CH3", "CZ", "CZ2", "CZ3"};

    const char *str = (const char *)atom_types.at(n);
    for( int i=0; i<(int)(sizeof(aro)/sizeof(aro[0])); i++ )
      if( !g_ascii_strcasecmp(str, aro[i]) )
        return true;
    return false;
}

void Model::DeleteSideChain(int which)
{
    GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
    int nr = c_alpha[which].residue;
    int ch = c_alpha[which].chain;
    for(int i=0; i<(int) Chains[ch].Residues[nr].atoms.size(); i++)
    {
        int n = Chains[ch].Residues[nr].atoms[i];
        if(IsSideChain(n))
           atm[n].deleted = true;
    }
    gl->Redraw ();
    PreparaTabelle();
}

void Model::DeleteRing(int which)
{
    GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
    int nr = c_alpha[which].residue;
    int ch = c_alpha[which].chain;
    for(int i=0; i<(int) Chains[ch].Residues[nr].atoms.size(); i++)
    {
        int n = Chains[ch].Residues[nr].atoms[i];
        if(IsAromatic(n))
           atm[n].deleted = true;
    }
    PreparaTabelle();
    gl->Redraw ();
}

void AddChain(const char *chain_id, CHAIN *cc)
{
    cc->id        = DuplicaStringa(chain_id);
    cc->nResidues = 0;
    cc->nAmino    = 0;
    cc->isDNA     = false;
    cc->isRNA     = false;
}

void AddResidue(std::string Label, int which, int istart, RESIDUE *rr)
{
    rr->breakF      = false;
    rr->name        = DuplicaStringa(get_res_name(Label));
    rr->num         = which+1;
    rr->PDBnum      = DuplicaStringa(get_res_num(Label));
    rr->atoms.clear();
    rr->atoms.push_back(istart);
    rr->secondary   = ' ';
    rr->ca          = -1;
    rr->c           = -1;
    rr->n           = -1;
    rr->o           = -1;
    rr->h           = -1;
    rr->code        = IsAminoNucle(rr->name);
//    g_print("Resnum %d, Resname %s, Rescode %d\n", rr->num, rr->name, rr->code);
}

int TypeS(char Code)
{
  switch(Code) 
  {
  case 'H':
  case 'G':
  case 'I':
        return HELIX;
  case 'E':
  case 'S':
        return STRAND;
  default:
  case 'C':
        return COIL;
  case '1':
  case '2':
  case '3':
  case '4':
  case '5':
  case '6':
  case '7':
  case '8':
  case '@':
  case '&':
  case 'T':
        return TURN;
  case 'D':
        return OTHER;
  }

}

const char *Translate(char Code)
{
  const char *Dictionary[] = {
    "AlphaHelix","310Helix","PiHelix","Strand","Bridge","Coil","TurnI","TurnI'",
    "TurnII","TurnII'","TurnVIa","TurnVIb","TurnVIII","TurnIV","GammaClassic",
    "GammaInv","Turn","Nucleic Acid"," ", "Unknown" };

  switch(Code) {
  case 'H': return(Dictionary[0]);
  case 'G': return(Dictionary[1]);
  case 'I': return(Dictionary[2]);
  case 'E': return(Dictionary[3]);
  case 'S': return(Dictionary[3]);
  case 'B':
  case 'b':
            return(Dictionary[4]);
  case ' ':
  case 'C': return(Dictionary[5]);
  case '1': return(Dictionary[6]);
  case '2': return(Dictionary[7]);
  case '3': return(Dictionary[8]);
  case '4': return(Dictionary[9]);
  case '5': return(Dictionary[10]);
  case '6': return(Dictionary[11]);
  case '7': return(Dictionary[12]);
  case '8': return(Dictionary[13]);
  case '@': return(Dictionary[14]);
  case '&': return(Dictionary[15]);
  case 'T': return(Dictionary[16]);
  case 'D': return(Dictionary[17]);
  case 'L': return(Dictionary[18]);
  default:  return(Dictionary[19]);
  }
}

void FillSecondary(Model *md)
{
   for(int i=0; i<(int)(sizeof(md->numbStructs)/sizeof(md->numbStructs[0])); i++)
      md->numbStructs[i] = 0;
   int tipo = -1;
   for(int ch=0; ch<md->nChains; ch++)
   {
       for(int j=0; j< md->Chains[ch].nAmino; j++)
       {
           RESIDUE *cur = &(md->Chains[ch].AminoRes[j]);
           if( tipo != TypeS(cur->secondary) )
           {
               tipo = TypeS(cur->secondary);
               (md->numbStructs[tipo])++;
           }
       }
   }
}

void PrintSecondary(Model *md, std::ostream &os)
{
   os << "---------------- Secondary structure summary ---------------\n";

   for(int ch=0; ch<md->nChains; ch++)
   {
       for(int j=0; j< md->Chains[ch].nAmino; j++)
       {
           RESIDUE *cur = &(md->Chains[ch].AminoRes[j]);
           char c = (cur->secondary != 'C')?cur->secondary:' ';
           os << c;
       }
       os << "\n";
   }
   os << "\n";
   os << "----------- Detailed Secondary structure summary -----------\n";
   os << "\n|--- Residue ---| |---- Structure ----|\n";
   for(int ch=0; ch<md->nChains; ch++)
   {
       for(int j=0; j< md->Chains[ch].nAmino; j++)
       {
           RESIDUE *cur = &(md->Chains[ch].AminoRes[j]);
           os << "     " << md->Chains[ch].id << " " <<  cur->name
              << " " << cur->PDBnum << "\t      " 
              << cur->secondary << " " << Translate(cur->secondary)
              << "\n";
       }
       os << "\n";
   }
}

//#ifdef JAV
void Model::AllCAlpha()
{
   if(!ifSymmetry)
   {
        for(int i=0; i<(int) c_alpha.size(); i++)
        {
             C_Alpha c  = c_alpha[i];
             if(i == (int) c_alpha.size() -1)
                c.breakF = true;
             c_alphab.push_back(c);
        }
        return;
   }
   std::vector<int> ca, o, o2, n, c;
   int nEqui;
   int **o_equi, **ca_equi, **o2_equi, **n_equi, **c_equi;
   o_equi = g_new0(int *, (int) c_alpha.size());
   o2_equi = g_new0(int *, (int) c_alpha.size());
   n_equi = g_new0(int *, (int) c_alpha.size());
   ca_equi = g_new0(int *, (int) c_alpha.size());
   c_equi = g_new0(int *, (int) c_alpha.size());
   for(int ic=0; ic<(int) c_alpha.size(); ic++)
   {
       C_Alpha c_alfa = c_alpha[ic];
       int CA = c_alfa.atom;
       int O  = c_alfa.o;
       int O2 = c_alfa.o2;
       int N  = c_alfa.n;
       int C  = c_alfa.c;
       ca.clear();
       o.clear();
       n.clear();
       c.clear();
       o2.clear();
       for(int i=0; i<nats; i++)
       {
           if(iEqui[i] == O)
              o.push_back(i);
           if(iEqui[i] == CA)
              ca.push_back(i);
           else if(iEqui[i] == O2)
              o2.push_back(i);
           else if(iEqui[i] == N)
              n.push_back(i);
           else if(iEqui[i] == C)
              c.push_back(i);
       }
       nEqui = ((int) ca.size());
       ca_equi[ic] = g_new0(int, nEqui);
       c_equi[ic]  = g_new0(int, nEqui);
       n_equi[ic]  = g_new0(int, nEqui);
       o_equi[ic]  = g_new0(int, nEqui);
       o2_equi[ic] = g_new0(int, nEqui);
       for(int k=0; k<nEqui; k++)
       {
           if(k < (int) o2.size())
              o2_equi[ic][k] = o2[k];
           else
              o2_equi[ic][k] = -1;
           if(k < (int) o.size())
              o_equi[ic][k] = o[k];
           else
              o_equi[ic][k] = -1;
           if(k < (int) n.size())
              n_equi[ic][k] = n[k];
           else
              n_equi[ic][k] = -1;
           if(k < (int) c.size())
              c_equi[ic][k] = c[k];
           else
              c_equi[ic][k] = -1;
           ca_equi[ic][k] = ca[k];
       }
   }
   for(int k=0; k<nEqui; k++)
   {
        for(int i=0; i<(int) c_alpha.size(); i++)
        {
             C_Alpha c  = c_alpha[i];
             c.o = o_equi[i][k];
             c.n = n_equi[i][k];
             c.c = c_equi[i][k];
             c.o2 = o2_equi[i][k];
             c.atom = ca_equi[i][k];
             c.selected = false;
             c_alphab.push_back(c);
        }
    }
    for(int i=0; i<(int) c_alpha.size(); i++)
    {
        g_free(n_equi[i]);
        g_free(o_equi[i]);
        g_free(o2_equi[i]);
        g_free(c_equi[i]);
        g_free(ca_equi[i]);
    }
    g_free(n_equi);
    g_free(o_equi);
    g_free(c_equi);
    g_free(o2_equi);
    g_free(ca_equi);
    ca.clear();
    o.clear();
    n.clear();
    c.clear();
    o2.clear();
}
//#endif

void Model::CreaLegsCA()
{
//   float ds[] = { 3.f, 4.3f, 4.f, 8.f};
   double ds[] = { 2.5, 4.3, 4., 8.};
   double dmin, dmax;

   int n = (int)c_alphab.size();
   int *ca = g_new0(int, n);
   std::vector<jav_bond_type> vett;
   for(int i=0; i<n; i++)
       ca[i]   = c_alphab[i].atom;
   for(int i=0; i<n-1; i++)
   {
       if((c_alphab[i].isAmino) && (c_alphab[i+1].isAmino))
       {
          dmin = ds[0];
          dmax = ds[1];
       }
       else if((!c_alphab[i].isAmino) && (!c_alphab[i+1].isAmino))
       {
          dmin = ds[2];
          dmax = ds[3];
       }
       else
       {
          c_alphab[i].breakF = true;
          int ch = c_alphab[i].chain;
          for(int r=0; r<Chains[ch].nResidues; r++)
              if(r == c_alphab[i].residue)
                 Chains[ch].Residues[r].breakF = true;
          continue;
       }
       if((ca[i] != -1) && (ca[i+1] != -1) && (!c_alphab[i].breakF))
       {
           vet3 x1 = {{atm[ca[i]].xyz[0], atm[ca[i]].xyz[1], atm[ca[i]].xyz[2]}};
           vet3 x2 = {{atm[ca[i+1]].xyz[0], atm[ca[i+1]].xyz[1], atm[ca[i+1]].xyz[2]}};
           double dist = MRVectorDistance(x1, x2);
           if((dist >= dmin) && (dist <= dmax))
           {
               jav_bond_type b;
               b.n1 = ca[i] + 1;
               b.n2 = ca[i+1] + 1;
               b.dist = (float) dist;
               vett.push_back(b);
           }
           else
           {
               c_alphab[i].breakF = true;
               int ch = c_alphab[i].chain;
               for(int r=0; r<Chains[ch].nResidues; r++)
                   if(r == c_alphab[i].residue)
                      Chains[ch].Residues[r].breakF = true;
           }
       }
   }
   g_free(ca);
   nconn_ca = (int) vett.size();
   if(bonds_ca != NULL)
      g_free(bonds_ca);
   bonds_ca = g_new0(jav_bond_type, nconn_ca);
   for(int i=0; i<nconn_ca; i++)
   {
      bonds_ca[i].n1 = vett[i].n1;
      bonds_ca[i].n2 = vett[i].n2;
      bonds_ca[i].dist = vett[i].dist;
      bonds_ca[i].selected = false;
   }
   vett.clear();
}

int Model::IsTypeBB(int at, const char *types[], int n)
{
    for(int i=0; i<n; i++)
    {
        if(IsType(at, types[i]))
           return (i+1);
    }
    return 0;
}

void Model::CreaLegsBB()
{
   int *ifbb = g_new0(int, nats);
   int *ifnuc = g_new0(int, nats);
   std::vector<int> vett;
   std::vector<int> vettnuc;
   const char *types[] = {"N", "C", "CA", "O", "CB", "CD", "CD1", "CD2",
    "CE", "CE1", "CE2", "CE3", "CG", "CG1", "CG2", "CH2", "CH3",
    "CZ", "CZ2", "CZ3", "SG", "NE2"};
   int ntypes = (int) sizeof(types)/sizeof(types[0]);
//   const char *types1[] = {"P", "C3'", "C4'", "C5'", "O3'", "O5'"};
   const char *types1[] = {"P", "C1'", "C2'", "C3'", "C4'", "C5'",
                           "O3'", "O4'", "O5'"};
   int ntypes1 = (int) sizeof(types1)/sizeof(types1[0]);
   const char *types2[] = {"P", "O3'", "O4'", "O5'"};
   int ntypes2 = (int) sizeof(types2)/sizeof(types2[0]);
   for(int ch=0; ch<nChains; ch++)
   {
       for(int j=0; j< Chains[ch].nResidues; j++)
       {
           RESIDUE cur = Chains[ch].Residues[j];
           if(cur.code == 0)
              continue;
           if(!Is_Acid( cur.code))
              continue;
           for(int i=0; i<(int)cur.atoms.size(); i++)
           {
               int at = cur.atoms[i];
               if(Is_AminoG(cur.code))
                  ifbb[at] = IsTypeBB(at, types, ntypes);
               else if(Is_DnaG(cur.code))
               {
                  ifbb[at] = IsTypeBB(at, types1, ntypes1);
                  ifnuc[at] = (IsTypeBB(at, types2, ntypes2))?0:1;
               }
               for(int ieq=1; ieq <NEquivs[at]; ieq++)
               {
                   int jat = Equivs[at].at(ieq);
                   ifbb[jat] = ifbb[at];
                   ifnuc[jat] = ifnuc[at];
               }
           }
       }
   }
/*
   for(int i=nats_au; i<nats; i++)
   {
       int k = iEqui[i];
       ifbb[i] = ifbb[k];
       ifnuc[i] = ifnuc[k];
   }
*/
   for(int i=0; i<nconn_prot; i++)
   {
       int i1 = bonds[i].n1 - 1;
       int i2 = bonds[i].n2 - 1;
       if(ifbb[i1] && ifbb[i2])
         vett.push_back(i);
       if(ifnuc[i1] && ifnuc[i2])
         vettnuc.push_back(i);
   }

   nconn_bb = (int) vett.size();
   if(bonds_bb != NULL)
      g_free(bonds_bb);
   bonds_bb = g_new0(jav_bond_type, nconn_bb);
   for(int i=0; i<nconn_bb; i++)
   {
      bonds_bb[i].n1 = bonds[vett[i]].n1;
      bonds_bb[i].n2 = bonds[vett[i]].n2;
      bonds_bb[i].dist = bonds[vett[i]].dist;
      bonds_bb[i].ordleg = bonds[vett[i]].ordleg;
      bonds_bb[i].selected = false;
   }
   g_free(ifbb);
   vett.clear();

   nconn_nuc = (int) vettnuc.size();
   if(bonds_nuc != NULL)
      g_free(bonds_nuc);
   bonds_nuc = g_new0(jav_bond_type, nconn_nuc);
   for(int i=0; i<nconn_nuc; i++)
   {
      bonds_nuc[i].n1 = bonds[vettnuc[i]].n1;
      bonds_nuc[i].n2 = bonds[vettnuc[i]].n2;
      bonds_nuc[i].dist = bonds[vettnuc[i]].dist;
      bonds_nuc[i].ordleg = bonds[vettnuc[i]].ordleg;
      bonds_nuc[i].selected = false;
   }
   g_free(ifnuc);
   vettnuc.clear();
}

void Model::CreaLegsLig()
{
   std::vector<int>ib, is;
   for(int i=0; i<nconn_prot; i++)
   {
       int n1 = bonds[i].n1;
       int n2 = bonds[i].n2;
       if(IsHetero(n1-1) && IsHetero(n2-1))
          ib.push_back(i);
       else if(IsHetero(n1-1) || IsHetero(n2-1))
          is.push_back(i);
   }
   nconn_lig = (int) ib.size();
   nconn_mix = (int) is.size();
   if(bonds_lig != NULL)
      g_free(bonds_lig);
   bonds_lig = g_new0(jav_bond_type, nconn_lig);
   for(int i=0; i<nconn_lig; i++)
   {
      bonds_lig[i].n1 = bonds[ib[i]].n1;
      bonds_lig[i].n2 = bonds[ib[i]].n2;
      bonds_lig[i].dist = bonds[ib[i]].dist;
      bonds_lig[i].ordleg = bonds[ib[i]].ordleg;
      bonds_lig[i].selected = false;
   }
   if(bonds_mix != NULL)
      g_free(bonds_mix);
   bonds_mix = g_new0(jav_bond_type, nconn_mix);
   for(int i=0; i<nconn_mix; i++)
   {
      bonds_mix[i].n1 = bonds[is[i]].n1;
      bonds_mix[i].n2 = bonds[is[i]].n2;
      bonds_mix[i].dist = bonds[is[i]].dist;
      bonds_mix[i].ordleg = bonds[is[i]].ordleg;
      bonds_mix[i].selected = false;
   }
}

void Model::PreparaTabelle()
{
using namespace gtkext;
    GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
    std::string msg;
    int ier = PreparaTabelle(msg);
    if(ifDebug && (ier <= 0))
       g_print("%s", msg.c_str());
    if(ier < 0)
    {
       dlg_errmsg(GTK_WINDOW(GLWINDOW), msg.c_str());
       gl->Destroy(-1);
    }
}

bool Model::IsHetero(int num)
{
    int ieq = iEqui[num];
    return(atm_au[ieq].heta != 0);
}

void Model::FillPhList()
{
   for(int ch=0; ch<nChains; ch++)
   {
       if(!Chains[ch].isDNA)
          continue;
       for(int j=0; j< Chains[ch].nResidues; j++)
       {
           RESIDUE cur = Chains[ch].Residues[j];
           if(!Is_Acid(cur.code))
              continue;
           if(!IsPurine(cur.code))
              continue;
           int n1 = FindAtom("N1", cur);
           if(n1 == -1)
              continue;
           double distmax = 25.; //12.5;
           //int at=-1, res=-1;
           for(int ch2=0; ch2<nChains; ch2++)
           {
           int at=-1, res=-1;
               if((ch==ch2) && !Chains[ch].isRNA )
                   continue;
               if(!Chains[ch2].isDNA)
                   continue;
               for(int k=0; k< Chains[ch2].nResidues; k++)
               {
                   if((ch == ch2) && (j == k))
                      continue;
                   RESIDUE next = Chains[ch2].Residues[k];
                   if(!Is_Acid(next.code))
                      continue;
                   if(IsPair(cur.code, next.code))
                   {
                      int n3 = FindAtom("N3", next);
                      if(n3 == -1)
                         continue;
                      double dist = MRVectorDistance2(
                         MRVectorMake(atm[n1].xyz), 
                         MRVectorMake(atm[n3].xyz));
                      if(dist >= distmax)
                         continue;
                      distmax = dist;
                      at = n3;
                      res = k;
                   }
               }
               if(at != -1)
               {
                  RESIDUE resi = Chains[ch2].Residues[res];
                  int ph1 = FindAtom("P", cur);
                  int ph2 = FindAtom("P", resi);
                  if(ph1 == -1) 
                     ph1 = FindAtom("O5'", cur);
                  if(ph2 == -1) 
                     ph2 = FindAtom("O5'", resi);
                  if((ph1 != -1) && (ph2 != -1))
                  {
                     phlist.push_back(ph1);
                     phlist.push_back(ph2);
                     nucleic = 1;
                  }
               }
           }
       }
   }
}

void Model::PopulateModel (GtkTreeView *list, int ifClear)
{
  GtkTreeIter iter;

  GtkTreeStore *model = GTK_TREE_STORE(gtk_tree_view_get_model(list));
  if(ifClear)
    gtk_tree_store_clear(GTK_TREE_STORE(model));

  for(int ch=0; ch<nChains; ch++)
  {
      std::string chain = "Chain ";
      chain.append(Chains[ch].id);
      gtk_tree_store_append (model, &iter, NULL);
      gtk_tree_store_set (model, &iter, 0, DuplicaStringa(chain.c_str()), -1);
      // add children (residues) 
      for(int r=0; r<Chains[ch].nResidues; r++)
      {
           GtkTreeIter child_iter;
           std::string rn = "CC XXX AAA";
           rn.assign(Chains[ch].id);
           rn.append(" ");
           RESIDUE rr = Chains[ch].Residues[r];
           rn.append(rr.PDBnum);
           rn.append(" ");
           rn.append(rr.name);
           gtk_tree_store_append (model, &child_iter, &iter);
           gtk_tree_store_set (model, &child_iter, 0, DuplicaStringa(rn.c_str()), -1);
      }
  }
}

int Model::PreparaTabelle(std::string &msg)
{
using namespace std;
using namespace gtkext;
  CHAIN cc;
  RESIDUE rr;
  gint64 t0, t1;

  freeStructs(true);
  if(ifDebug)
     t0 = g_get_real_time();
  for(int i=0; i<nats_au; i++)
  {
      int ieq = Equivs[i].at(0);
      std::string Label = atm_au[ieq].lab, types, Types;
      getAtomTypes(Label, types, Types);
      const char *chain_id = get_chain(Label);
      if(str_white(types.c_str()) || str_white(chain_id))
      {
         isPDB = false;
         return 0;
      }
      atom_types.push_back(DuplicaStringa(types.c_str()));
      AtomTypes.push_back(DuplicaStringa(Types.c_str()));
      if(atm_au[ieq].deleted)
         continue;
      int ch = 0;
      if(nChains == 0)
      {
         nChains = 1;
         AddChain(chain_id, &cc);
         Chains.push_back(cc);
      }
      else
      {
         ch = FindChain( chain_id);
         if(ch == -1)
         {
            ch = nChains;
            nChains++;
            AddChain(chain_id, &cc);
            Chains.push_back(cc);
         }
      }
      atm_au[ieq].catseq = ch;   //FindChain( chain_id );
      atm_au[ieq].catena = FindChainId( chain_id );
      int nr = IsAminoNucle( get_res_name(Label) );
      if(nr > 0)
         atm_au[ieq].numResiduo = nr - 1;
      else
         atm_au[ieq].numResiduo = N_AMINO + N_DNA_NUCLEOTIDES;
  }
  if(ifDebug)
  {
     t1 = g_get_real_time();
     g_print("Loop 1 on atoms %lg\n", (double)(t1-t0)/1e6);
  }
  int ch = 0;
  for(int i=0; i<nats_au; i++)
  {
      //int ieq = Equivs[i].at(0);
      int ieq = i;
      if(atm_au[ieq].deleted)
         continue;
      std::string Label = atm_au[ieq].lab;
      const char *chain_id = get_chain(Label);
      const char *res_num = get_res_num(Label);
      const char *res_nam = get_res_name(Label);
      if(str_white(chain_id) || str_white(res_num) || str_white(res_nam))
      {
         isPDB = false;
         return 0;
      }
      atm_au[ieq].ResNum = atoi(res_num) - 1;
      atm_au[ieq].isW = IsWater(res_nam);
//      g_print("%d %s %s %s %s \n", i+1, Label.c_str(), get_chain(Label),
//              get_res_num(Label), get_res_name(Label));

      ch = FindChain( chain_id);
      if(ch < 0)
      {
         msg = "Some trouble reading PDB.";
         return -1;
      }

      bool to_add = true;
      int curResidue;
      if(Chains[ch].nResidues != 0)
      {
         curResidue = FindResidue( ch, res_num, &rr, ieq);
         if(curResidue != -1)
            to_add = false;
      }
      if(to_add)
      {
         curResidue = Chains[ch].nResidues;
         AddResidue(Label, Chains[ch].nResidues, ieq, &rr);
         Chains[ch].nResidues++;
         Chains[ch].Residues.push_back(rr);
      }
      atm_au[ieq].seq = rr.num;
      atm_au[ieq].resseq = curResidue;
   }
   if(ifDebug)
   {
   t0 = g_get_real_time();
   g_print("Loop 2 on atoms %lg\n", (double)(t0-t1)/1e6);
   }

   nResAmino = 0;
   Bmin_ca = 1.e10;
   Bmax_ca = -1.e10;
   int n_c, n_o, n_n;
   n_c = n_o = n_n = 0;
   c_alpha.clear();
   for(int ch=0; ch<nChains; ch++)
   {  
       Chains[ch].nAmino = 0;
       for(int r=0; r<Chains[ch].nResidues; r++)
       {
           RESIDUE *rr = &(Chains[ch].Residues[r]);
           int tipo_residuo = 0;
           int at = -1;
           if(Is_Water(rr->code))
              continue;
           if(IsAmino(rr->name))
           {
              at = FindAtom("CA", *rr, 6);
              if(at == -1)
              {
                 g_print("warning: Calpha not found at residue %s (Chain %c) \n", 
                   rr->PDBnum, Chains[ch].id[0]);
                 continue;
              }
              tipo_residuo = 1;
           }
           else if(IsDNA(rr->name))
           {
              at = FindAtom("P", *rr);
              if(at == -1)
              {
                 at = FindAtom("O5'", *rr);
                 if(at == -1)
                    continue;
              }
              tipo_residuo = 2;
           }
           else
           {
              rr->code = N_AMINO + N_DNA_NUCLEOTIDES + 1;
              at = FindAtom("CA", *rr, 6);
              if(at != -1)
              {
                 if((FindAtom("O", *rr) != -1) &&
                    (FindAtom("N", *rr) != -1) &&
                    (FindAtom("C", *rr) != -1) )
                 {
                    tipo_residuo = 1;
                    rr->code = 500;     // non standard amino acid
                 }
              }
              else
              {
                 at = FindAtom("P", *rr);
                 if(at != -1)
                 {
                    if((FindAtom("OP1", *rr) != -1) &&
                       (FindAtom("OP2", *rr) != -1) &&
                       (FindAtom("O5'", *rr) != -1) )
                    {
                       tipo_residuo = 2;
                       rr->code = 600;     // non standard nucleic acid
                    }
//                    else
//                       continue;
                 }
                 else
                 {
                    at = FindAtom("O5'", *rr);
                    if(at == -1)
                       continue;
                 }
                 tipo_residuo = 2;
                 rr->code = 600;     // non standard nucleic acid
              }
           }   

           if(atm_au[at].B < Bmin_ca)
              Bmin_ca = atm_au[at].B;
           else if(atm_au[at].B > Bmax_ca)
              Bmax_ca = atm_au[at].B;
           C_Alpha c;
           c.atom = at;
           rr->ca = at;
           c.breakF = (r == Chains[ch].nResidues-1);   //temp
           std::string Label = atm_au[at].lab;
           c.ResLab.assign(get_chain(Label));
           c.ResLab.append(get_res_name(Label));
           c.ResLab.append(get_res_num(Label));
           c.chain = ch;
           c.residue = r;
           if(tipo_residuo == 1)                // amino acid
           {
              at = FindAtom("O", *rr);
              c.o = at;
              rr->o = at;
              if(at != -1)
                 n_o++;
              at = FindAtom("C", *rr);
              c.c = at;
              rr->c = at;
              if(at != -1)
                 n_c++;
              at = FindAtom("N", *rr);
              c.n = at;
              rr->n = at;
              if(at != -1)
                 n_n++;
              at = FindAtom("H", *rr);
              if((at == -1) && (!IsProline(rr->name)))
                 at = FindAtom("D", *rr);
              rr->h = at;
              c.h = at;
              c.o2 = -1;
              c.isAmino = true;
           }
           else if(tipo_residuo == 2)  //nucleic acid
           {
              at = FindAtom("O5'", *rr);
              c.o = at;
              c.isAmino = false;
              rr->o = at;
              if(at != -1)
                 n_o++;
              at = FindAtom("C5'", *rr);
              c.c = at;
              rr->c = at;
              if(at != -1)
                 n_c++;
              at = FindAtom("OP1", *rr);
              c.o2 = at;
              at = FindAtom("O2'", *rr);
              c.n = at;
              rr->n = at;
              if(at != -1)
                 n_n++;
              at = FindAtom("H", *rr);
              rr->h = at;
              c.h = at;
              Chains[ch].isDNA = true;
              if(IsUracil(rr->code))
                  Chains[ch].isRNA = true;
           }
           if(tipo_residuo != 0)
           {
              c.selected = false;
              c_alpha.push_back(c);
              RESIDUE am = *rr;
              am.num = Chains[ch].nAmino;
              Chains[ch].AminoRes.push_back(am);
              Chains[ch].nAmino++;
              nResAmino++;
           }
       }
   }
   nResTot = 0;
   for(int ch=0; ch<nChains; ch++)
      nResTot += Chains[ch].nAmino;

   if(ifDebug)
   {
   t1 = g_get_real_time();
   g_print("Loop on chains %lg\n", (double)(t1-t0)/1e6);
   }
   bool ifDetectSecondary = false;
   hasSecondary = true;
   nucleic = 0;
//#ifdef JAV
   AllCAlpha();
   if(ifDebug)
   {
   t0 = g_get_real_time();
   g_print("AllCAlpha %lg\n", (double)(t0-t1)/1e6);
   }
   CreaLegsBB();
   if(ifDebug)
   {
   t1 = g_get_real_time();
   g_print("Backbone bonds %lg\n", (double)(t1-t0)/1e6);
   }
   CreaLegsCA();
   if(ifDebug)
   {
   t0 = g_get_real_time();
   g_print("CAlpha bonds %lg\n", (double)(t0-t1)/1e6);
   }
   CreaLegsLig();
   if(ifDebug)
   {
   t1 = g_get_real_time();
   g_print("Ligand bonds %lg\n", (double)(t1-t0)/1e6);
   }
   FillPhList();
   if(ifDebug)
   {
   t0 = g_get_real_time();
   g_print("PhList %lg\n", (double)(t0-t1)/1e6);
   }

   if(IsProtein() && !noSecondary)
   {
      ifDetectSecondary = true;
      for(int i=0; i<nats_au; i++)
      {
          if((Secondary[i] != ' ') && (Secondary[i] != 'L'))
             ifDetectSecondary = false;
      }
   }

   if((n_n <= 0 ) || (n_c <= 0) || (n_o <= 0))
       ifDetectSecondary = false;

   if(ifDetectSecondary)
   {
      if(!DetectSecondary(this, msg))
      {
         hasSecondary = false;
         return 0;
      }
   }
   else
   {
      for(int ch=0; ch<nChains; ch++)
      {
          for(int j=0; j< Chains[ch].nAmino; j++)
          {
              RESIDUE *cur = &(Chains[ch].AminoRes[j]);
              if(Is_DnaG(cur->code))
              {
                 Secondary[cur->atoms[0]] = 'D';
                 cur->secondary = Secondary[cur->atoms[0]];
              }
              else
                 cur->secondary = Secondary[cur->atoms[0]];
          }
      }
   }
   for(int ch=0; ch<nChains; ch++)
   {
       for(int j=0; j< Chains[ch].nResidues; j++)
       {
           RESIDUE *cur = &(Chains[ch].Residues[j]);
           if(Is_DnaG(cur->code))
           {
              Secondary[cur->atoms[0]] = 'D';
              cur->secondary = Secondary[cur->atoms[0]];
           }
           else
              cur->secondary = Secondary[cur->atoms[0]];
       }
   }

   std::ofstream os;
   PrintSecondary(this, os);
   if(ifDebug)
      PrintSecondary(this, cout);
   FillSecondary(this);
   if(ifDebug)
   {
   t1 = g_get_real_time();
   g_print("Secondary structure detection %lg\n", (double)(t1-t0)/1e6);
   }
   return 1;
}

#define BREAKDIST   2.5f
#define ERR         -1
#define CUTOFF      6.0f
#define RmGRID      3.0
#define EmGRID      -2.8
#define CGRID       -3.0*EmGRID*pow(RmGRID,8)
#define DGRID       -4.0*EmGRID*pow(RmGRID,6)
#define K1GRID      0.9/pow(cos(DEGTORAD(110.0)),6)
#define K2GRID      pow(cos(DEGTORAD(110.0)),2)
#define MINACCANG_SP2   90.0
#define MAXACCANG_SP2   180.0
#define MINDONANG_SP2   90.0
#define MAXDONANG_SP2   180.0
#define ACCDONANG       60.0
#define DONACCANG       90.0
#define DEFNUMPIXEL     18
#define MAXRESDNR                 6
#define MAXRESACC                 6
#define DIST_N_H                  1.0f
#define MAXHYDRBOND               50000
#define Eps                       0.000001
#define Treshold_H1     -230.0
#define Treshold_H3      0.12
#define Treshold_H4      0.06
#define Treshold_E1     -240.0
#define Treshold_E2     -310.0
#define DENTRO(x, target, range)      ( (x >= (target - range)) && (x <= (target + range)) )

typedef struct {
                 float Phi, Psi;
                 int PhiZn, PsiZn;
                 char Asn, PdbAsn;
               } ResidueProperty;

typedef struct {
                 int HBondDnr[MAXRESDNR];
                 int HBondAcc[MAXRESACC];
                 int NBondDnr, NBondAcc;
                 bool InterchainHBonds;
               } Involved;

typedef struct {
                 int Chain;
                 int D_Res, DD_Res, DDI_Res;
                 int D_At, DD_At, DDI_At, H;
                 float HB_Radius;
               } Donor;

typedef struct {
                 int Chain;
                 int A_Res, AA_Res, AA2_Res;
                 int A_At, AA_At, AA2_At;
                 float HB_Radius;
               } Acceptor;

typedef struct {
                 Donor *Dnr;
                 Acceptor *Acc;
                 bool ExistPolarInter, ExistHydrBondRose, ExistHydrBondBaker;
                 float Energy, Er, Et, Ep, ti, to, p;
                 double AccDonDist, OHDist, AngNHO, AngCOH;
                 double AccAng, DonAng, AccDonAng, DonAccAng;
               } HBOND;

typedef struct PAT {
                 HBOND *Hb1, *Hb2;
                 struct PAT *Nei1, *Nei2;
                 bool ExistPattern;
                 nomefile Type;
               } PATTERN;

const float PhiPsiStep = 20.f;
const int   nPixel = DEFNUMPIXEL;
ResidueProperty *properties;  // per ogni residuo info geometriche 
Involved *involved;           // per ogni residuo n. res. coinv. in leg h

bool DefineDnr(Model *md, int ch, Donor **Dnr, int *dc, int Res)
{
  Dnr[*dc] = g_new0(Donor, 1);
  
  Dnr[*dc]->Chain = ch;
  Dnr[*dc]->D_Res = Res;
  Dnr[*dc]->DD_Res = Res-1;
  Dnr[*dc]->DDI_Res = Res;
  Dnr[*dc]->HB_Radius = 1.90f;
  
  if( Res != 0 )
  {
      Dnr[*dc]->D_At  = md->Chains[ch].AminoRes[Res].n;
      Dnr[*dc]->DD_At = md->Chains[ch].AminoRes[Res-1].c;
  }
  else 
  {
      Dnr[*dc]->D_At  = ERR;
      Dnr[*dc]->DD_At = ERR;
  }
  Dnr[*dc]->DDI_At = md->Chains[ch].AminoRes[Res].ca;
  Dnr[*dc]->H      = md->Chains[ch].AminoRes[Res].h;
  
  if( Dnr[*dc]->H == ERR || Dnr[*dc]->D_At   == ERR || Dnr[*dc]->DD_At  == ERR ||
      (Dnr[*dc]->DDI_At == ERR  ) )
  {
    g_free(Dnr[*dc]); 
    return(false);
  }
  else 
    (*dc)++;
  return(true);
}

int get_atom(Model *md, int abnum, int Res, int chain)
{
    RESIDUE rr = md->Chains[chain].AminoRes[Res];
    for (int i=0; i<(int)rr.atoms.size(); i++)
         if(abnum == rr.atoms[i])
            return i;
    return -1;
}

int FindDnr(Model *md, Donor **Dnr, int *NDnr)
{

  int dc = *NDnr;

  for(int ch=0; ch<md->nChains; ch++)
  {
      for(int Res=0; Res< md->Chains[ch].nAmino; Res++)
          if(Is_Amino(md->Chains[ch].AminoRes[Res].code))
             DefineDnr(md, ch,Dnr,&dc, Res);
  }
  *NDnr = dc;
  if(ifDebug)
     for(int i=0; i<dc; i++)
      g_print("Donor %d D_Res %d DD_Res %d DDI_Res %d D_At %d DD_At %d DDI_At %d\n",
       i, Dnr[i]->D_Res, Dnr[i]->DD_Res, Dnr[i]->DDI_Res,
       get_atom(md, Dnr[i]->D_At, Dnr[i]->D_Res, Dnr[i]->Chain), 
       get_atom(md, Dnr[i]->DD_At, Dnr[i]->DD_Res, Dnr[i]->Chain), 
       get_atom(md, Dnr[i]->DDI_At, Dnr[i]->DDI_Res, Dnr[i]->Chain));
  return(dc);
}

bool DefineAcceptor(Model *md, int nch, Acceptor **Acc, int *ac, int Res)
{

  Acc[*ac] = g_new0(Acceptor, 1);

  Acc[*ac]->Chain     = nch;
  Acc[*ac]->A_Res     = Res;
  Acc[*ac]->AA_Res    = Res;
  Acc[*ac]->AA2_Res   = Res;
  Acc[*ac]->HB_Radius = 1.60f;
  
    if( Res != (md->Chains[nch].nAmino-1))
    {
        Acc[*ac]->A_At   = md->Chains[nch].AminoRes[Res].o;
        Acc[*ac]->AA_At  = md->Chains[nch].AminoRes[Res].c;
    }
    else 
    {
        Acc[*ac]->A_At  = ERR;
        Acc[*ac]->AA_At = ERR;
    }
    Acc[*ac]->AA2_At = md->Chains[nch].AminoRes[Res].ca;
  
  if( Acc[*ac]->A_At   == ERR || Acc[*ac]->AA_At  == ERR ||
      (Acc[*ac]->AA2_At == ERR  ) ) 
  {
      g_free(Acc[*ac]); 
      return(false); 
  }
  else 
      (*ac)++;
  return(true);
}

int FindAcc(Model *md, Acceptor **Acc, int *NAcc)
{

  int ac;
  
  ac = *NAcc;

  for(int ch=0; ch<md->nChains; ch++)
  {
      for(int Res=0; Res< md->Chains[ch].nAmino; Res++)
          if(Is_Amino(md->Chains[ch].AminoRes[Res].code))
             DefineAcceptor(md, ch, Acc, &ac, Res);
  }
  *NAcc = ac;
 if(ifDebug)
     for(int i=0; i<ac; i++)
      g_print("Acceptor %d A_Res %d AA_Res %d AA2_Res %d A_At %d AA_At %d AA2_At %d\n",
       i, Acc[i]->A_Res, Acc[i]->AA_Res, Acc[i]->AA2_Res,
       get_atom(md, Acc[i]->A_At, Acc[i]->A_Res, Acc[i]->Chain),
       get_atom(md, Acc[i]->AA_At, Acc[i]->AA_Res, Acc[i]->Chain),
       get_atom(md, Acc[i]->AA2_At, Acc[i]->AA2_Res, Acc[i]->Chain));

  return(ac);
}

vet3 NEWProject4_123(vet3 Coord1, vet3 Coord2, vet3 Coord3, vet3 Coord4)
{

/*
                          Atom4
   Atom3                  .
   \                     .
    \                   .    
     \                 .  .Proj4_123
      \               .. 
      Atom2-------Atom1

*/
  vet3 Vector21 = MRVectorSub(Coord1, Coord2);
  vet3 Vector23 = MRVectorSub(Coord3, Coord2);
  vet3 Vector14 = MRVectorSub(Coord4, Coord1);
  double Length_14 = MRVectorLength(Vector14);
  
  vet3 VectorNormal_123 = MRVectorCrossProduct(Vector21,Vector23);
  double NormalLength = MRVectorLength(VectorNormal_123);

  VectorNormal_123 = MRVectorNormalize(VectorNormal_123);

  double COS_Norm_14 = MRVectorDotProduct(VectorNormal_123, Vector14);

  COS_Norm_14 /= (Length_14*NormalLength);

  if( COS_Norm_14 < 0.0 ) {
    COS_Norm_14 = fabs(COS_Norm_14);
    VectorNormal_123 = MRVectorNegate(VectorNormal_123);
  }

  double Proj_14_Norm = Length_14*COS_Norm_14;

  VectorNormal_123 = MRVectorMultiplyScalar(VectorNormal_123, Proj_14_Norm);
  vet3 Coord_Proj4_123  = MRVectorAdd(MRVectorSub(Vector14, VectorNormal_123),
                           Coord1);
//  g_print("NEW coord %f %f %f\n", Coord_Proj4_123.v[0], Coord_Proj4_123.v[1], Coord_Proj4_123.v[2]);
  return Coord_Proj4_123;
}

double NewAngolo (vet3 p1, vet3 p2, vet3 p3)
{
   vet3 a = MRVectorSub(p1, p2);
   vet3 b = MRVectorSub(p3, p2);
   return MRVectorAngle(a, b);
}

void NEWHBOND_Energy(vet3 CA2, vet3 C, vet3 O, vet3 H, vet3 N, HBOND *HBond)
{

  vet3 ProjH;

 /***** Distance dependence ( 8-6 potential ) ****/

  if( HBond->AccDonDist < RmGRID )
    HBond->AccDonDist = RmGRID;
  HBond->Er = (float) (CGRID/pow(HBond->AccDonDist,8) - DGRID/pow(HBond->AccDonDist,6));

 /************** Angular dependance ****************/

  if(ifDebug)
  {
     g_print("CA %.4f %.4f %.4f \n", CA2.x, CA2.y, CA2.z);
     g_print("C  %.4f %.4f %.4f \n", C.x, C.y, C.z);
     g_print("O  %.4f %.4f %.4f \n", O.x, O.y, O.z);
     g_print("H  %.4f %.4f %.4f \n", H.x, H.y, H.z);
     g_print("N  %.4f %.4f %.4f \n", N.x, N.y, N.z);
  }
 /* Find projection of the hydrogen on the O-C-CA plane */
  ProjH = NEWProject4_123(O, C, CA2, H);

  if(ifDebug)
     g_print("ProjH  %.4f %.4f %.4f \n", ProjH.v[0], ProjH.v[1], ProjH.v[2]);

 /* Three angles determining the direction of the hydrogen bond */
  HBond->ti = (float) fabs(180.0 - NewAngolo(ProjH, O, C));
  HBond->to = (float) NewAngolo(H, O, ProjH);
  HBond->p  = (float) NewAngolo(N, H, O);
//  g_print("NEWti to p %f %f %f\n", HBond->ti, HBond->to, HBond->p);

 /* Calculate both angle-dependent HB energy components Et and Ep */
  if( HBond->ti >= 0.0 && HBond->ti < 90.0 )
    HBond->Et = (float) (cos(DEGTORAD(HBond->to))*(0.9+0.1*sin(DEGTORAD(2*HBond->ti))));
  else
  if( HBond->ti >= 90.0 && HBond->ti < 110.0 )
    HBond->Et = (float) (K1GRID*cos(DEGTORAD(HBond->to))*
      (pow((K2GRID-pow(cos(DEGTORAD(HBond->ti)),2)),3)));
  else
    HBond->Et = 0.0f;

  if( HBond->p > 90.0 && HBond->p < 270.0 )
    HBond->Ep = (float) pow(cos(DEGTORAD(HBond->p)),2);
  else
    HBond->Ep = 0.0;

    /******** Full hydrogen bond energy *********************/
  HBond->Energy = 1000.f*HBond->Er*HBond->Et*HBond->Ep;
//  g_print("NEWHBond->Energy %f\n", HBond->Energy);
}

#if 0
void Project4_123(float *Coord1, float *Coord2, float *Coord3, float *Coord4, 
                  float *Coord_Proj4_123)
{

/*
                          Atom4
   Atom3                  .
   \                     .
    \                   .    
     \                 .  .Proj4_123
      \               .. 
      Atom2-------Atom1

*/


  float Vector21[3], Vector23[3], Vector14[3], VectorNormal_123[3];
  float Length_21 = 0.0, Length_23 = 0.0, Length_14 = 0.0, NormalLength;
  float COS_Norm_14, Proj_14_Norm;
  int i;

  for( i=0; i<3; i++ ) {
    Vector21[i] = Coord1[i] - Coord2[i];
    Vector23[i] = Coord3[i] - Coord2[i];
    Vector14[i] = Coord4[i] - Coord1[i];
    Length_21 += Vector21[i]*Vector21[i];
    Length_23 += Vector23[i]*Vector23[i];
    Length_14 += Vector14[i]*Vector14[i];
  }
  
  Length_21 = sqrt(Length_21);
  Length_23 = sqrt(Length_23);
  Length_14 = sqrt(Length_14);

  NormalLength = vettoreXvettore(Vector21,Vector23,VectorNormal_123);

  for( i=0; i<3; i++ ) 
    VectorNormal_123[i] /= NormalLength;

  COS_Norm_14 = 0.0;

  for( i=0; i<3; i++ ) 
    COS_Norm_14 += VectorNormal_123[i]*Vector14[i];

  COS_Norm_14 /= (Length_14*NormalLength);

  if( COS_Norm_14 < 0.0 ) {
    COS_Norm_14 = fabs(COS_Norm_14);
    for( i=0; i<3; i++ ) 
      VectorNormal_123[i] = -VectorNormal_123[i];
  }

  Proj_14_Norm = Length_14*COS_Norm_14;

  for( i=0; i<3; i++ ) {
    VectorNormal_123[i] *= Proj_14_Norm;
    Coord_Proj4_123[i] = (Vector14[i] - VectorNormal_123[i]) + Coord1[i];
  }
//  g_print("coord %f %f %f\n", Coord_Proj4_123[0], Coord_Proj4_123[1], Coord_Proj4_123[2]);
}

void HBOND_Energy(float *CA2, float *C, float *O, float *H, float *N, HBOND *HBond)
{

  float ProjH[3];

 /***** Distance dependence ( 8-6 potential ) ****/

  if( HBond->AccDonDist < RmGRID )
    HBond->AccDonDist = RmGRID;
  HBond->Er = (float) (CGRID/pow(HBond->AccDonDist,8) - DGRID/pow(HBond->AccDonDist,6));

 /************** Angular dependance ****************/

  if(ifDebug)
  {
     g_print("CA %.4f %.4f %.4f \n", CA2[0], CA2[1], CA2[2]);
     g_print("C  %.4f %.4f %.4f \n", C[0], C[1], C[2]);
     g_print("O  %.4f %.4f %.4f \n", O[0], O[1], O[2]);
     g_print("H  %.4f %.4f %.4f \n", H[0], H[1], H[2]);
     g_print("N  %.4f %.4f %.4f \n", N[0], N[1], N[2]);
  }
 /* Find projection of the hydrogen on the O-C-CA plane */
  Project4_123(O,C,CA2,H,ProjH);

  if(ifDebug)
     g_print("ProjH  %.4f %.4f %.4f \n", ProjH[0], ProjH[1], ProjH[2]);

 /* Three angles determining the direction of the hydrogen bond */
  HBond->ti = (float) fabs(180.0 - Angolo(ProjH,O,C));
  HBond->to = Angolo(H,O,ProjH);
  HBond->p  = Angolo(N,H,O);
//  g_print("ti to p %f %f %f\n", HBond->ti, HBond->to, HBond->p);

 /* Calculate both angle-dependent HB energy components Et and Ep */
  if( HBond->ti >= 0.0 && HBond->ti < 90.0 )
    HBond->Et = (float) (cos(DEGTORAD(HBond->to))*(0.9+0.1*sin(DEGTORAD(2*HBond->ti))));
  else
  if( HBond->ti >= 90.0 && HBond->ti < 110.0 )
    HBond->Et = (float) (K1GRID*cos(DEGTORAD(HBond->to))*
      (pow((K2GRID-pow(cos(DEGTORAD(HBond->ti)),2)),3)));
  else
    HBond->Et = 0.0f;

  if( HBond->p > 90.0 && HBond->p < 270.0 )
    HBond->Ep = (float) pow(cos(DEGTORAD(HBond->p)),2);
  else
    HBond->Ep = 0.0;

    /******** Full hydrogen bond energy *********************/
  HBond->Energy = 1000.f*HBond->Er*HBond->Et*HBond->Ep;
//  g_print("HBond->Energy %f\n", HBond->Energy);

}
#endif

void PrintHydrBond(Model *md, int i, HBOND *HBond)
{

  int d = HBond->Dnr->D_Res;
  int chd = HBond->Dnr->Chain;
  int a = HBond->Acc->A_Res;
  int cha = HBond->Acc->Chain;
  std::string complete_name = g_get_current_dir();
  complete_name.append(G_DIR_SEPARATOR_S);
  complete_name.append(md->modname);
  std::string resa = md->Chains[cha].AminoRes[a].name;
  std::string resd = md->Chains[chd].AminoRes[d].name;

  g_print("HB %3d %20s %12s %4d   <> %12s %4d  ",i,
          complete_name.c_str(),
          resd.c_str(),
          d,
          resa.c_str(),
          a);
  g_print(" %7.1f ",HBond->AccDonDist);
  if( HBond->ExistPolarInter )
    g_print("%7.1f ",HBond->Energy);
  else
    g_print("XXXXXXX ");

  if( HBond->ExistHydrBondRose )
    g_print("YES ");
  else
    g_print("NO ");

  if( HBond->ExistHydrBondBaker )
    g_print("YES\n");
  else
    g_print("NO\n");

}

int FindHydrogenBonds(Model *md, HBOND **HBond, std::string &msg)
{
using namespace gtkext;
    Donor **Dnr;
    Acceptor **Acc;
    bool *BondedDonor, *BondedAcceptor;
    int NDnr=0, NAcc=0;
    int dc, ac, hc=0, i;

    Dnr = g_new0( Donor *, md->nResTot);
    Acc = g_new0( Acceptor *, md->nResTot);
  
    FindDnr(md, Dnr,&NDnr);
    FindAcc(md, Acc,&NAcc);

    BondedDonor    = g_new0( bool, NDnr);
    BondedAcceptor = g_new0( bool, NAcc);
    for( i=0; i<NDnr; i++ )
      BondedDonor[i] = false;
    for( i=0; i<NAcc; i++ )
      BondedAcceptor[i] = false;

    if(ifDebug)
       g_print("Find Hydrogen Bonds\n");
    for( dc=0; dc<NDnr; dc++ ) 
    {
         for( ac=0; ac<NAcc; ac++ ) 
         {
              if( (abs(Acc[ac]->A_Res - Dnr[dc]->D_Res) < 2) && 
                  (Acc[ac]->Chain == Dnr[dc]->Chain) ) 
                   continue;
              if( hc == MAXHYDRBOND ) 
              {
                  msg = g_strdup_printf("Number of hydrogen bonds exceeds current limit of %d \n",
                      MAXHYDRBOND);
                  return 0;
              }
              HBond[hc] = g_new0(HBOND, 1);
              HBond[hc]->ExistHydrBondRose = false;
              HBond[hc]->ExistHydrBondBaker = false;
              HBond[hc]->ExistPolarInter = false;
      
              vet3 n_d = {{md->atm[Dnr[dc]->D_At].xyz[0], 
                           md->atm[Dnr[dc]->D_At].xyz[1], 
                           md->atm[Dnr[dc]->D_At].xyz[2]}}; 
              vet3 h_d;
              for(int jj=0; jj<3; jj++)
              {
                  if(Dnr[dc]->H >= 0)
                     h_d.v[jj] = md->atm[Dnr[dc]->H].xyz[jj];
                  else
                     h_d.v[jj] = md->c_alpha[-(1+Dnr[dc]->H)].Hxyz[jj];
              }
              vet3 c_d = {{md->atm[Dnr[dc]->DD_At].xyz[0], 
                           md->atm[Dnr[dc]->DD_At].xyz[1], 
                           md->atm[Dnr[dc]->DD_At].xyz[2]}}; 
              vet3 ca_d = {{md->atm[Dnr[dc]->DDI_At].xyz[0], 
                            md->atm[Dnr[dc]->DDI_At].xyz[1], 
                            md->atm[Dnr[dc]->DDI_At].xyz[2]}}; 
              vet3 o_a = {{md->atm[Acc[ac]->A_At].xyz[0], 
                           md->atm[Acc[ac]->A_At].xyz[1], 
                           md->atm[Acc[ac]->A_At].xyz[2]}}; 
              vet3 c_a = {{md->atm[Acc[ac]->AA_At].xyz[0], 
                           md->atm[Acc[ac]->AA_At].xyz[1], 
                           md->atm[Acc[ac]->AA_At].xyz[2]}}; 
              vet3 ca_a = {{md->atm[Acc[ac]->AA2_At].xyz[0], 
                            md->atm[Acc[ac]->AA2_At].xyz[1], 
                            md->atm[Acc[ac]->AA2_At].xyz[2]}}; 
              if( (HBond[hc]->AccDonDist = MRVectorDistance(n_d, o_a) ) <= CUTOFF ) 
              {
                 if( Dnr[dc]->H != ERR) 
                 {
                     if(ifDebug)
                        g_print("hc %d\n", hc);
//                     HBOND_Energy(ca_a, c_a, o_a, h_d, n_d, HBond[hc]); 
                     NEWHBOND_Energy(ca_a, c_a, o_a, h_d, n_d, HBond[hc]); 
                     if( (HBond[hc]->Energy < -10.0) ) 
                         HBond[hc]->ExistPolarInter = true;
                 }
                 HBond[hc]->OHDist = MRVectorDistance(h_d, o_a);
                 HBond[hc]->AngNHO = NewAngolo(n_d, h_d, o_a);
                 HBond[hc]->AngCOH = NewAngolo(c_a, o_a, h_d);
                 if( (HBond[hc]->OHDist <= 2.5) &&
                     (HBond[hc]->AngNHO  >= 90.0) &&
                     (HBond[hc]->AngNHO <= 180.0) &&
                     (HBond[hc]->AngCOH >= 90.0) &&
                     (HBond[hc]->AngCOH <= 180.0 ))
                          HBond[hc]->ExistHydrBondBaker = true;
                 if( HBond[hc]->AccDonDist <= Dnr[dc]->HB_Radius+Acc[ac]->HB_Radius ) 
                 {
                     HBond[hc]->AccAng = NewAngolo(n_d, o_a, c_a);
                     if( ( HBond[hc]->AccAng >= MINACCANG_SP2 ) && 
                         ( HBond[hc]->AccAng <= MAXACCANG_SP2 ) ) 
                     {
                         HBond[hc]->DonAng = NewAngolo(o_a, n_d, c_d);
                         if( ( HBond[hc]->DonAng >= MINDONANG_SP2 && 
                             HBond[hc]->DonAng <= MAXDONANG_SP2 ) ) 
                         {
                             HBond[hc]->AccDonAng = MRabs(MRVectorDihedral(ca_d, n_d, c_d, o_a));
                             if( HBond[hc]->AccDonAng > 90.0 && HBond[hc]->AccDonAng < 270.0 )
                                 HBond[hc]->AccDonAng = MRabs(180.0 - HBond[hc]->AccDonAng);
                             HBond[hc]->DonAccAng = MRabs(MRVectorDihedral(n_d, o_a, c_a, ca_a));
                             if(HBond[hc]->DonAccAng > 90.0 && HBond[hc]->DonAccAng < 270.0)
                                HBond[hc]->DonAccAng = MRabs(180.0 - HBond[hc]->DonAccAng);
                             if( (HBond[hc]->AccDonAng <= ACCDONANG) &&
                                 (HBond[hc]->DonAccAng <= DONACCANG ) )
                                  HBond[hc]->ExistHydrBondRose = true;
                          }
                      }
                 }
              }
              if( (HBond[hc]->ExistPolarInter && HBond[hc]->Energy < 0.0) 
                || HBond[hc]->ExistHydrBondRose || HBond[hc]->ExistHydrBondBaker ) 
              {
                 HBond[hc]->Dnr = Dnr[dc];
                 HBond[hc]->Acc = Acc[ac];
                 BondedDonor[dc] = true;
                 BondedAcceptor[ac] = true;
                 int ddc = md->GetNumResiduo(Dnr[dc]->Chain, Dnr[dc]->D_Res);
                 if(involved[ddc].NBondDnr < MAXRESDNR)
                    involved[ddc].HBondDnr[involved[ddc].NBondDnr++] = hc;
                 else
                 {
                     msg = "Residue involved in more hydrogen bonds than allowed";
//                     return 0;
                 }
                 int aac = md->GetNumResiduo(Acc[ac]->Chain, Acc[ac]->A_Res);
                 if(involved[aac].NBondAcc < MAXRESACC)
                    involved[aac].HBondAcc[involved[aac].NBondAcc++] = hc;
                 else
                 {
                     msg = "Residue involved in more hydrogen bonds than allowed";
//                     return 0;
                 }
                 int ccd = Dnr[dc]->Chain;
                 int cca = Acc[ac]->Chain;
                 if( ccd != cca)
                 {
                     involved[ddc].InterchainHBonds = true;
                     involved[aac].InterchainHBonds = true;
                 }
                 hc++;
              }
              else
                 g_free(HBond[hc]);
         }
    }

    if(ifDebug)
       g_print("------------------- Hydrogen Bonds -------------------\n");
    for( i=0; i<hc; i++ ) 
    {
      if( (HBond[i]->Energy < 0.0)  && ifDebug )
        PrintHydrBond(md, i,HBond[i]);
    }

    for( i=0; i<NDnr; i++ )
      if( !BondedDonor[i] )
        g_free(Dnr[i]);
    for( i=0; i<NAcc; i++ )
      if( !BondedAcceptor[i] )
        g_free(Acc[i]);
    if( NDnr )
      g_free(BondedDonor);
    if( NAcc )
      g_free(BondedAcceptor);
    return(hc);
}

int FindPolInt(HBOND **HBond, int Res1, int Res2)
{

  register int i, j, hb;
  Involved *p1, *p2;

  p1 = &involved[Res1];
  p2 = &involved[Res2];

  if( p1->NBondDnr && p2->NBondAcc ) 
  {
      for( i=0; i<p1->NBondDnr; i++ ) 
      {
           hb = p1->HBondDnr[i];
           for( j=0; j<p2->NBondAcc; j++ )
                if( hb == p2->HBondAcc[j] && HBond[hb]->ExistPolarInter ) 
                    return(hb);
      }
  }
  return(ERR);
}

int FindBnd(HBOND **HBond, int Res1, int Res2)
{

  register int i, j, hb;
  Involved *p1, *p2;

  p1 = &involved[Res1];
  p2 = &involved[Res2];

  if( p1->NBondDnr && p2->NBondAcc ) 
  {
      for( i=0; i<p1->NBondDnr; i++ ) 
      {
            hb = p1->HBondDnr[i];
            for( j=0; j<p2->NBondAcc; j++ )
                  if( hb == p2->HBondAcc[j] && HBond[hb]->ExistHydrBondRose ) 
                      return(hb);
      }
  }
  return(ERR);
}

int NoDoubleHBond(Model *md, HBOND **HBond, int NHBond)
{

  int i, j, NExcl=0;

  for( i=0; i<NHBond-1; i++ )
  {
       for( j=i+1; j<NHBond; j++ ) 
       {
           if( HBond[i]->Dnr->D_Res == HBond[j]->Dnr->D_Res &&
               HBond[i]->Dnr->Chain == HBond[j]->Dnr->Chain &&
               HBond[i]->ExistPolarInter && HBond[j]->ExistPolarInter ) 
           {
              if( HBond[i]->Energy < 5.0*HBond[j]->Energy ) 
              {
                  HBond[j]->ExistPolarInter = false;
                  NExcl++;
              }
              else if( HBond[j]->Energy < 5.0*HBond[i]->Energy ) 
              {
                   HBond[i]->ExistPolarInter = false;
                   NExcl++;
              }
           }
       }
  }
  if(ifDebug)
     g_print("Excluded %d hydrogen bonds\n", NExcl);
  return(NExcl);
}

float **DefaultHelixMap()
{
  register int i;

  float **Map;
  static float Data[DEFNUMPIXEL][DEFNUMPIXEL] = {

      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0009014423f, 0.0041898815f,
      0.0085105160f, 0.0133839026f, 0.0245425366f, 0.0407802090f, 0.0464176536f, 0.0330946408f,
      0.0134803243f, 0.0024038462f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0007370283f, 0.0077203326f, 0.0269849468f,
      0.0492307022f, 0.0621860325f, 0.0747849122f, 0.0919913873f, 0.0918549150f, 0.0617070347f,
      0.0241584498f, 0.0041428790f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0041416897f, 0.0287234355f, 0.0835687742f,
      0.1384727061f, 0.1562444866f, 0.1470608264f, 0.1360232681f, 0.1159155145f, 0.0742164999f,
      0.0290896539f, 0.0050673936f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0009375000f, 0.0156580955f, 0.0757770315f, 0.1856354773f,
      0.2785892785f, 0.2880102694f, 0.2332847565f, 0.1741978228f, 0.1281246394f, 0.0793832615f,
      0.0320557840f, 0.0058840578f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0048893229f, 0.0437000208f, 0.1617751122f, 0.3399706185f,
      0.4626395404f, 0.4418565035f, 0.3235570788f, 0.2100441158f, 0.1358627081f, 0.0776144490f,
      0.0297011137f, 0.0052390974f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0136979166f, 0.0917820632f, 0.2773087323f, 0.5047551394f,
      0.6214492917f, 0.5485223532f, 0.3655386865f, 0.2054343373f, 0.1121114418f, 0.0548815951f,
      0.0178668182f, 0.0025975490f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0246484373f, 0.1396044195f, 0.3594934344f, 0.5710113049f,
      0.6337110400f, 0.5133636594f, 0.3054708838f, 0.1402616948f, 0.0584463216f, 0.0228670351f,
      0.0058531328f, 0.0005151099f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0265885405f, 0.1365883052f, 0.3163702190f, 0.4545661211f,
      0.4628692269f, 0.3425511420f, 0.1761947423f, 0.0607788190f, 0.0158569515f, 0.0042061093f,
      0.0008107311f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0152018229f, 0.0738445148f, 0.1630392224f, 0.2269553691f,
      0.2237145752f, 0.1528334022f, 0.0652616471f, 0.0150429625f, 0.0014589608f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0035156249f, 0.0165251363f, 0.0379281938f, 0.0584417619f,
      0.0619409233f, 0.0404052660f, 0.0136552500f, 0.0016678370f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0011718750f, 0.0046875002f,
      0.0070312503f, 0.0046875002f, 0.0011718750f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0006944445f, 0.0036063762f, 0.0080820229f, 0.0101532144f, 0.0076146079f,
      0.0032324446f, 0.0006009616f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f }
  };

  Map = g_new0(float *, DEFNUMPIXEL);

  for( i=0; i<DEFNUMPIXEL; i++ )
    Map[i] = &(Data[i][0]);

  return(Map);
}

float **DefaultSheetMap()
{

  register int i;
  float **Map;
  static float Data[DEFNUMPIXEL][DEFNUMPIXEL] = {

      {0.2769023776f, 0.1408346891f, 0.0464910716f, 0.0073784725f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0047086575f, 0.0218229108f, 0.0569166169f,
      0.1254088134f, 0.2340224832f, 0.3511219919f, 0.4355685711f, 0.4584180117f, 0.4007356465f},
      {0.4067636132f, 0.2329865396f, 0.0927943364f, 0.0237838365f, 0.0055147060f, 0.0013786765f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0088186050f, 0.0420726910f, 0.1043856740f,
      0.2086037844f, 0.3677131534f, 0.5367187858f, 0.6412357688f, 0.6458424330f, 0.5580080152f},
      {0.4286311865f, 0.2678007782f, 0.1282834113f, 0.0529448465f, 0.0220588241f, 0.0055147060f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0086062262f, 0.0445192643f, 0.1197573245f,
      0.2487278134f, 0.4369854629f, 0.6241853237f, 0.7160459757f, 0.6829043031f, 0.5716546178f},
      {0.3639202416f, 0.2397334576f, 0.1305907220f, 0.0683420748f, 0.0330882370f, 0.0082720593f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0053559211f, 0.0328565054f, 0.1048930883f,
      0.2402425259f, 0.4295993447f, 0.6026929021f, 0.6669865251f, 0.6039550304f, 0.4841639400f},
      {0.2637948096f, 0.1723874062f, 0.0920098722f, 0.0464194641f, 0.0220588241f, 0.0055147060f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0030202419f, 0.0224239044f, 0.0804052502f,
      0.1923188865f, 0.3456886411f, 0.4811576009f, 0.5223571062f, 0.4586051404f, 0.3565762639f},
      {0.1628032923f, 0.0930610597f, 0.0400134660f, 0.0143100554f, 0.0055147060f, 0.0013786765f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0015453297f, 0.0132468110f, 0.0489843786f,
      0.1174781919f, 0.2150468081f, 0.3082944453f, 0.3439011276f, 0.3080393970f, 0.2371628135f},
      {0.0825822726f, 0.0338854715f, 0.0092895878f, 0.0012122844f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0003863324f, 0.0046614520f, 0.0186656341f,
      0.0477515720f, 0.0961741805f, 0.1546680480f, 0.1961039603f, 0.1944279373f, 0.1469529718f},
      {0.0326442868f, 0.0073916214f, 0.0008854167f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0006347656f, 0.0031504754f,
      0.0104655549f, 0.0272454955f, 0.0570511036f, 0.0941907763f, 0.1088592261f, 0.0785619915f},
      {0.0090501504f, 0.0007651417f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0007207961f, 0.0035958111f, 0.0131648667f, 0.0318824202f, 0.0425693691f, 0.0292618107f},
      {0.0013020834f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0013020834f, 0.0052083335f, 0.0078125000f, 0.0052083335f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f},
      {0.0210939310f, 0.0078523019f, 0.0013020834f, 0.0000000000f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0014204546f,
      0.0071634995f, 0.0169352461f, 0.0272206441f, 0.0357281528f, 0.0395361669f, 0.0343801714f},
      {0.1146211401f, 0.0503530800f, 0.0130920913f, 0.0015190972f, 0.0000000000f, 0.0000000000f,
      0.0000000000f, 0.0000000000f, 0.0000000000f, 0.0010016026f, 0.0046167620f, 0.0157516468f,
      0.0453012958f, 0.0937970504f, 0.1454590708f, 0.1861637682f, 0.2019522935f, 0.1764564067f}
  };

 
  Map = g_new0(float *, DEFNUMPIXEL);

  for( i=0; i<DEFNUMPIXEL; i++ )
    Map[i] = &(Data[i][0]);

  return(Map); 
}

int PlaceHydrogens(Model *md, std::vector<C_Alpha> &c_alpha)
{

  int i, PlacedCnt=0;
  float Length_N_C, Length_N_CA, Length_N_H;
  int hyd_pres = 0;

  if(ifDebug)
     g_print("Placing  hydrogens\n");
  for(int ch=0; ch<md->nChains; ch++)
  {
      for(int Res=1; Res<md->Chains[ch].nAmino; Res++)
      {
          if(!Is_Amino(md->Chains[ch].AminoRes[Res].code) || 
             !Is_Amino(md->Chains[ch].AminoRes[Res-1].code))
                 continue;
          int ca = md->GetNumResiduo(ch, Res);
          if(strcmp(md->Chains[ch].AminoRes[Res].name, "PRO") == 0)
             continue;
          if( (c_alpha[ca].h == -1) && (c_alpha[ca].n != -1) && 
              (c_alpha[ca-1].c != -1))
          {
              Length_N_C   = Distanza(md->atm[c_alpha[ca].n].xyz,md->atm[c_alpha[ca-1].c].xyz);
              Length_N_CA  = Distanza(md->atm[c_alpha[ca].n].xyz,md->atm[c_alpha[ca].atom].xyz);
              for( i=0; i<3; i++ )
                 c_alpha[ca].Hxyz[i] = md->atm[c_alpha[ca].n].xyz[i] -
                   ( (md->atm[c_alpha[ca-1].c].xyz[i] -  md->atm[c_alpha[ca].n].xyz[i])/Length_N_C +
                     (md->atm[c_alpha[ca].atom].xyz[i] -  md->atm[c_alpha[ca].n].xyz[i])/Length_N_CA );
              Length_N_H = Distanza(md->atm[c_alpha[ca].n].xyz, c_alpha[ca].Hxyz);
              for( i=0; i<3; i++ )
                    c_alpha[ca].Hxyz[i] = md->atm[c_alpha[ca].n].xyz[i] +
                    DIST_N_H*(c_alpha[ca].Hxyz[i]-md->atm[c_alpha[ca].n].xyz[i])/Length_N_H;
              md->Chains[ch].AminoRes[Res].h = -(ca+1);
              PlacedCnt++;
              if(ifDebug)
                 g_print("N. %d Res %d: %.4f %.4f %.4f\n", PlacedCnt, Res, 
                 c_alpha[ca].Hxyz[0], c_alpha[ca].Hxyz[1],c_alpha[ca].Hxyz[2]);
         }
         else if(c_alpha[ca-1].c != -1)
          hyd_pres = 1;
      }
  }
  if(ifDebug)
     g_print("Placed %d hydrogens\n", PlacedCnt);
  return(PlacedCnt+hyd_pres);
}

void DiscrPhiPsi(Model *md)
{
   for(int ch=0; ch<md->nChains; ch++)
   {
       for(int j=0; j< md->Chains[ch].nAmino; j++)
       {
           int curr = md->GetNumResiduo(ch, j);
           properties[curr].PhiZn = -1;
           properties[curr].PsiZn = -1;
           if( j != 0 )
           {
              for( int i=0; i<nPixel; i++ )
              {
                   if( properties[curr].Phi  >  -180+(float)(i)*PhiPsiStep && 
                       properties[curr].Phi <=  -180+(float)(i+1)*PhiPsiStep ) 
                   {
                       properties[curr].PhiZn = i;
                       break;
                   }
              }
           }
           if( j != (md->Chains[ch].nAmino-1) )
           {
               for( int i=0; i<nPixel; i++ )
               {
                    if( properties[curr].Psi  >  -180+(float)(i)*PhiPsiStep && 
                        properties[curr].Psi <=  -180+(float)(i+1)*PhiPsiStep ) 
                    {
                        properties[curr].PsiZn = i;
                        break;
                    }
               }
          }
       }
       for(int j=0; j< md->Chains[ch].nAmino; j++)
       {
           int curr = md->GetNumResiduo(ch, j);
           if( (j != 0) && (properties[curr].PsiZn == -1) )
               properties[curr].PsiZn = properties[curr-1].PsiZn;
           if( (j != (md->Chains[ch].nAmino-1)) && (properties[curr].PhiZn == -1) )
               properties[curr].PhiZn = properties[curr+1].PhiZn;
       }
   }
}

void Helix(Model *md,  int Cn, HBOND **HBond, float **PhiPsiMap)
{
  int BondNumb;
  float *Proba;
  ResidueProperty *rp;

  Proba = g_new0(float, md->Chains[Cn].nAmino);

  for( int i=0; i<md->Chains[Cn].nAmino; i++ )
    Proba[i] = 0.0; 

  for( int i=0; i<md->Chains[Cn].nAmino-5; i++ )
  {
    int Res = md->GetNumResiduo(Cn, i);    
    rp = &properties[Res];

    if( rp[0].PhiZn != ERR && rp[0].PsiZn != ERR &&
        rp[1].PhiZn != ERR && rp[1].PsiZn != ERR &&
        rp[2].PhiZn != ERR && rp[2].PsiZn != ERR &&
        rp[3].PhiZn != ERR && rp[3].PsiZn != ERR &&
        rp[4].PhiZn != ERR && rp[4].PsiZn != ERR )
    {
        if( (BondNumb = FindPolInt(HBond,Res+4,Res)) != ERR ) 
        {
             Proba[i] = HBond[BondNumb]->Energy*(0.5f*
                 (PhiPsiMap[rp[0].PhiZn][rp[0].PsiZn]+
                 PhiPsiMap[rp[4].PhiZn][rp[4].PsiZn]));
        }
    }
  }

  for( int i=0; i<md->Chains[Cn].nAmino-5; i++ )
  {
    if( (Proba[i] < Treshold_H1) && (Proba[i+1] < Treshold_H1) )
    {
      int Res = md->GetNumResiduo(Cn, i);    
      rp = &properties[Res];
      rp[1].Asn = 'H'; 
      rp[2].Asn = 'H'; 
      rp[3].Asn = 'H'; 
      if( rp[0].PhiZn!= ERR && rp[0].PsiZn != ERR &&
          PhiPsiMap[rp[0].PhiZn][rp[0].PsiZn] > Treshold_H3 )
          rp[0].Asn = 'H';
      if( rp[4].PhiZn != ERR && rp[4].PsiZn != ERR &&
          PhiPsiMap[rp[4].PhiZn][rp[4].PsiZn] > Treshold_H4 )
          rp[4].Asn = 'H';
    }
  }

  for( int i=0; i<md->Chains[Cn].nAmino-4; i++ )
  {
      int Res = md->GetNumResiduo(Cn, i);    
      rp = &properties[Res];
      if(  FindBnd(HBond,i+3,i) != ERR && FindBnd(HBond,i+4,i+1) != ERR &&
         ( (rp[1].Asn != 'H' && rp[2].Asn != 'H') ||
          (rp[2].Asn != 'H' && rp[3].Asn != 'H') ) ) 
      {
          rp[1].Asn = 'G'; 
          rp[2].Asn = 'G'; 
          rp[3].Asn = 'G'; 
      }
  }

  for( int i=0; i<md->Chains[Cn].nAmino-6; i++ )
  {
      int Res = md->GetNumResiduo(Cn, i);    
      rp = &properties[Res];
      if( FindBnd(HBond,i+5,i) != ERR && FindBnd(HBond,i+6,i+1) != ERR &&
        rp[1].Asn == 'C' && rp[2].Asn == 'C' && 
        rp[3].Asn == 'C' && rp[4].Asn == 'C' && 
        rp[5].Asn == 'C' ) 
      {
         rp[1].Asn = 'I'; 
         rp[2].Asn = 'I';
         rp[3].Asn = 'I'; 
         rp[4].Asn = 'I'; 
         rp[5].Asn = 'I'; 
      }
  }
  if( ifDebug)
  {
    g_print("After Helix Secondary structrure\n");
    for( int i=0; i<md->Chains[Cn].nAmino-4; i++ )
    {
      RESIDUE r = md->Chains[Cn].AminoRes[i];
      int Res = md->GetNumResiduo(Cn, i);    
      rp = &properties[Res];

      if( rp[0].PhiZn != ERR && rp[0].PsiZn != ERR &&
          rp[4].PhiZn != ERR && rp[4].PsiZn != ERR ) 
      {
        if(rp[0].Asn != ' ')
           g_print("%s (%d) %c %10.7f %8.5f %8.5f | %4d %4d\n", r.PDBnum,
              i, rp[0].Asn, Proba[i],
              PhiPsiMap[rp[0].PhiZn][rp[0].PsiZn],
              PhiPsiMap[rp[4].PhiZn][rp[4].PsiZn],
              rp[4].PhiZn,rp[4].PsiZn);
      }
    }
  }

  g_free(Proba);
}

int Link(HBOND **HBond, Model *md, int Cn1, int Cn2, int Res1_1, 
         int Res1_2, int Res2_2, int Res2_1, int CRes1, 
         int CRes2, float **PhiPsiMap, PATTERN **Pattern, int *NumPat, 
         const char *Text, float Treshold, int Test)
{

  int BondNumber1, BondNumber2, Flag = 0;
  float Prob1, Prob2, Conf, Coeff;
  float C1_E = -0.2f;
  float C2_E =  0.2f;

  if( (BondNumber1 = FindPolInt(HBond,Res1_1,Res1_2)) == ERR ) 
    return(0);

  if( (BondNumber2 = FindPolInt(HBond,Res2_2,Res2_1)) == ERR ) 
    return(0);

  if( CRes1 == -1 ) 
  {
    if( properties[CRes2].PhiZn == ERR || properties[CRes2].PsiZn == ERR )
      return(0);
    Conf = PhiPsiMap[properties[CRes2].PhiZn][properties[CRes2].PsiZn];
  }
  else if( CRes2 == -1 ) 
  {
    if( properties[CRes1].PhiZn == ERR || properties[CRes1].PsiZn == ERR )
      return(0);
    Conf = PhiPsiMap[properties[CRes1].PhiZn][properties[CRes1].PsiZn];
  }
  else 
  {
    if( properties[CRes2].PhiZn == ERR || properties[CRes2].PsiZn == ERR ||
        properties[CRes1].PhiZn == ERR || properties[CRes1].PsiZn == ERR )
      return(0);
    Conf = 
      0.5f*(PhiPsiMap[properties[CRes1].PhiZn][properties[CRes1].PsiZn]+
         PhiPsiMap[properties[CRes2].PhiZn][properties[CRes2].PsiZn]);
  }
  Coeff = 1+C1_E+C2_E*Conf;
  Prob1 = HBond[BondNumber1]->Energy*Coeff;
  Prob2 = HBond[BondNumber2]->Energy*Coeff;

  if( Prob1 < Treshold && Prob2 < Treshold ) {

    if( !Test ) {
      Pattern[*NumPat] = g_new0(PATTERN, 1);
      Pattern[*NumPat]->ExistPattern = true;
      Pattern[*NumPat]->Hb1 = HBond[BondNumber1];
      Pattern[*NumPat]->Hb2 = HBond[BondNumber2];
      Pattern[*NumPat]->Nei1 = NULL;
      Pattern[*NumPat]->Nei2 = NULL;
      CopiaStringa(Pattern[*NumPat]->Type, MaxLenFilename, Text);
      (*NumPat)++;
    }
    Flag = 1;
  }
  
  return(Flag);
  
}

bool MakeEnds(Model *md, int *Beg1, int ResBeg1, int NeiBeg1, char *Beg1Cn, char ResBeg1Cn, int *End1, 
               int ResEnd1, int NeiEnd1, char ResEnd1Cn, int *Beg2, int ResBeg2, int NeiBeg2, 
               char *Beg2Cn, char ResBeg2Cn, int *End2, int ResEnd2, int NeiEnd2, 
               char ResEnd2Cn, PATTERN **Pat, int NPat)
{

  register int i;
  int Flag1 = 0, Flag2 = 0;


  if( ResBeg1 <= NeiBeg1 && NeiBeg1 <= NeiEnd1 && NeiEnd1 <= ResEnd1 &&
      ResBeg2 <= NeiBeg2 && NeiBeg2 <= NeiEnd2 && NeiEnd2 <= ResEnd2 &&
      ResBeg1Cn == ResEnd1Cn && ResBeg2Cn == ResEnd2Cn ) {

    *Beg1 = ResBeg1;
    *End1 = ResEnd1;
    *Beg2 = ResBeg2;
    *End2 = ResEnd2;
    *Beg1Cn = ResBeg1Cn;
    *Beg2Cn = ResBeg2Cn;
    
    for( i=0; i<NPat && (Flag1 == 0 || Flag2 == 0); i++ ) {
      if( ( (Pat[i]->Hb1->Dnr->D_Res == (*Beg1) 
             && Pat[i]->Hb1->Acc->A_Res == (*End2)
             && md->Chains[Pat[i]->Hb1->Dnr->Chain].id[0] == (*Beg1Cn)   
             && md->Chains[Pat[i]->Hb1->Acc->Chain].id[0] == (*Beg2Cn) )
             ||
             (Pat[i]->Hb1->Acc->A_Res == (*Beg1) 
             && Pat[i]->Hb1->Dnr->D_Res == (*End2) 
             && md->Chains[Pat[i]->Hb1->Acc->Chain].id[0] == (*Beg1Cn) 
             && md->Chains[Pat[i]->Hb1->Dnr->Chain].id[0] == (*Beg2Cn) ) ) 
             && Pat[i]->Hb1->Dnr->D_Res == Pat[i]->Hb2->Acc->A_Res 
             && Pat[i]->Hb2->Dnr->D_Res == Pat[i]->Hb1->Acc->A_Res )
             Flag1 = 1; 
      if( ( (Pat[i]->Hb1->Dnr->D_Res == (*Beg2) 
             && Pat[i]->Hb1->Acc->A_Res == (*End1) 
             && md->Chains[Pat[i]->Hb1->Dnr->Chain].id[0] == (*Beg2Cn)    
             && md->Chains[Pat[i]->Hb1->Acc->Chain].id[0] == (*Beg1Cn) ) 
             ||
             (Pat[i]->Hb1->Acc->A_Res == (*Beg2) 
             && Pat[i]->Hb1->Dnr->D_Res == (*End1) 
             && md->Chains[Pat[i]->Hb1->Acc->Chain].id[0] == (*Beg2Cn)
             && md->Chains[Pat[i]->Hb1->Dnr->Chain].id[0] == (*Beg1Cn) ) ) 
             && Pat[i]->Hb1->Dnr->D_Res == Pat[i]->Hb2->Acc->A_Res 
             && Pat[i]->Hb2->Dnr->D_Res == Pat[i]->Hb1->Acc->A_Res )
             Flag2 = 1; 
    }
    
    if( !Flag1 ) {
      if( *Beg1 != NeiBeg1 ) (*Beg1)++;
      if( *End2 != NeiEnd2 ) (*End2)--;
    }
    
    if( !Flag2 ) {
      if( *End1 != NeiEnd1 ) (*End1)--;
      if( *Beg2 != NeiBeg2 ) (*Beg2)++;
    }
    return(true);
  }
  
  return(false);
}
  
void Alias(Model *md, int *D1,int *A1,int *D2,int *A2,char *D1Cn,char *A1Cn,char *D2Cn,char *A2Cn,
          PATTERN *Pat)
{
    *D1 = Pat->Hb1->Dnr->D_Res;
    *A1 = Pat->Hb1->Acc->A_Res;
    *D2 = Pat->Hb2->Dnr->D_Res;
    *A2 = Pat->Hb2->Acc->A_Res;
    *D1Cn = md->Chains[Pat->Hb1->Dnr->Chain].id[0]; 
    *A1Cn = md->Chains[Pat->Hb1->Acc->Chain].id[0];
    *D2Cn = md->Chains[Pat->Hb2->Dnr->Chain].id[0];
    *A2Cn = md->Chains[Pat->Hb2->Acc->Chain].id[0];
}

void FillAsnAntiPar(char *Asn1, char *Asn2, Model *md, int Cn1, int Cn2, 
             PATTERN **Pat, int NPat)
{
  register int i, j;
  int Beg1, Beg2, End1, End2;
  int B1D, B1A, B2D, B2A, E1D, E1A, E2D, E2A;
  char B1DCn, B1ACn, B2DCn, B2ACn, E1DCn, E1ACn, E2DCn, E2ACn, Beg1Cn, Beg2Cn; 
  PATTERN *CurrPat, *PrevPat;;

  for( i=0; i<NPat; i++ ) {

    if( Pat[i]->Nei1 != NULL && Pat[i]->Nei2 == NULL )
      CurrPat = Pat[i]->Nei1;
    else
    if( Pat[i]->Nei2 != NULL && Pat[i]->Nei1 == NULL ) 
      CurrPat = Pat[i]->Nei2;
    else 
      continue;
    
    PrevPat = Pat[i];
    while( CurrPat->Nei1 != NULL && CurrPat->Nei2 != NULL ) {
      
      if( (CurrPat->Nei1->Nei1 == CurrPat || CurrPat->Nei1->Nei2 == CurrPat) && 
             CurrPat->Nei1 != PrevPat ) {
             PrevPat = CurrPat;
             CurrPat = CurrPat->Nei1;
      }
      else 
      if( (CurrPat->Nei2->Nei1 == CurrPat || CurrPat->Nei2->Nei2 == CurrPat) && 
             CurrPat->Nei2 != PrevPat ) {
             PrevPat = CurrPat;
             CurrPat = CurrPat->Nei2;
      }
      else 
             break;
    }  
    
    Alias(md,&B1D,&B1A,&B2D,&B2A,&B1DCn,&B1ACn,&B2DCn,&B2ACn,Pat[i]);
    Alias(md,&E1D,&E1A,&E2D,&E2A,&E1DCn,&E1ACn,&E2DCn,&E2ACn,CurrPat);

    if( (Cn1 != Cn2 || E1D - B2A <  E2D - B2A ) &&
        ( MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E2A,E1D,E2ACn,&Beg2,E2D,E1A,&Beg2Cn,E2DCn,
             &End2,B1A,B2D,B1ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E1D,E2A,E1DCn,&Beg2,E1A,E2D,&Beg2Cn,E1ACn,
             &End2,B1A,B2D,B1ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || E2D - B2A <  E1D - B2A ) && 
        ( MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E1A,E2D,E1ACn,&Beg2,E1D,E2A,&Beg2Cn,E1DCn,
             &End2,B1A,B2D,B1ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E2D,E1A,E2DCn,&Beg2,E2A,E1D,&Beg2Cn,E2ACn,
             &End2,B1A,B2D,B1ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || B2A - E1D < B2A - E2D ) && 
        ( MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E2D,E1A,E2DCn,&Beg2,E2A,E1D,&Beg2Cn,E2ACn,
             &End2,B1D,B2A,B1DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E1A,E2D,E1ACn,&Beg2,E1D,E2A,&Beg2Cn,E1DCn,
             &End2,B1D,B2A,B1DCn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || B2A - E2D < B2A - E1D ) && 
        ( MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E1D,E2A,E1DCn,&Beg2,E1A,E2D,&Beg2Cn,E1ACn,
             &End2,B1D,B2A,B1DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E2A,E1D,E2ACn,&Beg2,E2D,E1A,&Beg2Cn,E2DCn,
             &End2,B1D,B2A,B1DCn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || B1D - E2A <  B2D - E2A ) && 
        ( MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B2A,B1D,B2ACn,&Beg2,B2D,B1A,&Beg2Cn,B2DCn,
             &End2,E1A,E2D,E1ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B1D,B2A,B1DCn,&Beg2,B1A,B2D,&Beg2Cn,B1ACn,
             &End2,E1A,E2D,E1ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || B2D - E2A <  B1D - E2A ) && 
        ( MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B1A,B2D,B1ACn,&Beg2,B1D,B2A,&Beg2Cn,B1DCn,
             &End2,E1A,E2D,E1ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B2D,B1A,B2DCn,&Beg2,B2A,B1D,&Beg2Cn,B2ACn,
             &End2,E1A,E2D,E1ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || E2A - B1D < E2A - B2D ) && 
        ( MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B2D,B1A,B2DCn,&Beg2,B2A,B1D,&Beg2Cn,B2ACn,
             &End2,E1D,E2A,E1DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B1A,B2D,B1ACn,&Beg2,B1D,B2A,&Beg2Cn,B1DCn,
             &End2,E1D,E2A,E1DCn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || E2A - B2D < E2A - B1D ) && 
        ( MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B1D,B2A,B1DCn,&Beg2,B1A,B2D,&Beg2Cn,B1ACn,
             &End2,E1D,E2A,E1DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B2A,B1D,B2ACn,&Beg2,B2D,B1A,&Beg2Cn,B2DCn,
             &End2,E1D,E2A,E1DCn,Pat,NPat) ) )
      ;
    else
      continue;


    if( Beg1Cn == md->Chains[Cn1].id[0] ) {
      for( j=Beg1; j<=End1; j++ ) 
             Asn1[j] = 'N';
      for( j=Beg2; j<=End2; j++ ) 
             Asn2[j] = 'N';
    }
    else {
      for( j=Beg1; j<=End1; j++ ) 
             Asn2[j] = 'N';
      for( j=Beg2; j<=End2; j++ ) 
             Asn1[j] = 'N';
    }

    Pat[i]->Nei1 = NULL;
    Pat[i]->Nei2 = NULL;
    CurrPat->Nei1 = NULL;
    CurrPat->Nei2 = NULL;

  }
}
  

void FillAsnPar(char *Asn1, char *Asn2, Model *md, int Cn1, int Cn2, 
             PATTERN **Pat, int NPat)
{
  register int i, j;
  int Beg1, Beg2, End1, End2;
  int B1D, B1A, B2D, B2A, E1D, E1A, E2D, E2A; 
  char B1DCn, B1ACn, B2DCn, B2ACn, E1DCn, E1ACn, E2DCn, E2ACn, Beg1Cn, Beg2Cn; 
  PATTERN *CurrPat, *PrevPat;;

  for( i=0; i<NPat; i++ ) {
    
    if( Pat[i]->Nei1 != NULL && Pat[i]->Nei2 == NULL )
      CurrPat = Pat[i]->Nei1;
    else
    if( Pat[i]->Nei2 != NULL && Pat[i]->Nei1 == NULL ) 
      CurrPat = Pat[i]->Nei2;
    else 
      continue;
    
    PrevPat = Pat[i];
    while( CurrPat->Nei1 != NULL && CurrPat->Nei2 != NULL ) {
      
      if( (CurrPat->Nei1->Nei1 == CurrPat || CurrPat->Nei1->Nei2 == CurrPat) && 
             CurrPat->Nei1 != PrevPat ) {
             PrevPat = CurrPat;
             CurrPat = CurrPat->Nei1;
      }
      else {
             PrevPat = CurrPat;
             CurrPat = CurrPat->Nei2;
      }
    }  

    Alias(md,&B1D,&B1A,&B2D,&B2A,&B1DCn,&B1ACn,&B2DCn,&B2ACn,Pat[i]);
    Alias(md,&E1D,&E1A,&E2D,&E2A,&E1DCn,&E1ACn,&E2DCn,&E2ACn,CurrPat);

    if( ( Cn1 != Cn2 || abs(E1D-B2A) < abs(E2D-B2A) ) && 
        ( MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E2A,E1D,E2ACn,&Beg2,B1A,B2D,&Beg2Cn,B1ACn,
             &End2,E2D,E1A,E2DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E1D,E2A,E1DCn,&Beg2,B1A,B2D,&Beg2Cn,B1ACn,
             &End2,E1A,E2D,E1ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || abs(E2D-B2A) < abs(E1D-B2A) ) && 
        ( MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E1A,E2D,E1ACn,&Beg2,B1A,B2D,&Beg2Cn,B1ACn,
             &End2,E1D,E2A,E1DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1D,B2A,&Beg1Cn,B1DCn,&End1,E2D,E1A,E2DCn,&Beg2,B1A,B2D,&Beg2Cn,B1ACn,
             &End2,E2A,E1D,E2ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || abs(B2A-E1D) < abs(B2A-E2D) ) && 
        ( MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E2D,E1A,E2DCn,&Beg2,B1D,B2A,&Beg2Cn,B1DCn,
             &End2,E2A,E1D,E2ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E1A,E2D,E1ACn,&Beg2,B1D,B2A,&Beg2Cn,B1DCn,
             &End2,E1D,E2A,E1DCn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || abs(B2A-E2D) < abs(B2A-E1D) ) && 
        ( MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E1D,E2A,E1DCn,&Beg2,B1D,B2A,&Beg2Cn,B1DCn,
             &End2,E1A,E2D,E1ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,B1A,B2D,&Beg1Cn,B1ACn,&End1,E2A,E1D,E2ACn,&Beg2,B1D,B2A,&Beg2Cn,B1DCn,
             &End2,E2D,E1A,E2DCn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || abs(B1D-E2A) < abs(B2D-E2A) ) && 
        ( MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B2A,B1D,B2ACn,&Beg2,E1A,E2D,&Beg2Cn,E1ACn,
             &End2,B2D,B1A,B2DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B1D,B2A,B1DCn,&Beg2,E1A,E2D,&Beg2Cn,E1ACn,
             &End2,B1A,B2D,B1ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || abs(B2D-E2A) < abs(B1D-E2A) ) && 
        ( MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B1A,B2D,B1ACn,&Beg2,E1A,E2D,&Beg2Cn,E1ACn,
             &End2,B1D,B2A,B1DCn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1D,E2A,&Beg1Cn,E1DCn,&End1,B2D,B1A,B2DCn,&Beg2,E1A,E2D,&Beg2Cn,E1ACn,
             &End2,B2A,B1D,B2ACn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || abs(E2A-B1D) < abs(E2A-B2D) ) && 
        ( MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B2D,B1A,B2DCn,&Beg2,E1D,E2A,&Beg2Cn,E1DCn,
             &End2,B2A,B1D,B2ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B1A,B2D,B1ACn,&Beg2,E1D,E2A,&Beg2Cn,E1DCn,
             &End2,B1D,B2A,B1DCn,Pat,NPat) ) )
      ;
    else
    if( ( Cn1 != Cn2 || abs(E2A-B2D) < abs(E2A-B1D) ) && 
        ( MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B1D,B2A,B1DCn,&Beg2,E1D,E2A,&Beg2Cn,E1DCn,
             &End2,B1A,B2D,B1ACn,Pat,NPat) ||
          MakeEnds(md, &Beg1,E1A,E2D,&Beg1Cn,E1ACn,&End1,B2A,B1D,B2ACn,&Beg2,E1D,E2A,&Beg2Cn,E1DCn,
             &End2,B2D,B1A,B2DCn,Pat,NPat) ) )
      ;
    else 
      continue;

    if( Beg1Cn == md->Chains[Cn1].id[0] ) {
      for( j=Beg1; j<=End1; j++ ) Asn1[j] = 'P';
      for( j=Beg2; j<=End2; j++ ) Asn2[j] = 'P';
    }
    else {
      for( j=Beg1; j<=End1; j++ ) Asn2[j] = 'P';
      for( j=Beg2; j<=End2; j++ ) Asn1[j] = 'P';
    }

    Pat[i]->Nei1 = NULL;
    Pat[i]->Nei2 = NULL;
    CurrPat->Nei1 = NULL;
    CurrPat->Nei2 = NULL;

  }
}
  

  
void FilterAntiPar(Model *md, PATTERN **Pat, int NPat)
{
 
  register int i, j;
  int I1A, I1D, I2A, I2D, J1A, J1D, J2A, J2D;
  char I1ACn, I1DCn, I2ACn, I2DCn, J1ACn, J1DCn, J2ACn, J2DCn;

  for( i=0; i<NPat; i++ ) {
    
    if( !Pat[i]->ExistPattern ) continue;

    Alias(md,&I1D,&I1A,&I2D,&I2A,&I1DCn,&I1ACn,&I2DCn,&I2ACn,Pat[i]);
    
    for( j=0; j<NPat; j++ ) {
      
      if( j == i || !Pat[j]->ExistPattern ) continue;
      
      Alias(md,&J1D,&J1A,&J2D,&J2A,&J1DCn,&J1ACn,&J2DCn,&J2ACn,Pat[j]);
      
      if( J1D == J2A && J2D == J1A && I1D != I2A && I2D != I1A &&
             ( (J1D == I1D && J1A == I1A) ||  (J1D == I1A && J1A == I1D) || 
             (J1D == I2A && J1A == I2D) ||  (J1D == I2D && J1A == I2A) ) ) continue;

      if( ( ( I1D < I2A || I2D < I1A ) && 
             ( (J1A <= I2A && J1A >= I1D && J2D <= I2A && J2D >= I1D && J2DCn == I1DCn &&
             J2A <= I1A && J2A >= I2D && J1D <= I1A && J1D >= I2D && J1DCn == I2DCn) ||
             (J2A <= I2A && J2A >= I1D && J1D <= I2A && J1D >= I1D && J1DCn == I1DCn &&
             J1A <= I1A && J1A >= I2D && J2D <= I1A && J2D >= I2D && J2DCn == I2DCn) ) ) || 
             ( ( I1D > I2A || I2D > I1A ) && 
             ( (J1A >= I2A && J1A <= I1D && J2D >= I2A && J2D <= I1D && J2DCn == I1DCn &&
             J2A >= I1A && J2A <= I2D && J1D >= I1A && J1D <= I2D && J1DCn == I2DCn) ||
             (J2A >= I2A && J2A <= I1D && J1D >= I2A && J1D <= I1D && J1DCn == I1DCn &&
             J1A >= I1A && J1A <= I2D && J2D >= I1A && J2D <= I2D && J2DCn == I2DCn) ) ) ) {
             Pat[j]->ExistPattern = false;
      }
    }
  }
}

void FilterPar(Model *md, PATTERN **Pat, int NPat)
{
 
  register int i, j;
  int I1A, I1D, I2A, I2D, J1A, J1D, J2A, J2D;
  char I1ACn, I1DCn, I2ACn, I2DCn, J1ACn, J1DCn, J2ACn, J2DCn;

  for( i=0; i<NPat; i++ ) {
    
    if( !Pat[i]->ExistPattern ) continue;

    Alias(md,&I1D,&I1A,&I2D,&I2A,&I1DCn,&I1ACn,&I2DCn,&I2ACn,Pat[i]);
    
    for( j=0; j<NPat; j++ ) {
      
      if( j == i || !Pat[j]->ExistPattern ) continue;
      
      Alias(md,&J1D,&J1A,&J2D,&J2A,&J1DCn,&J1ACn,&J2DCn,&J2ACn,Pat[j]);
      
      if( ( ( I1A >= I2D && I1D >= I2A ) && 
             ( (J1A >= I2A && J1A <= I1D && J2D >= I2A && J2D <= I1D && J2DCn == I1DCn &&
             J2A <= I1A && J2A >= I2D && J1D <= I1A && J1D >= I2D && J1DCn == I2DCn) ||
             (J2A >= I2A && J2A <= I1D && J1D >= I2A && J1D <= I1D && J1DCn == I1DCn &&
             J1A <= I1A && J1A >= I2D && J2D <= I1A && J2D >= I2D && J2DCn == I2DCn) ) ) || 

             ( I2A >= I1D && I2D >= I1A  && 
             ( (J1A <= I2A && J1A >= I1D && J2D <= I2A && J2D >= I1D && J2DCn == I1DCn &&
             J2A >= I1A && J2A <= I2D && J1D >= I1A && J1D <= I2D && J1DCn == I2DCn) ||

             (J2A <= I2A && J2A >= I1D && J1D <= I2A && J1D >= I1D && J1DCn == I1DCn &&
             J1A >= I1A && J1A <= I2D && J2D >= I1A && J2D <= I2D && J2DCn == I2DCn) ) ) ) {
             Pat[j]->ExistPattern = false;
      }
    }
  }
}
      
void PrintPatterns(PATTERN **Pat, int NPat, Model *md, int Cn1, int Cn2)
{

  int D1, A1, D2, A2;
  for( int i=0; i<NPat; i++ ) {
    if( !Pat[i]->ExistPattern ) continue;

    D1 = Pat[i]->Hb1->Dnr->D_Res;
    A1 = Pat[i]->Hb1->Acc->A_Res;
    D2 = Pat[i]->Hb2->Dnr->D_Res;
    A2 = Pat[i]->Hb2->Acc->A_Res;
    
    g_print("%3d %c %c ",
             i,md->Chains[Pat[i]->Hb1->Dnr->Chain].id[0],
               md->Chains[Pat[i]->Hb2->Dnr->Chain].id[0]);
    if( Pat[i]->Hb1->Dnr->Chain == Cn1)
      g_print("%3s(%3d) %3s(%3d) %3s(%3d) %3s(%3d)",
             md->Chains[Cn1].AminoRes[D1].PDBnum,D1,
             md->Chains[Cn2].AminoRes[A1].PDBnum,A1,
             md->Chains[Cn2].AminoRes[D2].PDBnum,D2,
             md->Chains[Cn1].AminoRes[A2].PDBnum,A2);
    else
      g_print("%3s(%3d) %3s(%3d) %3s(%3d) %3s(%3d)",
             md->Chains[Cn2].AminoRes[D1].PDBnum,D1,
             md->Chains[Cn1].AminoRes[A1].PDBnum,A1,
             md->Chains[Cn1].AminoRes[D2].PDBnum,D2,
             md->Chains[Cn2].AminoRes[A2].PDBnum,A2);
    
    if( Pat[i]->Nei1 != NULL ) {
      D1 = Pat[i]->Nei1->Hb1->Dnr->D_Res;
      A1 = Pat[i]->Nei1->Hb1->Acc->A_Res;
      D2 = Pat[i]->Nei1->Hb2->Dnr->D_Res;
      A2 = Pat[i]->Nei1->Hb2->Acc->A_Res;
      
      g_print(" N1 %c %c ",
             md->Chains[Pat[i]->Nei1->Hb1->Dnr->Chain].id[0],
             md->Chains[Pat[i]->Nei1->Hb2->Dnr->Chain].id[0]);
      if( Pat[i]->Nei1->Hb1->Dnr->Chain == Cn1)
             g_print("%3s(%3d) %3s(%3d) %3s(%3d) %3s(%3d) ",
             md->Chains[Cn1].AminoRes[D1].PDBnum,D1,
             md->Chains[Cn2].AminoRes[A1].PDBnum,A1,
             md->Chains[Cn2].AminoRes[D2].PDBnum,D2,
             md->Chains[Cn1].AminoRes[A2].PDBnum,A2);
      else
             g_print("%3s(%3d) %3s(%3d) %3s(%3d) %3s(%3d) ",
             md->Chains[Cn2].AminoRes[D1].PDBnum,D1,
             md->Chains[Cn1].AminoRes[A1].PDBnum,A1,
             md->Chains[Cn1].AminoRes[D2].PDBnum,D2,
             md->Chains[Cn2].AminoRes[A2].PDBnum,A2);
    }
    
    if( Pat[i]->Nei2 != NULL ) {
      D1 = Pat[i]->Nei2->Hb1->Dnr->D_Res;
      A1 = Pat[i]->Nei2->Hb1->Acc->A_Res;
      D2 = Pat[i]->Nei2->Hb2->Dnr->D_Res;
      A2 = Pat[i]->Nei2->Hb2->Acc->A_Res;
      g_print(" N2 %c %c ",
             md->Chains[Pat[i]->Nei2->Hb1->Dnr->Chain].id[0],
             md->Chains[Pat[i]->Nei2->Hb2->Dnr->Chain].id[0]);
      if( Pat[i]->Nei2->Hb1->Dnr->Chain == Cn1)
             g_print("%3s(%3d) %3s(%3d) %3s(%3d) %3s(%3d) ",
             md->Chains[Cn1].AminoRes[D1].PDBnum,D1,
             md->Chains[Cn2].AminoRes[A1].PDBnum,A1,
             md->Chains[Cn2].AminoRes[D2].PDBnum,D2,
             md->Chains[Cn1].AminoRes[A2].PDBnum,A2);
      else
             g_print("%3s(%3d) %3s(%3d) %3s(%3d) %3s(%3d) ",
             md->Chains[Cn2].AminoRes[D1].PDBnum,D1,
             md->Chains[Cn1].AminoRes[A1].PDBnum,A1,
             md->Chains[Cn1].AminoRes[D2].PDBnum,D2,
             md->Chains[Cn2].AminoRes[A2].PDBnum,A2);
    }
    g_print("\n");
  }
}  
    

void Bridge(char *Asn1, char *Asn2, Model *md, int Cn1, int Cn2, PATTERN **Pat, int NPat)
{

  register int i;
  int B_Res;
  
  for( i=0; i<NPat; i++ ) {
    if( Pat[i]->Nei1 != NULL || Pat[i]->Nei2 != NULL ) continue;

    if( !strcmp(Pat[i]->Type,"1331") && 
       ( Cn1 != Cn2 || abs(Pat[i]->Hb1->Dnr->D_Res-Pat[i]->Hb1->Acc->A_Res) >= 3 ) ) {
      
      if( Pat[i]->Hb1->Dnr->Chain == Cn1)
      {
             if( Asn1[Pat[i]->Hb1->Dnr->D_Res] == 'C' )
             Asn1[Pat[i]->Hb1->Dnr->D_Res] = 'B';
             if( Asn2[Pat[i]->Hb1->Acc->A_Res] == 'C' )
             Asn2[Pat[i]->Hb1->Acc->A_Res] = 'B';
      }
      else 
      {
             if( Asn2[Pat[i]->Hb1->Dnr->D_Res] == 'C' )
             Asn2[Pat[i]->Hb1->Dnr->D_Res] = 'B';
             if( Asn1[Pat[i]->Hb1->Acc->A_Res] == 'C' )
             Asn1[Pat[i]->Hb1->Acc->A_Res] = 'B';
      }

    }
    else
      if( !strcmp(Pat[i]->Type,"3124") && 
             ( Cn1 != Cn2 || 
             (abs(Pat[i]->Hb1->Dnr->D_Res-Pat[i]->Hb1->Acc->A_Res) >= 2 &&
             abs(Pat[i]->Hb2->Dnr->D_Res-Pat[i]->Hb2->Acc->A_Res) >= 2 ) ) ) {
      
             if( Pat[i]->Hb1->Dnr->Chain == Cn1 ) {
             
             if( Pat[i]->Hb1->Dnr->D_Res > Pat[i]->Hb2->Acc->A_Res )
             B_Res = Pat[i]->Hb1->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb1->Dnr->D_Res+1;

             if( Asn1[B_Res] == 'C' )
             Asn1[B_Res] = 'B';

             if( Pat[i]->Hb2->Dnr->D_Res > Pat[i]->Hb1->Acc->A_Res )
             B_Res = Pat[i]->Hb2->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb2->Dnr->D_Res+1;

             if( Asn2[B_Res] == 'C' )
             Asn2[B_Res] = 'B';
             }
             else {
             if( Pat[i]->Hb1->Dnr->D_Res > Pat[i]->Hb2->Acc->A_Res )
             B_Res = Pat[i]->Hb1->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb1->Dnr->D_Res+1;

             if( Asn2[B_Res] == 'C' )
             Asn2[B_Res] = 'B';

             if( Pat[i]->Hb2->Dnr->D_Res > Pat[i]->Hb1->Acc->A_Res )
             B_Res = Pat[i]->Hb2->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb2->Dnr->D_Res+1;

             if( Asn1[B_Res] == 'C' )
             Asn1[B_Res] = 'B';
             }
      }
      else
             if( ( ( !strcmp(Pat[i]->Type,"3123") || !strcmp(Pat[i]->Type,"1341") ) &&
             ( Cn1 != Cn2 ||
             (abs(Pat[i]->Hb1->Dnr->D_Res-Pat[i]->Hb1->Acc->A_Res) > 3 &&
             abs(Pat[i]->Hb2->Dnr->D_Res-Pat[i]->Hb2->Acc->A_Res) > 3 ) ) ) ) { 

             if( Pat[i]->Hb1->Dnr->Chain == Cn1 ) {
             
             if( Pat[i]->Hb1->Dnr->D_Res == Pat[i]->Hb2->Acc->A_Res ) {
             
             if( Asn1[Pat[i]->Hb1->Dnr->D_Res] == 'C' )
             Asn1[Pat[i]->Hb1->Dnr->D_Res] = 'B';
             
             if( Pat[i]->Hb2->Dnr->D_Res > Pat[i]->Hb1->Acc->A_Res )
             B_Res = Pat[i]->Hb2->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb2->Dnr->D_Res+1;
             
             if( Asn2[B_Res] == 'C' )
             Asn2[B_Res] = 'B';
             }
             else {
             if( Pat[i]->Hb2->Dnr->D_Res == Pat[i]->Hb1->Acc->A_Res )
             
             if( Asn2[Pat[i]->Hb2->Dnr->D_Res] == 'C' )
             Asn2[Pat[i]->Hb2->Dnr->D_Res] = 'B';
             
             if( Pat[i]->Hb1->Dnr->D_Res > Pat[i]->Hb2->Acc->A_Res )
             B_Res = Pat[i]->Hb1->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb1->Dnr->D_Res+1;
             
             if( Asn1[B_Res] == 'C' )
             Asn1[B_Res] = 'B';
             }
             }
             }
             else
             if( ( !strcmp(Pat[i]->Type,"13B1") || !strcmp(Pat[i]->Type,"133A") ) &&
             ( Cn1 != Cn2 ||
             (abs(Pat[i]->Hb1->Dnr->D_Res-Pat[i]->Hb1->Acc->A_Res) > 4 &&
             abs(Pat[i]->Hb2->Dnr->D_Res-Pat[i]->Hb2->Acc->A_Res) > 4 ) ) ) { 

             if( Pat[i]->Hb1->Dnr->Chain == Cn1 ) {
             
             if( Pat[i]->Hb1->Dnr->D_Res == Pat[i]->Hb2->Acc->A_Res ) {
             
             if( Asn1[Pat[i]->Hb1->Dnr->D_Res] == 'C' )
             Asn1[Pat[i]->Hb1->Dnr->D_Res] = 'B';
             
             if( Pat[i]->Hb2->Dnr->D_Res > Pat[i]->Hb1->Acc->A_Res )
             B_Res = Pat[i]->Hb2->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb2->Dnr->D_Res+1;
             
             if( Asn2[B_Res] == 'C' )
             Asn2[B_Res] = 'B';
             }
             else {
             if( Pat[i]->Hb2->Dnr->D_Res == Pat[i]->Hb1->Acc->A_Res )
             
             if( Asn2[Pat[i]->Hb2->Dnr->D_Res] == 'C' )
             Asn2[Pat[i]->Hb2->Dnr->D_Res] = 'b';
             
             if( Pat[i]->Hb1->Dnr->D_Res > Pat[i]->Hb2->Acc->A_Res )
             B_Res = Pat[i]->Hb1->Dnr->D_Res-1;
             else
             B_Res = Pat[i]->Hb1->Dnr->D_Res+1;
             
             if( Asn1[B_Res] == 'C' )
             Asn1[B_Res] = 'b';
             }
             }
             }
    
  }
}

bool RightSide2(int L_A1, int L_D1, int LnkD, int LnkA, int I1A, int I1D, int I2A, int I2D)
{

  if( ( I2A < I1D && LnkA <= I1D && LnkA <= I2A ) || 
      ( I2A > I1D && LnkA >= I1D && LnkA >= I2A ) ||
      ( I2D < I1A && LnkD <= I1A && LnkA <= I2D ) || 
      ( I2D > I1A && LnkD >= I1A && LnkD >= I2D )  )
    return(true);
  else
    if( I2A == I1D && I2D == I1A ) {
      if( L_A1 != ERR && 
             ( ( LnkD <= I2D && L_A1 <= I2D && LnkA >= I2A && L_D1 >= I2A ) || 
             ( LnkD >= I2D && L_A1 >= I2D && LnkA <= I2A && L_D1 <= I2A ) ) )
             return(false);
      else
             return(true);
    }

  return(false);
}

bool RightSide(int LnkA, int LnkD, int I1A, int I1D, int I2A, int I2D )
{

  if( ( I1A == I2D && I1D == I2A ) ||
      ( I1A < I2D && LnkA <= I2D && LnkA <= I1A ) || 
      ( I1A > I2D && LnkA >= I2D && LnkA >= I1A ) ||
      ( I1D < I2A && LnkD <= I2A && LnkD <= I1D ) ||
      ( I1D > I2A && LnkD >= I2A && LnkD >= I1D ) )
    return(true);
  
  return(false);
}

bool RightSidePar(int LnkA, int LnkD, int I1A, int I1D, int I2A, int I2D )
{

  if( ( I1A == I2D && I1D == I2A ) ||
      ( I1A < I2D && LnkA < I2D && LnkA <= I1A && I1D <= I2A && LnkD <= I2A && LnkD <= I1D ) || 
      ( I1A > I2D && LnkA > I2D && LnkA >= I1A && I1D >= I2A && LnkD >= I2A && LnkD >= I1D ) ||
      ( I1D < I2A && LnkD < I2A && LnkD <= I1D && I1A <= I2D && LnkA <= I2D && LnkA <= I1A ) ||
      ( I1D > I2A && LnkD > I2A && LnkD >= I1D && I1A >= I2D && LnkA >= I2D && LnkA >= I1A) )
    return(true);
  
  return(false);
}

void JoinNeighbours(int *Lnk1A, int Res1, int *Lnk1D, int Res2, PATTERN **Nei, 
             PATTERN *Pat, int *MinDB1, int DB, int *MinDW1, int DW, int *Min, int j)
{
  *Lnk1A = Res1;
  *Lnk1D = Res2;
  (*Nei) = Pat;
  *MinDB1 = DB;
  *MinDW1 = DW;
  *Min = j;
}

void JoinNeighb(PATTERN **Nei, PATTERN *Pat, int *MinDB2, int DB, int *MinDW2, int DW)
{
  (*Nei) = Pat;
  *MinDB2 = DB;
  *MinDW2 = DW;
}

bool NearPar(int Res1, int Res2, int Res3, int Res4, int Res5, int Res6, int Res7, int Res8,
             char Cn1, char Cn2, char Cn3, char Cn4, int *DistBest, int *DistWorst)
{

/*
   Res5 Res2 Res1
   Res6 Res3 Res4
*/

  int a, b, c1, d1, c, d, Nei1, Nei2;

  if( Cn1 != Cn2 || Cn3 != Cn4 ) return(false);

  if( Res1 >= Res2 && Res2 >= Res5 && Res7 >= Res1 && 
      Res4 >= Res3 && Res4 >= Res6 && Res8 >= Res4 ) {

    if( Res5 == Res2 ) 
      Nei1 = Res2;
    else
      Nei1 = Res2-1;

    if( Res1 == Res7 ) 
      Nei2 = Res1;
    else
      Nei2 = Res1+1;

    a = Nei2-Nei1;
    c1 = Nei2-Res5;

    if( Res3 == Res6 ) 
      Nei1 = Res3;
    else
      Nei1 = Res3-1;

    if( Res4 == Res8 ) 
      Nei2 = Res4;
    else
      Nei2 = Res4+1;

    b = Nei2-Nei1;
    d1 = Nei2-Res6;

  }
  else
  if( Res1 <= Res2 && Res2 <= Res5 && Res7 <= Res1 && 
      Res4 <= Res3 && Res4 <= Res6 && Res8 <= Res4 ) {

    if( Res5 == Res2 ) 
      Nei1 = Res2;
    else
      Nei1 = Res2+1;

    if( Res1 == Res7 ) 
      Nei2 = Res1;
    else
      Nei2 = Res1-1;

    a = Nei1-Nei2;
    c1 = Res1-Res7;

    if( Res3 == Res6 ) 
      Nei1 = Res3;
    else
      Nei1 = Res3+1;

    if( Res4 == Res8 ) 
      Nei2 = Res4;
    else
      Nei2 = Res4-1;

    b = Nei1-Nei2;
    d1 = Nei1-Res8;


  }
  else
   return(false);

  c = MRmax(c1,a);
  d = MRmax(d1,b);

  if( a >= 0 && b >= 0 && c >= 0 && d >= 0 && 
      ( (a <= 2 && b <= 5) || (a <= 5 && b <= 2) ) ) {
    *DistBest  = MRmin(a,b);
    *DistWorst = MRmax(c,d);
    if( *DistBest <= *DistWorst ) 
      return(true);
    else
      return(false);
  }
  
  return(false);
}

bool Near(int Res1, int Res2, int Res3, int Res4, int Res5, int Res6, int Res7, int Res8,
             char Cn1, char Cn2, char Cn3, char Cn4, int *DistBest, int *DistWorst)
{

/*
   Res5 Res2 Res1
   Res6 Res3 Res4
*/

  int a, b, c1, d1, c, d, Nei1, Nei2;


  if( Cn1 != Cn2 || Cn3 != Cn4 ) return(false);


  if( Res1 >= Res2 && Res2 >= Res5 && Res7 >= Res1 && 
      Res4 <= Res3 && Res4 <= Res6 && Res8 <= Res4 ) {

    if( Res5 == Res2 ) 
      Nei1 = Res2;
    else
      Nei1 = Res2-1;
    
    if( Res1 == Res7 ) 
      Nei2 = Res1;
    else
      Nei2 = Res1+1;
    
    a = Nei2-Nei1;
    c1 = Nei2-Res5;
    
    if( Res3 == Res6 ) 
      Nei1 = Res3;
    else
      Nei1 = Res3+1;
    
    if( Res4 == Res8 ) 
      Nei2 = Res4;
    else
      Nei2 = Res4-1;
    
    b = Nei1-Nei2;
    d1 = Res6-Nei2;
  }
  else
    return(false);
  
  c = MRmax(c1,a);
  d = MRmax(d1,b);
  
  if( a >= 0 && b >= 0 && c >= 0 && d >= 0 && 
      ( (a <= 2 && b <= 5) || (a <= 5 && b <= 2) ) ) {
    *DistBest  = MRmin(a,b);
    *DistWorst = MRmax(c,d);
    if( *DistBest <= *DistWorst ) 
      return(true);
    else
      return(false);
  }
  
  return(false);
}

void MergePatternsAntiPar(Model *md, PATTERN **Pat, int NPat)
{
  register int i, j;
  int DB, DW, MinDB1, MinDB2, MinDW1, MinDW2, Min, Lnk1A, Lnk1D;
  int I1A, I1D, I2A, I2D, J1A, J1D, J2A, J2D;
  char I1ACn, I1DCn, I2ACn, I2DCn, J1ACn, J1DCn, J2ACn, J2DCn;

  for( i=0; i<NPat; i++ ) {

    if( !Pat[i]->ExistPattern ) continue;

    MinDB1 = MinDB2 = MinDW1 = MinDW2 = 1000;
    Min = ERR;
    Lnk1D = Lnk1A = ERR;

    Alias(md,&I1D,&I1A,&I2D,&I2A,&I1DCn,&I1ACn,&I2DCn,&I2ACn,Pat[i]);

    for( j=0; j<NPat; j++ ) {

      if( i == j || !Pat[j]->ExistPattern ) continue;

      Alias(md,&J1D,&J1A,&J2D,&J2A,&J1DCn,&J1ACn,&J2DCn,&J2ACn,Pat[j]);
      
      if( Near(I1D,J1D,J1A,I1A,J2A,J2D,I2A,I2D,I1DCn,J1DCn,J1ACn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2D,&Lnk1D,J2A,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( Near(I1D,J1A,J1D,I1A,J2D,J2A,I2A,I2D,I1DCn,J1ACn,J1DCn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J1D,J1A,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2A,&Lnk1D,J2D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( Near(I1D,J2D,J2A,I1A,J1A,J1D,I2A,I2D,I1DCn,J2DCn,J2ACn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1D,&Lnk1D,J1A,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( Near(I1D,J2A,J2D,I1A,J1D,J1A,I2A,I2D,I1DCn,J2ACn,J2DCn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1A,&Lnk1D,J1D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( Near(I1A,J1D,J1A,I1D,J2A,J2D,I2D,I2A,I1ACn,J1DCn,J1ACn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2A,&Lnk1D,J2D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( Near(I1A,J1A,J1D,I1D,J2D,J2A,I2D,I2A,I1ACn,J1ACn,J1DCn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2D,&Lnk1D,J2A,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( Near(I1A,J2D,J2A,I1D,J1A,J1D,I2D,I2A,I1ACn,J2DCn,J2ACn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1A,&Lnk1D,J1D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( Near(I1A,J2A,J2D,I1D,J1D,J1A,I2D,I2A,I1ACn,J2ACn,J2DCn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSide(J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1D,&Lnk1D,J2D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

    }

    for( j=0; j<NPat; j++ ) {

      if( j == Min || j == i || !Pat[j]->ExistPattern ) continue;

      Alias(md,&J1D,&J1A,&J2D,&J2A,&J1DCn,&J1ACn,&J2DCn,&J2ACn,Pat[j]);

      if( Near(I2D,J1D,J1A,I2A,J2A,J2D,I1A,I1D,I2DCn,J1DCn,J1ACn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( Near(I2D,J1A,J1D,I2A,J2D,J2A,I1A,I1D,I2DCn,J1ACn,J1DCn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( Near(I2D,J2D,J2A,I2A,J1A,J1D,I1A,I1D,I2DCn,J2DCn,J2ACn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1A,J1D,I1A,I1D,I2A,I2D) ) 
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( Near(I2D,J2A,J2D,I2A,J1D,J1A,I1A,I1D,I2DCn,J2ACn,J2DCn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1D,J1A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( Near(I2A,J1D,J1A,I2D,J2A,J2D,I1D,I1A,I2ACn,J1DCn,J1ACn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( Near(I2A,J1A,J1D,I2D,J2D,J2A,I1D,I1A,I2ACn,J1ACn,J1DCn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( Near(I2A,J2D,J2A,I2D,J1A,J1D,I1D,I1A,I2ACn,J2DCn,J2ACn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1D,J1A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( Near(I2A,J2A,J2D,I2D,J1D,J1A,I1D,I1A,I2ACn,J2ACn,J2DCn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      }
  }
}

void MergePatternsPar(Model *md, PATTERN **Pat, int NPat)
{
  register int i, j;
  int DB, DW, MinDB1, MinDB2, MinDW1, MinDW2, Min, Lnk1A, Lnk1D;
  int I1A, I1D, I2A, I2D, J1A, J1D, J2A, J2D;
  char I1ACn, I1DCn, I2ACn, I2DCn, J1ACn, J1DCn, J2ACn, J2DCn;

  for( i=0; i<NPat; i++ ) {

    if( !Pat[i]->ExistPattern ) continue;

    MinDB1 = MinDB2 = MinDW1 = MinDW2 = 1000;
    Min = ERR;
    Lnk1D = Lnk1A = ERR;

    Alias(md,&I1D,&I1A,&I2D,&I2A,&I1DCn,&I1ACn,&I2DCn,&I2ACn,Pat[i]);

    for( j=0; j<NPat; j++ ) {

      if( i == j || !Pat[j]->ExistPattern ) continue;

      Alias(md,&J1D,&J1A,&J2D,&J2A,&J1DCn,&J1ACn,&J2DCn,&J2ACn,Pat[j]);
      
      if( NearPar(I1D,J1D,J1A,I1A,J2A,J2D,I2A,I2D,I1DCn,J1DCn,J1ACn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2D,&Lnk1D,J2A,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( NearPar(I1D,J1A,J1D,I1A,J2D,J2A,I2A,I2D,I1DCn,J1ACn,J1DCn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J1D,J1A,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2A,&Lnk1D,J2D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( NearPar(I1D,J2D,J2A,I1A,J1A,J1D,I2A,I2D,I1DCn,J2DCn,J2ACn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1D,&Lnk1D,J1A,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( NearPar(I1D,J2A,J2D,I1A,J1D,J1A,I2A,I2D,I1DCn,J2ACn,J2DCn,I1ACn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1A,&Lnk1D,J1D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( NearPar(I1A,J1D,J1A,I1D,J2A,J2D,I2D,I2A,I1ACn,J1DCn,J1ACn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2A,&Lnk1D,J2D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( NearPar(I1A,J1A,J1D,I1D,J2D,J2A,I2D,I2A,I1ACn,J1ACn,J1DCn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J2D,&Lnk1D,J2A,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( NearPar(I1A,J2D,J2A,I1D,J1A,J1D,I2D,I2A,I1ACn,J2DCn,J2ACn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1A,&Lnk1D,J1D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

      if( NearPar(I1A,J2A,J2D,I1D,J1D,J1A,I2D,I2A,I1ACn,J2ACn,J2DCn,I1DCn,&DB,&DW) && 
             ((DB < MinDB1 && DW <= MinDW1) || (DB <= MinDB1 && DW < MinDW1) ) && 
             RightSidePar(J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighbours(&Lnk1A,J1D,&Lnk1D,J2D,&Pat[i]->Nei1,Pat[j],&MinDB1,DB,&MinDW1,DW,&Min,j);

    }

    for( j=0; j<NPat; j++ ) {

      if( j == Min || j == i || !Pat[j]->ExistPattern ) continue;

      Alias(md,&J1D,&J1A,&J2D,&J2A,&J1DCn,&J1ACn,&J2DCn,&J2ACn,Pat[j]);

      if( NearPar(I2D,J1D,J1A,I2A,J2A,J2D,I1A,I1D,I2DCn,J1DCn,J1ACn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( NearPar(I2D,J1A,J1D,I2A,J2D,J2A,I1A,I1D,I2DCn,J1ACn,J1DCn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( NearPar(I2D,J2D,J2A,I2A,J1A,J1D,I1A,I1D,I2DCn,J2DCn,J2ACn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1A,J1D,I1A,I1D,I2A,I2D) ) 
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( NearPar(I2D,J2A,J2D,I2A,J1D,J1A,I1A,I1D,I2DCn,J2ACn,J2DCn,I2ACn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1D,J1A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( NearPar(I2A,J1D,J1A,I2D,J2A,J2D,I1D,I1A,I2ACn,J1DCn,J1ACn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2D,J2A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( NearPar(I2A,J1A,J1D,I2D,J2D,J2A,I1D,I1A,I2ACn,J1ACn,J1DCn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J2A,J2D,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( NearPar(I2A,J2D,J2A,I2D,J1A,J1D,I1D,I1A,I2ACn,J2DCn,J2ACn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1D,J1A,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      
      if( NearPar(I2A,J2A,J2D,I2D,J1D,J1A,I1D,I1A,I2ACn,J2ACn,J2DCn,I2DCn,&DB,&DW) && 
             (( DB < MinDB2 && DW <= MinDW2) || (DB <= MinDB2 && DW < MinDW2) ) &&
             RightSide2(Lnk1A,Lnk1D,J1A,J1D,I1A,I1D,I2A,I2D) )
             JoinNeighb(&Pat[i]->Nei2,Pat[j],&MinDB2,DB,&MinDW2,DW);
      }
  }
}

void Sheet(Model *md, int Cn1, int Cn2, HBOND **HBond, float **PhiPsiMap)
{
  PATTERN **PatN, **PatP;
  int  Res1, Res3, Res2, Res4, ResA, ResB, Res1m1, Res3p1;
  int R1, R3, R2, R4, RA, RB, PatCntN = 0, PatCntP = 0, Beg;
  char *AntiPar1, *Par1, *AntiPar2, *Par2;

  PatN = g_new0(PATTERN *, MAXHYDRBOND);
  PatP = g_new0(PATTERN *, MAXHYDRBOND);
  AntiPar1  = g_new0(char, md->Chains[Cn1].nAmino);       /* Antiparallel strands */
  Par1      = g_new0(char, md->Chains[Cn1].nAmino);       /* Parallel strands */
  AntiPar2  = g_new0(char, md->Chains[Cn2].nAmino);       /* Antiparallel strands */
  Par2      = g_new0(char, md->Chains[Cn2].nAmino);       /* Parallel strands */

  for( int i=0; i<md->Chains[Cn1].nAmino; i++ ) 
  { 
       AntiPar1[i] = 'C'; 
       Par1[i] = 'C'; 
  }

  for( int i=0; i<md->Chains[Cn2].nAmino; i++ ) 
  { 
       AntiPar2[i] = 'C'; 
       Par2[i] = 'C'; 
  }

  for( R1=0; R1<md->Chains[Cn1].nAmino; R1++ ) 
  {
        Res1   = md->GetNumResiduo(Cn1, R1);
        if((!involved[Res1].NBondDnr && !involved[Res1].NBondAcc) || 
            ((Cn1 != Cn2) && !involved[Res1].InterchainHBonds) )
           continue;
        RA     = R1+1;
        R2     = R1+2;
        Res1m1 = md->GetNumResiduo(Cn1, R1-1);
        ResA   = md->GetNumResiduo(Cn1, RA);
        Res2   = md->GetNumResiduo(Cn1, R2);

        if( R2 >= md->Chains[Cn1].nAmino || RA >= md->Chains[Cn1].nAmino || 
            properties[Res1].PhiZn == ERR || properties[Res1].PsiZn == ERR ||
            properties[Res2].PhiZn == ERR || properties[Res2].PsiZn == ERR ||
            properties[ResA].PhiZn == ERR || properties[ResA].PsiZn == ERR ) 
          continue;

        if( Cn1 != Cn2 ) 
          Beg = 0;
        else
          Beg = R1+1;

        for( R3=Beg; R3<md->Chains[Cn2].nAmino; R3++ )
        {
// Process anti-parallel strands
             Res3   = md->GetNumResiduo(Cn2, R3);
             if( (!involved[Res3].NBondAcc && !involved[Res3].NBondDnr ) || 
             ((Cn1 != Cn2) && !involved[Res3].InterchainHBonds) ) 
             continue;
             RB     = R3-1;
             R4     = R3-2;
             Res3p1 = md->GetNumResiduo(Cn2, R3+1);
             ResB   = md->GetNumResiduo(Cn2, RB);
             Res4   = md->GetNumResiduo(Cn2, R4);
      
             if( Cn1 != Cn2 || R3 - R1 >= 3  )
             Link(HBond,md,Cn1,Cn2,Res1,Res3,Res3,Res1,Res1,Res3,
             PhiPsiMap,PatN,&PatCntN,"1331",Treshold_E1,0);
      
             if( R2 < md->Chains[Cn1].nAmino && ((Cn1 != Cn2 && R4 >= 0) || R4-R2 >=2 ) )
             Link(HBond,md,Cn2,Cn1,Res3,Res1,Res2,Res4,ResB,ResA,
             PhiPsiMap,PatN,&PatCntN,"3124",Treshold_E1,0);
      
             if( ((Cn1 != Cn2 && RB >= 0 ) || RB-R1 > 4) && 
             ( RA >= md->Chains[Cn1].nAmino || (Cn1 == Cn2 && R3-RA <= 4 ) ||
             !Link(HBond,md,Cn1,Cn2,Res1,Res3,Res3,ResA,-1,Res3,
             PhiPsiMap,PatN,&PatCntN,"133A",Treshold_E1,1))
             &&
             ( R1-1 < 0 ||
             !Link(HBond,md,Cn1,Cn2,Res1m1,ResB,ResB,Res1,-1,ResB,
             PhiPsiMap,PatN,&PatCntN,"1-BB1",Treshold_E1,1)))
             Link(HBond,md,Cn1,Cn2,Res1,Res3,ResB,Res1,Res1,-1,
             PhiPsiMap,PatN,&PatCntN,"13B1",Treshold_E1,0);
      
             if( (RA < md->Chains[Cn1].nAmino && (Cn1 != Cn2 || R3-RA > 4)) &&
             ( (Cn1 == Cn2 && RB-R1 <= 4 ) || (Cn1 != Cn2 && RB < 0 ) || 
             !Link(HBond,md,Cn1,Cn2,Res1,Res3,ResB,Res1,Res1,-1,
             PhiPsiMap,PatN,&PatCntN,"13B1",Treshold_E1,1))
             &&
             ( R3+1 >= md->Chains[Cn2].nAmino ||
             !Link(HBond,md,Cn1,Cn2,ResA,Res3p1,Res3,ResA,ResA,-1,
             PhiPsiMap,PatN,&PatCntN,"A3+3A",Treshold_E1,1)))
             Link(HBond,md,Cn1,Cn2,Res1,Res3,Res3,ResA,-1,Res3,
             PhiPsiMap,PatN,&PatCntN,"133A",Treshold_E1,0);
      
// Process parallel strands */

                R4 = R3+2; 
                RB = R3+1;
                ResB   = md->GetNumResiduo(Cn2, RB);
                Res4   = md->GetNumResiduo(Cn2, R4);

                if( (Cn1 == Cn2 && abs(R3-R1) <= 3) || R4 >= md->Chains[Cn2].nAmino )
                     continue;
      
      if( R2 < md->Chains[Cn1].nAmino && (Cn1 != Cn2 || abs(R2-R3) > 3) )
             Link(HBond,md,Cn2,Cn1,Res3,Res1,Res2,Res3,Res3,ResA,
             PhiPsiMap,PatP,&PatCntP,"3123",Treshold_E2,0);
      
      if( R4 < md->Chains[Cn2].nAmino && (Cn1 != Cn2 || abs(R4-R1) > 3) )
             Link(HBond,md,Cn1,Cn2,Res1,Res3,Res4,Res1,Res1,ResB,
             PhiPsiMap,PatP,&PatCntP,"1341",Treshold_E2,0);
    }
  }
    
  FilterAntiPar(md,PatN,PatCntN);
  FilterPar(md,PatP,PatCntP);

  MergePatternsAntiPar(md,PatN,PatCntN);
  MergePatternsPar(md,PatP,PatCntP);

  if( ifDebug )  {
    g_print("PatN \n");
    PrintPatterns(PatN,PatCntN,md,Cn1,Cn2);
    g_print("PatP \n");
    PrintPatterns(PatP,PatCntP,md,Cn1,Cn2);
  }

  FillAsnAntiPar(AntiPar1,AntiPar2,md,Cn1,Cn2,PatN,PatCntN);
  FillAsnPar(Par1,Par2,md,Cn1,Cn2,PatP,PatCntP);

  Bridge(AntiPar1,AntiPar2,md,Cn1,Cn2,PatN,PatCntN);
  Bridge(Par1,Par2,md,Cn1,Cn2,PatP,PatCntP);

  for( int i=0; i<md->Chains[Cn1].nAmino; i++ )
  {
    int ir = md->GetNumResiduo(Cn1, i);
    if( AntiPar1[i] == 'N' || Par1[i] == 'P' ) 
      properties[ir].Asn = 'E';
    else if( AntiPar1[i] == 'B' || Par1[i] == 'B' )
      properties[ir].Asn = 'B';
    else if( AntiPar1[i] == 'b' || Par1[i] == 'b' )
      properties[ir].Asn = 'b';
  }
  for( int i=0; i<md->Chains[Cn2].nAmino; i++ )
  {
    int ir = md->GetNumResiduo(Cn2, i);
    if( properties[ir].Asn == 'E' )
      continue;
    else if( AntiPar2[i] == 'N' || Par2[i] == 'P' ) 
      properties[ir].Asn = 'E';
    else if( AntiPar2[i] == 'B' || Par2[i] == 'B' ) 
      properties[ir].Asn = 'B';
    else if( AntiPar2[i] == 'b' || Par2[i] == 'b' ) 
      properties[ir].Asn = 'b';
   }
  g_free(PatN);
  g_free(PatP);
  g_free(AntiPar1);
  g_free(Par1);
  g_free(AntiPar2);
  g_free(Par2);
}

bool TurnCondition(float Phi2,float Phi2S,float Psi2,float Psi2S,
                  float Phi3,float Phi3S,float Psi3,float Psi3S,
                  float Range1,float Range2)
{
  if((DENTRO(Phi2,Phi2S,Range2) && DENTRO(Psi2,Psi2S,Range1) &&
      DENTRO(Phi3,Phi3S,Range1) && DENTRO(Psi3,Psi3S,Range1))
     ||
     (DENTRO(Phi2,Phi2S,Range1) && DENTRO(Psi2,Psi2S,Range2) &&
      DENTRO(Phi3,Phi3S,Range1) && DENTRO(Psi3,Psi3S,Range1))
     ||
     (DENTRO(Phi2,Phi2S,Range1) && DENTRO(Psi2,Psi2S,Range1) &&
      DENTRO(Phi3,Phi3S,Range2) && DENTRO(Psi3,Psi3S,Range1))
     ||
     (DENTRO(Phi2,Phi2S,Range1) && DENTRO(Psi2,Psi2S,Range1) &&
      DENTRO(Phi3,Phi3S,Range1) && DENTRO(Psi3,Psi3S,Range2))
     )
    return(true);

  return(false);
}

void BetaTurn(Model *md, int Cn)
{

  RESIDUE r, r4;
  int CA1, CA4;
  float dist14;

  for(int i=0; i<md->Chains[Cn].nAmino-4; i++ ) 
  {
    r  = md->Chains[Cn].AminoRes[i];
    r4 = md->Chains[Cn].AminoRes[i+3];
    int Res  = md->GetNumResiduo(Cn, i);
    int Res1 = md->GetNumResiduo(Cn, i+1);
    int Res2 = md->GetNumResiduo(Cn, i+2);
    int Res3 = md->GetNumResiduo(Cn, i+3);

    char asn  = properties[Res ].Asn;
    char asn1 = properties[Res1].Asn;
    char asn2 = properties[Res2].Asn;
    char asn3 = properties[Res3].Asn;
    
    CA1 = md->FindAtom("CA",r, 6);
    CA4 = md->FindAtom("CA",r4, 6);
    if( (CA1 != -1) && (CA4 != -1) )
        dist14 = Distanza(md->atm[CA1].xyz,md->atm[CA4].xyz);
    else
        dist14 = 10;
    if( asn1 == 'H' || asn2 == 'H' || 
        asn1 == 'G' || asn2 == 'G' || 
        asn1 == 'I' || asn2 == 'G' || 
        (CA1 == -1)  || (CA4 == -1)  ||
        dist14 > 7.0 )
      continue;
    
// Informazioni dettagliate
/*
    float Phi2, Phi3, Psi2, Psi3, Range1 = 30.0, Range2 = 45.0;
    char TurnType;
    Phi2 = properties[Res1].Phi;
    Psi2 = properties[Res1].Psi;
    Phi3 = properties[Res2].Phi;
    Psi3 = properties[Res2].Psi;
    
    if( TurnCondition(Phi2,-60.0,Psi2,-30,Phi3,-90.0,Psi3,0,Range1,Range2) )
      TurnType = '1';
    else
    if( TurnCondition(Phi2,60.0,Psi2,30,Phi3,90.0,Psi3,0,Range1,Range2) )
      TurnType = '2';
    else
    if( TurnCondition(Phi2,-60.0,Psi2,120,Phi3,80.0,Psi3,0,Range1,Range2) )
      TurnType = '3';
    else
    if( TurnCondition(Phi2,60.0,Psi2,-120,Phi3,-80.0,Psi3,0,Range1,Range2) )
      TurnType = '4';
    else
    if( TurnCondition(Phi2,-60.0,Psi2,120,Phi3,-90.0,Psi3,0,Range1,Range2) )
      TurnType = '5';
    else
    if( TurnCondition(Phi2,-120.0,Psi2,120,Phi3,-60.0,Psi3,0,Range1,Range2) )
      TurnType = '6';
    else
    if( TurnCondition(Phi2,-60.0,Psi2,-30,Phi3,-120.0,Psi3,120,Range1,Range2) )
      TurnType = '7';
    else
      TurnType = '8';
*/
// Fine informazioni dettagliate
  
    if( asn == 'C' ) 
      properties[Res ].Asn= 'T';
    
    if( asn1 == 'C' )
      properties[Res1].Asn = 'T';
    
    if( asn2 == 'C' )
      properties[Res2].Asn = 'T';
    
    if( asn3 == 'C' )
      properties[Res3].Asn = 'T';
  }
}

void GammaTurn(Model *md, int Cn, HBOND **HBond)
{
  ResidueProperty rp, rp1, rp2, rp3, rp4;
  int Res, Res1, Res2, Res3, Res4;
  float Phi2, Psi2;
  char Asn;

  for( int i=0; i<md->Chains[Cn].nAmino-2; i++ ) 
  {
    if(i > 0)
    {
       Res = md->GetNumResiduo(Cn, i-1);
       rp = properties[Res];
    }
    Res1 = md->GetNumResiduo(Cn, i);
    rp1 = properties[Res1];
    Res2 = md->GetNumResiduo(Cn, i+1);
    rp2 = properties[Res2];
    Res3 = md->GetNumResiduo(Cn, i+2);
    rp3 = properties[Res3];
    if(i != md->Chains[Cn].nAmino-3)
    {
       Res4 = md->GetNumResiduo(Cn, i+3);
       rp4 = properties[Res4];
    }
    Asn = rp2.Asn;

    if( Asn == 'H' || Asn == 'T' || Asn == 'G' || Asn == 'I' ||
        FindBnd(HBond,i+2,i) == ERR ||
        (i > 0 && FindBnd(HBond,Res3,Res) != ERR) || 
        (i < md->Chains[Cn].nAmino-3 && FindBnd(HBond,Res4,Res1) != ERR) )
      continue;
    
    Phi2 = rp2.Phi;
    Psi2 = rp2.Psi;
    
/*
    char TurnType;
    if( Phi2 > 0.0 && Psi2 < 0.0 )
      TurnType = '@';
    else
    if( Phi2 < 0.0 && Psi2 > 0.0 )
      TurnType = '&';
    else 
      continue;
*/
    if( (Phi2*Psi2) >= 0)
      continue;

    if( rp1.Asn == 'C' )
      rp1.Asn = 'T';
    
    if( rp2.Asn == 'C' )
      rp2.Asn = 'T';
    
    if( rp3.Asn == 'C' )
      rp3.Asn = 'T';
  }
}

bool DetectSecondary(Model *md, std::string &msg)
{
      properties = g_new0(ResidueProperty, md->nResTot);
      involved = g_new0(Involved, md->nResTot);

//Backbone Angles
      for(int ch=0; ch<md->nChains; ch++)
      {
          for(int j=0; j< md->Chains[ch].nAmino; j++)
          {
              int curr = md->GetNumResiduo(ch, j);
              properties[curr].Asn = 'C';
              properties[curr].Phi = 360.;
              RESIDUE cur = md->Chains[ch].AminoRes[j];
              if(j != 0)
              {
                 RESIDUE prec = md->Chains[ch].AminoRes[j-1];
                 if( (prec.c != -1) && (cur.n != -1) &&
                     (cur.ca != -1) && (cur.c != -1) &&
                     (Distanza(md->atm[prec.c].xyz, md->atm[cur.n].xyz) < BREAKDIST) )
                        properties[curr].Phi = Torsione(md->atm[prec.c].xyz,
                                                     md->atm[cur.n].xyz,
                                                     md->atm[cur.ca].xyz,
                                                     md->atm[cur.c].xyz);
              }
              properties[curr].Psi = 360.;
              if(j != (md->Chains[ch].nAmino-1))
              {
                 RESIDUE next = md->Chains[ch].AminoRes[j+1];
                 if( (next.n != -1) && (cur.n != -1) &&
                     (cur.ca != -1) && (cur.c != -1) &&
                     (Distanza(md->atm[cur.c].xyz, md->atm[next.n].xyz) < BREAKDIST) )
                        properties[curr].Psi = Torsione(md->atm[cur.n].xyz,
                                                        md->atm[cur.ca].xyz,
                                                        md->atm[cur.c].xyz,
                                                        md->atm[next.n].xyz);
              }
          }
      }

      float **PhiPsiMapHelix, **PhiPsiMapSheet;
      PhiPsiMapHelix = DefaultHelixMap();
      PhiPsiMapSheet = DefaultSheetMap();
      int h_cnt = PlaceHydrogens(md, md->c_alpha);
      if(h_cnt == 0)
      {
         msg = "No hydrogen placed \n";
         return false;
      }
      int NHBond;
      HBOND **HBond = g_new0(HBOND *, 50000);
      if( (NHBond = FindHydrogenBonds(md, HBond, msg)) == 0 )
      {
          msg = "No hydrogen bonds found \n";
          return false;
      }
      NoDoubleHBond(md, HBond,NHBond);
   
      DiscrPhiPsi(md);
      for(int Cn=0; Cn<md->nChains; Cn++)
      {
        if(md->Chains[Cn].nAmino > 5)
        {
          Helix(md, Cn, HBond, PhiPsiMapHelix);
          for(int i=0; i<md->nChains; i++)
          {
              if(md->Chains[i].nAmino > 5)
                  Sheet(md, Cn, i, HBond, PhiPsiMapSheet);
          }
          BetaTurn(md,Cn);
          GammaTurn(md,Cn,HBond);
        }
      }
      for(int ch=0; ch<md->nChains; ch++)
      {
          for(int j=0; j< md->Chains[ch].nAmino; j++)
          {
              RESIDUE *cur = &(md->Chains[ch].AminoRes[j]);
              if(Is_Amino(cur->code))
              {
                 cur->secondary = properties[cur->num].Asn;
                 for(int na=0; na<(int) cur->atoms.size(); na++)
                 {
                     int kat = cur->atoms[na];
                     md->Secondary[kat] = cur->secondary;
                 }
              }
          }
      }
  
      if(properties)
         g_free(properties);
      if(involved)
         g_free(involved);
      for( int i=0; i<NHBond; i++ ) 
         g_free(HBond[i]);
      g_free(HBond);
      g_free(PhiPsiMapHelix);
      g_free(PhiPsiMapSheet);
      return true;
}

#ifdef MAPS

static float ***mappa;
static int ***color;

float side(float val, float lim1, float lim2)
{
   return((val-lim1)/(lim2-lim1));
}

int sidepp(int val, int nbin)
{
   int result = val;
   while(result < 0)
         result += nbin;
   while(result >= nbin)
         result -= nbin;
   return result;
}

void Model::CreaMappa(int whichmod, float pRadius, float step)
{
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");

   vet3 delt, xsmin={{1e6, 1e6, 1e6}}, xsmax={{-1e6, -1e6, -1e6}};
   double due = 2.0;
   double ln2 = log(due);

   gl->RiNormalizza();
   for(int i=0; i<nats_au; i++)
   {
       int ieq = Equivs[i].at(0);
       xsmin = MRVectorMinimum(xsmin,
                MRVectorAddScalar(MRVectorMake(atm[ieq].xyz), -radMax));
       xsmax = MRVectorMaximum(xsmax,
                MRVectorAddScalar(MRVectorMake(atm[ieq].xyz),  radMax));
   }
   delt = MRVectorSub(xsmax, xsmin);

   vet3 xcmin = gl->CartToFract(xsmin);
   vet3 xcmax = gl->CartToFract(xsmax);
   int np[] = {(int) (delt.x/step + 0.5f)+1,
               (int) (delt.y/step + 0.5f)+1,
               (int) (delt.z/step + 0.5f)+1};
   float deltx  = (float)np[0] / (float)delt.x;
   float delty  = (float)np[1] / (float)delt.y;
   float deltz  = (float)np[2] / (float)delt.z;
   float caChem = (float) cos(DEGTORAD(gl->Cell[3]));
   float cbChem = (float) cos(DEGTORAD(gl->Cell[4]));
   float cgChem = (float) cos(DEGTORAD(gl->Cell[5]));

   mappa = g_new0(float **, np[0]);
   color = g_new0(int **, np[0]);
   for(int i=0; i<np[0]; i++)
   {
       mappa[i] = g_new0(float *, np[1]);
       color[i] = g_new0(int *, np[1]);
       for(int j=0; j<np[1]; j++)
       {
           mappa[i][j] = g_new0(float , np[2]);
           color[i][j] = g_new0(int , np[2]);
       }
   }

   float xs1[3];
   float alim = (float) radMax;
   for(int i=0; i<nats_au; i++)
   {
       int ieq = Equivs[i].at(0);
       for(int j=0; j<3; j++)
           xs1[j] = side(atm[ieq].xyzc[j], (float) xcmin.v[j], (float) xcmax.v[j]);
       int index = atm[ieq].color;
       float raggioVDW = radii[index][1];
       float DmaxSq = alim*alim;

       int iszu = (int)(xs1[2]*np[2]+alim*deltz);
       int iszl = (int)(xs1[2]*np[2]-alim*deltz);
       int isyu = (int)(xs1[1]*np[1]+alim*delty);
       int isyl = (int)(xs1[1]*np[1]-alim*delty);
       int isxu = (int)(xs1[0]*np[0]+alim*deltx);
       int isxl = (int)(xs1[0]*np[0]-alim*deltx);

       for(int  iz=iszl; iz<=iszu; iz++)
       {
           for(int iy=isyl; iy<=isyu; iy++)
           {
               for(int  ix=isxl; ix<=isxu; ix++)
               {
                   float a1 = (float)ix/deltx-xs1[0]*(float)delt.x;
                   float a2 = (float)iy/delty-xs1[1]*(float)delt.y;
                   float a3 = (float)iz/deltz-xs1[2]*(float)delt.z;
/*
                   float a1 = (float)ix/deltx-xs1[0]*delt[0];
                   float a2 = (float)iy/delty-xs1[1]*delt[1];
                   float a3 = (float)iz/deltz-xs1[2]*delt[2];
*/
                   float ray = a1*a1 + a2*a2 + a3*a3 + 2.f*a1*a2*cgChem +
                       2.f*a1*a3*cbChem + 2.f*a2*a3*caChem;
                   if(ray < DmaxSq) 
                   {
                      int ix1 = sidepp(ix,np[0]);
                      int ix2 = sidepp(iy,np[1]);
                      int ix3 = sidepp(iz,np[2]);
                      double gauss = due*exp(-ln2*ray/(raggioVDW*raggioVDW));
//così è più smussata
//                      mappa[ix1][ix2][ix3] += (float)gauss;
                      if(mappa[ix1][ix2][ix3] < gauss)
                         color[ix1][ix2][ix3] = atm[ieq].Z-1;
                      mappa[ix1][ix2][ix3] = MRmax(mappa[ix1][ix2][ix3],
                                               (float) gauss);
                   }
               }
           }
       }
   }
   for(int i=0; i<np[0]; i++)
   {
       for(int j=0; j<np[1]; j++)
           for(int k=0; k<np[2];k++)
           mappa[i][j][k] *= pRadius;
   }
   int nmaps = gl->nMaps + 1;
   int nrefl = 0;
   float resm = 0;
   int ifd = 1;
   char *name = g_strdup("prova");
   PassaMappa(name, &nmaps, &whichmod, np, &nrefl, &resm, &ifd,
                NULL, NULL, NULL, NULL, mappa, color);
   setMinMax(xcmin, xcmax, pRadius, step, whichmod-1);
   for(int i=0; i<np[0]; i++)
   {
       for(int j=0; j<np[1]; j++)
       {
           g_free(mappa[i][j]);
           g_free(color[i][j]);
       }
       g_free(mappa[i]);
       g_free(color[i]);
   }
   g_free(mappa);
   g_free(color);
   Surface_c = true;
}
#endif

std::string Model::GetType(int na, ...)
{
    va_list vl;
    std::string labtype("");
    va_start(vl,na);
    int if_au = va_arg(vl,int);
    for (int i=0;i<na;i++)
    {
         int n1 = va_arg(vl,int);
         if(i)
            labtype.append("-");
         int ks;
         if(if_au)
            ks = atm_au[n1].color;
         else
            ks = atm[n1].color;
         if(ks >= 0)
         {
            std::string labtemp(LabSpec[ks], LEN_SPEC);
            labtype.append(trim(labtemp));
         }
         else
            labtype.append("Q");
    }
    va_end(vl);
    return labtype;
}

void Model::CopyAtoms(jav_atom_type *atml, int natoms)
{
     atm = g_new0(jav_atom_type, natoms);
     for(int i=0; i<natoms; i++)
         memcpy(&(atm[i]), &(atml[i]), sizeof(jav_atom_type));
}

void Model::FillAtoms(atom_type *atml)
{
//  structure_info_type StructInfo;

  atm = g_new0(jav_atom_type, nats);
  iEqui = g_new0(int, nats);
//  int ifCalc = 0;
//  int wm = GetActiveModel();
//  getstructureinfo_(&StructInfo, &ifCalc, &wm);
  for(int i=1; i<=nats_au; i++)
  {
      for(int l=0; l<MAX_LABLEN; l++)
          atm[i-1].lab[l] = '\0';
      CopiaStringan(atm[i-1].lab, MAX_LABLEN, atml[i-1].lab, 16);
      atm[i-1].deleted = false;
      int sp = MRabs(atml[i-1].nz)/nzc - 1;
      if(sp >= 0)
      {
         atm[i-1].color = sp % MAX_SPECIES;
         //atm[i-1].Z =  OTHERS;
      }
      else
      {
         atm[i-1].color = -1;
         //atm[i-1].Z =  OTHERS;
      }
      atm[i-1].ptab =  atml[i-1].ptab;
      atm[i-1].Z =  ZFromPtab(atm[i-1].ptab-1);
      if(atm[i-1].Z <= 0)
            atm[i-1].Z =  OTHERS;

      atm[i-1].op.op = 1;
      //atm[i-1].opcode = 1;
      iEqui[i-1] = i - 1;

      atm[i-1].numResiduo = 0;
      atm[i-1].catena = 0;
      atm[i-1].catseq = 0;
      for(int j=1; j<=3; j++)
      {
          atm[i-1].xyzc[j-1] = atml[i-1].xc[j-1];
          baricentro[j-1] += atm[i-1].xyzc[j-1];
      }
      atm[i-1].B = atml[i-1].biso;
      atm[i-1].ocry = atml[i-1].ocry;
      atm[i-1].och = atml[i-1].och;
      atm[i-1].H = atml[i-1].inte;
      if(atm[i-1].B < Bmin)
         Bmin = atm[i-1].B;
      else if(atm[i-1].B > Bmax)
         Bmax = atm[i-1].B;
      for(int ll=0; ll<6; ll++)
          atm[i-1].Bij[ll] = atml[i-1].bij[ll];
  }
  GetEquivs();
}

void Model::FillAtoms(atom_type *atml,  structure_info_type StructInfo, bool au,
                      int *nzo, int *het)
//                      symm_info_type *info, int *nzo, int *het)
{
  GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
  int natoms;
  jav_atom_type *atoms;
  if(au)
  {
     natoms = nats_au;
     atoms = atm_au;
  }
  else
  {
     natoms = nats;
     atoms = atm;
  }
  for(int i=1; i<=natoms; i++)
  {
      for(int l=0; l<MAX_LABLEN; l++)
          atoms[i-1].lab[l] = '\0';
      CopiaStringan(atoms[i-1].lab, MAX_LABLEN, atml[i-1].lab, 16);
      atoms[i-1].deleted = false;
      atoms[i-1].hidden = false;
      atoms[i-1].ptab =  atml[i-1].ptab;
      atoms[i-1].Z =  ZFromPtab(atoms[i-1].ptab-1);
      if(atoms[i-1].Z <= 0)
            atoms[i-1].Z =  OTHERS;
      int sp = MRabs(atml[i-1].nz)/nzc - 1;
      if(sp >= 0)
      {
         atoms[i-1].color = sp % MAX_SPECIES;
         //atoms[i-1].Z =  StructInfo.zspec[sp%MAX_SPECIES];
         //if(atoms[i-1].Z <= 0)
            //atoms[i-1].Z =  OTHERS;
      }
      else
      {
         atoms[i-1].color = -1;
         //atoms[i-1].Z =  OTHERS;
      }

      if(!au)
      //if((info != NULL)  && !au)
      {
         atoms[i-1].op.op = atml[i-1].op.op;
         memcpy(atoms[i-1].op.tra, atml[i-1].op.tra, 3*sizeof(int));
         //atoms[i-1].oper = info[i-1].oper;
         //atoms[i-1].opcode = info[i-1].opcode;
         if(atoms[i-1].op.op <= 0)
            atoms[i-1].op.op = 1;
         if((atoms[i-1].op.op <= 0) || (atoms[i-1].op.op > StructInfo.nsymmstr))
         {
             g_print("C\'e\' qualcosa di strano in oper \n");
             g_print("Atom %d oper=%d\n",i, atoms[i-1].op.op);
         }
         //iEqui[i-1] = info[i-1].asym - 1;
         iEqui[i-1] = (atml[i-1].asym)?(atml[i-1].asym - 1):i-1;
         if((iEqui[i-1] < 0) || (iEqui[i-1] >= nats_au))
         {
               g_print("C\'e\' qualcosa di strano in asym\n");
               g_print("Atom %d asym=%d\n",i, iEqui[i-1]);
         }
      }
      else
      {
         atoms[i-1].op.op = 1;
         //atoms[i-1].oper = 1;
         //atoms[i-1].opcode = 1;
      }

#if 0
      if(nzo != NULL)
      {
         atoms[i-1].deleted = false;
         atoms[i-1].hidden = false;
         if(nzo[iEqui[i-1]] == 1)
            atoms[i-1].hidden = true;
         else if(nzo[iEqui[i-1]] == 2)
            atoms[i-1].deleted = true;
      }
#endif

      atoms[i-1].numResiduo = 0;
      atoms[i-1].catena = 0;
      atoms[i-1].catseq = 0;
      float xyz[3];
      for(int j=1; j<=3; j++)
      {
          atoms[i-1].xyzc[j-1] = atml[i-1].xc[j-1];
          if(!au)
             baricentro[j-1] += atoms[i-1].xyzc[j-1];
          xyz[j-1] = atml[i-1].xc[j-1];
      }
      vet3 xyzo = gl->FractToCart(xyz);    //fraz. --> orto
      for(int j=1; j<=3; j++)
          atoms[i-1].xyz[j-1] = (float)xyzo.v[j-1];
      atoms[i-1].B = atml[i-1].biso;
      atoms[i-1].ocry = atml[i-1].ocry;
      atoms[i-1].och = atml[i-1].och;
      atoms[i-1].H = atml[i-1].inte;
      if(!au)
      {
      if(atoms[i-1].B < Bmin)
         Bmin = atoms[i-1].B;
      else if(atoms[i-1].B > Bmax)
         Bmax = atoms[i-1].B;
      }
      for(int ll=0; ll<6; ll++)
          atoms[i-1].Bij[ll] = atml[i-1].bij[ll];

      gl->CheckDistAng();
      atoms[i-1].heta = 0;
      if(isPDB && au)
      {
         char c = ' ';
#ifdef PROTEINS
         int jj = i;
         getsecondary_(&jj, &c, 1);
#endif
         Secondary[i-1] = c;
         if(het != NULL)
         {
            atoms[i-1].heta = het[i-1];
            if(het[i-1])
               Secondary[i-1] = 'L';
         }
      }
  }
}

void Model::FillLegs(int *nl, int *nl_au, f_bond_type *legm, bool if_edit)
{
  nconn_prot = *nl;
  nconn_bb = 0;
  nconn_lig = 0;
  nconn_ca = 0;
  nlegs_au = *nl_au;
  nconn = nconn_prot;
  bonds = g_new0(jav_bond_type,  nconn);
  for(int i=1; i<=nconn; i++)
  {
      int n1 = legm[i-1].n1;
      int n2 = legm[i-1].n2;
      bonds[i-1].n1 = n1;
      bonds[i-1].n2 = n2;
      bonds[i-1].dist = legm[i-1].dist;
      bonds[i-1].sigma = legm[i-1].sigma;
      if(legm[i-1].ord <= 0)
         bonds[i-1].ordleg = 1;
      else
         bonds[i-1].ordleg = legm[i-1].ord;
      if(!if_edit)
      {
         int numleg = atm[n1-1].numleg;
         atm[n1-1].leg[numleg] = n2-1;
         atm[n1-1].dist[numleg] = bonds[i-1].dist;
         numleg = atm[n2-1].numleg;
         atm[n2-1].leg[numleg] = n1-1;
         atm[n2-1].dist[numleg] = bonds[i-1].dist;
         atm[n1-1].numleg++;
         atm[n2-1].numleg++;
      }
  }
}

size_t Model::truesize()
{
    return (sizeof(*this) - sizeof(Chains) - sizeof(c_alpha) - sizeof(c_alphab)
            - sizeof(atoms_sel) - sizeof(bonds_sel) -sizeof(resid_sel)
            - sizeof(phlist) -sizeof(atom_types) -sizeof(AtomTypes)
            -sizeof(Conns) -sizeof(Equivs));
}

void GLWindow::IsAtom(int x, int y)
{
     for(int i=0;  i<nModels; i++)
     {
         if(ModelInfo[i].ifVis) 
         {
            int ind = 0;
            //if(ModelInfo[i].plotStyle == VANDERWAALS)
            //   ind = 2;
            //else if(ModelInfo[i].plotStyle <= RODS)
            //   ind = 1;
            int jat = ModelInfo[i].isAtom(x,y,ind);
            if(jat < 0)
               continue;
            char *str;
            if(ModelInfo[i].plotStyle >= POLYPEPTIDE)
            {
               std::string Label = ModelInfo[i].atm[jat].lab;
               std::string ResLab;
               ResLab.assign(get_chain(Label));
               ResLab.append(get_res_name(Label));
               ResLab.append(get_res_num(Label));
                str = g_strdup_printf("%s",
                 ResLab.c_str());

            }
            else
            {
               std::string str1 =
                     g_strdup_printf("%s #%d ",
                     g_strstrip(g_strdup(ModelInfo[i].atm[jat].lab)), jat+1);
               if(ModelInfo[i].atm[jat].numleg > 0)
                     str1.append("Dist: ");
               for(int kat=0; kat<ModelInfo[i].atm[jat].numleg; kat++)
               {
                   int j2 = ModelInfo[i].atm[jat].leg[kat];
                   jav_atom_type *atomo2 = &(ModelInfo[i].atm[j2]);
                   if(kat)
                      str1.append(", ");
                   str1.append(g_strdup_printf("%s: %.3f",
                     g_strstrip(g_strdup(atomo2->lab)),
                     ModelInfo[i].atm[jat].dist[kat]));
               }
               str = g_strdup(str1.c_str());
            }
            PrintMess(str);
            break;
         }
     }
}

int Model::isAtom(int x, int y, int ind)
{
  double *model = g_new0(double, 16);
  double *proj = g_new0(double, 16);
  GLint view[4];
  GLdouble winx, winy, winz;
  GLdouble winx1, winy1, winz1;
  GLdouble xx = (GLdouble)x;
  GLdouble yy = (GLdouble)y;
  glGetDoublev(GL_MODELVIEW_MATRIX, model);
  glGetDoublev(GL_PROJECTION_MATRIX, proj);
  glGetIntegerv(GL_VIEWPORT, view);

  for(int i=0; i<nats; i++)
  {
     FromCoordToWin(atm[i].xyzN[0],atm[i].xyzN[1], atm[i].xyzN[2],
                    model, proj, view, &winx, &winy, &winz);
     FromCoordToWin(atm[i].xyzN[0]+atm[i].radN[ind],atm[i].xyzN[1]+atm[i].radN[ind], atm[i].xyzN[2]+atm[i].radN[ind],
                    model, proj, view, &winx1, &winy1, &winz1);
     GLdouble rad = sqrt((winx-winx1)*(winx-winx1)+
                     (winy-winy1)*(winy-winy1));
     if((MRabs(winx - xx) < rad) && (MRabs(winy - yy) < rad))
        return i;
  }
  return -1;
}

}
