 MODULE RIETV

 USE progtype
!corr USE atom_basic, only: atom_type

 implicit none

!corr type(atom_type), dimension(:), allocatable      :: atm_sav   ! salva atomi durante affinamento
 logical, dimension(:), allocatable :: refop

 integer, parameter, private :: RIET_REFINE=0, LEBAIL_DECOMP=1, PAWLEY_DECOMP=2

 CONTAINS

   subroutine inizializza_rietveld
!
!  Variabili da inizializzare ogni volta che si parte con una nuova struttura
!
   USE RefinecomRef
   !USE RRESTR
   !USE constraints
   USE calculate_pdp
   USE variables, only: cryst,dataset
   USE crystal_phase
   integer          :: nph
!
   call init_refine_cond(rfcond)
!
   call init_data_profile(dataset(1),numphase(cryst))
   cryst%scal = 1.0
   dataset(1)%sdisp = 0.0
   dataset(1)%stran = 0.0
!
   kabsor=0
   do nph=1,numphase(cryst)
      cryst(nph)%kopcorr = 0
      cryst(nph)%hklpo = [0,0,1]
      cryst(nph)%gpar = 1.0
   enddo
!
   end subroutine inizializza_rietveld

!--------------------------------------------------------------------

   subroutine init_refine_cond(cond)
!
!  Default per le condizioni di affinamento
!
   USE RefinecomRef
   USE progtype, only: refine_condition_type
   USE genlsq, only: GAUNEW,LMDER,NL2SOL
   USE General, only:ifAutomatic
   type(refine_condition_type), intent(out) :: cond
!
   cond%algo = GAUNEW              ! tipo di algoritmo
   !cond%algo = LMDER              ! tipo di algoritmo
   cond%auto_prof = .false.        ! affinamento automatico profilo
   cond%auto_stru = .false.        ! affinamento automatico struttura
   cond%ncauto = 3                 ! num. di cicli di affinamento automatico
   if (ifAutomatic == 1) then
       cond%winmess = .false.      ! messagistica attiva?
   else
       cond%winmess = .true.
   endif
   cond%typewei = 2                ! tipo di peso
   cond%eps = 0.0001               ! parametro di convergenza
   cond%maxcy = 30                 ! numero massimo di cicli
   cond%raction = 0                 ! 0=Rietveld 1=Le Bail 2=Pawley
   !cond%sprint = 'SVW'             ! Opzioni di stampa
   cond%sprint = 'SW'              ! Opzioni di stampa
!   
   end subroutine init_refine_cond

!--------------------------------------------------------------------

   subroutine set_rcode_function(pears,psvoig,tchh,sref,code)
!  
!  Definisce codici di affinamento per parametri di profilo.
!  sref deve contenere l'elenco dei paramatri da affinare separati con una virgola. 
!  Se sref e' ALL tutti i parametri della funzione vengono affinati
!  code e' il valore da trasferire in funzione%parametro%rcod
!  
   !corr USE progtype
   USE pearsonf
   USE psvoigtf
   USE tchzf
   USE profile_function
   type(profile_function_t), intent(inout), optional :: pears
   type(profile_function_t), intent(inout), optional  :: psvoig
   type(profile_function_t), intent(inout), optional     :: tchh
   character(len=*), intent(in)                       :: sref
   integer, intent(in)                                :: code
   character(len=len_trim(sref)+2)                    :: str
   integer                                            :: kall
!  
   if (index(sref,'all') > 0) then
       str = ' '
       kall = 1
   else
       str = ','//trim(sref)//','
       kall = 0
   endif
!  
   if (present(pears)) then
       if (index(str,',u,') > 0 .or. kall==1) pears%par(P_UPAR)%rcod = code
       if (index(str,',v,') > 0 .or. kall==1) pears%par(P_VPAR)%rcod = code
       if (index(str,',w,') > 0 .or. kall==1) pears%par(P_WPAR)%rcod = code
       if (index(str,',beta0,') > 0 .or. kall==1) pears%par(P_B0PAR)%rcod = code
       if (index(str,',beta1,') > 0 .or. kall==1) pears%par(P_B1PAR)%rcod = code
       if (index(str,',beta2,') > 0 .or. kall==1) pears%par(P_B2PAR)%rcod = code                              
!corr-test       if (index(str,',asym,') > 0 .or. kall==1) pears%par(P_ASYM1PAR)%rcod = code
       if (index(str,',asym,') > 0 .or. kall==1) pears%par(P_ASYM1PAR:P_ASYM4PAR)%rcod = code
   endif
!  
   if (present(psvoig)) then
       if (index(str,',u,') > 0 .or. kall==1) psvoig%par(PV_UPAR)%rcod = code
       if (index(str,',v,') > 0 .or. kall==1) psvoig%par(PV_VPAR)%rcod = code
       if (index(str,',w,') > 0 .or. kall==1) psvoig%par(PV_WPAR)%rcod = code
       if (index(str,',eta0,') > 0 .or. kall==1) psvoig%par(PV_E0PAR)%rcod = code
       if (index(str,',eta1,') > 0 .or. kall==1) psvoig%par(PV_E1PAR)%rcod = code
       if (index(str,',eta2,') > 0 .or. kall==1) psvoig%par(PV_E2PAR)%rcod = code
!corr-test       if (index(str,',asym,') > 0 .or. kall==1) psvoig%par(PV_ASYM1PAR)%rcod = code
       if (index(str,',asym,') > 0 .or. kall==1) psvoig%par(PV_ASYM1PAR:PV_ASYM4PAR)%rcod = code
   endif
!  
   if (present(tchh)) then
       if (index(str,',u,') > 0 .or. kall==1) tchh%par(T_UPAR)%rcod = code
       if (index(str,',v,') > 0 .or. kall==1) tchh%par(T_VPAR)%rcod = code
       if (index(str,',w,') > 0 .or. kall==1) tchh%par(T_WPAR)%rcod = code
       if (index(str,',z,') > 0 .or. kall==1) tchh%par(T_ZPAR)%rcod = code
       if (index(str,',x,') > 0 .or. kall==1) tchh%par(T_XPAR)%rcod = code
       if (index(str,',y,') > 0 .or. kall==1) tchh%par(T_YPAR)%rcod = code
!corr-test       if (index(str,',asym,') > 0 .or. kall==1) tchh%par(T_ASYM1PAR)%rcod = code
       if (index(str,',asym,') > 0 .or. kall==1) tchh%par(T_ASYM1PAR:T_ASYM4PAR)%rcod = code
   endif
!  
   end subroutine set_rcode_function

!--------------------------------------------------------------------------------------

   subroutine calcola_profilo_leb (Nactot)
   USE RefinecomRef
   USE GENLSQ
   USE pearsonf
   USE psvoigtf
   USE tchzf
   USE profile_function
   USE reflection_type_util
   USE variables, only: dataset,cryst
   USE arrayutil
   USE crystal_phase
!
   integer, intent(in)           :: Nactot
   real, dimension(:), allocatable :: Iobs
   real, dimension(:), allocatable :: fold
   real, dimension(:), allocatable :: Iobsold
   real                          :: scaleRF
   integer                       :: n,ncl
   real                          :: norms
   real                          :: sumfc
   real                          :: tol
   integer                       :: ierp
   integer                       :: nph,nref,nrefu
!
   if (rfcond%raction /= LEBAIL_DECOMP) return
!
   nph = 1
   tol = lebtol
   do nph=1,numphase(cryst)
      ncl = 10
      nref = cryst(nph)%numref()
      call new_array(Iobs,nref)
      call new_array(fold,nref)
      call new_array(Iobsold,nref)
      sumfc = sum(cryst(nph)%ref%fo)
      Iobs = cryst(nph)%ref%fo**2 
!oldref      nrefu = count(cryst(nph)%ref%tthd(1) <= dataset(1)%xmaxc())  !!!! .and. ref%tthd(1) >= rfc%lim%ttmin)
      nrefu = cryst(nph)%numref()
      do n=1,ncl

         Iobsold = Iobs
         fold = cryst(nph)%ref%fo
         chisquared_old = chisquarednow

         select case (dataset(1)%typefun(nph))

           case (PEARSON7)
             call foleb_pears(cryst(nph)%ref,1,nrefu,cryst(nph)%ref%fo**2,Iobs,dataset(1)%pear(nph),cryst(nph)%scal,      &
                              dataset(1)%y,dataset(1)%yc,dataset(1)%x0,dataset(1)%yb,nph)

           case (PVOIG)
             call foleb_psvoi(cryst(nph)%ref,1,nrefu,cryst(nph)%ref%fo**2,Iobs,dataset(1)%pvoi(nph),cryst(nph)%scal,      &
                              dataset(1)%y,dataset(1)%yc,dataset(1)%x0,dataset(1)%yb,nph)

           case (TCHZ)
             call foleb_tchz(cryst(nph)%ref,1,nrefu,cryst(nph)%ref%fo**2,Iobs,dataset(1)%tch(nph),cryst(nph)%scal,        &
                              dataset(1)%y,dataset(1)%yc,dataset(1)%x0,dataset(1)%yb,nph)

         end select

         cryst(nph)%ref%fo = sqrt(Iobs)
         scaleRF = sum(cryst(nph)%ref%fo) / sumfc
         cryst(nph)%ref%fo = cryst(nph)%ref%fo / scaleRF

         call setRInt(cryst(nph)%ref)
!
         call calcola_profilo(Nactot,ierp)
!
         if (chisquarednow > tol*chisquared_old) then
             cryst(nph)%ref%fo = fold  
             call setRInt(cryst(nph)%ref)
             call calcola_profilo(Nactot,ierp)
             !write(6,*)'dopo ripristino chi=',chisquarednow
             !write(6,*)'prima ripristino chi=',chisquared_old
             exit
         endif
         norms = maxval(abs(Iobsold-Iobs)/max(Iobs,1.0))
         !write(6,*)'Ciclo n.',n,'norm=',norms
         !prova call erreF(ref%fo,Iobs,ref%fc**2)
         !if (knw == 1) rinfo%rfvo = Rvalue(ref%fv,ref%fo)
         if (norms < 0.01) exit
!
      enddo
   enddo
!
   end subroutine calcola_profilo_leb
   
!--------------------------------------------------------------------------------------
  
   subroutine get_rbragg(refl,nreff,rbragg,rf)
!
!  Calcola R-Bragg e R-F
!
   USE RefinecomRef
   USE stat, only:Rvalue
   USE counts
   USE profile_function
   USE reflection_type_util, only: reflection_type
   USE variables, only: dataset,cryst
   USE pearsonf
   USE psvoigtf
   USE tchzf
!
   type(reflection_type), dimension(:), allocatable, intent(in) :: refl
   integer, intent(in)                                          :: nreff
   real, intent(out)                                            :: rbragg,rf
   real, dimension(nreff)                                       :: Fobs2,Iobs
   real, dimension(nreff)                                       :: Fcal2,Ical
!corr   real, dimension(rfcond%lim%nrefu) :: Fobs2,Iobs
!corr   real, dimension(rfcond%lim%nrefu) :: Fcal2,Ical
   integer                                                      :: i,nph
!
   Fcal2 = refl%fc**2 
   nph = 1
   select case (dataset(1)%typefun(nph))

     case (PEARSON7)
       call foleb_pears(refl,1,nreff,Fcal2,Fobs2,dataset(1)%pear(nph),cryst(1)%scal,    &
                        dataset(1)%y,dataset(1)%yc,dataset(1)%x0,dataset(1)%yb,nph)

     case (PVOIG)
       call foleb_psvoi(refl,1,nreff,Fcal2,Fobs2,dataset(1)%pvoi(nph),cryst(1)%scal,    &
                        dataset(1)%y,dataset(1)%yc,dataset(1)%x0,dataset(1)%yb,nph)

     case (TCHZ)
       call foleb_tchz(refl,1,nreff,Fcal2,Fobs2,dataset(1)%tch(nph),cryst(1)%scal,      &
                       dataset(1)%y,dataset(1)%yc,dataset(1)%x0,dataset(1)%yb,nph)

   end select
   !call erreF(sqrt(Fobs2),Fobs2,Fcal2)
   do i=1,nreff
      Iobs(i) =  refl(i)%m*Fobs2(i)*refl(i)%po*refl(i)%lp(1)*refl(i)%ab(1)
      Ical(i) =  refl(i)%m*Fcal2(i)*refl(i)%po*refl(i)%lp(1)*refl(i)%ab(1)
   enddo
   rf = Rvalue(sqrt(Fobs2),refl%fc)
   rbragg = Rvalue(Iobs,Ical)
!
   end subroutine get_rbragg
   
  !------------------------------------------------------------------------------------------
  
   subroutine genera_profilo2(sumyco)
   USE variables, only: cryst,dataset
   USE cryutil
   USE crystal_phase
!
   real, intent(out) :: sumyco
   integer           :: nph
!
   do nph=1,numphase(cryst)
      call pearson_calc(cryst(nph),cryst(nph)%yc,sumyco,dataset(1)%nwave)
   enddo
!
   end subroutine genera_profilo2
   
!-----------------------------------------------------------------------

   subroutine CalcI()
   USE variables, only:cryst,dataset
   USE atom_type_util
   USE General, only: nsyn
   USE crystal_phase
   implicit none
   integer :: nph

   do nph=1,numphase(cryst)
      call cryst(nph)%LPcorr(dataset(1)%radtype,dataset(1)%sync,dataset(1)%nwave)
      call cryst(nph)%ABScorr(dataset(1)%nwave)
      call cryst(nph)%sfactor()
      cryst(nph)%ref%po = 1.0
!corr      do i=1,dataset(1)%nwave
!corr         cryst(nph)%ref%ab(i) = 1.0
!corr      enddo
      call setRInt(cryst(nph)%ref)   !!!TODO: rapIcalc for each phase
   enddo
!
   end subroutine CalcI
   
!-----------------------------------------------------------------------

   subroutine setRInt(refl)
   USE RefinecomRef, only: rfcond
   USE reflection_type_util, only : reflection_type
   type(reflection_type), dimension(:), intent(inout) :: refl
   real, dimension(size(refl))                        :: Icalc
   integer                                            :: i
   if (rfcond%raction > 0) then
       do i=1,size(refl)
          Icalc(i)=refl(i)%m*(refl(i)%fo**2)*refl(i)%po*refl(i)%lp(1)*refl(i)%ab(1)
       enddo
   else
       do i=1,size(refl)
          Icalc(i)=refl(i)%m*(refl(i)%fc**2)*refl(i)%po*refl(i)%lp(1)*refl(i)%ab(1)
       enddo
   endif
   refl(:)%rapI=1.0+Icalc(:)/maxval(Icalc(:))
!
   end subroutine setRInt

! ----------------------------------------------------------------------

   subroutine define_strategy_struct(codes,strategy,radtype)
   USE setref
   USE variables, only: cryst
   USE crystal_phase
   integer, intent(in)                                    :: codes
   type(refinement_strategy_type), intent(inout)          :: strategy
   integer, intent(in)                                    :: radtype
   integer                                                :: nph
!   
   nph = 1
   select case (codes)
      case (1)
        do nph=1,numphase(cryst)
           call strategy%add_stage('XYZ@SEQ',cryst(nph)%at,radtype,[nph])
           call strategy%add_stage('XYZ',cryst(nph)%at,radtype,[nph])
           call strategy%add_stage('B@SEQ',cryst(nph)%at,radtype,[nph])
           call strategy%add_stage('B',cryst(nph)%at,radtype,[nph])
           call strategy%add_stage('XYZB@SEQ',cryst(nph)%at,radtype,[nph])
           call strategy%add_stage('XYZB',cryst(nph)%at,radtype,[nph])
        enddo
        if (numphase(cryst) > 1) call strategy%add_stage('XYZB')

      case (2)
        do nph=1,numphase(cryst)
           call strategy%add_stage('XYZ',[nph])
           call strategy%add_stage('XYZB',[nph])
        enddo
        if (numphase(cryst) > 1)call strategy%add_stage('XYZB')

      case (3)
        do nph=1,numphase(cryst)
           call strategy%add_stage('XYZB',[nph])
        enddo
        if (numphase(cryst) > 1)call strategy%add_stage('XYZB')
   end select
!
   end subroutine define_strategy_struct

!--------------------------------------------------------------------

   subroutine define_strategy_prof(strategy,raction)
   USE setref
   USE profile_function
   USE RefinecomRef
   USE variables, only: cryst,dataset
   USE crystal_phase
   type(refinement_strategy_type), intent(inout) :: strategy
   integer, intent(in)                           :: raction
   integer                                       :: nph

!!!!!TEST
     !  call strategy%add_stage('scale')
     !  return
!!!!!TEST
   select case (raction)
!corr     case (RIET_REFINE, LEBAIL_DECOMP)
!corr       call strategy%add_stage('zero')
!corr       call strategy%add_stage('zero_scale')
!corr       call strategy%add_stage('zero_scale_back')
!corr       call strategy%add_stage('scale_cell')
!corr       call strategy%add_stage('zero_scale_back_cell')
!corr       select case(rfcond%typefun)
!corr         case(PEARSON7)
!corr           call strategy%add_stage('scale_w_beta0_asym')
!corr           call strategy%add_stage('zero_back_scale_cell_w_beta0_asym')
!corr           call strategy%add_stage('scale_u_v_w')
!corr           call strategy%add_stage('zero_back_scale_u_v_w_beta0_asym')
!corr           call strategy%add_stage('back_scale_asym_beta0_beta1_beta2')

     case (RIET_REFINE, LEBAIL_DECOMP)
!!!!!!!!!!!!!!test
      ! call strategy%add_stage('zero_scale')
      !    call strategy%add_stage('asym')
      !        call strategy%add_stage('u_v_w')
      !    return
!!!!!!!!!!!!!!test
       call strategy%add_stage('zero')
       call strategy%add_stage('zero_scale')
       call strategy%add_stage('zero_scale_back')
       do nph=1,numphase(cryst)
          call strategy%add_stage('scale_cell',ph=[nph])
       enddo
       if (numphase(cryst) > 1) call strategy%add_stage('scale_cell')
       call strategy%add_stage('zero_scale_back_cell')
       call strategy%add_stage('sd')
       call strategy%add_stage('zero_scale_back_cell_sd')
       call strategy%add_stage('st')
       call strategy%add_stage('scale_st')
       !call strategy%add_stage('zero_scale_back_cell_st')

       do nph=1,numphase(cryst)
          select case(dataset(1)%typefun(nph))
            case(PEARSON7)
              call strategy%add_stage('scale_w_beta0_asym',ph=[nph])
              call strategy%add_stage('zero_back_scale_cell_w_beta0_asym',ph=[nph])
              call strategy%add_stage('scale_u_v_w',ph=[nph])
              call strategy%add_stage('zero_back_scale_u_v_w_beta0_asym',ph=[nph])
              call strategy%add_stage('back_scale_asym_beta0_beta1_beta2',ph=[nph])
            case(PVOIG)
              call strategy%add_stage('w',ph=[nph])
              call strategy%add_stage('w_scale_back_cell_zero',ph=[nph])
              call strategy%add_stage('scale_w_eta0_asym',ph=[nph])
              call strategy%add_stage('zero_back_scale_cell_w_eta0_asym',ph=[nph])
              call strategy%add_stage('scale_u_v_w',ph=[nph])
              call strategy%add_stage('zero_back_scale_u_v_w_eta0_asym',ph=[nph])
              call strategy%add_stage('back_scale_asym_eta0_eta1_eta2',ph=[nph])
            case(TCHZ)
              call strategy%add_stage('w',ph=[nph])
              call strategy%add_stage('w_scale_back_cell_zero',ph=[nph])
              call strategy%add_stage('scale_w_xp_yp_asym',ph=[nph])
              call strategy%add_stage('zero_back_scale_cell_w_xp_yp_zp_asym',ph=[nph])
              call strategy%add_stage('scale_u_v_w',ph=[nph])
              call strategy%add_stage('zero_back_scale_u_v_w_xp_yp_asym',ph=[nph])
              call strategy%add_stage('back_scale_asym_xp_yp_zp',ph=[nph])
          end select
       enddo

     case (PAWLEY_DECOMP)
!test       call strategy%add_stage('zero_scale')
!test       call strategy%add_stage('fo1_scale')
!test       call strategy%add_stage('zero_scale')
!test       call strategy%add_stage('fo1_scale')
!test       call strategy%add_stage('zero_scale_back')
!test       call strategy%add_stage('fo1_scale')
!test       call strategy%add_stage('scale_cell')
!test       call strategy%add_stage('fo1_scale')
!test       call strategy%add_stage('zero_back_scale_cell')
!test       call strategy%add_stage('scale_fo1')
!test
!test           call strategy%add_stage('scale_w_beta0_asym')
!test           call strategy%add_stage('scale_fo1')
!test           call strategy%add_stage('scale_zero_back_w_beta0_asym_cell')
!test           call strategy%add_stage('scale_fo1')
!test           call strategy%add_stage('scale_u_v_w')
!test           call strategy%add_stage('scale_fo1')
!test           call strategy%add_stage('scale_zero_back_beta0_asym_u_v_w_fo1')
!test           call strategy%add_stage('scale_fo1')
!test           call strategy%add_stage('scale_back_beta0_beta1_beta2_asym')
!test           call strategy%add_stage('scale_back_beta0_beta1_beta2_asym_fo1')
!test       call strategy%add_stage('scale_fo1')
!test       call strategy%add_stage('scale_zero_back_cell_all_fo1')
!test       call strategy%add_stage('scale_fo1')

       call strategy%add_stage('zero_scale')
       call strategy%add_stage('fo4_scale')
       call strategy%add_stage('fo5_scale')
       call strategy%add_stage('zero_scale')
       call strategy%add_stage('fo4_scale')
       call strategy%add_stage('fo5_scale')
       call strategy%add_stage('zero_scale_back')
       call strategy%add_stage('fo4_scale')
       call strategy%add_stage('fo5_scale')
       call strategy%add_stage('scale_cell')
       call strategy%add_stage('fo4_scale')
       call strategy%add_stage('fo5_scale')
       call strategy%add_stage('zero_back_scale_cell')
       call strategy%add_stage('scale_fo4')
       call strategy%add_stage('scale_fo5')

!corr       select case(rfcond%typefun)
!!!!!FIX THIS PART
       select case(dataset(1)%typefun(nph))
         case(PEARSON7)
           call strategy%add_stage('scale_w_beta0_asym')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_zero_back_w_beta0_asym_cell')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_u_v_w')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_zero_back_beta0_asym_u_v_w_fo2')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_back_beta0_beta1_beta2_asym_fo2')
         case(PVOIG)
           call strategy%add_stage('scale_w_eta0_asym')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_zero_back_w_eta0_asym_cell')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_u_v_w')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_zero_back_eta0_asym_u_v_w_fo2')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_back_eta0_eta1_eta2_asym_fo2')
         case(TCHZ)
           call strategy%add_stage('scale_w_xp_yp_asym')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_zero_back_w_xp_yp_asym_cell')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_u_v_w')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_zero_back_xp_yp_asym_u_v_w_fo2')
           call strategy%add_stage('scale_fo4')
           call strategy%add_stage('scale_fo5')
           call strategy%add_stage('scale_back_xp_yp_zp_asym_fo2')
       end select
       call strategy%add_stage('scale_fo4')
       call strategy%add_stage('scale_fo5')
       call strategy%add_stage('scale_fo1')
       call strategy%add_stage('scale_zero_back_cell_all_fo2')
       call strategy%add_stage('scale_fo4')
       call strategy%add_stage('scale_fo6')
       call strategy%add_stage('scale_fo7')
   end select
   call strategy%add_stage('zero_back_scale_cell_all')

   end subroutine define_strategy_prof

!-------------------------------------------------------------------

   subroutine check_btcryst(outrange,btmin,btmax,sprint,nberr)
!
!  Check thermal factor. If kaction = 1 parameter is blocked
!
   USE constraints
   USE type_constants, only: DP
   USE atom_type_util
   USE crystal_phase
   USE strutil
   USE variables, only: cryst
!
   implicit none
   logical, dimension(:), intent(inout) :: outrange   ! true for parameters out of range
   real, intent(in)                     :: btmin,btmax  ! max e min per parametri termici
   logical, intent(in)                  :: sprint       ! codice di stampa
   integer, intent(out)                 :: nberr        ! codice di errore
   integer                              :: i,nph
   integer                              :: nco
   integer                              :: codej
!
   nberr = 0
   outrange(:) = .false.
   do nph=1,numphase(cryst)
      do i=1,cryst(nph)%natoms()
         nco = par_in_constraint(cryst(nph)%constr,i,4) 
         if (nco > 0) then
             codej = cryst(nph)%constr(nco)%ref
         else
             codej = cryst(nph)%at(i)%rcod(4)
         endif
         if (codej > 0) then
             if (cryst(nph)%at(i)%biso < btmin .or. cryst(nph)%at(i)%biso > btmax) then
                 if (sprint) then
                     write(6,'(a,a,a,a,a)')' WARNING: for atom ',cryst(nph)%at(i)%glab(i),' thermal factor ',  &
                           r_to_s(cryst(nph)%at(i)%biso,2),' is out of range '//r_to_s(btmin,2)//'-'//r_to_s(btmax,2)
                 endif
                 outrange(codej) = .true.
                 nberr = nberr + 1
             endif
         endif
      enddo
   enddo
!
   end subroutine check_btcryst

!-------------------------------------------------------------------

   subroutine check_occupancy(x,atom,constr,kaction,sprint,ierror)
!
!  Check thermal factor. If kaction = 1 parameter is blocked
!
   USE constraints
   USE type_constants, only: DP
   USE atom_type_util
!
   implicit none
   real(DP), dimension(:), intent(in)                        :: x            ! parametri affinati
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(constraint_type), dimension(:), allocatable          :: constr
   integer, intent(in)                                       :: kaction      ! codice per gestione del parametro
   character(len=*), intent(in)                              :: sprint       ! codice di stampa
   integer, intent(out)                                      :: ierror       ! codice di errore
   integer                                                   :: i
   integer                                                   :: nco
   integer                                                   :: codej
   real, parameter                                           :: ocmin=0.0,ocmax=1.0  ! min and max for occupancy
!
   ierror = 0
   do i=1,numatoms(atom)
      nco = par_in_constraint(constr,i,5) 
      if (nco > 0) then
          codej = constr(nco)%ref
      else
          codej = atom(i)%rcod(5)
      endif
      if (codej > 0) then
!!!!TODO: check every occ in constraint
          if (x(codej) < ocmin .or. x(codej) > ocmax) then
              if (index(sprint,'V') > 0) then
                  write(6,'(a,i0,a,f0.2,a,a,f0.2,a,f0.2,a)')' WARNING: for atom ',i,' occupancy factor ',  &
                        x(codej),' is out of range ','(',ocmin,'<-->',ocmax,')'
              endif
              if (kaction == 1) then
                  if (nco > 0) then
                      constr(nco)%ref = 0
                  else
                      atom(i)%rcod(5) = 0
                  endif
              endif
              ierror = 1
          endif
      endif
   enddo
!
   end subroutine check_occupancy

!--------------------------------------------------------------------
   subroutine check_fo(x,kaction,ierror)
!
!  Controlla i parametri termici, se kaction = 1 il parametro viene bloccato
!
   USE type_constants, only:DP
   USE refinecomref
   USE reflection_type_util
   USE variables, only: cryst
   implicit none
   real(DP), dimension(:), intent(in) :: x    
   integer                               :: kaction
   integer, intent(out)                  :: ierror
   integer                               :: nneg
   integer                               :: i
   integer                               :: nkref
!
              return
   nneg = 0
   nkref = 0
!corr   do i=1,rfcond%lim%nrefu
   do i=1,numrefl(cryst(1)%ref)
      if (fopar(i)%rcod > 0) then
          nkref = nkref + 1
          if (x(nkref) < 0) then
              if (kaction == 1) fopar(i)%rcod = 0
              nneg = nneg + 1
          endif
      endif
   enddo
   if (nneg > 0) then
       ierror = 2
       write (0,*)'fo negativi:',nneg
   endif
!
   end subroutine check_fo

!-------------------------------------------------------------------------------

   subroutine printfin(kpr,title,opt)
   USE RefinecomRef
   USE background
   USE RRESTR
   USE variables, only:cryst,dataset
   USE General, only:lo
   USE atom_type_util
   USE STRUTIL
   USE unit_cell
   USE constraints
   USE profile_function
   USE reflection_type_util
   USE crystal_phase
   USE quantmod
   USE cryutil
   USE pearsonf
   USE psvoigtf
   USE tchzf
!
   implicit none
   integer, intent(in)                    :: kpr
   character(len=*), optional, intent(in) :: title
   character(len=*), intent(in), optional :: opt
   integer                                :: nph,npar
   character(len=:), allocatable          :: popt
!corr   real, dimension(:), allocatable        :: wcryst,esdw
!
   if (present(opt)) then
       popt = opt
   else
       popt = 'S'
   endif
   write(kpr,'(/1x,80("="))')
   if (present(title)) then
       write(kpr,'(a)') centra_str(title,80)
   endif
!
   call print_rvalues(cryst(1)%ref,kpr,title=.true.)
   select case (rfcond%typewei)
     case (1)
       write(kpr,'(/,1x,a)') 'Weighting scheme: w=1.0'
     case (2)
       write(kpr,'(/,1x,a)') 'Weighting scheme: w=1.0/ycount'
     case (3)
       write(kpr,'(/,1x,a)') 'Weighting scheme: w=1.0/ycount^2'
   end select
   write(kpr,'(1x,a,f0.6,/)') 'Convergence criterium: eps=',rfcond%eps
!
   call print_background(kpr,dataset(1)%cond,dataset(1)%coef,dataset(1)%thzerob)
!corr!
!corr!  Stampa informazioni sul background
!corr   select case (backriet%btype)
!corr      case (POLY)
!corr        write(kpr,'(/1x,a,i0)')'Background modelled by a polynomial of order ',backriet%ncoef-1
!corr
!corr      case (CHEBY)
!corr        write(kpr,'(/1x,a,i0)')'Background modelled by a Chebyshev polynomial of degree ',backriet%ncoef
!corr
!corr      case (FOUR_SERIES)
!corr        write(kpr,'(/1x,a,i0)')'Background modelled by a cosine Fourier series of degree ',backriet%ncoef
!corr
!corr   end select
!corr   if (backriet%btype==POLY) write(kpr,'(9x,a,t20,a)')'Origin',trim(string_sigf(dataset(1)%thzerob,7))
!corr   do i=1,backriet%ncoef
!corr      write(kpr,'(9x,a,i0,t20,a)')'b',i,trim(string_sigf(dataset(1)%coef(i),7))
!corr   enddo
!
!  Zero
!corr   write(kpr,'(/9x,a,t20,a)')trim(zero%str),string_sigf(zero%val,7)
   write(kpr,'(/9x,a,t20,a)')'zero',string_sigf(dataset(1)%zero,7)
!
!  Sample displacement and transparency
   write(kpr,'(/9x,a,t20,a)')'sample di.',string_sigf(dataset(1)%sdisp,7)
   write(kpr,'(/9x,a,t20,a)')'sample tr.',string_sigf(dataset(1)%stran,7)
!
!  Informazioni sul profilo
   do nph=1,numphase(cryst)
      select case (dataset(1)%typefun(nph))

        case (PEARSON7)
          write(kpr,'(/1x,a)') 'Profile shape function: Pearson VII'
          do npar=P_UPAR,P_ASYM4PAR
             write(kpr,30)dataset(1)%pear(nph)%par(npar)%str,string_sigf(dataset(1)%pear(nph)%par(npar)%val,7)
          enddo

        case (PVOIG)
          write(kpr,'(/1x,a)') 'Profile shape function: Pseudo-Voigt '
          do npar=PV_UPAR,PV_ASYM4PAR
             write(kpr,30)dataset(1)%pvoi(nph)%par(npar)%str,string_sigf(dataset(1)%pvoi(nph)%par(npar)%val,7)
          enddo

        case (TCHZ)
          write(kpr,'(/1x,a)') 'Profile shape function: TCH Pseudo-Voigt '
          do npar=T_UPAR,T_ASYM4PAR
             write(kpr,30)dataset(1)%tch(nph)%par(npar)%str,string_sigf(dataset(1)%tch(nph)%par(npar)%val,7)
          enddo

      end select
!
      if (index(popt,'S') > 0) then
!
!         Scale
          write(kpr,'(/9x,a,t20,a)')'scale',string_sigf(cryst(nph)%scal,7)
!
!         Cell parameters
          write(kpr,'(/)')
          call print_cell(cryst(nph)%get_cellpar(),kpr=kpr)
!
!         Orientazione preferenziale
          if (cryst(nph)%kopcorr > 0) then
              write(kpr,'(/,a,3i3/)') '  Preferred orientation on plane: ',cryst(nph)%hklpo(:)
              write(kpr,'(9x,a,t20,a)')'G',trim(string_sigf(cryst(nph)%gpar,7))
          endif
!
          if (cryst(nph)%natoms() > 0) then
              call print_atoms(cryst(nph)%at,title='Structure')
              call print_constraints(cryst(nph)%constr,cryst(nph)%at,lo,.false.) ! print refined constraints
              call print_restraints_all(kpr,cryst(nph)%res,cryst(nph)%at,cryst(1)%cell)
              if (cryst(nph)%is_known()) call to_coord_cry(cryst(nph),kpr)
          endif
      endif
   enddo
30 format (9x,a,t20,a)

   call quantitative(cryst)
   call quantitative_print(cryst,lo)
!
   write(kpr,'(1x,80("="))')
!
   end subroutine printfin
!----------------------------------------------------------------------------------
   subroutine print_rvalues(refl,kpr,title)
   USE RefinecomRef
   USE stat, only:Rvalue
   USE reflection_type_util
   USE variables, only: cryst
   type(reflection_type), dimension(:), intent(in), allocatable :: refl
   integer, intent(in) :: kpr
   logical, intent(in) :: title
   real                :: rbragg,rf,rfvc,rfvo
!
   if (title) then
       write(kpr,'(/1x,a)') 'Agreement factors'
   endif
   write(kpr,"(2x,a,f10.3,a,f10.3,a,f10.3,a,f15.3)")" Rp =",rinfo%rp,"   Rwp =",rinfo%rwp,"   Re =",errexp, "   Chi2=",chi2
   write(kpr,"(2x,a,f10.3,a,f10.3,a,f10.3,a,f15.3)")" Rp'=",errep1,  "   Rwp'=",errepw1,  "   Re'=",errexp1,"   DW  =",rinfo%dw
   if (rfcond%raction == 0) then 
!
!      Calcola e stampa RF e R-bragg
       call get_rbragg(refl,numrefl(refl),rbragg,rf)
       write(kpr,30)rf,rbragg
30     format(3x,'R-structure factor =',f8.3,'   R-Bragg factor     =',f8.3)
!
!      Calcola e stampa R tra veri e calcolati
       if (cryst(1)%is_known()) then
           rfvc = Rvalue(refl%fv,refl%fc)
           write(kpr,"(3x,'RF(true-cal)       =',f8.3)")rfvc
       endif
   else
!
!      Calcola e stampa R tra veri e osservati
       if (cryst(1)%is_known()) then
           rfvo = Rvalue(refl%fv,refl%fo)
           write(kpr,"(3x,'RF(true-obs)       =',f8.3)")rfvo
       endif
   endif
!
   end subroutine print_rvalues

! ----------------------------------------------------------------------
   subroutine print_parameter(val,sd,rcod,str,sigsh,is_std,kpr,separ)
   USE progtype
   USE strutil
   USE genlsq, only:shiftx
   real, intent(in)              :: val,sd
   integer, intent(in)           :: rcod
   character(len=*), intent(in)  :: str
   integer, intent(in)           :: sigsh
   logical, intent(in)           :: is_std
   integer, intent(in)           :: kpr
   logical, intent(in), optional :: separ
   logical                       :: sep
   real                          :: shiftp   !!!!!corr,stdval
!
   if (present(separ)) then
       sep = separ
   else
       sep = .false.
   endif
   if (is_std) then
       if (abs(sd) >= epsilon(0.0)) then
           write(kpr,'(9x,a,t20,a)')trim(str),trim(string_esd(val,sd))
       else
           write(kpr,'(9x,a,t20,a)')trim(str),trim(string_sigf(val,7))
       endif
   else
       if (rcod > 0) then
           shiftp = shiftx(rcod)  
           if (sep) then
               write(kpr,'(9x,a,t20,a,t40,a/)')str,string_sigf(val,7),trim(string_sigf(shiftp,sigsh))
           else
               write(kpr,'(9x,a,t20,a,t40,a)')str,string_sigf(val,7),trim(string_sigf(shiftp,sigsh))
           endif
       endif
   endif
!
   end subroutine print_parameter

  !-------------------------------------------------------------------------------------

   subroutine print_background(kpr,cond,coef,zerob,sd,code)
!
!  Print background information
!
   USE strutil
   USE background
   integer, intent(in)                         :: kpr
   type(back_condition_type), intent(in)       :: cond
   real, dimension(:), intent(in)              :: coef
   real, intent(in)                            :: zerob
   real, dimension(:), intent(in), optional    :: sd
   integer, dimension(:), intent(in), optional :: code
   integer                                     :: i,sigsh=5
!
   select case (cond%btype)
      case (POLY)
        write(kpr,'(/1x,a,i0)')'Background modelled by a polynomial of order ',cond%ncoef-1

      case (CHEBY)
        write(kpr,'(/1x,a,i0)')'Background modelled by a Chebyshev polynomial of degree ',cond%ncoef

      case (FOUR_SERIES)
        write(kpr,'(/1x,a,i0)')'Background modelled by a cosine Fourier series of degree ',cond%ncoef

   end select
   if (cond%btype==POLY) write(kpr,'(9x,a,t20,a)')'Origin',trim(string_sigf(zerob,7))
   if (present(sd)) then
       do i=1,cond%ncoef
          call print_parameter(coef(i),sd(i),code(i),'b'//i_to_s(i),sigsh,.true.,kpr)
       enddo
   else
       do i=1,cond%ncoef
          write(kpr,'(9x,a,i0,t20,a)')'b',i,trim(string_sigf(coef(i),7))
       enddo
   endif

   end subroutine print_background

!--------------------------------------------------------------------

   subroutine azzeraref(zerop)
!
!  Se zerop = .false. l'OP non e azzerata
!
   USE RefinecomRef
   USE constraints
   USE unit_cell
   USE spginfom
   USE atom_type_util
   USE unit_cell
   USE setref
   USE variables, only: cryst,dataset
   USE crystal_phase
   implicit none
   logical, optional, intent(in)                                   :: zerop
   logical                                                         :: zerogpo
   integer                                                         :: nph
!   
!corr   zero%rcod = 0
   dataset(1)%zcode = 0
   dataset(1)%sdcode = 0
   dataset(1)%stcode = 0
!corr   coefbkg%rcod = 0
   dataset(1)%bcode(:) = 0
   if (allocated(fopar)) fopar(:)%rcod = 0
   if (present(zerop)) then
       zerogpo = zerop
   else
       zerogpo = .true.
   endif
          ! zerogpo = .true.
!corr   if (zerogpo) gpo%rcod = 0
   if (zerogpo) cryst(1)%gcode = 0
   rinfo%refine_profile = .false.
!
   do nph=1,numphase(cryst)
      call set_rcode_function(dataset(1)%pear(nph),dataset(1)%pvoi(nph),dataset(1)%tch(nph),sref='all',code=0)
      call cryst(nph)%set_refine_cell(0)
      cryst(nph)%is_refined = .false.
      cryst(nph)%scode = 0
      if (cryst(nph)%natoms() > 0) then
          call set_rcode_atoms(cryst(nph)%at,cryst(nph)%cell,cryst(nph)%spg,cryst(nph)%constr,'XYZBO',0)
          call refine_constraint(cryst(nph)%constr,cryst(nph)%at,0)
      endif
   enddo
!
   end subroutine azzeraref

!-----------------------------------------------------------------

!corr   subroutine refine_all(atom,legm,cell,spg,elem,radtype,constr,refop,nph)
   subroutine refine_all(refop,nph)
!corr   USE RefinecomRef, only:rfcond
   USE messagemod
   USE elements
   USE unit_cell
   USE atom_type_util
   USE profile_function
   USE setref
   USE spginfom
   USE constraints
   USE connect_mod
   USE variables, only: cryst,dataset
   implicit none
!corr   type(atom_type), dimension(:), allocatable, intent(inout)       :: atom
!corr   type(bond_type), dimension(:), allocatable, intent(in)          :: legm
!corr   type(cell_type), intent(inout)                                  :: cell
!corr   type(spaceg_type), intent(in)                                   :: spg
!corr   type(element_type), dimension(:), allocatable, intent(in)       :: elem
!corr   type(constraint_type), dimension(:), allocatable, intent(inout) :: constr
!corr   integer, intent(in)                                             :: radtype
   logical, intent(in)                                             :: refop
   integer, intent(in)                                             :: nph
!corr   integer                                                         :: i
!
   dataset(1)%zcode = 1
   dataset(1)%sdcode = 1
   dataset(1)%stcode = 1
   dataset(1)%bcode = 1
   call refine_crystal(cryst(nph),dataset(1),nph,refop)
!corr   call calcola_occ(atom,spg,cell)
!corr   call make_constraints(atom,legm,constr,spg,cell,rfcond%riding_model,rfcond%adp_factor)
!corr   cryst(nph)%scode = 1
!corr   if (refop) cryst(nph)%gcode = 1
!corr   call cell%set_refine(spg,1)
!corr   select case (dataset(1)%typefun(nph))
!corr      case (PEARSON7)
!corr        call set_rcode_function(dataset(1)%pear(nph),sref='all',code=1)
!corr      case (PVOIG)
!corr        call set_rcode_function(psvoig=dataset(1)%pvoi(nph),sref='all',code=1)
!corr      case (TCHZ)
!corr        call set_rcode_function(tchh=dataset(1)%tch(nph),sref='all',code=1)
!corr        call set_rcode_function(tchh=dataset(1)%tch(nph),sref='z',code=0)
!corr   end select
!corr!
!corr!  Refine all species excluded H-atoms
!corr   do i=1,numelem(elem)
!corr      if ((radtype == RX_SOURCE) .and. (elem(i)%z /= H_at)) then
!corr          call set_rcode_atoms(atom,cell,spg,constr,'XYZB',1,zspec=[elem(i)%z])
!corr      endif
!corr   enddo
!
   end subroutine refine_all

!------------------------------------------------------------------------------------------------------------------

   subroutine refine_crystal(crystal,datas,nph,refop)
   use datasetmod
   use profile_function
   use elements
   use crystal_phase
   use setref
   use atom_type_util
   type(crystal_phase_t), intent(inout) :: crystal
   type(dataset_type), intent(inout)    :: datas
   integer, intent(in)                  :: nph
   logical, intent(in)                  :: refop
   integer                              :: i
!
   call calcola_occ(crystal%at,crystal%spg,crystal%cell)
   call make_constraints(crystal%at,crystal%bond,crystal%constr,crystal%spg,crystal%cell,crystal%riding_model,crystal%adp_factor)
   crystal%scode = 1
   if (refop) crystal%gcode = 1
   call crystal%set_refine_cell(1)
   select case (datas%typefun(nph))
      case (PEARSON7)
        call set_rcode_function(datas%pear(nph),sref='all',code=1)
      case (PVOIG)
        call set_rcode_function(psvoig=datas%pvoi(nph),sref='all',code=1)
      case (TCHZ)
        call set_rcode_function(tchh=datas%tch(nph),sref='all',code=1)
        call set_rcode_function(tchh=datas%tch(nph),sref='z',code=0)
   end select
!
!  Refine all species excluded H-atoms for X-rays
   do i=1,crystal%numelem()
      if ((datas%radtype == RX_SOURCE) .and. (crystal%elem(i)%z /= H_at)) then
          call set_rcode_atoms(crystal%at,crystal%cell,crystal%spg,crystal%constr,'XYZB',1,zspec=[crystal%elem(i)%z])
      endif
   enddo
!
   end subroutine refine_crystal

!-----------------------------------------------------------------------------------------------

   subroutine pawley(code)
   USE progtype
   USE RefinecomRef
   USE genlsq
   USE reflection_type_util
   USE variables, only: cryst
   integer             :: i
   integer, intent(in) :: code
!
   if (code == 1) then
       allocate(fopar(numrefl(cryst(1)%ref)))
!
!      Associa ad ogni Fo un parametro
!corr       do i=1,rfcond%lim%nrefu
       do i=1,numrefl(cryst(1)%ref)
          fopar(i)%rcod = 0
          fopar%str = 'F'
       enddo
!
       rfcond%raction = 2
       rfcond%algo = GAUNEW              ! tipo di algoritmo
!
   else
       deallocate(fopar)
       rfcond%raction = 0
       rfcond%algo = GAUNEW              ! tipo di algoritmo
   endif
!
   end subroutine pawley

!----------------------------------------------------------------------

   subroutine rietveldexe(refine,init)
   USE RefinecomRef
   USE variables, only: cryst
   USE stat
   USE TestModel
   USE compare_struct
   USE General, only: fname,OUT_FILE,lo,structurename
   USE fileutil
   USE progtype, only: DISTRIBUTED_VER
   USE crystal_phase
   USE quantmod
!
   logical, intent(in), optional :: init
   character(len=*), intent(in)  :: refine
   integer                       :: i
!
   if (present(init)) then
       if (init)  call init_rietveld(1,.true.,.true.)
   endif
!
!corr   write(0,*)'refine: '//trim(refine)
   select case(trim(refine))
      case ('all')
        rfcond%raction = 0
        rfcond%auto_prof = .true.
        rfcond%auto_stru = .true.
        call riet_new(1)
        call quantitative_print(cryst,lo,'S',structurename) !uncomment for test on quantitative

      case ('structure')
        rfcond%raction = 0
        rfcond%auto_prof = .false.
        rfcond%auto_stru = .true.
        call riet_new(1)

      case ('profile')
        rfcond%raction = 1
        !rfcond%sprint = 'SVW'  !; rfcond%raction = 2   !!test
        rfcond%auto_prof = .true.
        rfcond%auto_stru = .false.
        if (rfcond%raction > 0) then
            call setRInt(cryst(1)%ref)
            call Count0()   !!!!cryst(1)%ref)
        endif
!
        if (rfcond%raction == 1) then
            do i=1,3
               rfcond%auto_prof = .true.
               rfcond%auto_stru = .false.
               call init_fobs(cryst(1)%ref)
               call riet_new(i)
            enddo
        else
            call pawley(1)
            call init_fobs(cryst(1)%ref)
            do i=1,3
               rfcond%auto_prof = .true.
               rfcond%auto_stru = .false.
               call riet_new(i)
            enddo
            call pawley(0)
        endif
        !call scrivi_lista6()
   end select
!
   call exitriet()
   if (.not.DISTRIBUTED_VER .and. cryst(1)%is_known()) then
       call to_coord_sir(1,cryst(1)%at,cryst(1)%ref,size(cryst(1)%ref),cryst(1)%cell,cryst(1)%spg,lo,cryst(1)%atpub)
       write(6,'(a,t30,a,3f10.3,a,i6,a,i0,2f10.4)')'TB1'//file_get_name(fname(OUT_FILE)),        &
                              ' R:',rinfo%rwp,rinfo%rp,Rvalue(cryst(1)%ref%fv,cryst(1)%ref%fo),                      &
                              ' trov:',int(vett_test(3)),'/',int(vett_test(1)),vett_test(4),rmsd
   endif
!
   end subroutine rietveldexe

!-------------------------------------------------------------------------------------------------------------

   subroutine print_structure(atom,constr,kpr)
!
!  Stampa solo i parametri affinati
!
   USE genlsq, only:shiftx, step_length
   USE STRUTIL
   USE atom_type_util
   USE constraints
   implicit none
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(constraint_type), dimension(:), allocatable, intent(in) :: constr
   integer, intent(in)               :: kpr
   integer, parameter                :: lenp = 13  ! lunghezza del campo in cui scrivere i parametri
   integer                           :: i,j
!corr   integer                           :: ier
!corr   real                              :: shiftp
   integer                           :: codepar
   integer, dimension(5)             :: id_constr, id_par
   logical, dimension(5)             :: id_constr_ref
   real :: coefc
   character(len=12) :: snopar = '         -  '
   character(len=12) :: snlab
!corr   character(len=:), allocatable :: kprs
!
   write(kpr,'(a)')'                        x           y           z           B          occ'
   do i=1,numatoms(atom)
!
!     Check if par. is in constraint (id_constr,id_par) and if the constraint is refined (id_constr_ref)
      do j=1,5
         id_constr(j) = par_in_constraint(constr,i,j,id_par(j))
         if (id_constr(j) > 0) then
             id_constr_ref(j) = (constr(id_constr(j))%ref > 0) 
         else
             id_constr_ref(j) = .false.
         endif
      enddo
      if (any(atom(i)%rcod > 0) .or. any(id_constr_ref)) then
          snlab = i_to_s(i)//' ('//trim(atom(i)%lab)//') '
          write(kpr,'(3x,a)',advance='no')snlab
!
!         Coordinate
          do j=1,3
!
!            prendo il codice di affinamento del parametro
             if (id_constr(j) > 0) then            
                 codepar = constr(id_constr(j))%ref
                 coefc = constr(id_constr(j))%coef(id_par(j))
             else
                 codepar = atom(i)%rcod(j)
                 coefc = 1.0
             endif
             if (codepar > 0) then
                 !write(kpr,10,advance='no')atom(i)%xc(j)
                 write(kpr,10,advance='no')atom(i)%xc(j)
             else
                 write(kpr,'(a)',advance='no')snopar
             endif
          enddo
!
!         Fattori termici
          if (id_constr(4) > 0) then            
              codepar = constr(id_constr(4))%ref
          else
              codepar = atom(i)%rcod(4)
          endif
          if (codepar > 0) then
              !write(stringp(4),'(f0.6)',iostat=ier)atom(i)%biso
              write(kpr,10,advance='no')atom(i)%biso
          else
              write(kpr,'(a)',advance='no')snopar
          endif
!
!         Occupancies
          if (id_constr(5) > 0) then            
              codepar = constr(id_constr(5))%ref
          else
              codepar = atom(i)%rcod(5)
          endif
          if (codepar > 0) then
              write(kpr,10,advance='yes')atom(i)%och
          else
              write(kpr,'(a)',advance='yes')snopar
          endif
!
!         Write shifts
          if (i==1) then
              snlab = 'shift'
              write(kpr,'(3x,a)',advance='no')snlab
          else
              write(kpr,20,advance='no')
          endif
!
!         Shifts on coordinates
          do j=1,3
!
!            prendo il codice di affinamento del parametro
             if (id_constr(j) > 0) then            
                 codepar = constr(id_constr(j))%ref
                 coefc = constr(id_constr(j))%coef(id_par(j))
             else
                 codepar = atom(i)%rcod(j)
                 coefc = 1.0
             endif
             if (codepar > 0) then
                 write(kpr,10,advance='no')shiftx(codepar)*coefc
             else
                 write(kpr,'(a)',advance='no')snopar
             endif
          enddo
!
!         Shifts on B
          if (id_constr(4) > 0) then            
              codepar = constr(id_constr(4))%ref
              coefc = constr(id_constr(4))%coef(id_par(4))
          else
              codepar = atom(i)%rcod(4)
              coefc = 1.0
          endif
          if (codepar > 0) then
              write(kpr,10,advance='no')shiftx(codepar)*coefc
          else
              write(kpr,'(a)',advance='no')snopar
          endif
!
!         Shifts on occupancies
          if (id_constr(5) > 0) then            
              codepar = constr(id_constr(5))%ref
          else
              codepar = atom(i)%rcod(5)
          endif
          if (codepar > 0) then
              write(kpr,10,advance='yes')shiftx(codepar)*step_length
          else
              write(kpr,'(a)',advance='yes')snopar
          endif
      endif
   enddo
!
!  Set format here!
10 format(f12.4)
20 format(3x,12x)
!
   end subroutine print_structure

!-------------------------------------------------------------------------------------------------------------

   subroutine prerefine_structure(atom,refl,constr,nkref)
   !USE RefinecomRef
   USE GENLSQ
   USE atom_type_util
   USE reflection_type_util
   USE constraints
   !USE variables, only: cryst
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(reflection_type), dimension(:), allocatable          :: refl
   type(constraint_type), dimension(:), allocatable          :: constr
   integer, intent(inout)                                    :: nkref
   character(4), dimension(5)                                :: atomstring = ['x   ','y   ','z   ','B   ','occ.']
   character(len=20)                                         :: strat
   integer                                                   :: i,j,kk,iv
   integer                                                   :: kpar
   real                                                      :: bfacmax
   integer                                                   :: katom
   real                                                      :: coefc
   integer                                                   :: npar
!
!  Struttura
!corr   if (rinfo%refine_structure) then
   !if (cryst(1)%is_refined) then
       bfacmax = 35. / maxval(refl%slaq)   ! 35 = 30*ln(10) / 2
       do j=1,numatoms(atom)
          write(strat,'(i0,a)')j,' ('//trim(atom(j)%lab)//')'
          do kk=1,3        ! x,y,z
             if (atom(j)%rcod(kk) > 0) then
                nkref = nkref + 1
                par_string(nkref) = trim(atomstring(kk))//strat
                xpar(nkref) = atom(j)%xc(kk)
                atom(j)%rcod(kk) = nkref
                typex(nkref) = 10.0
             endif
          enddo
          if (atom(j)%rcod(4) > 0) then
             nkref = nkref + 1
             par_string(nkref) = trim(atomstring(4))//strat
             xpar(nkref) = atom(j)%biso
             atom(j)%rcod(4) = nkref
             typex(nkref) = bfacmax
          endif
          if (atom(j)%rcod(5) > 0) then
             nkref = nkref + 1
             par_string(nkref) = trim(atomstring(5))//strat
             xpar(nkref) = atom(j)%och
             atom(j)%rcod(5) = nkref
          endif
       enddo
   !endif
!
   do i=1,nconstraints(constr)
      if (constr(i)%ref > 0) then
          select case (constr(i)%code)
            case (1)  ! constrain sui b, un 1 par. indipendente
              nkref = nkref + 1 
              write(par_string(nkref),'(a,i3)')' co.',i
              !katom = constr(i)%vat(1)
              katom = constr(i)%vat(size(constr(i)%vat))
              xpar(nkref) = atom(katom)%biso

            case (2)  ! constrain sulle coordinate, un 1 par. indipendente
              nkref = nkref + 1
              write(par_string(nkref),'(a,i3)')' co.',i
              katom = constr(i)%vat(1)
              kpar = constr(i)%vpar(1)
              coefc = constr(i)%coef(1)
              xpar(nkref) = atom(katom)%xc(kpar) / coefc

            case (3)  ! constrain sulle occ, npar-1 par. indipendenti
              npar = constr(i)%npar
              do iv=1,npar-1
                 nkref = nkref + 1
                 write(par_string(nkref),'(a,i3,a,i3)')' co.',i,' par.',iv
                 katom = constr(i)%vat(iv)
                 xpar(nkref) = atom(katom)%och
              enddo
              
          end select
          constr(i)%ref = nkref
      endif
   enddo
!
   end subroutine prerefine_structure

!corr!--------------------------------------------------------------------
!corr
!corr   subroutine setstagestru_new(rfc,atom,legm,cell,spg,elem,constr,sstage)
!corr   USE elements
!corr   USE unit_cell
!corr   USE atom_type_util
!corr   USE setref
!corr   USE spginfom
!corr   USE connect_mod
!corr   USE constraints
!corr   USE arrayutil
!corr!
!corr   implicit none
!corr   type(refine_condition_type), intent(in)                         :: rfc
!corr   type(atom_type), dimension(:), allocatable, intent(inout)       :: atom
!corr   type(bond_type), dimension(:), allocatable, intent(in)          :: legm
!corr   type(cell_type), intent(inout)                                  :: cell
!corr   type(spaceg_type), intent(in)                                   :: spg
!corr   type(element_type), dimension(:), allocatable, intent(in)       :: elem
!corr   type(constraint_type), dimension(:), allocatable, intent(inout) :: constr
!corr   type(refinement_stage_type), intent(in)                         :: sstage
!corr!
!corr   call calcola_occ(atom,spg,cell)   ! coordinates are modified
!corr   call azzeraref()
!corr   call make_constraints(atom,legm,constr,spg,cell,rfc%riding_model,rfc%adp_factor)
!corr   if (size_array(sstage%vet) > 0) then
!corr       call set_rcode_atoms(atom,cell,spg,constr,sstage%spar,1,zspec=sstage%vet)
!corr   else
!corr       call set_rcode_atoms(atom,cell,spg,constr,sstage%spar,1)
!corr   endif
!corr!
!corr   end subroutine setstagestru_new
!corr
!--------------------------------------------------------------------

   subroutine setstage(stage,refop,rfc)
   USE strutil
   USE setref
   USE RefinecomRef
   USE spginfom
   USE unit_cell
   USE profile_function
   USE arrayutil
   USE atom_type_util
   USE reflection_type_util
   USE elements
   USE connect_mod
   USE constraints
   USE variables, only: cryst,dataset
   USE crystal_phase
   implicit none
   type(refinement_stage_type), intent(in)    :: stage
   logical, dimension(:), intent(in)          :: refop
   type(refine_condition_type), intent(in)    :: rfc
   character(len=:), allocatable              :: sts  
   character(len=10), dimension(100)          :: wordv
   integer                                    :: nword,i,k,nph,np
   type(gref_type), allocatable, dimension(:) :: gref
   integer, dimension(:), allocatable         :: vetph
!   
   nph = 1
   if (rfc%raction == PAWLEY_DECOMP) then
       call make_ref_groups(cryst(nph)%ref,gref,sogi=0.6)
   endif

   sts = s_u2b(stage%spar)
   if (allocated(stage%ph)) then
       allocate(vetph(size(stage%ph)),source=stage%ph)
   else
       allocate(vetph(numphase(cryst)),source=[(nph,nph=1,numphase(cryst))])
   endif
   call get_words(sts,wordv,nword)
   do i=1,nword
      wordv(i) = lower(wordv(i))
      select case (wordv(i))
        case ('zero')
           dataset(1)%zcode = 1

        case ('sd')
           dataset(1)%sdcode = 1

        case ('st')
           dataset(1)%stcode = 1

        case ('scale') 
           if (allocated(stage%ph)) then
               cryst(stage%ph)%scode = 1
           else
               cryst(:)%scode = 1
           endif

        case ('back')
!corr           coefbkg%rcod = 1
           dataset(1)%bcode = 1

        case ('cell')
           if (allocated(stage%ph)) then
               do nph=1,size(stage%ph)
                  call cryst(stage%ph(nph))%set_refine_cell(1)
               enddo
           else
               do nph=1,numphase(cryst)
                  call cryst(nph)%set_refine_cell(1)
               enddo
           endif

        case ('u','v','w','asym')
           do np=1,size(vetph)
              nph=vetph(np)
!corr              select case(rfcond%typefun)
              select case (dataset(1)%typefun(nph))
                case(PEARSON7)
                   call set_rcode_function(dataset(1)%pear(nph),sref=wordv(i),code=1)
                case(PVOIG)
                   call set_rcode_function(psvoig=dataset(1)%pvoi(nph),sref=wordv(i),code=1)
                case(TCHZ)
                   call set_rcode_function(tchh=dataset(1)%tch(nph),sref=wordv(i),code=1)
              end select
           enddo
!!!!!FIX THIS loop for different profile function for each phase
        case ('beta0','beta1','beta2')
           do np=1,size(vetph)
              nph=vetph(np)
              call set_rcode_function(dataset(1)%pear(nph),sref=wordv(i),code=1)
           enddo

        case ('eta0','eta1','eta2')
           do np=1,size(vetph)
              nph=vetph(np)
              call set_rcode_function(psvoig=dataset(1)%pvoi(nph),sref=wordv(i),code=1)
           enddo

        case ('xp','yp','zp')
           do np=1,size(vetph)
              nph=vetph(np)
              call set_rcode_function(tchh=dataset(1)%tch(nph),sref=wordv(i)(1:1),code=1)
           enddo

        case ('all')
           do np=1,size(vetph)
              nph=vetph(np)
!corr              select case(rfcond%typefun)
              select case (dataset(1)%typefun(nph))
                case(PEARSON7)
                   call set_rcode_function(dataset(1)%pear(nph),sref=wordv(i),code=1)
                case(PVOIG)
                   call set_rcode_function(psvoig=dataset(1)%pvoi(nph),sref=wordv(i),code=1)
                case(TCHZ)
                   call set_rcode_function(tchh=dataset(1)%tch(nph),sref=wordv(i),code=1)
                   call set_rcode_function(tchh=dataset(1)%tch(nph),sref='z',code=0)
              end select
           enddo

        case ('xyz','xyzb','b')
           call azzeraref()
           do np=1,size(vetph)
              nph = vetph(np)
              call calcola_occ(cryst(nph)%at,cryst(nph)%spg,cryst(nph)%cell)   ! coordinates are modified
              call make_constraints(cryst(nph)%at,cryst(nph)%bond,cryst(nph)%constr,cryst(nph)%spg,    &
                                    cryst(nph)%cell,cryst(nph)%riding_model,cryst(nph)%adp_factor)
              if (size_array(stage%vet) > 0) then
                  call set_rcode_atoms(cryst(nph)%at,cryst(nph)%cell,cryst(nph)%spg,cryst(nph)%constr,wordv(i),1,zspec=stage%vet)
              else
                  call set_rcode_atoms(cryst(nph)%at,cryst(nph)%cell,cryst(nph)%spg,cryst(nph)%constr,wordv(i),1)
              endif
           enddo

        case ('allp')
           !if (allocated(stage%ph)) then
           !    allocate(vetph(size(stage%ph)),source=stage%ph)
           !else
           !    allocate(vetph(numphase(cryst)),source=[(nph,nph=1,numphase(cryst))])
           !endif
           do np=1,size(vetph)
              nph = vetph(np)
              call refine_all(refop(nph),nph)
           enddo

        case ('fo1')
           fopar(:)%rcod = 1
        
        case ('fo2')
           do k=1,size(fopar)
              if (fopar(k)%val > 0.000) then
                  fopar(k)%rcod = 1
              else
                  write(6,*)'riflesso non affinato:',k
              endif
           enddo

        case ('fo3')
           where(fopar(:)%val <= 0.000) fopar(:)%rcod = 1
  
        case ('fo4')
           do k=1,size(gref)
              if (gref(k)%code == 1) then
                  fopar(gref(k)%vref(1))%rcod = 1
              endif
           enddo
  
        case ('fo5')
           do k=1,size(gref)
              if (gref(k)%code >= 2) then
                  fopar(gref(k)%vref(:))%rcod = 1
              endif
           enddo
  
        case ('fo6')
           do k=1,size(gref)
              if (gref(k)%code >= 2 .and. gref(k)%code <=4) then
                  !do j=1,gref(k)%code
                  !   if (fopar(gref(k)%vref(j))%val > 0.00)fopar(gref(k)%vref(j))%rcod = 1
                  !enddo
                  fopar(gref(k)%vref(:))%rcod = 1
              endif
           enddo
  
        case ('fo7')
           do k=1,size(gref)
              if (gref(k)%code >= 5) then
                  fopar(gref(k)%vref(:))%rcod = 1
              endif
           enddo

      end select
   enddo
   do nph=1,numphase(cryst)
      if (refop(nph)) cryst(nph)%gcode = 1
   enddo
   
   end subroutine setstage


!corr!----------------------------------------------------------------------
!corr
!corr   function sample_displacement(tt,dis,geom) result(corr)
!corr!
!corr!  2theta = -2s*cos(theta)*(180/pi)/R = dis*cos(theta)*(180/pi)  [in degree]
!corr!
!corr   USE trig_constants
!corr   real, dimension(:), intent(inout) :: tt   ! 2theta in degree
!corr   real, intent(in)                  :: dis  ! refined parameter = -2s/R
!corr   integer, intent(in)               :: geom ! geometry
!corr   real, dimension(size(tt))         :: corr
!corr!
!corr   select case (geom)
!corr     case (BB_GEOMETRY)
!corr       corr = dis * cos(dtor*tt/2) * rtod
!corr
!corr     case (DS_GEOMETRY)    !!!TODO
!corr   end select
!corr!
!corr   end function sample_displacement
!corr
!corr!-----------------------------------------------------------------
!corr
!corr   function sample_transparency(tt,trasp,geom) result(corr)
!corr!
!corr!  2theta = (1/2*mu*R)*sin(2theta)*(180/pi) = trasp*sin(2theta)*(180/pi)  [in degree]
!corr!
!corr   USE trig_constants
!corr   real, dimension(:), intent(inout) :: tt    ! 2theta in degree
!corr   real, intent(in)                  :: trasp ! refined parameter = 1/2*mu*R
!corr   integer, intent(in)               :: geom  ! geometry
!corr   real, dimension(size(tt))         :: corr
!corr!
!corr   select case (geom)
!corr     case (BB_GEOMETRY)
!corr       corr = trasp * sin(dtor*tt) * rtod
!corr
!corr     case (DS_GEOMETRY)    !!!TODO
!corr   end select
!corr!
!corr   end function sample_transparency
!corr
!--------------------------------------------------------------

   subroutine calcola_profilo (Nactot,ierp)
!
   USE RefinecomRef
   USE calculate_pdp
   USE variables, only: dataset, cryst
   USE crystal_phase
   implicit none
   integer, intent(in)  :: Nactot
   integer, intent(out) :: ierp
   integer              :: ncall,nph
!
!!!!FIX THIS SUB for scale
   ierp = 0
!corr   ncall = gpo%rcod+rfcond%raction
   ncall = cryst(1)%gcode+rfcond%raction
!corr   if (zero%rcod > 0) ncall = ncall + 1
   if (dataset(1)%zcode > 0) ncall = ncall + 1
   if (dataset(1)%sdcode > 0) ncall = ncall + 1
   if (dataset(1)%stcode > 0) ncall = ncall + 1
   if (rinfo%refine_profile) ncall = ncall + 1
   do nph=1,numphase(cryst)
      if (cryst(nph)%refine_cell() .or. cryst(nph)%is_refined) ncall = ncall + 1 
   enddo
!corr   if (ncall > 0) call calculate_powder_pattern(rfcond,dataset,cryst,dataset(1)%pear(1),dataset(1)%pvoi(1),dataset(1)%tch(1),ierp)
   if (ncall > 0) call calculate_powder_pattern(rfcond,dataset,cryst,ierp)
!
   call erreacc(Nactot)
!
   end subroutine calcola_profilo

! ----------------------------------------------------------------------

   subroutine print_all_parameters(kpr,refcond,ref)
   USE RefinecomRef
   USE GENLSQ
   USE RRESTR
   USE variables, only:cryst,dataset
   USE compare_struct
   USE General, only:lo   !!!!!,cell
   USE STRUTIL
   USE constraints
   USE unit_cell
   USE profile_function
   USE reflection_type_util
   USE crystal_phase
   USE pearsonf
   USE psvoigtf
   USE tchzf
!
   implicit none
   integer, intent(in)         :: kpr
   type(refine_condition_type) :: refcond
   type(reflection_type), dimension(:), allocatable, intent(in) :: ref
   integer                     :: i,nph,npar
   integer                     :: back_code
   real                        :: shiftp
   integer                     :: sigsh,ipar1,ipar2
   character(len=*), dimension(6), parameter :: scell= ['a    ','b    ','c    ','alpha','beta ','gamma']
   logical :: is_std = .false.
   type(profile_function_t) :: prof
!
   sigsh = 5
!
!  Stampa intestazione
   write(kpr,'(2x,76("="))')
   if (refcond%auto_prof .or. refcond%auto_stru) then
      write(kpr,'(16x,a10,i4,a24,i4,a1,i0/)') '  Cycle # ',ncytot,        &
      '       Step refinement: ',kstager,'/',kstagemax
   else
      write(kpr,'(26x,a10,i4/)') '  Cycle # ',ncytot
   endif
!
   write(kpr,'(9x,a,i0)')'Number of refined parameters:',rinfo%npar
!
!  Stampa titolo colonne
!corr   back_code = count(coefbkg(:)%rcod > 0) 
   back_code = count(dataset(1)%bcode > 0)
   if (back_code+dataset(1)%zcode+dataset(1)%sdcode+dataset(1)%stcode+cryst(1)%scode+cryst(1)%gcode > 0 .or. rinfo%refine_profile &
       .or. cryst(1)%refine_cell()) then
      write(kpr,'(a)')'                    Parameter            Shift'
   endif
!
!  Background
   if (back_code > 0) then
       do i=1,dataset(1)%cond%ncoef
!corr          call print_parameter(coefbkg(i)%val,coefbkg(i)%sd,coefbkg(i)%rcod,coefbkg(i)%str,sigsh,is_std,kpr)
          call print_parameter(dataset(1)%coef(i),dataset(1)%bsd(i),dataset(1)%bcode(i),'b'//i_to_s(i),sigsh,is_std,kpr)
       enddo
       write(kpr,*)
   endif
!
!  Zero
!corr   call print_parameter(zero,sigsh,kpr,separ=.true.)
   call print_parameter(dataset(1)%zero,dataset(1)%zsd,dataset(1)%zcode,'zero',sigsh,is_std,kpr,separ=.true.)
!
!  Sample displacement and transparency
   !call print_parameter(sdisp,sigsh,kpr,separ=.true.)
   !call print_parameter(stran,sigsh,kpr,separ=.true.)
   call print_parameter(dataset(1)%sdisp,dataset(1)%sdsd,dataset(1)%sdcode,'sample di.',sigsh,is_std,kpr,separ=.true.)
   call print_parameter(dataset(1)%stran,dataset(1)%stsd,dataset(1)%stcode,'sample tr.',sigsh,is_std,kpr,separ=.true.)
!
!  Informazioni sul profilo
   do nph=1,numphase(cryst)
      select case (dataset(1)%typefun(nph))

        case (PEARSON7)
          ipar1=P_UPAR
          ipar2=P_ASYM4PAR
          prof = dataset(1)%pear(nph)

        case (PVOIG)
          ipar1=PV_UPAR
          ipar2=PV_ASYM4PAR
          prof = dataset(1)%pvoi(nph)

        case (TCHZ)
          ipar1=T_UPAR
          ipar2=T_ASYM4PAR
          prof = dataset(1)%tch(nph)

      end select
      do npar=ipar1,ipar2
         call print_parameter(prof%par(npar)%val,prof%par(npar)%sd,prof%par(npar)%rcod,prof%par(npar)%str,sigsh,is_std,kpr)
      enddo

!corr   enddo
!corr   if (rinfo%refine_profile)write(kpr,*)
!corr   do nph=1,numphase(cryst)
!
!     Scale
      call print_parameter(cryst(nph)%scal,cryst(nph)%ssd,cryst(nph)%scode,'scale',sigsh,is_std,kpr)
!
!     Cella
      do i=1,3
         if (cryst(nph)%get_cellcod(i) > 0) then
             shiftp = shiftx(cryst(nph)%get_cellcod(i))  
             write(kpr,'(9x,a,t20,f0.7,t40,a)')trim(scell(i)),cryst(nph)%get_cellpar(i),trim(string_sigf(shiftp,sigsh))
         endif
      enddo
!
!     Orientazione preferenziale
      call print_parameter(cryst(nph)%gpar,cryst(nph)%gsd,cryst(nph)%gcode,'G',sigsh,is_std,kpr,separ=.true.)
!
      do i=4,6
         !if (cellph%get_cod(i) > 0) then
         if (cryst(nph)%get_cellcod(i) > 0) then
             !shiftp = shiftx(cellph%get_cod(i))  
             shiftp = shiftx(cryst(nph)%get_cellcod(i))  
             !write(kpr,'(9x,a,t20,f0.4,t40,a)')trim(cellr(i)%str),cell(i),trim(string_sigf(shiftp,sigsh))
             write(kpr,'(9x,a,t20,f0.4,t40,a)')trim(scell(i)),cryst(nph)%get_cellpar(i),trim(string_sigf(shiftp,sigsh))
         endif
      enddo
      if (cryst(nph)%refine_cell())write(kpr,*)
!
!     Struttura
      if (cryst(nph)%is_refined) then
          call print_structure(cryst(nph)%at,cryst(1)%constr,kpr)
          write(kpr,*)
      endif
!
!     Constraints su x,y,z
      call print_constraints(cryst(nph)%constr,cryst(1)%at,lo,.false.)
   enddo
!
   if (refcond%algo == GAUNEW) then
       write(kpr,'(3x,a,a,t42,a,f0.5)') 'Chisquared = ',trim(string_sig(chisquarednow,10)),'Step lenght = ',step_length
   else
       write(kpr, '(3x,a,a)')  'Chisquared =',string_sig(chisquarednow,10)
   endif
!corr   if (nrestraints(resvr) > 0 .and. rinfo%refine_structure) call print_restraints_all(kpr,resvr,atm,set_cell_type(cell))
!corr   if (nrestraints(resvr) > 0 .and. rinfo%refine_structure) call print_restraints_all(kpr,resvr,atm,cryst(1)%cell)
!corr   if (cryst(1)%numres() > 0 .and. rinfo%refine_structure) call print_restraints_all(kpr,cryst(1)%res,cryst(1)%at,cryst(1)%cell)
   if (cryst(1)%numres() > 0 .and. cryst(1)%is_refined) call print_restraints_all(kpr,cryst(1)%res,cryst(1)%at,cryst(1)%cell)
   call print_rvalues(ref,kpr,title=.false.)
!corr   if (rinfo%refine_structure .and. cryst(1)%is_known()) then
   if (cryst(1)%is_refined .and. cryst(1)%is_known()) then
       call to_coord_sir(0,cryst(1)%at,ref,size(ref),cryst(1)%cell,cryst(1)%spg,lo,cryst(1)%atpub)
   endif
!
   end subroutine print_all_parameters

!-------------------------------------------------------------------------------

   subroutine printfin_esd(kpr,Nact,atom,ref,title,sdpar)
   USE RefinecomRef
   USE GENLSQ
   USE type_constants, only: DP
   USE RRESTR
   USE variables, only:cryst,dataset
   USE compare_struct
   USE STRUTIL
   USE background
   USE atom_type_util
   USE unit_cell
   USE profile_function
   USE constraints
   USE unit_cell
   USE reflection_type_util
   USE pearsonf
   USE psvoigtf
   USE tchzf
   USE crystal_phase
   USE cryutil
   USE quantmod
!
   implicit none
   integer, intent(in)                    :: Nact
   integer, intent(in)                    :: kpr
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(reflection_type), dimension(:), allocatable, intent(in) :: ref
   character(len=*), intent(in), optional :: title
   integer                                :: i,j,npar
   integer                                :: sigsh
   real, dimension(Nact)                  :: stdpar
!corr   real                                   :: stdval
!corr   integer                                :: ier
   integer, parameter                     :: lenp = 15 
   !character(len=lenp), dimension(5)      :: stringp
   character(len=12) :: snlab
   integer                                :: kconstr
   integer                                :: codepar
   real(DP), dimension(Nact), intent(in) :: sdpar
   logical :: is_std=.true.
   integer :: nph,ipar1,ipar2
   type(profile_function_t) :: prof
!
   write(kpr,'(/1x,80("="))')
   if (present(title)) then
       write(kpr,'(a)') centra_str(title,80)
   endif
!
   call print_rvalues(ref,kpr,title=.true.)
   select case (rfcond%typewei)
     case (1)
       write(kpr,'(/,1x,a)') 'Weighting scheme: w=1.0'
     case (2)
       write(kpr,'(/,1x,a)') 'Weighting scheme: w=1.0/ycount'
     case (3)
       write(kpr,'(/,1x,a)') 'Weighting scheme: w=1.0/ycount^2'
   end select
   write(kpr,'(1x,a,f0.6,/)') 'Convergence criterium: eps=',rfcond%eps
!
   sigsh = 5
!
!  Calcola le std
   !call solve_linlsq(jacob,-funcls,shiftx,ier,grad,hess,.true.,matinv)
   !esdd = chisquarednow/(refcond%lim%ncu+nrestraints(resvr)-Nact)
   !do i=1,Nact
   !   stdpar(i) = sqrt(matinv(i,i)*esdd)
   !enddo
     !call calculate_sd(jacob,funcls,chisquarednow,Nact,sdpar,ier)
     !call set_sd(sdpar)
!corr        write(0,*)'cell esd:',cellr%sd
   stdpar(:) = real(sdpar(:))
   
      !write(0,'(a,*(f10.4))')'SD2=',sdpar
!corr!
!corr!  Stampa informazioni sul background
!corr   select case (backriet%btype)
!corr      case (POLY)
!corr        write(kpr,'(/1x,a,i0)')'Background modelled by a polynomial of order ',backriet%ncoef-1
!corr
!corr      case (CHEBY)
!corr        write(kpr,'(/1x,a,i0)')'Background modelled by a Chebyshev polynomial of degree ',backriet%ncoef
!corr
!corr      case (FOUR_SERIES)
!corr        write(kpr,'(/1x,a,i0)')'Background modelled by a cosine Fourier series of degree ',backriet%ncoef
!corr
!corr   end select
!corr   if (backriet%btype==POLY) write(kpr,'(9x,a,t20,a)')'Origin',trim(string_sigf(dataset(1)%thzerob,7))
!corr!
!corr!  Background
!corr   do i=1,backriet%ncoef
!corr      call print_parameter(dataset(1)%coef(i),dataset(1)%bsd(i),dataset(1)%bcode(i),'b'//i_to_s(i),sigsh,is_std,kpr)
!corr   enddo
   call print_background(kpr,dataset(1)%cond,dataset(1)%coef,dataset(1)%thzerob,dataset(1)%bsd,dataset(1)%bcode)
   write(kpr,*)
!
!  Zero
   call print_parameter(dataset(1)%zero,dataset(1)%zsd,dataset(1)%zcode,'zero',sigsh,is_std,kpr,separ=.true.)
!
!  Sample displacement and transparency
   call print_parameter(dataset(1)%sdisp,dataset(1)%sdsd,dataset(1)%sdcode,'sample di.',sigsh,is_std,kpr,separ=.true.)
   call print_parameter(dataset(1)%stran,dataset(1)%stsd,dataset(1)%stcode,'sample tr.',sigsh,is_std,kpr,separ=.true.)
!
!  Scale
!corr   call print_parameter(scalr,sigsh,kpr,printsd)
   call print_parameter(cryst(1)%scal,cryst(1)%ssd,cryst(1)%scode,'scale',sigsh,is_std,kpr)
!
!  Cella
   write(kpr,'(/1x,a)')'Lattice parameters'
   !call cellph%write(kpr)
   call cryst(1)%print(kpr,'C')
!
!  Informazioni sul profilo
   do nph=1,numphase(cryst)
      select case (dataset(1)%typefun(nph))

        case (PEARSON7)
          write(kpr,'(/1x,a)') 'Profile shape function: Pearson VII'
          ipar1=P_UPAR
          ipar2=P_ASYM4PAR
          prof = dataset(1)%pear(nph)

        case (PVOIG)
          write(kpr,'(/1x,a)') 'Profile shape function: Pseudo-Voigt '
          ipar1=PV_UPAR
          ipar2=PV_ASYM4PAR
          prof = dataset(1)%pvoi(nph)

        case (TCHZ)
          write(kpr,'(/1x,a)') 'Profile shape function: TCH Pseudo-Voigt '
          ipar1=T_UPAR
          ipar2=T_ASYM4PAR
          prof = dataset(1)%tch(nph)

      end select

      do npar=ipar1,ipar2
         call print_parameter(prof%par(npar)%val,prof%par(npar)%sd,prof%par(npar)%rcod,prof%par(npar)%str,sigsh,is_std,kpr)
      enddo

!
      if (cryst(nph)%is_refined) then
          write(kpr,'(a)')'                      x           y            z           B        occ'
          do i=1,numatoms(atom)
             snlab = trim(atom(i)%lab)
             write(kpr,'(3x,a)',advance='no')snlab
             do j=1,3

                kconstr = par_in_constraint(cryst(nph)%constr,i,j)
                if (kconstr > 0) then            
                    codepar = cryst(nph)%constr(kconstr)%ref
                    !if (codepar > 0) stdval = stdpar(codepar)*abs(coef_of_constraint(cryst(nph)%constr(kconstr),i,j))
                else
                    codepar = atom(i)%rcod(j)
                    !if (codepar > 0) stdval = stdpar(codepar)
                endif
                if (codepar > 0) then
                    !stringp(j) = trim(string_esd(atom(i)%xc(j),stdval))
                    write(kpr,'(a12)',advance='no')string_esd(atom(i)%xc(j),atom(i)%xsd(j))
                else
                    write(kpr,10,advance='no')atom(i)%xc(j)
                    !write(stringp(j),'(f0.6)',iostat=ier)atom(i)%xc(j)
                endif
             enddo
             !stringp(4) = ' '
             kconstr = par_in_constraint(cryst(nph)%constr,i,4)
             if (kconstr > 0) then            
                 codepar = cryst(nph)%constr(kconstr)%ref
                 !if (codepar > 0) stdval = stdpar(codepar)*abs(coef_of_constraint(cryst(nph)%constr(kconstr),i,4))
             else
                 codepar = atom(i)%rcod(4)
                 !if (codepar > 0) stdval = stdpar(codepar)
             endif
             if (codepar > 0) then
                 !stringp(4) = trim(string_esd(atom(i)%biso,stdval))
                 write(kpr,'(a12)',advance='no')string_esd(atom(i)%biso,atom(i)%bsd)
             else
                 !write(stringp(4),'(f0.6)',iostat=ier)atom(i)%biso
                 write(kpr,10,advance='no')atom(i)%biso
             endif
             kconstr = par_in_constraint(cryst(nph)%constr,i,5)
             if (kconstr > 0) then            
                 codepar = cryst(nph)%constr(kconstr)%ref
             else
                 codepar = atom(i)%rcod(4)
             endif
             if (codepar > 0) then
                 write(kpr,'(a12)',advance='yes')string_esd(atom(i)%och,atom(i)%osd)
             else
                 write(kpr,10,advance='yes')atom(i)%och
             endif
          enddo
10    format(f12.4)
      else
          call print_atoms(atom(:),title='Structure')
      endif
      call print_restraints_all(kpr,cryst(nph)%res,atom,cryst(nph)%cell)
!
      write(kpr,'(2x,80("="))')
!
!     Orientazione preferenziale
      if (cryst(nph)%kopcorr > 0) then
          write(kpr,'(/,a,3i3/)') '  Preferred orientation on plane: ',cryst(nph)%hklpo(:)
          write(kpr,'(9x,a,t20,a)')'G',trim(string_sigf(cryst(nph)%gpar,7))
      endif
   enddo
!
   call quantitative(cryst)
   call quantitative_print(cryst,kpr)
!
   end subroutine printfin_esd

!----------------------------------------------------------------------------------

   subroutine stampa_risultati (kpr)
   USE RefinecomRef
   USE genlsq
           USE spginfom
           USE cif_frm
           USE sinterface
           USE prognames, only: package_alt_name
           USE variables, only:cryst,dataset
   USE unit_cell
   USE type_constants, only: DP
   USE reflection_type_util
   USE crystal_phase
   implicit none
   integer, intent(in) :: kpr
   logical                         :: stdprint,fprint   !,cprint,mprint
   real(DP), dimension(rinfo%npar) :: sdpar
   integer                         :: ier
!
   stdprint = index(rfcond%sprint,'E') > 0
   fprint = index(rfcond%sprint,'S') > 0
   if (stdprint .or. fprint) then
       if (stdprint) then
           ier = 0
           if (.not.sdavail) call compute_sd(ier)
           !call calculate_sd(jacob,funcls,rinfo%npar,sdpar,ier)
           if (ier == 0) then
               !call set_sdval(sdpar)
               call printfin_esd(kpr,rinfo%npar,cryst(1)%at,cryst(1)%ref,'Final Rietveld parameters and stdv',sdpar)
!
!              export cif file with std
               call create_ciffile('riet.cif',cell=cryst(1)%cell,atom=cryst(1)%at,progname=package_alt_name,    &
               spg=init_spaceg_type(spaceg_string()),elem=cryst(1)%elem,legm=cryst(1)%bond,                     &
               std=.true.,symm=.true.,datas=dataset(1))
           endif
       else
           if (numphase(cryst) > 0) then   ! sd for scale is required
               if (.not.sdavail) call compute_sd(ier)
           endif
           call printfin(kpr,'Final Rietveld parameters')
       endif
   endif
   if (index(rfcond%sprint,'C') > 0) call stampa_correlation(kpr)
   if (index(rfcond%sprint,'M') > 0) call stampa_matrici_lsq(kpr)
!
   end subroutine stampa_risultati


!-----------------------------------------------------------------------

   subroutine init_fobs(ref)
   USE Refinecomref
   USE Counts
   USE arrayutil
   USE reflection_type_util
   USE math_util
!corr   USE conteggi, only: npunti
   USE variables, only: dataset
   implicit none
   type(reflection_type), dimension(:), intent(inout) :: ref
   integer                                            :: i,j
   integer                                            :: pini,pfin
   real                                               :: yarea
   integer                                            :: npp
   real                                               :: deltath
   real, dimension(:), allocatable                    :: ycb 
   real, dimension(size(ref))                         :: xref
   type(gref_type), dimension(:), allocatable         :: gref
   integer                                            :: nref1,nref2,ngref,nrefmin,nrefmax
   integer, dimension(1)                              :: cloc
   real                                               :: tmed,ycv,const
   integer                                            :: nrefu
!
   deltath = 0.25
!
   nrefu = size(ref)  !cryst(nph)%numref()
   call make_ref_groups(ref(:nrefu),gref)   ! estrai gruppi
   allocate(ycb(dataset(1)%npoints()))
!
   do i=1,size(gref)
!
!     Individua i conteggi pini e pfin che delimitano il gruppo
      ngref = gref(i)%code
      nref1 = gref(i)%vref(1)
      nref2 = gref(i)%vref(ngref)
      cloc(:) = minloc(ref(nref1:nref2)%tthd(1))
      nrefmin = cloc(1) + nref1 - 1   ! primo riflesso del gruppo
      cloc(:) = maxloc(ref(nref1:nref2)%tthd(1))
      nrefmax = cloc(1) + nref1 - 1   ! ultimo riflesso del gruppo
      pini = clocate(dataset(1)%x,ref(nrefmin)%tthd(1)-deltath)
      pfin = clocate(dataset(1)%x,ref(nrefmax)%tthd(1)+deltath)
      !write(6,*)'gruppo n.',i,'ngr=',ngref,nref1,nref2
      !write(6,*)'min=',ttetaczero(pini)
      !write(6,*)'max=',ttetaczero(pfin)
!
!     Calcola integrale dell'area  tra pini e fin
      npp = pfin-pini+1
      if (pfin <= ubound(dataset(1)%yb,dim=1)) then
          ycb(:npp) = dataset(1)%y(pini:pfin) - dataset(1)%yb(pini:pfin)
      else
          ycb(:npp) = dataset(1)%y(pini:pfin)
      endif
!!      if (npp > 1) then
!!          xcb(:npp) = ttetacount(pini:pfin)
!!          yarea = 0.5*SUM((xcb(2:npp)-xcb(1:npp-1))*(ycb(2:npp) + ycb(1:npp-1))) 
!!      else
!!          yarea =  ycb(1)
!!      endif
!!        write(0,*)'AREA=',yarea,integrate(ttetacount(pini:pfin),ycb,1,npp)
      yarea = integrate(dataset(1)%x(pini:pfin),ycb,1,npp)
!
      if (yarea < 0) then
          yarea = 0.01
          ref(nref1:nref2)%fo = sqrt(yarea/(ref(nref1:nref2)%m*ref(nref1:nref2)%lp(1)))
      else
          if (ngref > 1) then
!
!             Ripartisci intensita' in basa alla distanza dal massimo
              cloc(:) = maxloc(ycb(:npp)) + pini - 1
              tmed = dataset(1)%x(cloc(1))
              xref(nref1:nref2) = 1.0/(0.001 + abs(ref(nref1:nref2)%tthd(1) - tmed))
              const = yarea/sum(xref(nref1:nref2))
              !write(6,*)'max at =',tmed
              do j=nref1,nref2
                 ycv = const*xref(j)
                 !write(6,*)'int=',j,ycv,yarea,ref(j)%tthd
                 ref(j)%fo = sqrt(ycv/(ref(j)%m*ref(j)%lp(1)))
                 !write(6,*)'INIT=',j,ref(j)%fo,ycv,ref(j)%m*ref(j)%lp
              enddo
          else
              ref(nref1)%fo = sqrt(yarea/(ref(nref1)%m*ref(nref1)%lp(1)))
          endif
      endif
   enddo
        !write(0,*)'SCAL=',cryst(1)%scal
      !ref%fo = ref%fo / cryst(1)%scal
           !ref%fo = 1000*ref%fo/maxval(ref%fo)
!
   end subroutine init_fobs

! ----------------------------------------------------------------------

   subroutine Count0()
!
!  Compute profile and a starting value for scale
!
   USE RefinecomRef, only: rfcond
   USE calculate_pdp
   USE variables, only: dataset,cryst
   USE crystal_phase
   USE profile_function
   USE arrayutil
   use genlsq
   use type_constants
!
   implicit none
   integer                               :: ierp,nref
   integer                               :: nph,nc,ier
   real(dp), dimension(:), allocatable   :: bm
   real(dp), dimension(:,:), allocatable :: am,ainv
   real(dp), dimension(:), allocatable   :: scal
   real(dp)                              :: esdd
!
!  Compute profile
   call pd_phase_init(numphase(cryst))   
   do nph=1,numphase(cryst)
      call pd_ref_init(nph,cryst(nph)%numref(),dataset(1)%nwave)
      call new_array(cryst(nph)%yc,dataset(1)%npointsc())
!oldref      nref = count(cryst(nph)%ref%tthd(1) <= dataset(1)%xmaxc())
      nref = cryst(nph)%numref()
       !write(0,*)'NREF=',nref,cryst(1)%numref(),dataset(1)%xmaxc(),cryst(1)%ref(cryst(1)%numref())%tthd(1),cryst(1)%ref(nref)%tthd(1)

      call calculate_powder_pattern(rfcond,dataset(1)%nc2,dataset(1)%x0,cryst(nph)%yc,cryst(nph)%ref,nref,dataset(1)%nwave,   &
      dataset(1)%ratio,dataset(1)%pear(nph),dataset(1)%pvoi(nph),dataset(1)%tch(nph),dataset(1)%typefun(nph),nph,ierp)
   enddo
!
!  Compute scale factor from lsq
   nc = dataset(1)%nc2
   allocate(am(nc,numphase(cryst)),bm(nc))
   allocate(scal(numphase(cryst)),ainv(numphase(cryst),numphase(cryst)))
   do nph=1,numphase(cryst)
      am(:,nph) = sqrt(weilsq)*cryst(nph)%yc
   enddo
   bm(:) = sqrt(weilsq)*(dataset(1)%y(:nc) - dataset(1)%yb(:nc))
   call solve_linlsq(am,bm,scal,ier,linv=.true.,ainv=ainv)
   if (ier == 0) then
       cryst(:)%scal = scal
   else
       cryst(:)%scal = 1.0
   endif
   dataset(1)%yc = cryst(1)%scal*cryst(1)%yc
   do nph=2,numphase(cryst)
      dataset(1)%yc = dataset(1)%yc + cryst(nph)%scal*cryst(nph)%yc
   enddo
   call erreacc(0)
!
!  Compute sd on scale factors
   if (ier == 0) then
       esdd = sum(funcls(:nc)**2) / (nc-numphase(cryst))
       do nph=1,numphase(cryst)
          cryst(nph)%ssd = sqrt(ainv(nph,nph)*esdd)
       enddo
   endif
!
   end subroutine Count0

 !-----------------------------------------------------------------------

   subroutine Count2()
   USE RefinecomRef, only:rfcond
   USE Counts
   USE background
   USE calculate_pdp
   !USE reflection_type_util
   USE variables, only: dataset,cryst
!
   implicit none
   !type(reflection_type), dimension(:), allocatable, intent(inout) :: ref
   integer :: posini,posfin
   integer :: ierp    !,nph,nref
!
   posini = 1
!corr   posfin = rfcond%lim%ncu  
   posfin = dataset(1)%nc2
   !nph = 1
   !nref = count(cryst(nph)%ref%tthd(1) <= rfcond%lim%ttmax)
   call getbackground(dataset(1)%cond%btype,dataset(1)%yb(:posfin-posini+1),      &
                      dataset(1)%x0(posini:posfin),dataset(1)%coef(:dataset(1)%cond%ncoef),dataset(1)%thzerob)
   call calculate_powder_pattern(rfcond,dataset,cryst,ierp)
   !call calculate_powder_pattern(rfcond,dataset(1)%x0,dataset(1)%yc,ref,nref,dataset(1)%nwave,dataset(1)%ratio,   &
   !                              dataset(1)%pear(1),dataset(1)%pvoi(1),dataset(1)%tch(1),dataset(1)%typefun(nph),1,ierp)
!
   call erreacc(0)
!
   end subroutine Count2

!--------------------------------------------------------------

   integer function count_parameters(refcond) result(Nactot)
   USE RefinecomRef
   USE variables, only:cryst,dataset
   USE RRESTR
   USE atom_type_util
   USE constraints
   USE profile_function
   USE crystal_phase
   USE calculate_pdp
   use absmod
!
   implicit none
   type(refine_condition_type), intent(in) :: refcond
   integer                                 :: Ncell,Nprof,Nstru
   integer                                 :: i,nph
   integer                                 :: nconstra
!
   Nactot = 0
!
!  Fo per Pawley
   if (refcond%raction == 2) then
       do i=1,size(fopar)
          if (fopar(i)%rcod > 0) then
              Nactot = Nactot + 1
          endif
       enddo
   endif
!
!  Profile parameters
   Nprof = num_pfunction_ref(dataset(1),numphase(cryst))
   rinfo%refine_profile = Nprof > 0
   do nph=1,numphase(cryst)
      if (cryst(nph)%scode > 0) Nprof = Nprof + 1
   enddo
!
   Ncell = 0
   Nstru = 0
   nconstra = 0
   do nph=1,numphase(cryst)
!
!     Cell parameters
      Ncell = Ncell + count(cryst(nph)%get_cellcod() > 0) 
!
!     Atomic parameters
      do i=1,cryst(nph)%natoms()
         Nstru = Nstru + count(cryst(nph)%at(i)%rcod > 0)
      enddo
      nconstra = nconstra + nconstraints_ref(cryst(nph)%constr)
      cryst(nph)%is_refined = Nstru + nconstra > 0
!
!     Corrections
      if (cryst(nph)%gcode > 0) Nactot = Nactot + 1
      if (cryst(nph)%absf == BB_SUOR .or. cryst(nph)%absf == BB_PITS) then
          if (cryst(nph)%abscode(1) > 0) Nactot = Nactot + 1
          if (cryst(nph)%abscode(2) > 0) Nactot = Nactot + 1
      endif
   enddo
!
   Nactot = Nactot + Ncell + Nprof + Nstru
!
   if (dataset(1)%zcode > 0) Nactot = Nactot + 1
   if (dataset(1)%sdcode > 0) Nactot = Nactot + 1
   if (dataset(1)%stcode > 0) Nactot = Nactot + 1
   Nactot = Nactot + count(dataset(1)%bcode(:dataset(1)%cond%ncoef) > 0) + nconstra
   rinfo%npar = Nactot
!
   end function count_parameters

!------------------------------------------------------------------------------------------------

   subroutine prerefine()
   USE RefinecomRef
   USE GENLSQ
   USE variables, only:cryst,dataset
   USE RRESTR
   USE profile_function
   USE crystal_phase
   USE pearsonf
   USE psvoigtf
   USE tchzf
   USE strutil, only: i_to_s
   use absmod
   implicit none
   integer                                   :: i,j,nkref,nph
   character(len=*), dimension(6), parameter :: scell= ['a    ','b    ','c    ','alpha','beta ','gamma']
   integer :: npar
!
!  Questa subroutine riempie i vettori krefp,par_string,typex
!
!  krefp(i)      = puntatore su irefmat
!  par_string(i) = stringa associata al parametro
!  typex(i)      = valore massimo
!  i = puntatore al parametro nella matrice di affinamento
!
   nkref = 0
   typex = 1.0
!
!  Affinamento fattori di struttura nel pawley
   if (rfcond%raction == 2) then
       do i=1,size(fopar)
          if (fopar(i)%rcod > 0) then
              nkref = nkref + 1
              fopar(i)%rcod = nkref
              !xpar(nkref) = ref(i)%fo
              xpar(nkref) = cryst(1)%ref(i)%fo**2
              par_string(nkref) = fopar(i)%str
          endif
       enddo
   endif
!
!  Zero
   if (dataset(1)%zcode > 0) then
       nkref = nkref + 1
       par_string(nkref) = 'zero'
       xpar(nkref) = dataset(1)%zero
       typex(nkref) = 0.001
       dataset(1)%zcode = nkref
   endif
!
!  Sample displacement
   if (dataset(1)%sdcode > 0) then
       nkref = nkref + 1
       par_string(nkref) = 'sample di.'
       xpar(nkref) = dataset(1)%sdisp
       typex(nkref) = 0.001
       dataset(1)%sdcode = nkref
   endif
!
!  Sample transparency
   if (dataset(1)%stcode > 0) then
       nkref = nkref + 1
       par_string(nkref) = 'sample tr.'
       xpar(nkref) = dataset(1)%stran
       typex(nkref) = 0.001
       dataset(1)%stcode = nkref
   endif
!
!  Background
   if (dataset(1)%bcode(1) > 0) then
       do i=1,dataset(1)%cond%ncoef
          nkref = nkref + 1
          par_string(nkref) = 'b'//i_to_s(i)
          xpar(nkref) = dataset(1)%coef(i)
          dataset(1)%bcode(i) = nkref
       enddo
   endif
!
!  Profilo
   do nph=1,numphase(cryst)
      select case (dataset(1)%typefun(nph))
        case (PEARSON7)
           do npar=P_UPAR,P_ASYM4PAR
              if (dataset(1)%pear(nph)%par(npar)%rcod > 0) then 
                  nkref = nkref + 1
                  par_string(nkref) = dataset(1)%pear(nph)%par(npar)%str
                  xpar(nkref) = dataset(1)%pear(nph)%par(npar)%val
                  dataset(1)%pear(nph)%par(npar)%rcod = nkref
                  if (npar == P_ASYM1PAR) typex(nkref) = 0.5
              endif
           enddo

        case (PVOIG)
           do npar=PV_UPAR,PV_ASYM4PAR
              if (dataset(1)%pvoi(nph)%par(npar)%rcod > 0) then 
                  nkref = nkref + 1
                  par_string(nkref) = dataset(1)%pvoi(nph)%par(npar)%str
                  xpar(nkref) = dataset(1)%pvoi(nph)%par(npar)%val
                  dataset(1)%pvoi(nph)%par(npar)%rcod = nkref
              endif
           enddo

        case (TCHZ)
           do npar=T_UPAR,T_ASYM4PAR
              if (dataset(1)%tch(nph)%par(npar)%rcod > 0) then 
                  nkref = nkref + 1
                  par_string(nkref) = dataset(1)%tch(nph)%par(npar)%str
                  xpar(nkref) = dataset(1)%tch(nph)%par(npar)%val
                  dataset(1)%tch(nph)%par(npar)%rcod = nkref
              endif
           enddo

      end select
   enddo
!
   do nph=1,numphase(cryst)
!
!     Scale
      if (cryst(nph)%scode > 0) then
          nkref = nkref + 1
          par_string(nkref) = 'scale'
          xpar(nkref) = cryst(nph)%scal
          cryst(nph)%scode = nkref
      endif
!
!     Cell
      if(cryst(nph)%refine_cell()) then
         do j=1,6
            if (cryst(nph)%get_cellcod(j) > 0) then
                nkref = nkref + 1
                par_string(nkref) = scell(j)
                xpar(nkref) = cryst(nph)%get_cellpar(j)
                call cryst(nph)%set_cellcod(j,nkref)
                call cryst(nph)%set_cellsd(j,0.)   ! reset sd for refined par.
            endif
         enddo
      endif
!
      if (cryst(nph)%is_refined) then
          call prerefine_structure(cryst(nph)%at,cryst(nph)%ref,cryst(nph)%constr,nkref)
      endif
!
!     Orientazione preferenziale
      if (cryst(nph)%gcode > 0) then
          nkref = nkref + 1
          par_string(nkref) = 'G'
          xpar(nkref) = cryst(nph)%gpar
          cryst(nph)%gcode = nkref
      endif
!
!     Absorption factor
      if (cryst(nph)%absf == BB_SUOR .or. cryst(nph)%absf == BB_PITS) then
          do j=1,2
             if (cryst(nph)%abscode(j) > 0) then
                 nkref = nkref + 1
                 par_string(nkref) = 'abs'//i_to_s(j)
                 cryst(nph)%abscode(j) = nkref
             endif
          enddo
      endif
   enddo
!
   end subroutine prerefine

!------------------------------------------------------------------------------------------------

   subroutine rietjacobian(ierp)
   USE Counts
   USE RefinecomRef
   USE nr
   USE GENLSQ
   USE background
   USE RRESTR
   USE progtype, only:deratom
   USE variables, only:cryst,dataset
   USE reflection_type_util
   USE pearsonf
   USE psvoigtf
   USE tchzf
   USE constraints
   USE unit_cell
   USE profile_function
   USE crystal_phase
   USE arrayutil
   use datasetmod
!
   implicit none

   type derivate_t
       type(deratom), dimension(:,:), allocatable :: derca
       real         , dimension(:,:), allocatable :: dercl
       real         , dimension(:,:), allocatable :: dfwhm
       real         , dimension(:,:), allocatable :: derpf
       real         , dimension(:),   allocatable :: derpG
   end type derivate_t

   integer, intent(out)                       :: ierp
   integer                                    :: nref
   integer                                    :: ncounts
   integer                                    :: i,j,k,kk,nph
   integer                                    :: kka
   integer                                    :: katom
   real                                       :: derzb
   real                                       :: thy,xche
   real                                       :: doc
   real, dimension(36)                        :: tt
   real                                       :: aa,bb
   type(deratom), dimension(:,:), allocatable :: dera
   real         , dimension(:),   allocatable :: dery2tcorr
   real         , dimension(:), allocatable   :: derfct
   real         , dimension(:), allocatable   :: derOP
   real         , dimension(:), allocatable   :: fstr
   real, dimension(:,:), allocatable          :: dycount
   real                                       :: difth
   integer                                    :: kp
   integer                                    :: kpar
   real                                       :: coef
   integer                                    :: np,kpind,npar     !!!!,natom
   type(derivate_t), dimension(:), allocatable :: derph
!
   ierp = 0
   ncounts = dataset(1)%nc2
!
   jacob(:,:) = 0
   allocate(derph(numphase(cryst)))
   do nph=1,numphase(cryst)
      nref = cryst(nph)%numref()     ! TOFIX with nrefu in cryst
      call new_array(derfct,nref)
      if (cryst(nph)%refine_cell() .or. cryst(nph)%is_refined) then
          if (rfcond%raction == 0) then
              if (allocated(dera)) deallocate(dera)
              allocate(dera(nref,cryst(nph)%natoms()),derph(nph)%derca(ncounts,cryst(nph)%natoms()))
              call fcalcadern(cryst(nph)%ref,cryst(nph)%at(:),cryst(nph)%spg,cryst(nph)%elem,dataset(1)%wave(1),   &
                              dera,derfct)
          else
              derfct = 0.0
          endif
          call setRInt(cryst(nph)%ref)
      endif
!
      call new_array(fstr,nref)
      if (rfcond%raction > 0) then
          fstr(:) = cryst(nph)%ref(:nref)%fo
          if (rfcond%raction == 2) then
              allocate(dycount(nref,ncounts))
              call decomponi_pearson(dataset(1)%pear(nph),cryst(nph)%ref(:nref),dataset(1)%x0(:ncounts),cryst(nph)%scal,   &
                                     dycount,dataset(1)%nwave,1.0,nph,ierp)
          endif
      else
          fstr(:) = cryst(nph)%ref(:nref)%fc
      endif
      if (cryst(nph)%kopcorr > 0) then   !!FIX THIS PART
          call new_array(derOP,nref)
          call calcola_derivataOP(cryst(nph)%ref,cryst(nph)%gpar,cryst(nph)%hklpo,cryst(nph)%cell,cryst(nph)%spg,derOP)
      endif
!
      if (allocated(dery2tcorr)) deallocate(dery2tcorr)
      allocate(dery2tcorr(ncounts),derph(nph)%dercl(ncounts,6),derph(nph)%derpG(ncounts)) 
      dery2tcorr(:) = 0
      select case (dataset(1)%typefun(nph))
         case(PEARSON7)
          !allocate(derph(nph)%dfwhm(ncounts,3),derph(nph)%derpf(ncounts,4),source=0.0)
          allocate(derph(nph)%dfwhm(ncounts,3),source=0.0)
          allocate(derph(nph)%derpf(ncounts,7),source=0.0)
          call derpears(fstr,dataset(1)%pear(nph),1,nref,dataset(1)%x0(:ncounts),cryst(nph)%ref,     &
                        cryst(nph)%get_cellpar(),derfct,derOP,dery2tcorr,                            &
                        !derph(nph)%dfwhm,derph(nph)%derpf,derph(nph)%dercl,derph(nph)%derpG,dataset(1)%ratio,   &
                        derph(nph)%dfwhm,derph(nph)%derpf,derph(nph)%dercl,jacob,dataset(1)%ratio,   &
                        cryst(nph)%scal,cryst(nph)%refine_cell(),                                    &
                        rinfo%refine_profile,dataset(1)%nwave,cryst(nph)%gcode,dataset(1)%radtype,   &                      
                        dataset(1)%sync,dataset(1)%wave(1),cryst(nph)%spg,nph)
          if (cryst(nph)%is_refined) call derpearsa(cryst(nph)%ref,dataset(1)%pear(nph),1,nref,    &
                 dataset(1)%x0(:ncounts),dera,derph(nph)%derca,dataset(1)%nwave,dataset(1)%ratio,cryst(nph)%scal,nph)

         case(PVOIG)
          !allocate(derph(nph)%dfwhm(ncounts,3),derph(nph)%derpf(ncounts,4),source=0.0)
          allocate(derph(nph)%dfwhm(ncounts,3),source=0.0)
          allocate(derph(nph)%derpf(ncounts,7),source=0.0)
          call derpsvoi(dataset(1)%pvoi(nph),1,nref,dataset(1)%x0(:ncounts),cryst(nph)%ref,        &
                        cryst(nph)%get_cellpar(),derfct,derOP,dery2tcorr,                          &
                        derph(nph)%dfwhm,derph(nph)%derpf,derph(nph)%dercl,derph(nph)%derpG,                  &
                        dataset(1)%ratio,cryst(nph)%scal,cryst(nph)%refine_cell(),                 &
                        rinfo%refine_profile,dataset(1)%nwave,cryst(nph)%gcode,dataset(1)%radtype, &
                        dataset(1)%sync,dataset(1)%wave(1),cryst(nph)%spg,nph)
          if (cryst(nph)%is_refined) call derpsvoia(cryst(nph)%ref,dataset(1)%pvoi(nph),1,nref,    &
                 dataset(1)%x0(:ncounts),dera,derph(nph)%derca,dataset(1)%nwave,dataset(1)%ratio,cryst(nph)%scal,nph)

         case(TCHZ)
          !allocate(derph(nph)%dfwhm(ncounts,6),derph(nph)%derpf(ncounts,1),source=0.0)
          allocate(derph(nph)%dfwhm(ncounts,6),source=0.0)
          allocate(derph(nph)%derpf(ncounts,4),source=0.0)
          call derpsvoi_tchz(dataset(1)%tch(nph),1,nref,dataset(1)%x0(:ncounts),cryst(nph)%ref,          &
                             cryst(nph)%get_cellpar(),derfct,derOP,dery2tcorr,                           &
                             derph(nph)%dfwhm,derph(nph)%derpf,derph(nph)%dercl,derph(nph)%derpG,                   &
                             dataset(1)%ratio,cryst(nph)%scal,cryst(nph)%refine_cell(),                  &
                             rinfo%refine_profile,dataset(1)%nwave,cryst(nph)%gcode,dataset(1)%radtype,  &                                    
                             dataset(1)%sync,dataset(1)%wave(1),cryst(nph)%spg)
          if (cryst(nph)%is_refined) call dertchza(cryst(nph)%ref,dataset(1)%tch(nph),1,nref,dataset(1)%x0(:ncounts),dera,    &
                                      derph(nph)%derca,dataset(1)%nwave,dataset(1)%ratio,cryst(nph)%scal,nph)
      end select
   enddo
!corr   nph=1
!
   LOOP_CONTEGGI : do i=1,ncounts
      kka = 0
!
!     Affinamento fo con pawley
      if (rfcond%raction == 2) then
          !do kk=1,rfcond%lim%nrefu
          do kk=1,size(fopar)
             if (fopar(kk)%rcod > 0) then
                 kka = fopar(kk)%rcod
                 !jacob(i,kka) = 2.0*dycount(kk,i)*ref(kk)%fo 
                 jacob(i,kka) = dycount(kk,i)
             endif
          enddo
      endif
!
!     Derivata dello zero.
      if (dataset(1)%zcode > 0) then
          kka = dataset(1)%zcode
!
!         Calcolo del contributo derzb del background alla derivato dello zero
          select case (dataset(1)%cond%btype)

            case (POLY)
              derzb=dataset(1)%coef(2)
              do k=3,dataset(1)%cond%ncoef
                 derzb = derzb + (k-1)*dataset(1)%coef(k)*(dataset(1)%x0(i)/dataset(1)%thzerob - 1.0)**(k-2)
              enddo
              derzb = -2.0*derzb / dataset(1)%thzerob

            case (CHEBY)
!!!!FIX FOR X0
              aa = -1.0
              bb = 1.0
              !difth = (rfcond%lim%ttmax-rfcond%lim%ttmin)
              !xche = 2.0*(dataset(1)%x(i) - rfcond%lim%ttmin)/difth - 1.0
              difth = (dataset(1)%xmaxc0() - dataset(1)%xminc0())
              xche = 2.0*(dataset(1)%x(i) - dataset(1)%xminc0())/difth - 1.0
              derzb = -4.0*chebev(aa,bb,chder(aa,bb,dataset(1)%coef(:dataset(1)%cond%ncoef)),xche)/difth

            case (FOUR_SERIES)
              derzb = 0.0

          end select
          jacob(i,kka) = -2.0 * dery2tcorr(i) + derzb
      endif
!
!     Derivative of sample displacement
      if (dataset(1)%sdcode > 0) then
          kka = dataset(1)%sdcode
          select case (dataset(1)%geom)
            case (BB_GEOMETRY)
              jacob(i,kka) = dery2tcorr(i) * cos(dtor*dataset(1)%x(i)/2)*rtod

            case (DS_GEOMETRY)  !!TODO
          end select
      endif
!
!     Derivative of sample transparency
      if (dataset(1)%stcode > 0) then
          kka = dataset(1)%stcode
          select case (dataset(1)%geom)
            case (BB_GEOMETRY)
              jacob(i,kka) = dery2tcorr(i) * sin(dtor*dataset(1)%x(i))*rtod

            case (DS_GEOMETRY)  !!TODO
          end select
      endif
!
!     Derivate del background
!corr      if (coefbkg(1)%rcod > 0) then
      if (dataset(1)%bcode(1) > 0) then

          select case (dataset(1)%cond%btype)

             case (POLY)
               thY=dataset(1)%x0(i)/dataset(1)%thzerob - 1.0
               jacob(i,kka+1) = 1.0
               do kk=2,dataset(1)%cond%ncoef
                  jacob(i,kk+kka) = thY**(kk-1)
               enddo

             case (CHEBY)
!!!!FIX FOR X0
               !xche = 2.0*(dataset(1)%x(i) - rfcond%lim%ttmin)/(rfcond%lim%ttmax-rfcond%lim%ttmin) - 1.0
               xche = 2.0*(dataset(1)%x(i) - dataset(1)%xminc0())/(dataset(1)%xmaxc0()-dataset(1)%xminc0()) - 1.0
               tt(1) = 1.0
               tt(2) = xche
               do j=2,dataset(1)%cond%ncoef-1
                  tt(j+1) = 2.*xche*tt(j) - tt(j-1)
               enddo
               do kk=1,dataset(1)%cond%ncoef
                   jacob(i,kka+kk) = tt(kk)
               enddo

             case (FOUR_SERIES)
               do kk=1,dataset(1)%cond%ncoef
                  jacob(i,kk+kka) = cos((kk-1)*dataset(1)%x0(i)*dtor)
               enddo

          end select
          kka = kka + dataset(1)%cond%ncoef

      endif
!
!     Derivate del profilo
      do nph=1,numphase(cryst)
         if (rinfo%refine_profile) then
             select case (dataset(1)%typefun(nph))
               case (PEARSON7)
                 if (dataset(1)%pear(nph)%par(P_UPAR)%rcod > 0) then
                     kka = kka + 1
                     jacob(i,kka) = derph(nph)%dfwhm(i,1)
                 endif
                 if (dataset(1)%pear(nph)%par(P_VPAR)%rcod > 0) then
                     kka = kka + 1
                     jacob(i,kka)= derph(nph)%dfwhm(i,2)
                 endif
                 if (dataset(1)%pear(nph)%par(P_WPAR)%rcod > 0) then
                     kka = kka + 1
                     jacob(i,kka) = derph(nph)%dfwhm(i,3)
                 endif
                 do npar=P_B0PAR,P_ASYM4PAR
                    if (dataset(1)%pear(nph)%par(npar)%rcod > 0) then
                        kka = kka + 1
                        jacob(i,kka)= derph(nph)%derpf(i,npar-P_B0PAR+1)
                    endif
                 enddo

               case (PVOIG)
                 if (dataset(1)%pvoi(nph)%par(PV_UPAR)%rcod > 0) then
                     kka = kka + 1
                     jacob(i,kka) = derph(nph)%dfwhm(i,1)
                 endif
                 if (dataset(1)%pvoi(nph)%par(PV_VPAR)%rcod > 0) then
                     kka = kka + 1
                     jacob(i,kka)= derph(nph)%dfwhm(i,2)
                 endif
                 if (dataset(1)%pvoi(nph)%par(PV_WPAR)%rcod > 0) then
                     kka = kka + 1
                     jacob(i,kka) = derph(nph)%dfwhm(i,3)
                 endif
                 do npar=PV_E0PAR,PV_ASYM4PAR
                    if (dataset(1)%pvoi(nph)%par(npar)%rcod > 0) then
                        kka = kka + 1
                        jacob(i,kka)= derph(nph)%derpf(i,npar-PV_E0PAR+1)
                    endif
                 enddo

               case (TCHZ)
                 do npar=T_UPAR,T_YPAR
                    if (dataset(1)%tch(nph)%par(npar)%rcod > 0) then
                        kka = kka + 1
                        jacob(i,kka) = derph(nph)%dfwhm(i,npar)
                    endif
                 enddo
                 do npar=T_ASYM1PAR,T_ASYM4PAR
                    if (dataset(1)%tch(nph)%par(npar)%rcod > 0) then
                        kka = kka + 1
                        jacob(i,kka) = derph(nph)%derpf(i,npar-T_ASYM1PAR+1)
                    endif
                 enddo
             end select
         endif
      enddo
!
      do nph=1,numphase(cryst)
!
!        Scale factor
         if (cryst(nph)%scode > 0) then
             kka = kka + 1
             !jacob(i,kka) = dataset(1)%yc(i) / cryst(1)%scal  !THIS CHANGE RESULTS!
             jacob(i,kka) = cryst(nph)%yc(i)
         endif
!
!        Derivate della cella
         if(cryst(nph)%refine_cell()) then
            do j=1,6
               if (cryst(nph)%get_cellcod(j) > 0) then
                   kka = kka + 1
                   jacob(i,kka) = derph(nph)%dercl(i,j)
               endif
               if (cryst(nph)%get_cellcod(j) == -1) jacob(i,kka) = jacob(i,kka) + derph(nph)%dercl(i,1) !constraints a=b o a=b=c
            enddo
         endif
!
!        Derivate parametri strutturali
         if (cryst(nph)%is_refined) then
             do j=1,cryst(nph)%natoms()
                do kk=1,5
                   if (cryst(nph)%at(j)%rcod(kk) > 0) then
                       kka = kka + 1
                       if (kk.eq.1) doc = derph(nph)%derca(i,j)%co(1)
                       if (kk.eq.2) doc = derph(nph)%derca(i,j)%co(2)
                       if (kk.eq.3) doc = derph(nph)%derca(i,j)%co(3)
                       if (kk.eq.4) doc = derph(nph)%derca(i,j)%b
                       if (kk.eq.5) doc = derph(nph)%derca(i,j)%occ
                       jacob(i,kka) = doc
                   endif
                enddo
             enddo
         endif
!
         do j=1,nconstraints(cryst(nph)%constr)
            if (cryst(nph)%constr(j)%ref > 0) then
                select case (cryst(nph)%constr(j)%code)
                  case (CONSTR_B)  ! constrain on b, 1 solo par. indipendente
                     kka = kka + 1
                     jacob(i,kka) = 0
                     do kp=1,cryst(nph)%constr(j)%npar
                        katom = cryst(nph)%constr(j)%vat(kp)
                        kpar = cryst(nph)%constr(j)%vpar(kp)
                        coef = cryst(nph)%constr(j)%coef(kp)
                        jacob(i,kka) = jacob(i,kka) + coef*derph(nph)%derca(i,katom)%b
                     enddo

                   case (CONSTR_CO) ! constrain on coord., 1 solo par. indipendente
                     kka = kka + 1
                     jacob(i,kka) = 0
                     do kp=1,cryst(nph)%constr(j)%npar
                        katom = cryst(nph)%constr(j)%vat(kp)
                        kpar = cryst(nph)%constr(j)%vpar(kp)
                        coef = cryst(nph)%constr(j)%coef(kp)
                        jacob(i,kka) = jacob(i,kka) + coef*derph(nph)%derca(i,katom)%co(kpar)
                     enddo

                   case (CONSTR_SOF) ! constrain on occupanze, npar-1 par. indipendenti
                     npar = cryst(nph)%constr(j)%npar
                     kpind= cryst(nph)%constr(j)%vat(npar)  ! parametro indipendente
                     do np=1,npar-1
                        kka = kka + 1
                        katom = cryst(nph)%constr(j)%vat(np)
!!!!TODO: if you use coef should be: jacob(i,kka) = derca(i,katom)%occ - derca(i,kpind)%occ*constrv(j)%coef(i)/constrv(j)%coef(kpind)
                        jacob(i,kka) = derph(nph)%derca(i,katom)%occ - derph(nph)%derca(i,kpind)%occ
                     enddo
                end select
            endif
         enddo
!
!        Derivate parametri di orientazione preferenziale
         if (cryst(nph)%gcode > 0) then
             kka = kka + 1
             !  write(0,*)'KKA=',kka,cryst(nph)%gcode
             !jacob(i,kka) = derph(nph)%derpG(i)
         endif
!
         !if (cryst(nph)%absf == BB_SUOR .or. cryst(nph)%absf == BB_PITS) then
         !endif
      enddo
!
      jacob(i,:kka) = -sqrt(weilsq(i))*jacob(i,:kka)
   enddo LOOP_CONTEGGI
!
!  Introduce nello Jacobiano ed in funcls il contributo dei restraints
   if (cryst(1)%numres()>0 .and. cryst(1)%is_refined) then
       call resapp(cryst(1)%at,cryst(1)%cell,cryst(1)%res,cryst(1)%wres,ncounts,1,coderwi=2)
   endif
!
   end subroutine rietjacobian

!------------------------------------------------------------------------------------------------

   subroutine set_parameter(Nact,parx)
   USE RefinecomRef
   USE Conteggi
   USE Counts
   USE background
   USE unit_cell
   USE variables, only:cryst,dataset
   USE RRESTR
   USE genlsq, only:shiftx,step_length
   USE reflection_type_util
   USE constraints
   USE unit_cell
   USE profile_function
   USE spginfom
   USE type_constants, only: DP
   USE crystal_phase
   USE pearsonf
   USE psvoigtf
   USE tchzf
!
   implicit none
   integer              , intent(in)          :: Nact
   real(DP), dimension(Nact), intent(in)      :: parx
   integer                                    :: i,j,nph
   integer                                    :: nkref
   integer                                    :: icalcback != 0
   logical, dimension(:), allocatable         :: icalcFc,icalcrapI
   integer                                    :: kk
   integer                                    :: katom,kparam
   integer                                    :: kp
   real                                       :: coefc
   integer                                    :: kcon
   type(gref_type), allocatable, dimension(:) :: gref
   integer                                    :: ng
   integer                                    :: code
   integer                                    :: npar,nref
   real                                       :: sumocc
   logical                                    :: refocc
   real, dimension(6)                         :: cellpar
!
   nkref = 0
   icalcback = 0
   !allocate(icalcFc(numphase(cryst)),icalcrapI(numphase(cryst)),source=.false.)
   allocate(icalcFc(numphase(cryst)),source=.false.)
   allocate(icalcrapI(numphase(cryst)),source=.false.)
!
!  Fo se pawley
   if (rfcond%raction == 2) then
              call make_ref_groups(cryst(1)%ref,gref,sogi=0.3)
       !do i=1,rfcond%lim%nrefu
       do i=1,size(fopar)
          if (fopar(i)%rcod > 0) then
              nkref = nkref + 1
              fopar(i)%rcod = nkref
              !fopar(i)%val = parx(nkref)
              if (parx(nkref) < 0.000) then
                      write(0,*)'NEG=',parx(nkref)
                  ng = number_of_group(i,gref)
                  if (gref(ng)%code == 1) then    ! riflesso singolo
                      fopar(i)%val = 0.000
                  else                            ! media delle intensita
                      fopar(i)%val = sum(cryst(1)%ref(gref(ng)%vref)%fo) / gref(ng)%code
                  endif
              else
                  fopar(i)%val = sqrt(parx(nkref))
              endif
              cryst(1)%ref(i)%fo = fopar(i)%val
          endif
       enddo
       icalcRapI = .true.
   endif
!
!  Zero
   if (dataset(1)%zcode > 0) then
       nkref = nkref + 1
       dataset(1)%zero = parx(nkref)
       !ttetaczero = ttetacount - 2.0 * zero%val
       !icalcback = 1          ! se affino lo zero ricalcola il background
   endif
!
!  Sample displacement
   if (dataset(1)%sdcode > 0) then
       nkref = nkref + 1
       dataset(1)%sdisp = parx(nkref)
   endif
 !
!  Sample transparency
   if (dataset(1)%stcode > 0) then
       nkref = nkref + 1
       dataset(1)%stran = parx(nkref)
   endif   
!
!  Apply shift !!!!TODO: better to move after
   if (dataset(1)%zcode > 0 .or. dataset(1)%sdcode > 0 .or. dataset(1)%stcode > 0) then
       !dataset(1)%x0 = dataset(1)%x - 2.0 * dataset(1)%zero + sample_displacement(dataset(1)%x0,dataset(1)%sdisp,diffr_geom)   &
       !                                         + sample_transparency(dataset(1)%x0,dataset(1)%stran,diffr_geom)
       call dataset(1)%apply_shifts()

       icalcback = 1          ! update background
   endif
!
!  Background
   if (dataset(1)%bcode(1) > 0) then
       do i=1,dataset(1)%cond%ncoef
          nkref = nkref + 1
          dataset(1)%coef(i) = parx(nkref)
       enddo
       icalcback = 1
   endif
!
!  Profilo
   do nph=1,numphase(cryst)
      select case (dataset(1)%typefun(nph))
        case (PEARSON7)
          do npar=P_UPAR,P_ASYM4PAR
             if (dataset(1)%pear(nph)%par(npar)%rcod > 0) then
                 nkref = nkref + 1
                 dataset(1)%pear(nph)%par(npar)%val = parx(nkref)
             endif
          enddo

        case (PVOIG)
          do npar=PV_UPAR,PV_ASYM4PAR
             if (dataset(1)%pvoi(nph)%par(npar)%rcod > 0) then
                 nkref = nkref + 1
                 dataset(1)%pvoi(nph)%par(npar)%val = parx(nkref)
             endif
          enddo

        case (TCHZ)
          do npar=T_UPAR,T_ASYM4PAR
             if (dataset(1)%tch(nph)%par(npar)%rcod > 0) then
                 nkref = nkref + 1
                 dataset(1)%tch(nph)%par(npar)%val = parx(nkref)
             endif
          enddo
      end select
   enddo
!
   do nph=1,numphase(cryst)
!
      nref = cryst(nph)%numref()
!
!     Scala
      if (cryst(nph)%scode > 0) then
          nkref = nkref + 1
          cryst(nph)%scal= parx(nkref)
      endif
!
!     Cell parameters
      if(cryst(nph)%refine_cell()) then
         cellpar = cryst(nph)%get_cellpar()
         do j=1,6
            !if (cellph%get_cod(j) > 0) then
            if (cryst(nph)%get_cellcod(j) > 0) then
                nkref = nkref + 1
                cellpar(j) = parx(nkref)
            endif
         enddo
         call cryst(nph)%set_cell(cellpar)
         call cryst(nph)%norm_cell()
!corr         cell(1:6) = cryst(nph)%get_cellpar()
!
!        recompute 2theta 
         call cryst(nph)%calc_2theta(dataset(1)%wave(:))
!
!        ricalcola le correzioni
         call cryst(nph)%LPcorr(dataset(1)%radtype,dataset(1)%sync,dataset(1)%nwave)
         call cryst(nph)%ABScorr(dataset(1)%nwave)
         if (cryst(nph)%kopcorr == 1) then
             call calcola_correzioneOP(cryst(nph)%ref(:nref),cryst(nph)%gpar,   &
                                       cryst(nph)%hklpo,cryst(nph)%cell,cryst(nph)%spg)
         endif
!
         icalcFc(nph) = .true.
         icalcrapI(nph) = .true.
      endif
!
!     Struttura
      if (cryst(nph)%is_refined) then
          refocc = .false.
          do j=1,cryst(nph)%natoms()
             do kk=1,3 
                if (cryst(nph)%at(j)%rcod(kk) > 0) then
                   nkref = nkref + 1
                   cryst(nph)%at(j)%xc(kk) = parx(nkref)
                endif
             enddo
             if (cryst(nph)%at(j)%rcod(4) > 0) then
                 nkref = nkref + 1
                 cryst(nph)%at(j)%biso = parx(nkref)
                 refocc = .true.
             endif
             if (cryst(nph)%at(j)%rcod(5) > 0) then
                 nkref = nkref + 1
                 cryst(nph)%at(j)%och = parx(nkref)
             endif
          enddo
!
          do kcon=1,nconstraints(cryst(nph)%constr)
             if (cryst(nph)%constr(kcon)%ref > 0) then
                 code = cryst(nph)%constr(kcon)%code
                        !write(6,*)'XPAR=',parx(nkref),shiftx(nkref)
                 select case(code)
                    case (CONSTR_B)   ! constraints on b
                      nkref = nkref + 1
                      do kp = 1,cryst(nph)%constr(kcon)%npar
                         katom = cryst(nph)%constr(kcon)%vat(kp)
                         coefc = cryst(nph)%constr(kcon)%coef(kp)
                         cryst(nph)%at(katom)%biso =  cryst(nph)%atmc(katom)%biso + coefc*shiftx(nkref)*step_length
                      enddo

                     case (CONSTR_CO)  ! constraints on coord.
                      nkref = nkref + 1
                      do kp = 1,cryst(nph)%constr(kcon)%npar
                         katom = cryst(nph)%constr(kcon)%vat(kp)
                         kparam = cryst(nph)%constr(kcon)%vpar(kp)
                         coefc = cryst(nph)%constr(kcon)%coef(kp)
                         cryst(nph)%at(katom)%xc(kparam) = cryst(nph)%atmc(katom)%xc(kparam) + coefc*shiftx(nkref)*step_length
                      enddo

                     case (CONSTR_SOF)  ! constraints on sof
                      npar = cryst(nph)%constr(kcon)%npar
                      sumocc = 0.0
                      do i=1,npar-1  ! loop sui par. indipendenti
                         nkref = nkref + 1
                         katom = cryst(nph)%constr(kcon)%vat(i)
                         !atm(katom)%och = cryst(nph)%atmc(katom)%och + shiftx(nkref)*step_length
                         !   write(0,*)'OCC=',atm(katom)%och,' XPAR=',parx(nkref)
                         cryst(nph)%at(katom)%och = parx(nkref)
                         sumocc = sumocc + cryst(nph)%at(katom)%och
                      enddo
                      katom = cryst(nph)%constr(kcon)%vat(npar)   ! parametro dipendente
                      cryst(nph)%at(katom)%och = cryst(nph)%constr(kcon)%sumc - sumocc
                      refocc = .true.
                      
                 end select
             endif
          enddo
!corr          icalcFc = 1
          icalcFc(nph) = .true.
          icalcrapI(nph) = .true.
!         recompute content if occupancy has been refined
!te   st       if (refocc) call compute_content(cryst(1)%at,cryst(1)%elem,cryst(1)%spg)
          !if (refocc) write(0,*)'NW=',cryst(1)%elem%nw
      endif
!
!     Orientazione preferenziale
      if (cryst(nph)%gcode > 0) then  
          nkref = nkref + 1
          cryst(nph)%gpar = parx(nkref)
          call calcola_correzioneOP(cryst(nph)%ref(:nref),cryst(nph)%gpar,   &
                                    cryst(nph)%hklpo,cryst(nph)%cell,cryst(nph)%spg)
          icalcrapI(nph) = .true.   
      endif
   enddo
   !call cryst(1)%get_atoms(atm)
   !call fillatom(cryst(1)%at)
   if (rfcond%raction == RIET_REFINE) then
       do nph=1,numphase(cryst)
          if (icalcFc(nph)) call cryst(nph)%sfactor()
       enddo
   endif
   do nph=1,numphase(cryst)
      if (icalcrapI(nph)) call setRInt(cryst(nph)%ref)
   enddo
!
   if (icalcback == 1) then
!corr       call getbackground(dataset(1)%cond%btype,dataset(1)%yb(:rfcond%lim%ncu),dataset(1)%x0,   &
       call getbackground(dataset(1)%cond%btype,dataset(1)%yb(:dataset(1)%nc2),dataset(1)%x0,   &
            dataset(1)%coef(:dataset(1)%cond%ncoef),dataset(1)%thzerob)
   endif
!
   end subroutine set_parameter

!------------------------------------------------------------------------------------------

   subroutine set_sdval(sdval)
   USE RefinecomRef
   USE variables, only: cryst,dataset
   USE RRESTR
   USE constraints
   USE type_constants, only: DP
   USE atom_type_util
   USE profile_function
   USE spginfom
   USE crystal_phase
   USE pearsonf
   USE psvoigtf
   USE tchzf
   implicit none
   real(DP), dimension(:), intent(in) :: sdval
   integer                            :: i,j,nph
   integer                            :: nkref
   integer                            :: kk
   integer                            :: katom,kparam
   integer                            :: kp
   real                               :: coefc
   integer                            :: kcon
   integer                            :: code
   integer                            :: npar
   real                               :: sumsd
   real(DP), dimension(6)             :: sdcell
!
   nkref = 0
!
!  Fo se pawley
   if (rfcond%raction == 2) then
       !do i=1,rfcond%lim%nrefu
       do i=1,size(fopar)
          if (fopar(i)%rcod > 0) then
              nkref = nkref + 1
          endif
       enddo
   endif
!
!  Zero
   if (dataset(1)%zcode > 0) then
       nkref = nkref + 1
       dataset(1)%zsd = sdval(nkref)
   endif
!
!  sample displacement
   if (dataset(1)%sdcode > 0) then
       nkref = nkref + 1
       dataset(1)%sdsd = sdval(nkref)
   endif
!
!  sample displacement
   if (dataset(1)%stcode > 0) then
       nkref = nkref + 1
       dataset(1)%stsd = sdval(nkref)
   endif
!
!  Background
   if (dataset(1)%bcode(1) > 0) then
       do i=1,dataset(1)%cond%ncoef
          nkref = nkref + 1
          dataset(1)%bsd(i) = sdval(nkref)
       enddo
   endif
!
!  Profilo
   do nph=1,numphase(cryst)
      select case (dataset(1)%typefun(nph))
        case (PEARSON7)
          do npar=P_UPAR,P_ASYM4PAR
             if (dataset(1)%pear(nph)%par(npar)%rcod > 0) then
                 nkref = nkref + 1
                 dataset(1)%pear(nph)%par(npar)%sd = sdval(nkref)
             endif
          enddo

        case (PVOIG)
          do npar=PV_UPAR,PV_ASYM4PAR
             if (dataset(1)%pvoi(nph)%par(npar)%rcod > 0) then
                 nkref = nkref + 1
                 dataset(1)%pvoi(nph)%par(npar)%sd = sdval(nkref)
             endif
          enddo

        case (TCHZ)
          do npar=T_UPAR,T_ASYM4PAR
             if (dataset(1)%tch(nph)%par(npar)%rcod > 0) then
                 nkref = nkref + 1
                 dataset(1)%tch(nph)%par(npar)%sd = sdval(nkref)
             endif
          enddo
      end select
   enddo
!
   do nph=1,numphase(cryst)
!
!     Scale
      if (cryst(nph)%scode > 0) then
          nkref = nkref + 1
          cryst(nph)%ssd = sdval(nkref)
      endif
!
!     Cell
      if(cryst(nph)%refine_cell()) then
         do j=1,6
            if (cryst(nph)%get_cellcod(j) > 0) then
                nkref = nkref + 1
                sdcell(j) = sdval(nkref)
            endif
         enddo
         call cryst(nph)%set_cellsdref(real(sdcell))
      endif
!
!     Struttura
      if (cryst(nph)%is_refined) then
          do j=1,cryst(nph)%natoms()
             do kk=1,3 
                if (cryst(nph)%at(j)%rcod(kk) > 0) then
                   nkref = nkref + 1
                   cryst(nph)%at(j)%xsd(kk) = sdval(nkref)
                endif
             enddo
             if (cryst(nph)%at(j)%rcod(4) > 0) then
                 nkref = nkref + 1
                 cryst(nph)%at(j)%bsd = sdval(nkref)
             endif
             if (cryst(nph)%at(j)%rcod(5) > 0) then
                 nkref = nkref + 1
                 cryst(nph)%at(j)%osd = sdval(nkref)
             endif
          enddo
!
          do kcon=1,nconstraints(cryst(nph)%constr)
             if (cryst(nph)%constr(kcon)%ref > 0) then
                 code = cryst(nph)%constr(kcon)%code
                 select case(code)
                    case (CONSTR_B)   ! constraints on b
                      nkref = nkref + 1
                      do kp = 1,cryst(nph)%constr(kcon)%npar
                         katom = cryst(nph)%constr(kcon)%vat(kp)
                         cryst(nph)%at(katom)%bsd =  sdval(nkref)
                      enddo

                     case (CONSTR_CO)  ! constraints on coord.
                      nkref = nkref + 1
                      do kp = 1,cryst(nph)%constr(kcon)%npar
                         katom = cryst(nph)%constr(kcon)%vat(kp)
                         kparam = cryst(nph)%constr(kcon)%vpar(kp)
                         coefc = cryst(nph)%constr(kcon)%coef(kp)
                         cryst(nph)%at(katom)%xsd(kparam) = coefc*sdval(nkref)
                      enddo

                     case (CONSTR_SOF)  ! constraints on sof
                      npar = cryst(nph)%constr(kcon)%npar
                      sumsd = 0.0
                      do i=1,npar-1  ! loop sui par. indipendenti
                         nkref = nkref + 1
                         katom = cryst(nph)%constr(kcon)%vat(i)
                         cryst(nph)%at(katom)%osd = sdval(nkref)
                         sumsd = sumsd + sdval(nkref)
                      enddo
                      katom = cryst(nph)%constr(kcon)%vat(i)   ! parametro dipendente
                      cryst(nph)%at(katom)%osd = sumsd
                                        
                 end select
             endif
          enddo
      endif
!
!     Orientazione preferenziale
      if (cryst(nph)%gcode > 0) then
          nkref = nkref + 1
          cryst(nph)%gsd = sdval(nkref)
      endif
   enddo
!
   end subroutine set_sdval


!-----------------------------------------------------------------------

   subroutine gesref(refop,exitcode)   !!!,refop)
   USE RefinecomRef
   USE PatternRef
   USE General, only: lo
   USE GENLSQ
   USE Molcom  
   USE variables, only : updgr,cryst,dataset
   USE progtype,only:DEALLOCA,ALLOCA
   USE RRESTR 
   USE prnutil
   USE grexpomod
   USE messagemod
   USE ilmviewmod
   USE atom_type_util
   USE connect_mod
   USE unit_cell
   USE setref
   USE constraints
   USE crystal_phase
!
   implicit none
   logical, dimension(:), intent(in)  :: refop
   integer, intent(out)               :: exitcode
   integer                            :: Nactot
   integer                            :: modes
   logical                            :: winact
   integer                            :: kstageprof
   integer                            :: nstage
   integer                            :: nstagestru
   integer                            :: stoplsq
   logical                            :: bothref   !!!!,continue_ref,resetcode
   integer                            :: nstageval
   integer                            :: numres
   character(len=200)                 :: msgpar,msgstep
   character(len=200)                 :: msginfo
   integer                            :: ierp,nph,ncu
   type(refinement_strategy_type)     :: riets
   external frietlm
   external frietgauss
!

   exitcode = 0
   call open_rietveld_viewer()
   ifBrekke = 0
   termcode = 0
   updgr = 5
   kstagemax = 0
   nstageval = 11
   bothref = rfcond%auto_prof .and. rfcond%auto_stru 
   numres = cryst(1)%numres()
   if (rfcond%raction == 2) nstageval = 35
   if (bothref) then
       kstagemax = nstageval  !!!!22  !!!11
       kstageprof = kstagemax
       !call define_strategy(1,sstage,specstage,nstage,numres)
       call riets%new()
       call define_strategy_prof(riets,rfcond%raction)
       call define_strategy_struct(1,riets,dataset(1)%radtype)
       call riets%add_stage('ALLP')
       !call riets%prn(0)
       !nstage = riets%get_num()
       !kstagemax = kstagemax+nstage+1
       kstagemax = riets%get_num()
       modes = 2
   else
       if (rfcond%auto_prof) then
           !kstagemax = nstageval  !!!!22  !!!!11
           modes = 2   !!!!1
           call riets%new()
           call define_strategy_prof(riets,rfcond%raction)
           kstagemax = riets%get_num()
       elseif (rfcond%auto_stru) then        
           !call define_strategy(1,sstage,specstage,nstage,numres)
           call riets%new()
           call define_strategy_struct(1,riets,dataset(1)%radtype)
           nstage = riets%get_num()
           kstagemax = kstagemax + nstage
           modes = 2
       endif
   endif
   do nph=1,numphase(cryst)
      call calcola_occ(cryst(nph)%at,cryst(nph)%spg,cryst(nph)%cell)
   enddo
!corr!!
!corr!!  save in refop if po refinement has been required
!corr!   allocate(refop(numphase(cryst)))
!corr!   do nph=1,numphase(cryst)
!corr!      refop(nph) = cryst(nph)%gcode > 0
!corr!   enddo
!
   call clear_messages(ipos=2)
!
   nstagestru = 0
   call setwei() ! il peso va riempito nuovamente
   loop_stage: do kstager = 1,kstagemax
!
      call azzeraref(.false.)                                  
      call setstage(riets%stages(kstager),refop,rfcond)
      msgpar = riets%stages(kstager)%get_string()
      msgstep = 'Automatic refinement '    !!!TODO:set correctly this string
      write(msginfo,'(a,i0,a,i0,a)')trim(msgstep)//' (step ',kstager,'/',kstagemax,').'
      call write_message(trim(msginfo),pos=2)
      call write_message('Parameters: '//trim(msgpar),pos=3)
      !call progress_bar_update(cval=kstager,fval=kstagemax)
!
      ncu = dataset(1)%nc2
      loop_restart: do
          Nactot = count_parameters(rfcond)
          if (Nactot == 0) cycle loop_stage           ! solo se ho atomi in posizione speciale
          if (cryst(1)%is_refined) then
              call Allocafuncls(ALLOCA,ncu+numres)
          else
              call Allocafuncls(ALLOCA,ncu)
          endif
          call setwei() ! il peso va riempito nuovamente
          call AllocaLSQ(DEALLOCA)
          if (numres > 0 .and. cryst(1)%is_refined) then
              call AllocaLSQ(ALLOCA,Nactot,ncu+numres)
          else
              call AllocaLSQ(ALLOCA,Nactot,ncu)
          endif
          call prerefine()
!
          call calcola_profilo(Nactot,ierp)
!
          if (rfcond%algo == LMDER) then
!corr              call levmarq_algo(frietlm,Nactot,rfcond%lim%ncu,modes,winact)
              call levmarq_algo(frietlm,Nactot,ncu,modes,winact)
              stoplsq = 0
          else
              call gaussn_algo(frietgauss,Nactot,rfcond%maxcy,rfcond%eps,rfcond%sprint,exitcode,stoplsq)
              if (exitcode == 1) cycle loop_stage  ! skip stage from user
              if (exitcode == 2) exit loop_stage   ! exit from user
          endif
!
          if (kstager == kstagemax) call stampa_risultati(lo)
          if (stoplsq == 0) exit
      enddo loop_restart
!
      call calcola_profilo_leb(Nactot) !prova
   enddo loop_stage
   call calcola_profilo_leb(Nactot) !prova
        
!
!  Ricalcola le occ. se hai affinato la struttura
   do nph=1,numphase(cryst)
      if (cryst(nph)%is_refined) then
          call calcola_occ(cryst(nph)%at,cryst(nph)%spg,cryst(nph)%cell)
          call bond_distance_update(cryst(nph)%at,cryst(nph)%bond,cryst(nph)%cell%get_g())
          if (nph == active_phase) then
              call ilmview_draw(cryst(active_phase)%at,cryst(active_phase)%bond,         &
                                cryst(active_phase)%elem,cryst(active_phase)%cell,cryst(active_phase)%spg)
          endif
          call update_pattern(ncytot)
      endif
   enddo
!
   call AllocaLSQ(DEALLOCA)
!
   call azzeraref()         ! riazzera irefmat per evitare selezioni attive con la grafica
   if (rfcond%raction == 0) then
       do nph=1,numphase(cryst)
          call make_constraints(cryst(nph)%at,cryst(nph)%bond,cryst(nph)%constr,cryst(nph)%spg,cryst(nph)%cell,   &
                             cryst(nph)%riding_model,cryst(nph)%adp_factor)
       enddo
   endif
!
   end subroutine gesref

!-----------------------------------------------------------------------------------------

   subroutine useref()
   USE RefinecomRef
   USE PatternRef
   USE General
   USE Molcom, only : ifBrekke
   USE GENLSQ
   USE variables, only : updgr,cryst,dataset
   USE progtype,only:DEALLOCA,ALLOCA
   USE RRESTR
   USE prnutil
   USE grexpomod
   USE messagemod
      USE ilmviewmod
      USE atom_type_util
   USE connect_mod
   USE unit_cell
   USE setref
   use crystal_phase
                                   use constraints
!
   integer  :: Nactot
   integer  :: exitcode
   integer  :: stoplsq
   logical  :: continue_ref,resetcode
   integer  :: numres
   integer  :: ierp,nph,ncu
   external frietlm
   external frietgauss
!
   fortran_work = .true.
   call open_rietveld_viewer()
   ifBrekke = 0
   termcode = 0
   updgr = 1
!
!  Inizializzazioni utili per la gestione della progressbar
   kstagemax = 1
   kstager = 1
   ncurrent_cycle = 1
   rfcond%ncauto = 1
   ncu = dataset(1)%nc2
!
   call progress_bar_init('Starting refinement')
   call clear_messages(ipos=2)
   if (cryst(1)%is_refined) call calcola_occ(cryst(1)%at,cryst(1)%spg,cryst(1)%cell)
   do 
!
       resetcode = .false.
            !write(lo,'(a)')'Print constraints to refine'
            !  call print_constraints(cryst(1)%constr,cryst(1)%at,lo,.false.) ! print refined constraints
            !write(lo,'(a)')'Print all constraints'
            !  call print_constraints(cryst(1)%constr,cryst(1)%at,lo,.true.) ! print refined constraints
       Nactot = count_parameters(rfcond)
       if (Nactot == 0) exit
       call AllocaLSQ(DEALLOCA)
!
       numres = cryst(1)%numres()
       if (numres > 0 .and. cryst(1)%is_refined) then
           call AllocaLSQ(ALLOCA,Nactot,ncu+numres)
       else
           call AllocaLSQ(ALLOCA,Nactot,ncu)
       endif
       call prerefine()
!
       call calcola_profilo(Nactot,ierp)
       if (rfcond%algo == LMDER) then
           call levmarq_algo(frietlm,Nactot,ncu,2,.TRUE.)
       else
           do 
              call gaussn_algo(frietgauss,Nactot,rfcond%maxcy,rfcond%eps,rfcond%sprint,exitcode,stoplsq)
              if (stoplsq > 0) exit
              call message_from_lsq(continue_ref,resetcode)
              if (.not.continue_ref) exit
           enddo
       endif
       if (stoplsq == 0) exit
   enddo
!
!  Ricalcola le occ. se hai affinato la struttura
   do nph=1,numphase(cryst)
      if (cryst(nph)%is_refined) then
          call calcola_occ(cryst(nph)%at,cryst(nph)%spg,cryst(nph)%cell)
          call bond_distance_update(cryst(nph)%at,cryst(nph)%bond,cryst(nph)%cell%get_g())
          if (nph == active_phase) then
              call ilmview_draw(cryst(active_phase)%at,cryst(active_phase)%bond,         &
                                cryst(active_phase)%elem,cryst(active_phase)%cell,cryst(active_phase)%spg)
          endif
          call update_pattern(ncytot)
      endif
   enddo

!
   call stampa_risultati(lo)
   if (resetcode) then
       call azzeraref()
       do nph=1,numphase(cryst)
          call make_constraints(cryst(nph)%at,cryst(nph)%bond,cryst(nph)%constr,cryst(nph)%spg,cryst(nph)%cell,   &
                                cryst(nph)%riding_model,cryst(nph)%adp_factor)
       enddo
   endif
!
   call progress_bar_close()
   call AllocaLSQ(DEALLOCA)
   fortran_work = .false.
!
   end subroutine useref

!-----------------------------------------------------------------

   subroutine unset_param(xset)
!
!  Unset refined parameters when xpar is false 
!
   use variables, only: cryst
   use crystal_phase
   use constraints
   logical, dimension(:), intent(in) :: xset
   integer                           :: np,na,nc
   integer                           :: nph
!
   do np=1,size(xset)
      if (xset(np)) then
          loop_phase: do nph=1,numphase(cryst)
             do nc=1,nconstraints(cryst(nph)%constr)
                if (cryst(nph)%constr(nc)%ref == np) then
                    cryst(nph)%constr(nc)%ref = 0
                       !write(0,*)'UNSET constr:',nc
                    exit loop_phase
                endif
             enddo
             do na=1,cryst(nph)%natoms()
                if (cryst(nph)%at(na)%rcod(4) == np) then  ! check only on b
                    cryst(nph)%at(na)%rcod(4) = 0
                       !write(0,*)'UNSET at:',na
                    exit loop_phase
                endif
             enddo
          enddo loop_phase
      endif
   enddo

!
   end subroutine unset_param

!-----------------------------------------------------------------------

   subroutine compute_sd(ier)
   USE genlsq
   USE type_constants, only: DP
   implicit none
   integer, intent(out)               :: ier
   real(DP), dimension(size(jacob,2)) :: sdpar
!
   call calculate_sd(jacob,funcls,size(jacob,2),sdpar,ier)
   if (ier == 0) then
       call set_sdval(sdpar)
   endif
!
   end subroutine compute_sd

 END MODULE RIETV

!-------------------------------------------------------------------

   subroutine riet_new(ncyr)
   USE RefinecomRef
   USE General, only: ERR_WINDOW, lo    !!!!!, cell
   USE Molcom, only : ModelToCompare
   USE RRESTR, only:nrestraints
   USE GENLSQ
   USE Counts
   USE progtype, only:ALLOCA,DEALLOCA
   USE variables, only:cryst,dataset
   USE grexpomod, only:fortran_work
   USE enable_amb
   USE messagemod
   USE ilmviewmod
   USE cryutil
   USE crystal_phase
   USE reflection_type_util
   USE rietv
   USE arrayutil
!
   implicit none
   integer, intent(in)               :: ncyr
   integer                           :: nExitCode
   logical                           :: kautomatic
   integer                           :: i,nph
   integer                           :: exitcode
   integer                           :: numres
!
   if (ncyr == 1) then   ! in a loop on riet_new restart refop only at first cycle
       call new_array(refop,numphase(cryst))
       refop(:) = cryst(:)%gcode > 0
   endif
   do nph=1,numphase(cryst)
      if (cryst(nph)%kopcorr == 1) then
          call calcola_correzioneOP(cryst(nph)%ref(:),cryst(nph)%gpar,cryst(nph)%hklpo,cryst(nph)%cell,cryst(nph)%spg)
      endif
   enddo
   kautomatic = rfcond%auto_prof .or. rfcond%auto_stru
!
   call jav_set_modify(0)   ! disabilita menu'
   call abilita_tasti('run')
   if (kautomatic) then
       if (ncytot == 0 .and. index(rfcond%sprint,'S') > 0) then
           call printfin(lo,'Starting Rietveld parameters')    ! stampa parametri+restraints
       endif
       fortran_work = .true.
       call progress_bar_init('Starting refinement')
       do i=1,rfcond%ncauto
          ncurrent_cycle = i
          call gesref(refop,exitcode)
          if (exitcode == 2) exit
       enddo
       call progress_bar_close()
       call to_coord_cry(cryst,-1,9)
       rfcond%auto_stru = .false.
       rfcond%auto_prof = .false.
       fortran_work = .false.
   else
       if (count_parameters(rfcond) == 0)  then        ! Se non affino niente ...
           call MsgWinErr('ERROR','User Refinement Stage. Please set the strategy before going on !',ERR_WINDOW,nExitCode)
           return
       else
           if (ncytot == 0 .and. index(rfcond%sprint,'S') > 0) then
               call printfin(lo,'Starting Rietveld parameters')    ! stampa parametri+restraints
           endif
!
!          Se nel frattempo ho introdotto attraverso la grafica dei restraints devo riallocare funcls
           numres = cryst(1)%numres()
           if (size(funcls) /= dataset(1)%nc2 + numres .and. cryst(1)%is_refined) then
               call Allocafuncls(ALLOCA,dataset(1)%nc2+numres)
               call setwei() ! il peso va riempito nuovamente
           endif
           call useref()
           call to_coord_cry(cryst,-1,9)
       endif
   endif
   call abilita_tasti('rietveld')
   call jav_set_modify(3)   ! disabilita menu'
!
   end subroutine riet_new

!------------------------------------------------------------------------------------------
   subroutine rietveld()
   USE RefinecomRef
   USE variables, only:cryst,dataset
   USE Molcom
   USE General
   USE genlsq
   USE ilmviewmod
   USE atom_type_util
   USE sinterface
   USE enable_amb
   USE rietv
   USE unit_cell
   USE rinterface
!
   implicit none
   logical                            :: modif
   integer                            :: numph, i
   integer, dimension(:), allocatable :: natoms,nelem
!
   if (jscreen == 0 .or. ifAutomatic == 1) then   
!      Rietveld run without GUI or in automatic mode
       call rietveldexe(refine='all',init=.true.)
       !call rietveldexe(refine='profile',init=.true.)
       return
   endif
!
!  FIX THIS!!!
   if (.not.cryst(1)%has_symmetry()) then
       call set_cell_and_spg(cryst(1),modif)
       if (modif .and. jav_opened()) then
           call ilmview_draw(cryst(1)%at,cryst(1)%bond,cryst(1)%elem,cryst(1)%cell,cryst(1)%spg)
       endif
   endif
!
   if (cryst(1)%natoms() == 0) then
       call open_file_selector('Import starting model for Rietveld Refinement'//char(0))
       return
   endif
!
   call init_rietveld(1,.true.,.true.)
!
   if (.not. jav_opened()) call ilmview_open(cryst(active_phase)%cr_name)
   call ilmview_draw(cryst(active_phase)%at,cryst(active_phase)%bond,cryst(active_phase)%elem,      &
                     cryst(active_phase)%cell,cryst(active_phase)%spg)
   call jav_set_modify(3)   ! abilita solo i menu' di info
!
!  Crystal phase section
   call get_info_crystal(numph,natoms,nelem)
   call make_crystal_t()
   call put_info_crystal(numph,natoms,nelem,cryph)
   do i=1, numph
      call put_info_ph(cryst(i)%elem, cryst(i)%at,i)
      call put_info_po(cryst(i)%kopcorr,cryst(i)%hklpo,cryst(i)%gcode, cryst(i)%gpar,i)
   enddo
!
!  Dataset section
   call make_dataset_t()
   call put_info_dataset(datas)
   do i=1, numph
      call put_info_pr(dataset(1)%pear(i),         &
             dataset(1)%pvoi(i),dataset(1)%tch(i),dataset(1)%typefun(i),i,1)
   enddo
!
!  Apri finestra grafica
   call abilita_tasti('rietveld')
   call open_rietdlg(rfcond, dataset(1)%cond)
!
   end subroutine rietveld

!------------------------------------------------------------------------------------------

   subroutine init_rietveld(kaction,initfo,scalefo)
   USE RefinecomRef
   USE PatternRef
   USE Molcom
   USE General, only: lo
   USE PhaseModule
   USE Atomi
   USE GENLSQ
   USE RIETV
   USE PRNUTIL
   USE VIEW
   USE Counts
   USE background
   USE variables, only:cryst,dataset,ref
   USE RRESTR
   USE constraints
   USE unit_cell
   USE atom_type_util
   USE reflection_type_util
   USE messagemod
   USE proginterface
   USE strutil
   USE profile_function
   USE setref
   USE pointmod
   USE calculate_pdp
   USE crystal_phase
       !USE Testmodel
   USE arrayutil
   USE datasetmod
   USE cryutil
!
   implicit none
   integer, intent(in) :: kaction ! 1 = Rietveld, 2 = SA
!corr   logical, intent(in) :: kprin
   logical, intent(in) :: initfo
   logical, intent(in) :: scalefo
   integer             :: nsize
   integer             :: ierp,i,nph
!
   if (kaction == 1) call write_title(lo,(/'Rietveld Refinement'/))
!
   iriet = 1
   ncytot = 0   
!  
!corr   do i=1,numphase(cryst)
!corr      call delete_ghosts(cryst(i)%at,cryst(i)%bond)
!corr   enddo
!
!  I riflessi
   if (iFirstRietveld == 0) then
       do nph=1,numphase(cryst)
          if (cryst(nph)%natoms() > 0) then
              call delete_ghosts(cryst(nph)%at,cryst(nph)%bond)
              if (kaction == 1) call calcola_occ(cryst(nph)%at,cryst(nph)%spg,cryst(nph)%cell)   ! avoid for SA because modify coordinates
              call make_constraints(cryst(nph)%at,cryst(nph)%bond,cryst(nph)%constr,cryst(nph)%spg,   &
                   cryst(nph)%cell,cryst(nph)%riding_model,cryst(nph)%adp_factor)
              call set_rcode_atoms(cryst(1)%at,cryst(1)%cell,cryst(1)%spg,cryst(1)%constr,'XYZBO',0)  ! reset ref code for atoms, shared with SA
          endif
          if (cryst(nph)%numref() == 0) then
              call cryst(nph)%make_reflections(thmin,thmax,dataset(1)%wave(:))
          endif
          call cryst(nph)%set_mac(dataset(1)%wave(1))
          cryst(nph)%riding_model = (dataset(1)%radtype == RX_SOURCE)
       enddo
   endif
!
   if (iFirstRietveld == 0) then
       call inizializza_rietveld()
   endif
!
   if (dataset(1)%datatype == POW_DATA) then
       if(iFirstRietveld == 0) then
          iFirstRietveld = 1
          call dataset(1)%set_limit()
          call dataset(1)%set_calculate()
          call Allocafuncls(ALLOCA,dataset(1)%npointsc()+cryst(1)%numres())
          call dataset(1)%make_background()

          call CalcI()  ! contains sfactor
          call setwei
          call Count0() 
          if (initfo) then
              do nph=1,numphase(cryst)
                 call init_fobs(cryst(nph)%ref)
              enddo
          endif
!         during rietveld ref. scale Fo on Fc scale to avoid scale problem switching from and to LeBail refinement
          do nph=1,numphase(cryst)
             if (scalefo .and. cryst(nph)%natoms() >0)    &
              cryst(nph)%ref%fo = cryst(nph)%ref%fo*(sum(cryst(nph)%ref%fc)/sum(cryst(nph)%ref%fo))  
          enddo
!
!         leggi direttive di rietveld solo la prima volta
          call riet_directives()
!
       else
!
!         Rebuild background if yb doesn't cover the complete range of observed
          if (size_array(dataset(1)%yb) /= dataset(1)%npoints()) then
              call dataset(1)%set_limit()
              call dataset(1)%make_background()
          endif
          call dataset(1)%set_calculate()
          call Allocafuncls(ALLOCA,dataset(1)%npointsc()+cryst(1)%numres())
          call CalcI()
          call setwei
          call calculate_powder_pattern(rfcond,dataset,cryst,ierp)
          call erreacc(0)
       endif
!
       if (kaction == 1) then
           call init_messages()
           call write_message('Rietveld Refinement',pos=1)
           call write_message('Rp=',rnum=rinfo%rp,pos=3)
           call write_message('Rwp=',rnum=rinfo%rwp,pos=2)
!
!          Disegna pattern
           if (jscreen /= 0) then
               kcount_status = 0; jjj=100
               call vedinew(8)  !!!!,1,rfcond%lim%nctot)   !!!!!!,posini,posfin)
           endif
!
           call to_coord_cry(cryst,-1,4)
       endif
   endif
!
   call defco(nsize)
   call inptf
!!!!TOFIX   call presearch
!             call abs_corr()
!             call calcola_correzioneAbs(1)
!!!!prova
!
   end subroutine init_rietveld

!-------------------------------------------------------------------

   subroutine riet_directives()
   USE comandiexpo, only: ipdir
   USE rrestr
   USE variables, only:cryst
   USE strutil
   USE commandsmod
   USE errormod
   USE constraints
   USE setref
   implicit none
   integer                 :: i
   integer                 :: icom
   integer                 :: ier
   type(restraint_type)    :: res
   character(len=80)       :: strdir,word
   integer                 :: nlongs,nlongw
   real, dimension(100)    :: vet
   integer, dimension(100) :: ivet
   integer                 :: iv
   type(constraint_type), dimension(:), allocatable :: constr
   type(error_type)        :: err
!
   icom = command_position(commands,'rietveld')
!
!  Read directives 
   if (icom /= 0) then
       do i=1,commands(icom)%ndir
          strdir = trim(commands(icom)%strdir(i))
          call Cutst(strdir,nlongs,word,nlongw)
          nlongw = min(4,nlongw)     ! bastano 4 caratteri per individuare la direttiva
          if (match_word(word,'LOAD')) then
              call read_restraints_from_file(cryst(1)%at,cryst(1)%cell,strdir,cryst(1)%res,cryst(1)%wres)
          elseif (match_word(word,'WEID')) then
              call Getnum(strdir(:nlongs),vet,iv=iv)
              if (iv > 0) cryst(1)%wres(1) = vet(1)
          elseif (match_word(word,'WEIA')) then
              call Getnum(strdir(:nlongs),vet,iv=iv)
              if (iv > 0) cryst(1)%wres(2) = vet(1)
          elseif (match_word(word,'WEIP')) then
              call Getnum(strdir(:nlongs),vet,iv=iv)
              if (iv > 0) cryst(1)%wres(3) = vet(1)
          elseif (match_word(word,'CONSTR')) then
              call constraint_from_equation(strdir,cryst(1)%at,cryst(1)%elem,constr,err)
              if (err%signal) then
                  call err%print()
              else
                  call push_back_constraint(cryst(1)%constr,constr)
              endif
          elseif (match_word(word,'PO')) then ! orientazione preferenziale
              call Getnum(strdir(:nlongs),vet,ivet,iv)
              if (iv == 3) then
                  cryst(1)%kopcorr = 1
                  cryst(1)%gcode = 1
                  cryst(1)%hklpo(:) = ivet(1:3)
              endif
          else
              call get_restraint_from_string(commands(icom)%strdir(i),cryst(1)%at,res,ier)
              if (ier == 0) then
                  call add_restraint_to_list(cryst(1)%res,res)       
              endif
          endif
       enddo
       ipdir = ipdir + commands(icom)%ndir
   endif
!
   end subroutine riet_directives

!-------------------------------------------------------------------

   subroutine exitriet()
   USE RefinecomRef
   USE PatternRef
   USE General, only: cell
   USE Molcom
   USE progtype, only:DEALLOCA
   USE genlsq
   USE grexpomod
   USE RIETV
   USE variables, only: cryst,ref,atm
   USE drawatoms, only:draw_atoms
   USE ilmviewmod
   USE enable_amb
   USE messagemod
   USE exportfiles
   USE profile_function
   !USE atom_type_util
   USE arrayutil
   USE sinterface, only: fillatom
!
   implicit none
   integer :: iExCode,interp
!
   call clear_plot(grplot,1)
   call gr_close_viewer()
!
!!   if (jscreen > 0 .and. ifAutomatic == 0) then
!!       Msg = 'Do you want to save parameters?'
!!       call MsgWinErr('WARNING',Msg,QUEST_WINDOW_YES_NO,iExCode)
!!   else
       iExCode = 1
!!   endif
!
   if (iExCode.eq.1) then
       iFirstRietveld = 1
       if (cryst(1)%natoms() > 0) then
           call cryst(1)%get_atoms(atm)
           call fillatom(cryst(1)%at)
           interp = 1
           call frm15e(cryst(1)%natoms(),interp)
           !call scrivi_lista15()
           call export_files()
       endif
       !cell(1:6) = cellph%get_par()
       call cryst(1)%get_reflections(ref)
       cell(1:6) = cryst(1)%get_cellpar()
       call scrivi_cella
!prova
       !call scrivi_lista6  ! i riflessi
!prova
   endif
!corr   call AllocaRietnew (DEALLOCA,0)
!corr   call delete_array(rapIcalc)
   call Allocafuncls(DEALLOCA)
!  
   if (jscreen > 0 .and. cryst(active_phase)%natoms() > 0) then
       call ilmview_draw(cryst(active_phase)%at,cryst(active_phase)%bond,cryst(active_phase)%elem,   &
                         cryst(active_phase)%cell,cryst(active_phase)%spg)
       call jav_set_modify(1)
   endif
!
   call clear_messages()
!
   kcount_status = 1
   iriet = 0
   call abilita_tasti('structure+pattern')
!
   end subroutine exitriet
!--------------------------------------------------------------------
   subroutine checkOP(kop,ref,cell,ier)
   !USE RefinecomRef
   USE General, only: lo
   USE unit_cell
   USE CGEOM
   USE unit_cell
   USE prog_constants
   USE reflection_type_util
   USE variables, only: cryst
   USE reflection_type_util
   implicit none
   type(reflection_type), dimension(:), intent(in) :: ref
   integer, intent(in)         :: kop
   type(cell_type), intent(in) :: cell
   integer, intent(out)        :: ier
   integer                     :: i
   real                        :: angolo
   integer                     :: nExitCode
   real, dimension(3,3)        :: gr
   character(len=300)          :: msg
!
!  Controllo per il piano di tipo 0,0,0
!
   if (sum(abs(cryst(1)%hklpo(:))) == 0) then
       ier = -1
   else
!
!     Controlla che il piano esista
      gr = cell%get_r()
      !do i=1,rfcond%lim%nrefu    
      do i=1,size(ref)
         angolo = angle_vectors(ref(i)%hkl,cryst(1)%hklpo(:),gr)
         if (angolo < 1.0e-6 .or. abs(angolo-180.) < 1.0e-06) then
             write(lo,*)i,angolo,ref(i)%hkl
             ier = 0
             return
         endif
      enddo
      ier = -1
   endif
   if (ier == -1) then
       write(msg,100) kop,cryst(1)%hklpo(:)
   100     format('Error on Preferred Orientation Plane',1x,i3,1x,'(',3(i3,1x),')')
       call MsgWinErr('Rietveld Refinement',Msg,WARN_WINDOW,nExitCode)
   endif
!
   return
   end
!--------------------------------------------------------------------
   subroutine setwei
   USE RefinecomRef
   USE GENLSQ
   USE Counts
   USE variables, only: dataset
   implicit none
   integer :: i
!
   select case (rfcond%typewei)

     case(1)
       weilsq(:dataset(1)%nc2) = 1.0

     case(2)
       do i=1,dataset(1)%nc2
          weilsq(i) = 0.0
          if (dataset(1)%y(i) > 0.0) weilsq(i) = 1.0/dataset(1)%y(i)
       enddo

     case(3)
       do i=1,dataset(1)%nc2
          weilsq(i) = 0.0
          if (dataset(1)%y(i) > 0.0) weilsq(i) = 1.0/(dataset(1)%y(i)**2)
       enddo

   end select
!
   end subroutine setwei
!----------------------------------------------------------------------------------
!----------------------------------------------------------------------------------
!
!     SUBROUTINES DI STAMPA
!
!----------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
   subroutine stampa_correlation(kpr)
   USE RefinecomRef
   USE PRNUTIL
   USE genlsq   !!!!, only:matinv,par_string
   USE type_constants, only: DP
!
   implicit none
   integer, intent(in)                                   :: kpr
   real(DP), dimension(size(matinv,1),size(matinv,1)) :: mat_corr
   integer                                               :: Nact
   integer                                               :: i,j
   integer                                               :: ier
!
   call solve_linlsq(jacob,-funcls,shiftx,ier,grad,hess,.true.,matinv)
   if (ier == 0) then
       Nact = size(matinv,1)
       forall (i=1:Nact, j=1:Nact) mat_corr(i,j) = matinv(i,j)/sqrt(matinv(i,i)*matinv(j,j))
       call stampa_matrice(mat_corr,par_string,4,kpr,'Correlation Matrix')
   endif
!
   end subroutine stampa_correlation

  !--------------------------------------------------------------------------------

   subroutine UserBack()
   USE RefinecomRef
   USE background
   USE counts
   USE RIETV
   USE variables, only: dataset
!
   implicit none
   real :: thmin0,thmax0
!
   if (dataset(1)%cond%auto) then
       call autobackground(dataset(1)%x0,dataset(1)%y,dataset(1)%yb,dataset(1)%cond%btype,    &
            dataset(1)%points,dataset(1)%thzerob,dataset(1)%coef,dataset(1)%cond%ncoef)
   else
       thmin0 = dataset(1)%xminc0()
       thmax0 = dataset(1)%xmaxc0()
       call param_back(dataset(1)%cond%btype,dataset(1)%points,dataset(1)%cond%ncoef,dataset(1)%coef,   &
                       thmin0,thmax0,dataset(1)%thzerob)
       call getbackground(dataset(1)%cond%btype,dataset(1)%yb(:dataset(1)%nc2),  &
            dataset(1)%x0,dataset(1)%coef(:dataset(1)%cond%ncoef),dataset(1)%thzerob)
   endif
!
   end subroutine Userback

  !--------------------------------------------------------------------------------

   integer function update_background(xpixel,ypixel) result(ier)
   USE RefinecomRef
   USE Molcom
   USE GREXPOMOD, only : scalex,dispx,scaley,dispy,gr_clear_all_area,iselview
   USE VIEW
   USE pointmod
   USE genlsq, only:funcls
   USE plotstyle
   USE proginterface
   USE arrayutil
   USE variables, only: dataset
!
   implicit none
   integer, intent(in) :: xpixel,ypixel
   character(len = 80) :: Message
   integer             :: i
   integer             :: ira
   integer             :: iix2, iiy2
   type(point_type)    :: padd
   integer             :: ipos
!
   if (button_ev == 1) then      ! aggiungo punti col tasto sinistro
!
!      Set background is not available
       if (.not.allocated(dataset(1)%yb)) then
           call fillcounts()
           call back_for_peaksearch(.true.)
       endif
       padd%x = (xpixel - dispx(iselview)) / scalex(iselview)
       padd%y = (ypixel - dispy(iselview)) / scaley(iselview)
       ipos = clocate(dataset(1)%points(:)%x,padd%x)
       call add_point(dataset(1)%points,padd,ipos)
       ier = 0
   elseif (button_ev == 3) then  ! elimino punti col tasto destro
       ira=5
       ier = -1
!corr       call dataset(1)%get_points(pointb)
       do i=1,dataset(1)%npoints_back()
          iix2 = dataset(1)%points(i)%x*scalex(iselview) + dispx(iselview)
          iiy2 = dataset(1)%points(i)%y*scaley(iselview) + dispy(iselview)
          if(iabs(xpixel-iix2) <= ira .and. iabs(ypixel-iiy2) <= ira) then
             call Beep(200,200)
             call del_point(dataset(1)%points,i)
!corr             npointback = npointback - 1  
!corr             call dataset(1)%set_points(pointb)
             ier = 0
             exit
          end if
       enddo
   endif
!
   if (ier == 0) then
       call UserBack()      ! ricalcola il background
!
       if (allocated(funcls)) then   ! calcola e mostra Rwp se  funcls e' allocato
           call erreacc(dataset(1)%cond%ncoef)
           write(Message,'(" Rp = ",f8.3," Rwp = ",f8.3)') rinfo%rp,rinfo%rwp
           call PrintMess(Message)
       endif
!
       if (style(STYLE_PEAKS)%vis == 1) then  ! peaksearch
           !call peaksearch(shiftzero,ier)
           call run_peaksearch()
       else
           call vedinew()
       endif
   endif
!
   end function update_background

!----------------------------------------------------------------------------------------
   subroutine abs_corr
   USE assorbimento
   USE RefinecomRef
!
!  Questa subroutine determina un'espressione della correzione di assorbimento
!  in funzione di sen(theta) utilizzando un metodo proposto sulle tabelle internazionali
!  rif. Tibbals,Acta Cryst.(1982).A38,161-163
!
   implicit none
   integer :: i,j
   real, dimension(7) :: abs
   real               :: muR
   real               :: sum_abs
!
!  Calcolo la correzione di assorbimento (abs) per theta=0,15,30,45,60,75,90 gradi
!
   muR = 0.1
   write(6,*)'muR = ',muR
   do j=1,7
      sum_abs = 0.0
      do i=1,7
         sum_abs = sum_abs + cabs(i,j)*muR**i
      enddo
      abs(j) = exp (sum_abs)
   enddo
!
!  Calcolo coefficienti coef della formula di correzione
!
   do j=1,7
      abscoef(j) = sum(abs*cinv(j,:)/3.0)
   enddo
   write(0,*)'OLD',abscoef(1:7)
!
   return
   end subroutine abs_corr

!-------------------------------------------------------------------------------------------------------------

   subroutine applyriet(numph,modzero,modcell,modop,modxyz,modfun,modwei)
!
!  Esegue determinate modifiche dopo aver modificato i parametri 
!
   USE RefinecomRef, only: rinfo
   USE View
   USE background
   USE variables, only:cryst,dataset
   USE RIETV
   USE reflection_type_util
   use crystal_phase
   use ilmviewmod
!
   implicit none
   integer, intent(in)                   :: numph
   integer, intent(in)                   :: modzero  ! modification on zero/sdisp/strans
   integer, intent(in), dimension(numph) :: modcell  ! modifica sulla cella
   integer, intent(in), dimension(numph) :: modop    ! modifica sull'orientazione preferenziale
   integer, intent(in), dimension(numph) :: modxyz   ! modifiche sui parametri strutturali x,y,z,B,och
   integer, intent(in), dimension(numph) :: modfun   ! modifica sulla funzione di profilo o sulla scala
   integer, intent(in)                   :: modwei   ! modifica sul tipo di peso
   logical                               :: calcback
   character(len=200)                    :: Msg
   logical, dimension(numph)             :: calcfc,calcrapi
   integer                               :: nph
!
   calcback = .false.
   calcfc(:) = .false.
   calcrapi(:) = .false.
!
!  Se ho modificato lo zero-shift ...
   if (modzero == 1) then
       call dataset(1)%apply_shifts()
       calcback = .true.                                     ! devo ricalcolare il background
   endif
   do nph=1,numphase(cryst)
!
!     Cell was modified
      if (modcell(nph) > 0) then
          call cryst(nph)%norm_cell()
          call cryst(nph)%calc_2theta(dataset(1)%wave(:))
          call cryst(nph)%LPcorr(dataset(1)%radtype,dataset(1)%sync,dataset(1)%nwave)
          call cryst(nph)%ABScorr(dataset(1)%nwave)
          calcfc(nph) = .true.                                   ! devo ricalcolare gli Fc
      endif
!
!     Modification on PO
      if (modop(nph) == 1) then
          cryst(nph)%kopcorr = 1                     
          call calcola_correzioneOP(cryst(nph)%ref,cryst(nph)%gpar,cryst(nph)%hklpo,cryst(nph)%cell,cryst(nph)%spg)     ! ricalcola OP
          calcrapi(nph) = .true.                                     ! devo ricalcolare i rapporti di intensita'
      endif
!
!     Modifications on structural parameters: x,y,z,B,occ ...
      if (modxyz(nph) > 0) then
          calcfc(nph) = .true.         
          calcrapi(nph) = .true.           
      endif
   enddo
!
!  Se ho cambiato il peso ..
   if (modwei == 1) call setwei()
!
!  Recompute background
   if(calcback)then
!corr      call getbackground(dataset(1)%cond%btype,dataset(1)%yb(:rfcond%lim%ncu),   &
      call getbackground(dataset(1)%cond%btype,dataset(1)%yb(:dataset(1)%nc2),   &
           dataset(1)%x0,dataset(1)%coef(:dataset(1)%cond%ncoef),dataset(1)%thzerob)
   endif
!
!  Compute Fc 
   do nph=1,numphase(cryst)
      if (calcfc(nph)) call cryst(nph)%sfactor()
      if (calcrapi(nph)) call setRInt(cryst(nph)%ref)
   enddo
!
!  Ricalcolo il profilo se ho modifica qulcosa
   if (any(modcell > 0) .or. any(modxyz > 0) .or. any(modop > 0) .or. any(modfun > 0) .or. (modzero+modwei > 0)) then
       call PrintMess('User Modification applied')
       call Count2()
       call vedinew()
       write(Msg,'(a6,f8.3,a7,f8.3)')' Rp = ',rinfo%rp,' Rwp = ',rinfo%rwp
       call PrintMess(Msg)
       do nph=1,numphase(cryst)
          if ((modxyz(nph) == 1) .and. (nph == active_phase)) then
              call ilmview_draw(cryst(active_phase)%at,cryst(active_phase)%bond,cryst(active_phase)%elem,   &
                                cryst(active_phase)%cell,cryst(active_phase)%spg)
          endif
       enddo
   endif
!
   end subroutine applyriet

!-----------------------------------------------------------------------------------------

   subroutine print_short_info(kpr)
   USE variables, only: cryst
   USE RefinecomRef, only:rinfo
   integer, intent(in) :: kpr
!corr   if (rinfo%refine_structure) then
   if (cryst(1)%is_refined) then
       write(kpr,'(a,f10.3)')'@',rinfo%rwp
   endif
   end subroutine print_short_info
!-----------------------------------------------------------------------------------------
   subroutine message_from_lsq(msgcontinue,resetcode)
   USE RefinecomRef
   USE prnutil
   USE genlsq
   USE General
   USE RIETV
   logical, intent(out) :: msgcontinue
   logical, intent(out) :: resetcode
   integer              :: exitcode
!
   msgcontinue = .false.
   resetcode = .false.
   write(Inte,'(a20,1x,i2)')'Termination criteria',termcode
   select case (termcode)
       case (0)
         if (rfcond%auto_stru .or. rfcond%auto_prof) then
             Msg = 'Automatic Refinement completed.'//char(10)//'Convergence NOT reached'
             call MsgWinErr(Inte,Msg,WARN_WINDOW,exitcode)
         else
             Msg = ' Convergence NOT reached'//char(10)//' Do you want to continue?'
             call MsgWinErr(Inte,Msg,QUEST_WINDOW_NO_YES,exitcode)
             if (exitcode == 1) msgcontinue = .true.
         endif
       case (-1,1,2,3)
         if (rfcond%auto_stru .or. rfcond%auto_prof) then
             Msg = 'Automatic Refinement completed.'//char(10)//'Convergence reached'
         else
             Msg = 'Refinement completed.'//char(10)//'Convergence reached'
         endif
         call MsgWinErr(Inte,Msg,WARN_WINDOW,exitcode)
         resetcode = .true.
       case (-2) 
         call MsgWinErr('Warning','Hessian matrix is not positive definite',WARN_WINDOW,exitcode)
         !call stampa_matrice(hess,par_string,7,6)
   end select
!
   end subroutine message_from_lsq
!-----------------------------------------------------------------------------------------
   subroutine open_rietveld_viewer()
   USE RefinecomRef
   USE grexpomod
   USE molcom  !for colors
   integer :: xdim,ydim
!
   if (index(rfcond%sprint,'W') > 0 .and. jscreen > 0) then
       call gr_create_graph(grgraph)
       call gr_set_graph(grgraph,stringx='cycles',stringy='Rwp',xscale=10000.0)
       if (.not.allocated(grplot)) then
            allocate(grplot(1))
            call gr_new_plot(grplot(1),1000)
            call gr_set_plot(grplot(1),existmarkers=.TRUE.,colorline=color_LIGHTRED,colormarkers=color_BLACK)
       endif
       call new_viewer_size(xdim,ydim)
       call gr_open_viewer(xdim,ydim,'CF vs. number of moves',newv=.true.)
       call gr_select_viewer(1)
       call gr_setbackground(color_BRIGHTWHITE)
       call gr_select_viewer(0)
   endif
!
   end subroutine open_rietveld_viewer
!--------------------------------------------------------------
   subroutine update_pattern(nls)
   USE Molcom, only:jscreen
   USE VIEW
   USE variables, only:updgr
   USE RefinecomRef, only:rfcond
   implicit none
   integer, intent(in) :: nls
   integer             :: kresto
!
   if (jscreen == 0) return      ! Rietveld senza grafica
!
!  updgr --> variabile che definisce dopo quanti cicli va rifatto il grafico
!
!  Il grafico viene aggiornato solo dopo veti_graph(2) cicli o nell'ultimo ciclo
!
   kresto = mod(nls,updgr)
   if (kresto == 0 .or. nls == rfcond%maxcy) then
       if (kzoomn) then
           call vedinew()  ! mantiene lo zoom
       else
           !call vedinew(8,1,rfcond%lim%nctot,lsetstyle=.false.)
           call vedinew(8,lsetstyle=.false.)
       endif
   endif
!
   end subroutine update_pattern


!--------------------------------------------------------------
   subroutine erreacc(Nactot)
   USE Conteggi
   USE RefinecomRef
   USE GENLSQ
   USE Counts
   USE RRESTR, only:nrestraints,resapp
   USE unit_cell
   USE variables, only: cryst,dataset
!
   implicit none
   integer, intent(in) :: Nactot
   integer             :: numrexp
   real                :: sumdeltay,sumyow,sdeltayw,sumyow1
!corr   real, dimension(rfcond%lim%ncu) :: diff
   real, dimension(:), allocatable :: diff
   integer             :: ncc
   integer             :: numres
!
!corr   ncc = rfcond%lim%ncu
   ncc = dataset(1)%nc2
!
!  Calcola contributo dei conteggi
   funcls(:ncc) = sqrt(weilsq(:ncc))*(dataset(1)%y(:ncc)-dataset(1)%yb(:ncc)-dataset(1)%yc(:ncc))
   sdeltayw  = SUM(funcls(:ncc)**2)
!
!  Calcola il contributo dei restraints se si affinano i parametri strutturali
   numres = cryst(1)%numres()
   if (numres > 0 .and. cryst(1)%is_refined) then
       call resapp(cryst(1)%at,cryst(1)%cell,cryst(1)%res,cryst(1)%wres,ncc,0,coderwi=2)  
       chisquarednow=sdeltayw+SUM(funcls(ncc+1:ncc+numres)**2)         ! Somma il contributo dovuto ai restraints
   else
       chisquarednow = sdeltayw                                       ! Chisquared
   endif
!
   sumyow    = SUM(weilsq(:ncc)*dataset(1)%y(:ncc)**2)
   sumyow1   = SUM(weilsq(:ncc)*(dataset(1)%y(:ncc)-dataset(1)%yb(:ncc))**2)
   sumdeltay = SUM(abs(dataset(1)%y(:ncc)-dataset(1)%yb(:ncc)-dataset(1)%yc(:ncc)))

   rinfo%rp     = (sumdeltay/SUM(dataset(1)%y(:ncc)))*100.               ! Rp (Conventional)
   errep1    = (sumdeltay/SUM(dataset(1)%y(:ncc)-dataset(1)%yb(:ncc)))*100.     ! Rp'(Back. subtrac.)
!
   rinfo%rwp         = sqrt(sdeltayw/sumyow) *100.                    ! Rwp (Conventional)
   errepw1        = sqrt(sdeltayw/sumyow1) *100.                      ! Rwp'(Back. subtrac.)
!corr   numrexp        = rfcond%lim%ncu - Nactot
   numrexp        = dataset(1)%nc2 - Nactot
   errexp         = sqrt(numrexp/sumyow) *100.                        ! Re (Conventional)
   errexp1        = sqrt(numrexp/sumyow1) *100.                       ! Re'(Back. subtrac.)
   chi2           = (rinfo%rwp/errexp)**2                             ! Chi-2
   allocate(diff(ncc))
   diff = weilsq(:ncc)*(dataset(1)%y(:ncc)-dataset(1)%yb(:ncc)-dataset(1)%yc(:ncc))
   rinfo%dw       = sum((diff(2:ncc) - diff(1:ncc-1))**2)/sum(diff**2)
!
   end subroutine erreacc


   subroutine setstageprof_pawley()
   USE RefinecomRef
   USE General
   USE RIETV
   USE variables, only: cryst,dataset
   USE reflection_type_util
   USE unit_cell
   USE profile_function
   implicit none
   character (len=100) :: msgpar
         integer :: i,nph
   integer :: refinefo
   type(gref_type), allocatable, dimension(:) :: gref
!
     call make_ref_groups(cryst(1)%ref,gref,sogi=0.6)
   refinefo = 0
              rfcond%maxcy = 30
   call azzeraref(.false.)                                  
!corr                 scalr%rcod = 1
       cryst(1)%scode = 1
!
   nph = 1  ! FIX THIS!
   select case(kstager)
     case(1)
       dataset(1)%zcode = 1
       msgpar = 'zero'

     case(2)
       refinefo = 4
       rfcond%maxcy = 1

     case(3)
       refinefo = 5
       rfcond%maxcy = 1

     case(4)
       dataset(1)%zcode = 1
!corr       scalr%rcod = 1
       cryst(1)%scode = 1
       msgpar = 'zero, scale'

     case(5)
       refinefo = 4
       rfcond%maxcy = 1

     case(6)
       refinefo = 5
       !if (rfcond%raction == 2) fopar(:)%rcod = 1
       rfcond%maxcy = 1

     case(7)
       dataset(1)%zcode = 1
!corr       coefbkg%rcod = 1
       dataset(1)%bcode = 1
!corr       scalr%rcod = 1
       cryst(1)%scode = 1
       msgpar = 'zero, scale, background'

     case(8)
       refinefo = 4
       rfcond%maxcy = 1

     case(9)
       refinefo = 5
       rfcond%maxcy = 1

     case(10)
!corr       scalr%rcod = 1
       cryst(1)%scode = 1
       !call cellph%set_refine(cryst(1)%spg,1)
       call cryst(1)%set_refine_cell(1)
       msgpar = 'scale, cell parameters'

     case(11)
       refinefo = 4   !!!!2
       rfcond%maxcy = 1

     case(12)
       refinefo = 5   !!!!2
       rfcond%maxcy = 1

     case(13)
       dataset(1)%zcode = 1
!corr       coefbkg%rcod = 1
       dataset(1)%bcode = 1
!corr       scalr%rcod = 1
       cryst(1)%scode = 1
       !call cellph%set_refine(cryst(1)%spg,1)
       call cryst(1)%set_refine_cell(1)
       msgpar = 'zero, background, scale, cell parameters'

     case(14)
       refinefo = 4
       rfcond%maxcy = 1

     case(15)
       refinefo = 5
       rfcond%maxcy = 1

     case(16)
!corr       scalr%rcod = 1
       cryst(1)%scode = 1
!corr       select case(rfcond%typefun)
       select case (dataset(1)%typefun(nph))
         case(PEARSON7)
            call set_rcode_function(dataset(1)%pear(1),sref='w,beta0,asym',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=dataset(1)%pvoi(1),sref='w,eta0,asym',code=1)
            !call set_rcode_function(psvoig=pvoi,sref='w',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=dataset(1)%tch(1),sref='w,x,y,asym',code=1)
       end select
       msgpar = 'scale, W, mixing, asymmetry'

     case(17)
       refinefo = 4
       rfcond%maxcy = 1

     case(18)
       refinefo = 5
       rfcond%maxcy = 1

     case(19)
       dataset(1)%zcode = 1
!corr       coefbkg%rcod = 1
       dataset(1)%bcode = 1
!corr       scalr%rcod = 1
       cryst(1)%scode = 1
       !call cellph%set_refine(cryst(1)%spg,1)
       call cryst(1)%set_refine_cell(1)
!corr       select case(rfcond%typefun)
       select case (dataset(1)%typefun(nph))
         case(PEARSON7)
            call set_rcode_function(dataset(1)%pear(1),sref='w,beta0,asym',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=dataset(1)%pvoi(1),sref='w,eta0,asym',code=1)
            !call set_rcode_function(psvoig=pvoi,sref='eta0',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=dataset(1)%tch(1),sref='w,x,y,z,asym',code=1)
       end select
       msgpar = 'zero, background, scale, cell, profile'

     case(20)
       refinefo = 4
       rfcond%maxcy = 1

     case(21)
       refinefo = 5
       rfcond%maxcy = 1

     case(22)
!corr       scalr%rcod = 1
       cryst(1)%scode = 1
!corr       select case(rfcond%typefun)
       select case (dataset(1)%typefun(nph))
         case(PEARSON7)
            call set_rcode_function(dataset(1)%pear(1),sref='u,v,w',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=dataset(1)%pvoi(1),sref='u,v,w',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=dataset(1)%tch(1),sref='u,v,w',code=1)
       end select
       msgpar = 'scale, FWHM'

     case(23)
       refinefo = 4
       rfcond%maxcy = 1

     case(24)
       refinefo = 5
       rfcond%maxcy = 1

     case(25)
       refinefo = 2
       dataset(1)%zcode = 1
!corr       coefbkg%rcod = 1
       dataset(1)%bcode = 1
!corr       scalr%rcod = 1
       cryst(1)%scode = 1
!corr       select case(rfcond%typefun)
       select case (dataset(1)%typefun(nph))
         case(PEARSON7)
            call set_rcode_function(dataset(1)%pear(1),sref='beta0,asym,u,v,w',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=dataset(1)%pvoi(1),sref='eta0,asym,u,v,w',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=dataset(1)%tch(1),sref='x,y,asym,u,v,w',code=1)
       end select
       msgpar = 'zero, background, scale, cell, profile'

     case(26)
       refinefo = 4
       rfcond%maxcy = 1

     case(27)
       refinefo = 5
       rfcond%maxcy = 1

     case(28)
       refinefo = 2
!corr       scalr%rcod = 1
       cryst(1)%scode = 1
!corr       coefbkg%rcod = 1
       dataset(1)%bcode = 1
!corr       select case(rfcond%typefun)
       select case (dataset(1)%typefun(nph))
         case(PEARSON7)
            call set_rcode_function(dataset(1)%pear(1),sref='beta0,beta1,beta2,asym',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=dataset(1)%pvoi(1),sref='eta0,eta1,eta2,asym',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=dataset(1)%tch(1),sref='x,y,z,asym',code=1)
       end select
       msgpar = 'scale, background, profile'
       if (rfcond%raction == 2) fopar(:)%rcod = 1

     case(29)
       refinefo = 4
       rfcond%maxcy = 1

     case(30)
       refinefo = 5
       rfcond%maxcy = 1

     case(31)
       refinefo = 1
       rfcond%maxcy = 1

     case(32)       
       refinefo = 2
       dataset(1)%zcode = 1
!corr       coefbkg%rcod = 1
       dataset(1)%bcode = 1
!corr       scalr%rcod = 1
       cryst(1)%scode = 1
       !call cellph%set_refine(cryst(1)%spg,1)
       call cryst(1)%set_refine_cell(1)
!corr       select case(rfcond%typefun)
       select case (dataset(1)%typefun(nph))
         case(PEARSON7)
            call set_rcode_function(dataset(1)%pear(1),sref='all',code=1)
         case(PVOIG)
            call set_rcode_function(psvoig=dataset(1)%pvoi(1),sref='all',code=1)
         case(TCHZ)
            call set_rcode_function(tchh=dataset(1)%tch(1),sref='all',code=1)
            call set_rcode_function(tchh=dataset(1)%tch(1),sref='z',code=0)
       end select
       msgpar = 'all'
       if (rfcond%raction == 2) fopar(:)%rcod = 1

     case(33)
       refinefo = 4
       rfcond%maxcy = 1

     case(34)
       refinefo = 6
       rfcond%maxcy = 1

     case(35)
       refinefo = 7
       rfcond%maxcy = 1

   end select
!
   select case (refinefo)
      case (1)   
       fopar(:)%rcod = 1

      case (2)
       do i=1,size(fopar)
          if (fopar(i)%val > 0.000) then
              fopar(i)%rcod = 1
          else
              !write(6,*)'riflesso non affinato:',i
          endif
       enddo

      case (3)
       where(fopar(:)%val <= 0.000) fopar(:)%rcod = 1

      case (4)
       do i=1,size(gref)
          if (gref(i)%code == 1) then
              fopar(gref(i)%vref(1))%rcod = 1
          endif
       enddo

      case (5)
       do i=1,size(gref)
          if (gref(i)%code >= 2) then
              fopar(gref(i)%vref(:))%rcod = 1
          endif
       enddo

      case (6)
       do i=1,size(gref)
          if (gref(i)%code >= 2 .and. gref(i)%code <=4) then
              !do j=1,gref(i)%code
              !   if (fopar(gref(i)%vref(j))%val > 0.00)fopar(gref(i)%vref(j))%rcod = 1
              !enddo
              fopar(gref(i)%vref(:))%rcod = 1
          endif
       enddo

      case (7)
       do i=1,size(gref)
          if (gref(i)%code >= 5) then
              fopar(gref(i)%vref(:))%rcod = 1
          endif
       enddo

   end select
!
   end subroutine setstageprof_pawley
!--------------------------------------------------------------------
   subroutine calcola_correzioneAbs(ref)
!
!  Calcolo dellla correzione di assorbimento
!  abscoef = coefficienti calcolati nella subroutine abs_corr
!
   USE RefinecomRef
   USE reflection_type_util
   USE trig_constants
   USE variables, only: dataset
   implicit none
   type(reflection_type), dimension(:), allocatable, intent(inout) :: ref
   integer :: i
   integer :: kal
   integer, dimension(7) :: mvet = (/0,2,4,6,8,10,12/)
!
   LOOP_KALPHA : do kal=1,dataset(1)%nwave
      do i=1,numrefl(ref)
         ref(i)%ab = sum(abscoef*sin(ref(i)%tthd(kal)*dtor)**mvet)
      enddo
   enddo LOOP_KALPHA
!
   end subroutine calcola_correzioneAbs


!-----------------------------------------------------------------

   subroutine frietgauss(npar,iflag,nls,err)
   USE RefinecomRef
   USE Molcom, only : viewerOpened, jscreen, ModelToCompare
   USE GENLSQ
   USE variables, only:cryst
   USE General, only:lo
   USE RIETV
   USE grexpomod
   USE rrestr
   USE atom_type_util
   USE ilmviewmod
   USE messagemod
   USE stat, only:Rvalue
   USE constraints
   USE type_constants, only: DP
   USE cryutil
   USE crystal_phase
   USE arrayutil
!
   implicit none
   integer, intent(in)             :: npar    ! numero di parametri da affinare
   integer, intent(in)             :: iflag   ! tipo di operazione da eseguire
   integer, intent(in), optional   :: nls     ! num. ciclo corrente
   integer, intent(out), optional  :: err
   real                            :: rbragg, rf
   integer                         :: ierp,nph,nberr,i,iersd
   !integer, dimension(:), allocatable :: bvet
   !logical, dimension(:), allocatable :: bconstr,outrange
   logical, dimension(:), allocatable :: outrange
!
   ierp = 0
   select case (iflag)

     case (-1)                    ! calcolo iniziale delle funzioni
       call calcola_profilo(npar,ierp)

     case (0)                     ! calcola jacobiano
       call rietjacobian(ierp)

     case (1)                     ! applica gli shift e calcola il profilo
       xpar = xparold + step_length*shiftx
       call set_parameter(npar,xpar)
       call calcola_profilo(npar,ierp)
      !     if (ierprof /= 0) write(0,*)'IERProf=',ierprof

     case (2)                     ! stampa e aggiorna la grafica
       if (index(rfcond%sprint,'V') > 0) call print_all_parameters(lo,rfcond,cryst(1)%ref)
       if (index(rfcond%sprint,'M') > 0) call stampa_matrici_lsq(lo)
       call write_message('Cycle # ',inum=ncytot,pos=4)
       call write_message('Rp =    ',rnum=rinfo%rp,pos=5)
       call write_message('Rwp =   ',rnum=rinfo%rwp,pos=6)
       call get_rbragg(cryst(1)%ref,cryst(1)%numref(),rbragg,rf)
       call write_message('R-Bragg =   ',rnum=rbragg,pos=7)
       if (cryst(1)%is_known()) then
           call write_message('R-F =   ',rnum=Rvalue(cryst(1)%ref%fv,cryst(1)%ref%fo),pos=8)
       else
           call write_message('R-F =   ',rnum=rf,pos=8)
       endif
       call progress_bar_update(cval=(ncurrent_cycle-1)*kstagemax*rfcond%maxcy+(kstager-1)*rfcond%maxcy+nls,   &
                                fval=rfcond%maxcy*kstagemax*rfcond%ncauto)
       do nph=1,numphase(cryst)
          if (cryst(nph)%is_known() .and. cryst(nph)%is_refined) then
              call to_coord_cry(cryst(nph),-1)
              call write_message_found(9+nph-1)
          endif
       enddo
!
!      Output grafico su viewer secondario
       if (index(rfcond%sprint,'W') > 0 .and. jscreen > 0) then
!           
!          Aggiorna sempre punti da plottare
           call gr_add_point_plot(grplot(1),real(ncytot),rinfo%rwp)
!       
!          Se il viewer è aperto esegui plot
           if (viewerOpened == 1) then
               call gr_select_viewer(1)
               call gr_clear_area(0.0,1.0,0.0,1.0)
               call gr_area(0.08, 0.99, 0.10, 0.99)
               call gr_draw_graph(grgraph,grplot)
               call zflush()
               call gr_select_viewer(0)
           endif
       endif
!
       call update_pattern(ncytot)
       if (cryst(active_phase)%is_refined) then
           call ilmview_draw(cryst(active_phase)%at,cryst(active_phase)%bond,cryst(active_phase)%elem,   &
                             cryst(active_phase)%cell,cryst(active_phase)%spg)
       endif
!
     case (3)                     
       nph = 1
       call new_array(outrange,npar)
       call check_btcryst(outrange,0.0,1000.,.false.,nberr) ! check thermal factors
       if (nberr > 0) ierp = 1
       if (rfcond%raction == 2) call check_fo(xpar,0,ierp)

     case (4)                     ! controlla e blocca i parametri
       call new_array(outrange,npar)
       call check_btcryst(outrange,0.0,1000.,index(rfcond%sprint,'V') > 0,nberr)
       if (nberr > 0) ierp = 1
       if ((index(rfcond%sprint,'E') > 0) .or. (index(rfcond%sprint,'S') > 0 .and. numphase(cryst) > 0)) then
           call compute_sd(iersd)
       endif
!
       step_length = 0.0
       xpar = xparold + step_length*shiftx
       call set_parameter(npar,xpar)
       call calcola_profilo(npar,ierp)
       call unset_param(outrange)

     case (5)                     ! save parameters
       xparold(:) = xpar(:)
       do nph=1,numphase(cryst)
          if (nconstraints(cryst(nph)%constr) > 0) then     ! se ci sono constraints e' necessario salvare anche gli atomi
              call copy_atoms(cryst(nph)%atmc,cryst(nph)%at)
          endif
       enddo

   end select
   if (present(err)) err=ierp
!
   end subroutine frietgauss
!-----------------------------------------------------------------
   subroutine frietlm(mfun,npar,x,fvec,fjac,ldfjac,iflag)
   USE Conteggi
   USE RefinecomRef
   USE Molcom, only : viewerOpened
   USE GENLSQ
   USE view
   USE grexpomod
   USE General, only:lo
   USE messagemod
   USE type_constants, only: DP
   USE variables, only: cryst
   USE rietv
!
   implicit none
   integer,                   intent(in)              :: mfun   !numero di funzioni
   integer,                   intent(in)              :: npar   !numero di parametri (n<=m)
   integer,                   intent(in)              :: ldfjac
   integer,                   intent(inout)           :: iflag
   real(DP), dimension(mfun),        intent(out) :: fvec   !funzioni che entrano nella somma di quadrati
   real(DP), dimension(ldfjac,npar), intent(out) :: fjac   !jacobiano di fvec
   real(DP), dimension(npar),        intent(in)  :: x      !i parametri da affinare
   integer :: ierp
!!!!!!!   character(80)                                      :: string
!
   select case (iflag)

     case (0)
       nclev = nclev + 1
       if (nclev == 0) then
       elseif (nclev <= rfcond%maxcy) then
           ncytot = ncytot + 1
           shiftx = xpar-xparold
           if (index(rfcond%sprint,'V') > 0) call print_all_parameters(lo,rfcond,cryst(1)%ref)
           !write(string,'(a6,f8.3,a9,f8.3,7x,a8,i4)') ' Rp = ',rinfo%rp,' - Rwp = ',rinfo%rwp,'Cycle # ',ncytot
           !call scrper(string,nclev,rfcond%maxcy)
           call progress_bar_update(cval=nclev,fval=rfcond%maxcy)
!
!          Output grafico su viewer secondario
           if (index(rfcond%sprint,'W') > 0) then
!           
!              Aggiorna sempre punti da plottare
               call gr_add_point_plot(grplot(1),real(ncytot),rinfo%rwp)
!       
!              Se il viewer è aperto esegui plot
               if (viewerOpened == 1) then
                   call gr_select_viewer(1)
                   call gr_clear_area(0.0,1.0,0.0,1.0)
                   call gr_area(0.08, 0.99, 0.10, 0.99)
                   call gr_draw_graph(grgraph,grplot)
                   call zflush()
                   call gr_select_viewer(0)
               endif
           endif
           call vedinew()
           if (nclev == rfcond%maxcy) iflag = -1
       endif
       chisquared_old = chisquarednow

     case (1)                  ! calcola le funzioni in x
       call set_parameter(npar,x)
       call calcola_profilo(npar,ierp)
       fvec = funcls
       xparold = xpar

     case (2)                  ! calcola lo jacobiano
       call rietjacobian(ierp)
       fjac = jacob

   end select
!
   end subroutine frietlm

!----------------------------------------------------------------------------------

   subroutine calcgradn(np,x,epsfcn)
   USE RefinecomRef
   USE nr
   USE PatternRef, only : iriet
   USE GENLSQ
   !USE nrtype
   USE type_constants
   !USE variables, only: cryst
   USE rietv
   implicit none
        INTERFACE
                FUNCTION func(x)
                USE nrtype; USE nrutil
                IMPLICIT NONE
                REAL(SP), DIMENSION(:), INTENT(IN) :: x
                REAL(SP), DIMENSION(size(x)) :: func
                END FUNCTION func
        END INTERFACE
!
   integer, intent(in) :: np
   real(DP), intent(in)    :: epsfcn
   real(DP), dimension(np)  :: x
   real                :: temp
   real(DP)       :: h
   integer             :: i
   integer, parameter  :: npoin=80
   integer(i4b), parameter  :: ncof=20  !!!!15
   real, dimension(ncof)  :: parceb
   real(sp) :: aa, bb
   integer :: ierp
!
   chisquared_old = chisquarednow
   nparfun = np
   write(0,'(/,a21,i3,2x,a6,e15.7)')'Gradiente al ciclo n.',ncytot,'chi2 =',chisquared_old
   write(0,*)'   Parametro   chi2 stimato       der. numerica      der. analitica'
   do i=1, np
      temp = x(i)
      h= abs(sqrt(epsfcn)*max(temp,1.0))
      ippar = i
!
      aa = temp-npoin*h
      bb = temp+npoin*h
      parceb = chebft(aa,bb,ncof,func)
      write(0,'(1x,a13,3(e15.7,4x))')par_string(i),chebev(aa,bb,parceb,temp), &
                                     chebev(aa,bb,chder(aa,bb,parceb),temp),grad(i)
      x(i) = temp
   enddo
   chisquarednow = chisquared_old
   call set_parameter(np,x)
   call calcola_profilo(np,ierp)
! scommenta anche in func per affinamento su F
!!! call calcerrefs(npref,kcscal,scalr%val)
!
   end subroutine calcgradn
!----------------------------------------------------------------------------------
   function func(x)
   USE RefinecomRef
   USE nrtype
   USE GENLSQ
   !USE variables, only: cryst
   USE rietv
!
   implicit none
   real(sp), dimension(:), intent(in) :: x
   real(sp), dimension(size(x)) :: func
   integer :: i
   integer :: nx
   integer :: ierp
!
   nx = size(x)
   do i=1,nx
      xpar(ippar) = x(i)
      call set_parameter(nparfun,xpar)
      call calcola_profilo(nparfun,ierp)
! scommenta anche in calcgradn per affinamento su F
! call calcerrefs(npref,kcscal,scalr%val)
      func(i) = chisquarednow
   !  write(0,*)xpar(ippar),chisquarednow
   enddo
!
   end function func
!----------------------------------------------------------------------------------
   FUNCTION func1(x)
   USE nrtype
   USE GENLSQ
   USE RefinecomRef
   !USE variables, only: cryst
   USE rietv
   IMPLICIT NONE
   REAL(SP), INTENT(IN) :: x
   REAL(SP) :: func1
   integer :: ierp
!
   xpar(ippar) = x
   call set_parameter(nparfun,xpar)
   call calcola_profilo(nparfun,ierp)
   func1 = chisquarednow
!
   END FUNCTION func1
!----------------------------------------------------------------------------------
   subroutine calcgrad2(np,x,h)
   USE nr
   USE RefinecomRef
   USE GENLSQ
   USE type_constants, only: DP
   !USE variables, only: cryst
   USE rietv
!
   implicit none
        INTERFACE
                FUNCTION func1(x)
                USE nrtype
                IMPLICIT NONE
                REAL(SP), INTENT(IN) :: x
                REAL(SP) :: func1
                END FUNCTION func1
        END INTERFACE
   integer, intent(in) :: np
   real, intent(in)    :: h
   real(DP), dimension(np) :: x
   integer             :: i
   real                :: dx
   real                :: err
   integer             :: ierp
!
   chisquared_old = chisquarednow
   nparfun = np
   write(0,*)'Gradiente al ciclo n.',ncytot,'eps =',h
   write(0,*)'   Parametro           der. numerica        der. analitica'
   do i=1, np
      ippar = i
      dx = dfridr(func1,real(x(i)),h,err)
      write(0,'(4x,a13,3e15.7)')par_string(i),dx,-2.0*grad(i),err
   enddo
   chisquarednow = chisquared_old
   call set_parameter(np,x)
   call calcola_profilo(np,ierp)
!
   end subroutine calcgrad2
!----------------------------------------------------------------------------------
!!!!!   subroutine calcdern(np,h)
!!!!!   USE nr
!!!!!   USE RefinecomRef
!!!!!   USE GENLSQ
!!!!!!
!!!!!   implicit none
!!!!!        INTERFACE
!!!!!                FUNCTION func2(x)
!!!!!                USE nrtype
!!!!!                IMPLICIT NONE
!!!!!                REAL(SP), INTENT(IN) :: x
!!!!!                REAL(SP) :: func2
!!!!!                END FUNCTION func2
!!!!!        END INTERFACE
!!!!!   integer :: i
!!!!!   real, intent(in)    :: h
!!!!!   real                :: err
!!!!!   integer, intent(in) :: np
!!!!!   real                :: dx
!!!!!!
!!!!!   do i=1,20
!!!!!      ippar = i
!!!!!      dx = dfridr(func2,real(xpar(np)),h,err)
!!!!!      write(0,'(4x,i3,3e15.7)')i,dx,derxph(i,1,1)*0.5/refph(1,i)%fc,err
!!!!!   enddo
!!!!!!
!!!!!   end subroutine calcdern
!----------------------------------------------------------------------------------
!!!!!   function func2(x)
!!!!!   USE nrtype
!!!!!   USE GENLSQ
!!!!!   USE RefinecomRef
!!!!!   USE variables, only:ref
!!!!!   IMPLICIT NONE
!!!!!   REAL(SP), INTENT(IN) :: x
!!!!!   REAL(SP) :: func2
!!!!!!
!!!!!   xpar(1) = x
!!!!!   call set_parameter(1,xpar)
!!!!!   func2 = ref(ippar)%fc
!!!!!!
!!!!!   end function func2
!----------------------------------------------------------------------------------
!corr   subroutine controlla2theta()
!corr   USE RefinecomRef
!corr   USE variables, only: ref
!corr!
!corr   implicit none
!corr   integer :: i
!corr!
!corr   do i=2,size(ref)
!corr      if (ref(i)%tthd < ref(i-1)%tthd) then
!corr          write(0,*) 'WARN THETA',i,ref(i)%tthd,i-1,ref(i-1)%tthd
!corr      endif
!corr   enddo
!corr!
!corr   end subroutine controlla2theta
!corr

!--------------------------------------------------------------------------------

   subroutine genera_lista_restraints(code)
   USE rrestr
   USE variables, only: cryst
   USE unit_cell
   USE errormod
   USE OBmodule
   USE atom_type_util
   USE arrayutil
   implicit none
   integer, intent(in) :: code
   type(restraint_type), dimension(:), allocatable :: reslist
   integer, dimension(:), allocatable              :: vetres
   integer                                         :: i
   integer                                         :: nrl
   integer                                         :: pos
   integer, parameter :: BOND_TAB=1,CURRENT=2,MOL_MEC=3
   integer :: gen_mode = 1
   type(atom_type),  dimension(:), allocatable :: atom
   type(error_type) :: err
   integer :: nuser
!
   call copy_atoms(atom,cryst(1)%at)
   if (gen_mode == MOL_MEC) then
       call OBOptimizeGeometry(atom, cryst(1)%bond, cryst(1)%cell, err)
   endif
!
   call conn_to_restraints(cryst(1)%bond,atom,cryst(1)%cell,code,reslist,gen_mode)
!
   if (gen_mode == MOL_MEC) then
!
!      set current value from non-optimized model
       do i=1,nrestraints(reslist)
          call set_value_for_restraint(reslist(i),cryst(1)%at,cryst(1)%cell)
       enddo
   endif
!
!  Genera vettore vetres che dice quali checks sono attivi
   nrl = nrestraints(reslist)
          !call print_restraints_all(0,reslist,atm,cellt)
   if (nrl > 0) then
       allocate(vetres(nrl))
       vetres(:) = 0
!
!      Now update array resvr containing only the active restraints
!corr       if (nrestraints(resvr) > 0) then
       if (cryst(1)%numres() > 0) then
!corr           do i=1,nrestraints(resvr)
           do i=1,cryst(1)%numres()
              if (cryst(1)%res(i)%code /= code) cycle
              pos = restraint_position(reslist,cryst(1)%res(i)) 
              if (pos > 0) then
                  vetres(pos) = 1
                  reslist(pos) = cryst(1)%res(i)      ! usa target e esd del restraint in memoria
              else
!!!!TODO: test plane
                  if (code == RESPLANE) then   ! check if restraints is compatible with num. of atoms
                      if (any(cryst(1)%res(i)%na > numatoms(atom))) then
                          cryst(1)%res(i)%code = DELETED_RES  ! mark as deleted
                      endif
                  else
                      cryst(1)%res(i)%code = DELETED_RES  ! mark as deleted
                  endif
              endif
           enddo
           call delete_restraints_code(cryst(1)%res)
       endif
   endif
!
!  For plane set by user to_genera is called from C
   nuser = 0
   do i=1,nrestraints(cryst(1)%res)
      if (cryst(1)%res(i)%code == RESPLANE) then
          nuser = nuser + 1
          call add_restraint_to_list(reslist,cryst(1)%res(i))
      endif
   enddo
   if (nuser > 0) then
       nrl = nrestraints(reslist)
       call resize_array(vetres,nrl)
       vetres(nrl-nuser+1:nrl) = 1  ! user restraints are active
                                    ! TODO: user restraints desappear if deactivated
   endif
!
!  aggiunte da mr 240709
   call c_deallocate_res()
   call passa_nres(nrl)
   if (nrl > 0) then
       call passa_vetres(nrl, vetres)
       do i=1, nrl
          call put_info_newrest(i-1,reslist(i)%code,reslist(i)%targ,reslist(i)%sigma,   &
                                reslist(i)%val,size(reslist(i)%na),reslist(i)%na)
       end do
   endif
!
   end subroutine genera_lista_restraints
!-----------------------------------------------------------------------
   subroutine init_newrestraints(cwqq)          !  aggiunta da mr 240709
!corr   USE RRESTR
   USE variables, only: cryst
   real    :: cwqq(3)  
   integer :: i      
!      
   do i=1,3
      cwqq(i) = cryst(1)%wres(i)
   enddo
!
   return
   end subroutine init_newrestraints

!-----------------------------------------------------------------------

   subroutine get_restraints_from_c(active,code,val,targ,esd, nna, na) 
!
!  Function called by C every time that a restraint is selected
!
   USE rrestr
   implicit none
   integer, intent(in)              :: code
   integer, intent(in)              :: nna
   integer, intent(in)              :: na(*)
   integer, intent(in)              :: active
   type(restraint_type)             :: restr
   real, intent(in)                 :: val, targ, esd
!
   restr%code  = code
   restr%val   = val
   restr%targ  = targ
   restr%sigma = esd
   allocate(restr%na(nna))
   restr%na(1:nna) = na(1:nna)
   call get_restraints_from_graphic(active,restr)  ! update global variable resvr
!
   end subroutine get_restraints_from_c

!--------------------------------------------------------------

   subroutine get_restraints_from_graphic(code,restr)
   USE rrestr
   USE variables, only: cryst
   implicit none
   integer, intent(in)              :: code
   type(restraint_type), intent(in) :: restr
   integer                          :: pos
!
!
!  prendo la posizione del restraint restr nella lista resvr
   pos = restraint_position(cryst(1)%res,restr) 
   if (code == 1) then   ! check attivo
       if (pos == 0) then
!
!          aggiungi il restraint se non esiste
           call add_restraint_to_list(cryst(1)%res,restr)
       else
!
!          se esiste aggiorna esd e target
           if (restr%code < 3) then
               cryst(1)%res(pos)%sigma = restr%sigma
               cryst(1)%res(pos)%targ = restr%targ
           else
               cryst(1)%res(pos)%sigma = restr%sigma
           endif
       endif

   else                  ! check disattivato
!
!      rimuovi il restraint dalla lista resvr
       if (pos > 0) call delete_restraints(cryst(1)%res,pos)
   endif
       !call print_restraints_all(0,resvr,atm,set_cell_type(cell))
!
   end subroutine get_restraints_from_graphic

!-----------------------------------------------------------------------
   subroutine finishnewrestraints(iExitC, cwqq) !  aggiunta da mr 240709
   USE RRESTR
!corr   USE rietv
   USE variables, only: cryst
   implicit none
   integer, intent(in)              :: iExitC
   integer                          :: i
   real, intent(in)                 :: cwqq(3)
!
!   iexitC = 0 (Cancel); = 1 (OK)
!
   if (iExitC.eq.1) then
       do i=1,3
          cryst(1)%wres(i) = cwqq(i)
          call set_weight(cryst(1)%res,i,cryst(1)%wres(i))
       enddo
   endif
!
   return
   end subroutine finishnewrestraints
! ----------------------------------------------------------------------
   subroutine LeggiRestraints(nomefile)         !  aggiunta da mr 240709
   USE rrestr
   USE rietv
   USE variables, only: cryst
   USE unit_cell
   implicit none
   character(len=*), intent(in)     :: nomefile
!
   call read_restraints_from_file(cryst(1)%at,cryst(1)%cell,nomefile,cryst(1)%res,cryst(1)%wres)
!
   return
   end subroutine LeggiRestraints
! ----------------------------------------------------------------------
   subroutine ScriviRestraints(nomefile, cwqq)   !  aggiunta da mr 240709
!corr   USE rietv
   USE variables, only: cryst
   USE rrestr
   implicit none
   character(len=*), intent(in)     :: nomefile
   real, intent(in)                 :: cwqq(3)
!
   call write_restraints_on_file(nomefile,cryst(1)%res,cryst(1)%at,cwqq)
!
   end subroutine ScriviRestraints
! ----------------------------------------------------------------------

   subroutine get_newrestraint_from_c(active,code,val,targ,esd, nna, na)
!  aggiunta da mr 250709
   USE rrestr
   USE rietv
   USE variables, only: cryst
   USE unit_cell
   implicit none
   integer, intent(in)              :: code
   integer, intent(in)              :: nna
   integer, intent(in)              :: na(*)
   integer, intent(in)              :: active
   type(restraint_type)             :: restr
   real, intent(in)                 :: val, targ, esd
   integer                          :: pos
!
   restr%code  = code
   restr%val   = val
   restr%targ  = targ
   restr%sigma = esd
   allocate(restr%na(nna))
   restr%na(1:nna) = na(1:nna)
   !write(0,*)'code, nna, na ',restr%code, nna, (restr%na(j),j=1, nna)
!
!  aggiungi il restraint solo se non esiste già
   pos = restraint_position(cryst(1)%res,restr) 
   if (pos == 0) then
       call set_value_for_restraint(restr,cryst(1)%at,cryst(1)%cell)
       call add_restraint_to_list(cryst(1)%res,restr)
   endif
!
   end subroutine get_newrestraint_from_c

!-----------------------------------------------------------------------
   subroutine get_resttodel_from_c(ind_l,active,code,val,targ,esd, nna, na)
!  aggiunta da mr 250709
   USE rrestr
   USE variables, only: cryst
!corr   USE rietv
   implicit none
   integer, intent(in)              :: ind_l
   integer, intent(in)              :: code
   integer, intent(in)              :: nna
   integer, intent(in)              :: na(*)
   integer, intent(in)              :: active
   real, intent(in)                 :: val, targ, esd
   type(restraint_type)             :: restr
   integer                          :: pos
!
   restr%code  = code
   restr%val   = val
   restr%targ  = targ
   restr%sigma = esd
   allocate(restr%na(nna))
   restr%na(1:nna) = na(1:nna)
   pos = restraint_position(cryst(1)%res,restr) 
   if (pos > 0) call delete_restraints(cryst(1)%res,pos)
!
   end subroutine get_resttodel_from_c
!-----------------------------------------------------------------------
   subroutine PassaWeight(w, icode)
   USE variables, only: cryst
!corr   USE RRESTR
!
   implicit none
   integer, intent(in)              :: icode
   real, intent(in)                 :: w
!
!  errore- se entri ed esci da rietveld dopo aver settato restraint su piano
!  arriva icode = 0. Da capire perche
   cryst(1)%wres(icode) = w
!
   return
   end subroutine PassaWeight
! ----------------------------------------------------------------------

   integer function num_restraints(code)
   USE rrestr
   USE variables, only: cryst
   integer, intent(in) :: code
   num_restraints = nrestraints(cryst(1)%res,code) 
   end function num_restraints

! ----------------------------------------------------------------------

!   subroutine test_esd()
!   USE tempp
!   implicit none
!   real    :: std
!   integer :: i
!   character(len=100) :: string
!!
!        std = 17.456
!      string = string_esdt(std,std)
!      write(0,*)'std=',std,trim(string)
!        return
!   do i=1,100
!      call random_number(std)
!      std = std*100
!      string = string_esdt(std,std)
!      write(0,*)'std=',std,trim(string)
!   enddo
!
!   end subroutine test_esd
