MODULE sannel
USE simulated_anneal, only:SimAnn_Conditions_type
!corr USE atom_basic, only: atom_type
USE rrestr, only: restraint_type
!corr USE constraints, only: constraint_type
!corr USE anti_bump, only: bump_settings
USE reflection_type_util, only: gref_type   !corr , reflection_type
USE progtype, only: opti_info_type, param_type
USE timer, only: timer_type
!corr use fastfcal, only: sfactor_type

implicit none

type shift_atom_type
  integer, dimension(3) :: rcod ! codice di affinamento  
end type

integer, parameter :: THERMALF_TYPE = 1
integer, parameter :: PO_TYPE = 2
integer, parameter :: OTHERS_TYPE   = 3
integer, parameter :: EXT_TYPE = 4
integer, parameter :: INT_TYPE = 5

enum, bind(c) 
  enumerator :: SA_LOCAL=1,BBBC,SA_GLOBAL,HARMONYS
endenum

integer, parameter :: RWPC_COSTF=1,  &  ! CF compacted pattern
                      CHI2I=2,       &  ! CF intensita'
                      CHI2FACT=3,    &  ! CF fattori di struttura (tutti i riflessi)
                      CHI2I_NEW=4,   &  ! CF intensita' integrate
                      RWP_COSTF=5,   &  ! CF profilo 
                      CHI2GRU=6,     &  ! CF intensities groups
                      CHI2_COSTF=7,  &  ! CHI2 pattern
                      CHI2I_SD=8        ! CF fattori di struttura (sing e doppietti)
character(len=*), dimension(8), parameter :: COSTF_NAME = ['R weighted profile','R intensities     ','R structure factor',   &
                                                           'Chisquared 1      ','Rwp 1             ','R groups          ',   &
                                                           'Chisquared 3      ','Chisquared 4      ']
integer, dimension(2), parameter :: XPD_COSTF = [RWPC_COSTF,CHI2I] ! CF for powder, XDP_COSTF(1) is the default
integer, dimension(2), parameter :: XSE_COSTF = [CHI2FACT,CHI2I]   ! CF for single crystal and electrons, XSE_COSTF(1) is the default

type(shift_atom_type), dimension(:), allocatable :: shift_atom         ! gli shift sugli atomi
type(shift_atom_type), dimension(:), allocatable :: shift_atom_copy    ! gli shift sugli atomi (copia)
type(opti_info_type)                             :: ginfo              ! informazioni generali sulla procedura
type(opti_info_type)                             :: goptim             ! informazioni generali sulla procedura
type(param_type), dimension(:), allocatable      :: gpar               ! i parametri dell'algoritmo di SA
real                                             :: shiftatomval       ! shift applicato agli atomi
integer                                          :: numcyc_calc        ! calculated number of CF evaluation for run
real                                             :: rwden              ! denominatore funzione costo
real, dimension(:), allocatable, private         :: intoss,intcal      ! intensita' per funzione costo CHI2I
!corr type(reflection_type), dimension(:), allocatable :: refi               ! copia di tutti i riflessi
#if defined(DISTRIB)
logical, private                                 :: print_tb = .false. ! stampa info per tabella riepilogativa
#else
logical, private                                 :: print_tb = .true.  ! stampa info per tabella riepilogativa
#endif
integer                                          :: ncuold 
!corr real, dimension(:), allocatable :: ycossb,ycountsa
!corr real, dimension(:), allocatable :: ycountsa
real :: sumycossb,sumintoss
!corr real, dimension(:), allocatable :: ycountonew,weichi
!corr real, dimension(:), allocatable :: weichi,ycossb
integer :: nco_new
!corrtype(rparam_type)                                  :: gpo_sav
!corr real                                  :: gpo_sav
real, parameter, private :: DISTDOC = 1.0   ! distanza limite per la d.o.c.
real, parameter, private :: DISTOCC = 0.9   ! distanza limite per atomi in posizione speciale
character(len=*), dimension(4), parameter :: algoname_long = (/'Simulated Annealing                             ',  &
                                                               'Hybrid Big Bang - Big Crunch combined with GRASP',  &
                                                               'Simulated Annealing                             ',  &
                                                               'Harmony Search                                  '/)  
character(len=*), dimension(4), parameter :: algoname_short= (/'SA      ','GHBB-BC ','SA      ','HS      '/)
logical :: map_enabled = .true.
real :: sasogmap
real :: radmap
!
! restraints variables
type(restraint_type), dimension(:), allocatable  :: bumptab
!corr type(constraint_type), dimension(:), allocatable :: sconstr

real, parameter :: SASIGD = 0.2
real, parameter :: SAWEID = 100
real, parameter :: SHIFT_DEF = 0.5

!corr type(atom_type), dimension(:), allocatable :: atom0

type(gref_type), dimension(:), allocatable :: sagref
!corr real, dimension(:,:,:), allocatable :: asymtab
integer, parameter :: SATIME_CF=1, SATIME_MOVES=2, SATIME_GRAPH=3, SATIME_RUN=4, SATIME_TOT=5
type(timer_type), dimension(5) :: timesa
character(len=*), private, parameter :: STRTAG='@@'
!corr type(sfactor_type), dimension(:), allocatable :: sfactorc !sfactor e una sua copia
!corr integer, dimension(:), allocatable  :: kpscat

CONTAINS

   subroutine sa_prelim(gopti,sc,bcond,read_dir,init_cond,saerr)
   USE connect_mod
   USE variables, only:cryst, dataset
   USE datasetmod
   USE General, only:lo,ifAutomatic
   USE simulated_anneal
   USE RIETV
   USE atom_type_util
   USE reflection_type_util
   USE view
   USE Conteggi, only:npunti
   USE PatternRef, only: kcount_status
   USE ilmviewmod
   USE messagemod
   USE Molcom, only:jscreen
   USE bb_bc
   USE compare_struct
   USE plotstyle
   USE anti_bump
   USE errormod
   USE constraints
   USE commandsmod
   USE mpi_prog
   USE fragmentmod
   USE rrestr
   USE crystal_phase
   USE cryutil
   type(opti_info_type), intent(inout)       :: gopti    
   type(SimAnn_Conditions_type), intent(out) :: sc                     
   type(bb_bc_condition), intent(out)        :: bcond                     
   logical, intent(in)                       :: read_dir
   logical, intent(in), optional             :: init_cond
   type(error_type), intent(out)             :: saerr
   logical                                   :: initc
   integer                                   :: i,j,nph
   real                                      :: resmaxg
   integer                                   :: ishifta
   real                                      :: resmin
   real, parameter                           :: RESLIMIT = 2.0
   type(error_type)                          :: errdir
   integer, dimension(1)                     :: pfrag
   integer :: icpos   !,posb
   integer :: numat    !!!!!,numfrag
   real, dimension(:), allocatable :: resv
!
   call saerr%reset()
!   
   initc = .true.
   if (present(init_cond)) initc = init_cond
!
   call init_rietveld(2,initc,.false.)
   if (jscreen > 0 .and. dataset(1)%datatype /= SCRYS_DATA) then
       call jav_set_modify(3)
       kcount_status = 1; 
       call set_plot_style(8)
       style(STYLE_CALC)%vis = 0
       style(STYLE_DIFF)%vis = 0
       call vedinew(8,1,npunti,lsetstyle=.false.)
   endif      
!
!  Inizializza
   if (initc) then ! if false the conditions are initialize externally
       call sa_set_default(sc)
       call bb_bc_initialize(bcond)
       gopti%cftype = set_default_cfunction(1)  ! set only cost function
   endif
   do i=1,numphase(cryst)
      call cryst(i)%clear_restraints()
      call reset_constraints(cryst(i)%sconstr)
   enddo
!
   if (read_dir) then
!
!      Read restraints from directives. Distance restraints can break bonds
       icpos = command_position(commands,'sannel')
       if (icpos > 0) then
           call read_restraints_from_directives(commands(icpos)%strdir,commands(icpos)%ndir,cryst(1)%at,cryst(1)%res,SASIGD,SAWEID)
       endif
!corr       do i=1,nrestraints(cryst(1)%res)
!corr          if (cryst(1)%res(i)%code == RESDIST) then
!corr              posb = bond_position(cryst(1)%bond,cryst(1)%res(i)%na(1),cryst(1)%res(i)%na(2))
!corr              if (posb > 0) call remove_bond(cryst(1)%bond,posb)
!corr          endif
!corr       enddo
   endif
!
!!!!!FIX THIS FOR MULTI
   numat = cryst(1)%natoms()
   call create_bump_table(cryst(1)%at,bumptab)
   call reallocate_shift(shift_atom,numat,.false.)
   do i=1,numat
      shift_atom(i)%rcod = 0
   enddo
   shiftatomval = SHIFT_DEF
!
   do nph=1,numphase(cryst)
      numat = cryst(nph)%natoms()
      call cryst(nph)%init_conn()
      call cryst(nph)%make_fragments()
      call cryst(nph)%make_rotations()
      forall (j=1:numat) cryst(nph)%at(j)%rcod = 0    ! reset refinement codes
      call cryst(nph)%load_bvparam()  !for 'BVPAR' directive and BV restraints
   enddo
!
!  Leggi le direttive se e' attivo il comando %sannel
   if (read_dir) then
       call sa_directives(gopti,sc,bcond,resmaxg,ishifta,errdir)  
       if (errdir%signal) then
           if (jscreen == 0 .or. ifAutomatic == 1) then
               saerr = errdir
               return   ! stop procedure only if no GUI and automatic run
           else
               call errdir%print()
           endif
       endif
   else
       resmaxg = -1      ! 
   endif
      !call cryst(1)%set_doc(.false.) ! disable doc for test
           !call set_antibump_auto(bump_sa,bumptab,atm,cellt,fragment,bumpset%scaledist)  !!!Force anti-bump for test
           !if (nrestraints(bump_sa) == 0) stop
!
!  Align bump restraints from directive with graphic
   if (jscreen == 1 .and. ifAutomatic == 0) then
       if (cryst(1)%numresab() > 0) then
           call table_from_bump(bumptab,cryst(1)%abres,cryst(1)%at,cryst(1)%abset)
       endif
   endif
   do nph=1,numphase(cryst)
      if (cryst(nph)%numresab() > 0) cryst(nph)%abres%wei = cryst(nph)%abset%weight ! apply the same weigth to all a-bump. res.
!
!     Porta i frammenti in cella
      call fragment_in_cell(cryst(nph)%frag,cryst(nph)%at)
!
!     Fix traslation in case of polar axis only for fragment with more atoms
!     uncomment only for test if (.not.spacegr%polar()) stop
      if (cryst(nph)%spg%polar()) then
          pfrag = maxloc(cryst(nph)%frag(:)%nat)
          call cryst(nph)%spg%code_for_polar(cryst(nph)%frag(pfrag(1))%rcod)
      endif
   enddo
!
!  Compute number of parameters for setting of number of moves and resolution
   call gopt_count_parameters(gopti)
!
!  Calcola valore iniziali per algoritmi di ottimizzazione
   if (gopti%autoalgo) call gopt_set_algo(gopti)   !!!!!!,cryst(1)%numrot(),cryst(1)%numfrag())
   if (sc%autont) call gopt_set_ncycle(gopti,sc%nt)  
   if (gopti%algo == BBBC) then
       call set_bb_bc_condition(gopti,bcond)
   endif
!
!  Memorizza tutti i riflessi
   do nph=1,numphase(cryst)
      !call copy_ref(cryst(nph)%refc,cryst(nph)%ref)
      call cryst(nph)%save_ref()
   enddo
   ncuold = 0
!
!  Definisci i limiti risoluzione dello spettro 
   if (resmaxg < 0) then
       resmin = resvalue(dataset(1)%tmax,dataset(1)%wave(1))
       if (resmin > RESLIMIT) then
           gopti%res = resmin
       else
           !gopti%res = RESLIMIT
           !call gopt_count_parameters(gopti)
           !call check_resp(cryst(1),gopti%res)
           allocate(resv(numphase(cryst)),source=RESLIMIT)
           do nph=1,numphase(cryst)
              call check_resp(cryst(nph),resv(nph))
              !write(0,*)'PH:',nph,resv(nph),RESLIMIT
           enddo
           gopti%res = minval(resv)
       endif
   else    
       gopti%res = resmaxg
   endif
!
   ginfo = gopti  ! save in ginfo for graphic and external function
!   
   if (jscreen > 0) then
       call clear_messages()
       call write_message(trim(algoname_long(gopti%algo)), pos=1)
       call write_message('Number of refined parameters: ',inum=gopti%npar,pos=2)
       call write_message('DOF(ext): ',inum=gopti%dofe,pos=3)
       call write_message('DOF(int): ',inum=gopti%dofi,pos=4)
       do nph=1,numphase(cryst)
          if (cryst(nph)%is_known()) call cryst(nph)%sfactor()
       enddo
       call to_coord_cry(cryst,-1,5)
           !call to_coord_sir(-1,cryst(1)%at,cryst(1)%ref,size(cryst(1)%ref),cryst(1)%cell,cryst(1)%spg,lo,cryst(1)%atpub)
           !call write_message_found1(5)
   endif
!   
!  Print info
   if (mpi_prog_rank == mpi_prog_master) then
       write(lo,'(1x,79("+"),//,30x,"Global Optimization Procedure",//,1x,79("+"))')
       do nph=1,numphase(cryst)
          write(lo,'(/2x,a,i0,a)')'Phase #',nph,': '//trim(cryst(nph)%cr_name)
          call cryst(nph)%print_bonds(lo,'C')
          call cryst(nph)%print_molecules(lo)
          call cryst(nph)%print_bonds(lo,'R')
          if (cryst(nph)%numresbv() > 0) then
              call cryst(nph)%print_bvparam(lo)
          endif
       enddo
   endif
!   
   end subroutine sa_prelim     

!----------------------------------------------------------------------------------------------------      

   subroutine global_init(gopti)
   type(opti_info_type), intent(out)  :: gopti     
!
   gopti%cftype = set_default_cfunction(1)
   !!gopti%cftype = CHI2GRU
   !!gopti%cftype = RWP_COSTF
   !!gopti%cftype = CHI2I_NEW
!corr   gopti%doc = .false.
   gopti%seed = 1      ! If different from zero, holds the seed for random number generator
   gopti%nrun = 10     ! Number of runs
   gopti%nrunc = 1     ! Contatore dei run
   !gopti%algo = SA_LOCAL  ! Tipo di algoritmo usato
   gopti%algo = BBBC   ! Tipo di algoritmo usato
   gopti%autoalgo = .true.
   gopti%popt = 'PMVESB'   ! Opzioni di output
!
   end subroutine global_init

!----------------------------------------------------------------------------------------------------      

   integer function set_default_cfunction(codedata)  result(cftype)
!
!  Set the default CF for the dataset codedata
!
   USE variables, only: dataset
   USE datasetmod
   integer, intent(in) :: codedata
!
   cftype = XPD_COSTF(1)
   if (ndataset(dataset) > 0) then
       if (dataset(codedata)%datatype == SCRYS_DATA) then
           cftype = XSE_COSTF(1)
       endif
   endif
!
   end function set_default_cfunction

!----------------------------------------------------------------------------------------------------      

   subroutine set_allowed_cfunction(codedata,cfvet)
!
!  Set allowed CF for the dataset codedata
!
   USE variables, only: dataset
   USE datasetmod
   integer, intent(in)                             :: codedata
   integer, dimension(:), allocatable, intent(out) :: cfvet
!
   if (ndataset(dataset) > 0) then
       if (dataset(codedata)%datatype == SCRYS_DATA) then
           allocate(cfvet(size(XSE_COSTF)),source=XSE_COSTF)
           return
       endif
   endif
   allocate(cfvet(size(XPD_COSTF)),source=XPD_COSTF)
!
   end subroutine set_allowed_cfunction

!----------------------------------------------------------------------------------------------------      
 
   subroutine sa_run(gopti,sc,bcond,extra,speedup)
   USE mpi_prog
   USE RefinecomRef, only:rinfo,rfcond
   USE genlsq
   USE variables, only:cryst,dataset
   USE Molcom, only:ifBrekke,jscreen
   USE General, only:lo
   USE simulated_anneal
   USE compare_struct
   USE atom_type_util
   USE Conteggi, only:npunti
   USE messagemod
   USE bb_bc
   USE plotstyle
   USE connect_mod
   USE constraints
   USE RIETV
   USE PatternRef, only: kcount_status
   USE View
   !USE background
   USE ilmviewmod
   USE enable_amb
   USE fouriermap
   USE anti_bump
   USE STAT, only:Rvalue
   !USE unit_cell
   !USE reflection_type_util
   USE arrayutil
   USE fragmentmod
   USE datasetmod
   USE rrestr
   USE grexpomod
   USE timer
   USE crystal_phase
   type array_temp_sav
        type(cell_type)                    :: cell 
        integer                            :: gcode
        real                               :: gpar
        integer, dimension(:), allocatable :: rcodb,rcodo       
   end type array_temp_sav
   type(opti_info_type), intent(inout)         :: gopti    
   type(SimAnn_Conditions_type), intent(inout) :: sc                     
   type(bb_bc_condition), intent(in)           :: bcond                     
   logical, intent(in), optional               :: extra
   real, intent(in), optional                  :: speedup
   real                                        :: resmaxg
   integer                                     :: i,nph
   real                                        :: res,resi,minres
   integer                                     :: ncc
!corr   integer, dimension(:), allocatable          :: rcodbsav,rcodosav
!corr   type(cell_type)                             :: cellold
   real                                        :: vol_per_atom
   type(bond_type), dimension(:), allocatable  :: legm
   type(atom_type), dimension(:), allocatable  :: atomcart
   logical :: extrac
   integer :: nrefu
   type(array_temp_sav), dimension(:), allocatable :: cry_sav
!
   extrac = .true.
   if (present(extra)) extrac = extra
!!!!!!!!!!test
          !call set_bump_exclusions(cryst(1)%at,cryst(1)%bond,cryst(1)%frag,cryst(1)%rot,cryst(1)%abres)
!!!!!!!!!!test
!
!  Conta gradi di liberta
   call gopt_count_parameters(gopti)
!   
!  Assegna valore per la risoluzione
   resmaxg = gopti%res
   resi = resmaxg
!   
!  Controlla che la risoluzione assegnata non sia inferiore al valore minimo gestibile
!corr   ncc = rfcond%lim%nctot   
   ncc = dataset(1)%npoints()
   if (npunti > 0) then
       minres = resvalue(dataset(1)%x0(ncc),dataset(1)%wave(1))
   else
       minres = resvalue(cryst(1)%refc(cryst(1)%numref())%tthd(1),dataset(1)%wave(1))
   endif
   res = max(resi,minres)
!   
   gopti%res = res
   gopti%ttmax = tthvalue(res,dataset(1)%wave(1))
   if (npunti > 0) then
       gopti%ttmin = dataset(1)%x0(1)  
       call dataset(1)%set_limit(-1.0,gopti%ttmax)
       gopti%ttmax = dataset(1)%xmaxc()
   else
       gopti%ttmin = cryst(1)%refc(1)%tthd(1)
   endif
   do nph=1,numphase(cryst)
      !nrefu = count(cryst(nph)%refc%tthd(1) <= gopti%ttmax .and. cryst(nph)%refc%tthd(1) >= dataset(1)%x(1))
      nrefu = count(cryst(nph)%refc%tthd(1) <= gopti%ttmax) 
      call cryst(nph)%set_used_ref(nrefu)
   enddo
!corr   !call set_limit_ref(cryst(1)%ref,rfcond%lim,cryst(1)%refc,gopti%ttmax)
!corr   !call set_limit_ref(cryst(1)%ref,cryst(1)%refc,gopti%ttmax)
!corr       call dataset(1)%set_limit(-1.0,gopti%ttmax)
       !write(0,*)'NREF_SA=',nrefu,cryst(1)%numref(),dataset(1)%xmaxc(),cryst(1)%ref(cryst(1)%numref())%tthd(1),cryst(1)%ref(nrefu)%tthd(1)
       !write(0,*)'NCU=',dataset(1)%nc2
       !write(0,*)'TMAX=',gopti%ttmax,dataset(1)%xmaxc(),dataset(1)%x(dataset(1)%nc2-1)
!      
!!!!prova
!      write(0,*)'nc prima=',rfcond%lim%ncu
!      ncusav = rfcond%lim%ncu
!      rfcond%lim%ncu = min(rfcond%lim%ncu*1.1,1.0*rfcond%lim%nctot)
!      write(0,*)'nc dopo =',rfcond%lim%ncu
!!!!prova
!corr   gopti%nref = rfcond%lim%nrefu
   !gopti%nco = rfcond%lim%ncu
   gopti%nco = dataset(1)%nc2
!   
!corr   if(ncuold /= rfcond%lim%ncu .and. npunti > 0 .and. extrac .eqv. .true.) then
   if(ncuold /= dataset(1)%nc2 .and. npunti > 0 .and. extrac .eqv. .true.) then
       !ncuold = rfcond%lim%ncu
       ncuold = dataset(1)%nc2
       ncytot = 0            
       do nph=1,numphase(cryst)
          cryst(1)%ref(:)%pk = 20
       enddo
!
       !call dataset(1)%set_limit(n2=rfcond%lim%ncu)
       call dataset(1)%make_background()
!
!      Disegna pattern
       if (jscreen /= 0) then
           kcount_status = 0; 
           call vedinew(8)   !!!!,1,rfcond%lim%nctot)
           style(STYLE_BACKP)%vis = 0
       endif      
!       
!      Affina il profilo
       rfcond%raction = 1
       !rfcond%raction = 2
       rfcond%winmess = .false.
       rfcond%sprint = 'W'
       !rfcond%sprint = 'WSV'
            !rfcond%typefun = PVOIG
            !rfcond%typefun = TCHZ
                   !call init_function_param(pear,pvoi,tch)
       call setRInt(cryst(1)%ref)
       !call allocafuncls(ALLOCA,rfcond%lim%ncu) ! rialloca prima della count0
       call allocafuncls(ALLOCA,dataset(1)%nc2) ! rialloca prima della count0
       call setwei()
       call Count0()
       if (index(rfcond%sprint,'S') > 0) call printfin(lo,'Starting Rietveld parameters')
!
!      Save parameters modified by refinement
       allocate(cry_sav(numphase(cryst)))
       do nph=1,numphase(cryst)
          cry_sav(nph)%cell = cryst(nph)%cell
          cry_sav(nph)%gcode = cryst(nph)%gcode
          cry_sav(nph)%gpar = cryst(nph)%gpar
          allocate(cry_sav(nph)%rcodb(cryst(nph)%natoms()),source=cryst(nph)%at%rcod(4))  ! salva il codice dei b perche' la riet_new li ricopre
          allocate(cry_sav(nph)%rcodo(cryst(nph)%natoms()),source=cryst(nph)%at%rcod(5))  ! e per occ
       enddo
       !cellold = cryst(1)%cell  
       !gcodesav = cryst(1)%gcode
       !gpo_sav = cryst(1)%gpar
       !call new_array(rcodbsav,cryst(1)%natoms())
       !call new_array(rcodosav,cryst(1)%natoms())
       !rcodbsav = cryst(1)%at%rcod(4)  ! salva il codice dei b perche' la riet_new li ricopre
       !rcodosav = cryst(1)%at%rcod(5)  ! e per occ
!
       call clear_plot(grplot,1)
       if (rfcond%raction == 1) then
           rfcond%ncauto = 1
           do nph=1,numphase(cryst)
              call init_fobs(cryst(nph)%ref)
           enddo
           do i=1,3
              rfcond%auto_prof = .true.
              rfcond%auto_stru = .false.
              !call init_fobs(cryst(1)%ref)
              call riet_new(i)
           enddo
       else
           call pawley(1)
           do nph=1,numphase(cryst)
              call init_fobs(cryst(nph)%ref)
           enddo
           do i=1,3
              rfcond%auto_prof = .true.
              rfcond%auto_stru = .false.
              call riet_new(i)
           enddo
           call pawley(0)
       endif
!
       !do nph=1,numphase(cryst)
       !   cryst(nph)%cell = cry_sav%cell 
       !enddo
!corr       gpo%rcod = gpo_sav%rcod  ! ripristina il codice
       !cryst(1)%gcode = gcodesav  ! ripristina il codice
       !gpo_sav = cryst(1)%gpar    ! tieni il valore di G affinato
       !cryst(1)%at%rcod(4) = rcodbsav
       !cryst(1)%at%rcod(5) = rcodosav
!
       gopti%rwp_leb = rinfo%rwp  ! salva Rwp alla fine dell'affinamento del profilo
!
       do nph=1,numphase(cryst)
          cryst(nph)%gcode = cry_sav(nph)%gcode ! ripristina il codice
          cryst(nph)%gpar = cry_sav(nph)%gpar   ! tieni il valore di G affinato
          cryst(nph)%at%rcod(4) = cry_sav(nph)%rcodb(:) 
          cryst(nph)%at%rcod(5) = cry_sav(nph)%rcodo(:) 
          !call coord_in_newcell(cryst(nph)%at,cellold,cryst(nph)%cell)
          call coord_in_newcell(cryst(nph)%at,cry_sav(nph)%cell,cryst(nph)%cell)
       enddo
!
!      Riaggiorna le coordinate dei frammenti           
       do nph=1,numphase(cryst)
          call set_atm_in_frag(cryst(nph)%frag,cryst(nph)%at)
       enddo
!
       if (mpi_prog_rank == mpi_prog_master) then
           call printfin(lo,'Parameters after profile refinement',opt='')
           !write(6,'(a10,3f10.3)')STRTAG//cryst(1)%cr_name,rinfo%rwp,rinfo%rp,Rvalue(cryst(1)%ref%fv,cryst(1)%ref%fo); return
       endif
!
       rfcond%raction = 0 
   endif
!
   fortran_work = .true.
   do nph=1,numphase(cryst)
!
!     Refined atoms are in general position
      do i=1,cryst(nph)%numfrag()
         if (any(cryst(nph)%frag(i)%rcod(:) > 0)) then
             cryst(nph)%at(cryst(nph)%frag(i)%pos)%ocry = 1.0
         endif
      enddo
!
      call set_equivalent_for_restraints(cryst(nph)%res,cryst(nph)%bond)
   enddo
!
!  Attiva la gestione dei tempi   
   call timesa(SATIME_CF)%set_type('Cost function calculation')
   call timesa(SATIME_MOVES)%set_type('Moves')
   call timesa(SATIME_GRAPH)%set_type('Graphic')
   call timesa(SATIME_RUN)%set_type('Time/run')
   call timesa(SATIME_TOT)%set_type('Total time')          
!
   call gopt_init_costf(gopti)   ! inizializzazione per funzioni costo
!
!  Inizializza gpar, randomizza se richiesto
   call sa_prerefine(gopti,sc%rand)
!
   do nph=1,numphase(cryst)
      if (cryst(nph)%doc_active() .or. cryst(nph)%numresbv() > 0) then
!
!        Set size of grid according to the active procedures
         if (cryst(nph)%doc_active() .and. cryst(nph)%numresbv() > 0) then  
             !kgr = ceiling(maxval(cryst(nph)%bvpar(:,:)%rmax))
             !call init_grid(cryst(nph)%cell,cryst(nph)%spg,cryst(nph)%natoms(),1.0,kgr)  ! grid for doc + bv
             call cryst(nph)%allocate_grid(cutoff=1.0)      ! grid only for doc + bv
         else
             if (cryst(nph)%doc_active()) then
                 !call init_grid(cryst(nph)%cell,cryst(nph)%spg,cryst(nph)%natoms(),1.0,1)  ! grid only for doc
                 call cryst(nph)%allocate_grid(cutoff=1.0)      ! grid only for doc
             elseif (cryst(nph)%numresbv() > 0) then
                 !call init_grid(cryst(nph)%cell,cryst(nph)%spg,cryst(nph)%natoms(),maxval(cryst(nph)%bvpar(:,:)%rmax),1)  ! grid only for bv
                 call cryst(nph)%allocate_grid(cutoff=maxval(cryst(nph)%bvpar(:,:)%rmax))  ! grid only for bv
             endif
         endif
      endif
   enddo
!
!  Stampa informazioni generali:restraints,range,num. di riflessi,dof,etc.   
   if (mpi_prog_rank == mpi_prog_master) then
       do nph=1,numphase(cryst)
          call print_atoms(cryst(nph)%at,title='Starting structure')
          if (cryst(nph)%is_known()) then
              call to_coord_sir(1,cryst(nph)%at,cryst(nph)%ref,size(cryst(nph)%ref),    &
                                cryst(nph)%cell,cryst(nph)%spg,lo,cryst(nph)%atpub)
          endif
!
!         Checking sul volume per atomo
          vol_per_atom = volume_per_atom(cryst(nph)%at, cryst(nph)%spg%nsymop, cryst(nph)%cell%volume())
          write(lo,'("   Volume per atom calculated by starting model = ",f0.3)') vol_per_atom
          if (vol_per_atom < 5 .or. vol_per_atom > 30)    &
          write(lo,'("   WARNING: volume per atom seems unresonable. Check the starting fragment")')
       enddo

       call gopt_print_info(gopti,sc,lo)
   endif
!
!  Operazioni per la grafica, imponi visualizzazione cella   
   if (jscreen > 0) then
       call ilmview_set_cell(1)
       call ilmview_open(cryst(active_phase)%cr_name)
       call ilmview_draw(cryst(active_phase)%at,cryst(active_phase)%bond,   &
                         cryst(active_phase)%elem,cryst(active_phase)%cell,cryst(active_phase)%spg)
       call jav_set_style(5,1)          !!mr 221014
       call jav_set_sym(0,0,1)
       call set_polyhedra_button(0)
       call set_contacts_button(0)
       call set_grow_fragment_button(0)
   endif
!   
   call timesa(SATIME_TOT)%tic()
!
   call abilita_tasti('run+skip')
!
!  Calcolo automatico del numero di cicli per temperatura
   if (sc%autont) call gopt_set_ncycle(gopti,sc%nt)
   if (present(speedup)) sc%nt = ceiling(sc%nt*speedup)
!
!  Calcolo automatico della temperatura iniziale
   if (sc%autoti) call sa_set_temperature_new(sc,gopti,gpar,safunc_temp)
   !ginfo = gopti; if (sc%autoti) call sa_set_temperature(sc,gopti,gpar,tfunk,safunc1)
!
!  Setting della convergenza
   call sa_set_convergence(sc,gopti%npar)
!
   call progress_bar_init('Starting simulated annealing')   !  Inizializza progress bar      
!
!  Definisce su che righe scrivere i vari messaggi
   if (.not.extrac) then
       call sa_set_messages(gopti%algo, 0, cryst(1)%numresab(),cryst(1)%numres())
   else
       call sa_set_messages(gopti%algo, npunti, cryst(1)%numresab(),cryst(1)%numres())
   endif
!
!  Initialize map visualization: compute radius and define sogmap
   call copy_atoms(atomcart,cryst(1)%at)
   if (cryst(1)%numfrag() > 1) then
       call regroup_fragment(cryst(1)%frag,atomcart,cryst(1)%cell,cryst(1)%spg)
   endif
   call frac_to_cart(atomcart,cryst(1)%cell%get_ortom())
   call get_radius_molecule(atomcart,radmap)
   call map_set_rad(radmap)
   sasogmap = 120
!!!   call sogmap_init(FOUR_FO_FC,ref,atm,sasogmap)   ! non funziona ad es. con clomipra
!!!   sasogmap = sasogmap * 0.003
!!!      write(0,*)'sogmap=',sasogmap
!
   ifBrekke = 0      !azzera codice di interazione con la grafica
   call simann(gopti,sc,bcond,extrac) 
   !call ptempering(sc) 
!
   call gopt_close_costf()            ! dealloca info
   call set_atm_in_frag(cryst(1)%frag,cryst(1)%at)
!
!  Chiudi gestione tempi   
   call timesa(SATIME_TOT)%toc()
   call timesa(SATIME_RUN)%set_print_ave()
   if (mpi_prog_rank == mpi_prog_master) then
       call print_time(timesa,lo)
   endif
!
   if (jscreen > 0) then
       call progress_bar_close()                             ! chiudi progress bar
!
!      disegna modello
       call copy_bonds(legm,cryst(1)%bond)
       if (cryst(1)%numfrag() > 1) then 
           call connect_fragment(cryst(1)%frag,cryst(1)%at,legm,cryst(1)%cell,.false.)
       endif
       call ilmview_draw(cryst(1)%at,legm,cryst(1)%elem,cryst(1)%cell,cryst(1)%spg)
!
!      disegna profilo
       call cryst(1)%sfactor()
       if (dataset(1)%datatype == POW_DATA) then
           kcount_status = 1
           call sa_profile(cryst(1)%ref%fc)
           call vedinew(8,1,gopti%nco)
       endif
!
       if (is_map_enabled()) then
           call map_set_centre(baricentro(cryst(1)%at))
           call jav_show_map(map_type_selected(),cryst(1)%ref,'SA Map',sogmap=sasogmap,fctype=2)
       endif
!
   endif
!   
   fortran_work = .false.
!
   end subroutine sa_run

!----------------------------------------------------------------------------------------------------      

   subroutine sa_set_messages(algo, data_type, nbump, nres)
!
!  Set order of messages for SA in the split window
!
   USE simulated_anneal
   integer, intent(in) :: algo
   integer, intent(in) :: data_type
   integer, intent(in) :: nbump, nres
   integer             :: nline
!
   nline = 1
   if (algo == BBBC) then
       nline = nline + 1; vmess(LINE_RUN) = nline
                          vmess(LINE_TEMP) = -100   ! non mostrare la temperatura
       nline = nline + 1; vmess(LINE_MOVES) = nline
       nline = nline + 1; vmess(LINE_CF) = nline
       nline = nline + 1; vmess(LINE_CFMIN) = 5
   else
       nline = nline + 1; vmess(LINE_RUN) = nline
       nline = nline + 1; vmess(LINE_TEMP) = nline
       nline = nline + 1; vmess(LINE_MOVES) = nline
       nline = nline + 1; vmess(LINE_CF) = nline
       nline = nline + 1; vmess(LINE_CFMIN) = nline
   endif
   if (nres > 0) then
       nline = nline + 1; vmess(LINE_CFRES) = nline
   endif
   if (nbump > 0) then
       nline = nline + 1; vmess(LINE_CFBUMP) = nline
   endif
   if (data_type > 0) then
       nline = nline + 1; vmess(LINE_RATIO) = nline
       nline = nline + 1; vmess(LINE_TIME) = nline
   else
       vmess(LINE_RATIO) = -100
       nline = nline + 1; vmess(LINE_TIME) = nline + 1
   endif
!
   end subroutine sa_set_messages

!----------------------------------------------------------------------------------------------------      

   subroutine gopt_init_costf(gopti)
!
!  Inizializzazione della funzione costo
!   
   USE fastfcal
   USE genlsq
   USE variables, only:cryst,dataset
   USE reflection_type_util
   USE arrayutil
   USE variables, only: dataset
   USE cryutil
   USE crystal_phase
   type(opti_info_type), intent(inout) :: gopti    
   integer                             :: i,kal
   integer                             :: nnn
   integer                             :: maxlim
   integer, dimension(:), allocatable  :: ivetc,puntc
   real, dimension(:), allocatable     :: ycountonew,ycossb,weichi
   integer                             :: ncc,nph,maxf
!   
   nph = 1
   select case (gopti%cftype)
      case (RWP_COSTF)     
        cryst(nph)%ref(:)%pk = 1.0
        call profile_init(dataset(1)%typefun(nph),gopti,cryst(nph),nph)
        rwden = sqrt(sum(weilsq(:gopti%nco)*dataset(1)%y(:gopti%nco)**2))/100.0
        gopti%Cost_function_name = COSTF_NAME(RWP_COSTF)
        gopti%Cost_function_name_short = 'Rwp'
!corr        allocate(ycountsa(gopti%nco))
        call new_array(cryst(nph)%yc,gopti%nco)
        call dataset(1)%set_ys(gopti%nco)
        sumycossb = sum(dataset(1)%ys)
               
      case (CHI2FACT)
        rwden = sqrt(sum(cryst(nph)%ref%fo**2))/100.0
        gopti%Cost_function_name = COSTF_NAME(CHI2FACT)
        gopti%Cost_function_name_short = 'R-factor'
      
      case (CHI2I)
        call new_array(intoss,cryst(1)%numref())
        call new_array(intcal,cryst(1)%numref())
        call new_array(weichi,cryst(1)%numref())
!        allocate(intoss(gopti%nref),intcal(gopti%nref))
        !intoss = ref%m*ref%lp*ref%fo**2      
        !intoss = ref%m*ref%fo**2      
        intoss = cryst(1)%ref%fo**2      
        !intoss = ref%lp*ref%fo**2      
        sumintoss = sum(intoss)
        rwden = sqrt(sum(intoss**2))/100.0
        weichi(:) = 1.0/(1.0+intoss(:))
        !rwden = sumintoss/100.0
        gopti%Cost_function_name = COSTF_NAME(CHI2I)
        gopti%Cost_function_name_short = 'R-int'
        
      case (CHI2I_SD)
        call select_ref(cryst(1)%ref,3)
!corr        gopti%nref = size(cryst(1)%ref)
        call new_array(intoss,cryst(1)%numref())
        call new_array(intcal,cryst(1)%numref())
        !allocate(intoss(gopti%nref),intcal(gopti%nref))
        intoss = cryst(1)%ref%m*cryst(1)%ref%lp(1)*cryst(1)%ref%fo**2      
        rwden = sqrt(sum(intoss**2))/100.0
!corr        gopti%Cost_function_name = 'R Bragg intensities on single and double reflections'
        gopti%Cost_function_name = COSTF_NAME(CHI2I_SD)
        gopti%Cost_function_name_short = 'R-Bragg'
        !rwden = sqrt(sum(ref%fo**2))/100.0
        !sc%Cost_function_name = 'R structure factor on single and double reflections'               
        !gopti%nref = size(ref)
        
      case (CHI2I_NEW)
        cryst(1)%ref(:)%pk = 1.0
        call new_array(intoss,cryst(1)%numref())
        call new_array(intcal,cryst(1)%numref())
        call profile_init(dataset(1)%typefun(nph),gopti,cryst(1),nph)
        sumycossb = 0
        kal = 1
        do i=1,size(cryst(1)%ref)
           intoss(i) = 0
           do nnn=cryst(1)%profi(i,kal)%iniz,cryst(1)%profi(i,kal)%ifin
              intoss(i) = intoss(i) + dataset(1)%y(nnn) - dataset(1)%yb(nnn)
           enddo 
           sumycossb = sumycossb + intoss(i)
        enddo
        rwden = sqrt(sum(intoss**2))/100.0
        gopti%Cost_function_name = COSTF_NAME(CHI2I_NEW)
        gopti%Cost_function_name_short = 'R-Bragg'

      case (RWPC_COSTF,CHI2_COSTF)     
        maxlim = 0
        do nph=1,numphase(cryst)
           cryst(nph)%ref(:)%pk = 1.0
           call profile_init(dataset(1)%typefun(nph),gopti,cryst(nph),nph)
           maxf = maxval(cryst(nph)%profi%ifin)
           if (maxf > maxlim) maxlim = maxf
        enddo
        call new_array(ivetc,maxlim)
        call new_array(puntc,maxlim)
        call new_array(ycountonew,maxlim)
        call new_array(weichi,maxlim)
        call new_array(ycossb,maxlim)
!corr        call new_array(ycountsa,maxlim)
        ivetc(:) = 0
        do kal=1,dataset(1)%nwave
           do nph=1,numphase(cryst)
              do i=1,size(cryst(nph)%ref)
                 do nnn=cryst(nph)%profi(i,kal)%iniz,cryst(nph)%profi(i,kal)%ifin
                    ivetc(nnn) = 1
                 enddo
                 !write(6,*)i,'refo=',profi(i)%iniz,profi(i)%ifin
                 !write(6,*)'yc=',dataset(1)%y(profi(i)%iniz:profi(i)%ifin)
              enddo
           enddo
        enddo
        ncc = 0
        do nnn=1,maxlim
           if (ivetc(nnn) == 1) then
               ncc = ncc + 1
               puntc(nnn) = ncc
               ycountonew(ncc) = dataset(1)%y(nnn)
               ycossb(ncc) = dataset(1)%y(nnn) - dataset(1)%yb(nnn)
               weichi(ncc) = real(weilsq(nnn))
           endif
        enddo
        call dataset(1)%set_ys(ycossb(:ncc))
        call dataset(1)%set_wei(weichi(:ncc))
        do kal=1,dataset(1)%nwave
           do nph=1,numphase(cryst)
              do i=1,size(cryst(nph)%ref)
                 cryst(nph)%profi(i,kal)%iniz = puntc(cryst(nph)%profi(i,kal)%iniz)
                 cryst(nph)%profi(i,kal)%ifin = puntc(cryst(nph)%profi(i,kal)%ifin)
              enddo
           enddo
        enddo
        nco_new = ncc
        do nph=1,numphase(cryst)
           call new_array(cryst(nph)%yc,ncc)
        enddo

        rwden = sqrt(sum(dataset(1)%wei*ycountonew(:ncc)**2))/100.0
        gopti%Cost_function_name = COSTF_NAME(gopti%cftype)
        gopti%Cost_function_name_short = 'Rwp'
        sumycossb = sum(dataset(1)%ys)

      case (CHI2GRU)
        call make_ref_groups(cryst(1)%ref,sagref,sogi=1.0)
        call print_refg(sagref)
        rwden = sqrt(sum(sagref(:)%Io**2))/100.0
        gopti%Cost_function_name = COSTF_NAME(CHI2GRU)
        gopti%Cost_function_name_short = 'Rg'
        
   end select
   do nph=1,numphase(cryst)
      call fcalc_init(cryst(nph)%ref,cryst(nph)%at,cryst(nph)%spg,cryst(nph)%elem,dataset(1)%radtype,   &
                   cryst(nph)%sfact,cryst(nph)%sfactc,cryst(nph)%kpscat)
   enddo
!  
   end subroutine gopt_init_costf
        
!----------------------------------------------------------------------------------------------------         

   subroutine profile_init(typef,gopti,crystal,nph)
   USE profile_function
   USE variables, only: dataset
   USE cryutil
   USE crystal_phase
   type(opti_info_type), intent(in)     :: gopti    
   type(crystal_phase_t), intent(inout) :: crystal
   integer, intent(in)                  :: nph
   integer, intent(in)                  :: typef
   integer                              :: ier

   select case (typef)
     case (PEARSON7)
       call pearson_init(crystal,dataset(1)%x0(:gopti%nco),dataset(1)%pear(1),dataset(1)%nwave,dataset(1)%ratio,nph,ier)

     case (PVOIG)
       call psvoigt_init(crystal,dataset(1)%x0(:gopti%nco),dataset(1)%pvoi(1),dataset(1)%nwave,dataset(1)%ratio,nph)

     case (TCHZ)

   end select

   end subroutine profile_init

!----------------------------------------------------------------------------------------------------         

   subroutine gopt_close_costf()
   USE fastfcal
   USE variables, only: cryst
   USE cryutil
!
   call fcalc_close(cryst(1)%sfact,cryst(1)%sfactc)     ! dealloca info sui riflessi
!
   call close_profile(cryst(1))   ! dealloca info sul profilo
!   
   end subroutine gopt_close_costf
   
!----------------------------------------------------------------------------------------------------         

   subroutine sa_set_convergence(sc,npar)
!
!  Setting della convergenza in base al numero di parametri
!
   type(SimAnn_Conditions_type), intent(inout) :: sc
   integer, intent(in)                         :: npar
!
   select case (npar)
      case (1:3)
        sc%eps = 0.01

      case (4:5)
        sc%eps = 0.1

      case (6:)
        sc%eps = 2.0

   end select
!
   end subroutine sa_set_convergence

!----------------------------------------------------------------------------------------------------         

   subroutine sa_directives(gopti,sc,bcond,resmaxg,ishifta,errdir) 
!
!  Read directives for global optimization
!
   USE simulated_anneal                  
   USE commandsmod
   USE bb_bc
   USE errormod
   USE connect_mod
   USE variables, only: cryst,dataset
   USE rrestr
   USE unit_cell
   USE constraints
   !USE atom_type_util
   USE rotationmod
   USE fragmentmod
   USE gen_frm
   type(opti_info_type), intent(inout)         :: gopti     
   type(SimAnn_Conditions_type), intent(inout) :: sc                     
   type(bb_bc_condition), intent(inout)        :: bcond                     
   real, intent(out)                           :: resmaxg
   type(error_type), intent(out)               :: errdir
   integer, intent(out)                        :: ishifta
   real                                        :: shiftaval
   logical                                     :: found
   integer                                     :: i
   logical, dimension(:), allocatable          :: vetshift   !!!, vetocc
   integer                                     :: icpos
   type(error_type)                            :: errf
!
!  Inizializza parametri dell'algoritmo
   resmaxg = -1
   ishifta = 0
   shiftaval = 0.0
   allocate(vetshift(cryst(1)%natoms()))
   vetshift(:) = .false.
!
!  Read directives
   icpos = command_position(commands,'sannel')
   if (icpos > 0) then
       call reset_constraints(cryst(1)%constr)  ! delete old constraints and load in constr new constraints by read_dir_frag
!
!      Read directives specific for SA
       do i=1,commands(icpos)%ndir
          call read_dir_sa(commands(icpos)%strdir(i),gopti,sc,bcond,resmaxg,errdir,found)
          if (errdir%signal) return
          if (found) cycle
!
!         Read directives specific for molecular fragments
          call read_dir_frag(commands(icpos)%strdir(i),cryst(1),  &
                ishifta,shiftaval,vetshift,dataset(1)%radtype,errdir,found)
          if (errdir%signal) return
          if (.not.found) then 
              call errf%set('Error reading directive "'//trim(commands(icpos)%strdir(i))//'"')
          endif
       enddo
       if (errf%signal) errdir = errf
   endif
!
   call errdir%as_severe()
!
!  Shift sugli atomi
   if (ishifta > 0) then
       shiftatomval = shiftaval
!
!      Azzera vetshift se l'atomo e' isolato
       do i=1,size(cryst(1)%at)
          if (vetshift(i) .and. fragment_natom(cryst(1)%frag,i) > 1) then
              shift_atom(i)%rcod(1:3) = 1
          else
              shift_atom(i)%rcod(1:3) = 0
          endif
       enddo
   endif 
!
!  Global optimization works on array sconstr 
   if (nconstraints(cryst(1)%constr) > 0) then
       call copy_constraints(cryst(1)%sconstr,cryst(1)%constr) 
!corr       call reset_constraints(cryst(1)%constr)
       call refine_constraint(cryst(1)%sconstr,cryst(1)%at,1)  ! activate constraints
   endif
!
   end subroutine sa_directives

 !----------------------------------------------------------------------------------------------------

   subroutine read_dir_sa(strdir0,gopti,sc,bcond,resmaxg,errdir,found)
   USE strutil
   USE simulated_anneal                 
   USE bb_bc
   USE errormod
   USE mpi_prog
   character(len=*), intent(in)                :: strdir0
   type(opti_info_type), intent(inout)         :: gopti     
   type(SimAnn_Conditions_type), intent(inout) :: sc                     
   type(bb_bc_condition), intent(inout)        :: bcond                     
   real, intent(inout)                         :: resmaxg
   type(error_type), intent(out)               :: errdir
   logical, intent(out)                        :: found
   character(len=len(strdir0))                 :: strdir,word
   real, dimension(100)                        :: vet
   integer, dimension(100)                     :: ivet
   integer                                     :: iv
   integer                                     :: nlongs
   logical                                     :: errpar
!
   strdir = strdir0
   call Cutst(strdir,nlongs,word) 
!
   errpar = .false.
   found = .true.
   if (match_word(word,'TEMP')) then ! temperature        
       call Getnum(strdir(:nlongs),vet,ivet,iv)
       if (iv >= 1) then
           sc%t_ini = vet(1)
           sc%autoti = .false.
       else
           errpar = .true.
       endif

   elseif (match_word(word,'ALGO')) then ! tipo di algoritmo
       call Getnum(strdir(:nlongs),vet,ivet,iv)
       if (iv > 0) then
           if (ivet(1) > 0 .and. ivet(1) < 5) then
               ! ignore this directive if PSA is activated
               if (sc%cooperate .and. mpi_prog_numprocs > 1 .and. ivet(1) /= SA_LOCAL) then   
                   call errdir%setw('Directive '//trim(strdir0)//' will be ignored')
               else
                   gopti%algo = ivet(1)
                   gopti%autoalgo = .false.
               endif
           endif
       endif

   elseif (match_word(word,'COST')) then ! cost function
       call Getnum(strdir(:nlongs),vet,ivet,iv)
       if (iv > 0) then
           gopti%cftype = ivet(1)
       else
           errpar = .true.
       endif
             
   elseif (match_word(word,'TFAC')) then ! temperature factor
       call Getnum(strdir(:nlongs),vet,ivet,iv)
       if (iv >= 1) then
           sc%temp_fact = vet(1)
       else
           errpar = .true.
       endif
               
   elseif (match_word(word,'NITE')) then ! number of iterations
       call Getnum(strdir(:nlongs),vet,ivet,iv)
       if (iv >= 1) then
           sc%nt = ivet(1)
           sc%autont = .false.
       else
           errpar = .true.
       endif

   elseif (match_word(word,'NRUN')) then ! number of runs
       call Getnum(strdir(:nlongs),vet,ivet,iv)
       if (iv >= 1) then
           gopti%nrun = ivet(1)
           call update_run_cooperate(gopti%nrun,sc%cooperate)
       else
           errpar = .true.
       endif

   elseif (match_word(word,'NCYC')) then ! numero di cicli per bb-bc
       call Getnum(strdir(:nlongs),vet,ivet,iv)
       if (iv >= 1) then
           bcond%niter = ivet(1)
           bcond%auto = .false.
       else
           errpar = .true.
       endif

   elseif (match_word(word,'RESM')) then ! max resolution
       call Getnum(strdir(:nlongs),vet,ivet,iv)
       if (iv > 0) then
           resmaxg = vet(1)
       else
           errpar = .true.
       endif

   elseif (match_word(word,'RAND')) then ! randomize fragment
       call Getnum(strdir(:nlongs),vet,ivet,iv)
       if (iv == 0) then
           sc%rand = 0
       else
           sc%rand = ivet(1)
       endif
   elseif (match_word(word,'COOP')) then ! cooperation in SA
       if (mpi_prog_numprocs == 1) then
           call errdir%setw('Directive '//trim(strdir0)//' will be ignored')
       else
           sc%cooperate = .true.
           gopti%algo = SA_LOCAL
           gopti%autoalgo = .false.
           call update_run_cooperate(gopti%nrun,sc%cooperate)
       endif
   elseif (match_word(word,'RARE')) then ! rare cooperation in SA
       if (mpi_prog_numprocs == 1) then
           call errdir%setw('Directive '//trim(strdir0)//' will be ignored')
       else
           sc%nrare = mpi_prog_numprocs / 5
           if (sc%nrare == 0) sc%nrare = 1
       endif
   else
       found = .false.
   endif
!   
!  Segnale errore nell'inserimento del paramatro da associare alla stringa
   if (errpar) then
       call errdir%set('Error in directive "'//trim(strdir0)//'"')
   endif        
!
   end subroutine read_dir_sa
  
!----------------------------------------------------------------------------------------------------

   subroutine update_run_cooperate(nrun,coop)
!
!  In case of cooperation the number of run is updated to avoid idle cpus
!
   use mpi_prog
   integer, intent(inout) :: nrun
   logical, intent(in)    :: coop
!
   if (mpi_prog_numprocs > 1 .and. coop) then
       if (mod(nrun,mpi_prog_numprocs) /= 0) nrun = mpi_prog_numprocs * (nrun/mpi_prog_numprocs + 1)
   endif
!
   end subroutine update_run_cooperate

!----------------------------------------------------------------------------------------------------

   subroutine gopt_count_parameters(info)   !!!!,atomo,frag,rotat,gpocode)
   USE constraints
   USE variables, only: cryst
   USE crystal_phase
   type(opti_info_type), intent(inout) :: info
   integer                             :: i,nph,numph
!
   numph = numphase(cryst)
   info%dofe = 0           
   info%npb = 0
   info%npshift = 0
   info%npar = 0
   info%nprot = 0
!   
   do nph=1,numph
!
!     Count external DOF
      info%dofe = info%dofe + cryst(nph)%edof()
!   
!     Count thermal factors B
      info%npb = info%npb + count(cryst(nph)%at%rcod(4) > 0)
      if (nconstraints(cryst(nph)%sconstr) > 0) info%npb = info%npb + count(cryst(nph)%sconstr%ref > 0)
!
!     Count rotations   
      info%nprot = info%nprot + cryst(nph)%idof()
!
!     Count shifts
!!!!!FIX THIS FOR MULTI
      do i=1,cryst(nph)%natoms()
         info%npshift = info%npshift + count(shift_atom(i)%rcod > 0)
      enddo
!
!     Preferred orientation
      if (cryst(nph)%gcode > 0) then
          info%npar = info%npar + 1
      endif
   enddo
!
!  DOF interni   
   info%dofi = info%nprot
!
!  Numero totale di parametri   
   info%npar = info%npar + info%dofi + info%dofe + info%npb + info%npshift
!
   end subroutine gopt_count_parameters

!----------------------------------------------------------------------------------------------------

   subroutine sa_prerefine(gopti,randomod)
!
!  Conta DOF, definisce i parametri (gpar) e randomizza i parametri se richiesto
!   
   USE variables, only: cryst, dataset
   USE simulated_anneal
   USE compare_struct
   USE trig_constants, only: twopi,dtor
   USE atom_type_util
   USE errormod
   USE rand_mod
   USE connect_mod
   USE elements
   USE constraints
   USE gen_frm
   USE rotationmod
   USE fragmentmod
   USE mpi_prog
   USE crystal_phase
   type(opti_info_type), intent(inout)       :: gopti     
   integer, intent(in)                       :: randomod 
   integer                                   :: i,j,nph
   integer                                   :: nkref
   character(len=30)                         :: strat
   real, dimension(3)                        :: bar0
   character(len=*), dimension(5), parameter :: atomstring = (/'x   ','y   ','z   ','B   ','occ.'/)
   character(len=*), dimension(3), parameter :: string_rot =(/'theta','phi  ','z    '/)
   real, dimension(3)                        :: xroti = (/0.5,0.5,0.0/) 
   real, dimension(3)                        :: rangef
   logical, dimension(3)                     :: not_periodic, out_range
   real, dimension(3)                        :: newpos
   real                                      :: newang
!
!  Randomize models if required
   if (randomod >= 0) then
                !write(0,*)'RAND=',randomod,mpi_prog_rank
                ! FIX problem with randmod=0 int on system clock, gfortran, see intel
       do nph=1,numphase(cryst)
          call Init_Ran(randomod)
          call randomize_molecule(cryst(nph)%at,cryst(nph)%bond,cryst(nph)%cell,cryst(nph)%frag,cryst(nph)%rot)
          call set_atm_in_frag(cryst(nph)%frag,cryst(nph)%at)                  ! update fragment info
          do i=1,cryst(nph)%numrot()                                         ! update torsion value
             cryst(nph)%rot(i)%torsval = torsion_value_for_rotat(cryst(nph)%rot(i),cryst(nph)%at,cryst(nph)%cell)
          enddo
!
!         Crea mol file dalle coordinate random
          if (mpi_prog_rank == mpi_prog_master) then
              call export_structure(trim(cryst(nph)%cr_name)//'_rand.mol',cryst(nph)%at,cryst(nph)%cell,cryst(nph)%spg,  &
                                    cryst(nph)%elem,MOL_FILE,cryst(nph)%cr_name,dataset(1)%wave(1),cryst(nph)%bond)
          endif
       enddo
   endif
!
!  Alloca xpar al numero di parametri       
   if (allocated(gpar)) deallocate(gpar)
   allocate(gpar(gopti%npar))
!
   gpar(:)%stp = 1.0
   nkref = 0      
   do nph=1,numphase(cryst)
!   
!     Parametri termici
      do j=1,size(cryst(nph)%at)
         write(strat,'(i0,a)')j,' ('//trim(cryst(nph)%at(j)%lab)//')'     
         if (cryst(nph)%at(j)%rcod(4) > 0) then      
            nkref = nkref + 1
            gpar(nkref)%str = trim(atomstring(4))//strat
            gpar(nkref)%val = cryst(nph)%at(j)%biso
            cryst(nph)%at(j)%rcod(4) = nkref
            gpar(nkref)%low = max(0.0,cryst(nph)%at(j)%biso-10)
            gpar(nkref)%high = cryst(nph)%at(j)%biso+10
            gpar(nkref)%bound = 0         
            allocate(gpar(nkref)%pat(1))
            gpar(nkref)%pat(1) = j         
            gpar(nkref)%typepar = THERMALF_TYPE
            gpar(nkref)%ph = nph
         endif
      enddo
!
      do i=1,nconstraints(cryst(nph)%sconstr)
         if (cryst(nph)%sconstr(i)%ref > 0) then
             select case (cryst(nph)%sconstr(i)%code)
               case (CONSTR_B)
                 nkref = nkref + 1
                 !gpar(nkref)%str = 'constraint on '//trim(specie_from_pxen(atm(sconstr(i)%vat(1))%ptab))
                 gpar(nkref)%str = 'constraint on B'
                 cryst(nph)%sconstr(i)%ref = nkref
                 gpar(nkref)%low = 0
                 gpar(nkref)%high = 20
                 gpar(nkref)%bound = 0         
                 gpar(nkref)%val = constr_aveb(cryst(nph)%sconstr(i),cryst(nph)%at,gpar(nkref)%low,gpar(nkref)%high)
             !write(0,*)'MED=',gpar(nkref)%val,atm(sconstr(i)%vat)%biso
                 allocate(gpar(nkref)%pat(cryst(nph)%sconstr(i)%npar))
                 gpar(nkref)%pat(:) = cryst(nph)%sconstr(i)%vat
                 gpar(nkref)%typepar = THERMALF_TYPE
                 gpar(nkref)%ph = nph
                case (4)
             end select
         endif
      enddo
!
!     Loop sui frammenti
      do i=1,cryst(nph)%numfrag()
!
!        Traslazioni del frammento
!        Calcola il baricentro del frammento
         bar0 = baricentro(cryst(nph)%at(cryst(nph)%frag(i)%pos)) 
!
!        calcola il range di movimento in ognuna delle 3 direzioni
         rangef(:) = cryst(nph)%frag(i)%thigh(:) - cryst(nph)%frag(i)%tlow(:)    
!
!        controlla se il range e' diverso da 1 e quindi non pu� essere periodico
         not_periodic(:) = abs(rangef(:) - 1.0) > 0.01
!
!        controlla se il frammento e' fouri range
         out_range(:) = any(bar0(:) > cryst(nph)%frag(i)%thigh) .or. any(bar0(:) < cryst(nph)%frag(i)%tlow)      
!
!        Traslo il frammento all'interno del range se si verificano contemporaneamente le 2 condizioni:
!        1) Il range non e' periodico in almeno una direzione
!        2) Il frammento e' fuori range in almeno una direzione
         if (any(not_periodic(:)) .and. (any(out_range(:))) ) then
!
!            Modifica le coordinate del frammento solo lungo le direzioni che non
!            soddisfano le 2 condizioni precedenti      
             where(not_periodic(:) .and. out_range) 
                   newpos(:) = rangef(:)/2
             elsewhere
                   newpos(:) = bar0(:)
             endwhere
!!!   !          write(6,*)'Frammento spostato in ',newpos,' da ',bar0
!
!            Ora sposta il frammento nella nuova posizione          
             call set_fragment_position(cryst(nph)%frag(i),newpos,cryst(nph)%at)   
         endif       
!
         do j=1,3      
            if (cryst(nph)%frag(i)%rcod(j) > 0) then             
                nkref = nkref + 1
                gpar(nkref)%str = atomstring(j)
                gpar(nkref)%val = 0  
                cryst(nph)%frag(i)%rcod(j) = nkref         
                if (not_periodic(j)) then     ! range non periodico
                    gpar(nkref)%low = cryst(nph)%frag(i)%tlow(j)/2.0
                    gpar(nkref)%high = cryst(nph)%frag(i)%thigh(j)/2.0
                    gpar(nkref)%bound = 0             
                else                          ! range periodico
                    gpar(nkref)%low = -bar0(j)
                    gpar(nkref)%high = 1-bar0(j)
                    gpar(nkref)%bound = 1             
                endif
                !no_H = fragment(i)%nat - number_of_hydrogens(atm(fragment(i)%pos))
                !write(0,*)'NO_H=',no_H
                !allocate(gpar(nkref)%pat(no_H))
                !gpar(nkref)%pat = pack(fragment(i)%pos, mask = atomic_number(atm(fragment(i)%pos)) /= H_at)
                !      write(0,*)'PAR=',gpar(nkref)%pat
                allocate(gpar(nkref)%pat(cryst(nph)%frag(i)%nat))
                gpar(nkref)%pat = cryst(nph)%frag(i)%pos   
                gpar(nkref)%typepar = EXT_TYPE
                gpar(nkref)%ph = nph
            endif
         enddo
!
!        Rotazione frammento      
!
!        calcola il range per la rotazione random del frammento
         rangef(1) = cryst(nph)%frag(i)%rbound(2) - cryst(nph)%frag(i)%rbound(1)    
!
!        controlla se il range e' diverso da 1 e quindi non puo' essere periodico
         not_periodic(1) = abs(rangef(1) - 1.0) > 0.01
         do j=4,6
            if (cryst(nph)%frag(i)%rcod(j) > 0) then
                nkref = nkref + 1
                gpar(nkref)%str = string_rot(j-3)
                gpar(nkref)%val = xroti(j-3)   
                cryst(nph)%frag(i)%rcod(j) = nkref
                if (not_periodic(1)) then
                    gpar(nkref)%low = max(xroti(j-3) - rangef(1)/2,0.0)
                    gpar(nkref)%high = min(xroti(j-3) + rangef(1)/2,1.0)
                    gpar(nkref)%bound = 0
                else
                    gpar(nkref)%low = 0.0
                    gpar(nkref)%high = 1.0
                    gpar(nkref)%bound = 1
                endif
                allocate(gpar(nkref)%pat(cryst(nph)%frag(i)%nat))
                gpar(nkref)%pat = cryst(nph)%frag(i)%pos  
                gpar(nkref)%typepar = EXT_TYPE
                gpar(nkref)%ph = nph
            endif          
         enddo
      enddo   
!
!     Loop sulle rotazioni
      do i=1,cryst(nph)%numrot()
         if (cryst(nph)%rot(i)%rcod > 0) then
             nkref = nkref + 1         
             write(gpar(nkref)%str,'(a,i0)')'DOF(int) n.',i 
             gpar(nkref)%val = 0.0
             cryst(nph)%rot(i)%rcod = nkref
             rangef(1) = cryst(nph)%rot(i)%trbound(2) - cryst(nph)%rot(i)%trbound(1)  ! range di rotazione
             not_periodic(1) = abs(rangef(1) - 360) > 0.1                 ! valuta periodicit�
             if (not_periodic(1)) then     ! range non periodico
!
!                Converti i limiti delle torsioni in limiti degli angoli di rotazione normalizzati 
                 !write(6,*)i,'tors lim=',rotation(i)%trbound(1),rotation(i)%trbound(2)
                 gpar(nkref)%low = -(cryst(nph)%rot(i)%torsval - cryst(nph)%rot(i)%trbound(1))*dtor/twopi
                 gpar(nkref)%high = (cryst(nph)%rot(i)%trbound(2) - cryst(nph)%rot(i)%torsval)*dtor/twopi
                      !write(0,*)'TORS=',gpar(nkref)%low,gpar(nkref)%high
                 gpar(nkref)%bound = 0          
                 !write(6,*)i,'ang     =', -(rotation(i)%torsval - rotation(i)%trbound(1)),   &
                 !                          (rotation(i)%trbound(2) - rotation(i)%torsval)
                 !write(6,*)i,'ang norm=',gpar(nkref)%low,gpar(nkref)%high
                 out_range(1) = 0.0 < gpar(nkref)%low .or. 0.0 > gpar(nkref)%high
                 if (out_range(1)) then   ! torsione fuori range
                 !    write(0,*)'torsione fuori range'
                     newang = (gpar(nkref)%high - gpar(nkref)%low) / 2
                 endif
             else                          ! range periodico
                 gpar(nkref)%low = -0.5
                 gpar(nkref)%high = 0.5
                 gpar(nkref)%bound = 1          
             endif
             allocate(gpar(nkref)%pat(cryst(nph)%rot(i)%nat))
             gpar(nkref)%pat = cryst(nph)%rot(i)%pat
             gpar(nkref)%typepar = INT_TYPE
             gpar(nkref)%ph = nph
         endif
      enddo
!
!     Loop per gli shifts
!FIX THIS FOR SHIFTS
      do i=1,size(cryst(nph)%at)  
         do j=1,3
            if (shift_atom(i)%rcod(j) > 0) then
                nkref = nkref + 1             
                gpar(nkref)%val = 0.0
                shift_atom(i)%rcod(j) = nkref
                gpar(nkref)%low = 0.0
                !  gpar(nkref)%high = shiftatomval  - old code with shift 
                gpar(nkref)%high = 1.0
                gpar(nkref)%bound = 0             
                allocate(gpar(nkref)%pat(1))
                gpar(nkref)%pat(1) = i  
                gpar(nkref)%typepar = INT_TYPE
                gpar(nkref)%ph = nph
                write(strat,'(i0,a)')gpar(nkref)%pat(1),' ('//trim(cryst(nph)%at(gpar(nkref)%pat(1))%lab)//')'
                gpar(nkref)%str = trim(atomstring(j))//strat
            endif
         enddo
      enddo
!
!     Correzione per PO
      if (cryst(nph)%gcode > 0) then
          nkref = nkref + 1
          cryst(nph)%gpar = 1.0
          cryst(nph)%gcode = nkref
          gpar(nkref)%str = 'G'
          gpar(nkref)%val = 1.0
          gpar(nkref)%low = 0.0
          gpar(nkref)%high = 5.0
          gpar(nkref)%bound = 0             
          gpar(nkref)%typepar = PO_TYPE
          gpar(nkref)%ph = nph
      endif
!
!     Set  starting model for moves
      call copy_atoms(cryst(nph)%atm0,cryst(nph)%at)
   enddo
!
   end subroutine sa_prerefine 
   
!----------------------------------------------------------------------------------------------------

   subroutine gopt_set_ncycle(info,ncy)
!
!  Compute automatically number of moves for each step of temperature
!   
   USE arrayutil
   USE rotationmod
   USE variables, only: cryst
   USE crystal_phase
   type(opti_info_type), intent(in) :: info
   integer, intent(out)             :: ncy
   real                             :: const
   real                             :: cfrag
   real                             :: cdofe
   real                             :: crot
   real                             :: ctherm
   real                             :: cshifta
   real                             :: crotc
   integer                          :: flex,ntotfrag,nph
!
   const = 10.0
   cfrag = 3.0    ! contributo per ogni frammento oltre al primo
   cdofe = 2.0    ! contributo per ogni grado di liberta esterno
   crot  = 1.0    ! contributo per ogni rotazione
   ctherm = 0.1   ! contributo per ogni fattore termico
   cshifta = 0.1  ! contributo per ogni shift
   crotc  = 0.8   ! contributo per ogni rotazione concatenata
!   
!  Compute total number of refined fragment
   ntotfrag = 0
   do nph=1,numphase(cryst)
      ntotfrag = ntotfrag + cryst(nph)%numfrag()
   enddo
!
!corr   if(cryst(1)%numfrag() == 1 .and. cryst(1)%numrot() <= 2) then 
   if(ntotfrag == 1 .and. info%nprot <= 2) then 
!           
!     Possibile formula: 20 + 10*2**numrot
      ncy = 20 + 10*2**info%nprot + const*(info%npb + info%npshift)
   else           
!
!     Molecular flexibility
      flex = 0
      do nph=1,numphase(cryst)
         flex = flex + num_connected_rotation(cryst(nph)%rot)
      enddo
! 
      ncy = nint ( const*(cfrag*(ntotfrag-1) + cdofe*info%dofe +  &
                   crot*info%nprot  + crotc*flex   +              &
                   ctherm*info%npb  + cshifta*info%npshift) )  
   endif                
!
   end subroutine gopt_set_ncycle
      
!----------------------------------------------------------------------------------------------------

   subroutine gopt_set_algo(info)
   USE variables, only: cryst
   USE crystal_phase
   type(opti_info_type), intent(inout) :: info
   integer                             :: nph,num_frag
!
   num_frag = 0
   do nph=1,numphase(cryst)
      num_frag = num_frag + cryst(nph)%numfrag()
   enddo
   if (info%nprot <= 5 .and. num_frag <= 2) then
       info%algo = BBBC
   else
       info%algo = SA_LOCAL
   endif
!
   end subroutine  gopt_set_algo

!----------------------------------------------------------------------------------------------------

   subroutine xpar_to_atm(x,crystal)
!
!  Convert x parameters in crystal structure parameters
!   
   USE fastfcal
   USE atom_type_util
   USE cgeom
   USE rrestr
   USE constraints
   USE trig_constants
   USE fragmentmod
   USE symmgrid
   USE crystal_phase
!
   real, dimension(:), intent(in)             :: x       ! parametri SA
   type(crystal_phase_t), intent(inout)       :: crystal
   integer                                    :: j
   real, dimension(3)                         :: xtra
   real, dimension(3)                         :: xrot
   integer                                    :: scodt,scodr
   integer                                    :: i
   real, dimension(3)                         :: pa1,pa2
   type(atom_type), dimension(:), allocatable :: atmr 
   real                                       :: theta 
   integer                                    :: natrot
   integer                                    :: natom
   integer                                    :: nshift
   real, dimension(3)                         :: xshift
!
   crystal%at = crystal%atm0
   natom = crystal%natoms()   
!
!  Thermal factor
   do j=1,natom
      if (crystal%at(j)%rcod(4) > 0) then      ! B
          crystal%at(j)%biso = x(crystal%atm0(j)%rcod(4))
      endif
   enddo
   do i=1,nconstraints(crystal%sconstr)
      if (crystal%sconstr(i)%ref > 0) then
          select case (crystal%sconstr(i)%code) 
             case (CONSTR_B)
                crystal%at(crystal%sconstr(i)%vat)%biso = crystal%sconstr(i)%coef*x(crystal%sconstr(i)%ref)
             case (4)
          end select
      endif
   enddo
!
!  Frammenti
   do i=1,crystal%numfrag()
      scodt = 0
      do j=1,3                           ! traslazione
         if (crystal%frag(i)%rcod(j) > 0) then
             xtra(j) = x(crystal%frag(i)%rcod(j))
             scodt = scodt + 1
         else
             xtra(j) = 0
         endif
      enddo
!      
      scodr = 0
      do j=4,6                           ! rotazione
         if (crystal%frag(i)%rcod(j) > 0) then
             xrot(j-3) = x(crystal%frag(i)%rcod(j))
             scodr = scodr + 1
         else
             xrot(j-3) = 0
         endif
      enddo      
      !write(6,*)'xrot=',xrot(:)
      if (scodt + scodr > 0) then
          !call move_fragment(atom,fragment(i),xtra,xrot,scodt,scodr)      
          !atom(fragment(i)%pos) = fragment(i)%atm0
          call move_fragment(crystal%at,crystal%frag(i)%pos,xtra,xrot,scodt,scodr,crystal%cell,crystal%frag(i)%rat)
!      else
!
!         se non affini i DOF esterni riposiziona il frammento nell'origine      
!corr-old          atom(fragment(i)%pos) = fragment(i)%atm0
          !do ic=1,3
          !   atom(fragment(i)%pos)%xc(ic) = fragment(i)%atm0%xc(ic)
          !enddo
      endif          
   enddo   
!
!  Rotazioni
   do i=1,crystal%numrot()
      if (crystal%rot(i)%rcod > 0) then
          pa1 = matmul(crystal%cell%get_ortom(),crystal%at(crystal%rot(i)%pax(1))%xc)
          pa2 = matmul(crystal%cell%get_ortom(),crystal%at(crystal%rot(i)%pax(2))%xc)
          theta = 2.0*pi*x(crystal%rot(i)%rcod)
          natrot = crystal%rot(i)%nat
          call new_atoms(atmr,crystal%natoms())
          atmr(:natrot) = crystal%at(crystal%rot(i)%pat)
          call rotate_atoms(atmr(:natrot),pa1,direction_cos(pa1,pa2),theta,crystal%cell)
          crystal%at(crystal%rot(i)%pat) = atmr(:natrot)
      endif
   enddo
!
!  Shift on atoms
   do i=1,natom 
      xshift(:) = 0
      nshift = 0
      do j=1,3
         if (shift_atom(i)%rcod(j) > 0) then
             xshift(j) =  x(shift_atom(i)%rcod(j))
             nshift = nshift + 1
         endif
      enddo
      if (nshift > 0) then
          crystal%at(i)%xc = crystal%at(i)%xc + matmul(crystal%cell%get_ortoi(),xshift)
      endif
   enddo
!
!old   if (crystal%doc_active() .or. crystal%numresbv() > 0) then
!old       if (crystal%doc_active_all() .or. crystal%numresbv() > 0) then
!old       !write(71,*)'GPAR=',hpar,gpar(hpar)%pat
!old!old_occ       call make_symmetry_table(atom,crystal%spg,asymtab)
!old           !call make_symmetry_grid_all(crystal%at,crystal%spg)
!old           
!old       else
!old           !call make_symmetry_grid_vet(crystal%at,crystal%spg,crystal%vetdoc)
!old       endif
!old   endif
!   
!  Dynamical occupancy correction
   if (crystal%doc_active()) then
       call crystal%make_symmetry_cell()
       call crystal%allocate_grid(cutoff=1.0)
       if (crystal%doc_active_all()) then
!old_occ           call compute_doc_st_all(atom,crystal%spg,crystal%cell%get_g(),1.0,asymtab)
           !call compute_doc_grid_all(crystal%at,crystal%cell,1.0)
           call crystal%fill_grid()
           call crystal%compute_doc(1.0)
           call fcalc_for_tfocc(crystal%ref,crystal%at,[(i,i=1,natom)],crystal%sfact,crystal%kpscat)
       else
           !call compute_doc_atoms(atom,crystal%spg,crystal%cell%get_g(),1.0)
           !call compute_doc(atom,crystal%spg,crystal%cell%get_g(),1.0,crystal%vetdoc)  ! test also this!
           !call compute_doc_grid_vet(crystal%at,crystal%cell,1.0,crystal%vetdoc)
           call crystal%fill_grid(crystal%vetdoc)
           call crystal%compute_doc(1.0,crystal%vetdoc)
           call fcalc_for_tfocc(crystal%ref,crystal%at(crystal%vetdoc),crystal%vetdoc,crystal%sfact,crystal%kpscat)
       endif
!TODO:   call fcalc_for_tfocc is unusefull when the par is B
   endif
!
!  Orientazione preferenziale
   if (crystal%gcode > 0) then
       crystal%gpar = x(crystal%gcode)
   endif
!  
   end subroutine xpar_to_atm

!----------------------------------------------------------------------------------------------------

   subroutine xpar_to_crystals(x)
   use variables, only: cryst
   use crystal_phase
   real, dimension(:), intent(in) :: x
   integer                        :: nph
   do nph=1,numphase(cryst)
      call xpar_to_atm(x,cryst(nph))
   enddo
   end subroutine xpar_to_crystals

!----------------------------------------------------------------------------------------------------

   subroutine search_overlapped_atoms(atom,gmat,spg,modifi)
   USE CGEOM
   USE spginfom, only: spaceg_type
   USE atom_type_util
   type(atom_type), dimension(:), intent(inout) :: atom   
   real, dimension(3,3), intent(in)             :: gmat
   type(spaceg_type), intent(in)                :: spg
   logical, intent(in), optional                :: modifi
   logical                                      :: modif
   integer                                      :: natc
   real, dimension(3,spg%nsymop)                :: xeq
   real                                         :: dd
   integer                                      :: i,j,k
   real, dimension(3) :: ktra
!
   natc = size(atom)
   if (present(modifi)) then
       modif = modifi
   else
       modif = .true.
   endif
!
!corr   ivetocc(:) = 1
   do i=1,natc-1
      call get_equivalent1(atom(i)%xc,spg,xeq)   !!!!!,neqa)
!corr      if (all(vet /= i)) cycle
      if (atom(i)%doc) then
          do j=i+1,natc
!corr             if (all(vet /= j)) cycle
             if (atom(j)%doc) then
                 if (atom(i)%get_nz() == atom(j)%get_nz()) then
                     do k=1,spg%nsymop
                        if (k == 1 .and. j < i) cycle  ! combinazione già testata
                        !dd = xdisteq(xeq(:,k),atom(j)%xc,xgg)
                        call xdisteqs(atom(j)%xc,xeq(:,k),gmat,dd,ktra)
                        !dd = distanzaC(atom(j)%xc,xeq(:,k),gmat)
                        if (dd < DISTDOC) then
!corr                            ivetocc(i) = ivetocc(i) + 1
!corr                            ivetocc(j) = ivetocc(j) + 1
                            if (modif) then
                                !if (dd < 0.01) then   !! this is test
                                !    atom(j)%xc = atom(i)%xc   !!!FIXME - but it is necessary reconnect j atom if k/=1
                                !else
                                    atom(j)%xc = atom(j)%xc + ktra  ! ora i 2 atomi sono nella stessa posizione
                                !endif
                        !call xdisteqs(atom(j)%xc,xeq(:,k),gmat,dd,ktra)
                            !write(0,'(a,a,a,a,f0.4,a,i0)')'vicinid=',atom(i)%lab,atom(j)%lab,'dd=',dd,' op=',k
                            !         write(0,*)'atom=',trim(atom(i)%lab),atom(i)%xc
                            !         write(0,*)'atom=',trim(atom(j)%lab),atom(j)%xc
                            endif
                            !non funziona atom(j)%xc = xeq(:,k) - ktra
                        endif
                     enddo
                 endif
             endif
          enddo
      endif
   enddo
!corr   do i=1,natc
!corr      atom(i)%och = atom(i)%och/ivetocc(i)
!corr   enddo
!
   end subroutine search_overlapped_atoms

!----------------------------------------------------------------------------------------------------

   subroutine search_overlapped_atoms_print(atom,gmat,spg)
   USE General, only:lo
   USE CGEOM
   USE strutil
   USE nr
   USE spginfom, only: spaceg_type
   USE atom_type_util
   type(atom_type), dimension(:), intent(in) :: atom   
   real, dimension(3,3), intent(in)          :: gmat
   type(spaceg_type), intent(in)             :: spg
   integer                                      :: natc
   real, dimension(3,spg%nsymop)                :: xeq 
   real                                         :: dd
   integer                                      :: i,j,k
   real, dimension(3)                           :: ktra
   integer, dimension(size(atom), size(atom))   :: tableat
   real, dimension(size(atom), size(atom))   :: tabled   !!!!FIXME  da migliorare
   integer                                      :: nrem
   integer, dimension(size(atom))               :: vrem
   integer, allocatable, dimension(:)           :: iord
!
   natc = size(atom)
   tableat(:,:) = 0
   do i=1,natc
      call get_equivalent1(atom(i)%xc,spg,xeq)   !!!!!,neqa)
      do j=1,natc
         if (i==j) cycle
         if (atom(i)%get_nz() == atom(j)%get_nz()) then
             do k=1,spg%nsymop
                if (k == 1 .and. j < i) cycle  ! combinazione già testata
                call xdisteqs(atom(j)%xc,xeq(:,k),gmat,dd,ktra)
                if (dd < DISTDOC) then
                    !write(0,'(" WARNING: Atoms ",a," and ",a," are in the same position")')   &
                    !trim(atom(i)%lab),trim(atom(j)%lab)
                    tableat(i,j) = 1
                    tableat(j,i) = 1
                else if (dd < DISTOCC + 0.3) then
                    !write(0,'(" WARNING: Critical distance between ",a," and ",a," (d=",f0.3,")")')   &
                    !trim(atom(i)%lab),trim(atom(j)%lab),dd
                    tableat(i,j) = -1
                    tableat(j,i) = -1
                    tabled(i,j) = dd
                    tabled(j,i) = dd
                endif
             enddo
         endif
      enddo
   enddo
!
   do i=1,natc-1
      do j=i+1,natc
         if (tableat(i,j) == 1) then
             write(lo,'(" WARNING: Atoms ",a," and ",a," are in the same position")')   &
             trim(atom(i)%lab),trim(atom(j)%lab)
         elseif (tableat(i,j) == -1) then
             write(lo,'(" WARNING: Critical distance between ",a," and ",a," (d=",f0.3,")")')   &
             trim(atom(i)%lab),trim(atom(j)%lab),tabled(i,j)
         endif
      enddo
   enddo
!
   nrem = 0
   do i=1,natc-1
      do j=i+1,natc
         if (abs(tableat(i,j)) > 0) then
!             
!            cosa rimuovere: i o j?
             !wej = sum(abs(tableat(i,:))) 
             !wei = sum(abs(tableat(j,:)))
             !if (wei > wej) then
             !    write(lo,*)'posso rimuovere ',trim(atom(i)%lab),wei,wej
             !    tableat(i,:) = 0
             !    tableat(:,i) = 0
             !    exit
             !else
              nrem = nrem + 1
              vrem(nrem) = j
                 !!!!write(lo,*)'posso rimuovere ',trim(atom(j)%lab)
                 tableat(j,:) = 0
                 tableat(:,j) = 0
                 cycle
             !endif
         endif
      enddo
   enddo
!
!  Stampa gli atomi da eliminare
   if (nrem > 0) then
       if (nrem > 1) then   ! ordina il vettore vrem
           allocate(iord(nrem))
           call indexx(vrem(:nrem),iord)
           vrem(:nrem) = vrem(iord)
       endif
       call write_svet(atom(vrem(:nrem))%lab,nrem,lo,' The following atoms could be removed: ')
   endif
!
   end subroutine search_overlapped_atoms_print

!----------------------------------------------------------------------------------------------------

   real function tfunk(x,hpar)
   USE variables, only:cryst
   USE anti_bump
   USE rrestr
!
   real, dimension(:), intent(in) :: x
   integer, intent(in)            :: hpar
   integer, parameter             :: kpr = 0
   real                           :: chid
!
!  Passa x in atm
   call timesa(SATIME_MOVES)%tic()
   call xpar_to_crystals(x)
   call timesa(SATIME_MOVES)%toc()
!
   call timesa(SATIME_CF)%tic()
         
   tfunk = chisquared_xdata(hpar,ginfo)   !,cryst(1)%at,cryst(1)%cell,cryst(1)%ref)
!
!  Contributo restraints
   if (cryst(1)%numres() > 0) then
       chid=chisquared_res(cryst(1))
       tfunk = tfunk + chid
   endif
!
!  Contributo anti-bump
   if (cryst(1)%numresab() > 0) then
       tfunk = tfunk + cryst(1)%abset%weight*   &
               antibump_function(cryst(1)%abres,cryst(1)%at,cryst(1)%spg,cryst(1)%cell%get_g(),.false.)
   endif
         
   call timesa(SATIME_CF)%toc()
!   
   end function tfunk
      

!----------------------------------------------------------------------------------------------------

   subroutine gopt_statusbar(ntot,ntemp)
   USE messagemod
   USE simulated_anneal
   USE timer
   integer, intent(in)                        :: ntot
   integer, intent(in)                        :: ntemp
   real                                       :: init_time = 0
   real                                       :: end_time
   real                                       :: range_time = 1.0 ! intervallo di tempo dopo il quale updatare la statusbar
   logical                                    :: oktime 
!
   call timesa(SATIME_GRAPH)%tic() ! calcola il tempo impiegato dalle operazioni grafiche  
!
!  Controlla se e trascorso un tempo pari a range_time
   call cpu_time(end_time)
   oktime = end_time - init_time > range_time
   if (oktime) init_time = end_time
!   
!  Aggiorna informazioni sulla statusbar
   if (oktime .or. ntemp == numcyc_calc) then
       call write_message('Move n.',inum=ntot,pos=vmess(LINE_MOVES))
       call write_message('Time: '//secs_to_string(timesa(SATIME_TOT)%get_time(),.true.),pos=vmess(LINE_TIME))
       call progress_bar_update(cval=ntemp,fval=numcyc_calc)
   endif
!
   call timesa(SATIME_GRAPH)%toc() ! calcola il tempo impiegato dalle operazioni grafiche  
!   
   end subroutine gopt_statusbar   
      
!----------------------------------------------------------------------------------------------------

   real function ratio_value(fopt,gopti) result(ratio)
   USE RefinecomRef, only:rinfo
   type(opti_info_type), intent(in) :: gopti     
   real, intent(in)                 :: fopt
!
   if (.not.is_profile_function(gopti%cftype)) then
       !call sa_profile(sqrt(ref%fc))
       ratio = rinfo%rwp/gopti%rwp_leb    ! migliorare forse si puo' usare sole questa funzione
   else
       ratio = fopt/gopti%rwp_leb
   endif
!
   end function ratio_value

!----------------------------------------------------------------------------------------------------

   subroutine sa_profile(fc)
   USE variables, only:cryst
   USE rietv
   real, dimension(:), intent(in) :: fc
!
   cryst(1)%ref(:)%fc = fc(:)
   cryst(1)%ref(:)%pk = 10
   call Count0()   !cryst(1)%ref)
!
   end subroutine sa_profile

!----------------------------------------------------------------------------------------------------
   
   subroutine gopt_draw(sdraw,pdraw,gopti)
   USE view
   USE atom_type_util
   USE connect_mod
   USE ilmviewmod
   USE fouriermap
   USE reflection_type_util
   USE variables, only: cryst
   USE fragmentmod
   logical, intent(in)                         :: sdraw,pdraw
   type(opti_info_type), intent(in)            :: gopti    
   type(atom_type), dimension(:), allocatable       :: atmsav
   type(bond_type), dimension(:), allocatable  :: legm
   type(reflection_type), dimension(:), allocatable :: refmap
   integer                                     :: nfrag
!
   call timesa(SATIME_GRAPH)%tic ! calcola il tempo impiegato dalle operazioni grafiche  
!
!  Disegna la struttura
   if (sdraw) then
       nfrag = cryst(1)%numfrag()
       !corr if (nfrag > 1 .or. sadoc%active()) atmsav(:) = atm(:)
       if (nfrag > 1 .or. cryst(1)%doc_active()) call copy_atoms(atmsav,cryst(1)%at)
       if (nfrag > 1) call regroup_fragment(cryst(1)%frag,cryst(1)%at,cryst(1)%cell,cryst(1)%spg)
       if (cryst(1)%doc_active()) then
           !corr call search_overlapped_atoms(cryst(1)%at,cryst(1)%cell%get_g(),cryst(1)%spg,sadoc%vet,.true.)
           call search_overlapped_atoms(cryst(1)%at,cryst(1)%cell%get_g(),cryst(1)%spg,.true.)
       endif
       if (nfrag > 1) then 
           call copy_bonds(legm,cryst(1)%bond)
           call connect_fragment(cryst(1)%frag,cryst(1)%at,legm,cryst(1)%cell,.false.)
           call ilmview_draw(cryst(1)%at,legm,cryst(1)%elem,cryst(1)%cell,cryst(1)%spg,resize=.not.(ilmview_info(1,1) == 1))  ! no resize with visible cell
       else
           call ilmview_draw(cryst(1)%at,cryst(1)%bond,cryst(1)%elem,cryst(1)%cell,   &
                             cryst(1)%spg,resize=.not.(ilmview_info(1,1) == 1))  ! no resize with visible cell
       endif
!corr       if (nfrag > 1 .or. sadoc%active()) cryst(1)%at(:) = atmsav(:)
       if (nfrag > 1 .or. cryst(1)%doc_active()) cryst(1)%at(:) = atmsav(:)
       if (is_map_enabled()) then
           call map_set_centre(baricentro(cryst(1)%at))
           !refmap(:) = ref(:)
           call copy_ref(refmap,cryst(1)%ref)
           call sfcalc(refmap,cryst(1)%at,cryst(1)%spg,cryst(1)%elem)  ! compute fase
           call jav_show_map(map_type_selected(),refmap,'SA map',sogmap=sasogmap,fctype=2)
       endif
   endif
!   
!  Disegna il profilo
   if (pdraw) then
       call sa_profile(sqrt(cryst(1)%ref%fc))
       if (kzoomn) then
           call vedinew(iAct_sav,xzleft,xzright,xzleft,xzright,lsetstyle=.false.)
       else
           call vedinew(iAct_sav,1,gopti%nco,1,gopti%nco,lsetstyle=.false.)
       endif
   endif
   call timesa(SATIME_GRAPH)%toc()   
!
   end subroutine gopt_draw

!----------------------------------------------------------------------------------------------------
!corr
!corr   real function chisquared_xdata(hpar,gopti) result(chix)
!corr   integer, intent(in)              :: hpar
!corr   type(opti_info_type), intent(in) :: gopti    
!corr!
!corr   select case(gopti%cftype)
!corr      case (RWPC_COSTF)
!corr        chix = sqrt(chisquared_profile_new(hpar))/rwden
!corr
!corr      case (RWP_COSTF)
!corr        chix = sqrt(chisquared_profile(hpar,gopti))/rwden
!corr
!corr      case (CHI2FACT)
!corr        chix = sqrt(chisquared_factor(hpar))/rwden
!corr
!corr      case (CHI2I_SD)
!corr        chix = sqrt(chisquared_int(hpar))/rwden
!corr
!corr      case (CHI2I)
!corr        chix = sqrt(chisquared_int(hpar))/rwden
!corr
!corr      case (CHI2I_NEW)
!corr        chix = sqrt(chisquared_int_new(hpar,gopti))/rwden
!corr
!corr      case (CHI2_COSTF)
!corr        chix = chisquared_profile_new(hpar)
!corr
!corr      case (CHI2GRU)
!corr        chix = sqrt(chisquared_int_groups(hpar))/rwden
!corr   end select
!corr!
!corr   end function chisquared_xdata
!corr
!----------------------------------------------------------------------------------------------------

   real function chisquared_xdata(hpar,gopti) result(chix)
   USE fastfcal
!   USE RIETV
   USE variables, only: cryst   !,dataset
!   USE genlsq
   USE reflection_type_util
   USE crystal_phase
   integer, intent(in)              :: hpar
   type(opti_info_type), intent(in) :: gopti    
!corr   real                             :: scal 
   integer                          :: i,natom,nph
!corr   real                             :: sumyco,chi
!
!
!  Calcola fattori di struttura
   if (hpar > 0) then       
       nph = gpar(hpar)%ph
       if (gpar(hpar)%typepar == THERMALF_TYPE) then 
           call fcalc_from_thermalf(cryst(nph)%ref,cryst(nph)%at(gpar(hpar)%pat),gpar(hpar)%pat,cryst(nph)%spg,cryst(nph)%sfact)
       elseif (gpar(hpar)%typepar == PO_TYPE) then 
           call calcola_correzioneOP(cryst(nph)%ref,cryst(nph)%gpar,cryst(nph)%hklpo,cryst(nph)%cell,cryst(nph)%spg)
       else
           call fcalc_from_atm(cryst(nph)%ref,cryst(nph)%at(gpar(hpar)%pat),gpar(hpar)%pat,cryst(nph)%spg,cryst(nph)%sfact)
       endif    
   else
       do nph=1,numphase(cryst)
          natom = cryst(nph)%natoms()
          if (cryst(nph)%gcode > 0) then
              call calcola_correzioneOP(cryst(nph)%ref,cryst(nph)%gpar,cryst(nph)%hklpo,cryst(nph)%cell,cryst(nph)%spg)
          endif
!TODO:    necessary if all refined param are thermal factors
!test          if (cryst(nph)%doc_active())call fcalc_for_tfocc(cryst(nph)%ref,cryst(nph)%at,(/(i,i=1,natom)/))
          call fcalc_from_atm(cryst(nph)%ref,cryst(nph)%at,(/(i,i=1,natom)/),cryst(nph)%spg,cryst(nph)%sfact)
          call fcalc_save_atm((/(i,i=1,natom)/),cryst(nph)%spg,cryst(nph)%sfact,cryst(nph)%sfactc)
       enddo
   endif    
!!corr   call check_fcalc(cryst(1)%ref,cryst(1)%at,hpar)   ! diagnostica degli fc

   select case(gopti%cftype)
      case (RWPC_COSTF)
        chix = sqrt(chisquared_profile_new())/rwden

      case (RWP_COSTF)
        chix = sqrt(chisquared_profile(gopti))/rwden

      case (CHI2FACT)
        chix = sqrt(chisquared_factor())/rwden

      case (CHI2I_SD)
        chix = sqrt(chisquared_int())/rwden

      case (CHI2I)
        chix = sqrt(chisquared_int())/rwden

      case (CHI2I_NEW)
        chix = sqrt(chisquared_int_new(gopti))/rwden

      case (CHI2_COSTF)
        chix = chisquared_profile_new()

      case (CHI2GRU)
        chix = sqrt(chisquared_int_groups())/rwden
   end select
!
   end function chisquared_xdata

!----------------------------------------------------------------------------------------------------

   real function chisquared_profile(gopti) result(chi)
   USE counts
!corr   USE fastfcal
   USE genlsq, only:weilsq
   USE RIETV
   USE variables, only: dataset,cryst
!corr   USE atom_type_util
!corr   USE reflection_type_util
!corr   integer, intent(in) :: hpar
   type(opti_info_type), intent(in) :: gopti    
   real                :: scal 
!corr   integer             :: i,natom  
     real :: sumyco
     !real :: diff
!corr!
!corr!  Calcola fattori di struttura
!corr   if (hpar > 0) then       
!corr       if (gpar(hpar)%typepar == THERMALF_TYPE) then 
!corr           call fcalc_from_thermalf(cryst(1)%ref,cryst(1)%at(gpar(hpar)%pat),gpar(hpar)%pat,cryst(1)%spg,cryst(1)%sfact)
!corr       elseif (gpar(hpar)%typepar == PO_TYPE) then 
!corr           call calcola_correzioneOP(cryst(1)%ref,cryst(1)%gpar,cryst(1)%hklpo,cryst(1)%cell,cryst(1)%spg)
!corr       else
!corr           call fcalc_from_atm(cryst(1)%ref,cryst(1)%at(gpar(hpar)%pat),gpar(hpar)%pat,cryst(1)%spg,cryst(1)%sfact)
!corr       endif    
!corr   else
!corr       !natom = numatoms(atom)
!corr       natom = cryst(1)%natoms()
!corr       if (cryst(1)%gcode > 0) call calcola_correzioneOP(cryst(1)%ref,cryst(1)%gpar,cryst(1)%hklpo,cryst(1)%cell,cryst(1)%spg)
!corr!corr       if (gpo%rcod > 0) call calcola_correzioneOP(cryst(1)%ref,gpo%val,cryst(1)%hklpo,cryst(1)%cell)
!corr!corr       if (cryst(1)%doc_active())call fcalc_for_tfocc(cryst(1)%ref,atom,(/(i,i=1,natom)/))
!corr       call fcalc_from_atm(cryst(1)%ref,cryst(1)%at,(/(i,i=1,natom)/),cryst(1)%spg,cryst(1)%sfact)
!corr       call fcalc_save_atm((/(i,i=1,natom)/),cryst(1)%spg,cryst(1)%sfact,cryst(1)%sfactc)
!corr!corr       gpo_sav = cryst(1)%gpar
!corr   endif    
!corr!!!!   call check_fcalc(cryst(1)%ref,cryst(1)%at,hpar)   ! diagnostica degli fc
!
!  Calcola il profilo
   call genera_profilo2(sumyco)   !!!!,dataset(1)%nwave)
   scal = sumycossb/sumyco
!
   call sscal(gopti%nco,scal,cryst(1)%yc,1)  
!
!  Calcola Rwp
   chi = sum(weilsq(:gopti%nco)*(dataset(1)%ys-cryst(1)%yc(:gopti%nco))**2)
!new   chi = 0
!new   do i=1,ginfo%nco
!new      diff = ycossb(i) - scal*ycounttot(i)
!new      chi = chi + weilsq(i)*diff**2
!new   enddo
!   
   end function chisquared_profile
      
!----------------------------------------------------------------------------------------------------

   real function chisquared_profile_new() result(chi)
   USE RIETV
   USE variables, only: dataset,cryst
   USE genlsq
   USE crystal_phase
   USE type_constants, only: dp
   real :: scal 
   real :: sumyco
   integer                               :: nph,nc,ier
   real(dp), dimension(:), allocatable   :: bm
   real(dp), dimension(:,:), allocatable :: am,ainv
   real(dp), dimension(:), allocatable   :: crscal
!
   call genera_profilo2(sumyco)   !!!!!,dataset(1)%nwave)
   if (numphase(cryst) == 1) then
               !call scalefit(cryst(1)%yc,dataset(1)%ys,scal,chi,1/sqrt(dataset(1)%wei))
               !call scalefit(cryst(1)%yc,dataset(1)%ys,scal,chi,sqrt(abs(dataset(1)%ys)))
        !call scalefit(cryst(1)%yc,dataset(1)%ys,scal,chi)
       scal = sumycossb/sumyco
       nc = size(dataset(1)%ys)
!
       !call sscal(size(cryst(1)%yc),scal,cryst(1)%yc,1)  
       call sscal(nc,scal,cryst(1)%yc(:nc),1)  
!
!      Calcola Rwp
       chi = sum(dataset(1)%wei*(dataset(1)%ys-cryst(1)%yc(:nc))**2)
   else
!
!      Compute scale factor from lsq
       nc = size(dataset(1)%ys)
       allocate(am(nc,numphase(cryst)),bm(nc))
       allocate(crscal(numphase(cryst)),ainv(numphase(cryst),numphase(cryst)))
       do nph=1,numphase(cryst)
          am(:,nph) = sqrt(dataset(1)%wei)*cryst(nph)%yc
       enddo
       bm(:) = sqrt(dataset(1)%wei)*dataset(1)%ys
       call solve_linlsq(am,bm,crscal,ier,linv=.true.,ainv=ainv)
       cryst%scal = crscal
       dataset(1)%yc(:nc) = cryst(1)%scal*cryst(1)%yc
       do nph=2,numphase(cryst)
          dataset(1)%yc(:nc) = dataset(1)%yc(:nc) + cryst(nph)%scal*cryst(nph)%yc
       enddo
       chi = sum(dataset(1)%wei*(dataset(1)%ys-dataset(1)%yc(:nc))**2)
   endif
!   
   end function chisquared_profile_new

!----------------------------------------------------------------------------------------------------

   real function chisquared_factor() result(chi)
!corr   USE fastfcal
!corr   USE RIETV
!corr   USE unit_cell
   USE variables, only: cryst
!corr   USE reflection_type_util
!corr   integer, intent(in)                                :: hpar   
!corr   type(atom_type), dimension(:), intent(in)          :: atom
!corr   type(cell_type), intent(in)                        :: cell
!corr   type(reflection_type), dimension(:), intent(inout) :: refl
!corr   integer                                            :: i, nat
   real                                               :: scal
!corr!
!corr!  Calcola fattori di struttura
!corr   if (hpar > 0) then       
!corr       if (gpar(hpar)%typepar == THERMALF_TYPE) then 
!corr           call fcalc_from_thermalf(cryst(1)%ref,cryst(1)%at(gpar(hpar)%pat),gpar(hpar)%pat,cryst(1)%spg,cryst(1)%sfact)
!corr       elseif (gpar(hpar)%typepar == PO_TYPE) then 
!corr           call calcola_correzioneOP(cryst(1)%ref,cryst(1)%gpar,cryst(1)%hklpo,cryst(1)%cell,cryst(1)%spg)
!corr       else
!corr           call fcalc_from_atm(cryst(1)%ref,cryst(1)%at(gpar(hpar)%pat),gpar(hpar)%pat,cryst(1)%spg,cryst(1)%sfact)
!corr       endif    
!corr   else
!corr       !nat = size(atom)
!corr       nat = cryst(1)%natoms()
!corr       if (cryst(1)%gcode > 0) call calcola_correzioneOP(cryst(1)%ref,cryst(1)%gpar,cryst(1)%hklpo,cryst(1)%cell,cryst(1)%spg)
!corr!corr       if (cryst(1)%doc_active())call fcalc_for_tfocc(cryst(1)%ref,cryst(1)%at,(/(i,i=1,nat)/))
!corr       call fcalc_from_atm(cryst(1)%ref,cryst(1)%at,(/(i,i=1,nat)/),cryst(1)%spg,cryst(1)%sfact)
!corr       call fcalc_save_atm((/(i,i=1,nat)/),cryst(1)%spg,cryst(1)%sfact,cryst(1)%sfactc)
!corr!corr       gpo_sav = cryst(1)%gpar
!corr   endif    
!corr   !call check_fcalc(ref,cryst(1)%at,hpar)   ! diagnostica degli fc
!
   cryst(1)%ref%fc = sqrt(cryst(1)%ref%fc)
   scal = sum(cryst(1)%ref%fo) / sum(cryst(1)%ref%fc)
   chi = sum((cryst(1)%ref%fo - scal*cryst(1)%ref%fc)**2)    
!   
   end function chisquared_factor

 !----------------------------------------------------------------------------------------------------

   real function chisquared_int_groups() result(chi)
!corr   USE fastfcal
!corr   USE RIETV
!corr   USE unit_cell
   USE reflection_type_util
   USE variables, only: cryst
!corr   integer, intent(in)                                :: hpar   
!corr   type(opti_info_type), intent(in)                   :: gopti    
!corr   type(atom_type), dimension(:), intent(in)          :: atom
!corr   type(cell_type), intent(in)                        :: cell
!corr   type(reflection_type), dimension(:), intent(inout) :: refl
!corr   integer                                            :: i, nat
   real                                               :: scal
!corr!
!corr!  Calcola fattori di struttura
!corr   if (hpar > 0) then       
!corr       if (gpar(hpar)%typepar == THERMALF_TYPE) then 
!corr           call fcalc_from_thermalf(cryst(1)%ref,cryst(1)%at(gpar(hpar)%pat),gpar(hpar)%pat,cryst(1)%spg,cryst(1)%sfact)
!corr       elseif (gpar(hpar)%typepar == PO_TYPE) then 
!corr           call calcola_correzioneOP(cryst(1)%ref,cryst(1)%gpar,cryst(1)%hklpo,cryst(1)%cell,cryst(1)%spg)
!corr       else
!corr           call fcalc_from_atm(cryst(1)%ref,cryst(1)%at(gpar(hpar)%pat),gpar(hpar)%pat,cryst(1)%spg,cryst(1)%sfact)
!corr       endif    
!corr   else
!corr       !nat = size(atom)
!corr       nat = cryst(1)%natoms()
!corr       if (cryst(1)%gcode > 0) call calcola_correzioneOP(cryst(1)%ref,cryst(1)%gpar,cryst(1)%hklpo,cryst(1)%cell,cryst(1)%spg)
!corr!corr       if (cryst(1)%doc_active())call fcalc_for_tfocc(cryst(1)%ref,cryst(1)%at,(/(i,i=1,nat)/))
!corr       call fcalc_from_atm(cryst(1)%ref,cryst(1)%at,(/(i,i=1,nat)/),cryst(1)%spg,cryst(1)%sfact)
!corr       call fcalc_save_atm((/(i,i=1,nat)/),cryst(1)%spg,cryst(1)%sfact,cryst(1)%sfactc)
!corr!corr       gpo_sav = cryst(1)%gpar
!corr   endif    
!
   cryst(1)%ref%fc = sqrt(cryst(1)%ref%fc)
   call update_int_refg(cryst(1)%ref,sagref)
   scal = sum(sagref(:)%Io) / sum(sagref(:)%Ic)
   chi = sum((sagref(:)%Io - scal*sagref(:)%Ic)**2)
!corr   scal = sum(cryst(1)%ref%fo) / sum(cryst(1)%ref%fc)
!corr   chi = sum((cryst(1)%ref%fo - scal*cryst(1)%ref%fc)**2)    
!   
   end function chisquared_int_groups  

!----------------------------------------------------------------------------------------------------

   real function chisquared_int() result(chi)
!corr   USE fastfcal
!corr   USE RIETV
!corr   USE unit_cell
   USE variables, only: cryst
!corr   USE atom_type_util
!corr   USE reflection_type_util
!corr   integer, intent(in)                                    :: hpar   
!corr   type(atom_type), dimension(:), allocatable, intent(in) :: atom
!corr   type(cell_type)                                        :: cell
!corr   type(reflection_type), dimension(:), intent(inout)     :: refl
!corr   integer                                                :: natom
   real                                                   :: scal
!  type(reflection_type), dimension(size(ref)) :: ref1

!corr!
!corr!  Calcola fattori di struttura
!corr   if (hpar > 0) then       
!corr       if (gpar(hpar)%typepar == THERMALF_TYPE) then 
!corr           call fcalc_from_thermalf(cryst(1)%ref,cryst(1)%at(gpar(hpar)%pat),gpar(hpar)%pat,cryst(1)%spg,cryst(1)%sfact)
!corr       elseif (gpar(hpar)%typepar == PO_TYPE) then 
!corr           call calcola_correzioneOP(cryst(1)%ref,cryst(1)%gpar,cryst(1)%hklpo,cryst(1)%cell,cryst(1)%spg)
!corr       else
!corr           call fcalc_from_atm(cryst(1)%ref,cryst(1)%at(gpar(hpar)%pat),gpar(hpar)%pat,cryst(1)%spg,cryst(1)%sfact)
!corr       endif    
!corr   else
!corr       !natom = numatoms(atom)
!corr       natom = cryst(1)%natoms()
!corr       if (cryst(1)%gcode > 0) call calcola_correzioneOP(cryst(1)%ref,cryst(1)%gpar,cryst(1)%hklpo,cryst(1)%cell,cryst(1)%spg)
!corr!corr       if (cryst(1)%doc_active())call fcalc_for_tfocc(cryst(1)%ref,cryst(1)%at,(/(i,i=1,natom)/))
!corr       call fcalc_from_atm(cryst(1)%ref,cryst(1)%at,(/(i,i=1,natom)/),cryst(1)%spg,cryst(1)%sfact)
!corr       call fcalc_save_atm((/(i,i=1,natom)/),cryst(1)%spg,cryst(1)%sfact,cryst(1)%sfactc)
!corr!corr       gpo_sav = cryst(1)%gpar
!corr   endif    
!corr   !!!!call check_fcalc(cryst(1)%ref,cryst(1)%at,1)   ! diagnostica degli fc
!
   !intcal = cryst(1)%ref%m*cryst(1)%ref%fc 
   intcal = cryst(1)%ref%po*cryst(1)%ref%fc  ! fc al quadrato
   !intoss = cryst(1)%ref%fo**2
   !intcal = cryst(1)%ref%lp*cryst(1)%ref%fc 
   !sumintoss = sum(intoss)
       !write(6,*)'sum=',sumintoss,maxval(intoss),maxval(cryst(1)%ref%po),gpo%val
   scal = sumintoss / sum(intcal)
   chi = sum((intoss - scal*intcal)**2)    
   !     rwden = sqrt(sumintoss)/100.0
   !chi = sum(abs(intoss - scal*intcal)) 
!   chi = sum(dataset(1)%wei(:)*abs(intoss - scal*intcal)) 
!   
   end function chisquared_int
               
!----------------------------------------------------------------------------------------------------

   real function chisquared_int_new(gopti) result(chi)
!corr   USE fastfcal
!corr   USE RIETV
   USE variables, only: cryst,dataset
!corr   USE atom_type_util
!corr   USE reflection_type_util
!corr   integer, intent(in)              :: hpar   
   type(opti_info_type), intent(in) :: gopti    
!corr   integer                          :: i,natom,kal
   integer                          :: i,kal
   real                             :: scal
   integer                          :: nnn
   real                             :: opT
   real                             :: ffx
   real                             :: sumyco
!corr!
!corr!  Calcola fattori di struttura
!corr   if (hpar > 0) then       
!corr       if (gpar(hpar)%typepar == THERMALF_TYPE) then 
!corr           call fcalc_from_thermalf(cryst(1)%ref,cryst(1)%at(gpar(hpar)%pat),gpar(hpar)%pat,cryst(1)%spg,cryst(1)%sfact)
!corr       elseif (gpar(hpar)%typepar == PO_TYPE) then 
!corr           call calcola_correzioneOP(cryst(1)%ref,cryst(1)%gpar,cryst(1)%hklpo,cryst(1)%cell,cryst(1)%spg)
!corr       else
!corr           call fcalc_from_atm(cryst(1)%ref,cryst(1)%at(gpar(hpar)%pat),gpar(hpar)%pat,cryst(1)%spg,cryst(1)%sfact)
!corr       endif    
!corr   else
!corr       natom = cryst(1)%natoms()
!corr       if (cryst(1)%gcode > 0) call calcola_correzioneOP(cryst(1)%ref,cryst(1)%gpar,cryst(1)%hklpo,cryst(1)%cell,cryst(1)%spg)
!corr!corr       if (cryst(1)%doc_active())call fcalc_for_tfocc(cryst(1)%ref,cryst(1)%at,(/(i,i=1,natom)/))
!corr       call fcalc_from_atm(cryst(1)%ref,cryst(1)%at,(/(i,i=1,natom)/),cryst(1)%spg,cryst(1)%sfact)
!corr       call fcalc_save_atm((/(i,i=1,natom)/),cryst(1)%spg,cryst(1)%sfact,cryst(1)%sfactc)
!corr!corr       gpo_sav = cryst(1)%gpar
!corr   endif    
!corr   !!!!call check_fcalc(ref,cryst(1)%at,1)   ! diagnostica degli fc
!
   !intcal = ref%m*ref%lp*ref%fc    !!!!!**2      

   sumyco = 0.0
   opT = 1
   dataset(1)%yc(:gopti%nco) = 0.0
   kal = 1
!corr   do i=1,size(refl)
   do i=1,size(cryst(1)%ref)
      ffx = cryst(1)%ref(i)%m*cryst(1)%ref(i)%fc*opT*cryst(1)%ref(i)%lp(1)
      do nnn=cryst(1)%profi(i,kal)%iniz,cryst(1)%profi(i,kal)%ifin
         dataset(1)%yc(nnn) = dataset(1)%yc(nnn) + ffx*cryst(1)%profi(i,kal)%func(nnn-cryst(1)%profi(i,kal)%iniz+1)
      enddo 
   enddo 
   do i=1,size(cryst(1)%ref)
      intcal(i) = 0
      do nnn=cryst(1)%profi(i,kal)%iniz,cryst(1)%profi(i,kal)%ifin
         intcal(i) = intcal(i) + dataset(1)%yc(nnn) 
      enddo 
      sumyco = sumyco + intcal(i)
   enddo 
   scal = sumycossb/sumyco
   chi = sum((intoss - scal*intcal)**2)    
!   
   end function chisquared_int_new
               
!----------------------------------------------------------------------------------------------------
   subroutine check_fcalc(ref1,atmi,pos)
!
!  Controlla che il calcolo degli fc sia corretto comparandoli 
!  con quelli calcolati usando la routine standard fcalcan
!
   USE reflection_type_util
   USE variables, only: cryst
   USE atom_type_util
   type(reflection_type), dimension(:), intent(in) :: ref1
   type(atom_type), dimension(:), intent(in)       :: atmi
   integer, intent(in)                             :: pos
   type(reflection_type), dimension(size(ref1))    :: ref2
   real                                            :: absf
   integer                                         :: i
!   
   ref2 = ref1
   call sfcalc(ref2,atmi,cryst(1)%spg,cryst(1)%elem)
   do i=1,size(ref1)
      absf = abs(2.0*sqrt(ref1(i)%fc)-ref2(i)%fc)
      if (absf > 0.005) then
!
!         Scrivi solo il primo fc diverso      
          write(76,'(i10,2x,i4,3f10.3)')pos,i,2.0*sqrt(ref1(i)%fc),ref2(i)%fc,absf
          exit
      endif    
   enddo           
!   
   end subroutine check_fcalc
   
!----------------------------------------------------------------------------------------------------

   real function chisquared_bond(kpr,gmat) result(chi)
!
!  Calcola chi2 sulle distanze di legame
!   
   USE CGEOM
   USE variables, only: cryst
   USE connect_mod
!
   integer, intent(in) :: kpr
   real, dimension(3,3), intent(in) :: gmat
   real                :: d, diffd
   real                :: delta = 0.02
   integer             :: i   
!
   chi = 0.0
   do i=1,numbonds(cryst(1)%bond)
      d = distanzaC(cryst(1)%at(cryst(1)%bond(i)%n1)%xc,cryst(1)%at(cryst(1)%bond(i)%n2)%xc,gmat)
      diffd = d-cryst(1)%bond(i)%dist
      if (diffd >= delta) then
          chi = chi + ((diffd-delta)/cryst(1)%bond(i)%sigma)**2
      elseif (diffd <= -delta) then
          chi = chi + ((diffd+delta)/cryst(1)%bond(i)%sigma)**2
      endif 
      if (kpr > 0) write(6,'(a,i0,a,i0,a,f0.4,a,f0.4)')        &
       ' at=',cryst(1)%bond(i)%n1,'-',cryst(1)%bond(i)%n2,' dcalc=',d,' datteso=',cryst(1)%bond(i)%dist
   enddo
!   
   end function chisquared_bond
         
!----------------------------------------------------------------------------------------------------

   real function chisquared_res(crystal)
   USE CGEOM
   USE rrestr
   USE trig_constants
   USE crystal_phase
   USE bond_valence
   USE symmgrid
   type(crystal_phase_t), intent(inout)  :: crystal
   integer                               :: i,k
   real, dimension(3,crystal%spg%nsymop) :: xeq 
   real                                  :: dd,dmin
   real, dimension(3)                    :: ktra
   integer                               :: n1,n2,n3
   integer                               :: kloc
   real, dimension(3)                    :: x3, x1eq, x3eq
!corr   integer                               :: kst
   real                                  :: d12, d23, val
   real, dimension(3,3)                  :: gmat
   logical                               :: kpr = .false.
   integer                               :: ks
!
   crystal%chi2resa = 0
   crystal%chi2resd = 0
   crystal%chi2resbv = 0
   gmat = crystal%cell%get_g()
!
   if (crystal%numresbv() > 0) then
!
!      Make grid for BVS restraints
       if (.not.crystal%doc_active()) call crystal%make_symmetry_cell()
       call crystal%allocate_grid(maxval(crystal%bvpar(:,:)%rmax))
       call crystal%fill_grid()
   endif
!
   do i=1, crystal%numres()
      select case (crystal%res(i)%code)
         case (RESDIST)   ! distance
!
!          atomi coinvolti nel restraint distanza
           n1 = crystal%res(i)%na(1)
           n2 = crystal%res(i)%na(2)
!
!          Genera equivalenti di n2
           call get_equivalent1(crystal%at(n2)%xc,crystal%spg,xeq)  
!
!          Cerca la distanza minima tra n1 e equivalenti di n2
           !if (crystal%res(i)%sym(1) == 0) then
           !    kst = 2                  ! restraint between the same atoms (n1=n2) or n1-n2 are in bond: jump operator 1
           !    if (crystal%spg%nsymop == 1) cycle  ! not possible!
           !else
           !    kst = 1
           !endif
           !call xdisteqs(crystal%at(n1)%xc,xeq(:,kst),gmat,dmin,ktra)
           !do k=kst+1,crystal%spg%nsymop
           !   call xdisteqs(crystal%at(n1)%xc,xeq(:,k),gmat,dd,ktra)
           !   if (dd < dmin) dmin = dd
           !enddo
           !crystal%res(i)%val = dmin
!!!!!new
           dmin = huge(1.0) !!!!distanzaC(atom(n2)%xc,atom(n1)%xc,gmat)
           do k=1,crystal%spg%nsymop
              call xdisteqs(crystal%at(n1)%xc,xeq(:,k),gmat,dd,ktra)
              if (crystal%res(i)%sym(1) == 0 .and. k == 1) then
!
!                 check if cell translation is 0: ktra - (xeq(:,k)-atom(n2)%xc) == 0
                  ! write(71,*)'KTRA=',ktra
                  ! write(71,*)'XEQ=',xeq(:,k)
                  ! write(71,*)'XC=',crystal%at(n1)%xc,all((crystal%at(n1)%xc + ktra) - xeq(:,k) <= epsilon(1.0))
                  ! write(71,*)'DIFF=',(crystal%at(n1)%xc + ktra) - xeq(:,k)
                  if (all((crystal%at(n1)%xc + ktra) - xeq(:,k) <= epsilon(1.0))) cycle  ! if true n1 is not an equivalent for translation
              endif
              !if ((k == 1) .or. (k > 2 .and. dd <= 3)) then
              !    crystal%res(i)%val = dd
              !    crystal%chi2resd =  crystal%chi2resd + chi2res_value(crystal%res(i))   !!!!!,deltad)
              !endif
              !write(6,'(i5,3(a,f0.3))')i,'chi2resd=',crystal%chi2resd,' dist=',dd,' expected=',crystal%res(i)%targ
              if (dd < dmin) dmin = dd
           enddo
           crystal%res(i)%val = dmin
!!!!!new
!
!          Valuta contributo alla funzione costo
           crystal%chi2resd =  crystal%chi2resd + chi2res_value(crystal%res(i))   !!!!!,deltad)
           if (kpr) write(6,'(i5,3(a,f0.3))')i,'chi2resd=',crystal%chi2resd,' dist=',dmin,' expected=',crystal%res(i)%targ

         case (RESANGLE)   ! angle
           n1 = crystal%res(i)%na(1)
           n2 = crystal%res(i)%na(2)
           n3 = crystal%res(i)%na(3)
           if (n1 == n3) then
               ks = 2   ! e.g., Si1-O1 ... Si1
           else
               ks = 1
           endif
!
!          generate equivalent if sym = 1
           if (crystal%res(i)%sym(1) == 1) then
               call distance_equivalent(crystal%at(n1)%xc,crystal%at(n2)%xc,gmat,crystal%spg,d12,x1eq,ks)
           else
               d12 = distanzaC(crystal%at(n1)%xc,crystal%at(n2)%xc,gmat)
               x1eq = crystal%at(n1)%xc
           endif
           if (crystal%res(i)%sym(3) == 1) then
               call distance_equivalent(crystal%at(n3)%xc,crystal%at(n2)%xc,gmat,crystal%spg,d23,x3eq,ks)
           else
               d23 = distanzaC(crystal%at(n3)%xc,crystal%at(n2)%xc,gmat)
               x3eq = crystal%at(n3)%xc
           endif
           crystal%res(i)%val = rtod*angleC(x1eq,crystal%at(n2)%xc,x3eq,gmat)
!
!          Evaluate contribution to cost function
           crystal%chi2resa = crystal%chi2resa + chi2res_value(crystal%res(i))   !!!!,deltaa)
           if (kpr) write(6,'(4(a,f0.3))')    &
               'chi2a=',crystal%chi2resa,' angl=',crystal%res(i)%val,' expected=',crystal%res(i)%targ,' dim=',dmin
           !write(66,'(3(a,f0.3))')'chi2resa=',crystal%chi2resa,' angl=',crystal%res(i)%val,' expected=',crystal%res(i)%targ

         case (RESBV)   ! bond valence
           n1 = crystal%res(i)%na(1)
           call crystal%compute_bvs(n1,val)
           crystal%res(i)%val = val
           crystal%chi2resbv = crystal%chi2resbv + chi2res_value(crystal%res(i))
           !write(70,*)'VAL(',crystal%at(n1)%glab()//')=',crystal%res(i)%val,crystal%chi2resbv

         case (10000)   ! angle
!
!          atomi coinvolti nel restraint angolo
           n1 = crystal%res(i)%na(1)
           n2 = crystal%res(i)%na(2)
           n3 = crystal%res(i)%na(3)
!
!          Genera equivalenti di n3
           call get_equivalent1(crystal%at(n3)%xc,crystal%spg,xeq)  !!!!!,neqa)
!
!          Cerca l'equivalente a distanza minima tra n2-n3
           call xdisteqs(crystal%at(n2)%xc,xeq(:,1),gmat,dmin,ktra)
           !call xdisteqs(xeq(:,1),atom(n2)%xc,gmat,dmin,ktra)
           kloc = 1
           do k=2,crystal%spg%nsymop
              call xdisteqs(crystal%at(n2)%xc,xeq(:,k),gmat,dd,ktra)
              !call xdisteqs(xeq(:,k),atom(n2)%xc,gmat,dd,ktra)
              if (dd < dmin) then
                  dmin = dd
                  kloc = k
              endif
           enddo
!
!         valuta angolo n1-n2-n3
          !xeq(:,kloc) = xeq(:,kloc) + ktra(:)
          x3(:) = crystal%at(n2)%xc + ktra(:)
            !write(6,*)'distanza=',distanzaC(atom(n2)%xc,x3,gmat),dmin
          !rest(i)%val = rtod*angleC(atom(n1)%xc,atom(n2)%xc,xeq(:,kloc),gmat)
          crystal%res(i)%val = rtod*angleC(crystal%at(n1)%xc,crystal%at(n2)%xc,x3,gmat)
!
!         Valuta contributo alla funzione costo
          crystal%chi2resa = crystal%chi2resa + chi2res_value(crystal%res(i))   !!!!,deltaa)
          if (kpr) write(6,'(4(a,f0.3))')    &
              'chi2a=',crystal%chi2resa,' angl=',crystal%res(i)%val,' expected=',crystal%res(i)%targ,' dim=',dmin
           !write(66,'(3(a,f0.3))')'chi2resa=',crystal%chi2resa,' angl=',rest(i)%val,' expected=',rest(i)%targ
      end select
   enddo
   chisquared_res = crystal%chi2resa + crystal%chi2resd + crystal%chi2resbv
   crystal%chi2res = chisquared_res 
!
   end function chisquared_res

!----------------------------------------------------------------------------------------------------        

   real function chi2res_value(rest)
   type(restraint_type), intent(in) :: rest
   real                             :: rplus,rless
!
   rplus = rest%targ + rest%sigma
   rless = rest%targ - rest%sigma
   if (rest%val <= rless) then
       chi2res_value = rest%wei*((rest%val-rless))**2
   elseif (rest%val >= rplus) then
       chi2res_value = rest%wei*((rest%val-rplus))**2
   else
       chi2res_value = 0
   endif
!
   end function chi2res_value

!----------------------------------------------------------------------------------------------------        

   subroutine chi2res_print(crystal,kpr)
   USE crystal_phase
   USE rrestr
   type(crystal_phase_t), intent(inout) :: crystal
   integer, intent(in)                  :: kpr
   write(kpr,'(2x,a,f15.4)')    'Chi-squared (restraints contribution): ',crystal%chi2res
   if (crystal%numres(RESDIST) > 0 .and. crystal%numres(RESDIST) /= crystal%numres()) then
       write(kpr,'(2x,a,f15.4)')'                            Distances: ',crystal%chi2resd
   endif
   if (crystal%numres(RESANGLE) > 0 .and. crystal%numres(RESANGLE) /= crystal%numres()) then
       write(kpr,'(2x,a,f15.4)')'                               Angles: ',crystal%chi2resa
   endif
   if (crystal%numres(RESBV) > 0 .and. crystal%numres(RESBV) /= crystal%numres()) then
       write(kpr,'(2x,a,f15.4)')'                         Bond valence: ',crystal%chi2resbv
   endif
   end subroutine chi2res_print

!----------------------------------------------------------------------------------------------------        
   
   subroutine gopt_print_info(info,sc,unt)
!
!  Print general information
!   
   USE mpi_prog
   USE variables, only: cryst
   USE reflection_type_util
   USE crystal_phase
   USE strutil
   type(opti_info_type), intent(in)         :: info
   type(SimAnn_Conditions_type), intent(in) :: sc                     
   integer, intent(in)                      :: unt
   integer                                  :: nph
!
   if (mpi_prog_rank /= mpi_prog_master) return
!
   write(unt,'(/1x,a,i0)')         'Number of CPU-cores:                 ',mpi_prog_numprocs
   write(unt,'(1x,a,i0)')          'Number of parameters:                ',info%npar
   write(unt,'(3x,a,i0)')            'DOF(ext):                          ',info%dofe
   write(unt,'(3x,a,i0)')            'DOF(int):                          ',info%dofi
   if (info%npb > 0) then
       write(unt,'(3x,a,i0)')        'Thermal factors:                   ',info%npb
   endif
   if (info%npshift > 0) then
       write(unt,'(3x,a,i0)')        'Shifts on atoms:                   ',info%npshift
   endif
   write(unt,'(1x,a,f0.2,a,f0.2)') 'Used data range:                     ',info%ttmin,'-',info%ttmax
   if (numphase(cryst) > 1) then
       do nph=1,numphase(cryst)
          write(unt,'(1x,a,i0,a)')'Settings for phase ',nph,' ('//trim(cryst(nph)%cr_name)//')'
          write(unt,'(1x,a,i0,a,i0)') 'Number of used reflections:          ',cryst(nph)%numref(),'/',numrefl(cryst(nph)%refc)
          call cryst(nph)%print_doc(unt)
          write(unt,*)
       enddo
   else
       write(unt,'(1x,a,i0,a,i0)') 'Number of used reflections:          ',cryst(1)%numref(),'/',numrefl(cryst(1)%refc)
       call cryst(1)%print_doc(unt)
   endif
   write(unt,'(1x,a,a)')           'Resolution:                          ',r_to_s(info%res,3)//' ang'
   write(unt,'(1x,a)')             'Cost function:                       '//trim(info%cost_function_name)
   if (cryst(1)%gcode > 0) then
       write(unt,'(1x,a,3(1x,i0))')'P.O. correction is active for plane: ',cryst(1)%hklpo(:)
   endif
   if (info%algo == SA_LOCAL .and. mpi_prog_numprocs > 1 .and. sc%cooperate) then
       write(unt,'(1x,a)')         'Algorithm:                           '//'Parallel '//algoname_long(info%algo)
   else
       write(unt,'(1x,a)')         'Algorithm:                           '//algoname_long(info%algo)
   endif
!   
   end subroutine gopt_print_info
   
!----------------------------------------------------------------------------------------------------        
   
   subroutine gopt_print_parameters(xopt,par,unt,title)
!
!  Stampa lista dei parametri affinati
!   
   USE strutil, only:centra_str
   USE atom_type_util, only:norm_torsion
   USE rrestr
   USE constraints
   USE trig_constants
   USE variables, only: cryst
   USE crystal_phase
   real, dimension(:), intent(in)             :: xopt
   type(param_type), dimension(:), intent(in) :: par
   integer, intent(in)                        :: unt
   integer                                    :: i,j,nph
   integer                                    :: scod,icod   
   integer                                    :: natts
   character(len=*), intent(in), optional     :: title
!
   if (present(title)) then
       write(unt,'(/a)')centra_str(title,80)
   else        
       write(unt,*)    
   endif        
   write(unt,'(20x,a)')'        Value        Step length          Bounds'
!
   do nph=1,numphase(cryst)
!
!     Stampa per traslazione e rotazione frammenti   
      do i=1,cryst(nph)%numfrag()
         scod = count(cryst(nph)%frag(i)%rcod > 0)   ! numero di paramtri affinati per il frammento i
         if (scod > 0) then
             write(unt,'(1x,a,i0)')'External DOF for fragment n.',i
             do j=1,6
                icod = cryst(nph)%frag(i)%rcod(j)
                if (icod > 0) then             
                    write(unt,'(3x,a,t20,f15.4,f15.4,2f13.2)')trim(par(icod)%str),xopt(icod),   &
                          par(icod)%stp,par(icod)%low,par(icod)%high
                endif
             enddo
         endif
      enddo
!
!     Stampa per rotazioni interne ai frammenti
      if (cryst(nph)%numrot() > 0) then
          scod = count(cryst(nph)%rot%rcod > 0)
          if (scod > 0) then
              write(unt,'(1x,a)')'Internal DOF'
              do i=1,cryst(nph)%numrot()
                 icod = cryst(nph)%rot(i)%rcod
                 if (icod > 0) then
                     write(unt,'(3x,a,t20,f15.4,f15.4,2f13.2)')trim(par(icod)%str),                                        &
                                                               norm_torsion(rtod*twopi*xopt(icod)+cryst(nph)%rot(i)%torsval),  &
                                                               par(icod)%stp,                                              &
                                                               cryst(nph)%rot(i)%trbound(1),cryst(nph)%rot(i)%trbound(2)
                 endif
              enddo
          endif
      endif
   enddo
!!!FIX THIS FOR SHIFTS
!
!  Stampa per shift sugli atomi
   scod = 0
!corr   natts = size(at)
   natts = cryst(1)%natoms()
   do i=1,natts 
      if (count(shift_atom(i)%rcod > 0) > 0) then
          scod = 1
          exit
      endif 
   enddo
   if (scod > 0) then
       write(unt,'(1x,a)')'Shift on atoms'
       do i=1,natts  
          do j=1,3
             icod = shift_atom(i)%rcod(j)
             if (icod > 0) then
                 write(unt,'(3x,a,t20,f15.4,f15.4,2f13.2)')trim(par(icod)%str),xopt(icod),par(icod)%stp,par(icod)%low,par(icod)%high
             endif
          enddo
       enddo        
   endif
!
   do nph=1,numphase(cryst)
!
!     Stampa per parametri termici
      if (count(cryst(nph)%at%rcod(4) > 0) > 0) then
          write(unt,'(1x,a)')'Thermal parameters'
          !do j=1,size(at)
          do j=1,cryst(nph)%natoms()
             icod = cryst(nph)%at(j)%rcod(4)
             if (icod > 0) then      
                 write(unt,'(3x,a,t20,f15.4,f15.4,2f13.2)')trim(par(icod)%str),xopt(icod),par(icod)%stp,par(icod)%low,par(icod)%high
             endif
          enddo       
      endif
      do i=1,nconstraints(cryst(nph)%sconstr)
         icod = cryst(nph)%sconstr(i)%ref
         if (icod > 0) then
             select case (cryst(nph)%sconstr(i)%code) 
                case (CONSTR_B)
                  write(unt,'(3x,a,t20,f15.4,f15.4,2f13.2)')trim(par(icod)%str),xopt(icod),  &
                        par(icod)%stp,par(icod)%low,par(icod)%high
             end select
         endif
      enddo
!
!     Orientazione Preferenziale
      if (cryst(nph)%gcode > 0) then
          write(unt,'(1x,a)')'Preferred Orientation'
          icod = cryst(nph)%gcode
          write(unt,'(3x,a,t20,f15.4,f15.4,2f13.2)')trim(par(icod)%str),cryst(nph)%gpar,par(icod)%stp,par(icod)%low,par(icod)%high
      endif
   enddo
!   
   end subroutine gopt_print_parameters
      
!----------------------------------------------------------------------------------------------------        
#if MPI
   subroutine simann(gopti,sc,bb_cond,extrac)
!
!  Routine principale: esegue multirun di SA, stampa parametri iniziali e finale, gestisce la lista dei modelli
!   
   USE simulated_anneal   !, only: sa_print_conditions,sa,sa_glob_old,sa_print_final
   USE variables, only:cryst
   USE General, only:fname,lo,OUT_FILE
   USE TestModel, only:vett_test,rmsd
   USE compare_struct
   USE atom_type_util, only: print_atoms, calcola_occ, make_symmetry_table
   USE RefinecomRef, only:rinfo
   USE strutil, only:s_s_delete
   USE Molcom
   !USE harmony_search
   USE bb_bc    !, only: bb_bc_condition,bb_bc_algo,bb_bc_print_conditions
   USE rand_mod
   USE messagemod
   USE model_util
   USE anti_bump, only: antibump_function, print_antibump
   USE rrestr, only: nrestraints,print_restraints,print_restraints_all,RESBV
   USE Conteggi, only: npunti
   USE connect_mod
   USE mpi_prog
   USE mpi
   USE fragmentmod, only: regroup_fragment, connect_fragment
   USE constraints, only: print_constraints
   USE grexpomod
   USE timer
   USE reflection_type_util
   type(opti_info_type), intent(inout)         :: gopti    
   type(SimAnn_Conditions_type), intent(inout) :: sc
   type(bb_bc_condition), intent(in)           :: bb_cond                     
   logical, intent(in)                         :: extrac
   real, dimension(gopti%npar)                 :: xopt
   real                                        :: fopt
   real                                        :: tfin
   integer                                     :: nfcnev
   integer                                     :: nacc, nobds 
   integer                                     :: i,j
   type(param_type), dimension(size(gpar))     :: gparini
   type(atom_type), dimension(:), allocatable  :: atbest
   real, dimension(7)                          :: infomodel 
   real                                        :: timerun
   integer                                     :: kevent
   integer                                     :: nsol      
   real                                     :: distnsol
   integer, dimension(0:6)                       :: vcolor 
   integer :: pcol
   type(atom_type), dimension(:), allocatable  :: atmr
   logical                                     :: is_profile
   integer                                     :: nplotr
   real                                        :: rappo
   real                                        :: resval
   integer                                     :: xdim,ydim
   type(bb_bc_condition)                       :: bcond
   type(bond_type), dimension(:), allocatable  :: legm
   real                                        :: frun_min, frun_max, frun_ave
   integer :: sum_eval
   integer :: neval
   integer :: nsol8
   integer :: myrank,nrun,mpi_err
   real, dimension(:,:), allocatable :: mpi_sol,mpi_rec
   real(DP) :: timetot
   integer :: nsamodel
   logical :: restart_par
!
   gparini = gpar
   if (jscreen > 0) then
       if(npunti == 0) call s_s_delete(gopti%popt,'P')
   else
      gopti%popt = 'SE'
   endif
   if (gopti%algo == BBBC) then
       call set_bb_bc_condition(gopti,bcond,sc)
       numcyc_calc = sc%num_temps*sc%ns*sc%nt*gopti%npar*bb_cond%niter*bb_cond%nlocmax  ! calculated for progress bar
   else
       numcyc_calc = sc%t_ini*sc%ns*sc%nt*gopti%npar  ! for progress bar
   endif
!
   ginfo = gopti
!
!  Stampa condizioni iniziali
   if (mpi_prog_rank == mpi_prog_master) then
       if (gopti%algo == BBBC) then
           call bb_bc_print_conditions(bb_cond,6)
           call sa_print_conditions(sc,6,'Local search conditions')
       else
           call sa_print_conditions(sc,6,'Simulated Annealing conditions')
       endif
       write(6,'(1x,a,f0.3)')     'Initial cost function value:  ',tfunk(gpar%val,0)
       if (cryst(1)%numresab() > 0)   &
           write(6,'(1x,a,f0.3)') 'Anti-bump cost function:      ',  &
           cryst(1)%abset%weight*antibump_function(cryst(1)%abres,cryst(1)%at,cryst(1)%spg,cryst(1)%cell%get_g(),.false.)
       if (cryst(1)%numres() > 0) then
           write(6,'(1x,a,f0.3)') 'Restraints cost function:     ',chisquared_res(cryst(1))
           call print_restraints_all(lo,cryst(1)%res,cryst(1)%at,cryst(1)%cell)
       endif
       call print_antibump(lo,cryst(1)%abres,cryst(1)%at,cryst(1)%cell)
       call print_constraints(cryst(1)%sconstr,cryst(1)%at,lo,.true.,.false.)
       call gopt_print_parameters(gpar%val,gpar,6,'Starting refined parameters')
!      
       if (print_tb) then
           write(6,'(4(a,i0),a,f0.2)')STRTAG//' DOFext=',gopti%dofe,' DOFint=',gopti%dofi,   &
                                       ' NREF=',cryst(1)%numref(),'/',numrefl(cryst(1)%refc),' RES=',gopti%res
           if (cryst(1)%is_known()) then
               write(6,'(a)')STRTAG//' NRUN    found     dist      RMSD       CF       n.moves          time'   
           else
               write(6,'(a)')STRTAG//' NRUN      CF       n.moves          time'   
           endif
       endif
   endif
   call new_models(cryst(1)%model,gopti%nrun)
   nsamodel = 0
!
!  Operazione preliminari per grafico della CF
   is_profile = is_profile_function(gopti%cftype)
   if (index(gopti%popt,'V') > 0) then  
       vcolor = (/color_BROWN,color_LIGHTBLUE,color_LIGHTRED,color_LIGHTGREEN,     &
                  color_LIGHTCYAN,color_LIGHTMAGENTA,color_YELLOW/)
       call gr_create_graph(grgraph)
       call gr_set_graph(grgraph,stringx='moves',stringy=trim(gopti%cost_function_name_short),xscale=10000.0)
       call gr_set_graph(grgraph,y_resize=.true.,y_resize_val=150.0)
       if (is_profile) then
           nplotr = gopti%nrun + 1
       else
           nplotr = gopti%nrun
       endif
       if (allocated(grplot)) then
           if (size(grplot) /= nplotr) then
               deallocate(grplot)
               allocate(grplot(nplotr))
           endif
       else
           allocate(grplot(nplotr))
       endif
       do i=1,gopti%nrun
          call gr_new_plot(grplot(i),1000)
          pcol = mod(i,size(vcolor)) 
          call gr_set_plot(grplot(i),colorline=vcolor(pcol))
       enddo
!
!      Per visualizzare la linea dell'Rwp alla fine dell'affinamento
       if (is_profile) then
           call gr_new_plot(grplot(gopti%nrun+1),2)
           call gr_set_plot(grplot(gopti%nrun+1),colorline=color_LIGHTRED)
           call gr_add_point_plot(grplot(gopti%nrun+1),0.,gopti%rwp_leb)
           call gr_add_point_plot(grplot(gopti%nrun+1),1.,gopti%rwp_leb)
       endif
!
       call new_viewer_size(xdim,ydim)
       call gr_open_viewer(xdim,ydim,'CF vs. number of moves',newv=.true.)
       call gr_select_viewer(1)  ! serve se no disegna su entrambi i viewers 
       call gr_setbackground(color_BRIGHTWHITE)
       call gr_select_viewer(0)
   endif
!   
!  gestione messaggi
   if (.not.cryst(1)%is_known()) call clear_messages(8) ! se la struttura e' incognita la riga 8 non viene usata  
!   
   myrank = mpi_prog_rank + 1
   nrun = mpi_prog_ntrials(gopti%nrun)
   gopti%nrun = max(gopti%nrun,mpi_prog_numprocs)
   if (mpi_prog_rank == 0 .and. mpi_prog_numprocs > 1) then
       write(0,'(2x,a,i0,a)')'Direct space method will run on ',mpi_prog_numprocs,' cpu-cores ...'
   endif
   allocate(mpi_rec(gopti%npar+2,gopti%nrun),source=0.0)
   allocate(mpi_sol(gopti%npar+2,nrun),source=0.0)   ! save solutions for each cpu 
!
   allocate(atmr(size(cryst(1)%at)))
   nsol = 0
   distnsol = 0
   gopti%fmin = huge(1.0)
   gopti%rmin = 1
   neval = 0
   sum_eval = 0
   restart_par = .true.   ! set false to use the parameters of the previous run
#ifdef MPI
   do i=1,nrun          ! inizio loop sui run di SA
#else
   do i=1,gopti%nrun          ! inizio loop sui run di SA
#endif
      call timesa(SATIME_RUN)%tic()
#ifdef MPI
      if (mpi_prog_rank == mpi_prog_master) then
          write(6,'(/2x,80("=")/,25x,"** ",a," Run n.",i0," **")')trim(algoname_long(gopti%algo)),i
      endif
#else
      write(6,'(/2x,80("=")/,25x,"** ",a," Run n.",i0," **")')trim(algoname_long(gopti%algo)),i
#endif
      if (restart_par .or. i == 1) then
          gpar(:)%val = gparini(:)%val
          gpar(:)%stp = gparini(:)%stp
          gopti%nrunc = i         ! memorizza a che run sono
      endif
!
!     Initialize random number generator
      if (gopti%seed == 0) then
          call Init_Ran(gopti%seed)  ! seed from system clock
      else
!bug on seed          call Init_Ran(gopti%seed+i-1+mpi_prog_rank)
          call Init_Ran(gopti%seed+nrun*mpi_prog_rank+i)
      endif
!
!     Applica ottimizzazione dei parametri con SA      
      if (gopti%algo == SA_LOCAL) then
          call sa(gpar, sc, gopti, tfin, xopt, fopt, nacc, nfcnev, nobds, kevent, tfunk,safunc1,-1.0)   
      elseif (gopti%algo == SA_GLOBAL) then
          !call sa_glob(gpar, sc, gopti, tfin,fopt, nfcnev, kevent,safunc,safuncg)
          call sa_glob_old(gpar, sc, gopti, tfin,fopt, nfcnev, kevent,safunc,safunc1)
      elseif (gopti%algo == HARMONYS) then
          !call hm_search(gpar,fopt,tfunk,safunc1,sc)
      elseif (gopti%algo == BBBC) then
          call bb_bc_algo(gpar,bb_cond,gopti,fopt,tfunk,safunc1,sc,nfcnev,kevent)
      endif
      !write(mpi_prog_rank+70,*)'END OF RUN:',i
!      
!     Stampa risultati alla fine del run
      if (mpi_prog_rank == mpi_prog_master) then
          write(6, '(/, 25x,"** Results at the end of ",a," Run n.",i0," **")')trim(algoname_short(gopti%algo)),i
          if (gopti%algo /= BBBC) call sa_print_final(fopt,nfcnev,nacc,nobds,tfin,6)      
          call gopt_print_parameters(gpar%val,gpar,6)      
          call print_atoms(cryst(1)%at,write_occ=.true.)
          if(cryst(1)%numres() > 0) then
             call print_restraints_all(lo,cryst(1)%res,cryst(1)%at,cryst(1)%cell)
             call chi2res_print(cryst(1),lo)
             !resval=chisquared_res(cryst(1),kpr=lo)
          endif
          if(cryst(1)%numresab() > 0)then
             call print_antibump(lo,cryst(1)%abres,cryst(1)%at,cryst(1)%cell)
             resval = antibump_function(cryst(1)%abres,cryst(1)%at,cryst(1)%spg,cryst(1)%cell%get_g(),.false.)
          endif
          if (cryst(1)%is_known()) then
             write(lo,*)
             call cryst(1)%sfactor()
             call to_coord_sir(1,cryst(1)%at,cryst(1)%ref,size(cryst(1)%ref),cryst(1)%cell,cryst(1)%spg,lo,cryst(1)%atpub)
!
!            Incrementa nsol se il risultato e una soluzione         
             if (vett_test(3) == vett_test(1)) then
                 nsol = nsol + 1
                 distnsol = distnsol + vett_test(4)
             endif
          endif
      endif
!old      if (cryst(1)%is_known()) then
!old         write(lo,*)
!old         call cryst(1)%sfactor()
!old         call to_coord_sir(1,cryst(1)%at,cryst(1)%ref,size(cryst(1)%ref),cryst(1)%cell,cryst(1)%spg,lo,cryst(1)%atpub)
!old!
!old!        Incrementa nsol se il risultato e una soluzione         
!old         if (vett_test(3) == vett_test(1)) then
!old             nsol = nsol + 1
!old             distnsol = distnsol + vett_test(4)
!old         endif
!old      else
!old         cryst(1)%ref%fc = sqrt(cryst(1)%ref%fc) ! rimetti fo in ref%fc
!old      endif
!
!     Calcola tempo per ogni run      
      call timesa(SATIME_RUN)%toc()
      timerun = timesa(SATIME_RUN)%get_interval()
!
!     save solutions
      mpi_sol(:gopti%npar,i) = gpar%val
      mpi_sol(gopti%npar+1,i) = timerun
      mpi_sol(gopti%npar+2,i) = nfcnev
!
!     Aggiorna info sul minimo su tutti i run
      if (fopt < gopti%fmin) then
          gopti%fmin = fopt
          gopti%rmin = i
      endif
!corr!      
!corr!     Write per tabella test
!corr      if(print_tb)write(6,"('TB1',i5,i6,'/',i0,2f10.4,f12.4,1x,i15,1x,a)")   &
!corr       i,int(vett_test(3)),int(vett_test(1)),vett_test(4),rmsd,fopt,nfcnev,trim(secs_to_string(timerun,.true.))
!corr      if (kevent == 2) exit  ! evento stop dell'utente
!
!     recompute the number of evaluation using the previous run
      neval = neval + 1
      sum_eval = sum_eval + nfcnev
      numcyc_calc = sum_eval / neval
   enddo                     ! fine loop sui run di SA
!
!corr   call timesa(SATIME_RUN)%set_average(gopti%nrun)
!
   call resize_models(cryst(1)%model,nsamodel)
!!!!!!!!!!!!!!!!!gestione modelli
!corr   if (mpi_prog_rank == mpi_prog_master) write(0,*)'COMMUNICATE MODELS',mpi_prog_rank
   call MPI_Barrier(MPI_COMM_WORLD,mpi_err)
   call model_communicate(nrun,mpi_sol,mpi_rec)
   call MPI_Barrier(MPI_COMM_WORLD,mpi_err)
   if (mpi_prog_rank  == mpi_prog_master) then
       call new_models(cryst(1)%model,gopti%nrun)
       nsamodel = 0
       nsol = 0
       do i=1,gopti%nrun
          fopt = tfunk(mpi_rec(:gopti%npar,i),0)   ! compute CF and fill atm
          write(6,'(a,i0,a,f10.3,a,*(f10.3))') 'Run ',i,' CF:',fopt,' Par: ',mpi_rec(:gopti%npar,i)
          !call xpar_to_atm(mpi_rec(:gopti%npar,i),sadoc%active(),atmr,cellt,ref)
          nsamodel = nsamodel + 1
          if (cryst(1)%numfrag() > 1) then      ! prima raggruppa i frammenti 
              call regroup_fragment(cryst(1)%frag,cryst(1)%at,cryst(1)%cell,cryst(1)%spg)
          endif
          if (cryst(1)%doc_active()) then
              call calcola_occ(cryst(1)%at,cryst(1)%spg,cryst(1)%cell,dist=DISTOCC)  
              call search_overlapped_atoms(cryst(1)%at,cryst(1)%cell%get_g(),cryst(1)%spg)
          else
              call calcola_occ(cryst(1)%at,cryst(1)%spg,cryst(1)%cell)  ! dist=default (0.6)
          endif

          call print_atoms(cryst(1)%at,title='Structure solution',write_occ=.true.)
          if (cryst(1)%doc_active()) then
              call search_overlapped_atoms_print(cryst(1)%at,cryst(1)%cell%get_g(),cryst(1)%spg)
          endif
          if (cryst(1)%is_known()) then
              write(6,*)
              call cryst(1)%sfactor()
              call to_coord_sir(1,cryst(1)%at,cryst(1)%ref,size(cryst(1)%ref),cryst(1)%cell,cryst(1)%spg,lo,cryst(1)%atpub)
!
!             Incrementa nsol se il risultato e una soluzione         
              if (vett_test(3) == vett_test(1)) then
                  nsol = nsol + 1               
                  distnsol = distnsol + vett_test(4)
              endif
          else
              cryst(1)%ref%fc = sqrt(cryst(1)%ref%fc) ! rimetti fo in ref%fc
          endif
          if (npunti > 0 .and. extrac) then
              if (is_profile) then
                  rappo = fopt/gopti%rwp_leb
              else
                  call sa_profile(cryst(1)%ref%fc)
                  rappo = rinfo%rwp/gopti%rwp_leb
              endif
          else
              rappo = 0  ! da migliorare
          endif
          timerun = mpi_rec(gopti%npar+1,i)
          nfcnev = mpi_rec(gopti%npar+2,i)
          infomodel = (/fopt,vett_test(3),vett_test(4),timerun,real(i),rappo,rmsd/)
          call model_set(cryst(1)%model(i),cryst(1)%at,fom=infomodel)
!      
!         Write per tabella test
          if(print_tb)then
             if (cryst(1)%is_known()) then
                 write(lo,"(a,i5,i6,'/',i0,2f10.4,f12.4,1x,i15,1x,a15)")   &
                 STRTAG,i,int(vett_test(3)),int(vett_test(1)),vett_test(4),rmsd,fopt,nfcnev,trim(secs_to_string(timerun,.true.))
             else
                 write(lo,"(a,i5,f12.4,1x,i15,1x,a15)")   &
                 STRTAG,i,fopt,nfcnev,trim(secs_to_string(timerun,.true.))
             endif
          endif
       enddo
!
!      Ragruppa e calcola occupanza
       do j=1,nsamodel
          call model_get(cryst(1)%model(j),atmr)
          if (cryst(1)%numfrag() > 1) then      ! prima raggruppa i frammenti 
              call regroup_fragment(cryst(1)%frag,atmr,cryst(1)%cell,cryst(1)%spg)
          endif
          if (cryst(1)%doc_active()) then
              call calcola_occ(atmr,cryst(1)%spg,cryst(1)%cell,dist=DISTOCC)  
              call search_overlapped_atoms(atmr,cryst(1)%cell%get_g(),cryst(1)%spg)
              call search_overlapped_atoms_print(atmr,cryst(1)%cell%get_g(),cryst(1)%spg)
          else
              call calcola_occ(atmr,cryst(1)%spg,cryst(1)%cell)  ! dist=default (0.6)
          endif
          call model_set(cryst(1)%model(j),atmr)
       enddo
!
!      Overlay and export sorted model
       call model_sort(cryst(1)%model,1)
       call overlay_models_list(cryst(1)%model,cryst(1)%ref,cryst(1)%numref())
       if (index(gopti%popt,'E') > 0) then
           do j=1,nsamodel
              !call model_get(cryst(1)%model(j),atmr,fom=infomodel)
              !call export_model(j,int(infomodel(5)),infomodel(1),gopti%algo)
              call export_model(j,gopti%algo)
           enddo
       endif
   else
       return
   endif
!!!!!!!!!!!!!!!!!gestione modelli
!
!  Ordina per CF e stampa la lista
   call print_listsa(cryst(1)%model,6)
   call print_listsa(cryst(1)%model,0)
!
!corr   if (cryst(1)%numres() > 0) then
!corr       do i=1,nsamodel  !!!!listsa%nstru
!corr          call model_get(cryst(1)%model(i),atmr,fom=infomodel)
!corr          cryst(1)%at(:) = atmr(:)
!corr          resval=chisquared_res(cryst(1))
!corr          call chi2res_print(cryst(1),lo)
!corr       enddo
!corr   endif
!
!  Associa connettivita' ai modelli
   if (cryst(1)%numfrag() > 1) then  ! riconnetti i frammenti fra loro
       do i=1,nsamodel
          call model_get(cryst(1)%model(i),atmr)
          call copy_bonds(legm,cryst(1)%bond)
          call connect_fragment(cryst(1)%frag,atmr,legm,cryst(1)%cell,.false.)
          call model_set(cryst(1)%model(i),leg=legm)
       enddo
   else
       do i=1,nsamodel
          call model_set(cryst(1)%model(i),leg=cryst(1)%bond)
       enddo
   endif
!
!  Seleziona il primo modello   
   call model_get(cryst(1)%model(1),atbest,fom=infomodel)
   fopt = infomodel(1)
   vett_test(3) = infomodel(2)
   vett_test(4) = infomodel(3)
   timerun = infomodel(4)   
   rappo = infomodel(6)
   rmsd = infomodel(7)
!   
   cryst(1)%at(:) = atbest(:)         ! aggiorna il modello
!   
   timetot = timesa(SATIME_TOT)%get_time()
   if(print_tb)then
      frun_min = minval(cryst(1)%model(:)%info(1))
      frun_max = maxval(cryst(1)%model(:)%info(1))
      frun_ave = sum(cryst(1)%model(:)%info(1)) / nsamodel
      if (cryst(1)%is_known()) then
          nsol8 =  count(cryst(1)%model(:)%info(7) <= 0.08)   ! number of solutions with rmsd <= 0.08
          write(lo,"(a,'best ',i6,'/',i0,2f10.4,f12.4,1x,a,' NSOL=',i0,' distm=',f0.3)")   &
          STRTAG,int(vett_test(3)),int(vett_test(1)),vett_test(4),rmsd,                    &
          fopt,secs_to_string(timetot),nsol,distnsol/nsol 
          write(lo,"(a,'info: ','CF=',f0.3,'-',f0.3,' CF(ave)=',f0.3,' Nsol_0.08=',i0,' Nsol_0.08 per min=',f0.3)")  &
          STRTAG,frun_min,frun_max,frun_ave,nsol8,60*nsol8/timetot
      else
          write(lo,"(a,'best ',f12.4,1x,a)")STRTAG,fopt,secs_to_string(timetot)
          write(lo,"(a,'info: ','CF=',f0.3,'-',f0.3,' CF(ave)=',f0.3)")  &
          STRTAG,frun_min,frun_max,frun_ave
      endif
   endif
   write(lo,'(/a,i0,1x,a)')' The structure of run n.',int(infomodel(5)),' was selected:'
   call print_atoms(cryst(1)%at,title='Structure solution',write_occ=.true.)
   if (cryst(1)%numres() > 0) then
!corr       if (cryst(1)%numresbv() > 0)call make_symmetry_table(cryst(1)%at,cryst(1)%spg,asymtab)
       resval = chisquared_res(cryst(1))
       call print_restraints_all(lo,cryst(1)%res,cryst(1)%at,cryst(1)%cell)
       call chi2res_print(cryst(1),lo)
   endif
!
   call write_message('Best structure found at run n. ',inum=int(infomodel(5)),pos=2)
   call write_message('Saved in file '//trim(cryst(1)%cr_name)//'_best1.cif',pos=3)
   call write_message('CF = ',rnum=fopt,pos=4)
   if (is_profile)call write_message('Ratio = ',rnum=rappo,pos=0)
   call write_message('Time: '//secs_to_string(timetot),pos=0)
   if (cryst(1)%is_known()) call write_message_found1(0)
   call write_message("Click on 'Solutions' to explore the other structures",pos=0)
   if (cryst(1)%doc_active()) call search_overlapped_atoms_print(cryst(1)%at,cryst(1)%cell%get_g(),cryst(1)%spg)
!
   if (cryst(1)%is_known()) then
       call cryst(1)%sfactor()
       call to_coord_sir(0,cryst(1)%at,cryst(1)%ref,size(cryst(1)%ref),cryst(1)%cell,cryst(1)%spg,lo,cryst(1)%atpub)
   endif
!
!  Write per tabella riepilogativa       
   if (print_tb) &
   write(6,100)  &
         '"'//trim(fname(OUT_FILE)(scan(fname(OUT_FILE),'/\',back=.true.)+1:))//'",', &
         gopti%dofe,gopti%dofi,                                           &
         '"',gopti%ttmin,gopti%ttmax,'"',cryst(1)%numref(),gopti%res,            &
         sc%nt,'"',nsol,gopti%nrun,'"',                                   &
         '"',int(vett_test(3)),int(vett_test(1)),'"',vett_test(4),rmsd,   &
         fopt,frun_min,frun_max,frun_ave,rappo, ',"'//trim(secs_to_string(timetot))//'"'
 100 format('TB2',a,i0,',',i0,',',a,f0.2,'-',f0.2,a,',',i0,',',f0.2,',',  &
      i0,',',a, i0,'/',i0,a,',',a,i0,'/',i0,a,7(',',f0.3),a)
!
!   
   end subroutine simann

!----------------------------------------------------------------------------------------------------

   subroutine model_communicate(nrun,mpi_sol,mpi_rec)
!
!  Send all models mpi_sol to root process (cpu 0) in mpi_rec
!
   USE mpi
   USE mpi_prog
   implicit none
   integer, intent(in)               :: nrun
   real, dimension(:,:), intent(in)  :: mpi_sol
   real, dimension(:,:), intent(out) :: mpi_rec
   integer                           :: mpi_err
   integer                           :: tag
   integer                           :: i,j
   integer                           :: status(MPI_STATUS_SIZE)
   integer                           :: nruntot
!
   if (mpi_prog_rank > 0) then     ! is a slave process
!
!      mpi_prog_rank send number of models to master process
       tag = 1
       call MPI_Ssend(nrun, 1, MPI_INTEGER, 0, tag, MPI_COMM_WORLD,mpi_err)   
!
!      mpi_prog_rank send models to master process
       tag = 2
       call MPI_Ssend(mpi_sol, size(mpi_sol), MPI_REAL, 0, tag, MPI_COMM_WORLD,mpi_err) 
       !write(6,'(a,i0,a,i0,a)')'Cpu ',mpi_prog_rank,' sends ',nrun,' solutions to cpu 1'
       !do j=1,nrun
       !   write(6,'(a,i0,a,*(f10.3))') 'Run ',j,' Par: ',mpi_sol(:,j)
       !enddo
   else                            ! master process
!
!      Add model of root to array mpi_rec
       mpi_rec(:,:nrun) = mpi_sol(:,:nrun)
       nruntot = nrun
!
!      Now add models of other cpu
       do i=1,mpi_prog_numprocs-1
          tag = 1
          call MPI_Recv(nrun, 1, MPI_INTEGER, i, tag, MPI_COMM_WORLD,status,mpi_err)
          tag = 2
          call MPI_Recv(mpi_sol, size(mpi_sol), MPI_REAL, i, tag, MPI_COMM_WORLD,status,mpi_err)
          write(6,'(a,i0,a,i0)')'Cpu ',mpi_prog_rank,' recieve solutions from cpu ',i
          do j=1,nrun
             write(6,'(a,i0,a,*(f10.3))') 'Run ',j,' Par: ',mpi_sol(:,j)
          enddo
          do j=1,nrun
             nruntot = nruntot + 1
             mpi_rec(:,nruntot) = mpi_sol(:,j)
          enddo
       enddo
   endif
!
   end subroutine model_communicate
#else
!----------------------------------------------------------------------------------------------------        

   subroutine simann(gopti,sc,bb_cond,extrac)
!
!  Routine principale: esegue multirun di SA, stampa parametri iniziali e finale, gestisce la lista dei modelli
!   
   USE simulated_anneal
   USE variables, only:cryst
   USE General, only:fname,lo,OUT_FILE
   USE TestModel, only:vett_test,rmsd
   USE compare_struct
   USE atom_type_util   
   USE RefinecomRef, only:rinfo
   USE strutil, only:s_s_delete
   USE Molcom
   USE bb_bc
   USE rand_mod
   USE messagemod
   USE model_util
   USE anti_bump
   USE Conteggi, only: npunti
   USE connect_mod
   USE fragmentmod
   USE constraints
   USE rrestr
   USE grexpomod
   USE timer
   USE reflection_type_util
   type(opti_info_type), intent(inout)         :: gopti    
   type(SimAnn_Conditions_type), intent(inout) :: sc
   type(bb_bc_condition), intent(in)           :: bb_cond                     
   logical, intent(in)                         :: extrac
   real, dimension(gopti%npar)                 :: xopt
   real                                        :: fopt
   real                                        :: tfin
   integer                                     :: nfcnev
   integer                                     :: nacc, nobds 
   integer                                     :: i,j
   type(param_type), dimension(size(gpar))     :: gparini
   type(atom_type), dimension(:), allocatable  :: atbest
   real, dimension(7)                          :: infomodel 
   real                                        :: timerun
   integer                                     :: kevent
   integer                                     :: nsol      
   real                                     :: distnsol
   integer, dimension(0:6)                       :: vcolor 
   integer :: pcol
   type(atom_type), dimension(:), allocatable  :: atmr
   logical                                     :: is_profile
   integer                                     :: nplotr
   real                                        :: rappo
   real                                        :: resval
   integer                                     :: xdim,ydim
   type(bb_bc_condition)                       :: bcond
   type(bond_type), dimension(:), allocatable  :: legm
   real                                        :: frun_min, frun_max, frun_ave
   integer :: sum_eval
   integer :: neval
   integer :: nsol8
   real(DP) :: timetot
   integer :: nsamodel
   logical :: restart_par
!
   gparini = gpar
   if (jscreen > 0) then
       if(npunti == 0) call s_s_delete(gopti%popt,'P')
       !if (npunti == 0) then
       !    gopti%popt = 'VME'
       !endif
   else
      gopti%popt = 'SE'
   endif
   if (gopti%algo == BBBC) then
       call set_bb_bc_condition(gopti,bcond,sc)
       numcyc_calc = sc%num_temps*sc%ns*sc%nt*gopti%npar*bb_cond%niter*bb_cond%nlocmax  ! calculated for progress bar
       call bb_bc_print_conditions(bb_cond,6)
       call sa_print_conditions(sc,6,'Local search conditions')
   else
       numcyc_calc = sc%t_ini*sc%ns*sc%nt*gopti%npar  ! for progress bar
       call sa_print_conditions(sc,6,'Simulated Annealing conditions')
   endif
!
   ginfo = gopti
!
!  Stampa condizioni iniziali
   write(6,'(1x,a,f0.3)')     'Initial cost function value:  ',tfunk(gpar%val,0)
   if (cryst(1)%numresab() > 0)   &
       write(6,'(1x,a,f0.3)') 'Anti-bump cost function:      ',  &
       cryst(1)%abset%weight*antibump_function(cryst(1)%abres,cryst(1)%at,cryst(1)%spg,cryst(1)%cell%get_g(),.false.)
   if (cryst(1)%numres() > 0) then
       write(6,'(1x,a,f0.3)') 'Restraints cost function:     ',chisquared_res(cryst(1))
       call print_restraints_all(lo,cryst(1)%res,cryst(1)%at,cryst(1)%cell)
   endif
   call print_antibump(lo,cryst(1)%abres,cryst(1)%at,cryst(1)%cell)
   call print_constraints(cryst(1)%sconstr,cryst(1)%at,lo,.true.,.false.)
   call gopt_print_parameters(gpar%val,gpar,6,'Starting refined parameters')
!  
   if (print_tb) then
       write(6,'(4(a,i0),a,f0.2)')STRTAG//' DOFext=',gopti%dofe,' DOFint=',gopti%dofi,   &
                                   ' NREF=',cryst(1)%numref(),'/',numrefl(cryst(1)%refc),' RES=',gopti%res
       if (cryst(1)%is_known()) then
           write(6,'(a)')STRTAG//' NRUN    found     dist      RMSD       CF       n.moves          time'   
       else
           write(6,'(a)')STRTAG//' NRUN      CF       n.moves          time'   
       endif
   endif
   call new_models(cryst(1)%model,gopti%nrun)
   nsamodel = 0
!
!  Operazione preliminari per grafico della CF
   is_profile = is_profile_function(gopti%cftype)
   if (index(gopti%popt,'V') > 0) then  
       vcolor = (/color_BROWN,color_LIGHTBLUE,color_LIGHTRED,color_LIGHTGREEN,     &
                  color_LIGHTCYAN,color_LIGHTMAGENTA,color_YELLOW/)
       call gr_create_graph(grgraph)
       call gr_set_graph(grgraph,stringx='moves',stringy=trim(gopti%cost_function_name_short),xscale=10000.0)
       call gr_set_graph(grgraph,y_resize=.true.,y_resize_val=150.0)
       if (is_profile) then
           nplotr = gopti%nrun + 1
       else
           nplotr = gopti%nrun
       endif
       if (allocated(grplot)) then
           if (size(grplot) /= nplotr) then
               deallocate(grplot)
               allocate(grplot(nplotr))
           endif
       else
           allocate(grplot(nplotr))
       endif
       do i=1,gopti%nrun
          call gr_new_plot(grplot(i),1000)
          pcol = mod(i,size(vcolor)) 
          call gr_set_plot(grplot(i),colorline=vcolor(pcol))
       enddo
!
!      Per visualizzare la linea dell'Rwp alla fine dell'affinamento
       if (is_profile) then
           call gr_new_plot(grplot(gopti%nrun+1),2)
           call gr_set_plot(grplot(gopti%nrun+1),colorline=color_LIGHTRED)
           call gr_add_point_plot(grplot(gopti%nrun+1),0.,gopti%rwp_leb)
           call gr_add_point_plot(grplot(gopti%nrun+1),1.,gopti%rwp_leb)
       endif
!
       call new_viewer_size(xdim,ydim)
       call gr_open_viewer(xdim,ydim,'CF vs. number of moves',newv=.true.)
       call gr_select_viewer(1)  ! serve se no disegna su entrambi i viewers 
       call gr_setbackground(color_BRIGHTWHITE)
       call gr_select_viewer(0)
   endif
!   
!  gestione messaggi
   if (.not.cryst(1)%is_known()) call clear_messages(8) ! se la struttura e' incognita la riga 8 non viene usata  
!   
   allocate(atmr(size(cryst(1)%at)))
   nsol = 0
   distnsol = 0
   gopti%fmin = huge(1.0)
   gopti%rmin = 1
   neval = 0
   sum_eval = 0
     restart_par = .true.
   do i=1,gopti%nrun          ! inizio loop sui run di SA
      call timesa(SATIME_RUN)%tic()
      write(6,'(/2x,80("=")/,25x,"** ",a," Run n.",i0," **")')trim(algoname_long(gopti%algo)),i
      if (restart_par .or. i == 1) then
          gpar(:)%val = gparini(:)%val
          gpar(:)%stp = gparini(:)%stp
          gopti%nrunc = i         ! memorizza a che run sono
      endif
!
!     Initialize random number generator
      if (gopti%seed == 0) then
          call Init_Ran(gopti%seed)  ! seed from system clock
      else
          call Init_Ran(gopti%seed+i-1)
      endif
!
!     Applica ottimizzazione dei parametri con SA      
      if (gopti%algo == SA_LOCAL) then
          call sa(gpar, sc, gopti, tfin, xopt, fopt, nacc, nfcnev, nobds, kevent, tfunk,safunc1,-1.0)   
      elseif (gopti%algo == SA_GLOBAL) then
          !call sa_glob(gpar, sc, gopti, tfin,fopt, nfcnev, kevent,safunc,safuncg)
          call sa_glob_old(gpar, sc, gopti, tfin,fopt, nfcnev, kevent,safunc,safunc1)
      elseif (gopti%algo == HARMONYS) then
          !call hm_search(gpar,fopt,tfunk,safunc1,sc)
      elseif (gopti%algo == BBBC) then
          call bb_bc_algo(gpar,bb_cond,gopti,fopt,tfunk,safunc1,sc,nfcnev,kevent)
      endif
!      
!     Stampa risultati alla fine del run
      write(6, '(/, 25x,"** Results at the end of ",a," Run n.",i0," **")')trim(algoname_short(gopti%algo)),i
      if (gopti%algo /= BBBC) call sa_print_final(fopt,nfcnev,nacc,nobds,tfin,6)      
      call gopt_print_parameters(gpar%val,gpar,6)      
      call print_atoms(cryst(1)%at,write_occ=.true.)
      if(cryst(1)%numres() > 0)then
         call print_restraints_all(lo,cryst(1)%res,cryst(1)%at,cryst(1)%cell)
         resval=chisquared_res(cryst(1))
         call chi2res_print(cryst(1),lo)
      endif
      if(cryst(1)%numresab() > 0)then
         call print_antibump(lo,cryst(1)%abres,cryst(1)%at,cryst(1)%cell)
         resval = antibump_function(cryst(1)%abres,cryst(1)%at,cryst(1)%spg,cryst(1)%cell%get_g(),.false.)
      endif
      if (cryst(1)%is_known()) then
         write(lo,*)
         call cryst(1)%sfactor()
         call to_coord_sir(1,cryst(1)%at,cryst(1)%ref,size(cryst(1)%ref),cryst(1)%cell,cryst(1)%spg,lo,cryst(1)%atpub)
!
!        Incrementa nsol se il risultato e una soluzione         
         if (vett_test(3) == vett_test(1)) then
             nsol = nsol + 1               
             distnsol = distnsol + vett_test(4)
         endif
      else
         cryst(1)%ref%fc = sqrt(cryst(1)%ref%fc) ! rimetti fo in ref%fc
      endif
!
!     Calcola tempo per ogni run      
      call timesa(SATIME_RUN)%toc()
      timerun = timesa(SATIME_RUN)%get_interval()
!
!     Calcola rapporto Rwp(calc)/Rwp(Le Bail)
      if (npunti > 0 .and. extrac) then
          if (is_profile) then
              rappo = fopt/gopti%rwp_leb
          else
              call sa_profile(cryst(1)%ref%fc)
              rappo = rinfo%rwp/gopti%rwp_leb
          endif
      else
          rappo = 0  ! da migliorare
      endif
!
!     Ragruppa e calcola occupanza
      atmr = cryst(1)%at
      if (cryst(1)%numfrag() > 1) then      ! prima raggruppa i frammenti 
          call regroup_fragment(cryst(1)%frag,atmr,cryst(1)%cell,cryst(1)%spg)
      endif
      if (cryst(1)%doc_active()) then
          call calcola_occ(atmr,cryst(1)%spg,cryst(1)%cell,dist=DISTOCC)  
          call search_overlapped_atoms(atmr,cryst(1)%cell%get_g(),cryst(1)%spg)
          call search_overlapped_atoms_print(atmr,cryst(1)%cell%get_g(),cryst(1)%spg)
      else
          call calcola_occ(atmr,cryst(1)%spg,cryst(1)%cell)  ! dist=default (0.6)
      endif
!
!     Aggiungi modello alla lista       
      infomodel = (/fopt,vett_test(3),vett_test(4),timerun,real(i),rappo,rmsd/)
      nsamodel = nsamodel + 1
      call model_set(cryst(1)%model(nsamodel),atmr,fom=infomodel)
!
!     esporta i modelli ordinati per fom
      call model_sort(cryst(1)%model(1:nsamodel),1)
      call overlay_models_list(cryst(1)%model(1:nsamodel),cryst(1)%ref,cryst(1)%numref())  ! sovrapponi
      if (index(gopti%popt,'E') > 0) then
          do j=1,nsamodel
             !call model_get(cryst(1)%model(j),atmr,fom=infomodel)
             call export_model(j,gopti%algo)
          enddo
      endif
!
!     Aggiorna info sul minimo su tutti i run
      if (fopt < gopti%fmin) then
          gopti%fmin = fopt
          gopti%rmin = i
      endif
!      
!     Write per tabella test
      if(print_tb)then
         if (cryst(1)%is_known()) then
             write(lo,"(a,i5,i6,'/',i0,2f10.4,f12.4,1x,i15,1x,a)")   &
             STRTAG,i,int(vett_test(3)),int(vett_test(1)),vett_test(4),rmsd,fopt,nfcnev,trim(secs_to_string(timerun,.true.))
         else
             write(lo,"(a,i5,f12.4,1x,i15,1x,a)")   &
             STRTAG,i,fopt,nfcnev,trim(secs_to_string(timerun,.true.))
         endif
      endif
!corr      if(print_tb)write(6,"(a,i5,i6,'/',i0,2f10.4,f12.4,1x,i15,1x,a)")   &
!corr       STRTAG,i,int(vett_test(3)),int(vett_test(1)),vett_test(4),rmsd,fopt,nfcnev,trim(secs_to_string(timerun,.true.))
      if (kevent == 2) exit  ! evento stop dell'utente
!
!     recompute the number of evaluation using the previous run
      neval = neval + 1
      sum_eval = sum_eval + nfcnev
      numcyc_calc = sum_eval / neval
   enddo                     ! fine loop sui run di SA
!
   call resize_models(cryst(1)%model,nsamodel)
!
!  Ordina per CF e stampa la lista
   call print_listsa(cryst(1)%model,6)
   call print_listsa(cryst(1)%model,0)
!
!  Associa connettivita' ai modelli
   if (cryst(1)%numfrag() > 1) then  ! riconnetti i frammenti fra loro
       do i=1,cryst(1)%nmodels()
          call model_get(cryst(1)%model(i),atmr)
          call copy_bonds(legm,cryst(1)%bond)
          call connect_fragment(cryst(1)%frag,atmr,legm,cryst(1)%cell,.false.)
          call model_set(cryst(1)%model(i),leg=legm)
       enddo
   else
       do i=1,cryst(1)%nmodels()
          call model_set(cryst(1)%model(i),leg=cryst(1)%bond)
       enddo
   endif
!
!  Select the first/best model
   call model_get(cryst(1)%model(1),atbest,fom=infomodel)
   fopt = infomodel(1)
   vett_test(3) = infomodel(2)
   vett_test(4) = infomodel(3)
   timerun = infomodel(4)   
   rappo = infomodel(6)
   rmsd = infomodel(7)
!   
   cryst(1)%at(:) = atbest(:)         ! update coordinates
!   
   timetot = timesa(SATIME_TOT)%get_time()
   if(print_tb)then
      frun_min = minval(cryst(1)%model(:)%info(1))
      frun_max = maxval(cryst(1)%model(:)%info(1))
      frun_ave = sum(cryst(1)%model(:)%info(1)) / nsamodel
      if (cryst(1)%is_known()) then
          nsol8 =  count(cryst(1)%model(:)%info(7) <= 0.08)   ! number of solutions with rmsd <= 0.08
          write(lo,"(a,'best ',i6,'/',i0,2f10.4,f12.4,1x,a,' NSOL=',i0,' distm=',f0.3)")   &
          STRTAG,int(vett_test(3)),int(vett_test(1)),vett_test(4),rmsd,                    &
          fopt,secs_to_string(timetot),nsol,distnsol/nsol 
          write(lo,"(a,'info: ','CF=',f0.3,'-',f0.3,' CF(ave)=',f0.3,' Nsol_0.08=',i0,' Nsol_0.08 per min=',f0.3)")  &
          STRTAG,frun_min,frun_max,frun_ave,nsol8,60*nsol8/timetot
      else
          write(lo,"(a,'best ',f12.4,1x,a)")STRTAG,fopt,secs_to_string(timetot)
          write(lo,"(a,'info: ','CF=',f0.3,'-',f0.3,' CF(ave)=',f0.3)")  &
          STRTAG,frun_min,frun_max,frun_ave
      endif
   endif
   write(lo,'(/a,i0,1x,a)')' The structure of run n.',int(infomodel(5)),' was selected:'
   call print_atoms(cryst(1)%at,title='Structure solution',write_occ=.true.)
   if (cryst(1)%numres() > 0) then
       resval = chisquared_res(cryst(1))
       call print_restraints_all(lo,cryst(1)%res,cryst(1)%at,cryst(1)%cell)
       call chi2res_print(cryst(1),lo)
   endif
!
   call write_message('Best structure found at run n. ',inum=int(infomodel(5)),pos=2)
   call write_message('Saved in file '//trim(cryst(1)%cr_name)//'_best1.cif',pos=3)
   call write_message('CF = ',rnum=fopt,pos=4)
   if (is_profile)call write_message('Ratio = ',rnum=rappo,pos=0)
   call write_message('Time: '//secs_to_string(timetot),pos=0)
   if (cryst(1)%is_known()) call write_message_found1(0)
   call write_message("Click on 'Solutions' to explore the other structures",pos=0)
   if (cryst(1)%doc_active()) call search_overlapped_atoms_print(cryst(1)%at,cryst(1)%cell%get_g(),cryst(1)%spg)
!
   if (cryst(1)%is_known()) then
       call cryst(1)%sfactor()
       call to_coord_sir(0,cryst(1)%at,cryst(1)%ref,size(cryst(1)%ref),cryst(1)%cell,cryst(1)%spg,lo,cryst(1)%atpub)
   endif
!
!  Write per tabella riepilogativa       
   if (print_tb) &
   write(6,100)  &
         '"'//trim(fname(OUT_FILE)(scan(fname(OUT_FILE),'/\',back=.true.)+1:))//'",', &
         gopti%dofe,gopti%dofi,                                           &
         '"',gopti%ttmin,gopti%ttmax,'"',cryst(1)%numref(),gopti%res,            &
         sc%nt,'"',nsol,gopti%nrun,'"',                                   &
         '"',int(vett_test(3)),int(vett_test(1)),'"',vett_test(4),rmsd,   &
         fopt,frun_min,frun_max,frun_ave,rappo, ',"'//trim(secs_to_string(timetot))//'"'
 100 format('TB2',a,i0,',',i0,',',a,f0.2,'-',f0.2,a,',',i0,',',f0.2,',',  &
      i0,',',a, i0,'/',i0,a,',',a,i0,'/',i0,a,7(',',f0.3),a)
!   
   end subroutine simann

#endif         
!----------------------------------------------------------------------------------------------------

   subroutine set_bb_bc_condition(gopti,cond,sc)
   USE bb_bc
   USE variables, only: cryst
   USE crystal_phase
   type(opti_info_type), intent(in)                      :: gopti    
   type(bb_bc_condition), intent(out)                    :: cond
   type(SimAnn_Conditions_type), intent(inout), optional :: sc
   integer                                               :: nph,ntotfrag
!
   if (cond%auto) then
!corr       select case(cryst(1)%numrot())
       select case(gopti%nprot)
          case (0)
            cond%niter = 5
          case (1:4)
            cond%niter = 20
          case (5:7)
            cond%niter = 30
          case (8:9)
            cond%niter = 40
          case (10:)
            cond%niter = 80
      
       end select
       ntotfrag = 0
       do nph=1,numphase(cryst)
          ntotfrag = ntotfrag + cryst(nph)%numfrag()
       enddo
       !corr cond%niter = cond%niter + nint(0.5*(cryst(1)%numfrag() - 1))*cond%niter
       cond%niter = cond%niter + nint(0.5*(ntotfrag - 1))*cond%niter
   endif
!
!  condizioni per SA all'interno del BB-BC
   if (present(sc)) then
       sc%num_temps = 5                 
       sc%nt = sc%nt / 10
       sc%temp_fact = sc%temp_fact / 4 
       sc%t_ini = 0.40*sc%t_ini 
       sc%eps = 10                    
       sc%ns = sc%ns
   endif
!
   end subroutine set_bb_bc_condition

!----------------------------------------------------------------------------------------------------

   subroutine safunc(xpar,h,hpar)
   implicit none
   real , intent(in)   :: xpar(:)
   real , intent(out)  :: h
   integer, intent(in) :: hpar
!   
   h = tfunk(xpar,hpar)
!
   end subroutine safunc

!----------------------------------------------------------------------------------------------------

   subroutine safunc_temp(xpar,h,hpar,gopti)
   implicit none
   real , intent(in)   :: xpar(:)
   real , intent(out)  :: h
   integer, intent(in) :: hpar
   type(opti_info_type), intent(in) :: gopti    
!
   call xpar_to_crystals(xpar)
   h = chisquared_xdata(hpar,gopti) !!!!,cryst(1)%at,cryst(1)%cell,cryst(1)%ref)
!
   end subroutine safunc_temp

!----------------------------------------------------------------------------------------------------        

   real function safuncg(x)
   real, dimension(:), intent(in) :: x
   safuncg = tfunk(x,0)
   end function safuncg

!----------------------------------------------------------------------------------------------------        
  
   subroutine safunc1(xp,fval,grad,icod)
   USE lsqalgo
!
!  Funzione che per  icod = 1  valuta la funzione costo
!                    icod = 2  valuta il gradiente
   real, dimension(:), intent(in)  :: xp
   real, intent(out)               :: fval
   real, dimension(:), intent(out), optional :: grad
   integer, intent(in)             :: icod
!
   select case(icod)
      case (1)
        fval = tfunk(xp,0)

      case (2)
        call calc_gradient(grad,xp,0.0,safuncg,icod=2)

   end select

!   CONTAINS
! 
!   real function funcgrad(x)
!   real, dimension(:), intent(in) :: x
!   funcgrad = tfunk(x,0)
!   end function funcgrad
!!
   end subroutine safunc1

!----------------------------------------------------------------------------------------------------        

   logical function is_profile_function(cftype) result(lval)
   integer, intent(in) :: cftype
!
   select case(cftype)

      case(RWPC_COSTF,RWP_COSTF,CHI2_COSTF)
           lval = .true.
      case(CHI2FACT,CHI2I,CHI2I_SD,CHI2I_NEW)
           lval = .false.
      case default 
           lval = .false.
   end select
!
   end function is_profile_function

!----------------------------------------------------------------------------------------------------        
   subroutine print_listsa(model,unitp)
!
!  Stampa riepilogativa della lista dei modelli
!   
   USE TestModel, only:vett_test
   USE model_util
   USE variables, only: cryst
   USE timer
   type(model_type), dimension(:), intent(in) :: model
   integer, intent(in)                        :: unitp
   integer                                    :: i
   character(len=10)                          :: snum
   integer                                    :: nmodel
!
   nmodel = size(model)
   write(unitp,'(/3x,90("-"),/a)')'           List of structural models sorted according to cost function '
   if (cryst(1)%is_known()) then
       write(unitp,'(a/,3x,90("-"))') &
   '      RUN        CF                Ratio       found        dist    RMSD    time         saved on'
       do i=1,nmodel
          write(snum,'(i0)')i
          write(unitp,'(2x,i5,2x,f15.6,f15.3,5x,i5,"/",i0,t55,2f10.4,2x,a,4x,a)')   &
          int(model(i)%info(5)),model(i)%info(1),model(i)%info(6),   &
          int(model(i)%info(2)),int(vett_test(1)),  &
          model(i)%info(3),model(i)%info(7),secs_to_string(model(i)%info(4)),    &
          trim(cryst(1)%cr_name)//'_best'//trim(snum)// '.cif'
       enddo 
   else
       write(unitp,'(a/,3x,90("-"))') '      RUN        CF                Ratio         time         saved on'
       do i=1,nmodel
          write(snum,'(i0)')i
          write(unitp,'(2x,i5,2x,f15.6,f15.3,8x,a,4x,a)')   &
          int(model(i)%info(5)),model(i)%info(1),model(i)%info(6),    &
          secs_to_string(model(i)%info(4)),trim(cryst(1)%cr_name)//'_best'//trim(snum)// '.cif'
       enddo 
   endif
 !
   end subroutine print_listsa
   
!----------------------------------------------------------------------------------------------------

   subroutine overlay_models_list(model,ref,numref)
!
!  Fa in modo che i modelli possano essere sovrapponibili
!
   USE compare_struct
   USE variables, only:cryst
   USE PhaseModule, only:shiftexam
   USE model_util
   USE reflection_type_util
   USE unit_cell
   type(model_type), dimension(:), intent(inout) :: model
   type(reflection_type), dimension(:), intent(in) :: ref
   integer, intent(in)                           :: numref
   integer                                       :: i   
   type(atom_type), allocatable, dimension(:)    :: atm1,atm2
   type(reflection_type), dimension(numref)      :: ref1,ref2
   integer                                       :: nmodel
!
   nmodel = size(model)
   if (nmodel <= 1) return
!
   do i=1,numref
      ref1(i) = ref(i)
      ref2(i) = ref(i)
   enddo
!
!  Prendi il primo modello della lista
   call model_get(model(1),atm1)
   call sfcalc(ref1,atm1,cryst(1)%spg,cryst(1)%elem)
!
!  Sovrapponi gli altri modelli al primo
   do i=2,nmodel
      call model_get(model(i),atm2)
      call sfcalc(ref2,atm2,cryst(1)%spg,cryst(1)%elem)
      call examing(ref1,ref2)
      call drawup_models(atm1,cryst(1)%bond,atm2,cryst(1)%bond,cryst(1)%cell,cryst(1)%spg,   &
           shiftexam,kfix=.true.,kenant=.false.,kinorg=.false.)
      call model_set(model(i),atm2)
   enddo
!
   end subroutine overlay_models_list

!----------------------------------------------------------------------------------------------------

   subroutine reallocate_shift(vetr,n,savevet)
!
!  Rialloca ad n un vettore di tipo fragment_type
!  Se savevet = .true. o non esiste si salva il suo contenuto.
!
   type(shift_atom_type), allocatable, intent(inout) :: vetr(:)
   integer, intent(in)              :: n
   logical, optional, intent(in)    :: savevet
   logical                          :: savev
   integer                          :: nv
   type(shift_atom_type), allocatable                :: vsav(:)
   integer                          :: nsav
!
!  se n = 0 (riallocazione a 0): dealloca ed esci
   if (n == 0) then
       if (allocated(vetr)) deallocate(vetr)
       return
   endif
!
   if (.not.allocated(vetr)) then
       allocate(vetr(n))
   else
!
       nv = size(vetr)
       if (present(savevet)) then
           savev = savevet
       else
           savev = .true.
       endif
!
       if (savev) then
!
!          nsav contiene qual � la porzione di vetr da salvare
           select case(nv-n)
             case (1:)       ! compatta x ad n
               nsav = n
             case (:-1)      ! espandi x ad n
               nsav = nv
             case (0)
               return        ! n=nv non fare niente
           end select
!
!          salva vetr fino a nsav
           allocate(vsav(nsav))
           vsav = vetr(:nsav)
!
!          dealloca vetr
           deallocate(vetr)
!
!          rialloca vetr su n
           allocate(vetr(n))
!
!          ripristina vetr fino a nsav
           vetr(:nsav) = vsav(:nsav)
       else
           deallocate(vetr)
           allocate(vetr(n))
       endif
   endif
!
   end subroutine reallocate_shift

!corr!----------------------------------------------------------------------------------------------------
!corr
!corr   real function tthvalue(resval,wave)
!corr!   
!corr!  Calcola 2theta dalla risoluzione
!corr!
!corr   USE trig_constants
!corr   real, intent(in) :: resval,wave
!corr!   
!corr   tthvalue = 2*rtod*asin(wave/(2*resval))
!corr!   
!corr   end function tthvalue
!corr         
!corr!----------------------------------------------------------------------------------------------------
!corr
!corr   real function resvalue(tthval,wave)
!corr!   
!corr!  Calcola la risoluzione dal 2theta
!corr!
!corr   USE trig_constants
!corr   real, intent(in) :: tthval,wave
!corr!   
!corr   resvalue = wave / (2*sin(dtor*tthval/2))       
!corr!  
!corr   end function resvalue
!corr
!corr!----------------------------------------------------------------------------------------------------

   integer function numref_from_res(resval,refl,wave)   result(numr)
!   
!  Calcola il numero di riflessi associati fino risoluzione resval
!
   USE datasetmod, only: tthvalue
   USE reflection_type_util
   real, intent(in)                                :: resval,wave
   type(reflection_type), dimension(:), intent(in) :: refl 
   real                                            :: tthrval
!   
   tthrval = tthvalue(resval,wave)
   numr = count(refl%tthd(1) <= tthrval)
!
   end function numref_from_res

!----------------------------------------------------------------------------------------------------
   
   subroutine export_model(codefile,algo)
   USE gen_frm, only: CIF_FILE, export_structure
   USE variables, only: cryst,dataset
   USE model_util
!
   integer, intent(in)                        :: codefile
   integer                                    :: nrun
   real                                       :: cfval
   integer, intent(in)                        :: algo
   character(len=250)                         :: filename
   character(len=30)                          :: snum,snum1
   character(len=200)                         :: commline
   type(atom_type), dimension(:), allocatable :: atmr
   real, dimension(7)                         :: infomodel 
!
   write(snum,'(i0)')codefile
   filename = trim(cryst(1)%cr_name)//'_best'//trim(snum)// '.cif'
   call model_get(cryst(1)%model(codefile),atmr,fom=infomodel)
   nrun = int(infomodel(5))
   cfval = infomodel(1)
   write(snum,'(f15.3)')cfval
   write(snum1,'(i0)')nrun
   commline = trim(algoname_short(algo))//' at run '//trim(snum1)//' CF = '//trim(adjustl(snum))
   call export_structure(filename,atmr,cryst(1)%cell,cryst(1)%spg,cryst(1)%elem,CIF_FILE,   &
                         cryst(1)%cr_name,dataset(1)%wave(1),comm=commline)
!
   end subroutine export_model

!----------------------------------------------------------------------------------------------------

   subroutine save_sa(unitp,gopt)
   integer, intent(in)              :: unitp
   type(opti_info_type), intent(in) :: gopt   
   write(unitp)gopt
   end subroutine save_sa

!----------------------------------------------------------------------------------------------------

   subroutine read_sa(unitp,gopt,err)
   USE errormod
   integer, intent(in)               :: unitp
   type(opti_info_type), intent(out) :: gopt   
   type(error_type), intent(out)     :: err
   integer                           :: ier
   read(unitp, iostat = ier)gopt
   if (ier /= 0) call err%set("Error on reading global optimization conditions")
   end subroutine read_sa

!----------------------------------------------------------------------------------------------------
!corr
!corr   subroutine set_doc(doc,atcode)
!corr   USE arrayutil
!corr   type(doc_type), intent(inout)     :: doc
!corr   integer, dimension(:), intent(in) :: atcode
!corr   integer                           :: ndoc
!corr   integer                           :: nat
!corr   integer                           :: i
!corr   ndoc = count(atcode == 1)
!corr   nat = size(atcode)
!corr   if (ndoc == 0) then             ! DOC non active
!corr       doc%dtype = 0
!corr       !call reallocate(doc%vet,0)
!corr       call delete_array(doc%vet)
!corr   else
!corr       !call reallocate(doc%vet,ndoc)
!corr       call resize_array(doc%vet,ndoc)
!corr       if (ndoc == nat) then  ! DOC active on all atoms
!corr           doc%dtype = 1
!corr           doc%vet = [(i,i=1,nat)]
!corr       else                        ! DOC active on selected atoms
!corr           doc%dtype = 2
!corr           doc%vet = pack([(i,i=1,nat)],mask=atcode == 1)
!corr       endif
!corr   endif
!corr   end subroutine set_doc
!corr
!corr!----------------------------------------------------------------------------------------------------
!corr
!corr   logical function doc_active(doc) 
!corr   class(doc_type), intent(in) :: doc
!corr   doc_active = (doc%dtype > 0) 
!corr   end function doc_active
!corr
!corr!----------------------------------------------------------------------------------------------------
!corr
!corr   logical function doc_all_atoms(doc)
!corr   class(doc_type), intent(in) :: doc
!corr   doc_all_atoms = (doc%dtype == 1)
!corr   end function doc_all_atoms
!corr
!corr!----------------------------------------------------------------------------------------------------
!corr
!corr   subroutine doc_print(doc,kpr,atom)
!corr   USE atom_basic
!corr   class(doc_type), intent(in)               :: doc
!corr   integer, intent(in)                       :: kpr
!corr   type(atom_type), dimension(:), intent(in) :: atom
!corr   integer, parameter                        :: MAXLENDOC = 60
!corr   integer                                   :: natdoc
!corr   if (.not.doc%active()) return
!corr   if (doc%all_atoms()) then
!corr       write(kpr,'(1x,a)')'Dynamical Occupancy Correction is active on all atoms'
!corr   else
!corr       natdoc = size(doc%vet)
!corr       if (natdoc == 1) then
!corr           write(kpr,'(1x,a,i0,a)')'Dynamical Occupancy Correction is active on selected atom: '//  &
!corr           slabvet(doc%vet,atom%lab)
!corr       else
!corr           write(kpr,'(1x,a,i0,a)')'Dynamical Occupancy Correction is active on ',size(doc%vet),' selected atoms: '//  &
!corr           slabvet(doc%vet,atom%lab,maxlen=MAXLENDOC)
!corr       endif
!corr   endif
!corr   end subroutine doc_print
!----------------------------------------------------------------------------------------------------

   subroutine check_resp(crystal,resval)
!   
!  Check if resolution is comparable with the number of parameters
!
   USE variables, only: dataset
   USE datasetmod
   USE crystal_phase
   implicit none
   type(crystal_phase_t), intent(in) :: crystal
   real, intent(inout)               :: resval
   integer                           :: nrefpar
   integer                           :: nrefval
   integer, parameter                :: OBS_FOR_PAR = 7
!   
   nrefval = numref_from_res(resval,crystal%refc,dataset(1)%wave(1)) ! number of reflections up to resval
   if (nrefval < crystal%numref()) then
!
!      Numero minimo di riflessi in base al numero di paramatri (edof+idof)
       !nrefpar = nparam * OBS_FOR_PAR
       nrefpar = (crystal%edof() + crystal%idof()) * OBS_FOR_PAR
       if (nrefpar > crystal%numref()) nrefpar = crystal%numref()
!   
       if (nrefval < nrefpar) then
           resval = resvalue(crystal%refc(nrefpar)%tthd(1),dataset(1)%wave(1))    
       endif           
   endif
!
   end subroutine check_resp

END MODULE sannel
   
!----------------------------------------------------------------------------------------------------

   subroutine ges_atom_parameters_win(code,atomw,shifta,shiftval)
   USE sannel, only: shift_atom, shift_atom_copy, shiftatomval, reallocate_shift
   USE variables, only: cryst
   USE atom_type_util
   implicit none
   integer, intent(in)                       :: code
   real, save                                :: shiftatomval_copy
   type(atom_type), dimension(*), intent(in) :: atomw
   integer, dimension(*), intent(inout)      :: shifta
   real, intent(in)                          :: shiftval
   integer                                   :: i,natom
!
   select case (code)
      case (0)        ! salva i parametri all'apertura della finestra
        call save_atoms(cryst(1)%at)
        call reallocate_shift(shift_atom_copy,cryst(1)%natoms())
        shift_atom_copy(:) = shift_atom(:)
        shiftatomval_copy = shiftatomval
        !write(0,*)'Atom parameters window: salva parametri'

      case (1)        ! applica modifiche su 'OK'
!
!       aggiorna i parametri
        do i=1,cryst(1)%natoms()
           shift_atom(i)%rcod(:) = shifta(i)
           cryst(1)%at(i) = atomw(i)
        enddo
        call cryst(1)%set_doc(1,atomw(1)%doc)  ! update doc settings
            !call cryst(1)%print_doc(0)
        shiftatomval = shiftval
!
!       disegna il nuovo modello
        call clear_saved_atoms()
        call reallocate_shift(shift_atom_copy,0)
        !write(0,*)'Atom parameters window: applica modifiche'
        
      case (2)        ! rispristina parametri su 'Cancel'
        call restore_atoms(cryst(1)%at,natom)
        call cryst(1)%set_doc(1,atomw(1)%doc)  ! update doc settings
            !call cryst(1)%print_doc(0)
        call clear_saved_atoms()
        shift_atom(:) = shift_atom_copy(:)
        call reallocate_shift(shift_atom_copy,0)
        shiftatomval = shiftatomval_copy
        do i=1,cryst(1)%natoms()
           if (count(shift_atom(i)%rcod > 0) > 0) then
               shifta(i) = 1
           else
               shifta(i) = 0
           endif
        enddo
!
   end select
!
   end subroutine ges_atom_parameters_win

!----------------------------------------------------------------------------------------------------
   subroutine get_info_for_gtklist(nrow,nrcol,cfcol,strcol)
   USE model_util
   USE variables, only: cryst
   implicit none
   integer, intent(out)                                     :: nrow    ! numero di righe della gtklist
   integer, dimension(*), intent(out)                       :: nrcol   ! colonna 'Run nr.'
   real, dimension(*), intent(out)                          :: cfcol   ! colonna 'Cost Function'
   character(len=*) , intent(out)                           :: strcol  ! nome file
   integer                                                  :: i
   real, dimension(5)                                       :: infolist
!   
   nrow = cryst(1)%nmodels() 
!   
   strcol = trim(cryst(1)%cr_name)//char(0)
   do i=1,nrow  
      call model_get(cryst(1)%model(i),fom=infolist)
      nrcol(i) = nint(infolist(5))
      cfcol(i) = infolist(1)
   enddo
!
   end subroutine get_info_for_gtklist

!----------------------------------------------------------------------------------------------------

   subroutine ShowSolution(n)
   USE sannel, only: ginfo, sasogmap, sa_profile
   USE variables, only: cryst,dataset
   USE ilmviewmod
   USE view
   USE messagemod
   USE strutil
   USE TestModel, only:vett_test, rmsd
   USE model_util
   USE fouriermap
   USE atom_type_util
   USE datasetmod
   implicit none
   integer                                    :: n
   real, dimension(7)                         :: infomodel
   type(bond_type), dimension(:), allocatable :: legm
!   
!  Seleziona il modello n dalla lista
   call model_get(cryst(1)%model(n),cryst(1)%at,leg=legm,fom=infomodel)
!
!  Disegna il modello
   call ilmview_open()
   call ilmview_draw(cryst(1)%at,legm,cryst(1)%elem,cryst(1)%cell,cryst(1)%spg)
!
!  Disegna il profilo
   if (dataset(1)%datatype == POW_DATA) then
       call cryst(1)%sfactor()
       call sa_profile(cryst(1)%ref%fc)
       call vedinew(8,1,ginfo%nco)
   endif
!
   if (is_map_enabled()) then
       call map_set_centre(baricentro(cryst(1)%at))
       call jav_show_map(map_type_selected(),cryst(1)%ref,'SA Map',sogmap=sasogmap,fctype=2)
   endif
!
   call clear_messages(ipos=2)
   if (n == 1) then
       call write_message('The best structure (trial n.'//trim(i_to_s(int(infomodel(5))))//') was selected',pos=2)
   else
       call write_message('The structure of trial n.'//trim(i_to_s(int(infomodel(5))))//' was selected',pos=2)
   endif
   call write_message('Saved in file '//trim(cryst(1)%cr_name)//'_best'//trim(i_to_s(n))//'.cif',pos=3)
   call write_message('CF = ',rnum=infomodel(1),pos=4)
   if (dataset(1)%datatype == POW_DATA) call write_message('Ratio = ',rnum=infomodel(6),pos=0)
   if (cryst(1)%is_known()) then
       vett_test(3) = infomodel(2)           ! trovati
       vett_test(4) = infomodel(3)           ! distanza
       rmsd = infomodel(7)
       call write_message_found1(0)
   endif
!   
   end subroutine ShowSolution

!----------------------------------------------------------------------------------------------------

   subroutine SaveSolution(n)
   USE variables, only: cryst
   USE atom_type_util
   USE sannel, only: search_overlapped_atoms_print
!
   implicit none
   integer       ::n
   call SetToggleVal('SymmOn'//char(0), 0)
   write(6,'("Solution n. ",i0," was selected using Solutions menu")')n
   call print_atoms(cryst(1)%at,title='Structure solution',write_occ=.true.)
   call search_overlapped_atoms_print(cryst(1)%at,cryst(1)%cell%get_g(),cryst(1)%spg)
   end subroutine SaveSolution

!----------------------------------------------------------------------------------------------------

   subroutine CancelSolution(n)
   implicit none
   integer       ::n
   call SetToggleVal('SymmOn'//char(0), 0)
   call ShowSolution(n)
   end subroutine CancelSolution

!----------------------------------------------------------------------------------------------------

   subroutine fcnprint(gopti,popt,xopt,fopt,temp,nftot,nmtemp)
!
!  Stampa informazioni alla fine di ogni step di temperatura
!  Opzioni di output: S = stampa parametri
!                     P = draw pattern su viewer principale
!                     M = draw molecule su viewer principale
!                     V = draw on secondary viewer
!                     B = manage status and progress bar
   USE variables, only:cryst   !fcnprint
   USE sannel, only: timesa, SATIME_TOT, SATIME_GRAPH, is_profile_function, chisquared_res,  &
                     SA_LOCAL, ratio_value, gopt_statusbar, gopt_draw, xpar_to_crystals
   USE simulated_anneal
   USE General, only:lo
   USE TestModel
   USE reflection_type_util
   USE compare_struct
   USE Conteggi, only:npunti
   USE grexpomod
   USE Molcom
   USE messagemod
   USE strutil
   USE timer
   USE unit_cell
       USE anti_bump
   USE mpi_prog
   USE rrestr
!                
   implicit none
   type(opti_info_type), intent(in)           :: gopti    
   character(len=*), intent(in)               :: popt     ! stringa opzioni di stampa
   real, dimension(:), intent(in)             :: xopt     ! parametri minimo
   real, intent(in)                           :: fopt     ! cf per parametri minimo
   real, intent(in)                           :: temp     ! temperatura
   integer, intent(in)                        :: nftot    ! num. totale di movimenti
   integer, intent(in)                        :: nmtemp   ! num. di movimenti alla temperatura temp
   integer                                    :: nplot
   real, save                                 :: ratiopt
   character(len=300)                         :: stringa
   type(reflection_type), dimension(:), allocatable :: refc
!new   character(len=:), allocatable              :: stringa
   logical                                    :: is_anti_bump, is_restraints
   real                                       :: totaltime
   real                                       :: fbump,fres
   character(len=3) :: adv
   type(timer_type),save :: t_draw
   logical :: sdraw
!
   if (mpi_prog_rank /= mpi_prog_master) return
!
!  Aggiorna la status e progress bar
   if (index(popt,'B') > 0) then
!
!      aggiorna ratio perche un nuovo minimo e stato trovato
       if (npunti > 0) then     ! da migliorare
           ratiopt = ratio_value(fopt,gopti)
       else
           ratiopt = 0.0
       endif
       call gopt_statusbar(nftot,nmtemp)
   endif
!
!  Stampa informazioni alla fine di uno step di temperatura
   if (index(popt,'S') > 0) then    !S=brevi informazioni
       call write_message('Ratio = ',rnum=ratiopt,pos=vmess(LINE_RATIO))
       adv = 'no'
       write(6,'(a,i0)',advance=adv)'Run:',gopti%nrunc
       if (gopti%algo == SA_LOCAL) then
           write(6,'(a,f8.2)',advance=adv)'  Temp:',temp
       endif
       is_anti_bump = cryst(1)%numresab() > 0
       is_restraints = cryst(1)%numres() > 0
       if (is_anti_bump .or. is_restraints) then
           write(6,'(1x,a,f10.3)',advance=adv)'CF:',fopt
       else
           write(6,'(1x,a,f10.3)',advance=adv)trim(gopti%cost_function_name_short)//':',fopt
       endif
!!!!FIX: should be better saving CFs of xopt-rewrite func with array of CFs in output
       if (is_anti_bump .or. is_restraints .or. cryst(1)%is_known()) then
           call xpar_to_crystals(xopt)
       endif
       if (is_anti_bump) then
           fbump = cryst(1)%abset%weight*antibump_function(cryst(1)%abres,cryst(1)%at,cryst(1)%spg,cryst(1)%cell%get_g(),.false.)
           write(6,'(a,f10.3)',advance=adv)' A-bumpCF:',fbump
       endif
       if (is_restraints) then
           fres = chisquared_res(cryst(1))
           write(6,'(a,f10.3)',advance=adv)' CF-res:',fres
       endif
       totaltime = timesa(SATIME_TOT)%get_time()
       if (cryst(1)%is_known()) then
!   
!          Passa xopt in atm
!corrr           call xpar_to_crystals(xopt)
!
!          calcola fasi senza ricoprire ref
           call copy_ref(refc,cryst(1)%ref)
           !corr refc(:) = cryst(1)%ref(:)  
           call sfcalc(refc,cryst(1)%at,cryst(1)%spg,cryst(1)%elem)
           !ref(:)%ph = refc(:)%ph
           call to_coord_sir(-1,cryst(1)%at,refc,size(refc),cryst(1)%cell,cryst(1)%spg,lo,cryst(1)%atpub)
           stringa = ' (found: '//trim(i_to_s(int(vett_test(3))))//'/'//trim(i_to_s(int(vett_test(1))))//   &
           ' dist: '//trim(r_to_s(vett_test(4),3))//' RMSD: '//trim(r_to_s(rmsd,3))//')'
           call write_message( 'CF = '//trim(r_to_s(fopt,3))//trim(stringa),pos=vmess(LINE_CF))
           write(6,'(a,a13,a,i3,a,i0,a,f0.4,a,f0.4,a,i0)')                          &
             '  Time:',secs_to_string(totaltime),'  found:',int(vett_test(3)),   &
             '/',int(vett_test(1)),'  dist:',vett_test(4),' RMSD:',rmsd,'  Moves:',nftot
           if (gopti%fmin < fopt) then
               stringa = '(found: '//trim(i_to_s(int(cryst(1)%model(1)%info(2))))//'/'//trim(i_to_s(int(vett_test(1))))//   &
               ' dist: '//trim(r_to_s(cryst(1)%model(1)%info(3),3))//' RMSD: '//trim(r_to_s(cryst(1)%model(1)%info(7),3))//')'
               call write_message('CFmin = '//trim(r_to_s(gopti%fmin,3))//' at run.'    &
                    //trim(i_to_s(gopti%rmin))//' '//trim(stringa),pos=vmess(LINE_CFMIN))
           else
               call write_message('CFmin = '//trim(r_to_s(fopt,3))//' at run.'//trim(i_to_s(gopti%nrunc)),pos=vmess(LINE_CFMIN))
           endif
       else
           call write_message('CF = ',rnum=fopt,pos=vmess(LINE_CF))
           write(6,'(a,a10,4x,a,i0)')'  Time:',secs_to_string(totaltime),'Moves:',nftot
           if (gopti%fmin < fopt) then
               call write_message('CFmin = '//trim(r_to_s(gopti%fmin,3))//' at run.'  &
                    //trim(i_to_s(gopti%rmin)),pos=vmess(LINE_CFMIN))
           else
               call write_message('CFmin = '//trim(r_to_s(fopt,3))//' at run.'//trim(i_to_s(gopti%nrunc)),pos=vmess(LINE_CFMIN))
           endif
       endif   
       if (is_anti_bump) call write_message('Anti-bump CF = ',rnum=fbump,pos=vmess(LINE_CFBUMP))
       if (is_restraints) call write_message('Restraints CF = ',rnum=fres,pos=vmess(LINE_CFRES))
   endif
!   
!  Output grafico su xopt
   if (index(popt,'M') > 0) then
       sdraw = diff_time(t_draw,0.5)
   else
       sdraw = .false.
   endif
    !if (sdraw) write(0,*)'SDRAW=',sdraw
   call gopt_draw(sdraw,index(popt,'P')>0,gopti)
   !old call gopt_draw(index(popt,'M')>0,index(popt,'P')>0,gopti)
!
!  Output grafico su viewer secondario
   if (index(popt,'V') > 0) then  
!corr       call timestep(4) ! calcola il tempo impiegato dalle operazioni grafiche  
       call timesa(SATIME_GRAPH)%tic() ! calcola il tempo impiegato dalle operazioni grafiche  
!           
!      Aggiorna plot CF
       call gr_add_point_plot(grplot(gopti%nrunc),real(nftot),fopt)
!
!      Aggiorna linea che segnala Rwp alla fine dell'affinamento
       if(is_profile_function(gopti%cftype)) then
          nplot = size(grplot)
          if (grplot(nplot)%point(2)%x < nftot) grplot(nplot)%point(2)%x = nftot
       endif
!       
!      Se il viewer e' aperto esegui plot
       if (viewerOpened == 1) then  
           call gr_select_viewer(1)
           call gr_clear_area(0.0,1.0,0.0,1.0)
           call gr_area(0.08, 0.99, 0.15, 0.99)
           call gr_set_graph(grgraph,color=grplot(gopti%nrunc)%colorline)
           call gr_draw_graph(grgraph,grplot)
           call zflush()
           call gr_select_viewer(0)
       endif
!corr       call timestep(4) 
       call timesa(SATIME_GRAPH)%toc() ! calcola il tempo impiegato dalle operazioni grafiche  
   endif
!
   end subroutine fcnprint         
   
!----------------------------------------------------------------------------------------------------   
   
   subroutine sa_restorepar(hpar,xpar)
!
!  Ripristina i parametri se il movimento non e' accettato
!   
   USE fastfcal
   USE sannel, only: gpar, PO_TYPE, THERMALF_TYPE
   USE variables, only:cryst
   USE reflection_type_util
   implicit none
   integer, intent(in) :: hpar
   real, intent(in)    :: xpar
   integer             :: nph
!
   nph = gpar(hpar)%ph
   if (gpar(hpar)%typepar == THERMALF_TYPE) then 
       call fcalc_ripr_thermalf(gpar(hpar)%pat,cryst(nph)%sfact,cryst(nph)%sfactc)
   elseif (gpar(hpar)%typepar == PO_TYPE) then
       cryst(nph)%gpar = xpar !!!!gpo_sav
       call calcola_correzioneOP(cryst(nph)%ref,cryst(nph)%gpar,cryst(nph)%hklpo,cryst(nph)%cell,cryst(nph)%spg)
   else
       call fcalc_ripr_atm(gpar(hpar)%pat,cryst(nph)%spg,cryst(nph)%sfact,cryst(nph)%sfactc)
       if (cryst(nph)%doc_active()) then
           call fcalc_ripr_thermalf(cryst(nph)%vetdoc,cryst(nph)%sfact,cryst(nph)%sfactc)
       endif
   endif
!   
   end subroutine sa_restorepar

!----------------------------------------------------------------------------------------------------      
   subroutine sa_savepar(hpar)
!
!  Salva i parametri se il movimento � accettato
!   
   USE fastfcal
   USE sannel, only: gpar, THERMALF_TYPE, PO_TYPE
   USE variables, only: cryst
   implicit none
   integer, intent(in) :: hpar
   integer             :: i,nph
!
   nph = gpar(hpar)%ph
   if (gpar(hpar)%typepar == THERMALF_TYPE) then 
       call fcalc_save_thermalf(gpar(hpar)%pat,cryst(nph)%sfact,cryst(nph)%sfactc)
!corr   elseif (gpar(hpar)%typepar == PO_TYPE) then
!corr       gpo_sav = cryst(nph)%gpar
   else
       call fcalc_save_atm(gpar(hpar)%pat,cryst(nph)%spg,cryst(nph)%sfact,cryst(nph)%sfactc)
       if (cryst(nph)%doc_active()) then
           call fcalc_save_thermalf((/(i,i=1,cryst(nph)%natoms())/),cryst(nph)%sfact,cryst(nph)%sfactc)
       endif
   endif    
!   
   end subroutine sa_savepar

!corr!----------------------------------------------------------------------------------------------------
!corr
!corr   subroutine sa_savepar_all()
!corr   USE fastfcal
!corr   USE variables, only: cryst
!corr   implicit none
!corr   integer :: i
!corr!
!corr   call fcalc_save_atm((/(i,i=1,cryst(1)%natoms())/),cryst(1)%spg,cryst(1)%sfact,cryst(1)%sfactc)
!corr!
!corr   end subroutine sa_savepar_all
!corr
!----------------------------------------------------------------------------------------------------

   subroutine sannelexe()   
!
!  Questa routine e' chiamata quando e' attivo il comando %sannel
!
   USE sannel, only: goptim,SimAnn_Conditions_type,sa_prelim,sa_run
   USE General, only:ifAutomatic
   USE molcom, only:jscreen
   USE errormod
   USE bb_bc
   USE enable_amb
   USE variables, only: cryst
   implicit none
   type(SimAnn_Conditions_type) :: sc                     
   type(bb_bc_condition)        :: bcond                     
   type(error_type)             :: saerr
!
   if (cryst(1)%natoms() == 0 .and. jscreen > 0) then
       call open_file_selector('Import starting model for Simulated Annealing' //char(0))
   endif
   if (cryst(1)%natoms() == 0) return
!
   if (ifAutomatic == 1 .or. jscreen == 0) then
!
!      Inizializza e legge le direttive associate al comando %sannel 
       call c_fortran_al_lavoro(0)
       call sa_prelim(goptim,sc,bcond,.true.,saerr=saerr)
       if (saerr%signal) then
           call saerr%print()
       else
           call sa_run(goptim,sc,bcond)
           call exitsannel()
       endif
       call c_fine_lavoro(0)
   else
       call aprisannelwin()
   endif
!
   end subroutine sannelexe

!----------------------------------------------------------------------------------------------------
   
   subroutine sannelexewin(ierr, resmin, resmax, info, sc, bcond, ifButton, bumpsetC)   
!
!  Questa routine e' chiamata alla pressione del tasto 'Simulated Annealing'
!
   USE sannel, only: bumptab,goptim,sa_prelim,shiftatomval, &
                     set_allowed_cfunction
   USE anti_bump, only: bump_settings
   USE simulated_anneal, only: SimAnn_Conditions_type
!corr   USE patternref, only:thmax
   USE variables, only:cryst,dataset
   USE rotationmod
   USE sinterface
   USE bb_bc
   USE enable_amb
   USE ilmviewmod
   USE rrestr
   USE errormod
   USE datasetmod
   USE rinterface
   implicit none
   type(SimAnn_Conditions_type)        :: sc                     
   type(bb_bc_condition)               :: bcond                     
   real*8                              :: resmin,resmax
   type(opti_info_type), intent(inout) :: info
   integer                             :: nfrag
   integer, intent(in)                 :: ifButton    ! (=1 se arrivo qui da menubar)
   integer, intent(out)                :: ierr
   integer                             :: i
   logical                             :: modif
   real                                :: bump_sc
   integer                             :: nrl
   type(bump_settings)                 :: bumpsetC
   type(error_type)                    :: saerr
   integer, dimension(:), allocatable  :: cfvet
   integer                             :: numph, nph
   integer, dimension(:), allocatable  :: natoms,nelem

!
   if (.not.cryst(1)%has_symmetry()) then
       call set_cell_and_spg(cryst(1),modif)
       if (modif .and. jav_opened()) then
           call ilmview_draw(cryst(1)%at,cryst(1)%bond,cryst(1)%elem,cryst(1)%cell,cryst(1)%spg)
       endif
   endif
!
   if (cryst(1)%natoms() == 0) then
       call open_file_selector('Import starting model for Simulated Annealing' //char(0))
       if (cryst(1)%natoms() == 0) then
           ierr = -1
           return
       endif
   endif
!
   if (ifButton == 0) then
!
!      Sto chiamando SA da comando %sannel nell'input: mi leggo le direttive
       call sa_prelim(goptim,sc,bcond,.true.,saerr=saerr)
   else
!
!      Sto chiamando SA da menubar: non mi leggo le direttive
       call sa_prelim(goptim,sc,bcond,.false.,saerr=saerr)
   endif
!
   if (saerr%signal) then
       call saerr%print()
       ierr = -1
   else
!
       call abilita_tasti('window')
!
!      Calcola risoluzione minima e massima per la grafica
       resmin = resvalue(dataset(1)%tmax,dataset(1)%wave(1))
       resmax = resvalue(cryst(1)%refc(2)%tthd(1),dataset(1)%wave(1))   ! non va bene se i riflessi non sono ordinati (ncount=0)
!
       call set_allowed_cfunction(1,cfvet)
       call passa_cfvect(size(cfvet), cfvet)
!       
       call get_info_crystal(numph,natoms,nelem)
       call make_crystal_t()
       call put_info_crystal(numph,natoms,nelem,cryph)
       do nph=1, numph
          call put_info_ph(cryst(nph)%elem, cryst(nph)%at,nph)
          call put_info_po(cryst(nph)%kopcorr,cryst(nph)%hklpo,cryst(nph)%gcode, cryst(nph)%gpar,nph)
       enddo
!
       bump_sc = cryst(1)%abset%weight
       bumpsetC = cryst(1)%abset
       nrl = nrestraints(bumptab)
!
       call passa_abres(nrl)
       do i=1, nrl
          call put_info_abres(i-1,bumptab(i)%code,bumptab(i)%targ, &
               size(bumptab(i)%na),bumptab(i)%na)
       end do
!
!      Apre finestra della grafica
       ierr = cryst(1)%natoms()
       info = goptim
       nfrag = cryst(1)%numfrag()
       call passa_info(cryst(1)%at, cryst(1)%frag%nat, nfrag, cryst(1)%numrot(),cryst(1)%natoms(), shiftatomval)
       do i = 1, nfrag
          call passa_frag(cryst(1)%frag(i)%pos, cryst(1)%frag(i)%rcod, cryst(1)%frag(i)%tlow, &
             cryst(1)%frag(i)%thigh, cryst(1)%frag(i)%rbound, cryst(1)%frag(i)%rat, i-1)
       end do
   endif
!
   end subroutine sannelexewin

!----------------------------------------------------------------------------------------------------

   subroutine passa_rcod(i, c_rcod, c_tlow, c_thigh, c_rbound, c_rat)
   USE variables, only: cryst
   implicit none
                                                                                                                            
   integer      :: i
   integer      :: c_rcod(6)
   integer      :: c_rat
   real         :: c_tlow(3)
   real         :: c_thigh(3)
   real         :: c_rbound(2)
!
   cryst(1)%frag(i)%rcod = c_rcod
   cryst(1)%frag(i)%tlow = c_tlow
   cryst(1)%frag(i)%thigh = c_thigh
   cryst(1)%frag(i)%rbound = c_rbound
   cryst(1)%frag(i)%rat = c_rat
!                
   end subroutine passa_rcod

!----------------------------------------------------------------------------------------------------

   integer function get_srcod(i, j)
   USE sannel, only: shift_atom
   implicit none
                                                                                                                
   integer      :: i
   integer      :: j
!
   get_srcod = shift_atom(i)%rcod(j)
!
   end function get_srcod

!----------------------------------------------------------------------------------------------------

   integer function get_rrcod(i)
   USE variables, only: cryst
   implicit none
                                                                                                                
   integer      :: i
!
   get_rrcod = cryst(1)%rot(i)%rcod
!
   end function get_rrcod

!----------------------------------------------------------------------------------------------------

   subroutine  get_rators(i, vettore, cod, val, bounds)
   USE variables, only: cryst
   implicit none
   integer      :: i, j, cod
   integer      :: vettore(4)
   real         :: val, bounds(2)
!
   do j=1, 4
      vettore(j) = cryst(1)%rot(i)%ators(j)
   end do
   do j=1, 2
      bounds(j) = cryst(1)%rot(i)%trbound(j)
   end do
   cod = cryst(1)%rot(i)%rcod
   val = cryst(1)%rot(i)%torsval
!
   end subroutine get_rators

!----------------------------------------------------------------------------------------------------

   subroutine  put_rators(i, cod, val, bounds)
   USE variables, only: cryst
   implicit none
                                                                                                                
   integer      :: i, j, cod
   real         :: val, bounds(2)
!
   do j=1, 2
      cryst(1)%rot(i)%trbound(j) = bounds(j)
   end do
   cryst(1)%rot(i)%rcod = cod
   cryst(1)%rot(i)%torsval = val
!
   end subroutine put_rators

!----------------------------------------------------------------------------------------------------

   subroutine  put_srcod(i, cod)
   USE sannel, only: shift_atom
   implicit none
                                                                                                                
   integer      :: i, cod(3), j
!
   do j=1, 3
      shift_atom(i)%rcod(j) = cod(j)
   end do
!
   end subroutine put_srcod

!----------------------------------------------------------------------------------------------------

   subroutine finishsannel(info, sc, bcond, shiftatomv, bumpsetC)
!
!  Routine chiamata dalla grafica alla pressione del tasto 'Execute'
!
   USE sannel, only: SimAnn_Conditions_type,goptim,shiftatomval,bumptab,sa_run
   USE rotationmod
   USE variables, only:cryst
   USE bb_bc
   USE ilmviewmod
   USE anti_bump
   USE rrestr
   implicit none
   type(SimAnn_Conditions_type)         :: sc                     
   type(bb_bc_condition)                :: bcond                     
   type(opti_info_type), intent(inout)  :: info
   integer                              :: i
   real                                 :: shiftatomv
   type(bump_settings)                  :: bumpsetC    
!
   call jav_set_modify(0)
   goptim = info
   shiftatomval = shiftatomv
   cryst(1)%abset = bumpsetC     
   if (cryst(1)%abset%baction > 0) then
       call bump_from_table(bumptab,cryst(1)%abres,cryst(1)%at,cryst(1)%bond,cryst(1)%cell,   &
                            cryst(1)%frag,cryst(1)%rot,cryst(1)%abset)
   else    ! 'None' was selected
       call delete_restraints(cryst(1)%abres)
   endif
   call sa_run(goptim,sc,bcond)
   info = goptim                           !sa_run modifica alcuni campi di gopti
!
!  Calcola nuovi valori per angoli di torsione per la grafica
   do i=1,cryst(1)%numrot()   
      cryst(1)%rot(i)%torsval = torsion_value_for_rotat(cryst(1)%rot(i),cryst(1)%at,cryst(1)%cell)
      !write(0,*)'TORS=',i,rotation(i)%torsval
   enddo
   call jav_set_modify(3)
!
   end subroutine finishsannel

!----------------------------------------------------------------------------------------------------

   subroutine exitsannel()
!
!  Routine chiamata dalla grafica alla pressione del tasto 'Exit'
!
   USE Molcom, only:jscreen
   USE sannel, only: gpar
   USE genlsq
   USE counts
!corr   USE RefinecomRef, only:rfcond    !!!!!,iFirstRietveld    
!corr   USE patternref, only:thmax   !,kcount_status
   USE RIETV
   USE variables, only:cryst   !!!,ref
   USE exportfiles
   USE ilmviewmod
   !USE atom_type_util
   USE grexpomod
   USE messagemod
   USE enable_amb
   USE profile_function
   USE arrayutil
   USE sinterface, only: fillatom
   USE crystal_phase
   USE reflection_type_util
   implicit none
   integer :: nph
!
!  dealloca parametri
   if (allocated(gpar)) deallocate(gpar)
!
!  Ripristina riflessi
   do nph=1,numphase(cryst)
      call cryst(nph)%set_used_ref(numrefl(cryst(nph)%refc))
   enddo
   !call set_limit_ref(cryst(1)%ref,rfcond%lim,cryst(1)%refc,thmax)
   !call set_limit_ref(cryst(1)%ref,cryst(1)%refc,thmax)
   cryst(1)%ref(:)%pk = 20
!
!!!!!   iFirstRietveld = 0  ! non passare affinamento SA al rietveld
!
!  dealloca riflessi e conteggi
   call Allocafuncls(DEALLOCA)
!
!  dealloca i plot
   if (jscreen > 0) then
       call clear_plot(grplot,1)
       call gr_close_viewer()
       call jav_set_modify(1)
       call abilita_tasti('structure+pattern')
!corr       kcount_status = 1; 
   endif
   call fillatom(cryst(1)%at)
   call clear_messages()
!
   call export_files()
!
   end subroutine exitsannel

!----------------------------------------------------------------------------------------------------

   subroutine evaluate_res(res,numref,tthmaxr,ier)
!
!  Data la risoluzione calcola il numero di riflessi e il 2theta massimo
!
!corr   USE patternref, only:thmax,thmin
   USE datasetmod
   USE variables, only: dataset,cryst
   USE reflection_type_util
!
   implicit none
   real, intent(inout)  :: res      ! risoluzione
   integer, intent(out) :: numref   ! numero di riflessi
   real, intent(out)    :: tthmaxr  ! 2theta massimo
   integer              :: ier      ! >0 se res � inferiore al valore minimo
   real                 :: resmin 
!
!  Calcola il valore minimo consentito per res
   resmin = resvalue(dataset(1)%tmax,dataset(1)%wave(1))
!
   if (res < resmin) then
       tthmaxr = dataset(1)%tmax
       numref = numrefl(cryst(1)%refc)
       ier = 1   ! segnala che res e' fuori range
   else
!corr       tthmaxr = 2*rtod*asin(alambda/(2*res))
       tthmaxr = tthvalue(res,dataset(1)%wave(1))
       numref = count(cryst(1)%refc%tthd(1) <= tthmaxr .and. cryst(1)%refc%tthd(1) >= dataset(1)%tmin)   
       ier = 0
   endif
!
   end subroutine evaluate_res

!----------------------------------------------------------------------------------------------------

   subroutine gestione_eventi(kevent)
   USE Molcom
   USE grexpomod
   USE ilmviewmod
   implicit none
   integer, intent(out) :: kevent ! se > 0 l'informazione va passata all'esterno
   integer              :: xsize,ysize
!
   if (jscreen > 0 .and. ifBrekke /= 0) then
       select case (ifBrekke)
          case (1)    ! tasto skip
            kevent = ifBrekke
          case (2)    ! tasto stop
            kevent = ifBrekke
          case (3)    ! tasto zoom + sul viewer secondario
            kevent = 0
            !call OnZoom_event(1)
          case (4)    ! tasto zoom - sul viewer secondario
            kevent = 0
            !call OnZoom_event(-1)
          case (5)    ! tasto annulla zoom sul viewer secondario
            kevent = 0
            !call OnZoom_event(0)
          case (6)    ! resize sul viewer secondario
            kevent = 0
            call new_viewer_size(xsize,ysize)
            call OnResize_event(xsize, ysize)
          case (7)    ! open viewer secondario
            kevent = 0
            !call OnOpen_event()
          case (8)    ! close viewer secondario
            kevent = 0
            !call gr_close_viewer()
          case (9)   
            kevent = 0
            call DoSingleIteration()

          case (11)  ! show structure
            kevent = 0
            call ilmview_open()

          case default
             kevent = 0
       end select
       ifBrekke = 0
   else
       kevent = 0
   endif
!
   end subroutine gestione_eventi

!corr   subroutine test_calcg()
!corr   USE lsqalgo
!corr   implicit none
!corr   real, dimension(2) :: xgrad,xval
!corr   real               :: h,fx
!corr   integer         :: i
!corr     interface
!corr       real function funcprova(x)
!corr       real, dimension(:), intent(in) :: x
!corr       end function funcprova
!corr     end interface
!corr!
!corr   xval(1) = 8.0
!corr   xval(2) = 1.0
!corr   fx = funcprova(xval)
!corr   write(0,*)'eps=',epsilon(1.0)
!corr   do i=1,10
!corr      h=10.0**(-i)
!corr      call calc_gradient(xgrad,xval,fx,funcprova,h,icod=1)
!corr      write(0,*)'1 h=',h,' xgrad=',xgrad(:)
!corr      call calc_gradient(xgrad,xval,fx,funcprova,h,icod=2)
!corr      write(0,*)'2 h=',h,' xgrad=',xgrad(:)
!corr   enddo
!corr   call calc_gradient(xgrad,xval,fx,funcprova,icod=1)
!corr   write(0,*)'1 xgrad=',xgrad(:)
!corr   call calc_gradient(xgrad,xval,fx,funcprova,icod=2)
!corr   write(0,*)'2 xgrad=',xgrad(:)
!corr!
!corr   end subroutine test_calcg
!corr
!corr   real function funcprova(x)
!corr   real, dimension(:), intent(in) :: x
!corr!
!corr   funcprova = 2*x(1)*x(2)**2 + 3*x(1)*x(2)
!corr!
!corr   end function funcprova

!----------------------------------------------------------------------
   subroutine SetValuesForPattern(cgpo, chklpo, ckopcorr)
!
   USE progtype
   USE variables, only: cryst
!corr   USE RefinecomRef, only: gpo
!
   implicit none
   type(rparam_type) :: cgpo
   integer           :: i, ckopcorr, chklpo(3)
!
   cryst(1)%kopcorr = ckopcorr
   cryst(1)%gpar = cgpo%val
   cryst(1)%gcode = cgpo%rcod
   do i=1, 3
      cryst(1)%hklpo(i) = chklpo(i)
   end do
!
   end subroutine SetValuesForPattern
!----------------------------------------------------------------------

   subroutine GetValuesForPattern(cgpo, chklpo, ckopcorr)
!
   USE progtype
   USE variables, only: cryst
!corr   USE RefinecomRef, only: gpo
!
   implicit none
   type(rparam_type) :: cgpo
   integer           :: i, ckopcorr, chklpo(3)
!
   ckopcorr = cryst(1)%kopcorr
!corr   cgpo = gpo
   cgpo%val  = cryst(1)%gpar
   cgpo%rcod = cryst(1)%gcode
   do i=1, 3
      chklpo(i) = cryst(1)%hklpo(i)
   end do
!
   end subroutine GetValuesForPattern
!----------------------------------------------------------------------

   subroutine Passa_Info_Bumptab(i,targ,code)
   USE sannel, only: bumptab
   implicit none
   integer, intent(in)              :: code
   integer, intent(in)              :: i
   real, intent(in)                 :: targ
!
   bumptab(i)%code  = code
   bumptab(i)%targ  = targ
!
   end subroutine Passa_Info_Bumptab

!----------------------------------------------------------------------

   subroutine randomize_parameters()
!
!  'Randomize button'
!
   !USE sannel
   USE fragmentmod
   USE variables, only: cryst
   USE ilmviewmod
   USE General, only:lo
   USE compare_struct
   USE reflection_type_util
   USE rotationmod
   implicit none
   integer :: i
!
   call randomize_molecule(cryst(1)%at,cryst(1)%bond,cryst(1)%cell,cryst(1)%frag,cryst(1)%rot)  ! randomize atm
   call set_atm_in_frag(cryst(1)%frag,cryst(1)%at)                    ! update fragment info
   do i=1,cryst(1)%numrot()                                         ! update torsion value
      cryst(1)%rot(i)%torsval = torsion_value_for_rotat(cryst(1)%rot(i),cryst(1)%at,cryst(1)%cell)
   enddo
   call ilmview_draw(cryst(1)%at,cryst(1)%bond,cryst(1)%elem,cryst(1)%cell,cryst(1)%spg)                          ! update graphic view
   if (cryst(1)%is_known()) then
       call cryst(1)%sfactor()
       call to_coord_sir(-1,cryst(1)%at,cryst(1)%ref,size(cryst(1)%ref),cryst(1)%cell,cryst(1)%spg,lo,cryst(1)%atpub)
       call write_message_found1(5)
   endif
!
   end subroutine randomize_parameters
